@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -11,6 +11,10 @@
  *     Oracle - initial API and implementation from Oracle TopLink
  *     07/16/2009-2.0 Guy Pelletier 
  *       - 277039: JPA 2.0 Cache Usage Settings
+ *     10/29/2010-2.2 Michael O'Brien 
+ *       - 325167: Make reserved # bind parameter char generic to enable native SQL pass through
+ *     06/30/2011-2.3.1 Guy Pelletier 
+ *       - 341940: Add disable/enable allowing native queries    
  ******************************************************************************/  
 package org.eclipse.persistence.config;
 
@@ -47,25 +51,35 @@ public class QueryHints {
     public static final String BIND_PARAMETERS = "eclipselink.jdbc.bind-parameters";
 
     /**
-     * "javax.persistence.cacheRetrieveMode"
+     * "eclipselink.jdbc.parameter-delimiter"
+     * <p>Configures parameter binding char to a user defined character (overrides default hash symbol).
+     * Valid values are:  ParameterDelimiterType.DEFAULT, other valid single character,
+     * "" can not be used and will instead default to ParameterDelimiterType.DEFAULT
+     * @see PersistenceUnitProperties#JDBC_BIND_PARAMETERS
+     * @see org.eclipse.persistence.queries.DatabaseQuery#setShouldBindAllParameters(boolean)
+     */
+    public static final String PARAMETER_DELIMITER = "eclipselink.jdbc.parameter-delimiter";
+    
+    /**
+     * "javax.persistence.cache.retrieveMode"
      * <p>Configures the behavior when data is retrieved by the find methods and 
-     * by the execution of queries. The cacheRetrieveMode is ignored for the 
+     * by the execution of queries. The cache retrieve mode is ignored for the 
      * refresh method, which always causes data to be retrieved from the 
-     * database and not the cache.
+     * database and not the cache.</p>
      * @see org.eclipse.persistence.queries.DatabaseQuery#dontMaintainCache()
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#shouldCheckDescriptorForCacheUsage()
      */
-    public static final String CACHE_RETRIEVE_MODE = "javax.persistence.cacheRetrieveMode";
+    public static final String CACHE_RETRIEVE_MODE = "javax.persistence.cache.retrieveMode";
     
     /**
-     * "javax.persistence.cacheStoreMode"
-     * <p> Configures the behavior when data is read from the database and when 
-     * data is committed into the database.
+     * "javax.persistence.cache.storeMode"
+     * <p>Configures the behavior when data is read from the database and when 
+     * data is committed into the database.</p>
      * @see org.eclipse.persistence.queries.DatabaseQuery#dontMaintainCache()
      * @see org.eclipse.persistence.queries.DatabaseQuery#maintainCache()
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#refreshIdentityMapResult()
      */
-    public static final String CACHE_STORE_MODE = "javax.persistence.cacheStoreMode";
+    public static final String CACHE_STORE_MODE = "javax.persistence.cache.storeMode";
     
     /**
      * "eclipselink.cache-usage"
@@ -167,6 +181,15 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setRedirector(org.eclipse.persistence.queries.QueryRedirector)
      */
     public static final String QUERY_REDIRECTOR = "eclipselink.query.redirector";
+    
+    /**
+     * "eclipselink.partitioning"
+     * <p>Used to provide a PartitioningPolicy instance or name to the query.
+     * This allows the query to execute on a specific, or on multiple connection pools.
+     * @see org.eclipse.persistence.annotations.Partitioning
+     * @see org.eclipse.persistence.descriptors.partitioning.PartitioningPolicy
+     */
+    public static final String PARTITIONING = "eclipselink.partitioning";
 
     /**
      * "eclipselink.query-type"
@@ -228,6 +251,12 @@ public class QueryHints {
      * Batch reading is normally more efficient than join fetch, especially for collection relationships.
      * Valid values are strings that represent JPQL style navigations to a relationship.
      * <p>e.g. "e.manager.phoneNumbers"
+     * <p>Note:  Batch reading produces two lines of SQL to read the requested object graph and uses the
+     *  'where clause' of the first SQL as part of the 'where clause' of the second.  When indirect attributes
+     *  of the objects returned from the first query are referenced the second query is executed to return those attributes.
+     *  If changes are made to data referenced in the 'where clause' after the first SQL results are returned but before the
+     *  second query executes then these attributes may not be returned by the second query. To prevent this reference these 
+     *  attributes before changing data associated with the query."
      * @see BatchFetch
      * @see #BATCH_TYPE
      * @see BatchFetchType
@@ -383,7 +412,7 @@ public class QueryHints {
      * By default queries generate their SQL the first time they are executed,
      * and avoid the cost of generating the SQL on subsequent executions.
      * This can be used to generate the SQL on every execution if the 
-     * query requires usage of dynamic SQL.
+     * query requires usage of dynamic SQL, such as to handle null parameters (use IS NULL instead of = null).
      * This only effects the SQL generation, not parameter binding or statement caching.
      * Valid values are:  HintValues.FALSE, HintValues.TRUE,
      * "" could be used instead of default value HintValues.FALSE
@@ -620,7 +649,16 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.FetchGroup#setShouldLoad(boolean)
      */
     public static final String FETCH_GROUP_LOAD = "eclipselink.fetch-group.load";
-
+    
+    /**
+     * "javax.persistence.fetchgraph"
+     * <p> Configures a jpa entity graph to be used as a fetch graph template.   
+     * Those attributes included in the entity graph will have their values fetched and populated.
+     * Attributes of the represented entity that are not present in the entity graph will be treated
+     * as FetchType.LAZY.
+     */
+    public static final String JPA_FETCH_GRAPH = "javax.persistence.fetchgraph";
+    
     /**
      * "eclipselink.fetch-group.default"
      * <p>Configures the query not to use the default fetch group.
@@ -651,7 +689,16 @@ public class QueryHints {
     public static final String LOAD_GROUP = "eclipselink.load-group";
     
     /**
-     * "eclipselink.fetch-group.attribute"
+     * "javax.persistence.loadgraph"
+     * <p> Configures a jpa entity graph to be used as a load graph template.   
+     * Those attributes included in the entity graph will have their values fetched and populated.
+     * Attributes of the represented entity that are not present in the entity graph will be loaded based
+     * on their mapping based FetchType settings.
+     */
+    public static final String JPA_LOAD_GRAPH = "javax.persistence.loadgraph";
+    
+    /**
+     * "eclipselink.load-group.attribute"
      * <p>Configures the query to use load group that includes a list of attributes.
      * Each attribute must be defined using a separate hint.
      * The query will load all relational attributes defined in the load group.
@@ -693,6 +740,24 @@ public class QueryHints {
      */
     public static final String INHERITANCE_OUTER_JOIN = "eclipselink.inheritance.outer-join";
 
+    /**
+     * The <code>"eclipselink.jdbc.allow-native-sql-query"</code> property 
+     * specifies whether a single native SQL query should override a persistence 
+     * unit level setting (eclipselink.jdbc.allow-native-sql-queries). The PU 
+     * level flag is of particular importance within a multitenant to minimize 
+     * the potential impact of revealing multitenant. However in some cases the
+     * application may need to allow certain native SQL queries through.
+     * <p>
+     * Allowed Values (String):
+     * <ul>
+     * <li>"true" - allow native SQL (and override the persistence unit flag)
+     * <li>"false" - (DEFAULT) do not allow native SQL (and respect the persistence unit flag if set.)
+     * </ul>
+     * 
+     * @see DatabaseQuery#setAllowNativeSQLQuery(boolean) {
+     */
+    public static final String ALLOW_NATIVE_SQL_QUERY = "eclipselink.jdbc.allow-native-sql-query";
+    
     /**
      * "eclipselink.history.as-of"
      * <p>Configures the query to query the state of the object as-of a point in time.
@@ -754,4 +819,53 @@ public class QueryHints {
      */
     public static final String PESSIMISTIC_LOCK_SCOPE = "javax.persistence.lock.scope";
 
+    /**
+     * "eclipselink.composite-unit.member"
+     * <p>This must be used on a native query executed on composite persistence unit.
+     * It specifies the name of composite member persistence unit to execute the query on.
+     * @see org.eclipse.persistence.queries.DatabaseQuery#setSessionName(String)
+     */
+    public static final String COMPOSITE_UNIT_MEMBER = "eclipselink.composite-unit.member";
+    
+    /**
+     * "eclipselink.jdbc.batch-writing"
+     * <p>Configures if this modify query can be batched through batch writing.
+     * Some types of queries cannot be batched, such as DDL on some databases.
+     * Disabling batch writing will also allow the row count to be returned.
+     * Valid values are:  HintValues.PERSISTENCE_UNIT_DEFAULT, HintValues.TRUE, HintValues.FALSE,
+     * "" could be used instead of default value HintValues.PERSISTENCE_UNIT_DEFAULT
+     * @see org.eclipse.persistence.queries.ModifyQuery#setIsBatchExecutionSupported(boolean)
+     */
+    public static final String BATCH_WRITING = "eclipselink.jdbc.batch-writing";
+
+    /**
+     * "eclipselink.result-set-access"
+     * <p>Configures if this read object(s) query should keep result set until all result object(s) are built.
+     * That allows to avoid getting potentially huge objects (LOBs) from ResultSet in case they would not be used
+     * because the object is found in the cache.   
+     * Valid values are:  HintValues.PERSISTENCE_UNIT_DEFAULT, HintValues.TRUE, HintValues.FALSE,
+     * "" could be used instead of default value HintValues.PERSISTENCE_UNIT_DEFAULT
+     * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setIsResultSetAccessOptimizedQuery(boolean)
+     * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#clearIsResultSetOptimizedQuery
+     */
+    public static final String RESULT_SET_ACCESS = "eclipselink.result-set-access";
+
+    /**
+     * "eclipselink.serialized-object"
+     * <p>Configures if this read object(s) query use SerializedObjectPolicy if it's set on the descriptor.
+     * Valid values are:  HintValues.TRUE, HintValues.FALSE,
+     * "" could be used instead of default value HintValues.FALSE
+     * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setShouldUseSerializedObjectPolicy(boolean)
+     */
+    public static final String SERIALIZED_OBJECT = "eclipselink.serialized-object";
+
+    /**
+     * "eclipselink.query-return-name-value-pairs"
+     * <p>Configures a ResultSetMappingQuery to return populated DatabaseRecord(s)
+     * instead of raw data.  This is particularly useful when the structure of the
+     * returned data is not known.
+     * @see org.eclipse.persistence.queries.ResultSetMappingQuery#setShouldReturnNameValuePairs(boolean)
+     * @see org.eclipse.persistence.queries.ResultSetMappingQuery#shouldReturnNameValuePairs()
+     */
+    public static final String RETURN_NAME_VALUE_PAIRS = "eclipselink.query-return-name-value-pairs";
 }
