@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
+ * which accompanies this distribution.
  * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+ * and the Eclipse Distribution License is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
  * Contributors:
@@ -27,6 +27,7 @@ import static javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI;
 import org.eclipse.persistence.descriptors.ClassExtractor;
 import org.eclipse.persistence.exceptions.DescriptorException;
 import org.eclipse.persistence.internal.xr.Attachment;
+import org.eclipse.persistence.internal.xr.BatchQueryOperation;
 import org.eclipse.persistence.internal.xr.CollectionResult;
 import org.eclipse.persistence.internal.xr.DeleteOperation;
 import org.eclipse.persistence.internal.xr.InsertOperation;
@@ -52,6 +53,7 @@ import org.eclipse.persistence.oxm.XMLField;
 import org.eclipse.persistence.oxm.mappings.XMLChoiceCollectionMapping;
 import org.eclipse.persistence.oxm.mappings.XMLChoiceObjectMapping;
 import org.eclipse.persistence.oxm.mappings.XMLCompositeCollectionMapping;
+import org.eclipse.persistence.oxm.mappings.XMLCompositeDirectCollectionMapping;
 import org.eclipse.persistence.oxm.mappings.XMLCompositeObjectMapping;
 import org.eclipse.persistence.oxm.mappings.XMLDirectMapping;
 import org.eclipse.persistence.oxm.mappings.XMLTransformationMapping;
@@ -63,7 +65,7 @@ import static org.eclipse.persistence.internal.xr.sxf.SimpleXMLFormat.DEFAULT_SI
 import static org.eclipse.persistence.internal.xr.sxf.SimpleXMLFormat.DEFAULT_SIMPLE_XML_TAG;
 import static org.eclipse.persistence.oxm.XMLConstants.BOOLEAN_QNAME;
 
-@SuppressWarnings({"serial", "unchecked", "rawtypes"})
+@SuppressWarnings({"serial", "rawtypes"})
 public class DBWSModelProject extends Project {
 
     public NamespaceResolver ns;
@@ -92,6 +94,7 @@ public class DBWSModelProject extends Project {
         addDescriptor(buildInsertDescriptor());
         addDescriptor(buildDeleteDescriptor());
         addDescriptor(buildUpdateDescriptor());
+        addDescriptor(buildBatchQueryDescriptor());
 
         for (Iterator descriptors = getDescriptors().values().iterator(); descriptors.hasNext();) {
             XMLDescriptor descriptor = (XMLDescriptor)descriptors.next();
@@ -137,6 +140,7 @@ public class DBWSModelProject extends Project {
         operationsMapping.addChoiceElement("query", QueryOperation.class);
         operationsMapping.addChoiceElement("update", UpdateOperation.class);
         operationsMapping.addChoiceElement("delete", DeleteOperation.class);
+        operationsMapping.addChoiceElement("batch-sql-query", BatchQueryOperation.class);
         descriptor.addMapping(operationsMapping);
         return descriptor;
     }
@@ -181,6 +185,28 @@ public class DBWSModelProject extends Project {
         return descriptor;
     }
 
+    /**
+     * Build an XMLDescriptor for BatchQueryOperation.
+     */
+    protected XMLDescriptor buildBatchQueryDescriptor() {
+        XMLDescriptor descriptor = new XMLDescriptor();
+        descriptor.setJavaClass(BatchQueryOperation.class);
+        descriptor.setDefaultRootElement("batch-sql-query");
+
+        XMLDirectMapping name = new XMLDirectMapping();
+        name.setAttributeName("name");
+        name.setXPath("name/text()");
+        descriptor.addMapping(name);
+
+        XMLCompositeDirectCollectionMapping statementsMapping = new XMLCompositeDirectCollectionMapping();
+        statementsMapping.setAttributeName("batchSql");
+        XMLField f1 = new XMLField("batch-statement/text()");
+        f1.setIsCDATA(true);
+        statementsMapping.setField(f1);
+        descriptor.addMapping(statementsMapping);
+        return descriptor;
+    }
+
     // parameter
     //
     // name,type
@@ -200,6 +226,12 @@ public class DBWSModelProject extends Project {
         type.setAttributeTransformer(qNameTransformer);
         descriptor.addMapping(type);
 
+        XMLDirectMapping optional = new XMLDirectMapping();
+        optional.setAttributeName("optional");
+        optional.setXPath("@optional");
+        optional.setNullValue(false);
+        descriptor.addMapping(optional);
+
         return descriptor;
     }
 
@@ -265,7 +297,7 @@ public class DBWSModelProject extends Project {
         });
         isCollection.setXPath("@isCollection");
         descriptor.addMapping(isCollection);
-        
+
         XMLField isColl = new XMLField("@isCollection");
         isColl.setSchemaType(BOOLEAN_QNAME);
         descriptor.getInheritancePolicy().setClassIndicatorField(isColl);
@@ -291,6 +323,10 @@ public class DBWSModelProject extends Project {
             }
         });
 
+        XMLDirectMapping jdbcTypeMapping = new XMLDirectMapping();
+        jdbcTypeMapping.setAttributeName("jdbcType");
+        jdbcTypeMapping.setXPath("@jdbcType");
+        descriptor.addMapping(jdbcTypeMapping);
         return descriptor;
     }
 
@@ -465,6 +501,10 @@ public class DBWSModelProject extends Project {
         complexTypeName.setXPath("complex-type");
         descriptor.addMapping(complexTypeName);
 
+        XMLDirectMapping jdbcTypeMapping = new XMLDirectMapping();
+        jdbcTypeMapping.setAttributeName("jdbcType");
+        jdbcTypeMapping.setXPath("@jdbcType");
+        descriptor.addMapping(jdbcTypeMapping);
         return descriptor;
     }
 
@@ -482,7 +522,7 @@ public class DBWSModelProject extends Project {
         resultType.addFieldTransformer("type/text()", qNameTransformer);
         resultType.setAttributeTransformer(qNameTransformer);
         descriptor.addMapping(resultType);
-        
+
         return descriptor;
     }
 
@@ -522,12 +562,17 @@ public class DBWSModelProject extends Project {
         name.setAttributeName("name");
         name.setXPath("name/text()");
         descriptor.addMapping(name);
-        
+
         XMLDirectMapping descriptorName = new XMLDirectMapping();
         descriptorName.setAttributeName("descriptorName");
         descriptorName.setXPath("descriptor-name/text()");
         descriptor.addMapping(descriptorName);
-        
+
+        XMLDirectMapping findByPKQuery = new XMLDirectMapping();
+        findByPKQuery.setAttributeName("findByPKQuery");
+        findByPKQuery.setXPath("find-by-pk-query/text()");
+        descriptor.addMapping(findByPKQuery);
+
         XMLCompositeCollectionMapping parameters = new XMLCompositeCollectionMapping();
         parameters.setAttributeName("parameters");
         parameters.setReferenceClass(Parameter.class);
