@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,23 +12,25 @@
  *       http://wiki.eclipse.org/EclipseLink/Development/Dynamic 
  *       (https://bugs.eclipse.org/bugs/show_bug.cgi?id=200045)
  *
+ *     11/10/2011-2.4 Guy Pelletier 
+ *       - 357474: Address primaryKey option from tenant discriminator column
  ******************************************************************************/
 package org.eclipse.persistence.dynamic;
 
 //javase imports
 import java.io.IOException;
 import java.io.InputStream;
+import java.lang.reflect.Field;
 import java.util.Iterator;
 import org.w3c.dom.Document;
 
 //EclipseLink imports
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.descriptors.RelationalDescriptor;
-import org.eclipse.persistence.descriptors.changetracking.AttributeChangeTrackingPolicy;
 import org.eclipse.persistence.exceptions.ValidationException;
 import org.eclipse.persistence.indirection.IndirectList;
+import org.eclipse.persistence.internal.dynamic.DynamicPropertiesManager;
 import org.eclipse.persistence.internal.dynamic.DynamicTypeImpl;
-import org.eclipse.persistence.internal.dynamic.DynamicTypeInstantiationPolicy;
 import org.eclipse.persistence.internal.dynamic.ValuesAccessor;
 import org.eclipse.persistence.internal.helper.ConversionManager;
 import org.eclipse.persistence.internal.helper.DatabaseField;
@@ -99,7 +101,20 @@ public class DynamicTypeBuilder {
         RelationalDescriptor descriptor = new RelationalDescriptor();
         descriptor.setJavaClass(dynamicClass);
         this.entityType = new DynamicTypeImpl(descriptor, parentType);
-
+        // JAXB generates some classes that do not conform to DynamicEntity interface - ignore
+        if (DynamicEntity.class.isAssignableFrom(dynamicClass)) {
+            try {
+                Field dpmField = dynamicClass.getField(DynamicPropertiesManager.PROPERTIES_MANAGER_FIELD);
+                DynamicPropertiesManager dpm = (DynamicPropertiesManager)dpmField.get(null);
+                dpm.setType(entityType);
+                entityType.setDynamicPropertiesManager(dpm);
+            }
+            catch (Exception e) {
+                // this is bad! Without the dpmField, not much to do but die :-( ...
+                e.printStackTrace();
+                return;
+            }
+        }
         configure(descriptor, tableNames);
     }
 
@@ -114,11 +129,27 @@ public class DynamicTypeBuilder {
      */
     public DynamicTypeBuilder(DynamicClassLoader dcl, ClassDescriptor descriptor, DynamicType parentType) {
         this.entityType = new DynamicTypeImpl(descriptor, parentType);
-
-        if (descriptor.getJavaClass() == null) {
+        Class<?> dynamicClass = descriptor.getJavaClass();
+        if (dynamicClass == null) {
             addDynamicClasses(dcl, descriptor.getJavaClassName(), parentType);
         }
-
+        else {
+            // JAXB generates some classes that do not conform to DynamicEntity interface - ignore
+            if (DynamicEntity.class.isAssignableFrom(dynamicClass)) {
+                try {
+                    Field dpmField = 
+                        descriptor.getJavaClass().getField(DynamicPropertiesManager.PROPERTIES_MANAGER_FIELD);
+                    DynamicPropertiesManager dpm = (DynamicPropertiesManager)dpmField.get(null);
+                    dpm.setType(entityType);
+                    entityType.setDynamicPropertiesManager(dpm);
+                }
+                catch (Exception e) {
+                    // this is bad! Without the dpmField, not much to do but die :-( ...
+                    e.printStackTrace();
+                    return;
+                }
+            }
+        }
         configure(descriptor);
     }
 
@@ -158,14 +189,11 @@ public class DynamicTypeBuilder {
 
         }
 
-        descriptor.setObjectChangePolicy(new AttributeChangeTrackingPolicy());
-        descriptor.setInstantiationPolicy(new DynamicTypeInstantiationPolicy((DynamicTypeImpl) getType()));
-
         for (int index = 0; index < descriptor.getMappings().size(); index++) {
             addMapping(descriptor.getMappings().get(index));
         }
 
-        descriptor.setProperty(DynamicTypeImpl.DESCRIPTOR_PROPERTY, entityType);
+        descriptor.setProperty(DynamicType.DESCRIPTOR_PROPERTY, entityType);
     }
 
     public DynamicType getType() {
@@ -191,7 +219,7 @@ public class DynamicTypeBuilder {
             ForeignReferenceMapping frMapping = (ForeignReferenceMapping) mapping;
             return frMapping.usesIndirection() || frMapping.isCollectionMapping();
         }
-        if (mapping.isAggregateMapping() && !mapping.isXMLMapping()) {
+        if (mapping.isAggregateObjectMapping() && !mapping.isXMLMapping()) {
             return !((AggregateObjectMapping) mapping).isNullAllowed();
         }
         return false;
@@ -417,7 +445,7 @@ public class DynamicTypeBuilder {
      * position of the mapping in the descriptor is captured to use as its
      * index.
      */
-    protected DatabaseMapping addMapping(DatabaseMapping mapping) {
+    public DatabaseMapping addMapping(DatabaseMapping mapping) {
         ClassDescriptor descriptor = getType().getDescriptor();
 
         if (!descriptor.getMappings().contains(mapping)) {
@@ -443,12 +471,15 @@ public class DynamicTypeBuilder {
             if (typeName != null) {
                 // Remove any additional padding
                 typeName = typeName.trim();
-                Class<?> attrType = ConversionManager.getDefaultManager().convertClassNameToClass(typeName);
-                ((AbstractDirectMapping) mapping).setAttributeClassification(attrType);
+                try {
+                    Class<?> attrType = ConversionManager.getDefaultManager().convertClassNameToClass(typeName);
+                    ((AbstractDirectMapping) mapping).setAttributeClassification(attrType);
+                } catch (Exception e) {
+                }
             }
         }
 
-        mapping.setAttributeAccessor(new ValuesAccessor(getType(), mapping, index));
+        mapping.setAttributeAccessor(new ValuesAccessor(mapping));
 
         if (requiresInitialization(mapping)) {
             this.entityType.getMappingsRequiringInitialization().add(mapping);
@@ -566,13 +597,33 @@ public class DynamicTypeBuilder {
 
             project.getLogin().getPlatform().getConversionManager().setLoader(dynamicClassLoader);
 
-            for (Iterator<?> i = project.getAliasDescriptors().values().iterator(); i.hasNext();) {
+            for (Iterator<?> i = project.getOrderedDescriptors().iterator(); i.hasNext();) {
                 ClassDescriptor descriptor = (ClassDescriptor) i.next();
                 if (descriptor.getJavaClass() == null) {
                     createType(dynamicClassLoader, descriptor, project);
                 }
             }
             project.convertClassNamesToClasses(dynamicClassLoader);
+            for (Iterator<?> i = project.getOrderedDescriptors().iterator(); i.hasNext();) {
+                ClassDescriptor descriptor = (ClassDescriptor) i.next();
+                Class<?> dynamicClass = descriptor.getJavaClass();
+                // JAXB generates some classes that do not conform to DynamicEntity interface - ignore
+                if (dynamicClass != null && DynamicEntity.class.isAssignableFrom(dynamicClass)) {
+                    DynamicType type = DynamicHelper.getType(descriptor);
+                    try {
+                        Field dpmField = 
+                            descriptor.getJavaClass().getField(DynamicPropertiesManager.PROPERTIES_MANAGER_FIELD);
+                        DynamicPropertiesManager dpm = (DynamicPropertiesManager)dpmField.get(null);
+                        dpm.setType(type);
+                        ((DynamicTypeImpl)type).setDynamicPropertiesManager(dpm);
+                    }
+                    catch (Exception e) {
+                        // this is bad! Without the dpmField, not much to do but die :-( ...
+                        e.printStackTrace();
+                        return null;
+                    }
+                }
+            }
         }
 
         return project;
@@ -620,6 +671,21 @@ public class DynamicTypeBuilder {
             type = new DynamicTypeBuilder(dcl, descriptor, parent).getType();
         }
 
+        // JAXB generates some classes that do not conform to DynamicEntity interface - ignore
+        if (javaClass != null && DynamicEntity.class.isAssignableFrom(javaClass)) {
+            try {
+                Field dpmField = javaClass.getField(DynamicPropertiesManager.PROPERTIES_MANAGER_FIELD);
+                DynamicPropertiesManager dpm = (DynamicPropertiesManager)dpmField.get(null);
+                dpm.setType(type);
+                ((DynamicTypeImpl)type).setDynamicPropertiesManager(dpm);
+            }
+            catch (Exception e) {
+                // this is bad! Without the dpmField, not much to do but die :-( ...
+                e.printStackTrace();
+                return null;
+            }
+        }
+        
         return type;
     }
-}
+}
\ No newline at end of file
