@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -23,8 +23,10 @@ import org.eclipse.persistence.exceptions.DatabaseException;
 import org.eclipse.persistence.exceptions.DescriptorException;
 import org.eclipse.persistence.expressions.Expression;
 import org.eclipse.persistence.expressions.ExpressionBuilder;
+import org.eclipse.persistence.indirection.ValueHolder;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.identitymaps.CacheId;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.queries.JoinedAttributeManager;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
@@ -111,6 +113,7 @@ public class EISOneToOneMapping extends ObjectReferenceMapping implements EISMap
      * Both the source foreign key field name and the corresponding
      * target primary key field name must be specified.
      */
+    @Override
     public void addForeignKeyField(DatabaseField sourceForeignKeyField, DatabaseField targetKeyField) {
         getSourceToTargetKeyFields().put(sourceForeignKeyField, targetKeyField);
         getTargetToSourceKeyFields().put(targetKeyField, sourceForeignKeyField);
@@ -211,7 +214,7 @@ public class EISOneToOneMapping extends ObjectReferenceMapping implements EISMap
                 return null;
             }
             result[index] = row.get(sourceKeyField);
-            if (getReferenceDescriptor().getCacheKeyType() == CacheKeyType.ID_VALUE) {
+            if (getReferenceDescriptor().getCachePolicy().getCacheKeyType() == CacheKeyType.ID_VALUE) {
                 return result[index];
             }
         }
@@ -345,7 +348,24 @@ public class EISOneToOneMapping extends ObjectReferenceMapping implements EISMap
      * Check for batch + aggregation reading.
      */
     @Override
-    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, AbstractSession session) throws DatabaseException {
+    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, CacheKey cacheKey, AbstractSession session, boolean isTargetProtected, Boolean[] wasCacheUsed) throws DatabaseException {
+        if (this.descriptor.getCachePolicy().isProtectedIsolation()) {
+            if (this.isCacheable && isTargetProtected && cacheKey != null) {
+                //cachekey will be null when isolating to uow
+                //used cached collection
+                Object result = null;
+                Object cached = cacheKey.getObject();
+                if (cached != null) {
+                    if (wasCacheUsed != null){
+                        wasCacheUsed[0] = Boolean.TRUE;
+                    }
+                    return this.getAttributeValueFromObject(cached);
+                }
+                return result;
+            } else if (!this.isCacheable && !isTargetProtected && cacheKey != null) {
+                return this.indirectionPolicy.buildIndirectObject(new ValueHolder(null));
+            }
+        }
         // If any field in the foreign key is null then it means there are no referenced objects
         // Skip for partial objects as fk may not be present.
         if (!query.hasPartialAttributeExpressions()) {
@@ -512,24 +532,4 @@ public class EISOneToOneMapping extends ObjectReferenceMapping implements EISMap
     public void setSelectionSQLString(String sqlString) {
         throw DescriptorException.invalidMappingOperation(this, "setSelectionSQLString");
     }
-
-    /**
-     * INTERNAL:
-     * This method is not supported in an EIS environment.
-     */
-    @Override
-    public void setUsesBatchReading(boolean usesBatchReading) {
-        if (usesBatchReading) {
-            throw DescriptorException.invalidMappingOperation(this, "setUsesBatchReading");
-        }
-    }
-
-    /**
-     * INTERNAL:
-     * This method is not supported in an EIS environment.
-     */
-    @Override
-    public void useBatchReading() {
-        throw DescriptorException.invalidMappingOperation(this, "useBatchReading");
-    }
 }
