@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -42,6 +42,10 @@ public class EntityManagerSetupException extends EclipseLinkException {
     public static final int NO_TEMPORARY_CLASSLOADER_AVAILABLE=28025;    
     public static final int CREATE_CONTAINER_EMF_NOT_SUPPORTED_IN_OSGI=28026;  
     public static final int COULD_NOT_FIND_PERSISTENCE_UNIT_BUNDLE=28027;  
+    public static final int FAILED_TO_INSTANTIATE_PROPERTY = 28028;
+    public static final int COMPOSITE_INCOMPATIBLE_WITH_SESSIONS_XML = 28029;
+    public static final int COMPOSITE_MEMBER_CANNOT_BE_USED_STANDALONE = 28030;
+    public static final int MISSING_PROPERTY = 28031;
     
     /**
      * INTERNAL:
@@ -139,6 +143,22 @@ public class EntityManagerSetupException extends EclipseLinkException {
         setupException.setErrorCode(FAILED_TO_INSTANTIATE_SERVER_PLATFORM);
         return setupException;
     }
+
+    public static EntityManagerSetupException missingProperty(String property) {
+        Object[] args = { property };
+
+        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, MISSING_PROPERTY, args));
+        setupException.setErrorCode(MISSING_PROPERTY);
+        return setupException;
+    }
+
+    public static EntityManagerSetupException failedToInstantiateProperty(String className, String property, Exception exception) {
+        Object[] args = { className, property };
+
+        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, FAILED_TO_INSTANTIATE_PROPERTY, args), exception);
+        setupException.setErrorCode(FAILED_TO_INSTANTIATE_PROPERTY);
+        return setupException;
+    }
     
     public static EntityManagerSetupException classNotFoundWhileProcessingAnnotations(String className, Exception exception) {
         Object[] args = { className };
@@ -164,10 +184,10 @@ public class EntityManagerSetupException extends EclipseLinkException {
         return setupException;
     }
     
-    public static EntityManagerSetupException cannotDeployWithoutPredeploy(String persistenceUnitName, String state) {
+    public static EntityManagerSetupException cannotDeployWithoutPredeploy(String persistenceUnitName, String state, Exception exception) {
         Object[] args = { persistenceUnitName, state };
 
-        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, CANNOT_DEPLOY_WITHOUT_PREDEPLOY, args));
+        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, CANNOT_DEPLOY_WITHOUT_PREDEPLOY, args), exception);
         setupException.setErrorCode(CANNOT_DEPLOY_WITHOUT_PREDEPLOY);
         return setupException;
     }
@@ -196,15 +216,15 @@ public class EntityManagerSetupException extends EclipseLinkException {
         return setupException;
     }
 
-    public static EntityManagerSetupException cannotPredeploy(String persistenceUnitName, String state) {
+    public static EntityManagerSetupException cannotPredeploy(String persistenceUnitName, String state, Exception exception) {
         Object[] args = { persistenceUnitName, state };
 
-        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, CANNOT_PREDEPLOY, args));
+        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, CANNOT_PREDEPLOY, args), exception);
         setupException.setErrorCode(CANNOT_PREDEPLOY);
         return setupException;
     }
 
-    public static EntityManagerSetupException predeployFailed(String persistenceUnitName, RuntimeException exception) {
+    public static EntityManagerSetupException predeployFailed(String persistenceUnitName, Throwable exception) {
         Object[] args = { persistenceUnitName };
 
         EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, PREDEPLOY_FAILED, args), exception);
@@ -212,7 +232,7 @@ public class EntityManagerSetupException extends EclipseLinkException {
         return setupException;
     }
 
-    public static EntityManagerSetupException deployFailed(String persistenceUnitName, RuntimeException exception) {
+    public static EntityManagerSetupException deployFailed(String persistenceUnitName, Throwable exception) {
         Object[] args = { persistenceUnitName };
 
         EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(EntityManagerSetupException.class, DEPLOY_FAILED, args), exception);
@@ -289,7 +309,7 @@ public class EntityManagerSetupException extends EclipseLinkException {
     
     /**
      * INTERNAL:
-     * An OSGi application is trying to instantiate a persistence unit for which a bundle does nto exist
+     * An OSGi application is trying to instantiate a persistence unit for which a bundle does not exist
      * @param PUName
      * @return
      */
@@ -301,5 +321,32 @@ public class EntityManagerSetupException extends EclipseLinkException {
         return setupException;
     }
 
+    /**
+     * INTERNAL:
+     * Persistence unit tries to use sessions.xml and to be a composite.
+     * @param PUName
+     * @return
+     */
+    public static EntityManagerSetupException compositeIncompatibleWithSessionsXml(String persistenceUnitName) {
+        Object[] args = { persistenceUnitName };
+        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(
+                EntityManagerSetupException.class, COMPOSITE_INCOMPATIBLE_WITH_SESSIONS_XML, args));
+        setupException.setErrorCode(COMPOSITE_INCOMPATIBLE_WITH_SESSIONS_XML);
+        return setupException;
+    }
+
+    /**
+     * INTERNAL:
+     * Persistence unit tries to use sessions.xml and to be a composite.
+     * @param PUName
+     * @return
+     */
+    public static EntityManagerSetupException compositeMemberCannotBeUsedStandalone(String persistenceUnitName) {
+        Object[] args = { persistenceUnitName };
+        EntityManagerSetupException setupException = new EntityManagerSetupException(ExceptionMessageGenerator.buildMessage(
+                EntityManagerSetupException.class, COMPOSITE_MEMBER_CANNOT_BE_USED_STANDALONE, args));
+        setupException.setErrorCode(COMPOSITE_MEMBER_CANNOT_BE_USED_STANDALONE);
+        return setupException;
+    }
 }
 
