@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,16 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     10/15/2010-2.2 Guy Pelletier 
+ *       - 322008: Improve usability of additional criteria applied to queries at the session/EM
+ *     05/24/2011-2.3 Guy Pelletier 
+ *       - 345962: Join fetch query when using tenant discriminator column fails.
+ *     06/30/2011-2.3.1 Guy Pelletier 
+ *       - 341940: Add disable/enable allowing native queries
+ *     02/08/2012-2.4 Guy Pelletier 
+ *       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls 
+ *     02/06/2013-2.5 Guy Pelletier 
+ *       - 382503: Use of @ConstructorResult with createNativeQuery(sqlString, resultSetMapping) results in NullPointerException
  ******************************************************************************/  
 package org.eclipse.persistence.exceptions;
 
@@ -17,6 +27,7 @@ import java.util.List;
 import org.eclipse.persistence.queries.*;
 import org.eclipse.persistence.internal.queries.*;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
+import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.expressions.*;
 import org.eclipse.persistence.mappings.*;
@@ -183,7 +194,20 @@ public class QueryException extends ValidationException {
     public final static int CAST_MUST_USE_INHERITANCE = 6167;
     public final static int PREPARE_FAILED = 6168;
     public final static int ORIGINAL_QUERY_MUST_USE_IN = 6169;
-
+    public final static int PARTIONING_NOT_SUPPORTED = 6171;
+    public final static int MISSING_CONNECTION_POOL = 6172;
+    public final static int FAILOVER_FAILED = 6173;
+    public final static int MISSING_CONTEXT_PROPERTY_FOR_PROPERTY_PARAMETER_EXPRESSION = 6174;
+    public static final int NATIVE_SQL_QUERIES_ARE_DISABLED = 6175;
+    public final static int EXCEPTION_WHILE_LOADING_CONSTRUCTOR = 6176;
+    public final static int COLUMN_RESULT_NOT_FOUND = 6177;
+    public final static int RESULT_SET_ACCESS_OPTIMIZATION_IS_NOT_POSSIBLE = 6178;
+    public final static int SOP_OBJECT_DESERIALIZE_FAILED = 6179;
+    public final static int SOP_OBJECT_IS_NOT_FOUND = 6180;
+    public final static int SOP_OBJECT_WRONG_VERSION = 6181;
+    public final static int SOP_OBJECT_WRONG_PK = 6182;
+    
+    
     /**
      * INTERNAL:
      * Constructor.
@@ -234,7 +258,15 @@ public class QueryException extends ValidationException {
         queryException.setErrorCode(ARGUMENT_SIZE_MISMATCH_IN_QUERY_AND_QUERY_DEFINITION);
         return queryException;
     }
+    
+    public static QueryException missingContextPropertyForPropertyParameterExpression(DatabaseQuery query, String argumentName) {
+        Object[] args = { argumentName };
 
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, MISSING_CONTEXT_PROPERTY_FOR_PROPERTY_PARAMETER_EXPRESSION, args), query);
+        queryException.setErrorCode(MISSING_CONTEXT_PROPERTY_FOR_PROPERTY_PARAMETER_EXPRESSION);
+        return queryException;
+    }
+    
     public static QueryException namedArgumentNotFoundInQueryParameters(String argumentName) {
         Object[] args = {argumentName};
         
@@ -243,6 +275,13 @@ public class QueryException extends ValidationException {
         return queryException;
     }
     
+    public static QueryException nativeSQLQueriesAreDisabled(DatabaseQuery query) {
+        Object[] args = {};        
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, NATIVE_SQL_QUERIES_ARE_DISABLED, args), query);
+        queryException.setErrorCode(NATIVE_SQL_QUERIES_ARE_DISABLED);
+        return queryException;
+    }
+    
     public static QueryException backupCloneIsDeleted(Object clone) {
         Object[] args = { clone, clone.getClass(), Integer.valueOf(System.identityHashCode(clone)), CR };
 
@@ -469,13 +508,27 @@ public class QueryException extends ValidationException {
     }
 
     /**
-     * An exception was throwing while using a ReportQuery with a constructor expession
+     * An exception was thrown while initializing the constructor from the class.
+     */
+    public static QueryException exceptionWhileInitializingConstructor(Exception thrownException, DatabaseQuery query, Class targetClass) {
+        Object[] args = { targetClass, thrownException };
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, EXCEPTION_WHILE_LOADING_CONSTRUCTOR, args));
+        queryException.setErrorCode(EXCEPTION_WHILE_LOADING_CONSTRUCTOR);
+        queryException.setInternalException(thrownException);
+        queryException.setQuery(query);
+        return queryException;
+    }
+
+    /**
+     * An exception was throwing while using a ReportQuery with a constructor expression
      */
     public static QueryException exceptionWhileUsingConstructorExpression(Exception thrownException, DatabaseQuery query) {
         Object[] args = { thrownException };
 
         QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, EXCEPTION_WHILE_USING_CONSTRUCTOR_EXPRESSION, args));
         queryException.setErrorCode(EXCEPTION_WHILE_USING_CONSTRUCTOR_EXPRESSION);
+        queryException.setInternalException(thrownException);
         queryException.setQuery(query);
         return queryException;
     }
@@ -970,7 +1023,7 @@ public class QueryException extends ValidationException {
         queryException.setErrorCode(REDIRECTION_CLASS_OR_METHOD_NOT_SET);
         return queryException;
     }
-
+    
     public static QueryException unableToSetRedirectorOnQueryFromHint(DatabaseQuery query, String hint, String redirectorClass, Exception ex) {
         Object[] args = {hint, redirectorClass  };
 
@@ -1386,6 +1439,14 @@ public class QueryException extends ValidationException {
         return queryException;
     }
 
+    public static QueryException columnResultNotFound(DatabaseField dbField) {
+        Object[] args = { dbField };
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, COLUMN_RESULT_NOT_FOUND, args));
+        queryException.setErrorCode(COLUMN_RESULT_NOT_FOUND);
+        return queryException;
+    }
+    
     public static QueryException compatibleTypeNotSet(DatabaseType type) {
         Object[] args = { type };
 
@@ -1537,6 +1598,69 @@ public class QueryException extends ValidationException {
         queryException.setErrorCode(ORIGINAL_QUERY_MUST_USE_IN);
         return queryException;
     }
+
+    public static QueryException partitioningNotSupported(AbstractSession session, DatabaseQuery query) {
+        Object[] args = { session };
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, PARTIONING_NOT_SUPPORTED, args), query);
+        queryException.setErrorCode(PARTIONING_NOT_SUPPORTED);
+        return queryException;
+    }
+
+    public static QueryException missingConnectionPool(String poolName, DatabaseQuery query) {
+        Object[] args = { poolName };
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, MISSING_CONNECTION_POOL, args), query);
+        queryException.setErrorCode(MISSING_CONNECTION_POOL);
+        return queryException;
+    }
+
+    public static QueryException failoverFailed(String poolName) {
+        Object[] args = { poolName };
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, FAILOVER_FAILED, args));
+        queryException.setErrorCode(FAILOVER_FAILED);
+        return queryException;
+    }
     
+    public static QueryException resultSetAccessOptimizationIsNotPossible(DatabaseQuery query) {
+        Object[] args = {};
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, RESULT_SET_ACCESS_OPTIMIZATION_IS_NOT_POSSIBLE, args), query);
+        queryException.setErrorCode(RESULT_SET_ACCESS_OPTIMIZATION_IS_NOT_POSSIBLE);
+        return queryException;
+    }
+
+    public static QueryException sopObjectDeserializeFailed(DatabaseQuery query, DatabaseField sopField, AbstractRecord row, Exception exception) {
+        Object[] args = {sopField, row};
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, SOP_OBJECT_DESERIALIZE_FAILED, args), query, exception);
+        queryException.setErrorCode(SOP_OBJECT_DESERIALIZE_FAILED);
+        return queryException;
+    }
+
+    public static QueryException sopObjectIsNotFound(DatabaseQuery query, DatabaseField sopField, AbstractRecord row) {
+        Object[] args = {sopField, row};
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, SOP_OBJECT_IS_NOT_FOUND, args), query);
+        queryException.setErrorCode(SOP_OBJECT_IS_NOT_FOUND);
+        return queryException;
+    }
+
+    public static QueryException sopObjectWrongVersion(DatabaseQuery query, Object sopObjectVersion, DatabaseField sopField, AbstractRecord row) {
+        Object[] args = {sopObjectVersion, sopField, row};
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, SOP_OBJECT_WRONG_VERSION, args), query);
+        queryException.setErrorCode(SOP_OBJECT_WRONG_VERSION);
+        return queryException;
+    }
+
+    public static QueryException sopObjectWrongPk(DatabaseQuery query, Object sopObjectPk, DatabaseField sopField, AbstractRecord row) {
+        Object[] args = {sopObjectPk, sopField, row};
+
+        QueryException queryException = new QueryException(ExceptionMessageGenerator.buildMessage(QueryException.class, SOP_OBJECT_WRONG_PK, args), query);
+        queryException.setErrorCode(SOP_OBJECT_WRONG_PK);
+        return queryException;
+    }
 }
 
