@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -16,7 +16,13 @@ import org.eclipse.persistence.exceptions.ValidationException;
 import org.eclipse.persistence.mappings.DatabaseMapping;
 import org.eclipse.persistence.oxm.XMLDescriptor;
 import org.eclipse.persistence.exceptions.i18n.ExceptionMessageGenerator;
+import org.eclipse.persistence.internal.oxm.mappings.Mapping;
 
+/**
+ * <P><B>Purpose</B>: XMLMarshalExceptions are raised when issues are encountered
+ * during XMLMarshaller or XMLUnmarshaller operations.
+ * </P>
+ */
 public class XMLMarshalException extends ValidationException {
     public static final int INVALID_XPATH_STRING = 25001;
     public static final int INVALID_XPATH_INDEX_STRING = 25002;
@@ -44,6 +50,21 @@ public class XMLMarshalException extends ValidationException {
     public static final int UNMAPPED_CONTENTHANDLER_DOESNT_IMPLEMENT = 25025;
     public static final int OBJ_NOT_FOUND_IN_CACHE = 25026; 
     public static final int NO_ATTACHMENT_UNMARSHALLER_SET = 25027;
+    public static final int UNKNOWN_XSI_TYPE = 25028;
+    public static final int SUBCLASS_ATTEMPTED_TO_OVERRIDE_NAMESPACE_DECLARATION = 25029;
+    public static final int ERROR_INVOKING_NAMESPACE_PREFIX_MAPPER = 25030;
+    public static final int ERROR_PROCESSING_PREFIX_MAPPER = 25031;
+    public static final int ERROR_INVOKING_CHARACTER_ESCAPE_HANDLER = 25032;
+    public static final int ERROR_PROCESSING_CHARACTER_ESCAPE_HANDLER = 25033;
+    public static final int ERROR_INVOKING_ID_RESOLVER = 25034;
+    public static final int ERROR_PROCESSING_ID_RESOLVER = 25035;
+    public static final int WRAPPED_ID_RESOLVER_WITH_MULTI_ID = 25036;
+    public static final int OBJECT_CYCLE_DETECTED = 25037;
+    public static final int PLATFORM_NOT_SUPPORTED_WITH_JSON_MEDIA_TYPE = 25038;
+    public static final int UNMARSHAL_FROM_STRING_FAILED = 25039;
+    public static final int MISSING_ID_FOR_IDREF = 25040;
+    public static final int INVALID_ATTRIBUTE_GROUP_NAME = 25041;
+    public static final int NULL_VALUE_NOT_ALLOWED_FOR_VARIABLE = 25042;
 
     // ==========================================================================================
     protected XMLMarshalException(String message) {
@@ -240,6 +261,17 @@ public class XMLMarshalException extends ValidationException {
         return exception;
     }
 
+    /**
+     * @since EclipseLink 2.5.0
+     */
+    public static XMLMarshalException noDescriptorFound(Mapping mapping) {
+        Object[] args = { mapping.getAttributeName() };
+
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, NO_DESCRIPTOR_FOUND, args));
+        exception.setErrorCode(NO_DESCRIPTOR_FOUND);
+        return exception;
+    }
+
     public static XMLMarshalException noDescriptorFound(DatabaseMapping mapping) {
         Object[] args = { mapping.getAttributeName() };
 
@@ -282,4 +314,135 @@ public class XMLMarshalException extends ValidationException {
         return exception;
     }
     
-}
+    /**
+     * @since EclipseLink 2.5.0
+     */
+    public static XMLMarshalException unknownXsiTypeValue(String xsiType, Mapping mapping) {
+        Object[] args = {xsiType, mapping};
+        
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, UNKNOWN_XSI_TYPE, args));
+        exception.setErrorCode(UNKNOWN_XSI_TYPE);
+        return exception;
+    }
+
+    public static XMLMarshalException unknownXsiTypeValue(String xsiType, DatabaseMapping mapping) {
+        Object[] args = {xsiType, mapping};
+        
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, UNKNOWN_XSI_TYPE, args));
+        exception.setErrorCode(UNKNOWN_XSI_TYPE);
+        return exception;
+    }
+
+    public static XMLMarshalException subclassAttemptedToOverrideNamespaceDeclaration(String prefix, String subClassName, String subClassNamespaceURI, String parentClassName, String parentClassNamespaceURI) {
+        Object[] args = {prefix, subClassName, subClassNamespaceURI, parentClassName, parentClassNamespaceURI};
+
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, SUBCLASS_ATTEMPTED_TO_OVERRIDE_NAMESPACE_DECLARATION, args));
+        exception.setErrorCode(SUBCLASS_ATTEMPTED_TO_OVERRIDE_NAMESPACE_DECLARATION);
+        return exception;
+    }
+    
+    public static XMLMarshalException errorInvokingPrefixMapperMethod(String methodName, Object prefixMapper) {
+        Object[] args = {methodName, prefixMapper};
+        
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, ERROR_INVOKING_NAMESPACE_PREFIX_MAPPER, args));
+        exception.setErrorCode(ERROR_INVOKING_NAMESPACE_PREFIX_MAPPER);
+        return exception;
+    }
+    
+    public static XMLMarshalException errorProcessingPrefixMapper(String methodName, Object prefixMapper) {
+        Object[] args = {methodName, prefixMapper};
+        
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, ERROR_PROCESSING_PREFIX_MAPPER, args));
+        exception.setErrorCode(ERROR_PROCESSING_PREFIX_MAPPER);
+        return exception;
+    }    
+
+    public static XMLMarshalException errorInvokingCharacterEscapeHandler(String methodName, Object handler, Throwable nestedException) {
+        Object[] args = {methodName, handler};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, ERROR_INVOKING_CHARACTER_ESCAPE_HANDLER, args));
+        exception.setErrorCode(ERROR_INVOKING_CHARACTER_ESCAPE_HANDLER);
+        exception.setInternalException(nestedException);
+        return exception;
+    }
+
+    public static XMLMarshalException errorProcessingCharacterEscapeHandler(String methodName, Object handler, Throwable nestedException) {
+        Object[] args = {methodName, handler};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, ERROR_PROCESSING_CHARACTER_ESCAPE_HANDLER, args));
+        exception.setErrorCode(ERROR_PROCESSING_CHARACTER_ESCAPE_HANDLER);
+        exception.setInternalException(nestedException);
+        return exception;
+    }
+
+    public static XMLMarshalException errorInvokingIDResolver(String methodName, Object resolver, Throwable nestedException) {
+        Object[] args = {methodName, resolver};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, ERROR_INVOKING_ID_RESOLVER, args));
+        exception.setErrorCode(ERROR_INVOKING_ID_RESOLVER);
+        exception.setInternalException(nestedException);
+        return exception;
+    }
+
+    public static XMLMarshalException errorProcessingIDResolver(String methodName, Object resolver, Throwable nestedException) {
+        Object[] args = {methodName, resolver};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, ERROR_PROCESSING_ID_RESOLVER, args));
+        exception.setErrorCode(ERROR_PROCESSING_ID_RESOLVER);
+        exception.setInternalException(nestedException);
+        return exception;
+    }
+
+    public static XMLMarshalException wrappedIDResolverWithMultiID(String keyString, Object resolver) {
+        Object[] args = {keyString, resolver};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, WRAPPED_ID_RESOLVER_WITH_MULTI_ID, args));
+        exception.setErrorCode(WRAPPED_ID_RESOLVER_WITH_MULTI_ID);
+        return exception;
+    }
+
+    public static XMLMarshalException objectCycleDetected(String objectCycleString) {
+        Object[] args = { objectCycleString };
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, OBJECT_CYCLE_DETECTED, args));
+        exception.setErrorCode(OBJECT_CYCLE_DETECTED);
+        return exception;
+    }
+    
+    public static XMLMarshalException unsupportedMediaTypeForPlatform() {
+        Object[] args = { };
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, PLATFORM_NOT_SUPPORTED_WITH_JSON_MEDIA_TYPE, args));
+        exception.setErrorCode(PLATFORM_NOT_SUPPORTED_WITH_JSON_MEDIA_TYPE);
+        return exception;
+    }
+
+    public static XMLMarshalException unmarshalFromStringException(String systemId, Exception nestedException) {
+        Object[] args = {systemId };
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, UNMARSHAL_FROM_STRING_FAILED, args));
+        exception.setInternalException(nestedException);
+        exception.setErrorCode(PLATFORM_NOT_SUPPORTED_WITH_JSON_MEDIA_TYPE);
+        return exception;
+    } 
+    
+    public static XMLMarshalException missingIDForIDRef(String classname, Object[] primaryKey) {   
+        String id = "";
+        for(int i=0;i<primaryKey.length; i++){
+            id += primaryKey[i];
+            if(i < primaryKey.length -1){
+                id += ", ";
+            }
+        }
+        Object[] args = {classname, id};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, MISSING_ID_FOR_IDREF, args));        
+        exception.setErrorCode(MISSING_ID_FOR_IDREF);
+        return exception;
+    }
+    
+    public static XMLMarshalException invalidAttributeGroupName(String groupName, String className) {
+        Object[] args = {groupName, className};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, INVALID_ATTRIBUTE_GROUP_NAME, args));
+        exception.setErrorCode(INVALID_ATTRIBUTE_GROUP_NAME);
+        return exception;
+    }
+    
+    public static XMLMarshalException nullValueNotAllowed(String variableName, String className) {
+        Object[] args = {variableName, className};
+        XMLMarshalException exception = new XMLMarshalException(ExceptionMessageGenerator.buildMessage(XMLMarshalException.class, NULL_VALUE_NOT_ALLOWED_FOR_VARIABLE, args));
+        exception.setErrorCode(NULL_VALUE_NOT_ALLOWED_FOR_VARIABLE);
+        return exception;
+    }
+}
\ No newline at end of file
