@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,11 +9,24 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     10/15/2010-2.2 Guy Pelletier 
+ *       - 322008: Improve usability of additional criteria applied to queries at the session/EM
+ *     05/24/2011-2.3 Guy Pelletier 
+ *       - 345962: Join fetch query when using tenant discriminator column fails.
+ *     06/30/2011-2.3.1 Guy Pelletier 
+ *       - 341940: Add disable/enable allowing native queries
+ *     02/08/2012-2.4 Guy Pelletier 
+ *       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls 
+ *     02/06/2013-2.5 Guy Pelletier 
+ *       - 382503: Use of @ConstructorResult with createNativeQuery(sqlString, resultSetMapping) results in NullPointerException
  ******************************************************************************/  
 package org.eclipse.persistence.exceptions.i18n;
 
 import java.util.ListResourceBundle;
 
+import org.eclipse.persistence.config.PersistenceUnitProperties;
+import org.eclipse.persistence.config.QueryHints;
+
 /**
  * INTERNAL: English ResourceBundle for QueryException messages.
  *
@@ -148,7 +161,7 @@ public class QueryExceptionResource extends ListResourceBundle {
        { "6142", "The value {1} supplied to the query hint {0} navigated an illegal relationship.  The relationship {2} is not a OneToOne or a OneToMany relationship."},
        { "6143", "The value {1} supplied to the query hint {0} navigated a non-existant relationship.  The relationship {2} does not exist."},
        { "6144", "The value {1} supplied to the query hint {0} did not contain enough tokens.  The join must start with the identification variable of the query.  For instance, in the query 'SELECT x from X x', to refer to a 'y' belonging to 'x', you should use the hint 'x.y'."},
-       { "6145", "Count distinct on an outer joined class [{0}] having a composite primary key is not supported. Descriptor [{1}] "},
+       { "6145", "Count distinct on a composite primary key class [{0}] is not supported. Descriptor [{1}] "},
        { "6146", "The value {1} supplied to the query hint {0} is not a valid value, valid values are Integer or Strings that can be parsed to int values."},
        { "6147", "The expression {0} is not valid for partial attribute reading." },
        { "6148", "Adding {0} to PLSQLStoredProcedureCall is not supported." },
@@ -169,10 +182,23 @@ public class QueryExceptionResource extends ListResourceBundle {
        { "6163", "index() requires QueryKeyExpression, cannot be applied to [{0}]"},
        { "6164", "index() requires QueryKeyExpression with CollectionMapping that has non-null list order column. [{1}] in does not meet this condition in [{0}]"},
        { "6165", "Batch fetch using IN requires singleton primary key."},
-       { "6166", "An attempt was made to cast outside of an inheritance hierarchy.  [{0}] does not appear in the class heirarchy for [{1}]. Note: This exception could also indicate you our casting on a Table-Per-Class inheritance relationship which is not supported for casting."},
+       { "6166", "An attempt was made to cast outside of an inheritance hierarchy.  [{0}] does not appear in the class heirarchy for [{1}]. Note: This exception could also indicate you are casting on a Table-Per-Class inheritance relationship which is not supported for casting."},
        { "6167", "A cast has been invoked on an expression that does not use inheritance: [{0}]."},
        { "6168", "Query failed to prepare, unexpected error occurred: [{0}]."},
-       { "6169", "A mapping was configured to use IN batch fetching, but the original query was not configured to use IN batch fetching and must be: [{0}]."}
+       { "6169", "A mapping was configured to use IN batch fetching, but the original query was not configured to use IN batch fetching and must be: [{0}]."},
+       { "6171", "Partitioning not support for the session type [{0}]. Only ServerSession and ClientSession are supported."},
+       { "6172", "Missing connection pool for partitioning [{0}]."},
+       { "6173", "Connection pool [{0}] failed to fail-over, all servers are dead."},
+       { "6174", "No value was provided for the session property [{0}]. This exception is possible when using additional criteria or tenant discriminator columns without specifying the associated contextual property. These properties must be set through Entity Manager, Entity Manager Factory or persistence unit properties. If using native EclipseLink, these properties should be set directly on the session."},
+       { "6175", "Native SQL queries have been disabled. This is done either by setting the persistence unit property '" + PersistenceUnitProperties.ALLOW_NATIVE_SQL_QUERIES + "' to false or having at least one multitenant entity defined in your persistence unit. Check your persistence unit specification. To allow native sql queries, set this property to true. Alternatively, individual queries may bypass this setting by setting the '" + QueryHints.ALLOW_NATIVE_SQL_QUERY + "' query hint to true."},
+       { "6176", "An exception was thrown while initializing the constructor from the class [{0}]:  [{1}]"},
+       { "6177", "The column result [{0}] was not found in the results of the query."},
+       { "6178", "isResultSetAccessOptimizedQuery set to true conflicts with other query settings."},
+       { "6179", "Failed to deserialize sopObject from [{0}] in [{1}]"},
+       { "6180", "serialized sopObject is not found in [{0}] in [{1}]"},
+       { "6181", "sopObject has a wrong version [{0}] in [{1}] in [{2}]"},
+       { "6182", "sopObject has a wrong primary key [{0}] in [{1}] in [{2}]"}
+       
     };
 
     /**
