@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -20,12 +20,11 @@ import org.eclipse.persistence.descriptors.changetracking.*;
  * IndirectMap allows a domain class to take advantage of TopLink indirection
  * without having to declare its instance variable as a ValueHolderInterface.
  * <p>To use an IndirectMap:<ul>
- * <li> Declare the appropriate instance variable with type Hashtable (jdk1.1)
- * or Map (jdk1.2).
+ * <li> Declare the appropriate instance variable with type Map or Hashtable
  * <li> Send the message #useTransparentMap(String) to the appropriate
  * CollectionMapping.
  * </ul>
- * TopLink will place an
+ * EclipseLink will place an
  * IndirectMap in the instance variable when the containing domain object is read from
  * the datatabase. With the first message sent to the IndirectMap, the contents
  * are fetched from the database and normal Hashtable/Map behavior is resumed.
@@ -107,7 +106,11 @@ public class IndirectMap extends Hashtable implements CollectionChangeTracker, I
      * Return the freshly-built delegate.
      */
     protected Hashtable buildDelegate() {
-        return (Hashtable)getValueHolder().getValue();
+        Hashtable value = (Hashtable)getValueHolder().getValue();
+        if (value == null) {
+            value = new Hashtable(this.initialCapacity, this.loadFactor);
+        }
+        return value;
     }
 
     /**
@@ -126,6 +129,15 @@ public class IndirectMap extends Hashtable implements CollectionChangeTracker, I
         }
     }
 
+    
+    /**
+     * INTERNAL:
+     * clear any changes that have been deferred to instantiation.
+     * Indirect collections with change tracking avoid instantiation on add/remove.
+     */
+    public void clearDeferredChanges(){
+    }
+    
     /**
      * @see java.util.Hashtable#clone()
      * This will result in a database query if necessary.
@@ -146,6 +158,7 @@ public class IndirectMap extends Hashtable implements CollectionChangeTracker, I
     public synchronized Object clone() {
         IndirectMap result = (IndirectMap)super.clone();
         result.delegate = (Hashtable)this.getDelegate().clone();
+        result.valueHolder = new ValueHolder(result.delegate);
         result.attributeName = null;
         result.changeListener = null;
         return result;
@@ -564,7 +577,7 @@ public class IndirectMap extends Hashtable implements CollectionChangeTracker, I
      */
     protected void raiseAddChangeEvent(Object key, Object value) {
         if (hasTrackedPropertyChangeListener()) {
-            _persistence_getPropertyChangeListener().propertyChange(new MapChangeEvent(this, getTrackedAttributeName(), this, key, value, CollectionChangeEvent.ADD));
+            _persistence_getPropertyChangeListener().propertyChange(new MapChangeEvent(this, getTrackedAttributeName(), this, key, value, CollectionChangeEvent.ADD, true));
         }
         // this is where relationship maintenance would go
     }
@@ -574,7 +587,7 @@ public class IndirectMap extends Hashtable implements CollectionChangeTracker, I
      */
     protected void raiseRemoveChangeEvent(Object key, Object value) {
         if (hasTrackedPropertyChangeListener()) {
-            _persistence_getPropertyChangeListener().propertyChange(new MapChangeEvent(this, getTrackedAttributeName(), this, key, value, CollectionChangeEvent.REMOVE));
+            _persistence_getPropertyChangeListener().propertyChange(new MapChangeEvent(this, getTrackedAttributeName(), this, key, value, CollectionChangeEvent.REMOVE, true));
         }
         // this is where relationship maintenance would go
     }
@@ -622,7 +635,15 @@ public class IndirectMap extends Hashtable implements CollectionChangeTracker, I
     public int size() {
         return this.getDelegate().size();
     }
-              
+    
+    /**
+     * INTERNAL
+     * Set whether this collection should attempt do deal with adds and removes without retrieving the 
+     * collection from the dB
+     */
+    public void setUseLazyInstantiation(boolean useLazyInstantiation){
+    }
+    
     /**
      * INTERNAL:
      * Return the elements that have been removed before instantiation.
