@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -25,6 +25,7 @@ import org.eclipse.persistence.internal.indirection.*;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
 import org.eclipse.persistence.internal.security.PrivilegedGetMethod;
 import org.eclipse.persistence.internal.security.PrivilegedMethodInvoker;
+import org.eclipse.persistence.mappings.DatabaseMapping;
 
 /**
  * IndirectSet is an example implementation of the Set protocol that
@@ -99,6 +100,14 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
     /** Store load factor for lazy init. */
     protected float loadFactor = 0.75f;
     
+    /**
+     * This value is used to determine if we should attempt to do adds and removes from the list without
+     * actually instantiating the list from the database. By default, this is set to false.  When set to
+     * true, adding duplicate elements to the set will result in the element being added when the transaction
+     * is committed.
+     */
+    private boolean useLazyInstantiation = false;
+    
     /**
      * Construct an empty IndirectSet.
      */
@@ -145,7 +154,15 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
         this.delegate = null;
         this.valueHolder = new ValueHolder(new HashSet(c));
     }
-
+    
+    protected boolean isRelationshipMaintenanceRequired() {
+        if (this.valueHolder instanceof UnitOfWorkQueryValueHolder) {
+            DatabaseMapping mapping = ((UnitOfWorkQueryValueHolder)this.valueHolder).getMapping();
+            return (mapping != null) && (mapping.getRelationshipPartner() != null);
+        }
+        return false;
+    }
+    
     /**
      * @see java.util.Set#add(java.lang.Object)
      */
@@ -155,7 +172,7 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
         if (shouldAvoidInstantiation()) {
             if (hasRemovedElements() && getRemovedElements().contains(element)) {
                 getRemovedElements().remove(element);
-            } else if (getAddedElements().contains(element)) {
+            } else if (isRelationshipMaintenanceRequired() && getAddedElements().contains(element)) {
                 // Must avoid recursion for relationship maintenance.                
                 return false;
             } else {
@@ -164,7 +181,9 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
         } else {
             added = getDelegate().add(element);
         }
-        raiseAddChangeEvent(element);
+        if (added) {
+            raiseAddChangeEvent(element);
+        }
         return added;
     }
 
@@ -190,6 +209,9 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
      */
     protected Set buildDelegate() {
         Set delegate = (Set)getValueHolder().getValue();
+        if (delegate == null) {
+            delegate = new HashSet(this.initialCapacity, this.loadFactor);
+        }
         // This can either be another indirect set or a HashSet.
         // It can be another indirect list because the mapping's query uses the same container policy.
         // Unwrap any redundent indirection layers, which can cause issues and impact performance.
@@ -217,17 +239,26 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
      */
     public void clear() {
         if (hasBeenRegistered() || hasTrackedPropertyChangeListener()) {
-            Iterator objects = this.iterator();
+            Iterator objects = iterator();
             while (objects.hasNext()) {
-                Object o = objects.next();
+                objects.next();
                 objects.remove();
-                this.raiseRemoveChangeEvent(o);
             }
-        } else {
-            this.getDelegate().clear();
+            // clear delegate in case it's still not empty, see bug 338393 
         }
+        getDelegate().clear();
     }
-
+    
+    /**
+     * INTERNAL:
+     * clear any changes that have been deferred to instantiation.
+     * Indirect collections with change tracking avoid instantiation on add/remove.
+     */
+    public void clearDeferredChanges(){
+        addedElements = null;
+        removedElements = null;
+    }
+    
     /**
      * @see java.lang.Object#clone()
      * This will result in a database query if necessary.
@@ -494,6 +525,15 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
         this.valueHolder = valueHolder;
     }
 
+    /**
+     * INTERNAL
+     * Set whether this collection should attempt do deal with adds and removes without retrieving the 
+     * collection from the dB
+     */
+    public void setUseLazyInstantiation(boolean useLazyInstantiation){
+        this.useLazyInstantiation = useLazyInstantiation;
+    }
+    
     /**
      * @see java.util.Set#size()
      */
@@ -501,6 +541,15 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
         return this.getDelegate().size();
     }
 
+    /**
+     * Return whether this collection should attempt do deal with adds and removes without retrieving the 
+     * collection from the dB
+     * @return
+     */
+    protected boolean shouldUseLazyInstantiation(){
+        return useLazyInstantiation;
+    }
+    
     /**
      * @see java.util.Set#toArray()
      */
@@ -538,9 +587,9 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
      */
     protected void raiseAddChangeEvent(Object element) {
         if (hasTrackedPropertyChangeListener()) {
-            _persistence_getPropertyChangeListener().propertyChange(new CollectionChangeEvent(this, getTrackedAttributeName(), this, element, CollectionChangeEvent.ADD));
+            _persistence_getPropertyChangeListener().propertyChange(new CollectionChangeEvent(this, getTrackedAttributeName(), this, element, CollectionChangeEvent.ADD, true));
         }
-        if (hasBeenRegistered()) {
+        if (isRelationshipMaintenanceRequired()) {
             ((UnitOfWorkQueryValueHolder)getValueHolder()).updateForeignReferenceSet(element, null);
         }
     }
@@ -550,9 +599,9 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
      */
     protected void raiseRemoveChangeEvent(Object element) {
         if (hasTrackedPropertyChangeListener()) {
-            _persistence_getPropertyChangeListener().propertyChange(new CollectionChangeEvent(this, getTrackedAttributeName(), this, element, CollectionChangeEvent.REMOVE));
+            _persistence_getPropertyChangeListener().propertyChange(new CollectionChangeEvent(this, getTrackedAttributeName(), this, element, CollectionChangeEvent.REMOVE, true));
         }
-        if (hasBeenRegistered()) {
+        if (isRelationshipMaintenanceRequired()) {
             ((UnitOfWorkQueryValueHolder)getValueHolder()).updateForeignReferenceRemove(element);
         }
     }
@@ -650,6 +699,6 @@ public class IndirectSet implements CollectionChangeTracker, Set, IndirectCollec
      * Current instantiation is avoided is using change tracking.
      */
     protected boolean shouldAvoidInstantiation() {
-        return (!isInstantiated()) && (_persistence_getPropertyChangeListener() instanceof AttributeChangeListener) && ((WeavedAttributeValueHolderInterface)getValueHolder()).shouldAllowInstantiationDeferral();
+        return (!isInstantiated()) && (shouldUseLazyInstantiation()) && (_persistence_getPropertyChangeListener() instanceof AttributeChangeListener) && ((WeavedAttributeValueHolderInterface)getValueHolder()).shouldAllowInstantiationDeferral();
     }
 }
