@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,6 +12,8 @@
  ******************************************************************************/  
 package org.eclipse.persistence.internal.databaseaccess;
 
+import java.io.Serializable;
+
 import org.eclipse.persistence.descriptors.DescriptorQueryManager;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 
@@ -26,7 +28,7 @@ import org.eclipse.persistence.internal.sessions.AbstractSession;
  *
  *    @since OracleAS TopLink 10<i>g</i> (9.0.4)
  */
-public abstract class BatchWritingMechanism {
+public abstract class BatchWritingMechanism implements Cloneable, Serializable {
 
     /**
      * This member variable stores the reference to the DatabaseAccessor that is
@@ -47,6 +49,17 @@ public abstract class BatchWritingMechanism {
     //bug 4241441: increments with each addBatch call.  Used to compare against value returned from driver for 
     //  optimistic locking
     protected int statementCount;
+    
+    /** Allow for the batch size to be set as many database have strict limits. **/
+    protected int maxBatchSize;
+
+    public int getMaxBatchSize() {
+        return maxBatchSize;
+    }
+
+    public void setMaxBatchSize(int maxBatchSize) {
+        this.maxBatchSize = maxBatchSize;
+    }
 
     /**
      * INTERNAL:
@@ -84,7 +97,7 @@ public abstract class BatchWritingMechanism {
      * INTERNAL:
      * Sets the accessor that this mechanism will use
      */
-    public void setAccessor(DatabaseAccessor accessor) {
+    public void setAccessor(DatabaseAccessor accessor, AbstractSession session) {
         databaseAccessor = accessor;
     }
 	
@@ -95,20 +108,40 @@ public abstract class BatchWritingMechanism {
      * being batched.  This call may result in the Mechanism executing the batched statements and
      * possibly, switching out the mechanisms
      */
-    abstract public void appendCall(AbstractSession session, DatabaseCall call);
+    public abstract void appendCall(AbstractSession session, DatabaseCall call);
 
     /**
      * INTERNAL:
      * This method is used to clear the batched statements without the need to execute the statements first
      * This is used in the case of rollback.
      */
-    abstract public void clear();
+    public abstract void clear();
 
     /**
      * INTERNAL:
      * This method is used by the DatabaseAccessor to clear the batched statements in the
      * case that a non batchable statement is being execute
      */
-    abstract public void executeBatchedStatements(AbstractSession session);
+    public abstract void executeBatchedStatements(AbstractSession session);
+    
+    /**
+     * INTERNAL:
+     * The mechanism will be cloned to be set into each accessor.
+     */
+    public BatchWritingMechanism clone() {
+        try {
+            return (BatchWritingMechanism)super.clone();
+        } catch (CloneNotSupportedException notPossible) {
+            throw new InternalError(notPossible.toString());
+        }
+    }
+
+    /**
+     * INTERNAL:
+     * Allow initialization with the session after login.
+     */
+    public void initialize(AbstractSession session) {
+        
+    }
 
 }
