@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -24,6 +24,7 @@ import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.logging.SessionLog;
 import org.eclipse.persistence.sessions.SessionProfiler;
+import org.eclipse.persistence.sessions.server.ConnectionPool;
 
 /**
  * INTERNAL:
@@ -106,6 +107,14 @@ public abstract class DatasourceAccessor implements Accessor {
      */
     protected transient SequencingCallback sequencingCallback;
     
+    /**
+     * This attribute is used to track failures on an accessor that may be communication based.
+     * If a failure is detected executing a call with a query timeout this flag is set.  If an error happens
+     * twice in a row on the same accessor then that accessor will be checked for a comm error.  If there is no
+     * query timeout then the flag is not set and the accessor will be checked immediately.
+     */
+    protected boolean possibleFailure;
+
     /**
      *  Used only in externalConnectionPooling case. Indicates which session is currently using connection.
      *  Allows to rise appropriate session's event when connection is already/still alive.
@@ -138,6 +147,8 @@ public abstract class DatasourceAccessor implements Accessor {
      */
     protected ConnectionCustomizer customizer;
 
+    protected ConnectionPool pool;
+    
     /**
      *    Default Constructor.
      */
@@ -209,6 +220,14 @@ public abstract class DatasourceAccessor implements Accessor {
         return this.isValid;
     }
     
+    public boolean isPossibleFailure() {
+        return possibleFailure;
+    }
+
+    public void setPossibleFailure(boolean possibleFailure) {
+        this.possibleFailure = possibleFailure;
+    }
+
     /**
      * Return true if some external connection pool is in use.
      */
@@ -231,13 +250,13 @@ public abstract class DatasourceAccessor implements Accessor {
         session.log(SessionLog.FINER, SessionLog.TRANSACTION, "begin_transaction", (Object[])null, this);
 
         try {
-            session.startOperationProfile(SessionProfiler.TRANSACTION);
+            session.startOperationProfile(SessionProfiler.Transaction);
             incrementCallCount(session);
             basicBeginTransaction(session);
             this.isInTransaction = true;
         } finally {
             decrementCallCount();
-            session.endOperationProfile(SessionProfiler.TRANSACTION);
+            session.endOperationProfile(SessionProfiler.Transaction);
         }
     }
 
@@ -300,7 +319,7 @@ public abstract class DatasourceAccessor implements Accessor {
                 // If ExternalConnectionPooling is used, the connection can be re-established.
                 if (this.usesExternalConnectionPooling) {
                     reconnect(session);
-                    session.postConnectExternalConnection(this);
+                    session.postAcquireConnection(this);
                     currentSession = session;
                 } else {
                     throw DatabaseException.databaseAccessorNotConnected();
@@ -330,7 +349,7 @@ public abstract class DatasourceAccessor implements Accessor {
                 customizer.customize();
             }
         }catch (DatabaseException ex){
-            //Set the accessor to ensure the retry code has an oportunity to retry.
+            //Set the accessor to ensure the retry code has an opportunity to retry.
             ex.setAccessor(this);
             throw ex;
         }
@@ -381,7 +400,7 @@ public abstract class DatasourceAccessor implements Accessor {
         session.log(SessionLog.FINER, SessionLog.TRANSACTION, "commit_transaction", (Object[])null, this);
 
         try {
-            session.startOperationProfile(SessionProfiler.TRANSACTION);
+            session.startOperationProfile(SessionProfiler.Transaction);
             incrementCallCount(session);
             basicCommitTransaction(session);
 
@@ -392,7 +411,7 @@ public abstract class DatasourceAccessor implements Accessor {
         } finally {
             sequencingCallback = null;
             decrementCallCount();
-            session.endOperationProfile(SessionProfiler.TRANSACTION);
+            session.endOperationProfile(SessionProfiler.Transaction);
         }
     }
 
@@ -401,8 +420,8 @@ public abstract class DatasourceAccessor implements Accessor {
      * Catch exceptions and re-throw as EclipseLink exceptions.
      */
     public void connect(Login login, AbstractSession session) throws DatabaseException {
-        session.startOperationProfile(SessionProfiler.CONNECT);
-        session.incrementProfile(SessionProfiler.TlConnects);
+        session.startOperationProfile(SessionProfiler.ConnectionManagement);
+        session.incrementProfile(SessionProfiler.Connects);
 
         try {
             if (session.shouldLog(SessionLog.CONFIG, SessionLog.CONNECTION)) {// Avoid printing if no logging required.
@@ -427,7 +446,7 @@ public abstract class DatasourceAccessor implements Accessor {
                 decrementCallCount();
             }
         } finally {
-            session.endOperationProfile(SessionProfiler.CONNECT);
+            session.endOperationProfile(SessionProfiler.ConnectionManagement);
         }
     }
 
@@ -471,13 +490,16 @@ public abstract class DatasourceAccessor implements Accessor {
         if (this.datasourceConnection == null) {
             return;
         }
-        session.incrementProfile(SessionProfiler.TlDisconnects);
-        session.startOperationProfile(SessionProfiler.CONNECT);
-        releaseCustomizer();
-        closeDatasourceConnection();
-        this.datasourceConnection = null;
-        this.isInTransaction = true;
-        session.endOperationProfile(SessionProfiler.CONNECT);
+        session.incrementProfile(SessionProfiler.Disconnects);
+        session.startOperationProfile(SessionProfiler.ConnectionManagement);
+        try {
+            releaseCustomizer();
+            closeDatasourceConnection();
+            this.datasourceConnection = null;
+            this.isInTransaction = true;
+        } finally {
+            session.endOperationProfile(SessionProfiler.ConnectionManagement);
+        }
     }
 
     /**
@@ -490,7 +512,7 @@ public abstract class DatasourceAccessor implements Accessor {
             if (this.datasourceConnection != null) {
                 if (isDatasourceConnected()) {
                     if(currentSession != null) {
-                        currentSession.preDisconnectExternalConnection(this);
+                        currentSession.preReleaseConnection(this);
                     }
                     if(customizer != null && customizer.isActive()) {
                         customizer.clear();
@@ -554,9 +576,12 @@ public abstract class DatasourceAccessor implements Accessor {
      */
     protected void reconnect(AbstractSession session) throws DatabaseException {
         session.log(SessionLog.FINEST, SessionLog.CONNECTION, "reconnecting_to_external_connection_pool");
-        session.startOperationProfile(SessionProfiler.CONNECT);
-        connectInternal(this.login, session);
-        session.endOperationProfile(SessionProfiler.CONNECT);
+        session.startOperationProfile(SessionProfiler.ConnectionManagement);
+        try {
+            connectInternal(this.login, session);
+        } finally {
+            session.endOperationProfile(SessionProfiler.ConnectionManagement);
+        }
     }
 
     /**
@@ -658,14 +683,14 @@ public abstract class DatasourceAccessor implements Accessor {
         session.log(SessionLog.FINER, SessionLog.TRANSACTION, "rollback_transaction", (Object[])null, this);
 
         try {
-            session.startOperationProfile(SessionProfiler.TRANSACTION);
+            session.startOperationProfile(SessionProfiler.Transaction);
             incrementCallCount(session);
             basicRollbackTransaction(session);
         } finally {
             this.isInTransaction = false;
             sequencingCallback = null;
             decrementCallCount();
-            session.endOperationProfile(SessionProfiler.TRANSACTION);
+            session.endOperationProfile(SessionProfiler.Transaction);
         }
     }
 
@@ -872,5 +897,19 @@ public abstract class DatasourceAccessor implements Accessor {
                createCustomizer(customizerSession);
            }
        }
-   }  
+   }
+
+   /**
+    * Return the associated connection pool this connection was obtained from.
+    */
+   public ConnectionPool getPool() {
+       return pool;
+   }
+
+   /**
+    * Set the associated connection pool this connection was obtained from.
+    */
+   public void setPool(ConnectionPool pool) {
+       this.pool = pool;
+   }
 }
