@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
  * which accompanies this distribution.
@@ -14,12 +14,21 @@
 package org.eclipse.persistence.internal.dbws;
 
 // Javase imports
+import static javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI;
+import static javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI;
+import static javax.xml.soap.SOAPConstants.SOAP_1_2_PROTOCOL;
+import static javax.xml.soap.SOAPConstants.URI_NS_SOAP_1_1_ENVELOPE;
+import static javax.xml.soap.SOAPConstants.URI_NS_SOAP_1_2_ENVELOPE;
+import static org.eclipse.persistence.internal.xr.Util.SERVICE_NAMESPACE_PREFIX;
+import static org.eclipse.persistence.oxm.XMLConstants.BASE_64_BINARY;
+import static org.eclipse.persistence.oxm.XMLConstants.SCHEMA_PREFIX;
+import static org.eclipse.persistence.oxm.XMLConstants.SCHEMA_INSTANCE_PREFIX;
+
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
 
-// Java extension imports
 import javax.activation.DataHandler;
 import javax.xml.namespace.QName;
 import javax.xml.soap.AttachmentPart;
@@ -27,12 +36,7 @@ import javax.xml.soap.MessageFactory;
 import javax.xml.soap.SOAPBody;
 import javax.xml.soap.SOAPException;
 import javax.xml.soap.SOAPMessage;
-import static javax.xml.soap.SOAPConstants.SOAP_1_2_PROTOCOL;
-import static javax.xml.soap.SOAPConstants.URI_NS_SOAP_1_1_ENVELOPE;
-import static javax.xml.soap.SOAPConstants.URI_NS_SOAP_1_2_ENVELOPE;
-import static javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI;
 
-// EclipseLink imports
 import org.eclipse.persistence.internal.oxm.schema.model.ComplexType;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.sessions.DatabaseSessionImpl;
@@ -48,13 +52,13 @@ import org.eclipse.persistence.oxm.mappings.XMLBinaryDataMapping;
 import org.eclipse.persistence.oxm.mappings.XMLCompositeCollectionMapping;
 import org.eclipse.persistence.oxm.mappings.XMLCompositeObjectMapping;
 import org.eclipse.persistence.oxm.mappings.XMLDirectMapping;
-import static org.eclipse.persistence.internal.xr.Util.SERVICE_NAMESPACE_PREFIX;
-import static org.eclipse.persistence.oxm.XMLConstants.BASE_64_BINARY;
 
 public class SOAPResponseWriter {
 
     protected DBWSAdapter dbwsAdapter;
     protected Map<String, XMLDescriptor> resultDescriptors = new HashMap<String, XMLDescriptor>();
+    public static final QName RECEIVER_QNAME = new QName(URI_NS_SOAP_1_2_ENVELOPE, "Receiver");
+    public static final QName SERVER_QNAME = new QName(URI_NS_SOAP_1_1_ENVELOPE, "Server");
 
     public SOAPResponseWriter(DBWSAdapter dbwsAdapter) {
         this.dbwsAdapter = dbwsAdapter;
@@ -97,7 +101,7 @@ public class SOAPResponseWriter {
                     QName type = queryOperation.getResult().getType();
                     String localElement = type.getLocalPart();
                     // look for top-level complex types
-                    Set<Map.Entry<String, ComplexType>> entrySet = 
+                    Set<Map.Entry<String, ComplexType>> entrySet =
                         dbwsAdapter.getSchema().getTopLevelComplexTypes().entrySet();
                     for (Map.Entry<String, ComplexType> me : entrySet) {
                         if (me.getValue().getName().equals(type.getLocalPart())) {
@@ -163,25 +167,24 @@ public class SOAPResponseWriter {
         }
     }
 
-    public SOAPMessage generateResponse(Operation op, boolean useSOAP12, Exception e)
-    throws SOAPException {
+    public SOAPMessage generateResponse(Operation op, boolean useSOAP12, Exception e) throws SOAPException {
         MessageFactory messageFactory = null;
         if (useSOAP12) {
             messageFactory = MessageFactory.newInstance(SOAP_1_2_PROTOCOL);
-        }
-        else {
+        } else {
             messageFactory = MessageFactory.newInstance();
         }
         SOAPMessage message = messageFactory.createMessage();
+        message.getSOAPPart().getEnvelope().addNamespaceDeclaration(SCHEMA_PREFIX, W3C_XML_SCHEMA_NS_URI);
+        message.getSOAPPart().getEnvelope().addNamespaceDeclaration(SCHEMA_INSTANCE_PREFIX, W3C_XML_SCHEMA_INSTANCE_NS_URI);
         SOAPBody body = message.getSOAPPart().getEnvelope().getBody();
-        QName serverQName = null;
+        QName faultCodeQName = null;
         if (useSOAP12) {
-            serverQName = new QName(URI_NS_SOAP_1_2_ENVELOPE, "Server");
+            faultCodeQName = RECEIVER_QNAME;
+        } else {
+            faultCodeQName = SERVER_QNAME;
         }
-        else {
-            serverQName = new QName(URI_NS_SOAP_1_1_ENVELOPE, "Server");
-        }
-        body.addFault(serverQName, op.getName() + " failed: " + e);
+        body.addFault(faultCodeQName, op.getName() + " failed: " + e);
         return message;
     }
 
@@ -194,6 +197,8 @@ public class SOAPResponseWriter {
             messageFactory = MessageFactory.newInstance();
         }
         SOAPMessage message = messageFactory.createMessage();
+        message.getSOAPPart().getEnvelope().addNamespaceDeclaration(SCHEMA_PREFIX, W3C_XML_SCHEMA_NS_URI);
+        message.getSOAPPart().getEnvelope().addNamespaceDeclaration(SCHEMA_INSTANCE_PREFIX, W3C_XML_SCHEMA_INSTANCE_NS_URI);
         SOAPBody body = message.getSOAPPart().getEnvelope().getBody();
 
         XMLDescriptor descriptor = resultDescriptors.get(op.getName());
