@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,22 +12,24 @@
  ******************************************************************************/  
 package org.eclipse.persistence.internal.indirection;
 
+import java.rmi.server.ObjID;
 import java.util.*;
 
 import org.eclipse.persistence.mappings.DatabaseMapping.WriteType;
 import org.eclipse.persistence.queries.*;
-import org.eclipse.persistence.sessions.remote.RemoteSession;
+import org.eclipse.persistence.sessions.remote.DistributedSession;
 import org.eclipse.persistence.indirection.*;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.internal.descriptors.*;
 import org.eclipse.persistence.internal.helper.*;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.sessions.remote.RemoteSessionController;
 import org.eclipse.persistence.internal.sessions.remote.RemoteUnitOfWork;
 import org.eclipse.persistence.internal.sessions.remote.RemoteValueHolder;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
+import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.sessions.MergeManager;
 import org.eclipse.persistence.internal.sessions.UnitOfWorkImpl;
-import org.eclipse.persistence.internal.sessions.AbstractSession;
 
 /**
  * <h2>Purpose</h2>:
@@ -88,22 +90,22 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      *  directly from a row as opposed to building the original from the
      *  row, putting it in the shared cache, and then cloning the original.
      */
-    public Object cloneAttribute(Object attributeValue, Object original, Object clone, UnitOfWorkImpl unitOfWork, boolean buildDirectlyFromRow) {
+    public Object cloneAttribute(Object attributeValue, Object original, CacheKey cacheKey, Object clone, Integer refreshCascade, AbstractSession cloningSession, boolean buildDirectlyFromRow) {
         ValueHolderInterface valueHolder = (ValueHolderInterface) attributeValue;
         ValueHolderInterface result;
         
-        if (!buildDirectlyFromRow && unitOfWork.isOriginalNewObject(original)) {
+        if (!buildDirectlyFromRow && cloningSession.isUnitOfWork() && ((UnitOfWorkImpl)cloningSession).isOriginalNewObject(original)) {
             // CR#3156435 Throw a meaningful exception if a serialized/dead value holder is detected.
             // This can occur if an existing serialized object is attempt to be registered as new.
             if ((valueHolder instanceof DatabaseValueHolder)
                     && (! ((DatabaseValueHolder) valueHolder).isInstantiated())
                     && (((DatabaseValueHolder) valueHolder).getSession() == null)
                     && (! ((DatabaseValueHolder) valueHolder).isSerializedRemoteUnitOfWorkValueHolder())) {
-                throw DescriptorException.attemptToRegisterDeadIndirection(original, getMapping());
+                throw DescriptorException.attemptToRegisterDeadIndirection(original, this.mapping);
             }
-            if (this.getMapping().getRelationshipPartner() == null) {
+            if (this.mapping.getRelationshipPartner() == null) {
                 result = new ValueHolder();
-                result.setValue(this.getMapping().buildCloneForPartObject(valueHolder.getValue(), original, clone, unitOfWork, false));
+                result.setValue(this.mapping.buildCloneForPartObject(valueHolder.getValue(), original, null, clone, cloningSession, refreshCascade, false, false));
             } else {
                 //if I have a relationship partner trigger the indirection so that the value will be inserted
                 // because of this call the entire tree should be recursively cloned
@@ -111,9 +113,9 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
                 if (valueHolder instanceof DatabaseValueHolder) {
                     row = ((DatabaseValueHolder)valueHolder).getRow();
                 }
-                result = this.getMapping().createUnitOfWorkValueHolder(valueHolder, original, clone, row, unitOfWork, buildDirectlyFromRow);
+                result = this.mapping.createCloneValueHolder(valueHolder, original, clone, row, cloningSession, buildDirectlyFromRow);
 
-                Object newObject = this.getMapping().buildCloneForPartObject(valueHolder.getValue(), original, clone, unitOfWork, false);
+                Object newObject = this.mapping.buildCloneForPartObject(valueHolder.getValue(), original, cacheKey, clone, cloningSession, refreshCascade, false, false);
                 ((UnitOfWorkValueHolder)result).privilegedSetValue(newObject);
                 ((UnitOfWorkValueHolder)result).setInstantiated();
             }
@@ -122,7 +124,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
             if (valueHolder instanceof DatabaseValueHolder) {
                 row = ((DatabaseValueHolder)valueHolder).getRow();
             }
-            result = this.getMapping().createUnitOfWorkValueHolder(valueHolder, original, clone, row, unitOfWork, buildDirectlyFromRow);
+            result = this.mapping.createCloneValueHolder(valueHolder, original, clone, row, cloningSession, buildDirectlyFromRow);
         }
         return result;
     }
@@ -135,7 +137,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      */
     @Override
     public Object extractPrimaryKeyForReferenceObject(Object referenceObject, AbstractSession session) {
-        if (objectIsInstantiated(referenceObject)) {
+        if (objectIsEasilyInstantiated(referenceObject)) {
             return super.extractPrimaryKeyForReferenceObject(((ValueHolderInterface)referenceObject).getValue(), session);
         } else {
             return getOneToOneMapping().extractPrimaryKeysForReferenceObjectFromRow(extractReferenceRow(referenceObject));
@@ -163,15 +165,15 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      * Replace the transient attributes of the remote value holders
      * with client-side objects.
      */
-    public void fixObjectReferences(Object object, Map objectDescriptors, Map processedObjects, ObjectLevelReadQuery query, RemoteSession session) {
-        Object attributeValue = this.getMapping().getAttributeValueFromObject(object);
+    public void fixObjectReferences(Object object, Map objectDescriptors, Map processedObjects, ObjectLevelReadQuery query, DistributedSession session) {
+        Object attributeValue = this.mapping.getAttributeValueFromObject(object);
         //bug 4147755 if it is not a Remote Valueholder then treat as if there was no VH...
         if (attributeValue instanceof RemoteValueHolder){
-            RemoteValueHolder rvh = (RemoteValueHolder)this.getMapping().getAttributeValueFromObject(object);
+            RemoteValueHolder rvh = (RemoteValueHolder)this.mapping.getAttributeValueFromObject(object);
             rvh.setSession(session);
-            rvh.setMapping(this.getMapping());
+            rvh.setMapping(this.mapping);
     
-            if ((!query.shouldMaintainCache()) && ((!query.shouldCascadeParts()) || (query.shouldCascadePrivateParts() && (!getMapping().isPrivateOwned())))) {
+            if ((!query.shouldMaintainCache()) && ((!query.shouldCascadeParts()) || (query.shouldCascadePrivateParts() && (!this.mapping.isPrivateOwned())))) {
                 rvh.setQuery(null);
             } else {
                 rvh.setQuery(query);
@@ -180,7 +182,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
             // set to uninstantiated since no objects are serialized past remote value holders
             rvh.setUninstantiated();
         }else{
-            this.getMapping().fixRealObjectReferences(object, objectDescriptors, processedObjects, query, session);        
+            this.mapping.fixRealObjectReferences(object, objectDescriptors, processedObjects, query, session);        
         }
     }
 
@@ -189,6 +191,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      * Return the original indirection object for a unit of work indirection object.
      * This is used when building a new object from the unit of work when the original fell out of the cache.
      */
+    @Override
     public Object getOriginalIndirectionObject(Object unitOfWorkIndirectionObject, AbstractSession session) {
         return this.getOriginalValueHolder(unitOfWorkIndirectionObject, session);
     }
@@ -197,6 +200,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      * INTERNAL:
      *    Return the original indirection object for a unit of work indirection object.
      */
+    @Override
     public Object getOriginalIndirectionObjectForMerge(Object unitOfWorkIndirectionObject, AbstractSession session) {
         DatabaseValueHolder holder = (DatabaseValueHolder)getOriginalIndirectionObject(unitOfWorkIndirectionObject, session);
         if (holder != null && holder.getSession()!= null){
@@ -210,38 +214,60 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      * underlying valueholder may be required when serializing the valueholder
      * or converting the valueHolder to another type.
      */
+    @Override
     public Object getOriginalValueHolder(Object unitOfWorkIndirectionObject, AbstractSession session) {
-        if (unitOfWorkIndirectionObject instanceof UnitOfWorkValueHolder) {
+        if ((unitOfWorkIndirectionObject instanceof UnitOfWorkValueHolder)
+                && (((UnitOfWorkValueHolder)unitOfWorkIndirectionObject).getRemoteUnitOfWork() != null)) {
             ValueHolderInterface valueHolder = ((UnitOfWorkValueHolder) unitOfWorkIndirectionObject).getWrappedValueHolder();
-            if ((valueHolder == null) && session.isRemoteUnitOfWork()) {
+            if (valueHolder == null) {
                 // For remote session the original value holder is transient,
                 // so the value must be found in the registry or created.
-                RemoteSessionController controller = ((RemoteUnitOfWork) session).getParentSessionController();
-                Object id = ((UnitOfWorkValueHolder) unitOfWorkIndirectionObject).getWrappedValueHolderRemoteID();
-                if (id == null) {
+                RemoteUnitOfWork remoteUnitOfWork = (RemoteUnitOfWork)((UnitOfWorkValueHolder)unitOfWorkIndirectionObject).getRemoteUnitOfWork();
+                RemoteSessionController controller = remoteUnitOfWork.getParentSessionController();
+                ObjID id = ((UnitOfWorkValueHolder) unitOfWorkIndirectionObject).getWrappedValueHolderRemoteID();
+                if (id != null) {
+                    // This value holder may be on the server, or the client,
+                    // on the server, the controller should exists, so can lock up in it,
+                    // on the client, the id should be enough to create a new remote value holder.
+                    if (controller != null) {
+                        valueHolder = controller.getRemoteValueHolders().get(id);
+                    } else if (session.isRemoteSession()) {
+                        valueHolder = new RemoteValueHolder(id);
+                        ((RemoteValueHolder)valueHolder).setSession(session);
+                    }
+                }
+                if (valueHolder == null) {
                     // Must build a new value holder.
                     Object object = ((UnitOfWorkValueHolder) unitOfWorkIndirectionObject).getSourceObject();
-                    AbstractRecord row = getMapping().getDescriptor().getObjectBuilder().buildRow(object, session, WriteType.UNDEFINED);
+                    AbstractRecord row = this.mapping.getDescriptor().getObjectBuilder().buildRow(object, session, WriteType.UNDEFINED);
                     ReadObjectQuery query = new ReadObjectQuery();
-                    query.setSession(((RemoteUnitOfWork) session).getParent());
-                    valueHolder = (ValueHolderInterface) getMapping().valueFromRow(row, null, query);
-                } else {
-                    valueHolder = (ValueHolderInterface) controller.getRemoteValueHolders().get(id);
+                    query.setSession(session);
+                    valueHolder = (ValueHolderInterface) this.mapping.valueFromRow(row, null, query, true);
+                }
+                return valueHolder;
+            }
+        }
+        if (unitOfWorkIndirectionObject instanceof WrappingValueHolder) {
+            ValueHolderInterface valueHolder =  ((WrappingValueHolder)unitOfWorkIndirectionObject).getWrappedValueHolder();
+            if (!session.isProtectedSession()){
+                while (valueHolder instanceof WrappingValueHolder && ((WrappingValueHolder)valueHolder).getWrappedValueHolder() != null){
+                    valueHolder = ((WrappingValueHolder)valueHolder).getWrappedValueHolder();
                 }
             }
             if ((valueHolder != null) && (valueHolder instanceof DatabaseValueHolder)) {
-                ((DatabaseValueHolder) valueHolder).releaseWrappedValueHolder();
+                ((DatabaseValueHolder) valueHolder).releaseWrappedValueHolder(session);
             }
             return valueHolder;
         } else {
             return unitOfWorkIndirectionObject;
         }
     }
+    
     /**
      * Reset the wrapper used to store the value.
      */
     public void reset(Object target) {
-        getMapping().setAttributeValueInObject(target, new ValueHolder());
+        this.mapping.setAttributeValueInObject(target, new ValueHolder());
     }
     
     /**
@@ -266,12 +292,26 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
         return remoteValueHolder.getValue();
     }
 
+    /**
+     * INTERNAL:
+     * The method validateAttributeOfInstantiatedObject(Object attributeValue) fixes the value of the attributeValue 
+     * in cases where it is null and indirection requires that it contain some specific data structure.  Return whether this will happen.
+     * This method is used to help determine if indirection has been triggered
+     * @param attributeValue
+     * @return
+     * @see validateAttributeOfInstantiatedObject(Object attributeValue)
+     */
+    @Override
+    public boolean isAttributeValueFullyBuilt(Object attributeValue){
+        return attributeValue != null;
+    }
+    
     /**
      * INTERNAL:
      *    Iterate over the specified attribute value,
      */
     public void iterateOnAttributeValue(DescriptorIterator iterator, Object attributeValue) {
-        iterator.iterateValueHolderForMapping((ValueHolderInterface)attributeValue, this.getMapping());
+        iterator.iterateValueHolderForMapping((ValueHolderInterface)attributeValue, this.mapping);
     }
 
     /**
@@ -281,10 +321,10 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      */
     public void mergeRemoteValueHolder(Object clientSideDomainObject, Object serverSideDomainObject, MergeManager mergeManager) {
         // This will always be a remote value holder coming from the server,
-        RemoteValueHolder serverValueHolder = (RemoteValueHolder)getMapping().getAttributeValueFromObject(serverSideDomainObject);
+        RemoteValueHolder serverValueHolder = (RemoteValueHolder)this.mapping.getAttributeValueFromObject(serverSideDomainObject);
         mergeClientIntoServerValueHolder(serverValueHolder, mergeManager);
 
-        getMapping().setAttributeValueInObject(clientSideDomainObject, serverValueHolder);
+        this.mapping.setAttributeValueInObject(clientSideDomainObject, serverValueHolder);
     }
 
     /**
@@ -305,6 +345,18 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
     public boolean objectIsInstantiated(Object object) {
         return ((ValueHolderInterface)object).isInstantiated();
     }
+    
+    /**
+     * INTERNAL:
+     * Return whether the specified object can be instantiated without database access.
+     */
+    public boolean objectIsEasilyInstantiated(Object object) {
+        if (object instanceof DatabaseValueHolder) {
+            return ((DatabaseValueHolder)object).isEasilyInstantiated();
+        } else {
+            return true;
+        }
+    }
 
     /**
      * INTERNAL:
@@ -312,7 +364,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      * In this case, place the value inside the target's ValueHolder.
      */
     public void setRealAttributeValueInObject(Object target, Object attributeValue) {
-        ValueHolderInterface holder = (ValueHolderInterface)this.getMapping().getAttributeValueFromObject(target);
+        ValueHolderInterface holder = (ValueHolderInterface)this.mapping.getAttributeValueFromObject(target);
         if (holder == null) {
             holder = new ValueHolder(attributeValue);
         } else {
@@ -321,6 +373,17 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
         super.setRealAttributeValueInObject(target, holder);
     }
 
+    /**
+     * INTERNAL:
+     * set the source object into QueryBasedValueHolder.
+     * Used only by transparent indirection.
+     */
+    public void setSourceObject(Object sourceObject, Object attributeValue) {        
+        if (attributeValue instanceof QueryBasedValueHolder) {
+            ((QueryBasedValueHolder)attributeValue).setSourceObject(sourceObject);
+        }
+    }
+    
     /**
      * INTERNAL:
      *    Return whether the type is appropriate for the indirection policy.
@@ -347,11 +410,11 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
             return new ValueHolder();
         }
         if (!(attributeValue instanceof ValueHolderInterface)) {
-            throw DescriptorException.valueHolderInstantiationMismatch(attributeValue, this.getMapping());
+            throw DescriptorException.valueHolderInstantiationMismatch(attributeValue, this.mapping);
         }
         return attributeValue;
     }
-
+    
     /**
      * INTERNAL:
      *    Verify that attributeType is correct for the
@@ -362,7 +425,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
     public void validateDeclaredAttributeType(Class attributeType, IntegrityChecker checker) throws DescriptorException {
         super.validateDeclaredAttributeType(attributeType, checker);
         if (!this.typeIsValid(attributeType)) {
-            checker.handleError(DescriptorException.attributeAndMappingWithIndirectionMismatch(this.getMapping()));
+            checker.handleError(DescriptorException.attributeAndMappingWithIndirectionMismatch(this.mapping));
         }
     }
 
@@ -376,7 +439,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
     public void validateGetMethodReturnType(Class returnType, IntegrityChecker checker) throws DescriptorException {
         super.validateGetMethodReturnType(returnType, checker);
         if (!this.typeIsValid(returnType)) {
-            checker.handleError(DescriptorException.returnAndMappingWithIndirectionMismatch(this.getMapping()));
+            checker.handleError(DescriptorException.returnAndMappingWithIndirectionMismatch(this.mapping));
         }
     }
 
@@ -390,7 +453,7 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
     public void validateSetMethodParameterType(Class parameterType, IntegrityChecker checker) throws DescriptorException {
         super.validateSetMethodParameterType(parameterType, checker);
         if (!this.typeIsValid(parameterType)) {
-            checker.handleError(DescriptorException.parameterAndMappingWithIndirectionMismatch(this.getMapping()));
+            checker.handleError(DescriptorException.parameterAndMappingWithIndirectionMismatch(this.mapping));
         }
     }
 
@@ -400,8 +463,8 @@ public class BasicIndirectionPolicy extends IndirectionPolicy {
      *    This value is determined by the batchQuery.
      * In this case, wrap the query in a ValueHolder for later invocation.
      */
-    public Object valueFromBatchQuery(ReadQuery batchQuery, AbstractRecord row, ObjectLevelReadQuery originalQuery) {
-        return new BatchValueHolder(batchQuery, row, this.getForeignReferenceMapping(), originalQuery);
+    public Object valueFromBatchQuery(ReadQuery batchQuery, AbstractRecord row, ObjectLevelReadQuery originalQuery, CacheKey parentCacheKey) {
+        return new BatchValueHolder(batchQuery, row, this.getForeignReferenceMapping(), originalQuery, parentCacheKey);
     }
 
     /**
