@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -15,9 +15,18 @@
  *       - 270011: JPA 2.0 MappedById support
  *     10/21/2009-2.0 Guy Pelletier 
  *       - 290567: mappedbyid support incomplete
+ *     12/17/2010-2.2 Guy Pelletier 
+ *       - 330755: Nested embeddables can't be used as embedded ids
+ *     11/10/2011-2.4 Guy Pelletier 
+ *       - 357474: Address primaryKey option from tenant discriminator column
+ *     14/05/2012-2.4 Guy Pelletier   
+ *       - 376603: Provide for table per tenant support for multitenant applications
+ *     08/20/2012-2.4 Guy Pelletier 
+ *       - 381079: EclipseLink dynamic entity does not support embedded-id
  ******************************************************************************/  
 package org.eclipse.persistence.internal.jpa;
 
+import java.io.Serializable;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 import java.security.AccessController;
@@ -30,10 +39,12 @@ import org.eclipse.persistence.internal.helper.ConversionManager;
 import org.eclipse.persistence.internal.localization.ExceptionLocalization;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.mappings.DatabaseMapping;
-import org.eclipse.persistence.mappings.foundation.AbstractDirectMapping;
+import org.eclipse.persistence.mappings.ObjectReferenceMapping;
+import org.eclipse.persistence.mappings.foundation.AbstractColumnMapping;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.identitymaps.CacheId;
+import org.eclipse.persistence.internal.indirection.WeavedObjectBasicIndirectionPolicy;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
 import org.eclipse.persistence.internal.security.PrivilegedGetField;
 import org.eclipse.persistence.internal.security.PrivilegedClassForName;
@@ -53,11 +64,11 @@ import org.eclipse.persistence.mappings.OneToOneMapping;
 public class CMP3Policy extends CMPPolicy {
 
     /** Stores the fields for this classes compound primary key class if required. */
-    protected KeyElementAccessor[] keyClassFields;
+    protected transient KeyElementAccessor[] keyClassFields;
 
     /** Used to look up the KeyElementAccessor for a specific DatabaseField, used for 
       resolving DerivedIds */
-    protected HashMap<DatabaseField,KeyElementAccessor> fieldToAccessorMap;
+    protected transient HashMap<DatabaseField,KeyElementAccessor> fieldToAccessorMap;
     
     // Store the primary key class name
     protected String pkClassName;
@@ -102,6 +113,18 @@ public class CMP3Policy extends CMPPolicy {
         }
     }
     
+    /**
+     * INTERNAL:
+     * Clone the CMP3Policy
+     */
+    @Override
+    public CMP3Policy clone() {
+        CMP3Policy policy = new CMP3Policy();
+        policy.setPrimaryKeyClassName(getPKClassName());
+        policy.setPKClass(getPKClass());
+        return policy;
+    }
+    
     /**
      * INTERNAL:
      * Convert all the class-name-based settings in this object to actual class-based
@@ -109,6 +132,7 @@ public class CMP3Policy extends CMPPolicy {
      * with class names to a project with classes.
      * @param classLoader 
      */
+    @Override
     public void convertClassNamesToClasses(ClassLoader classLoader){
         if(getPKClassName() != null){
             try{
@@ -134,6 +158,7 @@ public class CMP3Policy extends CMPPolicy {
      * INTERNAL:
      * Return if this policy is for CMP3.
      */
+    @Override
     public boolean isCMP3Policy() {
         return true;
     }
@@ -150,9 +175,6 @@ public class CMP3Policy extends CMPPolicy {
      * Return the java Class representing the primary key class name 
      */
     public Class getPKClass() {
-        if(this.pkClass == null && getPKClassName() == null) {
-            getKeyClassFields(null);
-        }
         return this.pkClass;
     }
 
@@ -162,7 +184,7 @@ public class CMP3Policy extends CMPPolicy {
     public void setPKClass(Class pkClass) {
         this.pkClass = pkClass;
     }
-
+ 
     /**
      * INTERNAL:
      */
@@ -173,12 +195,14 @@ public class CMP3Policy extends CMPPolicy {
     /**
      * INTERNAL:
      */
+    @Override
     public Object getPKClassInstance() {
         try {
             return getPKClass().newInstance();
-        } catch (Exception ex) {
-            return null;
-            // WIP - this should throw an exception
+        } catch (IllegalAccessException ex) {
+            throw DescriptorException.exceptionAccessingPrimaryKeyInstance(this.getDescriptor(), ex);
+        } catch (InstantiationException ex){
+            throw DescriptorException.exceptionAccessingPrimaryKeyInstance(this.getDescriptor(), ex);
         }
     }
     
@@ -194,7 +218,7 @@ public class CMP3Policy extends CMPPolicy {
         // If the descriptor primary key is mapped through direct-to-field mappings,
         // then no elaborate conversion is required.
         // If key is compound, add each value to the vector.
-        KeyElementAccessor[] pkElementArray = this.getKeyClassFields(key.getClass());
+        KeyElementAccessor[] pkElementArray = this.getKeyClassFields();
         Object[] primaryKey = null;
         if (getDescriptor().getCacheKeyType() != CacheKeyType.ID_VALUE) {
             primaryKey = new Object[pkElementArray.length];
@@ -202,8 +226,30 @@ public class CMP3Policy extends CMPPolicy {
         for (int index = 0; index < pkElementArray.length; index++) {
             DatabaseMapping mapping = pkElementArray[index].getMapping();
             Object fieldValue = null;
-            if (mapping.isDirectToFieldMapping()) {
-                fieldValue = ((AbstractDirectMapping)mapping).getFieldValue(pkElementArray[index].getValue(key, session), session);
+            if (mapping.isAbstractColumnMapping()) {    
+                if (pkElementArray[index].isNestedAccessor()) {
+                    // We have nested aggregate(s) in the embedded id pkclass.
+                    DatabaseField keyField = pkElementArray[index].getDatabaseField();
+                    Object keyToUse = key;
+                    DatabaseMapping keyMapping = getDescriptor().getObjectBuilder().getMappingForField(keyField);
+                    
+                    if (keyMapping.isAggregateMapping()) {
+                        keyMapping = keyMapping.getReferenceDescriptor().getObjectBuilder().getMappingForField(keyField);
+                        
+                        // Keep driving down the nested aggregates ...
+                        while (keyMapping.isAggregateMapping()) {
+                            keyToUse = keyMapping.getRealAttributeValueFromObject(keyToUse, session);
+                            keyMapping = keyMapping.getReferenceDescriptor().getObjectBuilder().getMappingForField(keyField);
+                        }
+                        
+                        fieldValue = ((AbstractColumnMapping)mapping).getFieldValue(pkElementArray[index].getValue(keyToUse, session), session);
+                    } else {
+                        // This should never hit but just in case ... better to get a proper exception rather than a NPE etc.
+                        fieldValue = ((AbstractColumnMapping)mapping).getFieldValue(pkElementArray[index].getValue(keyToUse, session), session);
+                    }
+                } else {
+                    fieldValue = ((AbstractColumnMapping)mapping).getFieldValue(pkElementArray[index].getValue(key, session), session);
+                }
             } else {
                 fieldValue = pkElementArray[index].getValue(key, session);
                 if ( (fieldValue !=null) && (pkClass != null) && (mapping.isOneToOneMapping()) ){
@@ -261,10 +307,11 @@ public class CMP3Policy extends CMPPolicy {
      *            corresponding pk fields
      * @return Object
      */
+    @Override
     public Object createBeanUsingKey(Object key, AbstractSession session) {
         try {
             Object bean = this.getDescriptor().getInstantiationPolicy().buildNewInstance();
-            KeyElementAccessor[] keyElements = this.getKeyClassFields(key.getClass());
+            KeyElementAccessor[] keyElements = this.getKeyClassFields();
             for (int index = 0; index < keyElements.length; ++index) {
                 Object toWriteInto = bean;
                 Object keyFieldValue = keyElements[index].getValue(key, session);
@@ -329,6 +376,9 @@ public class CMP3Policy extends CMPPolicy {
             // the mapping was found on the key class was found or not and throw an exception otherwise.
             Exception noSuchElementException = null;
             
+            // Set the current key class ...
+            Class currentKeyClass = keyClass;
+            
             // We always start by looking at the writable mappings first. Our preference is to use the 
             // writable mappings unless a derived id mapping is specified in which case we'll want to use 
             // that mapping instead when we find it.
@@ -368,19 +418,28 @@ public class CMP3Policy extends CMPPolicy {
                     allMappings.remove(mapping);
                     
                     // Update the index to parse the next mapping correctly.
-                    index = allMappings.size();                        
+                    index = allMappings.size();
+                    
+                    // Update the key class now ...
+                    currentKeyClass = mapping.getReferenceDescriptor().getJavaClass();
                 } else {
                     String fieldName = (mapping.hasMapsIdValue()) ? mapping.getMapsIdValue() : mapping.getAttributeName();
                 
-                    if (keyClass == null){                        
-                        // must be a primitive since key class was not set
+                    if (currentKeyClass == null || mapping.isMultitenantPrimaryKeyMapping()) {
+                        // Without a currentKeyClass, the primary key is a non compound key but
+                        // we may need to add any multitenant primary key mappings that are
+                        // defined and we need an accessor for them. The same case will hold
+                        // true when we do have a currentKeyClass. Multitenant primary keys
+                        // must still be added.
                         pkAttributes[i] = new KeyIsElementAccessor(fieldName, field, mapping);
-                        if (mapping.isDirectToFieldMapping()){
+                        if (mapping.isAbstractDirectMapping()) {
                             setPKClass(ConversionManager.getObjectClass(mapping.getAttributeClassification()));
                         } else if (mapping.isOneToOneMapping()) {
                             ClassDescriptor refDescriptor = mapping.getReferenceDescriptor();
                             // ensure the referenced descriptor was initialized
-                            refDescriptor.initialize(session);
+                            if (!session.isRemoteSession()) {
+                                refDescriptor.initialize(session);
+                            }
                             CMPPolicy refPolicy = refDescriptor.getCMPPolicy();
                             setPKClass(refPolicy.getPKClass());
                         } 
@@ -390,9 +449,11 @@ public class CMP3Policy extends CMPPolicy {
                         if (mapping.isOneToOneMapping()){
                             ClassDescriptor refDescriptor = mapping.getReferenceDescriptor();
                             // ensure the referenced descriptor was initialized
-                            refDescriptor.initialize(session);
+                            if (!session.isRemoteSession()) {
+                                refDescriptor.initialize(session);
+                            }
                             CMPPolicy refPolicy = refDescriptor.getCMPPolicy();
-                            if ((refPolicy!=null) && refPolicy.isCMP3Policy() && (refPolicy.getPKClass() == keyClass)){
+                            if ((refPolicy!=null) && refPolicy.isCMP3Policy() && (refPolicy.getPKClass() == currentKeyClass)){
                                 //Since the ref pk class is our pk class, get the accessor we need to pull the value out of the PK class for our field
                                 OneToOneMapping refmapping = (OneToOneMapping)mapping;
                                 DatabaseField targetKey = refmapping.getSourceToTargetKeyFields().get(field);
@@ -403,35 +464,48 @@ public class CMP3Policy extends CMPPolicy {
                                 break;
                             }
                         }
+                        
                         try {
-                            pkAttributes[i] = new FieldAccessor(getField(keyClass, fieldName), fieldName, field, mapping);
+                            pkAttributes[i] = new FieldAccessor(getField(currentKeyClass, fieldName), fieldName, field, mapping, currentKeyClass != keyClass);
                             fieldToAccessorMap.put(field, pkAttributes[i]);
                             noSuchElementException = null;
                         } catch (NoSuchFieldException ex) {
-                            if (mapping.getGetMethodName() != null) {
+                            String getMethodName = null; 
+                            String setMethodName = null; 
+                            if(mapping.isObjectReferenceMapping() && ((ObjectReferenceMapping)mapping).getIndirectionPolicy().isWeavedObjectBasicIndirectionPolicy()) {
+                                WeavedObjectBasicIndirectionPolicy weavedIndirectionPolicy = (WeavedObjectBasicIndirectionPolicy)((ObjectReferenceMapping)mapping).getIndirectionPolicy();
+                                if (weavedIndirectionPolicy.hasUsedMethodAccess()) {
+                                    getMethodName = weavedIndirectionPolicy.getGetMethodName();
+                                    setMethodName = weavedIndirectionPolicy.getSetMethodName();
+                                }
+                            } else {
+                                getMethodName = mapping.getGetMethodName(); 
+                                setMethodName = mapping.getSetMethodName(); 
+                            }
+                            if (getMethodName != null) {
                                 // Must be a property.
                                 try {
                                     Method getMethod = null;
                                     if (PrivilegedAccessHelper.shouldUsePrivilegedAccess()){
                                         try {
-                                            getMethod = AccessController.doPrivileged(new PrivilegedGetMethod(keyClass, mapping.getGetMethodName(), new Class[] {}, true));
+                                            getMethod = AccessController.doPrivileged(new PrivilegedGetMethod(currentKeyClass, getMethodName, new Class[] {}, true));
                                         } catch (PrivilegedActionException exception) {
                                             throw (NoSuchMethodException)exception.getException();
                                         }
                                     } else {
-                                        getMethod = PrivilegedAccessHelper.getMethod(keyClass, mapping.getGetMethodName(), new Class[] {}, true);
+                                        getMethod = PrivilegedAccessHelper.getMethod(currentKeyClass, getMethodName, new Class[] {}, true);
                                     }
                                     Method setMethod = null;
                                     if (PrivilegedAccessHelper.shouldUsePrivilegedAccess()){
                                         try {
-                                            setMethod = AccessController.doPrivileged(new PrivilegedGetMethod(keyClass, mapping.getSetMethodName(), new Class[] {getMethod.getReturnType()}, true));
+                                            setMethod = AccessController.doPrivileged(new PrivilegedGetMethod(currentKeyClass, setMethodName, new Class[] {getMethod.getReturnType()}, true));
                                         } catch (PrivilegedActionException exception) {
                                             throw (NoSuchMethodException)exception.getException();
                                         }
                                     } else {
-                                        setMethod = PrivilegedAccessHelper.getMethod(keyClass, mapping.getSetMethodName(), new Class[] {getMethod.getReturnType()}, true);
+                                        setMethod = PrivilegedAccessHelper.getMethod(currentKeyClass, setMethodName, new Class[] {getMethod.getReturnType()}, true);
                                     }
-                                    pkAttributes[i] = new PropertyAccessor(getMethod, setMethod, fieldName, field, mapping);
+                                    pkAttributes[i] = new PropertyAccessor(getMethod, setMethod, fieldName, field, mapping, currentKeyClass != keyClass);
                                     this.fieldToAccessorMap.put(field, pkAttributes[i]);
                                     noSuchElementException = null;
                                 } catch (NoSuchMethodException exs) {
@@ -440,7 +514,14 @@ public class CMP3Policy extends CMPPolicy {
                                 }
                             } else {
                                 noSuchElementException = ex;
-                            }
+	                        }
+	                            
+	                        // If we can't load the field or methods and the attribute accessor is a values
+	                        // accessor then we're dealing with a dynamic entity.
+	                        if (noSuchElementException != null && mapping.getAttributeAccessor().isValuesAccessor()) {
+	                        	pkAttributes[i] = new ValuesFieldAccessor(fieldName, field, mapping, currentKeyClass != keyClass);
+	                        	noSuchElementException = null;
+	                        }
                         }
                     }
                  
@@ -464,7 +545,7 @@ public class CMP3Policy extends CMPPolicy {
      * INTERNAL:
      * @return Returns the keyClassFields.
      */
-    protected KeyElementAccessor[] getKeyClassFields(Class clazz) {
+    protected KeyElementAccessor[] getKeyClassFields() {
         return this.keyClassFields;
     }
     
@@ -478,13 +559,15 @@ public class CMP3Policy extends CMPPolicy {
         protected String attributeName;
         protected DatabaseField databaseField;
         protected DatabaseMapping mapping;
+        protected boolean isNestedAccessor;
 
-        public PropertyAccessor(Method getMethod, Method setMethod, String attributeName, DatabaseField field, DatabaseMapping mapping) {
+        public PropertyAccessor(Method getMethod, Method setMethod, String attributeName, DatabaseField field, DatabaseMapping mapping, boolean isNestedAccessor) {
             this.getMethod = getMethod;
             this.setMethod = setMethod;
             this.attributeName = attributeName;
             this.databaseField = field;
             this.mapping = mapping;
+            this.isNestedAccessor = isNestedAccessor;
         }
 
         public String getAttributeName() {
@@ -520,6 +603,10 @@ public class CMP3Policy extends CMPPolicy {
             }
         }
         
+        public boolean isNestedAccessor() {
+            return isNestedAccessor;
+        }
+        
         public void setValue(Object object, Object value) {
             try {
                 if (PrivilegedAccessHelper.shouldUsePrivilegedAccess()){
@@ -541,22 +628,24 @@ public class CMP3Policy extends CMPPolicy {
             }
         }
     }
-
+    
     /**
      * INTERNAL:
      * This class will be used when the element of the keyclass is a field
      */
-    private class FieldAccessor implements KeyElementAccessor {
+    private class FieldAccessor implements KeyElementAccessor, Serializable {
         protected Field field;
         protected String attributeName;
         protected DatabaseField databaseField;
         protected DatabaseMapping mapping;
+        protected boolean isNestedAccessor;
 
-        public FieldAccessor(Field field, String attributeName, DatabaseField databaseField, DatabaseMapping mapping) {
+        public FieldAccessor(Field field, String attributeName, DatabaseField databaseField, DatabaseMapping mapping, boolean isNestedAccessor) {
             this.field = field;
             this.attributeName = attributeName;
             this.databaseField = databaseField;
             this.mapping = mapping;
+            this.isNestedAccessor = isNestedAccessor;
         }
 
         public String getAttributeName() {
@@ -586,6 +675,10 @@ public class CMP3Policy extends CMPPolicy {
             }
         }
         
+        public boolean isNestedAccessor() {
+            return isNestedAccessor;
+        }
+        
         public void setValue(Object object, Object value) {
             try {
                 Field pkField = null;
@@ -605,6 +698,48 @@ public class CMP3Policy extends CMPPolicy {
             }
         }
     }
+    
+    /**
+     * INTERNAL:
+     * This class will be used when the element of the keyclass is a virtual field.
+     */
+    private class ValuesFieldAccessor implements KeyElementAccessor {
+        protected String attributeName;
+        protected DatabaseField databaseField;
+        protected DatabaseMapping mapping;
+        protected boolean isNestedAccessor;
+
+        public ValuesFieldAccessor(String attributeName, DatabaseField databaseField, DatabaseMapping mapping, boolean isNestedAccessor) {
+            this.attributeName = attributeName;
+            this.databaseField = databaseField;
+            this.mapping = mapping;
+            this.isNestedAccessor = isNestedAccessor;
+        }
+
+        public String getAttributeName() {
+            return this.attributeName;
+        }
+
+        public DatabaseField getDatabaseField() {
+            return this.databaseField;
+        }
+        
+        public DatabaseMapping getMapping() {
+            return this.mapping;
+        }
+        
+        public Object getValue(Object object, AbstractSession session) {
+        	return mapping.getRealAttributeValueFromObject(object, session);
+        }
+        
+        public boolean isNestedAccessor() {
+            return isNestedAccessor;
+        }
+        
+        public void setValue(Object object, Object value) {
+        	mapping.setRealAttributeValueInObject(object, value);
+        }
+    }
 
     /**
      * INTERNAL:
@@ -616,11 +751,10 @@ public class CMP3Policy extends CMPPolicy {
      */
     public Object getPkValueFromKeyForField(Object key, DatabaseField field, AbstractSession session){
         Object fieldValue = null;
-        this.getKeyClassFields(key.getClass());
         KeyElementAccessor accessor = this.fieldToAccessorMap.get(field);
         DatabaseMapping mapping = accessor.getMapping();
-        if (mapping.isDirectToFieldMapping()) {
-            fieldValue = ((AbstractDirectMapping)mapping).getFieldValue(accessor.getValue(key, session), session);
+        if (mapping.isAbstractColumnMapping()) {
+            fieldValue = ((AbstractColumnMapping)mapping).getFieldValue(accessor.getValue(key, session), session);
         } else {
             fieldValue = accessor.getValue(key, session);
             if (mapping.isOneToOneMapping()){
@@ -643,9 +777,21 @@ public class CMP3Policy extends CMPPolicy {
      * INTERNAL:
      * Initialize the CMPPolicy settings.
      */
+    @Override
     public void initialize(ClassDescriptor descriptor, AbstractSession session) throws DescriptorException {
         super.initialize(descriptor, session);
         
         this.keyClassFields = initializePrimaryKeyFields(this.pkClass, session);
     }
+    
+    /**
+     * INTERNAL:
+     * Initialize the CMPPolicy settings for remote sessions.
+     */
+    @Override
+    public void remoteInitialize(ClassDescriptor descriptor, AbstractSession session) throws DescriptorException {
+        super.remoteInitialize(descriptor, session);
+        
+        this.keyClassFields = initializePrimaryKeyFields(null, session);
+    }
 }
