@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -42,6 +42,18 @@
  *       - 253083: Add support for dynamic persistence using ORM.xml/eclipselink-orm.xml
  *     06/01/2010-2.1 Guy Pelletier 
  *       - 315195: Add new property to avoid reading XML during the canonical model generation
+ *     06/14/2010-2.2 Guy Pelletier 
+ *       - 264417: Table generation is incorrect for JoinTables in AssociationOverrides
+ *     07/05/2010-2.1.1 Guy Pelletier 
+ *       - 317708: Exception thrown when using LAZY fetch on VIRTUAL mapping
+ *     09/16/2010-2.2 Guy Pelletier 
+ *       - 283028: Add support for letting an @Embeddable extend a @MappedSuperclass
+ *     12/01/2010-2.2 Guy Pelletier 
+ *       - 331234: xml-mapping-metadata-complete overriden by metadata-complete specification
+ *     12/02/2010-2.2 Guy Pelletier 
+ *       - 251554: ExcludeDefaultMapping annotation needed 
+ *     03/24/2011-2.3 Guy Pelletier 
+ *       - 337323: Multi-tenant with shared schema support (part 1)
  ******************************************************************************/  
 package org.eclipse.persistence.internal.jpa.metadata.accessors.classes;
 
@@ -51,17 +63,28 @@ import java.util.Map;
 
 import org.eclipse.persistence.annotations.Cache;
 import org.eclipse.persistence.exceptions.ValidationException;
-import org.eclipse.persistence.internal.jpa.metadata.MetadataConstants;
 import org.eclipse.persistence.internal.jpa.metadata.MetadataDescriptor;
 import org.eclipse.persistence.internal.jpa.metadata.MetadataLogger;
 import org.eclipse.persistence.internal.jpa.metadata.MetadataProject;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAnnotation;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataClass;
 
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_ACCESS_FIELD;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_ACCESS_PROPERTY;
+
 /**
  * INTERNAL:
  * An embeddable accessor.
  * 
+ * Key notes:
+ * - any metadata mapped from XML to this class must be compared in the
+ *   equals method.
+ * - any metadata mapped from XML to this class must be handled in the merge
+ *   method. (merging is done at the accessor/mapping level)
+ * - any metadata mapped from XML to this class must be initialized in the
+ *   initXMLObject  method.
+ * - methods should be preserved in alphabetical order.
+ * 
  * @author Guy Pelletier
  * @since EclipseLink 1.0
  */ 
@@ -139,6 +162,45 @@ public class EmbeddableAccessor extends ClassAccessor {
         }
     }
     
+    /**
+     * INTERNAL:
+     * Build a list of classes that are decorated with a MappedSuperclass
+     * annotation or that are tagged as a mapped-superclass in an XML document.
+     * 
+     * This method will also do a couple other things as well since we are
+     * traversing the parent classes:
+     *  - Build a map of generic types specified and will be used to resolve 
+     *    actual class types for mappings.
+     *  - save mapped-superclass descriptors on the project for later use
+     *    by the Metamodel API
+     * 
+     * We don't support embeddable inheritance yet. When that is added, this
+     * method will need to change and in fact we may be able to re-use the
+     * existing discover method from EntityAccessor (with minor tweaks).
+     */
+    protected void discoverMappedSuperclassesAndInheritanceParents(boolean addMappedSuperclassAccessors) {
+        // Clear any previous discovery.
+        clearMappedSuperclassesAndInheritanceParents();
+        
+        MetadataClass parentClass = getJavaClass().getSuperclass();
+        List<String> genericTypes = getJavaClass().getGenericType();
+        
+        while (parentClass != null && ! parentClass.isObject()) {
+            // Our parent might be a mapped superclass, check and add as needed.
+            addPotentialMappedSuperclass(parentClass, addMappedSuperclassAccessors);
+                
+            // Resolve any generic types from the generic parent onto the 
+            // current entity accessor.
+            resolveGenericTypes(genericTypes, parentClass);
+                
+            // Grab the generic types from the parent class.
+            genericTypes = parentClass.getGenericType();
+                
+            // Finally, get the next parent and keep processing ...
+            parentClass = parentClass.getSuperclass();  
+        }
+    }
+    
     /**
      * INTERNAL:
      */
@@ -184,9 +246,12 @@ public class EmbeddableAccessor extends ClassAccessor {
      */
     @Override
     public MetadataDescriptor getOwningDescriptor() {
-        // Return the first owning descriptor. In most cases this will be OK
-        // since in most cases there is only one.
-        return getOwningDescriptors().get(0);
+        if (getOwningDescriptors() != null && getOwningDescriptors().size() > 0) {
+            // Return the first owning descriptor. In most cases this will be OK
+            // since in most cases there is only one.
+            return getOwningDescriptors().get(0);
+        } 
+        return this.getDescriptor();
     }
     
     /** 
@@ -206,15 +271,15 @@ public class EmbeddableAccessor extends ClassAccessor {
      * This method is called after each entity of the persistence unit has had 
      * an opportunity to pre-process itself first since we'll rely on owning 
      * entities for things like access type etc. The pre-process will run some 
-     * validation. The order of processing is important, care must be taken if 
-     * changes must be made. 
+     * validation. 
+     * 
+     * The order of processing is important, care must be taken if changes must 
+     * be made. 
      */
     @Override
     public void preProcess() {
-        setIsPreProcessed();
-        
-        // Process the parent class if specified.
-        processParentClass();
+        // Perform the parent discovery process before processing any further.  
+        discoverMappedSuperclassesAndInheritanceParents(true);
         
         // Process the correct access type before any other processing.
         processAccessType();
@@ -225,19 +290,12 @@ public class EmbeddableAccessor extends ClassAccessor {
         // Process the default access methods after determining access type.
         processAccessMethods();
         
-        // Set a metadata complete flag if specified.
-        if (getMetadataComplete() != null) {
-            getDescriptor().setIgnoreAnnotations(isMetadataComplete());
-        } 
+        // Process a @Struct and @EIS annotation to create the correct type of descriptor.
+        processStruct();
+        processNoSql();
         
-        // Set an exclude default mappings flag if specified.
-        if (getExcludeDefaultMappings() != null) {
-            getDescriptor().setIgnoreDefaultMappings(excludeDefaultMappings());
-        } 
-
-        // Add the accessors and converters on this embeddable.
-        addAccessors();
-        addConverters();
+        // Process our parents metadata after processing our own.
+        super.preProcess();
     }
     
     /**
@@ -246,34 +304,31 @@ public class EmbeddableAccessor extends ClassAccessor {
      * during the canonical model generation. The use of this pre-process allows
      * us to remove some items from the regular pre-process that do not apply
      * to the canonical model generation.
+     * 
+     * The order of processing is important, care must be taken if changes must 
+     * be made.
      */
     @Override
-    public void preProcessForCanonicalModel() {        
-        setIsPreProcessed();
+    public void preProcessForCanonicalModel() {
+        // Perform the parent discovery process before processing any further.  
+        discoverMappedSuperclassesAndInheritanceParents(false);
         
-        // Process the parent class if specified.
-        processParentClass();
+        // Process our parents metadata after processing our own.
+        super.preProcessForCanonicalModel();
+    }
+    
+    /**
+     * INTERNAL
+     * Sub classes (Entity and Embeddable) must override this method to control 
+     * the metadata that is processed for their context. 
+     */
+    @Override
+    protected void preProcessMappedSuperclassMetadata(MappedSuperclassAccessor mappedSuperclass) {
+        // Process the global converters.
+        mappedSuperclass.processConverters();
         
-        // Process the correct access type before any other processing.
-        processAccessType();
-        
-        // Set a metadata complete flag if specified.
-        if (getMetadataComplete() != null) {
-            getDescriptor().setIgnoreAnnotations(isMetadataComplete());
-        } 
-        
-        // Set an exclude default mappings flag if specified.
-        if (getExcludeDefaultMappings() != null) {
-            getDescriptor().setIgnoreDefaultMappings(excludeDefaultMappings());
-        } 
-
-        // Before gathering our accessors, clear any accessors previously 
-        // gathered. When generating the canonical model the accessors need 
-        // to be re-gathered in each compile round.
-        getDescriptor().clearAccessors();
-        
-        // Add the accessors and converters on this embeddable.
-        addAccessors();
+        // Add the accessors and converters from this mapped superclass.
+        mappedSuperclass.addAccessors();
     }
     
     /**
@@ -287,23 +342,11 @@ public class EmbeddableAccessor extends ClassAccessor {
             throw ValidationException.cacheNotSupportedWithEmbeddable(getJavaClass());
         } 
         
-        // Process the customizer metadata.
-        processCustomizer();
-        
-        // Process the copy policy metadata.
-        processCopyPolicy();
-        
-        // Process the change tracking metadata.
-        processChangeTracking();
-        
-        // Process the properties metadata.
-        processProperties();
+        // Process our parents metadata after processing our own.
+        super.process();
 
-        // Process the accessors on this embeddable.
-        processAccessors();
-        
-        // Set the processed flag.
-        setIsProcessed();
+        // Process the mapping accessors on this embeddable now.
+        processMappingAccessors();
     }
     
     /**
@@ -312,35 +355,33 @@ public class EmbeddableAccessor extends ClassAccessor {
      * need to use with our mapping attributes.
      */
     public void processAccessMethods() {
-        if (usesVirtualAccess()) {
-            // If we use virtual access and do not have any access methods
-            // specified then get the default access methods from our owning
-            // entity.
-            if (hasAccessMethods()) {
-                getDescriptor().setDefaultAccessMethods(getAccessMethods());
-            } else {
-                // The embeddable does not define default access methods. We 
-                // need to look at our owning entities and 1) validate that
-                // they all use the same default access methods and 2) grab
-                // the default access methods from them.
-                ClassAccessor embeddingAccessor = null;
-                for (ClassAccessor currentEmbeddingAccessor : m_embeddingAccessors.values()) {
-                    if (embeddingAccessor == null) {
-                        embeddingAccessor = currentEmbeddingAccessor;
-                        continue;
-                    }
-                    
-                    if (! embeddingAccessor.getDescriptor().getDefaultAccessMethods().equals(currentEmbeddingAccessor.getDescriptor().getDefaultAccessMethods())) {
-                        throw ValidationException.conflictingAccessMethodsForEmbeddable(getJavaClassName(), embeddingAccessor.getJavaClassName(), embeddingAccessor.getDescriptor().getDefaultAccessMethods(), currentEmbeddingAccessor.getJavaClassName(), currentEmbeddingAccessor.getDescriptor().getDefaultAccessMethods());
-                    }
+        // If we use virtual access and do not have any access methods
+        // specified then get the default access methods from our owning
+        // entity.
+        if (hasAccessMethods()) {
+            getDescriptor().setDefaultAccessMethods(getAccessMethods());
+        } else {
+            // The embeddable does not define default access methods. We 
+            // need to look at our owning entities and 1) validate that
+            // they all use the same default access methods and 2) grab
+            // the default access methods from them.
+            ClassAccessor embeddingAccessor = null;
+            for (ClassAccessor currentEmbeddingAccessor : m_embeddingAccessors.values()) {
+                if (embeddingAccessor == null) {
+                    embeddingAccessor = currentEmbeddingAccessor;
+                    continue;
                 }
+                
+                if (! embeddingAccessor.getDescriptor().getDefaultAccessMethods().equals(currentEmbeddingAccessor.getDescriptor().getDefaultAccessMethods())) {
+                    throw ValidationException.conflictingAccessMethodsForEmbeddable(getJavaClassName(), embeddingAccessor.getJavaClassName(), embeddingAccessor.getDescriptor().getDefaultAccessMethods(), currentEmbeddingAccessor.getJavaClassName(), currentEmbeddingAccessor.getDescriptor().getDefaultAccessMethods());
+                }
+            }
 
-                // Set the default access methods on the descriptor and log a
-                // message if there is an embedding accessor. Otherwise we'll
-                // use the default set on our descriptor.
-                if (embeddingAccessor != null) {
-                    getDescriptor().setDefaultAccessMethods(embeddingAccessor.getDescriptor().getDefaultAccessMethods());
-                }
+            // Set the default access methods on the descriptor and log a
+            // message if there is an embedding accessor. Otherwise we'll
+            // use the default set on our descriptor.
+            if (embeddingAccessor != null) {
+                getDescriptor().setDefaultAccessMethods(embeddingAccessor.getDescriptor().getDefaultAccessMethods());
             }
         }
     }
@@ -377,9 +418,23 @@ public class EmbeddableAccessor extends ClassAccessor {
             // Set the default access type on the descriptor and log a message
             // that we are defaulting the access type for this embeddable.
             if (embeddingAccessor == null) {
-                // No class embedded this embeddable and we have no explicit 
-                // access type set, just default it to FIELD.
-                getDescriptor().setDefaultAccess(MetadataConstants.FIELD);
+                // We don't have an owning entity (only possible during
+                // canonical model generation) so look at the mapped 
+                // superclasses. Ultimate default will be FIELD.
+                String defaultAccessType = JPA_ACCESS_FIELD;
+                for (MappedSuperclassAccessor mappedSuperclass : getMappedSuperclasses()) {
+                    if (! mappedSuperclass.hasAccess()) {
+                        if (mappedSuperclass.hasObjectRelationalFieldMappingAnnotationsDefined()) {
+                            defaultAccessType = JPA_ACCESS_FIELD;
+                        } else if (mappedSuperclass.hasObjectRelationalMethodMappingAnnotationsDefined()) {
+                            defaultAccessType = JPA_ACCESS_PROPERTY;
+                        }
+                            
+                        break;
+                    }
+                }
+
+                getDescriptor().setDefaultAccess(defaultAccessType);
             } else {
                 // Use the access type from the embedding accessor.
                 getDescriptor().setDefaultAccess(embeddingAccessor.getAccessType());
@@ -387,7 +442,33 @@ public class EmbeddableAccessor extends ClassAccessor {
             
             getLogger().logConfigMessage(MetadataLogger.ACCESS_TYPE, getDescriptor().getDefaultAccess(), getJavaClass());
             
-            getDescriptor().setAccessTypeOnClassDescriptor(this.getAccessType());
+            getDescriptor().setAccessTypeOnClassDescriptor(getAccessType());
         } 
     }
+    
+    /**
+     * INTERNAL
+     * From an embeddable we need pair down what we process as things like
+     * ID metadata does not apply. 
+     */
+    @Override
+    protected void processMappedSuperclassMetadata(MappedSuperclassAccessor mappedSuperclass) {
+        // Process the attribute override metadata.
+        mappedSuperclass.processAttributeOverrides();
+                    
+        // Process the association override metadata.
+        mappedSuperclass.processAssociationOverrides();
+        
+        // Process the change tracking metadata.
+        mappedSuperclass.processChangeTracking();
+        
+        // Process the customizer metadata.
+        mappedSuperclass.processCustomizer();
+        
+        // Process the copy policy metadata.
+        mappedSuperclass.processCopyPolicy();
+        
+        // Process the property metadata.
+        mappedSuperclass.processProperties();
+    }
 }
