@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -35,68 +35,100 @@
  *       - 267217: Add Named Access Type to EclipseLink-ORM
  *     04/27/2010-2.1 Guy Pelletier 
  *       - 309856: MappedSuperclasses from XML are not being initialized properly
- ******************************************************************************/  
+ *     06/14/2010-2.2 Guy Pelletier 
+ *       - 264417: Table generation is incorrect for JoinTables in AssociationOverrides
+ *     08/11/2010-2.2 Guy Pelletier 
+ *       - 312123: JPA: Validation error during Id processing on parameterized generic OneToOne Entity relationship from MappedSuperclass
+ *     09/03/2010-2.2 Guy Pelletier 
+ *       - 317286: DB column lenght not in sync between @Column and @JoinColumn
+ *     03/24/2011-2.3 Guy Pelletier 
+ *       - 337323: Multi-tenant with shared schema support (part 1)
+ *     07/03/2011-2.3.1 Guy Pelletier 
+ *       - 348756: m_cascadeOnDelete boolean should be changed to Boolean
+ *     05/17/2012-2.3.3 Guy Pelletier  
+ *       - 379829: NPE Thrown with OneToOne Relationship
+ *     11/19/2012-2.5 Guy Pelletier 
+ *       - 389090: JPA 2.1 DDL Generation Support (foreign key metadata support)
+ *     11/28/2012-2.5 Guy Pelletier 
+ *       - 374688: JPA 2.1 Converter support
+ ******************************************************************************/
 package org.eclipse.persistence.internal.jpa.metadata.accessors.mappings;
 
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_COLUMN;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_FETCH_LAZY;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_JOIN_COLUMN;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_JOIN_COLUMNS;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_JOIN_TABLE;
+
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Vector;
 
-import javax.persistence.FetchType;
-import javax.persistence.CascadeType;
-import javax.persistence.JoinColumn;
-import javax.persistence.JoinColumns;
-import javax.persistence.JoinTable;
-
-import org.eclipse.persistence.mappings.DatabaseMapping;
-import org.eclipse.persistence.mappings.ForeignReferenceMapping;
-import org.eclipse.persistence.mappings.RelationTableMechanism;
-
 import org.eclipse.persistence.annotations.BatchFetch;
+import org.eclipse.persistence.annotations.CascadeOnDelete;
+import org.eclipse.persistence.annotations.Convert;
 import org.eclipse.persistence.annotations.JoinFetch;
+import org.eclipse.persistence.annotations.Noncacheable;
 import org.eclipse.persistence.annotations.PrivateOwned;
 import org.eclipse.persistence.exceptions.ValidationException;
 import org.eclipse.persistence.indirection.ValueHolderInterface;
-
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.helper.DatabaseTable;
 import org.eclipse.persistence.internal.jpa.metadata.MetadataDescriptor;
 import org.eclipse.persistence.internal.jpa.metadata.MetadataLogger;
-
+import org.eclipse.persistence.internal.jpa.metadata.MetadataProcessor;
+import org.eclipse.persistence.internal.jpa.metadata.MetadataProject;
+import org.eclipse.persistence.internal.jpa.metadata.accessors.classes.ClassAccessor;
+import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAccessibleObject;
+import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAnnotatedElement;
+import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAnnotation;
+import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataClass;
+import org.eclipse.persistence.internal.jpa.metadata.columns.ForeignKeyMetadata;
 import org.eclipse.persistence.internal.jpa.metadata.columns.JoinColumnMetadata;
+import org.eclipse.persistence.internal.jpa.metadata.columns.JoinFieldMetadata;
+import org.eclipse.persistence.internal.jpa.metadata.mappings.BatchFetchMetadata;
 import org.eclipse.persistence.internal.jpa.metadata.mappings.CascadeMetadata;
 import org.eclipse.persistence.internal.jpa.metadata.tables.JoinTableMetadata;
 import org.eclipse.persistence.internal.jpa.metadata.xml.XMLEntityMappings;
-
-import org.eclipse.persistence.internal.jpa.metadata.accessors.classes.ClassAccessor;
-import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAccessibleObject;
-import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAnnotation;
-import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataClass;
+import org.eclipse.persistence.mappings.DatabaseMapping;
+import org.eclipse.persistence.mappings.ForeignReferenceMapping;
+import org.eclipse.persistence.mappings.RelationTableMechanism;
 
 /**
  * INTERNAL:
  * A relational accessor.
  * 
+ * Key notes:
+ * - any metadata mapped from XML to this class must be compared in the
+ *   equals method.
+ * - any metadata mapped from XML to this class must be handled in the merge
+ *   method. (merging is done at the accessor/mapping level)
+ * - any metadata mapped from XML to this class must be initialized in the
+ *   initXMLObject  method.
+ * - methods should be preserved in alphabetical order.
+ * 
  * @author Guy Pelletier
  * @since TopLink EJB 3.0 Reference Implementation
  */
 public abstract class RelationshipAccessor extends MappingAccessor {
-    // Note: Any metadata mapped from XML to this class must be compared in the equals method.
-
     private Boolean m_orphanRemoval;
-    private boolean m_privateOwned;
+    private Boolean m_cascadeOnDelete;
+    private Boolean m_nonCacheable;
+    private Boolean m_privateOwned;
+    
+    private BatchFetchMetadata m_batchFetch;
     private CascadeMetadata m_cascade;
+    private ForeignKeyMetadata m_foreignKey;
+    private JoinTableMetadata m_joinTable;
     protected MetadataClass m_referenceClass;
     private MetadataClass m_targetEntity;
     
-    private String m_fetch;
-    private String m_joinFetch;
-    private String m_batchFetch;
-    private Integer m_batchFetchSize;
-
-    private JoinTableMetadata m_joinTable;
     private List<JoinColumnMetadata> m_joinColumns = new ArrayList<JoinColumnMetadata>();
-  
+    private List<JoinFieldMetadata> m_joinFields = new ArrayList<JoinFieldMetadata>();
+    
+    private String m_fetch;
+    private String m_mappedBy;
+    private String m_joinFetch;
     private String m_targetEntityName;
     
     /**
@@ -112,47 +144,68 @@ public abstract class RelationshipAccessor extends MappingAccessor {
     protected RelationshipAccessor(MetadataAnnotation annotation, MetadataAccessibleObject accessibleObject, ClassAccessor classAccessor) {
         super(annotation, accessibleObject, classAccessor);
         
-        m_fetch = (annotation == null) ? getDefaultFetchType() : (String) annotation.getAttribute("fetch");
-        m_targetEntity = getMetadataClass((annotation == null) ? "void" : (String) annotation.getAttributeString("targetEntity"));         
-        m_cascade = (annotation == null) ? null : new CascadeMetadata((Object[]) annotation.getAttributeArray("cascade"), accessibleObject);
+        m_fetch = (annotation == null) ? getDefaultFetchType() : annotation.getAttributeString("fetch");
+        m_targetEntity = getMetadataClass((annotation == null) ? "void" : annotation.getAttributeString("targetEntity"));         
+        m_cascade = (annotation == null) ? null : new CascadeMetadata(annotation.getAttributeArray("cascade"), this);
         
-        // Set the join fetch if one is present.           
-        MetadataAnnotation joinFetch = getAnnotation(JoinFetch.class);            
-        if (joinFetch != null) {
-            // Get attribute string will return the default ""
-            m_joinFetch = (String) joinFetch.getAttributeString("value");
+        // Set the join fetch if one is present.
+        if (isAnnotationPresent(JoinFetch.class)) {
+            m_joinFetch = getAnnotation(JoinFetch.class).getAttributeString("value");
         }
         
-        // Set the batch fetch if one is present.           
-        MetadataAnnotation batchFetch = getAnnotation(BatchFetch.class);            
-        if (batchFetch != null) {
-            // Get attribute string will return the default ""
-            m_batchFetch = (String) batchFetch.getAttributeString("value");
-            m_batchFetchSize = (Integer) batchFetch.getAttribute("size");
+        // Set the batch fetch if one is present. 
+        if (isAnnotationPresent(BatchFetch.class)) {
+            m_batchFetch = new BatchFetchMetadata(getAnnotation(BatchFetch.class), this);
+        }
+        
+        // Set the join columns if some are present. 
+        // Process all the join columns first.
+        if (isAnnotationPresent(JPA_JOIN_COLUMNS)) {
+            MetadataAnnotation joinColumns = getAnnotation(JPA_JOIN_COLUMNS);
+            for (Object joinColumn : joinColumns.getAttributeArray("value")) {
+                m_joinColumns.add(new JoinColumnMetadata((MetadataAnnotation) joinColumn, this));
+            }
+            
+            // Set the foreign key metadata if one is specified.
+            if (joinColumns.hasAttribute("foreignKey")) {
+                setForeignKey(new ForeignKeyMetadata(joinColumns.getAttributeAnnotation("foreignKey"), this));
+            }
+        }
+        
+        // Process the single key join column second.
+        if (isAnnotationPresent(JPA_JOIN_COLUMN)) {
+            JoinColumnMetadata joinColumn = new JoinColumnMetadata(getAnnotation(JPA_JOIN_COLUMN), this);
+            m_joinColumns.add(joinColumn);
+            
+            // Set the foreign key metadata.
+            setForeignKey(joinColumn.getForeignKey());
+        }
+        
+        // Set the join fields if some are present.
+        if (isAnnotationPresent("org.eclipse.persistence.nosql.annotations.JoinFields")) {
+            for (Object joinColumn : getAnnotation("org.eclipse.persistence.nosql.annotations.JoinFields").getAttributeArray("value")) {
+                m_joinColumns.add(new JoinColumnMetadata((MetadataAnnotation) joinColumn, this));
+            }
+        }
+        
+        // Process EIS/NoSQL join field.
+        if (isAnnotationPresent("org.eclipse.persistence.nosql.annotations.JoinField")) {
+            m_joinColumns.add(new JoinColumnMetadata(getAnnotation("org.eclipse.persistence.nosql.annotations.JoinField"), this));
+        }
+        
+        // Set the join table if one is present.
+        if (isAnnotationPresent(JPA_JOIN_TABLE)) {
+            m_joinTable = new JoinTableMetadata(getAnnotation(JPA_JOIN_TABLE), this);
         }
         
         // Set the private owned if one is present.
         m_privateOwned = isAnnotationPresent(PrivateOwned.class);
         
-        // Set the join columns if some are present. 
-        // Process all the join columns first.
-        MetadataAnnotation joinColumns = getAnnotation(JoinColumns.class);
-        if (joinColumns != null) {
-            for (Object jColumn : (Object[]) joinColumns.getAttributeArray("value")) {
-                m_joinColumns.add(new JoinColumnMetadata((MetadataAnnotation)jColumn, accessibleObject));
-            }
-        }
+        // Set the cascade on delete if one is present.
+        m_cascadeOnDelete = isAnnotationPresent(CascadeOnDelete.class);
         
-        // Process the single key join column second.
-        MetadataAnnotation joinColumn = getAnnotation(JoinColumn.class);
-        if (joinColumn != null) {
-            m_joinColumns.add(new JoinColumnMetadata(joinColumn, accessibleObject));
-        }
-        
-        // Set the join table if one is present.
-        if (isAnnotationPresent(JoinTable.class)) {
-            m_joinTable = new JoinTableMetadata(getAnnotation(JoinTable.class), accessibleObject);
-        }
+        // Set the non cacheable if one is present.
+        m_nonCacheable = isAnnotationPresent(Noncacheable.class);
     }
     
     /**
@@ -172,12 +225,8 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         }
         
         for (JoinColumnMetadata joinColumn : joinColumns) {
-            // If the pk field (referencedColumnName) is not specified, it 
-            // defaults to the primary key of the referenced table.
-            String defaultPKFieldName = descriptor.getPrimaryKeyFieldName();
-            DatabaseField pkField = joinColumn.getPrimaryKeyField();
-            setFieldName(pkField, defaultPKFieldName, PK_CTX);
-            pkField.setTable(descriptor.getPrimaryKeyTable());
+            // Look up the primary key field from the referenced column name.
+            DatabaseField pkField = getReferencedField(joinColumn.getReferencedColumnName(), descriptor, PK_CTX);
             
             // If the fk field (name) is not specified, it defaults to the 
             // name of the referencing relationship property or field of the 
@@ -187,8 +236,8 @@ public abstract class RelationshipAccessor extends MappingAccessor {
             // the join column name is formed as the concatenation of the 
             // following: the name of the entity + "_" + the name of the 
             // referenced primary key column.
-            DatabaseField fkField = joinColumn.getForeignKeyField();
-            String defaultFKFieldName = defaultFieldName + "_" + defaultPKFieldName;
+            DatabaseField fkField = joinColumn.getForeignKeyField(pkField);
+            String defaultFKFieldName = defaultFieldName + "_" + descriptor.getPrimaryKeyFieldName();
             setFieldName(fkField, defaultFKFieldName, FK_CTX);
             
             // Target table name here is the join table name.
@@ -218,7 +267,11 @@ public abstract class RelationshipAccessor extends MappingAccessor {
                 return false;
             }
             
-            if (! valuesMatch(m_privateOwned, relationshipAccessor.isPrivateOwned())) {
+            if (! valuesMatch(m_privateOwned, relationshipAccessor.getPrivateOwned())) {
+                return false;
+            }
+            
+            if (! valuesMatch(m_nonCacheable, relationshipAccessor.getNonCacheable())) {
                 return false;
             }
 
@@ -226,6 +279,10 @@ public abstract class RelationshipAccessor extends MappingAccessor {
                 return false;
             }
             
+            if (! valuesMatch(m_mappedBy, relationshipAccessor.getMappedBy())) {
+                return false;
+            }
+            
             if (! valuesMatch(m_fetch, relationshipAccessor.getFetch())) {
                 return false;
             }
@@ -238,7 +295,7 @@ public abstract class RelationshipAccessor extends MappingAccessor {
                 return false;
             }
             
-            if (! valuesMatch(m_batchFetchSize, relationshipAccessor.getBatchFetchSize())) {
+            if (! valuesMatch(m_batchFetch, relationshipAccessor.getBatchFetch())) {
                 return false;
             }
             
@@ -250,6 +307,10 @@ public abstract class RelationshipAccessor extends MappingAccessor {
                 return false;
             }
             
+            if (! valuesMatch(m_foreignKey, relationshipAccessor.getForeignKey())) {
+                return false;
+            }
+            
             return valuesMatch(m_targetEntityName, relationshipAccessor.getTargetEntityName());
         }
         
@@ -260,7 +321,7 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      * INTERNAL:
      * Used for OX mapping.
      */
-    public String getBatchFetch() {
+    public BatchFetchMetadata getBatchFetch() {
         return m_batchFetch;
     }
     
@@ -268,8 +329,8 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      * INTERNAL:
      * Used for OX mapping.
      */
-    public Integer getBatchFetchSize() {
-        return m_batchFetchSize;
+    public Boolean getCascadeOnDelete() {
+        return m_cascadeOnDelete;
     }
     
     /**
@@ -283,17 +344,17 @@ public abstract class RelationshipAccessor extends MappingAccessor {
     /**
      * INTERNAL:
      */
+    @Override
     public abstract String getDefaultFetchType();
     
     /**
      * INTERNAL:
      * Return the default table to hold the foreign key of a MapKey when
      * and Entity is used as the MapKey
-     * @return
      */
     @Override
     protected DatabaseTable getDefaultTableForEntityMapKey(){
-        if (getJoinTable() != null){
+        if (getJoinTable() != null) {
             return getJoinTable().getDatabaseTable();
         } else {
             return super.getDefaultTableForEntityMapKey();
@@ -308,6 +369,14 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         return m_fetch;
     }
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public ForeignKeyMetadata getForeignKey() {
+        return m_foreignKey;
+    }
+    
     /**
      * INTERNAL:
      * Used for OX mapping.
@@ -316,6 +385,14 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         return m_joinColumns;
     }
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */    
+    public List<JoinFieldMetadata> getJoinFields() {
+        return m_joinFields;
+    }
+    
     /**
      * INTERNAL:
      * Used for OX mapping.
@@ -341,17 +418,14 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      */
     protected JoinTableMetadata getJoinTableMetadata() {
         if (getDescriptor().hasAssociationOverrideFor(getAttributeName())) {
-            if (m_joinTable != null) {
-                // TODO: Log an override message ...
-            }
-            m_joinTable = getDescriptor().getAssociationOverrideFor(getAttributeName()).getJoinTable();
+            return getDescriptor().getAssociationOverrideFor(getAttributeName()).getJoinTable();
         } else {
             if (m_joinTable == null) {
-                // TODO: Log a defaulting message.
-                m_joinTable = new JoinTableMetadata(null, getAccessibleObject());
+                m_joinTable = new JoinTableMetadata(getClassAccessor());
             }
+            
+            return m_joinTable;
         }
-        return m_joinTable;
     }
     
     /**
@@ -360,6 +434,22 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      */
     protected abstract String getLoggingContext();
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public String getMappedBy() {
+        return m_mappedBy;
+    }
+    
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public Boolean getNonCacheable() {
+        return m_nonCacheable;
+    }
+    
     /**
      * INTERNAL:
      * Used for OX mapping.
@@ -372,28 +462,49 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      * INTERNAL:
      * Method to return an owner mapping. It will tell the owner class to
      * process itself if it hasn't already done so. Assumes that a mapped by
-     * value has been specified and that a check against mappedByValue has been
+     * value has been specified and that a check against mappedBy has been
      * done.
      */
-    protected DatabaseMapping getOwningMapping(String mappedBy) {
+    protected DatabaseMapping getOwningMapping() {
         MetadataDescriptor ownerDescriptor = getReferenceDescriptor();
-        DatabaseMapping mapping = ownerDescriptor.getMappingForAttributeName(mappedBy, this);
+        MappingAccessor mappingAccessor = ownerDescriptor.getMappingAccessor(getMappedBy());
         
         // If no mapping was found, there is an error in the mappedBy field, 
         // therefore, throw an exception.
-        if (mapping == null) {
-            throw ValidationException.noMappedByAttributeFound(ownerDescriptor.getJavaClass(), mappedBy, getJavaClass(), getAttributeName());
+        if (mappingAccessor == null) {
+            throw ValidationException.noMappedByAttributeFound(ownerDescriptor.getJavaClass(), getMappedBy(), getJavaClass(), getAttributeName());
+        } else if (mappingAccessor.isRelationship()) {
+            RelationshipAccessor relationshipAccessor = (RelationshipAccessor) mappingAccessor;
+            
+            // Check that we don't have circular mappedBy values which will 
+            // cause an infinite loop.
+            String mappedBy = relationshipAccessor.getMappedBy();
+                
+            if (mappedBy != null && mappedBy.equals(getAttributeName())) {
+                throw ValidationException.circularMappedByReferences(getJavaClass(), getAttributeName(), getJavaClass(), getMappedBy());
+            }
+            
+            // Make sure the mapping accessor is processed.
+            if (! mappingAccessor.isProcessed()) {
+                mappingAccessor.process();
+            }
+            
+            if (this.getMapping() != null
+                    && this.getMapping().isForeignReferenceMapping()) {
+                ((ForeignReferenceMapping) this.getMapping())
+                        .setMappedBy(mappedBy);
+            }
         }
         
-        return mapping;
+        return mappingAccessor.getMapping();
     }
     
     /**
      * INTERNAL:
      * Used for OX mapping.
      */
-    public String getPrivateOwned() {
-        return null;
+    public Boolean getPrivateOwned() {
+        return m_privateOwned;
     }
     
     /**
@@ -404,19 +515,45 @@ public abstract class RelationshipAccessor extends MappingAccessor {
       */
     @Override
     public MetadataDescriptor getReferenceDescriptor() {
-        MetadataDescriptor descriptor;
-       
-        try {
-            descriptor = super.getReferenceDescriptor();
-        } catch (Exception exception) {
-            descriptor = null;
+        MetadataDescriptor referenceDescriptor;
+        
+        // When processing metamodel mapped superclasses, we don't have the 
+        // luxury of a full type context as we would during regular metadata
+        // processing (e.g. to figure out generic types). The metamodel mapped
+        // superclass descriptors must therefore rely on a real child descriptor
+        // to extract this information and process correctly. When a type is 
+        // unknown, the reference class name currently defaults to java.lang.string.
+        // @see MetadataAnnotatedElement getRawClass(MetadataDescriptor)
+        if (getDescriptor().isMappedSuperclass() && getReferenceClassName().equals(MetadataAnnotatedElement.DEFAULT_RAW_CLASS) || getReferenceClass().isVoid()) {
+            MappingAccessor childMappingAccessor = getDescriptor().getMetamodelMappedSuperclassChildDescriptor().getMappingAccessor(getAttributeName());
+            referenceDescriptor = childMappingAccessor.getReferenceDescriptor();
+            
+            if (referenceDescriptor.isInheritanceSubclass()) {
+                referenceDescriptor = referenceDescriptor.getInheritanceRootDescriptor();
+            }
+        } else {
+            ClassAccessor accessor = getProject().getAccessor(getReferenceClassName());
+            referenceDescriptor = (accessor != null) ? accessor.getDescriptor() : null;
+            if (referenceDescriptor == null) {
+                MetadataProcessor compositeProcessor = getProject().getCompositeProcessor();
+                if (compositeProcessor != null) {
+                    for (MetadataProject pearProject : compositeProcessor.getPearProjects(getProject())) {
+                        accessor = pearProject.getAccessor(getReferenceClassName());
+                        if (accessor != null) {
+                            referenceDescriptor = accessor.getDescriptor();
+                            break;
+                        }
+                    }
+                }
+            }
         }
        
-        if (descriptor == null || descriptor.isEmbeddable() || descriptor.isEmbeddableCollection()) {
+        // Validate the reference descriptor is valid.
+        if (referenceDescriptor == null || referenceDescriptor.isEmbeddable() || referenceDescriptor.isEmbeddableCollection()) {
             throw ValidationException.nonEntityTargetInRelationship(getJavaClass(), getReferenceClass(), getAnnotatedElement());
         }
        
-        return descriptor;
+        return referenceDescriptor;
     }
     
     /**
@@ -437,12 +574,22 @@ public abstract class RelationshipAccessor extends MappingAccessor {
     
     /**
      * INTERNAL:
-     * Return true if a join table has been explicitly set on this accessor.
+     * Return true if a join table exists for this accessor (either directly
+     * set or through an association override).
      */
     protected boolean hasJoinTable() {
         return m_joinTable != null;
     }
     
+    /**
+     * INTERNAL:
+     * Return true if this accessor is the non owning side of the relationship,
+     * that is, has a mapped by value. 
+     */
+    public boolean hasMappedBy() {
+        return getMappedBy() != null && ! getMappedBy().equals("");
+    }
+    
     /**
      * INTERNAL:
      */
@@ -450,17 +597,29 @@ public abstract class RelationshipAccessor extends MappingAccessor {
     public void initXMLObject(MetadataAccessibleObject accessibleObject, XMLEntityMappings entityMappings) {
         super.initXMLObject(accessibleObject, entityMappings);
         
+        if (m_joinFields != null) {
+            m_joinColumns.addAll(m_joinFields);
+        }
+        
         // Initialize lists of objects.
         initXMLObjects(m_joinColumns, accessibleObject);
         
         // Initialize single objects.
         initXMLObject(m_joinTable, accessibleObject);
         initXMLObject(m_cascade, accessibleObject);
+        initXMLObject(m_foreignKey, accessibleObject);
         
         // Initialize the target entity name we read from XML.
         m_targetEntity = initXMLClassName(m_targetEntityName);
     }
     
+    /**
+     * INTERNAL:
+     */
+    public boolean isCascadeOnDelete() {
+        return m_cascadeOnDelete != null && m_cascadeOnDelete.booleanValue();
+    }
+    
     /**
      * INTERNAL:
      * Return if the accessor should be lazy fetched.
@@ -472,9 +631,17 @@ public abstract class RelationshipAccessor extends MappingAccessor {
             fetchType = getDefaultFetchType();
         }
         
-        return fetchType.equals(FetchType.LAZY.name());
+        return fetchType.equals(JPA_FETCH_LAZY);
     }
   
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public boolean isNonCacheable() {
+        return m_nonCacheable != null && m_nonCacheable.booleanValue();
+    }
+    
     /**
      * INTERNAL:
      * Return true is this relationship employs orphanRemoval.
@@ -488,7 +655,7 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      * Used for OX mapping.
      */
     public boolean isPrivateOwned() {
-        return m_privateOwned;
+        return m_privateOwned != null && m_privateOwned.booleanValue();
     }
     
     /**
@@ -500,19 +667,47 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         return getTargetEntity().getName().equals(ValueHolderInterface.class.getName()) || (getTargetEntity().getName().equals(void.class.getName()) && getReferenceClass().getName().equals(ValueHolderInterface.class.getName()));
     }
     
+    /**
+     * INTERNAL:
+     * Common validation done by all relationship accessors.
+     */
+    public void process() {
+        // The processing of this accessor may have been fast tracked through a 
+        // non-owning relationship. If so, no processing is required.
+        if (! isProcessed()) {
+            // If a Column annotation is specified then throw an exception.
+            if (isAnnotationPresent(JPA_COLUMN)) {
+                throw ValidationException.invalidColumnAnnotationOnRelationship(getJavaClass(), getAttributeName());
+            }
+                
+            // If a Convert annotation is specified then throw an exception.
+            if (isAnnotationPresent(Convert.class)) {
+                throw ValidationException.invalidMappingForConverter(getJavaClass(), getAttributeName());
+            }
+        }
+    }
+    
+    /**
+     * INTERNAL:
+     * Set the batch fetch type on the foreign reference mapping.
+     */
+    protected void processBatchFetch(ForeignReferenceMapping mapping) {
+        if (m_batchFetch != null) {
+            m_batchFetch.process(mapping);
+        }   
+    }
+    
     /**
      * INTERNAL:
      */
     protected void processCascadeTypes(ForeignReferenceMapping mapping) {
         if (m_cascade != null) {
-            for (String cascadeType : m_cascade.getTypes()) {
-                setCascadeType(cascadeType, mapping);
-            }
+            m_cascade.process(mapping);
         }
         
         // Apply the persistence unit default cascade-persist if necessary.
         if (getDescriptor().isCascadePersist() && ! mapping.isCascadePersist()) {
-            setCascadeType(CascadeType.PERSIST.name(), mapping);
+            mapping.setCascadePersist(true);
         }
     }
     
@@ -605,25 +800,37 @@ public abstract class RelationshipAccessor extends MappingAccessor {
     
     /**
      * INTERNAL:
-     * Front end validation before actually processing the relationship 
-     * accessor. The process() method should not be called directly.
+     * Process settings common to ForeignReferenceMapping.
      */
-    public void processRelationship() {
-        // The processing of this accessor may have been fast tracked through a 
-        // non-owning relationship. If so, no processing is required.
-        if (! isProcessed()) {
-            // If a Column annotation is specified then throw an exception.
-            if (hasColumn()) {
-                throw ValidationException.invalidColumnAnnotationOnRelationship(getJavaClass(), getAttributeName());
-            }
-                
-            // If a Convert annotation is specified then throw an exception.
-            if (hasConvert(false)) {
-                throw ValidationException.invalidMappingForConverter(getJavaClass(), getAttributeName());
-            }
-                        
-            process();
-        }
+    protected void processRelationshipMapping(ForeignReferenceMapping mapping) {
+        // Set the mapping, this must be done first.
+        setMapping(mapping);
+        
+        mapping.setIsLazy(isLazy());
+        mapping.setAttributeName(getAttributeName());
+        mapping.setReferenceClassName(getReferenceClassName());
+        mapping.setIsCascadeOnDeleteSetOnDatabase(isCascadeOnDelete());
+        
+        // Process join fetch type.
+        processJoinFetch(getJoinFetch(), mapping);
+        
+        // Process the batch fetch if specified.
+        processBatchFetch(mapping);
+            
+        // Process the orphanRemoval or PrivateOwned
+        processOrphanRemoval(mapping);
+        
+        // Will check for PROPERTY access
+        setAccessorMethods(mapping);
+
+        // Process the cascade types.
+        processCascadeTypes(mapping);
+        
+        // Process any partitioning policies if specified.
+        processPartitioning();
+        
+        // Process a non-cacheable setting.
+        mapping.setIsCacheable(!isNonCacheable());
     }
     
     /**
@@ -635,12 +842,11 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         super.setAccessorMethods(mapping);
         
         // If we have property access and the owning class has field access, 
-        // mark the mapping to weave transient field value holders (if it 
-        // so applies at weaving time). Setting the accessor methods 
-        // previously told us the type of access in turn indicating if we 
-        // needed to weave  transient value holder fields on the class. 
-        // With JPA 2.0 and the possibility of mixed access types this 
-        // assumption no longer applies.
+        // mark the mapping to weave transient field value holders (if it so 
+        // applies at weaving time). Setting the accessor methods previously 
+        // told us the type of access in turn indicating if we needed to weave
+        // transient value holder fields on the class. With JPA 2.0 and the 
+        // possibility of mixed access types this assumption no longer applies.
         ((ForeignReferenceMapping) mapping).setRequiresTransientWeavedFields(usesPropertyAccess() && ! getClassAccessor().usesPropertyAccess());
     }
     
@@ -648,38 +854,10 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      * INTERNAL:
      * Used for OX mapping.
      */
-    public void setBatchFetch(String batchFetch) {
+    public void setBatchFetch(BatchFetchMetadata batchFetch) {
         m_batchFetch = batchFetch;
     }
     
-    /**
-     * INTERNAL:
-     * Used for OX mapping.
-     */
-    public void setBatchFetchSize(Integer batchFetchSize) {
-        m_batchFetchSize = batchFetchSize;
-    }
-    
-    /**
-     * INTERNAL:
-     * Set the cascade type on a mapping.
-     */
-    protected void setCascadeType(String type, ForeignReferenceMapping mapping) {
-        if (type.equals(CascadeType.ALL.name())) {
-            mapping.setCascadeAll(true);
-        } else if(type.equals(CascadeType.MERGE.name())) {
-            mapping.setCascadeMerge(true);
-        } else if(type.equals(CascadeType.PERSIST.name())) {
-            mapping.setCascadePersist(true);
-        } else if(type.equals(CascadeType.REFRESH.name())) {
-            mapping.setCascadeRefresh(true);
-        } else if(type.equals(CascadeType.REMOVE.name())) {
-            mapping.setCascadeRemove(true);
-        } else if (type.equals(CascadeType.DETACH.name())) {
-            mapping.setCascadeDetach(true);
-        }
-    }
-    
     /**
      * INTERNAL:
      * Used for OX mapping.
@@ -688,6 +866,14 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         m_cascade = cascade;
     }
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public void setCascadeOnDelete(Boolean cascadeOnDelete) {
+        m_cascadeOnDelete = cascadeOnDelete;
+    }
+    
     /**
      * INTERNAL: 
      * Used for OX mapping.
@@ -696,6 +882,14 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         m_fetch = fetch;
     }
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public void setForeignKey(ForeignKeyMetadata foreignKey) {
+        m_foreignKey = foreignKey;
+    }
+    
     /**
      * INTERNAL: 
      * Used for OX mapping.
@@ -704,6 +898,14 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         m_joinColumns = joinColumns;
     }
     
+    /**
+     * INTERNAL: 
+     * Used for OX mapping.
+     */
+    public void setJoinFields(List<JoinFieldMetadata> joinFields) {
+        m_joinFields = joinFields;
+    }
+    
     /**
      * INTERNAL:
      * Used for OX mapping.
@@ -720,6 +922,22 @@ public abstract class RelationshipAccessor extends MappingAccessor {
         m_joinTable = joinTable;
     }
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public void setMappedBy(String mappedBy) {
+        m_mappedBy = mappedBy;
+    }
+    
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public void setNonCacheable(Boolean noncacheable) {
+        m_nonCacheable = noncacheable;
+    }
+
     /**
      * INTERNAL:
      * Used for OX mapping.
@@ -732,8 +950,8 @@ public abstract class RelationshipAccessor extends MappingAccessor {
      * INTERNAL:
      * Used for OX mapping.
      */
-    public void setPrivateOwned(String ignore) {
-        m_privateOwned = true;
+    public void setPrivateOwned(Boolean privateOwned) {
+        m_privateOwned = privateOwned;
     }
     
     /**
@@ -757,7 +975,7 @@ public abstract class RelationshipAccessor extends MappingAccessor {
     @Override
     protected boolean usesIndirection() {
         // If eager weaving is enabled, indirection is always used.
-        if (getProject().weaveEager()) {
+        if (getProject().isWeavingEagerEnabled()) {
             return true;
         }
         
