@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -17,6 +17,12 @@
  *       - 211324: Add additional event(s) support to the EclipseLink-ORM.XML Schema
  *     04/27/2010-2.1 Guy Pelletier 
  *       - 309856: MappedSuperclasses from XML are not being initialized properly
+ *     07/15/2010-2.2 Guy Pelletier 
+ *       -311395 : Multiple lifecycle callback methods for the same lifecycle event
+ *     12/01/2010-2.2 Guy Pelletier 
+ *       - 331234: xml-mapping-metadata-complete overriden by metadata-complete specification 
+ *     03/24/2011-2.3 Guy Pelletier 
+ *       - 337323: Multi-tenant with shared schema support (part 1)
  ******************************************************************************/  
 package org.eclipse.persistence.internal.jpa.metadata.listeners;
 
@@ -27,21 +33,14 @@ import java.security.PrivilegedActionException;
 
 import java.util.HashSet;
 
-import javax.persistence.PostLoad;
-import javax.persistence.PostPersist;
-import javax.persistence.PostRemove;
-import javax.persistence.PostUpdate;
-import javax.persistence.PrePersist;
-import javax.persistence.PreRemove;
-import javax.persistence.PreUpdate;
-
 import org.eclipse.persistence.descriptors.DescriptorEventListener;
 
 import org.eclipse.persistence.exceptions.ValidationException;
 
-import org.eclipse.persistence.internal.jpa.metadata.MetadataDescriptor;
 import org.eclipse.persistence.internal.jpa.metadata.ORMetadata;
 
+import org.eclipse.persistence.internal.jpa.metadata.accessors.MetadataAccessor;
+import org.eclipse.persistence.internal.jpa.metadata.accessors.classes.ClassAccessor;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAccessibleObject;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataAnnotation;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataClass;
@@ -55,21 +54,34 @@ import org.eclipse.persistence.internal.security.PrivilegedGetDeclaredMethods;
 import org.eclipse.persistence.internal.security.PrivilegedGetMethods;
 import org.eclipse.persistence.internal.security.PrivilegedNewInstanceFromClass;
 
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_POST_LOAD;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_POST_PERSIST;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_POST_REMOVE;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_POST_UPDATE;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_PRE_PERSIST;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_PRE_REMOVE;
+import static org.eclipse.persistence.internal.jpa.metadata.MetadataConstants.JPA_PRE_UPDATE;
+
 /**
  * A MetadataEntityListener and is placed on the owning entity's descriptor. 
  * Callback methods from an EntityListener require a signature on the method. 
  * Namely, they must have an Object parameter.
  * 
+ * Key notes:
+ * - any metadata mapped from XML to this class must be compared in the
+ *   equals method.
+ * - when loading from annotations, the constructor accepts the metadata
+ *   accessor this metadata was loaded from. Used it to look up any 
+ *   'companion' annotation needed for processing.
+ * - methods should be preserved in alphabetical order.
+ * 
  * @author Guy Pelletier
  * @since TopLink 10.1.3/EJB 3.0 Preview
  */
 public class EntityListenerMetadata extends ORMetadata implements Cloneable {
-    // Note: Any metadata mapped from XML to this class must be compared in the equals method.
-
     private MetadataClass m_entityListenerClass;
     
     protected EntityListener m_listener;
-    
     private String m_className;
     private String m_postLoad;
     private String m_postPersist;
@@ -81,7 +93,7 @@ public class EntityListenerMetadata extends ORMetadata implements Cloneable {
 
     /**
      * INTERNAL:
-     * Used for OX mapping.
+     * Used for XML loading.
      */
     public EntityListenerMetadata() {
         super("<entity-listener>");
@@ -89,9 +101,10 @@ public class EntityListenerMetadata extends ORMetadata implements Cloneable {
 
     /**
      * INTERNAL:
+     * Used for annotation loading.
      */
-    public EntityListenerMetadata(MetadataAnnotation entityListeners, MetadataClass entityListenerClass, MetadataAccessibleObject accessibleObject) {
-        super(entityListeners, accessibleObject);
+    public EntityListenerMetadata(MetadataAnnotation entityListeners, MetadataClass entityListenerClass, MetadataAccessor accessor) {
+        super(entityListeners, accessor);
         
         m_entityListenerClass = entityListenerClass;
     }
@@ -253,7 +266,7 @@ public class EntityListenerMetadata extends ORMetadata implements Cloneable {
     /**
      * INTERNAL:
      */
-    Object getInstance(Class cls) {
+    protected Object getInstance(Class cls) {
         try {
             if (PrivilegedAccessHelper.shouldUsePrivilegedAccess()){
                 try {
@@ -374,41 +387,38 @@ public class EntityListenerMetadata extends ORMetadata implements Cloneable {
     /**
      * INTERNAL: 
      */
-    public void process(MetadataDescriptor descriptor, ClassLoader loader, boolean isDefaultListener) {
+    public void process(ClassAccessor classAccessor, ClassLoader loader, boolean isDefaultListener) {
         // Make sure the entityListenerClass is initialized (default listeners
         // are cloned and m_entityListenerClass may be null)
         if (m_entityListenerClass == null) {
             m_entityListenerClass = getMetadataFactory().getMetadataClass(m_className);
         }
+        JPAEntityListenerHolder holder = new JPAEntityListenerHolder();
+        holder.setIsDefaultListener(Boolean.valueOf(isDefaultListener));
 
-        DescriptorEventListener listener;
-        Object entityListenerClassInstance = getInstance(getClass(m_entityListenerClass, loader));
+        holder.listenerClassName = m_entityListenerClass.getName();
         
         if (m_entityListenerClass.extendsInterface(DescriptorEventListener.class)) {
-            listener = (DescriptorEventListener) entityListenerClassInstance;
+            holder.listener = (DescriptorEventListener)getInstance(getClass(m_entityListenerClass, loader));
         } else {
             // Initialize the listener class before processing the callback methods.
-            m_listener = new EntityListener(entityListenerClassInstance, getClass(descriptor.getJavaClass(), loader));
-            
+            m_listener = new EntityListener(getClass(m_entityListenerClass, loader), getClass(classAccessor.getDescriptorJavaClass(), loader));
             // Process the callback methods defined from XML and annotations.
-            processCallbackMethods(getCandidateCallbackMethodsForEntityListener(), descriptor);
-            
-            listener = m_listener;
-        }
-        
-        // Add the listener to the descriptor.
-        if (isDefaultListener) {
-            descriptor.addDefaultEventListener(listener);
-        } else {
-            descriptor.addEntityListenerEventListener(listener);
+            processCallbackMethods(getCandidateCallbackMethodsForEntityListener(), classAccessor);
+            holder.convertToSerializableMethods(m_listener.getAllEventMethods());
+            holder.listener = m_listener;
+            m_listener.setOwningSession(getProject().getSession());
         }
+        classAccessor.getDescriptor().getClassDescriptor().getEventManager().addEntityListenerHolder(holder);
     }
-    
+
     /**
      * INTERNAL:
-     * Process the XML defined call back methods.
+     * Process the the callback methods. The XML defined callback methods are
+     * always added first, followed by those defined by annotations (only if not
+     * already defined in XML)
      */
-    protected void processCallbackMethods(Method[] methods, MetadataDescriptor descriptor) {    
+    protected void processCallbackMethods(Method[] methods, ClassAccessor classAccessor) {    
         // 1 - Set the XML specified methods first.
         if (m_postLoad != null) {
             setPostLoad(getCallbackMethod(m_postLoad, methods));
@@ -438,38 +448,39 @@ public class EntityListenerMetadata extends ORMetadata implements Cloneable {
             setPreUpdate(getCallbackMethod(m_preUpdate, methods));
         }
         
-        // 2 - Set any annotation defined methods second.
+        // 2 - Set any annotation defined methods second. We should only add
+        // add them if they were not overridden in XML.
         for (Method method : methods) {
-            MetadataMethod metadataMethod = getMetadataClass(method.getDeclaringClass()).getMethod(method.getName(), method.getParameterTypes());
-            if (metadataMethod == null) {
-                continue;
-            }
-            if (metadataMethod.isAnnotationPresent(PostLoad.class, descriptor)) {
-                setPostLoad(method);
-            }
-            
-            if (metadataMethod.isAnnotationPresent(PostPersist.class, descriptor)) {
-                setPostPersist(method);
-            }
-            
-            if (metadataMethod.isAnnotationPresent(PostRemove.class, descriptor)) {
-                setPostRemove(method);
-            }
-            
-            if (metadataMethod.isAnnotationPresent(PostUpdate.class, descriptor)) {
-                setPostUpdate(method);
-            }
-            
-            if (metadataMethod.isAnnotationPresent(PrePersist.class, descriptor)) {
-                setPrePersist(method);
-            }
-            
-            if (metadataMethod.isAnnotationPresent(PreRemove.class, descriptor)) {
-                setPreRemove(method);
-            }
-            
-            if (metadataMethod.isAnnotationPresent(PreUpdate.class, descriptor)) {
-                setPreUpdate(method);
+            MetadataMethod metadataMethod = getMetadataClass(method.getDeclaringClass().getName(), false).getMethod(method.getName(), method.getParameterTypes());
+            // Metadata method can be null when dealing with jdk methods: equals, notify, toString, wait etc.. 
+            if (metadataMethod != null) {
+                if (metadataMethod.isAnnotationPresent(JPA_POST_LOAD, classAccessor) && m_postLoad == null) {
+                    setPostLoad(method);
+                }
+                
+                if (metadataMethod.isAnnotationPresent(JPA_POST_PERSIST, classAccessor) && m_postPersist == null) {
+                    setPostPersist(method);
+                }
+                
+                if (metadataMethod.isAnnotationPresent(JPA_POST_REMOVE, classAccessor) && m_postRemove == null) {
+                    setPostRemove(method);
+                }
+                
+                if (metadataMethod.isAnnotationPresent(JPA_POST_UPDATE, classAccessor) && m_postUpdate == null) {
+                    setPostUpdate(method);
+                }
+                
+                if (metadataMethod.isAnnotationPresent(JPA_PRE_PERSIST, classAccessor) && m_prePersist == null) {
+                    setPrePersist(method);
+                }
+                
+                if (metadataMethod.isAnnotationPresent(JPA_PRE_REMOVE, classAccessor) && m_preRemove == null) {
+                    setPreRemove(method);
+                }
+                
+                if (metadataMethod.isAnnotationPresent(JPA_PRE_UPDATE, classAccessor) && m_preUpdate == null) {
+                    setPreUpdate(method);
+                }
             }
         }
     }
