@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,17 +12,28 @@
  *       - 267391: JPA 2.0 implement/extend/use an APT tooling library for MetaModel API canonical classes
  *     06/01/2010-2.1 Guy Pelletier 
  *       - 315195: Add new property to avoid reading XML during the canonical model generation
+ *     08/25/2010-2.2 Guy Pelletier 
+ *       - 309445: CannonicalModelProcessor process all files
+ *     11/23/2010-2.2 Guy Pelletier 
+ *       - 330660: Canonical model generator throws ClassCastException when using package-info.java
+ *     04/01/2011-2.3 Guy Pelletier 
+ *       - 337323: Multi-tenant with shared schema support (part 2)
+ *     09/20/2011-2.3.1 Guy Pelletier 
+ *       - 357476: Change caching default to ISOLATED for multitenant's using a shared EMF.
  ******************************************************************************/  
 package org.eclipse.persistence.internal.jpa.modelgen;
 
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 
 import javax.annotation.processing.ProcessingEnvironment;
 import javax.annotation.processing.RoundEnvironment;
 import javax.lang.model.element.Element;
+import javax.lang.model.element.TypeElement;
+import javax.lang.model.element.TypeParameterElement;
 import javax.lang.model.type.TypeMirror;
 import javax.tools.Diagnostic.Kind;
 
@@ -35,6 +46,7 @@ import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataC
 import org.eclipse.persistence.internal.jpa.modelgen.MetadataMirrorFactory;
 import org.eclipse.persistence.internal.jpa.modelgen.objects.PersistenceUnit;
 import org.eclipse.persistence.internal.jpa.modelgen.visitors.ElementVisitor;
+import org.eclipse.persistence.logging.SessionLog;
 import org.eclipse.persistence.sessions.DatabaseLogin;
 import org.eclipse.persistence.sessions.Project;
 import org.eclipse.persistence.sessions.server.ServerSession;
@@ -48,7 +60,6 @@ import org.eclipse.persistence.sessions.server.ServerSession;
  */
 public class MetadataMirrorFactory extends MetadataFactory {
     private ElementVisitor<MetadataClass, MetadataClass> elementVisitor;
-    private HashMap<MetadataClass, Boolean> roundElements;
     
     // Thing to note: persistence units can be reloaded. We do not however
     // reload their associated projects. Once the project is created, it remains 
@@ -56,10 +67,15 @@ public class MetadataMirrorFactory extends MetadataFactory {
     private Map<String, PersistenceUnit> persistenceUnits;
     private Map<String, MetadataProject> metadataProjects;
     
-    private Map<Element, MetadataClass> metadataClassesFromElements;
+    // This is a map of element/metadata classes built per compile round. This 
+    // map is cleared for each compile round.
+    private Map<Element, MetadataClass> roundElements;
+    
+    // This is a hash set of metadata classes per compile round. This set is
+    // cleared for each compile round.
+    private HashSet<MetadataClass> roundMetadataClasses;
     
     private ProcessingEnvironment processingEnv;
-    private RoundEnvironment roundEnv;
 
     /**
      * INTERNAL:
@@ -72,10 +88,10 @@ public class MetadataMirrorFactory extends MetadataFactory {
      */
     protected MetadataMirrorFactory(MetadataLogger logger, ClassLoader loader) {
         super(logger, loader);
-        roundElements = new HashMap<MetadataClass, Boolean>();
+        roundElements = new HashMap<Element, MetadataClass>();
+        roundMetadataClasses = new HashSet<MetadataClass>();
         persistenceUnits = new HashMap<String, PersistenceUnit>();
         metadataProjects = new HashMap<String, MetadataProject>();
-        metadataClassesFromElements = new HashMap<Element, MetadataClass>();
     }
     
     /**
@@ -87,42 +103,91 @@ public class MetadataMirrorFactory extends MetadataFactory {
     
     /**
      * INTERNAL:
+     * If the metadata class doesn't exist for the given element, we will build 
+     * one and add it to our map of MetadataClasses before returning it. We keep 
+     * our own map of metadata classes for each round element for a couple 
+     * reasons:
+     *   1- Most importantly, we use this map to avoid an infinite loop. Once we 
+     *      kick off the visiting of a class, it snow balls into visiting and
+     *      building other classes from the round (referenced from that class).
+     *   2- For our own safety we cache metadata class keyed on the element we 
+     *      built it for. This ensures we are always dealing with the correct 
+     *      related metadata class.
+     *   3- For each round we must update all metadata classes for each round
+     *      element. 
      */
-    protected MetadataClass buildMetadataClass(Element element) {
-        MetadataClass metadataClass = new MetadataClass(MetadataMirrorFactory.this, "");
+    public MetadataClass getMetadataClass(Element element) {
+        MetadataClass metadataClass = roundElements.get(element);
         
-        // We keep our own map of classes for a couple reasons:
-        // 1- Most importantly, we use this map to avoid an infinite loop. Once
-        // we kick off the visiting of this class, it snow balls into visiting
-        // the other classes.
-        // 2- For our own safety we cache metadata class keyed on the Element we 
-        // built it for. This ensures we are always dealing with the correct 
-        // related metadata class.
-        // 3- we can't call addMetadataClass here since it adds the class to the
-        // map keyed on its name. We won't know the class name till we visit it.
-        metadataClassesFromElements.put(element, metadataClass);
-        
-        // Kick off the visiting of elements.
-        element.accept(elementVisitor, metadataClass);
-        
-        // The name off the metadata class is a qualified name from a type
-        // element. Set this on the MetadataFactory map.
-        addMetadataClass(metadataClass);
+        if (metadataClass == null) {
+            // Only log if logging on finest.
+            boolean shouldLog = getLogger().getSession().getLogLevel() == SessionLog.FINEST;
+            // As a performance gain, avoid visiting this class if it is not a 
+            // round element. We must re-visit round elements.
+            if (isRoundElement(element)) {
+                if (shouldLog) {
+                    processingEnv.getMessager().printMessage(Kind.NOTE, "Building metadata class for round element: " + element);
+                }
+                metadataClass = new MetadataClass(MetadataMirrorFactory.this, "");
+                roundElements.put(element, metadataClass);
+                roundMetadataClasses.add(metadataClass);
+                
+                // Kick off the visiting of elements.
+                element.accept(elementVisitor, metadataClass);
+              
+                // The name of the metadata class is a qualified name from a 
+                // type element. Set this name on the MetadataFactory map. We 
+                // can't call addMetadataClass till we have visited the class.
+                addMetadataClass(metadataClass);
+            } else {
+                String name = element.toString();
+                if (metadataClassExists(name))  {
+                    return getMetadataClass(name);
+                }
+                // So we are not a round element, the outcome is as follows:
+                //  - TypeElement or TypeParameterElement in existing class map, 
+                //    return it.
+                //  - TypeElement, and not in the existing class map, return 
+                //    simple non-visited MetadataClass with only a name/type.
+                //  - TypeParameterElement, and not in the existing class map,
+                //    visit it to ensure we get the correct generic type set
+                //    and return it.
+                //  - Everything else, return simple non-visited MetadataClass 
+                //    with only a name/type from the toString value.
+                if (element instanceof TypeElement || element instanceof TypeParameterElement) {
+                    if (element instanceof TypeElement) {
+                        if (shouldLog) {
+                            processingEnv.getMessager().printMessage(Kind.NOTE, "Building metadata class for type element: " + name);
+                        }
+                        metadataClass = new MetadataClass(MetadataMirrorFactory.this, name);
+                        addMetadataClass(metadataClass);
+                        element.accept(elementVisitor, metadataClass);
+                        addMetadataClass(metadataClass);
+                    } else {
+                        // Only thing going to get through at this point are
+                        // TypeParameterElements (presumably generic ones). Look 
+                        // at those further since they 'should' be simple visits.
+                        if (shouldLog) {
+                            processingEnv.getMessager().printMessage(Kind.NOTE, "Building type parameter element: " + name);
+                        }
+                        metadataClass = new MetadataClass(MetadataMirrorFactory.this, name);
+                        addMetadataClass(metadataClass);
+                        element.accept(elementVisitor, metadataClass);
+                        addMetadataClass(metadataClass);
+                    }
+                } else {
+                    // Array types etc ...
+                    metadataClass = getMetadataClass(element.toString());
+                }
+            }
+        }
         
         return metadataClass;
     }
-    
-    /**
-     * INTERNAL:
-     * If the element doesn't exist we will build it and add it to our list of
-     * MetadataClasses before returning it.
-     */
-    public MetadataClass getMetadataClass(Element element) {
-        if (metadataClassesFromElements.containsKey(element)) {
-            return metadataClassesFromElements.get(element);
-        } else {
-            return buildMetadataClass(element);
-        }
+
+    @Override
+    public MetadataClass getMetadataClass(String className, boolean isLazy) {
+        return getMetadataClass(className);
     }
     
     /**
@@ -177,7 +242,7 @@ public class MetadataMirrorFactory extends MetadataFactory {
      */
     public MetadataProject getMetadataProject(SEPersistenceUnitInfo puInfo) {
         if (! metadataProjects.containsKey(puInfo.getPersistenceUnitName())) {
-            MetadataProject project = new MetadataProject(puInfo, new ServerSession(new Project(new DatabaseLogin())), false, false);
+            MetadataProject project = new MetadataProject(puInfo, new ServerSession(new Project(new DatabaseLogin())), false, false, false, false, false);
             metadataProjects.put(puInfo.getPersistenceUnitName(), project);
             return project;
         } else {
@@ -198,26 +263,26 @@ public class MetadataMirrorFactory extends MetadataFactory {
     public ProcessingEnvironment getProcessingEnvironment() {
         return processingEnv;
     }
-
+    
     /**
      * INTERNAL:
      */
-    public boolean isPreProcessedRoundElement(MetadataClass cls) {
-        return isRoundElement(cls) && roundElements.get(cls);
+    public Map<Element, MetadataClass> getRoundElements() {
+        return roundElements;
     }
     
     /**
      * INTENAL:
      */
     public boolean isRoundElement(Element element) {
-        return roundEnv.getRootElements().contains(element);
+        return roundElements.containsKey(element);
     }
     
     /**
      * INTENAL:
      */
     public boolean isRoundElement(MetadataClass cls) {
-        return roundElements.containsKey(cls);
+        return roundMetadataClasses.contains(cls);
     }
     
     /**
@@ -238,32 +303,39 @@ public class MetadataMirrorFactory extends MetadataFactory {
      */
     public void setEnvironments(ProcessingEnvironment processingEnvironment, RoundEnvironment roundEnvironment) {
         processingEnv = processingEnvironment;
-        roundEnv = roundEnvironment;
         roundElements.clear();
+        roundMetadataClasses.clear();
         
         // Initialize the element visitor if it is null.
         if (elementVisitor == null) {
             elementVisitor = new ElementVisitor<MetadataClass, MetadataClass>(processingEnv);
         }
         
-        // Visit all the root elements now. These may be new elements or 
+        // Go through the root elements and gather the round elements that
+        // we care about. It is crucial to not call getMetadataClass(element)
+        // before we gather our list of round elements. Calling this method will
+        // trigger the visiting of elements which has a dependency on round
+        // elements.
+        for (Element element : roundEnvironment.getRootElements()) {
+            // Look at only class elements.
+            if (element.getKind().isClass()) {
+                // Don't look at the generated classes. We must look at all the
+                // classes and not only those decorated with @Entity, 
+                // @MappedSuperclass or @Embeddable, since it may be a class
+                // defined solely in XML and we want to make sure we look at
+                // the changes for those classes as well.
+                if (element.getAnnotation(javax.annotation.Generated.class) == null) { 
+                    roundElements.put(element, null);
+                }
+            }
+        }
+        
+        // Visit all the round elements now. These may be new elements or 
         // existing elements that were changed. We must build or re-build the 
         // class metadata for that element to be re-used with new accessors
         // needing to pre-processed.
-        for (Element element : roundEnvironment.getRootElements()) {
-            if (element.getAnnotation(javax.annotation.Generated.class) == null) { 
-                processingEnv.getMessager().printMessage(Kind.NOTE, "Building metadata class for round element: " + element);
-                roundElements.put(buildMetadataClass(element), false);
-            }
-        }
-    }
-
-    /**
-     * INTERNAL:
-     */
-    public void setIsPreProcessedRoundElement(MetadataClass cls) {
-        if (isRoundElement(cls)) {
-            roundElements.put(cls, true);
+        for (Element element : roundElements.keySet()) {
+            getMetadataClass(element);
         }
     }
 }
