@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -16,6 +16,7 @@ import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.exceptions.JPQLException;
 import org.eclipse.persistence.expressions.*;
 import org.eclipse.persistence.mappings.DatabaseMapping;
+import org.eclipse.persistence.mappings.querykeys.QueryKey;
 
 /**
  * INTERNAL
@@ -39,6 +40,8 @@ public class AttributeNode extends Node {
     private DatabaseMapping mapping;
     
     private String castClassName = null;
+    
+    private QueryKey attributeQueryKey = null;
 
     /**
      * Create a new AttributeNode
@@ -81,7 +84,7 @@ public class AttributeNode extends Node {
         }
         TypeHelper typeHelper = context.getParseTreeContext().getTypeHelper();
         Class cast = (Class)typeHelper.resolveSchema(castClassName);
-        return exp.as(cast);
+        return exp.treat(cast);
     }
     
     public Object computeActualType(Object initialType, TypeHelper typeHelper){
@@ -91,6 +94,10 @@ public class AttributeNode extends Node {
         return initialType;
     }
 
+    public void checkForQueryKey(Object ownerType, TypeHelper typeHelper){
+        attributeQueryKey = typeHelper.resolveQueryKey(ownerType, name);
+    }
+    
     /** */
     public Expression addToExpression(Expression parentExpression, GenerationContext context) {
         if (isCollectionAttribute()) {
@@ -175,7 +182,7 @@ public class AttributeNode extends Node {
     /** */
     public boolean isCollectionAttribute() {
         DatabaseMapping mapping = getMapping();
-        return (mapping != null) && mapping.isCollectionMapping();
+        return ((mapping != null) && mapping.isCollectionMapping()) || (attributeQueryKey != null && attributeQueryKey.isCollectionQueryKey());
     }
 
     /**
