@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010 Oracle. All rights reserved.
+ * Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
  * which accompanies this distribution.
@@ -23,6 +23,7 @@ import java.util.Set;
 
 import javax.persistence.criteria.AbstractQuery;
 import javax.persistence.criteria.CollectionJoin;
+import javax.persistence.criteria.CommonAbstractCriteria;
 import javax.persistence.criteria.Expression;
 import javax.persistence.criteria.Join;
 import javax.persistence.criteria.ListJoin;
@@ -43,6 +44,7 @@ import org.eclipse.persistence.internal.helper.ClassConstants;
 import org.eclipse.persistence.internal.jpa.metamodel.MetamodelImpl;
 import org.eclipse.persistence.internal.jpa.metamodel.TypeImpl;
 import org.eclipse.persistence.internal.localization.ExceptionLocalization;
+import org.eclipse.persistence.queries.DatabaseQuery;
 import org.eclipse.persistence.queries.ObjectLevelReadQuery;
 import org.eclipse.persistence.queries.ReportQuery;
 
@@ -68,13 +70,13 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
     protected String alias;
     protected ReportQuery subQuery;
     protected Set<Join<?,?>> correlatedJoins;
-    protected AbstractQuery parent;
+    protected CommonAbstractCriteria parent;
     protected Set<FromImpl> processedJoins;
     protected Set<org.eclipse.persistence.expressions.Expression> correlations;
         
 
 
-    public SubQueryImpl(Metamodel metamodel, Class result, CriteriaBuilderImpl queryBuilder, AbstractQuery parent){
+    public SubQueryImpl(Metamodel metamodel, Class result, CriteriaBuilderImpl queryBuilder, CommonAbstractCriteria parent){
         super(metamodel, ResultType.OTHER, queryBuilder, result);
         this.subQuery = new ReportQuery();
         TypeImpl queryType = ((MetamodelImpl)metamodel).getType(result);
@@ -97,10 +99,6 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
      */
     public Subquery<T> select(Expression<T> selection) {
         findRootAndParameters(selection);
-        if (((InternalSelection)selection).isFrom()){
-            ((FromImpl)selection).isLeaf = false;
-        }
-        
         for (Iterator iterator = this.getRoots().iterator(); iterator.hasNext();){
             findJoins((FromImpl)iterator.next());
         }
@@ -368,14 +366,22 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
         return correlated;
 
     }
+
+    public Set<ParameterExpression<?>> getParameters() {
+        return ((CommonAbstractCriteriaImpl)this.getContainingQuery()).getParameters();
+    }
+
     /**
      * Return the query of which this is a subquery.
      * @return the enclosing query or subquery
      */
     public AbstractQuery<?> getParent(){
-        return this.parent;
+        if (parent == null || parent instanceof AbstractQuery) {
+            return (AbstractQuery) this.parent;
+        } 
+        throw new IllegalStateException("TODO.. write a better message");
     }
-    
+
     /**
      * Specify whether duplicate query results will be eliminated. A true value
      * will cause duplicates to be eliminated. A false value will cause
@@ -425,14 +431,16 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
     }
 
     public void addParameter(ParameterExpression<?> parameter){
-        ((AbstractQueryImpl)this.getParent()).addParameter(parameter);
+        ((CommonAbstractCriteriaImpl)this.getContainingQuery()).addParameter(parameter);
     }
 
     public void addJoin(FromImpl join){
-        if (this.processedJoins == null ) {this.processedJoins = new HashSet<FromImpl>();}
+        if (this.processedJoins == null ) {
+            this.processedJoins = new HashSet<FromImpl>();
+        }
         if (! this.processedJoins.contains(join)){
             this.processedJoins.add(join);
-        this.subQuery.addNonFetchJoinedAttribute(join.getCurrentNode());
+            this.subQuery.addNonFetchJoinedAttribute(join.getCurrentNode());
         }
     }
 
@@ -541,6 +549,9 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
     public boolean isConstructor(){
         return false;
     }
+    public boolean isJunction(){
+        return false;
+    }
     public boolean isPredicate(){
         return false;
     }
@@ -561,9 +572,7 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
         if (this.roots.isEmpty()) {
             TypeImpl type = ((MetamodelImpl)this.metamodel).getType(this.queryType);
             if ((type != null && type.getPersistenceType() == PersistenceType.ENTITY) || queryType.equals(ClassConstants.OBJECT)) {
-                // this is the first root, set return type and selection and
-                // query
-                // type
+                // this is the first root, set return type and selection and query type
                 if (this.selection == null) {
                     this.selection = root;
                     this.subQuery.getItems().clear();
@@ -574,11 +583,9 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
             this.subQuery.setReferenceClass(root.getJavaType());
             this.subQuery.setExpressionBuilder(root.getCurrentNode().getBuilder());
             this.queryType = root.getJavaType();
+            this.currentNode.setBaseExpression(((CommonAbstractCriteriaImpl)this.parent).getBaseExpression());
         }
-        if (!this.roots.contains(root)) {
-            this.roots.add(root);
-        }
-
+        super.integrateRoot(root);
     }
 
     public boolean isCompoundExpression(){
@@ -595,10 +602,22 @@ public class SubQueryImpl<T> extends AbstractQueryImpl<T> implements Subquery<T>
     public boolean isLiteral(){
         return false;
     }
-    public void findRootAndParameters(AbstractQueryImpl query){
+
+    public void findRootAndParameters(CommonAbstractCriteriaImpl query){
         for (Join join: this.correlatedJoins){
             ((JoinImpl)join).findRootAndParameters(query);
         }
     }
 
+    protected org.eclipse.persistence.expressions.Expression getBaseExpression() {
+        return this.currentNode.getSubQuery().getExpressionBuilder();
+    }
+
+    public CommonAbstractCriteria getContainingQuery() {
+        return this.parent;
+    }
+
+    public DatabaseQuery getDatabaseQuery() {
+        return this.subQuery;
+    }
 }
