@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,28 +9,37 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     11/10/2011-2.4 Guy Pelletier 
+ *       - 357474: Address primaryKey option from tenant discriminator column
+ *     11/29/2012-2.5 Guy Pelletier 
+ *       - 395406: Fix nightly static weave test errors
  ******************************************************************************/
 package org.eclipse.persistence.internal.jpa.weaving;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
-import org.eclipse.persistence.internal.libraries.asm.Type;
-
-import org.eclipse.persistence.indirection.ValueHolderInterface;
-import org.eclipse.persistence.logging.SessionLog;
-import org.eclipse.persistence.mappings.*;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
-import org.eclipse.persistence.descriptors.RelationalDescriptor;
-import org.eclipse.persistence.sessions.Session;
-import org.eclipse.persistence.sessions.Project;
+import org.eclipse.persistence.indirection.ValueHolderInterface;
+import org.eclipse.persistence.internal.descriptors.VirtualAttributeMethodInfo;
+import org.eclipse.persistence.internal.helper.Helper;
 import org.eclipse.persistence.internal.indirection.BasicIndirectionPolicy;
-import org.eclipse.persistence.internal.indirection.WeavedObjectBasicIndirectionPolicy;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataClass;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataField;
 import org.eclipse.persistence.internal.jpa.metadata.accessors.objects.MetadataMethod;
+import org.eclipse.persistence.internal.libraries.asm.Type;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.weaving.PersistenceWeavedChangeTracking;
-import org.eclipse.persistence.internal.helper.Helper;
+import org.eclipse.persistence.logging.SessionLog;
+import org.eclipse.persistence.mappings.DatabaseMapping;
+import org.eclipse.persistence.mappings.ForeignReferenceMapping;
+import org.eclipse.persistence.mappings.foundation.AbstractDirectMapping;
+import org.eclipse.persistence.sessions.Project;
+import org.eclipse.persistence.sessions.Session;
 
 /**
  * This class creates a ClassFileTransformer that is used for dynamic bytecode
@@ -49,7 +58,8 @@ public class TransformerFactory {
     public static final String WEAVER_CLASS_NOT_IN_PROJECT = "weaver_class_not_in_project";
     public static final String WEAVER_PROCESSING_CLASS = "weaver_processing_class";
     public static final String CANNOT_WEAVE_CHANGETRACKING = "cannot_weave_changetracking";
-    
+    public static final String CANNOT_WEAVE_VIRTUAL_ONE_TO_ONE = "cannot_weave_virtual_one_to_one";
+
     protected Session session;
     protected Collection<MetadataClass> entityClasses;
     protected Map<String, ClassDetails> classDetailsMap;
@@ -58,10 +68,11 @@ public class TransformerFactory {
     protected boolean weaveLazy;
     protected boolean weaveFetchGroups;
     protected boolean weaveInternal;
-    
+    protected boolean weaveRest;
+
     public static PersistenceWeaver createTransformerAndModifyProject(
             Session session, Collection<MetadataClass> entityClasses, ClassLoader classLoader,
-            boolean weaveLazy, boolean weaveChangeTracking, boolean weaveFetchGroups, boolean weaveInternal) {
+            boolean weaveLazy, boolean weaveChangeTracking, boolean weaveFetchGroups, boolean weaveInternal, boolean weaveRest) {
         if (session == null) {
             throw new IllegalArgumentException("Weaver session cannot be null");
         }
@@ -69,12 +80,12 @@ public class TransformerFactory {
             ((AbstractSession)session).log(SessionLog.SEVERE, SessionLog.WEAVER, WEAVER_NULL_PROJECT, null);
             throw new IllegalArgumentException("Weaver session's project cannot be null");
         }
-        TransformerFactory tf = new TransformerFactory(session, entityClasses, classLoader, weaveLazy, weaveChangeTracking, weaveFetchGroups, weaveInternal);
+        TransformerFactory tf = new TransformerFactory(session, entityClasses, classLoader, weaveLazy, weaveChangeTracking, weaveFetchGroups, weaveInternal, weaveRest);
         tf.buildClassDetailsAndModifyProject();
         return tf.buildPersistenceWeaver();
     }
-    
-    public TransformerFactory(Session session, Collection<MetadataClass> entityClasses, ClassLoader classLoader, boolean weaveLazy, boolean weaveChangeTracking, boolean weaveFetchGroups, boolean weaveInternal) {
+
+    public TransformerFactory(Session session, Collection<MetadataClass> entityClasses, ClassLoader classLoader, boolean weaveLazy, boolean weaveChangeTracking, boolean weaveFetchGroups, boolean weaveInternal, boolean weaveRest) {
         this.session = session;
         this.entityClasses = entityClasses;
         this.classLoader = classLoader;
@@ -83,8 +94,9 @@ public class TransformerFactory {
         this.weaveChangeTracking = weaveChangeTracking;
         this.weaveFetchGroups = weaveFetchGroups;
         this.weaveInternal = weaveInternal;
+        this.weaveRest = weaveRest;
     }
-    
+
     /**
      * INTERNAL:
      * Look higher in the hierarchy for the mappings listed in the unMappedAttribute list.
@@ -92,39 +104,39 @@ public class TransformerFactory {
      * We assume that if a mapping exists, the attribute must either be mapped from the owning
      * class or from a superclass.
      */
-    public void addClassDetailsForMappedSuperClasses(MetadataClass clz, RelationalDescriptor initialDescriptor, ClassDetails classDetails, Map classDetailsMap, List unMappedAttributes, boolean weaveChangeTracking){
-        // This class has inheritance to a mapped entity rather than a MappedSuperClass
-        if (initialDescriptor.getInheritancePolicyOrNull() != null && initialDescriptor.getInheritancePolicyOrNull().getParentClass() != null){
-            return;
-        }
-        
-        if (unMappedAttributes.isEmpty()){
-            return;
-        }
-        
+    public void addClassDetailsForMappedSuperClasses(MetadataClass clz, ClassDescriptor initialDescriptor, ClassDetails classDetails, Map classDetailsMap, List unMappedAttributes, boolean weaveChangeTracking){
         MetadataClass superClz = clz.getSuperclass();
         if (superClz == null || superClz.isObject()){
             return;
         }
-       
+        
+        ClassDescriptor mappedSuperclassDescriptor = ((AbstractSession) session).getMappedSuperclass(superClz.getName());
+        if (mappedSuperclassDescriptor == null) {
+            ClassDescriptor descriptor = findDescriptor(session.getProject(), clz.getSuperclass().getName());
+            if (descriptor != null) {
+                return;
+            }
+        }
+        
         boolean weaveValueHolders = canWeaveValueHolders(superClz, unMappedAttributes);
 
         List stillUnMappedMappings = null;
-        ClassDetails superClassDetails = createClassDetails(superClz, weaveValueHolders, weaveChangeTracking, weaveFetchGroups, weaveInternal);
+        ClassDetails superClassDetails = createClassDetails(superClz, weaveValueHolders, weaveChangeTracking, weaveFetchGroups, weaveInternal, weaveRest);
         superClassDetails.setIsMappedSuperClass(true);
-        if (!initialDescriptor.usesPropertyAccessForWeaving()){
+        
+        if (mappedSuperclassDescriptor != null && ! mappedSuperclassDescriptor.usesPropertyAccessForWeaving()) {
+            superClassDetails.useAttributeAccess();
+        } else if (!initialDescriptor.usesPropertyAccessForWeaving()){
             superClassDetails.useAttributeAccess();
         }
+        
         if (!classDetailsMap.containsKey(superClassDetails.getClassName())){
             stillUnMappedMappings = storeAttributeMappings(superClz, superClassDetails, unMappedAttributes, weaveValueHolders);
             classDetailsMap.put(superClassDetails.getClassName() ,superClassDetails);
-        }
-       
-        if (((stillUnMappedMappings != null) && (stillUnMappedMappings.size() > 0))){
             addClassDetailsForMappedSuperClasses(superClz, initialDescriptor, classDetails, classDetailsMap, stillUnMappedMappings, weaveChangeTracking);
         }
     }
-    
+
     public PersistenceWeaver buildPersistenceWeaver() {
         return new PersistenceWeaver(session, classDetailsMap);
     }
@@ -139,7 +151,7 @@ public class TransformerFactory {
             for (MetadataClass metaClass : entityClasses) {
                 // check to ensure that class is present in project
                 // this will be a relational descriptor because MetadataClass only describes relational descriptors
-                RelationalDescriptor descriptor = (RelationalDescriptor)findDescriptor(session.getProject(), metaClass.getName());
+                ClassDescriptor descriptor = findDescriptor(session.getProject(), metaClass.getName());
                 if (descriptor == null) {
                     log(SessionLog.FINER, WEAVER_CLASS_NOT_IN_PROJECT, new Object[]{metaClass.getName()});
                 } else {
@@ -148,30 +160,25 @@ public class TransformerFactory {
                     boolean weaveValueHoldersForClass = weaveLazy && canWeaveValueHolders(metaClass, descriptor.getMappings());
                     boolean weaveChangeTrackingForClass = canChangeTrackingBeEnabled(descriptor, metaClass, weaveChangeTracking);
                     
-                    ClassDetails classDetails = createClassDetails(metaClass, weaveValueHoldersForClass, weaveChangeTrackingForClass, weaveFetchGroups, weaveInternal);
+                    ClassDetails classDetails = createClassDetails(metaClass, weaveValueHoldersForClass, weaveChangeTrackingForClass, weaveFetchGroups, weaveInternal, weaveRest);
                     if (descriptor.isDescriptorTypeAggregate()) {
                         classDetails.setIsEmbedable(true);
-                        classDetails.setShouldWeaveFetchGroups(false);
+//                        classDetails.setShouldWeaveFetchGroups(false);
                     }
                     if (!descriptor.usesPropertyAccessForWeaving()){
                         classDetails.useAttributeAccess();
                     }
+                    
+                    classDetails.getVirtualAccessMethods().addAll(descriptor.getVirtualAttributeMethods());
+
                     List unMappedAttributes = storeAttributeMappings(metaClass, classDetails, descriptor.getMappings(), weaveValueHoldersForClass);
                     classDetailsMap.put(classDetails.getClassName() ,classDetails);
-                     
+
                     classDetails.setShouldWeaveConstructorOptimization((classDetails.getDescribedClass().getFields().size() - (descriptor.getMappings().size() - unMappedAttributes.size()))<=0);
 
-                    if (!unMappedAttributes.isEmpty()){
+                    if (classDetails.getSuperClassName() != null) {
                         addClassDetailsForMappedSuperClasses(metaClass, descriptor, classDetails, classDetailsMap, unMappedAttributes, weaveChangeTracking);
                     }
-                    if (classDetails.getLazyMappings() != null){
-                        Iterator iterator = classDetails.getLazyMappings().iterator();
-                        while (iterator.hasNext()) {
-                            ForeignReferenceMapping mapping = (ForeignReferenceMapping)iterator.next();
-                            mapping.setGetMethodName(ClassWeaver.getWeavedValueHolderGetMethodName(mapping.getAttributeName()));
-                            mapping.setSetMethodName(ClassWeaver.getWeavedValueHolderSetMethodName(mapping.getAttributeName()));
-                        }
-                    }
                 }
             }
 
@@ -180,7 +187,7 @@ public class TransformerFactory {
                 ClassDetails classDetails = (ClassDetails)i.next();
                 ClassDetails superClassDetails = classDetailsMap.get(classDetails.getSuperClassName());
                 if (superClassDetails == null) {
-                    ClassDescriptor descriptor = findDescriptor(session.getProject(), classDetails.getClassName());
+                    ClassDescriptor descriptor = findDescriptor(session.getProject(), classDetails.getDescribedClass().getName());
                     if (descriptor != null && descriptor.hasInheritance()){
                         superClassDetails = classDetailsMap.get(descriptor.getInheritancePolicy().getParentClassName());
                     }
@@ -189,7 +196,7 @@ public class TransformerFactory {
                     classDetails.setSuperClassDetails(superClassDetails);
                 }
             }
-            
+
             // Fix weaveChangeTracking based on superclasses,
             // we should only weave change tracking if our whole hierarchy can.
             for (Iterator i = classDetailsMap.values().iterator(); i.hasNext();) {
@@ -198,7 +205,7 @@ public class TransformerFactory {
             }
         }
     }
-    
+
     /**
      * Check to ensure the class meets all the conditions necessary to enable change tracking
      * This could occur either if the class already has change tracking implemented or if the
@@ -223,7 +230,7 @@ public class TransformerFactory {
         }
         return canWeaveChangeTracking;
     }
-    
+
     protected boolean wasChangeTrackingAlreadyWeaved(Class clz){
         Class[] interfaces = clz.getInterfaces();
         for (int i = 0; i < interfaces.length; i++) {
@@ -256,7 +263,7 @@ public class TransformerFactory {
         return weaveValueHolders;
     }
 
-    private ClassDetails createClassDetails(MetadataClass metadataClass, boolean weaveValueHolders, boolean weaveChangeTracking, boolean weaveFetchGroups, boolean weaveInternal) {
+    private ClassDetails createClassDetails(MetadataClass metadataClass, boolean weaveValueHolders, boolean weaveChangeTracking, boolean weaveFetchGroups, boolean weaveInternal, boolean weaveRest) {
         // compose className in JVM 'slash' format
         // instead of regular Java 'dotted' format
         String className = Helper.toSlashedClassName(metadataClass.getName());
@@ -269,11 +276,12 @@ public class TransformerFactory {
         classDetails.setShouldWeaveChangeTracking(weaveChangeTracking);
         classDetails.setShouldWeaveFetchGroups(weaveFetchGroups);
         classDetails.setShouldWeaveInternal(weaveInternal);
-        MetadataMethod method = metadataClass.getMethod("clone", new ArrayList());
+        classDetails.setShouldWeaveREST(weaveRest);
+        MetadataMethod method = metadataClass.getMethod("clone", new ArrayList(), false);
         classDetails.setImplementsCloneMethod(method != null);
         return classDetails;
     }
-    
+
     /**
      * Find a descriptor by name in the given project
      * used to avoid referring to descriptors by class.
@@ -289,7 +297,7 @@ public class TransformerFactory {
         }
         return null;
     }
-    
+
     /**
      * Return if the class contains the field.
      */
@@ -305,14 +313,16 @@ public class TransformerFactory {
         MetadataField field = metadataClass.getField(attributeName);
         return field.getDeclaringClass();
     }
-    
+
     /**
      * Use the database mapping for an attribute to find it's type.  The type returned will either be
      * the field type of the field in the object or the type returned by the getter method.
      */
     private MetadataClass getAttributeTypeFromClass(MetadataClass metadataClass, String attributeName, DatabaseMapping mapping, boolean checkSuperclass){       
         String getterMethod = mapping.getGetMethodName();
-        if (mapping != null && getterMethod != null) {
+        if (mapping.isAbstractDirectMapping() && mapping.getAttributeAccessor().isVirtualAttributeAccessor()){
+            return metadataClass.getMetadataClass(((AbstractDirectMapping)mapping).getAttributeClassificationName());
+        } else if (mapping != null && getterMethod != null) {
             MetadataMethod method = metadataClass.getMethod(getterMethod, new ArrayList(), checkSuperclass);
             if (method == null) {
                 return null;
@@ -338,20 +348,32 @@ public class TransformerFactory {
         Map<String, AttributeDetails> attributesMap = new HashMap<String, AttributeDetails>();
         Map<String, AttributeDetails> settersMap = new HashMap<String, AttributeDetails>();
         Map<String, AttributeDetails> gettersMap = new HashMap<String, AttributeDetails>();
-        List lazyMappings = new ArrayList();
 
         for (Iterator iterator = mappings.iterator(); iterator.hasNext();) {
             DatabaseMapping mapping = (DatabaseMapping)iterator.next();
+            
+            // Can't weave something that isn't really there and not going to be there.
+            if (mapping.isMultitenantPrimaryKeyMapping()) {
+                continue;
+            }
+            
             String attribute = mapping.getAttributeName();
             AttributeDetails attributeDetails = new AttributeDetails(attribute, mapping);
-            
+
+            if (mapping.getAttributeAccessor().isVirtualAttributeAccessor()){
+                attributeDetails.setVirtualProperty(mapping.getAttributeAccessor().isVirtualAttributeAccessor());
+                if ((classDetails.getInfoForVirtualGetMethod(mapping.getGetMethodName()) == null) && (classDetails.getInfoForVirtualSetMethod(mapping.getSetMethodName()) == null)){
+                    VirtualAttributeMethodInfo info = new VirtualAttributeMethodInfo(mapping.getGetMethodName(), mapping.getSetMethodName());
+                    classDetails.getVirtualAccessMethods().add(info);
+                }
+            }
             // Initial look for the type of this attribute.
             MetadataClass typeClass = getAttributeTypeFromClass(metadataClass, attribute, mapping, false);
             if (typeClass == null) {
                 attributeDetails.setAttributeOnSuperClass(true);
                 unMappedAttributes.add(mapping);
             }
-            
+
             // Set the getter and setter method names if the mapping uses property access.
             if (mapping.getGetMethodName() != null) {
                 gettersMap.put(mapping.getGetMethodName(), attributeDetails);
@@ -360,7 +382,7 @@ public class TransformerFactory {
                     settersMap.put(mapping.getSetMethodName(), attributeDetails);
                     attributeDetails.setSetterMethodName(mapping.getSetMethodName());
                 }
-                
+
                 if (mapping.isForeignReferenceMapping() && ((ForeignReferenceMapping) mapping).requiresTransientWeavedFields()) {
                     attributeDetails.setWeaveTransientFieldValueHolders();
                 }
@@ -380,25 +402,29 @@ public class TransformerFactory {
             // Check for lazy/value-holder indirection.
             if (mapping.isForeignReferenceMapping()) {
                 ForeignReferenceMapping foreignReferenceMapping = (ForeignReferenceMapping)mapping;
-                
+
                 // repopulate the reference class with the target of this mapping
                 attributeDetails.setReferenceClassName(foreignReferenceMapping.getReferenceClassName());
                 if (attributeDetails.getReferenceClassName() != null) {
                     MetadataClass referenceClass = metadataClass.getMetadataFactory().getMetadataClass(attributeDetails.getReferenceClassName());
                     attributeDetails.setReferenceClassType(Type.getType(referenceClass.getTypeName()));
                 }
-                
+
                 // This time, look up the type class and include the superclass so we can check for lazy.
                 if (typeClass == null){
                     typeClass = getAttributeTypeFromClass(metadataClass, attribute, foreignReferenceMapping, true);
                 }
                 if (weaveValueHolders && (foreignReferenceMapping.getIndirectionPolicy() instanceof BasicIndirectionPolicy) &&
                         (typeClass != null)  && (!typeClass.extendsInterface(ValueHolderInterface.class))) {
-                    lazyMappings.add(foreignReferenceMapping);
-                    attributeDetails.weaveVH(weaveValueHolders, foreignReferenceMapping);
+                    if (mapping.isObjectReferenceMapping() && attributeDetails.isVirtualProperty()){
+                        classDetails.setShouldWeaveValueHolders(false);
+                        log(SessionLog.WARNING, CANNOT_WEAVE_VIRTUAL_ONE_TO_ONE, new Object[]{classDetails.getClassName(), attributeDetails.getAttributeName()});
+                    } else {
+                        attributeDetails.weaveVH(weaveValueHolders, foreignReferenceMapping);
+                    }
                 }
             }
-            
+
             if (attributeDetails.getReferenceClassType() == null){
                 if (typeClass == null){
                     typeClass = getAttributeTypeFromClass(metadataClass, attribute, mapping, true);
@@ -413,13 +439,12 @@ public class TransformerFactory {
         classDetails.setAttributesMap(attributesMap);
         classDetails.setGetterMethodToAttributeDetails(gettersMap);
         classDetails.setSetterMethodToAttributeDetails(settersMap);
-        classDetails.setLazyMappings(lazyMappings);
         return unMappedAttributes;
     }
-    
+
     protected void log(int level, String msg, Object[] params) {
         ((org.eclipse.persistence.internal.sessions.AbstractSession)session).log(level,
             SessionLog.WEAVER, msg, params);
     }
-    
+
 }
