@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,8 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     09/13/2013-2.5 Guy Pelletier 
+ *       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls
  ******************************************************************************/  
 package org.eclipse.persistence.internal.localization.i18n;
 
@@ -33,6 +35,8 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                            { "stack_of_visited_objects_that_refer_to_the_corrupt_object", "stack of visited objects that refer to the corrupt object: {0}" },
                                            { "corrupt_object_referenced_through_mapping", "corrupt object referenced through mapping: {0}" },
                                            { "corrupt_object", "corrupt object: {0}" },
+                                           { "begin_unit_of_work_flush", "begin unit of work flush" },
+                                           { "end_unit_of_work_flush", "end unit of work flush" },
                                            { "begin_unit_of_work_commit", "begin unit of work commit" },
                                            { "end_unit_of_work_commit", "end unit of work commit" },
                                            { "resume_unit_of_work", "resume unit of work" },
@@ -48,8 +52,8 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                            { "register_new", "Register the new object {0}" },
                                            { "register_new_bean", "Register the new bean {0}" },
                                            { "register", "Register the object {0}" },
-                                           { "register_new_for_persist", "PERSIST operation called on: {0}." },
-                                           { "deleting_object", "The remove operation has been performed on: {0}"},
+                                           { "register_new_for_persist", "persist() operation called on: {0}." },
+                                           { "deleting_object", "remove() operation called on: {0}"},
                                            { "revert", "Revert the object''s attributes {0}" },
                                            { "unregister", "Unregister the object {0}" },
                                            { "begin_batch_statements", "Begin batch statements" },
@@ -224,7 +228,7 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                            { "acquire_client_session_broker", "acquire client session broker" },
                                            { "releasing_client_session_broker", "releasing client session broker" },
                                            { "client_released", "client released" },
-                                           { "client_acquired", "client acquired" },
+                                           { "client_acquired", "client acquired: {0}" },
                                            { "tracking_pl_object", "track pessimistic locked object {0} with UnitOfWork {1}" },
                                            { "instantiate_pl_relationship", "instantiate pessimistic locking relationship when relationship is accessed in a new transaction." },
                                            { "descriptor_xml_not_in_jar", "The descriptor file ({0}) is not found in jar({1}) file, no migration therefore will be performed for this jar." },
@@ -246,6 +250,7 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                            { "weaved_fetchgroups", "Weaved fetch groups (FetchGroupTracker) [{0}]."},
                                            { "weaved_changetracker", "Weaved change tracking (ChangeTracker) [{0}]."},
                                            { "weaved_persistenceentity", "Weaved persistence (PersistenceEntity) [{0}]."},
+                                           { "weaved_rest", "Weaved REST [{0}]."},
                                            { "cmp_init_invoke_predeploy", "JavaSECMPInitializer - predeploying {0}."},
                                            { "cmp_init_register_transformer", "JavaSECMPInitializer - registering transformer for {0}."},
                                            { "cmp_init_tempLoader_created", "JavaSECMPInitializer - created temporary ClassLoader: {0}."},
@@ -259,9 +264,9 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                            { "cmp_init_initialize_from_main", "JavaSECMPInitializer - initializing from main."},
                                            { "cmp_init_initialize_from_agent", "JavaSECMPInitializer - initializing from agent."},
 
-                                           { "dbPlatformHelper_detectedVendorPlatform", "Detected Vendor platform: {0}"},
-                                           { "dbPlatformHelper_regExprDbPlatform", "DBPlatform: {1}, RegularExpression: {0}."},
-                                           { "dbPlatformHelper_patternSyntaxException", "Exception while using regExpr : {0}." },
+                                           { "dbPlatformHelper_detectedVendorPlatform", "Detected database platform: {0}"},
+                                           { "dbPlatformHelper_regExprDbPlatform", "Database platform: {1}, regular expression: {0}"},
+                                           { "dbPlatformHelper_patternSyntaxException", "Exception while using regular expression: {0}" },
                                            { "unknown_query_hint", "query {0}: unknown query hint {1} will be ignored"},
                                            { "query_hint", "query {0}: query hint {1}; value {2}"},
                                            { "property_value_specified", "property={0}; value={1}"},
@@ -275,6 +280,8 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                            { "deploy_end", "End deploying Persistence Unit {0}; session {1}; state {2}; factoryCount {3}"},
                                            { "undeploy_begin", "Begin undeploying Persistence Unit {0}; session {1}; state {2}; factoryCount {3}"},
                                            { "undeploy_end", "End undeploying Persistence Unit {0}; session {1}; state {2}; factoryCount {3}"},
+                                           { "composite_member_begin_call", "Begin {0} on composite member Persistence Unit {1}; state {2}"},
+                                           { "composite_member_end_call", "End {0} on composite member Persistence Unit {1}; state {2}"},
                                            { "loading_session_xml", "Loading persistence unit from sessions-xml file: {0}, session-name: {1}"},
                                            { "sessions_xml_path_where_session_load_from", "Resource path found for sessions-xml file: {0}"},
 
@@ -305,8 +312,8 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                             { "sdo_type_generation_processing_type_as", "{0}: Generating Type  [{1}] as [{2}]."},
                                             { "sdo_type_generation_modified_class_naming_format_to", "{0}: Generated Type   [{1}] java class name capitalized to [{2}] to follow class naming conventions."},
                                             { "sdo_type_generation_modified_function_naming_format_to", "{0}: Generated Type   [{1}] java get/set method name capitalized to [{2}] to follow class naming conventions."},
-                                            { "registered_mbean", "Registered MBean: {0}" },
-                                            { "unregistering_mbean", "Unregistering MBean: {0}" },
+                                            { "registered_mbean", "Registered MBean: {0} on server {1}" },
+                                            { "unregistering_mbean", "Unregistering MBean: {0} on server {1}" },
                                             { "mbean_get_application_name", "The applicationName for the MBean attached to session [{0}] is [{1}]" },
                                             { "mbean_get_module_name", "The moduleName for the MBean attached to session [{0}] is [{1}]" },
                                             { "active_thread_is_different_from_current_thread", "Forcing the activeThread \"{0}\" on the mergeManager \"{1}\" to be the currentThread \"{2}\" because they are different." },
@@ -321,8 +328,34 @@ public class TraceLocalizationResource extends ListResourceBundle {
                                             { "metamodel_canonical_model_class_found", "Canonical Metamodel class [{0}] found and instantiated during initialization."},
                                             { "metamodel_relationaldescriptor_javaclass_null_on_managedType", "Metamodel processing: The javaClass field is null for the relationalDescriptor [{0}] for the managedType [{1}]." },                                            
                                             { "metamodel_attribute_getmember_is_null", "The java Member is null for the Attribute [{0}] with managedType [{1}] and descriptor [{2}]." },                                            
-                                            { "metamodel_typeImpl_javaClass_should_not_be_null", "The metamodel TypeImpl.javaClass field should not be set to null for the Type [{0}]." }                                            
-
+                                            { "metamodel_typeImpl_javaClass_should_not_be_null", "The metamodel TypeImpl.javaClass field should not be set to null for the Type [{0}] with name [{1}]." },                                            
+                                            { "jmx_mbean_classloader_in_use", "EclipseLink JMX Runtime Services is referencing the [{0}] ClassLoader at: [{1}]" },
+                                            { "metamodel_itentifiableType_javaclass_null_cannot_set_supertype", "Metamodel processing: Unable to set the superclass Hierarchy because the javaClass field is null for the relationalDescriptor [{0}] for the identifiableType [{1}]." },
+                                            { "metamodel_relationaldescriptor_not_fully_initialized_yet", "Metamodel processing: The relationalDescriptor [{0}] for the managedType [{1}] is not fully initialized yet - the Metamodel instance will be incomplete before at least one entityManger session login (after a full deploy)." },
+                                            { "no_weaved_vh_method_found_verify_weaving_and_module_order", "An expected weaving method [{0}] was not found on the accessor [{2}] on the mapping [{1}] - verify that the processing order of your modules places the one containing a persistence unit ahead of modules that use it in your deployment descriptor, or disable weaving for the persistence context or the mapping using FetchType.EAGER." },
+                                            { "proxy_connection_customizer_already_proxy_session", "{0}:{1}: proxy session with unknown properties is already opened. Closing it."},
+                                            { "proxy_connection_customizer_opened_proxy_session",  "{0}:{1}: opened proxy session."},
+                                            { "proxy_connection_customizer_closing_proxy_session", "{0}:{1}: closing proxy session."},
+                                            // Following 5 logs are from LoggingLocalization - required here when using a non-platform WeblogicEclipseLinkLog
+                                            { "jmx_mbean_runtime_services_registration_encountered_multiple_mbeanserver_instances", "Multiple [{0}] JMX MBeanServer instances exist, we will use the server at index [{1}] : [{2}]." },
+                                            { "jmx_mbean_runtime_services_registration_mbeanserver_print", "JMX MBeanServer instance found: [{0}], # of beans: [{1}], domain: [{2}] at index: [{3}]." },
+                                            { "jmx_mbean_runtime_services_switching_to_alternate_mbeanserver", "JMX MBeanServer in use: [{0}] from index [{1}] " },
+                                            { "jmx_unregistered_mbean", "Unregistered MBean [{0}] from MBeanServer [{1}]." },
+                                            { "jmx_unable_to_unregister_mbean", "Unable to unregister MBean [{0}] because the MBeanServer is null. Verify that your ServerPlatform is JMX enabled." },
+                                            { "releasing_invalid_lock", "A lock has been encountered where the thread: {0} is no longer active.  The lock on object class :{1} id: {2} has been forcibly released"},
+                                            { "acquire_connection", "Connection acquired from connection pool [{0}]." },
+                                            { "release_connection", "Connection released to connection pool [{0}]." },
+                                            { "failover", "Connection pool [{0}] is dead, failing over to poll [{1}]." },
+                                            { "release_connection", "Connection released to connection pool [{0}]." },
+                                            { "max_time_exceeded_for_acquirerequiredlocks_wait", "MAX TIME {0} seconds EXCEEDED FOR WRITELOCKMANAGER WAIT.  Waiting on Entity type: {1}with pk: {2} currently locked by thread: {3} with the following trace:\n"},
+                                            { "metamodel_type_collection_empty", "The collection of metamodel types is empty. Model classes may not have been found during entity search for Java SE and some Java EE container managed persistence units.  Please verify that your entity classes are referenced in persistence.xml using either <class> elements or a global <exclude-unlisted-classes>false</exclude-unlisted-classes> element" },
+                                            { "dcn_registering", "Registering for database change event notification." },
+                                            { "dcn_change_event", "Receieved database change event [{0}]." },
+                                            { "dcn_invalidate", "Invalidating cache key [{0}] from database change event for class [{1}]." },
+                                            { "dcn_register_table", "Registering table [{0}] for database change event notification." },
+                                            { "dcn_unregister", "Removing registering for database change event notification." },
+                                            
+                                            { "exception_caught_closing_statement", "Exception caught when trying to close the query statement [{0}]." },
     };
 
     /**
