@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -20,7 +20,12 @@ import java.awt.image.BufferedImage;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.StringReader;
+import java.util.ArrayList;
 import java.util.Iterator;
+import java.util.List;
+
 import javax.activation.DataHandler;
 import javax.imageio.ImageIO;
 import javax.imageio.ImageWriter;
@@ -33,10 +38,10 @@ import javax.xml.transform.stream.StreamSource;
 
 import org.eclipse.persistence.exceptions.ConversionException;
 import org.eclipse.persistence.exceptions.XMLMarshalException;
-import org.eclipse.persistence.internal.helper.ClassConstants;
+import org.eclipse.persistence.internal.core.helper.CoreClassConstants;
+import org.eclipse.persistence.internal.core.queries.CoreContainerPolicy;
+import org.eclipse.persistence.internal.core.sessions.CoreAbstractSession;
 import org.eclipse.persistence.internal.helper.Helper;
-import org.eclipse.persistence.internal.sessions.AbstractSession;
-import org.eclipse.persistence.oxm.XMLMarshaller;
 
 public class XMLBinaryDataHelper {
     protected static XMLBinaryDataHelper binaryDataHelper;
@@ -70,7 +75,20 @@ public class XMLBinaryDataHelper {
         MULTIPART = javax.mail.internet.MimeMultipart.class;
     }
 
-    public Object convertObject(Object obj, Class classification, AbstractSession session) {
+    public Object convertObject(Object obj, Class classification, CoreAbstractSession session, CoreContainerPolicy cp) {
+    	if( obj instanceof List && cp != null){
+    		List theList = (List)obj;
+    		Object container = cp.containerInstance(theList.size());
+    		for(int i=0; i<theList.size(); i++){
+    			Object next = theList.get(i);
+    			cp.addInto(convertSingleObject(next, classification, session), container, session);
+    		}
+    		return container;
+    	}
+    	return convertSingleObject(obj, classification, session);
+        
+    }
+    public Object convertSingleObject(Object obj, Class classification, CoreAbstractSession session) {
         if (classification == DATA_HANDLER) {
             return convertObjectToDataHandler(obj, session);
         } else if (classification == IMAGE) {
@@ -94,8 +112,24 @@ public class XMLBinaryDataHelper {
 
         return new EncodedData(output.toByteArray(), handler.getContentType());
     }
+    
+    public List<byte[]> getBytesListForBinaryValues(List attributeValue, Marshaller marshaller, String mimeType){
+		List returnList = new ArrayList(attributeValue.size());
+		for(int i=0;i<attributeValue.size(); i++){
+			Object next = attributeValue.get(i);
+			EncodedData nextEncodedData = getBytesForSingleBinaryValue(next, marshaller, mimeType); 
+			returnList.add(nextEncodedData.getData());
+		}
+		return returnList;
+    }
+
+    public EncodedData getBytesForBinaryValue(Object attributeValue, Marshaller marshaller, String mimeType) {
+    	return getBytesForSingleBinaryValue(attributeValue, marshaller, mimeType);
+    	
+    }
+    
+    public EncodedData getBytesForSingleBinaryValue(Object attributeValue, Marshaller marshaller, String mimeType) {
 
-    public EncodedData getBytesForBinaryValue(Object attributeValue, XMLMarshaller marshaller, String mimeType) {
         if (attributeValue instanceof DataHandler) {
             return getBytesFromDataHandler((DataHandler) attributeValue);
         } else if (attributeValue instanceof Image) {
@@ -104,16 +138,16 @@ public class XMLBinaryDataHelper {
             return getBytesFromSource((Source) attributeValue, marshaller, mimeType);
         } else if (attributeValue instanceof MimeMultipart) {
             return getBytesFromMultipart((MimeMultipart) attributeValue, marshaller);
-        } else if (attributeValue.getClass() == ClassConstants.APBYTE) {
+        } else if (attributeValue.getClass() == CoreClassConstants.APBYTE) {
             return new EncodedData((byte[]) attributeValue, mimeType);
-        } else if (attributeValue.getClass() == ClassConstants.ABYTE) {
+        } else if (attributeValue.getClass() == CoreClassConstants.ABYTE) {
             return getBytesFromByteObjectArray((Byte[]) attributeValue, mimeType);
         }
 
         return new EncodedData(new byte[0], null);
     }
 
-    public EncodedData getBytesFromMultipart(MimeMultipart value, XMLMarshaller marshaller) {
+    public EncodedData getBytesFromMultipart(MimeMultipart value, Marshaller marshaller) {
         ByteArrayOutputStream output = new ByteArrayOutputStream();
         try {
             ContentType contentType = new ContentType(value.getContentType());
@@ -133,7 +167,7 @@ public class XMLBinaryDataHelper {
         return new EncodedData(output.toByteArray(), value.getContentType());
     }
 
-    public EncodedData getBytesFromSource(Source source, XMLMarshaller marshaller, String mimeType) {
+    public EncodedData getBytesFromSource(Source source, Marshaller marshaller, String mimeType) {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         StreamResult result = new StreamResult(out);
         marshaller.getTransformer().transform(source, result);
@@ -273,21 +307,21 @@ public class XMLBinaryDataHelper {
         return null;
     }
 
-    public String stringFromDataHandler(DataHandler source, QName schemaTypeQName, AbstractSession session) {
+    public String stringFromDataHandler(DataHandler source, QName schemaTypeQName, CoreAbstractSession session) {
         ByteArrayOutputStream output = new ByteArrayOutputStream();
         try {
             source.writeTo(output);
             return (String) ((XMLConversionManager) session.getDatasourcePlatform().getConversionManager()).convertObject(output.toByteArray(), String.class, schemaTypeQName);
         } catch (Exception ex) {
-            throw ConversionException.couldNotBeConverted(source, ClassConstants.STRING, ex);
+            throw ConversionException.couldNotBeConverted(source, CoreClassConstants.STRING, ex);
         }
     }
 
-    public String stringFromDataHandler(Object source, QName schemaTypeQName, AbstractSession session) {
+    public String stringFromDataHandler(Object source, QName schemaTypeQName, CoreAbstractSession session) {
         return stringFromDataHandler((DataHandler) source, schemaTypeQName, session);
     }
 
-    public String stringFromImage(Image image, QName schemaTypeQName, AbstractSession session) {
+    public String stringFromImage(Image image, QName schemaTypeQName, CoreAbstractSession session) {
         ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
         try {
             String mimeType = null;
@@ -310,17 +344,17 @@ public class XMLBinaryDataHelper {
         }
     }
 
-    public String stringFromSource(Source source, QName schemaTypeQName, AbstractSession session) {
+    public String stringFromSource(Source source, QName schemaTypeQName, CoreAbstractSession session) {
         DataHandler handler = new DataHandler(source, "text/xml");
         return stringFromDataHandler(handler, schemaTypeQName, session);
     }
 
-    public String stringFromMultipart(MimeMultipart multipart, QName schemaTypeQName, AbstractSession session) {
+    public String stringFromMultipart(MimeMultipart multipart, QName schemaTypeQName, CoreAbstractSession session) {
         DataHandler handler = new DataHandler(multipart, multipart.getContentType());
         return stringFromDataHandler(handler, schemaTypeQName, session);
     }
 
-    public DataHandler convertObjectToDataHandler(Object sourceObject, AbstractSession session) {
+    public DataHandler convertObjectToDataHandler(Object sourceObject, CoreAbstractSession session) {
         DataHandler handler = null;
         if (sourceObject instanceof DataHandler) {
             return (DataHandler) sourceObject;
@@ -329,7 +363,7 @@ public class XMLBinaryDataHelper {
             handler = new DataHandler(new ByteArrayDataSource(bytes, "application/octet-stream"));
 
         } else if (sourceObject instanceof Byte[]) {
-            byte[] bytes = (byte[]) session.getDatasourcePlatform().getConversionManager().convertObject(sourceObject, ClassConstants.APBYTE);
+            byte[] bytes = (byte[]) session.getDatasourcePlatform().getConversionManager().convertObject(sourceObject, CoreClassConstants.APBYTE);
             handler = new DataHandler(new ByteArrayDataSource(bytes, "application/octet-stream"));
         }
         if (sourceObject instanceof String) {
@@ -347,26 +381,38 @@ public class XMLBinaryDataHelper {
     }
 
     public Object convertObjectToSource(Object obj) {
+        if(obj == null) {
+            return null;
+        }
         if (obj instanceof Source) {
             return obj;
         }
+        if(obj.getClass() == CoreClassConstants.STRING) {
+            return new StreamSource(new StringReader((String)obj));
+        }        
         if (obj instanceof DataHandler) {
             try {
-                Source source = (Source) ((DataHandler) obj).getContent();
-                return source;
+                InputStream object = ((DataHandler)obj).getInputStream();
+                return new StreamSource(object);
             } catch (Exception ex) {
+                try {
+                    Object object = ((DataHandler)obj).getContent();
+                    convertObjectToSource(object);
+                } catch(Exception ioException) {
+                    throw ConversionException.couldNotBeConverted(obj, Source.class);
+                }
             }
         } else if (obj instanceof byte[]) {
-            ByteArrayInputStream stream = new ByteArrayInputStream((byte[]) obj);
-            return new StreamSource(stream);
+            return new ByteArraySource((byte[])obj);
         } else if (obj instanceof Byte[]) {
             Byte[] objectBytes = (Byte[]) obj;
             byte[] bytes = new byte[objectBytes.length];
             for (int i = 0; i < objectBytes.length; i++) {
                 bytes[i] = objectBytes[i].byteValue();
             }
-            ByteArrayInputStream stream = new ByteArrayInputStream(bytes);
-            return new StreamSource(stream);
+            return new ByteArraySource(bytes);
+        } else if(obj instanceof InputStream) {
+            return new StreamSource((InputStream)obj);
         }
         return null;
     }
