@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -16,20 +16,21 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+
+import org.eclipse.persistence.internal.core.sessions.CoreAbstractSession;
 import org.eclipse.persistence.internal.oxm.XPathFragment;
+import org.eclipse.persistence.internal.oxm.mappings.Mapping;
 import org.eclipse.persistence.internal.oxm.record.MarshalContext;
+import org.eclipse.persistence.internal.oxm.record.MarshalRecord;
 import org.eclipse.persistence.internal.oxm.record.ObjectMarshalContext;
-import org.eclipse.persistence.internal.sessions.AbstractSession;
-import org.eclipse.persistence.oxm.NamespaceResolver;
-import org.eclipse.persistence.oxm.XMLMarshaller;
-import org.eclipse.persistence.oxm.record.MarshalRecord;
+import org.eclipse.persistence.core.queries.CoreAttributeGroup;
 
 /**
  * INTERNAL:
  * <p><b>Purpose</b>:  XPathNodes are used together to build a tree.  The tree
  * is built from all of the XPath statements specified in the mapping metadata 
- * (mappings and policies).  This tree is then navigated by the 
- * TreeObjectBuilder to perform marshal and unmarshal operations.</p>
+ * (mappings and policies).  This tree is then navigated by an 
+ * EventObjectBuilder to perform marshal and unmarshal operations.</p>
  * <p>The XPaths "a/b" and "a/c" would result in a tree with the root "a" and 
  * two child nodes "b" and "c".</p>
  * <p><b>Responsibilities</b>:<ul>
@@ -47,18 +48,23 @@ import org.eclipse.persistence.oxm.record.MarshalRecord;
 public class XPathNode {
     private NodeValue unmarshalNodeValue;
     private NodeValue marshalNodeValue;
+    private boolean isMarshalOnlyNodeValue;
     private XPathFragment xPathFragment;
     private XPathNode parent;
     private List<XPathNode> attributeChildren;
     private List<XPathNode> nonAttributeChildren;
-    private List selfChildren;
+    private List<XPathNode> selfChildren;
     private Map<XPathFragment, XPathNode> attributeChildrenMap;
     private Map<XPathFragment, XPathNode> nonAttributeChildrenMap;
-    private XMLAnyAttributeMappingNodeValue anyAttributeNodeValue;
+    private MappingNodeValue anyAttributeNodeValue;
     private XPathNode anyAttributeNode;
     private XPathNode textNode;
     private XPathNode anyNode;
-
+    private boolean hasTypeChild;
+    private boolean hasPredicateSiblings;
+    private boolean hasPredicateChildren;
+    private NullCapableValue nullCapableValue;
+	
     public XPathFragment getXPathFragment() {
         return xPathFragment;
     }
@@ -76,6 +82,7 @@ public class XPathNode {
         this.unmarshalNodeValue = nodeValue;
         if (null != nodeValue) {
             nodeValue.setXPathNode(this);
+            isMarshalOnlyNodeValue =  nodeValue.isMarshalOnlyNodeValue();
         }
     }
     
@@ -99,6 +106,15 @@ public class XPathNode {
             nodeValue.setXPathNode(this);
         }
         this.marshalNodeValue = nodeValue;
+        isMarshalOnlyNodeValue =  marshalNodeValue.isMarshalOnlyNodeValue();
+    }
+    
+    public NullCapableValue getNullCapableValue() {
+        return nullCapableValue;
+    }
+
+    public void setNullCapableValue(NullCapableValue nullCapableValue) {
+        this.nullCapableValue = nullCapableValue;
     }
     
     public XPathNode getParent() {
@@ -129,11 +145,11 @@ public class XPathNode {
         return this.attributeChildrenMap;
     }
 
-    public void setAnyAttributeNodeValue(XMLAnyAttributeMappingNodeValue nodeValue) {
+    public void setAnyAttributeNodeValue(MappingNodeValue nodeValue) {
         this.anyAttributeNodeValue = nodeValue;
     }
 
-    public XMLAnyAttributeMappingNodeValue getAnyAttributeNodeValue() {
+    public MappingNodeValue getAnyAttributeNodeValue() {
         return this.anyAttributeNodeValue;
     }
 
@@ -157,20 +173,21 @@ public class XPathNode {
         this.textNode = xPathNode;
     }
 
+    public boolean hasTypeChild() {
+        return hasTypeChild;
+    }
+
     public boolean equals(Object object) {
         try {
-            XPathNode perfNode = (XPathNode)object;
-
-            if ((getXPathFragment() == null) && (perfNode.getXPathFragment() != null)) {
-                return false;
-            }
-            if ((getXPathFragment() != null) && (perfNode.getXPathFragment() == null)) {
-                return false;
-            }
-            if (getXPathFragment() == perfNode.getXPathFragment()) {
+            XPathFragment perfNodeXPathFragment = ((XPathNode)object).getXPathFragment();
+            if(xPathFragment == perfNodeXPathFragment) {
                 return true;
+            } else if(null == xPathFragment) {
+                return false;
+            } else if(null == perfNodeXPathFragment) {
+                return false;
             }
-            return this.getXPathFragment().equals(perfNode.getXPathFragment());
+            return xPathFragment.equals(perfNodeXPathFragment);
 
             // turn fix off for now until we re-enable XMLAnyObjectAndAnyCollectionTestCases
             //          } catch (NullPointerException npe) {
@@ -186,7 +203,10 @@ public class XPathNode {
     public XPathNode addChild(XPathFragment anXPathFragment, NodeValue aNodeValue, NamespaceResolver namespaceResolver) {
         if (null != anXPathFragment && anXPathFragment.nameIsText()) {
             if (aNodeValue.isOwningNode(anXPathFragment)) {
-                XPathNode textXPathNode = new XPathNode();
+                XPathNode textXPathNode = this.getTextNode();
+                if(textXPathNode == null) {
+                    textXPathNode = new XPathNode();
+                }
                 textXPathNode.setParent(this);
                 textXPathNode.setXPathFragment(anXPathFragment);
                 if (aNodeValue.isMarshalNodeValue()) {
@@ -196,10 +216,15 @@ public class XPathNode {
                     textXPathNode.setUnmarshalNodeValue(aNodeValue);
                 }
                 this.setTextNode(textXPathNode);
-                if (null == nonAttributeChildren) {
-                    nonAttributeChildren = new ArrayList();
+                if(null != nonAttributeChildren && !nonAttributeChildren.contains(textXPathNode)) {
+                    nonAttributeChildren.add(textXPathNode);
+                }
+                if(aNodeValue instanceof XMLCompositeObjectMappingNodeValue) {
+                    if (null == selfChildren) {
+                        selfChildren = new ArrayList<XPathNode>();
+                    }
+                    selfChildren.add(textXPathNode);
                 }
-                nonAttributeChildren.add(textXPathNode);
                 return textXPathNode;
             }
         }
@@ -230,10 +255,16 @@ public class XPathNode {
         } else {
             if (null == nonAttributeChildren) {
                 nonAttributeChildren = new ArrayList();
+                if(null != textNode) {
+                    nonAttributeChildren.add(textNode);
+                }
             }
             if (null == nonAttributeChildrenMap) {
                 nonAttributeChildrenMap = new HashMap();
             }
+            if(anXPathFragment !=null && Constants.SCHEMA_TYPE_ATTRIBUTE.equals(anXPathFragment.getLocalName())){
+            	hasTypeChild = true;
+            }
             children = nonAttributeChildren;
             childrenMap = nonAttributeChildrenMap;
         }
@@ -246,8 +277,8 @@ public class XPathNode {
                 xPathNode.setUnmarshalNodeValue(aNodeValue);
             }
             xPathNode.setParent(this);
-            if (aNodeValue instanceof XMLAnyAttributeMappingNodeValue) {
-                setAnyAttributeNodeValue((XMLAnyAttributeMappingNodeValue)aNodeValue);
+            if (aNodeValue instanceof XMLAnyAttributeMappingNodeValue || (aNodeValue instanceof XMLVariableXPathObjectMappingNodeValue && ((XMLVariableXPathObjectMappingNodeValue)aNodeValue).getMapping().isAttribute() ) || (aNodeValue instanceof XMLVariableXPathCollectionMappingNodeValue && ((XMLVariableXPathCollectionMappingNodeValue)aNodeValue).getMapping().isAttribute() )) {
+                setAnyAttributeNodeValue((MappingNodeValue)aNodeValue);
                 anyAttributeNode = xPathNode;
             } else {
                 if(!children.contains(xPathNode)) {
@@ -257,12 +288,25 @@ public class XPathNode {
             }
             return xPathNode;
         }
-        boolean isSelfFragment = XPathFragment.SELF_FRAGMENT.equals(anXPathFragment);
+        this.hasPredicateChildren = hasPredicateChildren || anXPathFragment.getPredicate() != null;
+        if(this.getNonAttributeChildren() != null && this.hasPredicateChildren) {
+            for(XPathNode nextChild: this.getNonAttributeChildren()) {
+                XPathFragment nextFrag = nextChild.getXPathFragment();
+                if(nextFrag != null && nextFrag.equals(anXPathFragment, true)) {
+                    if(nextFrag.getPredicate() == null && anXPathFragment.getPredicate() != null) {
+                        nextChild.setHasPredicateSiblings(true);
+                    } else if(anXPathFragment.getPredicate() == null && nextFrag.getPredicate() != null) {
+                        xPathNode.setHasPredicateSiblings(true);
+                    }
+                }
+            }
+        }
 
+        boolean isSelfFragment = XPathFragment.SELF_FRAGMENT.equals(anXPathFragment);
         if(isSelfFragment){
             children.add(xPathNode);
             if (null == selfChildren) {
-                selfChildren = new ArrayList();
+                selfChildren = new ArrayList<XPathNode>();
             }
             selfChildren.add(xPathNode);
         }else{
@@ -277,7 +321,7 @@ public class XPathNode {
                 childrenMap.put(anXPathFragment, xPathNode);
             }
         }
-
+        
         if (aNodeValue.isOwningNode(anXPathFragment)) {
             if(aNodeValue.isMarshalNodeValue()) {
                 xPathNode.setMarshalNodeValue(aNodeValue);
@@ -291,12 +335,30 @@ public class XPathNode {
         }
         return xPathNode;
     }
-    public boolean marshal(MarshalRecord marshalRecord, Object object, AbstractSession session, NamespaceResolver namespaceResolver, XMLMarshaller marshaller, MarshalContext marshalContext) {
-        return marshal(marshalRecord, object, session, namespaceResolver, marshaller, marshalContext, null);
+
+    private void setHasPredicateSiblings(boolean b) {
+        this.hasPredicateSiblings = b;
+    }
+    
+    public boolean hasPredicateSiblings() {
+        return this.hasPredicateSiblings;
     }
 
-    public boolean marshal(MarshalRecord marshalRecord, Object object, AbstractSession session, NamespaceResolver namespaceResolver, XMLMarshaller marshaller, MarshalContext marshalContext, XPathFragment rootFragment) {
-        if ((null == marshalNodeValue) || marshalNodeValue.isMarshalOnlyNodeValue()) {
+    public boolean marshal(MarshalRecord marshalRecord, Object object, CoreAbstractSession session, NamespaceResolver namespaceResolver, Marshaller marshaller, MarshalContext marshalContext, XPathFragment rootFragment) {
+        if ((null == marshalNodeValue) || isMarshalOnlyNodeValue) {
+            if(marshalRecord.isWrapperAsCollectionName() && null != nonAttributeChildren && nonAttributeChildren.size() == 1) {
+                XPathNode childXPathNode = nonAttributeChildren.get(0);
+                NodeValue childXPathNodeUnmarshalNodeValue = childXPathNode.getUnmarshalNodeValue();
+                if(childXPathNodeUnmarshalNodeValue != null && childXPathNodeUnmarshalNodeValue.isContainerValue()) {
+                    ContainerValue containerValue = (ContainerValue) childXPathNodeUnmarshalNodeValue;
+                    if(containerValue.isWrapperAllowedAsCollectionName()) {
+                        XPathNode wrapperXPathNode = new XPathNode();
+                        wrapperXPathNode.setXPathFragment(this.getXPathFragment());
+                        wrapperXPathNode.setMarshalNodeValue(childXPathNode.getMarshalNodeValue());
+                        return wrapperXPathNode.marshal(marshalRecord, object, session, namespaceResolver, marshaller, marshalContext, rootFragment);
+                    }
+                }
+            }
             marshalRecord.addGroupingElement(this);
 
             boolean hasValue = false;
@@ -307,9 +369,13 @@ public class XPathNode {
                 }
             }
             if (anyAttributeNode != null) {
-                hasValue = anyAttributeNode.marshal(marshalRecord, object, session, namespaceResolver, marshaller, ObjectMarshalContext.getInstance()) || hasValue;
+                hasValue = anyAttributeNode.marshal(marshalRecord, object, session, namespaceResolver, marshaller, ObjectMarshalContext.getInstance(), null) || hasValue;
             }
-            if (null != nonAttributeChildren) {
+            if (null == nonAttributeChildren) {
+                if (textNode != null) {
+                    hasValue = textNode.marshal(marshalRecord, object, session, namespaceResolver, marshaller, ObjectMarshalContext.getInstance(), null) || hasValue;
+                }
+            } else {
                 for (int x = 0, size = marshalContext.getNonAttributeChildrenSize(this); x < size; x++) {
                     XPathNode xPathNode = (XPathNode)marshalContext.getNonAttributeChild(x, this);
                     MarshalContext childMarshalContext = marshalContext.getMarshalContext(x);
@@ -325,25 +391,36 @@ public class XPathNode {
 
             return hasValue;
         } else {
+            if(marshalNodeValue.isMappingNodeValue()) {
+                Mapping mapping = ((MappingNodeValue)marshalNodeValue).getMapping();
+                CoreAttributeGroup currentGroup = marshalRecord.getCurrentAttributeGroup();
+                if(!(currentGroup.containsAttributeInternal(mapping.getAttributeName()))) {
+                    return false;
+                }
+            }
             return marshalContext.marshal(marshalNodeValue, xPathFragment, marshalRecord, object, session, namespaceResolver, rootFragment);
         }
     }
 
-    public boolean startElement(MarshalRecord marshalRecord, XPathFragment anXPathFragment, Object object, AbstractSession session, NamespaceResolver namespaceResolver, TreeObjectBuilder compositeObjectBuilder, Object compositeObject) {
+    public boolean startElement(MarshalRecord marshalRecord, XPathFragment anXPathFragment, Object object, CoreAbstractSession session, NamespaceResolver namespaceResolver, ObjectBuilder compositeObjectBuilder, Object compositeObject) {    
         if (null == anXPathFragment) {
             return false;
         }
+
         marshalRecord.openStartElement(anXPathFragment, namespaceResolver);
         boolean hasValue = false;
+
+        marshalRecord.predicateAttribute(anXPathFragment, namespaceResolver);
+
         if (null != attributeChildren) {
             for (int x = 0, size = attributeChildren.size(); x < size; x++) {
                 XPathNode attributeNode = attributeChildren.get(x);
-                hasValue = attributeNode.marshal(marshalRecord, object, session, namespaceResolver, null, ObjectMarshalContext.getInstance()) || hasValue;
+                hasValue = attributeNode.marshal(marshalRecord, object, session, namespaceResolver, null, ObjectMarshalContext.getInstance(), null) || hasValue;
             }
         }
         if (anyAttributeNode != null) {
             //marshal the anyAttribute node here before closeStartElement()
-            hasValue = anyAttributeNode.marshal(marshalRecord, object, session, namespaceResolver, null, ObjectMarshalContext.getInstance()) || hasValue;
+            hasValue = anyAttributeNode.marshal(marshalRecord, object, session, namespaceResolver, null, ObjectMarshalContext.getInstance(), null) || hasValue;
         }
 
         if (null != compositeObjectBuilder) {
@@ -363,7 +440,7 @@ public class XPathNode {
      * @param marshaller
      * @return
      */
-    public boolean marshalSelfAttributes(MarshalRecord marshalRecord, Object object, AbstractSession session, NamespaceResolver namespaceResolver, XMLMarshaller marshaller) {
+    public boolean marshalSelfAttributes(MarshalRecord marshalRecord, Object object, CoreAbstractSession session, NamespaceResolver namespaceResolver, Marshaller marshaller) {
         if (marshalNodeValue == null) {
             return false;
         }
@@ -371,7 +448,7 @@ public class XPathNode {
     }
 
     public boolean isWhitespaceAware() {
-        return this.getNodeValue().isWhitespaceAware();
+        return unmarshalNodeValue.isWhitespaceAware();
     }
 
-}
+}
\ No newline at end of file
