@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -25,15 +25,17 @@ import javax.xml.transform.Source;
 import javax.xml.validation.Schema;
 
 import org.eclipse.persistence.exceptions.XMLMarshalException;
-import org.eclipse.persistence.internal.helper.ClassConstants;
+import org.eclipse.persistence.internal.core.helper.CoreClassConstants;
+import org.eclipse.persistence.internal.core.sessions.CoreAbstractSession;
+import org.eclipse.persistence.internal.oxm.Constants;
+import org.eclipse.persistence.internal.oxm.Root;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
 import org.eclipse.persistence.internal.oxm.XMLObjectBuilder;
 import org.eclipse.persistence.internal.oxm.XPathFragment;
+import org.eclipse.persistence.internal.oxm.mappings.Descriptor;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
-import org.eclipse.persistence.oxm.XMLConstants;
 import org.eclipse.persistence.oxm.XMLContext;
-import org.eclipse.persistence.oxm.XMLDescriptor;
 import org.eclipse.persistence.oxm.XMLRoot;
 import org.eclipse.persistence.oxm.XMLUnmarshaller;
 import org.eclipse.persistence.oxm.record.DOMRecord;
@@ -132,66 +134,99 @@ public class DOMUnmarshaller implements PlatformUnmarshaller {
     }
 
     public Object unmarshal(File file) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(file);
             return xmlToObject(new DOMRecord(document));
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(File file, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(file);
             return xmlToObject(new DOMRecord(document), clazz);
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(InputStream inputStream) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(inputStream);
             return xmlToObject(new DOMRecord(document));
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(InputStream inputStream, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(inputStream);
             return xmlToObject(new DOMRecord(document), clazz);
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(InputSource inputSource) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(inputSource);
             return xmlToObject(new DOMRecord(document));
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(InputSource inputSource, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(inputSource);
             return xmlToObject(new DOMRecord(document), clazz);
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(Node node) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         Element element = null;
         switch (node.getNodeType()) {
         case Node.DOCUMENT_NODE: {
@@ -209,6 +244,9 @@ public class DOMUnmarshaller implements PlatformUnmarshaller {
     }
 
     public Object unmarshal(Node node, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         Element element = null;
         switch (node.getNodeType()) {
         case Node.DOCUMENT_NODE: {
@@ -226,66 +264,99 @@ public class DOMUnmarshaller implements PlatformUnmarshaller {
     }
 
     public Object unmarshal(Reader reader) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(reader);
             return xmlToObject(new DOMRecord(document));
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(Reader reader, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(reader);
             return xmlToObject(new DOMRecord(document), clazz);
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(Source source) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(source);
             return xmlToObject(new DOMRecord(document));
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(Source source, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(source);
             return xmlToObject(new DOMRecord(document), clazz);
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(URL url) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(url);
             return xmlToObject(new DOMRecord(document));
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(URL url, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             Document document = null;
             document = parser.parse(url);
             return xmlToObject(new DOMRecord(document), clazz);
         } catch (XMLPlatformException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(XMLReader xmlReader, InputSource inputSource) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             SAXDocumentBuilder saxDocumentBuilder = new SAXDocumentBuilder();
             xmlReader.setContentHandler(saxDocumentBuilder);
@@ -295,10 +366,15 @@ public class DOMUnmarshaller implements PlatformUnmarshaller {
             throw XMLMarshalException.unmarshalException(e);
         } catch(SAXException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
     public Object unmarshal(XMLReader xmlReader, InputSource inputSource, Class clazz) {
+    	if(!xmlUnmarshaller.getMediaType().isApplicationXML()){
+    		throw XMLMarshalException.unsupportedMediaTypeForPlatform();
+    	}
         try {
             SAXDocumentBuilder saxDocumentBuilder = new SAXDocumentBuilder();
             xmlReader.setContentHandler(saxDocumentBuilder);
@@ -308,36 +384,11 @@ public class DOMUnmarshaller implements PlatformUnmarshaller {
             throw XMLMarshalException.unmarshalException(e);
         } catch(SAXException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } finally {
+        	xmlUnmarshaller.getStringBuffer().reset();
         }
     }
 
-    /**
-     * INTERNAL: Return the descriptor for the document.
-     */
-    protected XMLDescriptor getDescriptor(DOMRecord xmlRecord) throws XMLMarshalException {
-    	XMLDescriptor xmlDescriptor = null;
-    	
-    	XMLContext xmlContext = xmlUnmarshaller.getXMLContext();        
-        // Try to find a descriptor based on the schema type
-        String type = ((Element) xmlRecord.getDOM()).getAttributeNS(XMLConstants.SCHEMA_INSTANCE_URL, "type");
-        if (null != type) {
-            XPathFragment typeFragment = new XPathFragment(type);
-            String namespaceURI = xmlRecord.resolveNamespacePrefix(typeFragment.getPrefix());
-            typeFragment.setNamespaceURI(namespaceURI);
-            xmlDescriptor = xmlContext.getDescriptorByGlobalType(typeFragment);
-        }
-                    
-        if (null == xmlDescriptor) {
-        	QName rootQName = new QName(xmlRecord.getNamespaceURI(), xmlRecord.getLocalName());
-        	xmlDescriptor = xmlContext.getDescriptor(rootQName);
-        	if (null == xmlDescriptor) {
-                throw XMLMarshalException.noDescriptorWithMatchingRootElement(rootQName.toString());
-            }
-        }
-        
-        return xmlDescriptor;
-    }
-
     /**
      * INTERNAL: Find the Descriptor corresponding to the context node of the
      * XMLRecord, and then convert the XMLRecord to an instance of the
@@ -349,94 +400,123 @@ public class DOMUnmarshaller implements PlatformUnmarshaller {
      * @throws XMLMarshalException
      *             if an error occurred during unmarshalling
      */
-    public Object xmlToObject(DOMRecord xmlRecord) throws XMLMarshalException {
-        XMLDescriptor xmlDescriptor = getDescriptor(xmlRecord);
-        return xmlToObject(xmlRecord, xmlDescriptor.getJavaClass());
+    public Object xmlToObject(DOMRecord xmlRecord) throws XMLMarshalException {   
+    	 return xmlToObject(xmlRecord, null);
     }
 
     /**
      * INTERNAL: Convert the Oracle XMLDocument to the reference-class.
      */
     public Object xmlToObject(DOMRecord xmlRow, Class referenceClass) throws XMLMarshalException {
-        //Try to get the Encoding and Version from DOM3 APIs if available
-        String xmlEncoding = "UTF-8";
-        String xmlVersion = "1.0";
-
-        try {
-            Method getEncoding = PrivilegedAccessHelper.getMethod(xmlRow.getDocument().getClass(), "getXmlEncoding", new Class[] {}, true);
-            Method getVersion = PrivilegedAccessHelper.getMethod(xmlRow.getDocument().getClass(), "getXmlVersion", new Class[] {}, true);
-            xmlEncoding = (String) PrivilegedAccessHelper.invokeMethod(getEncoding, xmlRow.getDocument(), new Object[] {});
-            xmlVersion = (String) PrivilegedAccessHelper.invokeMethod(getVersion, xmlRow.getDocument(), new Object[] {});
-        } catch (Exception ex) {
-            //if the methods aren't available, then just use the default values
-        }
-
-        XMLContext xmlContext = xmlUnmarshaller.getXMLContext();
-
-        // handle case where the reference class is a primitive wrapper - in
-        // this case, we need to use the conversion manager to convert the 
-        // node's value to the primitive wrapper class, then create, 
-        // populate and return an XMLRoot
-        if (XMLConversionManager.getDefaultJavaTypes().get(referenceClass) != null ||ClassConstants.XML_GREGORIAN_CALENDAR.isAssignableFrom(referenceClass)
-        	    ||ClassConstants.DURATION.isAssignableFrom(referenceClass)){
-            // we're assuming that since we're unmarshalling to a primitive
-            // wrapper, the root element has a single text node
-            Object nodeVal;
-            try {
-                Text rootTxt = (Text) xmlRow.getDOM().getFirstChild();
-                nodeVal = rootTxt.getNodeValue();
-            } catch (Exception ex) {
-                // here, either the root element doesn't have a text node as a
-                // first child, or there is no first child at all - in any case,
-                // try converting null
-                nodeVal = null;
+    	try{
+	        //Try to get the Encoding and Version from DOM3 APIs if available
+	        String xmlEncoding = "UTF-8";
+	        String xmlVersion = "1.0";
+	
+	        try {
+	            Method getEncoding = PrivilegedAccessHelper.getMethod(xmlRow.getDocument().getClass(), "getXmlEncoding", new Class[] {}, true);
+	            Method getVersion = PrivilegedAccessHelper.getMethod(xmlRow.getDocument().getClass(), "getXmlVersion", new Class[] {}, true);
+	            xmlEncoding = (String) PrivilegedAccessHelper.invokeMethod(getEncoding, xmlRow.getDocument(), new Object[] {});
+	            xmlVersion = (String) PrivilegedAccessHelper.invokeMethod(getVersion, xmlRow.getDocument(), new Object[] {});
+	        } catch (Exception ex) {
+	            //if the methods aren't available, then just use the default values
+	        }
+	
+	        XMLContext xmlContext = xmlUnmarshaller.getXMLContext();
+	
+	        // handle case where the reference class is a primitive wrapper - in
+	        // this case, we need to use the conversion manager to convert the 
+	        // node's value to the primitive wrapper class, then create, 
+	        // populate and return an XMLRoot
+	        if (referenceClass != null && (XMLConversionManager.getDefaultJavaTypes().get(referenceClass) != null ||CoreClassConstants.XML_GREGORIAN_CALENDAR.isAssignableFrom(referenceClass)
+	        	    ||CoreClassConstants.DURATION.isAssignableFrom(referenceClass))){
+	            // we're assuming that since we're unmarshalling to a primitive
+	            // wrapper, the root element has a single text node
+	            Object nodeVal;
+	            try {
+	                Text rootTxt = (Text) xmlRow.getDOM().getFirstChild();
+	                nodeVal = rootTxt.getNodeValue();
+	            } catch (Exception ex) {
+	                // here, either the root element doesn't have a text node as a
+	                // first child, or there is no first child at all - in any case,
+	                // try converting null
+	                nodeVal = null;
+	            }
+	  
+	            Object obj = ((XMLConversionManager) xmlContext.getSession().getDatasourcePlatform().getConversionManager()).convertObject(nodeVal, referenceClass);
+	            Root xmlRoot = new XMLRoot();
+	            xmlRoot.setObject(obj);
+	            String lName = xmlRow.getDOM().getLocalName();
+	            if (lName == null) {
+	                lName = xmlRow.getDOM().getNodeName();
+	            }
+	            xmlRoot.setLocalName(lName);
+	            xmlRoot.setNamespaceURI(xmlRow.getDOM().getNamespaceURI());
+	            xmlRoot.setEncoding(xmlEncoding);
+	            xmlRoot.setVersion(xmlVersion);
+	            return xmlRoot;
+	        }
+            Descriptor descriptor = null;
+            CoreAbstractSession readSession = null;
+            boolean shouldWrap = true;            
+            if(referenceClass == null){
+                QName rootQName = new QName(xmlRow.getNamespaceURI(), xmlRow.getLocalName());
+                descriptor = xmlContext.getDescriptor(rootQName);   
+                if (null == descriptor) {
+                    String type = ((Element) xmlRow.getDOM()).getAttributeNS(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI, "type");
+                    if (null != type) {
+     	                XPathFragment typeFragment = new XPathFragment(type);
+     	                String namespaceURI = xmlRow.resolveNamespacePrefix(typeFragment.getPrefix());
+     	                typeFragment.setNamespaceURI(namespaceURI);
+     	                descriptor = xmlContext.getDescriptorByGlobalType(typeFragment);    	               
+     	            }
+                 } else if(descriptor.getTables().size() == 1){
+                     shouldWrap = false;
+                 }                 	    		
+                 if (null == descriptor) {
+                     throw XMLMarshalException.noDescriptorWithMatchingRootElement(rootQName.toString());
+                 }else{
+                     readSession = xmlContext.getSession(descriptor.getJavaClass());
+                 }
+            } else {
+                // for XMLObjectReferenceMappings we need a non-shared cache, so
+                // try and get a Unit Of Work from the XMLContext
+                readSession = xmlContext.getSession(referenceClass);
+                descriptor = (Descriptor)readSession.getDescriptor(referenceClass);
+                if (descriptor == null) {
+                    throw XMLMarshalException.descriptorNotFoundInProject(referenceClass.getName());
+                } 
             }
-  
-            Object obj = ((XMLConversionManager) xmlContext.getSession(0).getDatasourcePlatform().getConversionManager()).convertObject(nodeVal, referenceClass);
-            XMLRoot xmlRoot = new XMLRoot();
-            xmlRoot.setObject(obj);
-            String lName = xmlRow.getDOM().getLocalName();
-            if (lName == null) {
-                lName = xmlRow.getDOM().getNodeName();
-            }
-            xmlRoot.setLocalName(lName);
-            xmlRoot.setNamespaceURI(xmlRow.getDOM().getNamespaceURI());
-            xmlRoot.setEncoding(xmlEncoding);
-            xmlRoot.setVersion(xmlVersion);
-            return xmlRoot;
-        }
-
-        // for XMLObjectReferenceMappings we need a non-shared cache, so
-        // try and get a Unit Of Work from the XMLContext
-        AbstractSession readSession = xmlContext.getReadSession(referenceClass);
-
-        XMLDescriptor descriptor = (XMLDescriptor) readSession.getDescriptor(referenceClass);
-        if (descriptor == null) {
-            throw XMLMarshalException.descriptorNotFoundInProject(referenceClass.getName());
-        }
-
-        Object object = null;
-        if(null == xmlRow.getDOM().getAttributes().getNamedItemNS(XMLConstants.SCHEMA_INSTANCE_URL, XMLConstants.SCHEMA_NIL_ATTRIBUTE)) {
-            xmlRow.setUnmarshaller(xmlUnmarshaller);
-            xmlRow.setDocPresPolicy(xmlContext.getDocumentPreservationPolicy(readSession));
-            XMLObjectBuilder objectBuilder = (XMLObjectBuilder) descriptor.getObjectBuilder();
-
-            ReadObjectQuery query = new ReadObjectQuery();
-            query.setReferenceClass(referenceClass);
-            query.setSession(readSession);
-            object = objectBuilder.buildObject(query, xmlRow, null);
-
-            // resolve mapping references
-            xmlUnmarshaller.resolveReferences(readSession);
-        }
-
-        String elementNamespaceUri = xmlRow.getDOM().getNamespaceURI();
-        String elementLocalName = xmlRow.getDOM().getLocalName();
-        if (elementLocalName == null) {
-            elementLocalName = xmlRow.getDOM().getNodeName();
-        }
-        String elementPrefix = xmlRow.getDOM().getPrefix();
-        return descriptor.wrapObjectInXMLRoot(object, elementNamespaceUri, elementLocalName, elementPrefix, xmlEncoding, xmlVersion, this.isResultAlwaysXMLRoot);
+	    		
+	        Object object = null;
+	        if(null == xmlRow.getDOM().getAttributes().getNamedItemNS(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI, Constants.SCHEMA_NIL_ATTRIBUTE)) {
+	            xmlRow.setUnmarshaller(xmlUnmarshaller);
+	            xmlRow.setDocPresPolicy(xmlContext.getDocumentPreservationPolicy((AbstractSession) readSession));
+	            XMLObjectBuilder objectBuilder = (XMLObjectBuilder) descriptor.getObjectBuilder();
+	
+	            ReadObjectQuery query = new ReadObjectQuery();
+	            query.setReferenceClass(referenceClass);
+	            query.setSession((AbstractSession) readSession);
+	            object = objectBuilder.buildObject(query, xmlRow, null);
+	
+	            // resolve mapping references
+	            xmlRow.resolveReferences(readSession, xmlUnmarshaller.getIDResolver());
+	        }
+	
+	        String elementNamespaceUri = xmlRow.getDOM().getNamespaceURI();
+	        String elementLocalName = xmlRow.getDOM().getLocalName();
+	        if (elementLocalName == null) {
+	            elementLocalName = xmlRow.getDOM().getNodeName();
+	        }
+	        String elementPrefix = xmlRow.getDOM().getPrefix();
+	        if(shouldWrap || descriptor.isResultAlwaysXMLRoot() || isResultAlwaysXMLRoot){
+	            return descriptor.wrapObjectInXMLRoot(object, elementNamespaceUri, elementLocalName, elementPrefix, xmlEncoding, xmlVersion, this.isResultAlwaysXMLRoot, true, xmlUnmarshaller);
+	        }else{
+	        	return object;
+	        }
+    	}finally{    		
+            xmlUnmarshaller.getStringBuffer().reset();           
+    	}
     }
 
     public boolean isResultAlwaysXMLRoot() {
@@ -446,4 +526,9 @@ public class DOMUnmarshaller implements PlatformUnmarshaller {
     public void setResultAlwaysXMLRoot(boolean alwaysReturnRoot) {
         this.isResultAlwaysXMLRoot = alwaysReturnRoot;
     }
+
+    @Override
+    public void mediaTypeChanged() {
+       //do nothing
+    }
 }
