@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,13 +9,16 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     07/13/2012-2.5 Guy Pelletier 
+ *       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls
+ *     08/24/2012-2.5 Guy Pelletier 
+ *       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls
  ******************************************************************************/  
 package org.eclipse.persistence.internal.queries;
 
 import java.util.*;
 import java.io.*;
 
-import org.eclipse.persistence.internal.descriptors.ObjectBuilder;
 import org.eclipse.persistence.internal.descriptors.OptimisticLockingPolicy;
 import org.eclipse.persistence.descriptors.VersionLockingPolicy;
 import org.eclipse.persistence.descriptors.DescriptorEvent;
@@ -29,11 +32,9 @@ import org.eclipse.persistence.tools.profiler.QueryMonitor;
 import org.eclipse.persistence.expressions.*;
 import org.eclipse.persistence.queries.*;
 import org.eclipse.persistence.exceptions.*;
-import org.eclipse.persistence.mappings.*;
 import org.eclipse.persistence.mappings.DatabaseMapping.WriteType;
-import org.eclipse.persistence.mappings.foundation.AbstractTransformationMapping;
-import org.eclipse.persistence.logging.SessionLog;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
+import org.eclipse.persistence.internal.sessions.RepeatableWriteUnitOfWork;
 import org.eclipse.persistence.internal.sessions.UnitOfWorkImpl;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
@@ -131,10 +132,10 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
             return null;
         }
         
-        if (descriptor.shouldAcquireCascadedLocks()) {
+        if (query.isObjectBuildingQuery() && ((ObjectBuildingQuery)query).requiresDeferredLocks() || descriptor.shouldAcquireCascadedLocks()) {
             return session.getIdentityMapAccessorInstance().getFromIdentityMapWithDeferredLock(primaryKey, getReadObjectQuery().getReferenceClass(), false, descriptor);
         } else {
-            return session.getIdentityMapAccessorInstance().getFromIdentityMap(primaryKey, getReadObjectQuery().getReferenceClass(), false, descriptor);
+            return session.getIdentityMapAccessorInstance().getFromLocalIdentityMap(primaryKey, getReadObjectQuery().getReferenceClass(), false, descriptor);
         }        
     }
 
@@ -183,6 +184,16 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
      */
     public abstract Integer deleteObject() throws DatabaseException;
 
+    /**
+     * Execute a execute SQL call. 
+     * This should be overridden by subclasses.
+     * @exception DatabaseException
+     * @return true if the first result is a result set and false if it is an 
+     *   update count or there are no results other than through INOUT and OUT
+     *   parameterts, if any.
+     */
+    public abstract Object execute() throws DatabaseException;
+
     /**
      * Execute a non selecting SQL call
      * This should be overridden by subclasses.
@@ -210,7 +221,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         CommitManager commitManager = getSession().getCommitManager();
 
         // if the object has already been committed, no work is required
-        if (commitManager.isCommitCompletedOrInPost(object)) {
+        if (commitManager.isCommitCompletedInPostOrIgnore(object)) {
             return object;
         }
 
@@ -262,7 +273,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
     }
     
     /**
-     * Check whether the object already exists on the database; then
+     * Check whether the object already exists on the cadatabase; then
      * perform an insert or update, as appropriate.
      * This method was moved here, from WriteObjectQuery.execute(),
      * so we can hide the source.
@@ -283,7 +294,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         }
         // If the object has already been committed, no work is required
         // need to check for the object to ensure insert wasn't completed already.
-        if (commitManager.isCommitCompletedOrInPost(object)) {
+        if (commitManager.isCommitCompletedInPostOrIgnore(object)) {
             return object;
         }
         try {
@@ -353,6 +364,13 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         return this.query.getSession();
     }
 
+    /**
+     * Convenience method
+     */
+    protected AbstractSession getExecutionSession() {
+        return this.query.getExecutionSession();
+    }
+
     /**
      * Convenience method
      */
@@ -389,6 +407,8 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         WriteObjectQuery writeQuery = getWriteObjectQuery();
         ClassDescriptor descriptor = getDescriptor();
         DescriptorQueryManager queryManager = descriptor.getQueryManager();
+        boolean isFKUpdate = false; // Bug 319276
+        
         // check for user-defined query
         if ((!writeQuery.isUserDefined())// this is not a user-defined query
                  && queryManager.hasInsertQuery()// there is a user-defined query
@@ -422,6 +442,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         // In a unit of work/writeObjects the preInsert may have caused a shallow insert of this object,
         // in this case this second write must do an update.
         if (commitManager.isShallowCommitted(object)) {
+            isFKUpdate = true; // Bug 319276
             updateForeignKeyFieldAfterInsert();
         } else {
             AbstractRecord modifyRow = writeQuery.getModifyRow();
@@ -445,6 +466,9 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
                 writeQuery.setPrimaryKey(descriptor.getObjectBuilder().extractPrimaryKeyFromObject(object, session));
             }
             addWriteLockFieldForInsert();
+            if (descriptor.hasSerializedObjectPolicy()) {
+                descriptor.getSerializedObjectPolicy().putObjectIntoRow(modifyRow, object, session);
+            }
 
             // CR#3237
             // Store the size of the modify row so we can determine if the user has added to the row in the insert.
@@ -473,8 +497,14 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         if (writeQuery.shouldCascadeParts()) {
             queryManager.postInsert(writeQuery);
         }
+        
         if ((descriptor.getHistoryPolicy() != null) && descriptor.getHistoryPolicy().shouldHandleWrites()) {
-            descriptor.getHistoryPolicy().postInsert(writeQuery);
+            if (isFKUpdate) { // Bug 319276
+                descriptor.getHistoryPolicy().postUpdate(writeQuery, true);
+            }
+            else {
+                descriptor.getHistoryPolicy().postInsert(writeQuery);
+            }
         }
 
         // PERF: Avoid events if no listeners.
@@ -595,6 +625,12 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
      */
     public abstract void prepareExecuteNoSelect() throws QueryException;
 
+    /**
+     * Prepare for a raw execute call.
+     * This is sent to the original query before cloning.
+     */
+    public abstract void prepareExecute() throws QueryException;
+    
     /**
      * Prepare for a raw (non-object) select call.
      * This is sent to the original query before cloning.
@@ -752,6 +788,15 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
      */
     protected abstract void updateForeignKeyFieldAfterInsert(WriteObjectQuery writeQuery);
 
+    /**
+     * Update the foreign key fields to null when resolving a deletion cycle.
+     * This must always be dynamic as it is called within an delete query and is really part of the delete
+     * and does not fire update events or worry about locking.
+     */
+    public void updateForeignKeyFieldBeforeDelete() {
+        // Nothing by default.
+    }
+    
     protected void updateObjectAndRowWithReturnRow(Collection returnFields, boolean isFirstCallForInsert) {
         WriteObjectQuery writeQuery = getWriteObjectQuery();
         AbstractRecord outputRow = (AbstractRecord)writeQuery.getProperties().get("output");
@@ -771,7 +816,14 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
 
         Object object = writeQuery.getObject();
 
-        getDescriptor().getObjectBuilder().assignReturnRow(object, getSession(), row);
+        ObjectChangeSet objectChangeSet = null;
+        if (getSession().isUnitOfWork()) {
+            objectChangeSet = writeQuery.getObjectChangeSet();
+            if ((objectChangeSet == null) && (((UnitOfWorkImpl)getSession()).getUnitOfWorkChangeSet() != null)) {
+                objectChangeSet = (ObjectChangeSet)((UnitOfWorkImpl)getSession()).getUnitOfWorkChangeSet().getObjectChangeSetForClone(object);
+            }
+        }
+        getDescriptor().getObjectBuilder().assignReturnRow(object, writeQuery.getSession(), row, objectChangeSet);
 
         Object primaryKey = null;
         if (isFirstCallForInsert) {
@@ -792,119 +844,10 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
             }
         }
 
-        // update the changeSet if there is one
-        if (getSession().isUnitOfWork()) {
-            ObjectChangeSet objectChangeSet = writeQuery.getObjectChangeSet();
-            if ((objectChangeSet == null) && (((UnitOfWorkImpl)getSession()).getUnitOfWorkChangeSet() != null)) {
-                objectChangeSet = (ObjectChangeSet)((UnitOfWorkImpl)getSession()).getUnitOfWorkChangeSet().getObjectChangeSetForClone(object);
+        if (objectChangeSet != null) {
+            if (primaryKey != null) {
+                objectChangeSet.setId(primaryKey);
             }
-            if (objectChangeSet != null) {
-                updateChangeSet(getDescriptor(), objectChangeSet, row, object);
-                if (primaryKey != null) {
-                    objectChangeSet.setId(primaryKey);
-                }
-            }
-        }
-    }
-
-    /**
-     * Update the change set with all of the field values in the row.
-     * This handles writable and read-only mappings, direct and nested aggregates.
-     * It is used from ReturningPolicy and VersionLockingPolicy.
-     */
-    public void updateChangeSet(ClassDescriptor desc, ObjectChangeSet objectChangeSet, AbstractRecord row, Object object) {
-        int size = row.size();
-        HashSet handledMappings = null;
-        if (size > 1) {
-            // PERF: Only create set if more than one fields, used to avoid duplicate in transformation mappings.
-            handledMappings = new HashSet(size);
-        }
-        for (int i = 0; i < size; i++) {
-            DatabaseField field = (DatabaseField)row.getFields().get(i);
-            updateChangeSet(desc, objectChangeSet, field, object, handledMappings);
-        }
-    }
-    
-    /**
-     * Update the change set with the field value.
-     * This handles writable and read-only mappings, direct and nested aggregates.
-     * It is used from ReturningPolicy and VersionLockingPolicy.
-     */
-    protected void updateChangeSet(ClassDescriptor desc, ObjectChangeSet objectChangeSet, DatabaseField field, Object object) {
-        updateChangeSet(desc, objectChangeSet, field, object, null);
-    }
-    
-    /**
-     * Update the change set with the field value.
-     * This handles writable and read-only mappings, direct and nested aggregates.
-     * It is used from ReturningPolicy and VersionLockingPolicy.
-     */
-    protected void updateChangeSet(ClassDescriptor desc, ObjectChangeSet objectChangeSet, DatabaseField field, Object object, Collection handledMappings) {
-        DatabaseMapping mapping;
-        List readOnlyMappings = desc.getObjectBuilder().getReadOnlyMappingsForField(field);
-        if (readOnlyMappings != null) {
-            int size = readOnlyMappings.size();
-            for (int index = 0; index < size; index++) {
-                mapping = (DatabaseMapping)readOnlyMappings.get(index);
-                updateChangeSet(mapping, objectChangeSet, field, object, handledMappings);
-            }
-        }
-        mapping = desc.getObjectBuilder().getMappingForField(field);
-        if (mapping != null) {
-            updateChangeSet(mapping, objectChangeSet, field, object, handledMappings);
-        }
-    }
-
-    protected void updateChangeSet(DatabaseMapping mapping, ObjectChangeSet objectChangeSet, DatabaseField field, Object object, Collection handledMappings) {
-        if ((handledMappings != null) && handledMappings.contains(mapping)) {
-            return;
-        }
-        if (mapping.isDirectToFieldMapping()) {
-            Object attributeValue = mapping.getAttributeValueFromObject(object);
-            objectChangeSet.updateChangeRecordForAttribute(mapping, attributeValue, getSession());
-        } else if (mapping.isAggregateObjectMapping()) {
-            Object aggregate = mapping.getAttributeValueFromObject(object);
-            AggregateChangeRecord record = (AggregateChangeRecord)objectChangeSet.getChangesForAttributeNamed(mapping.getAttributeName());
-            if (aggregate != null) {
-                if (record == null) {
-                    record = new AggregateChangeRecord(objectChangeSet);
-                    record.setAttribute(mapping.getAttributeName());
-                    record.setMapping(mapping);
-                    objectChangeSet.addChange(record);
-                }
-                ObjectChangeSet aggregateChangeSet = (ObjectChangeSet)record.getChangedObject();
-                ClassDescriptor aggregateDescriptor = ((AggregateObjectMapping)mapping).getReferenceDescriptor();
-                if (aggregateChangeSet == null) {
-                    aggregateChangeSet = aggregateDescriptor.getObjectBuilder().createObjectChangeSet(aggregate, (UnitOfWorkChangeSet)((UnitOfWorkImpl)getSession()).getUnitOfWorkChangeSet(), getSession());
-                    record.setChangedObject(aggregateChangeSet);
-                }
-                updateChangeSet(aggregateDescriptor, aggregateChangeSet, field, aggregate, handledMappings);
-            } else {
-                if (record != null) {
-                    record.setChangedObject(null);
-                }
-            }
-        } else if (mapping.isTransformationMapping()) {
-            TransformationMappingChangeRecord record = (TransformationMappingChangeRecord)objectChangeSet.getChangesForAttributeNamed(mapping.getAttributeName());
-            if (record == null) {
-                record = new TransformationMappingChangeRecord(objectChangeSet);
-                record.setAttribute(mapping.getAttributeName());
-                record.setMapping(mapping);
-                objectChangeSet.addChange(record);
-            }
-            AbstractRecord transformationRow = new DatabaseRecord(mapping.getFields().size());
-            int size = mapping.getFields().size();
-            for (int i = 0; i < size; i++) {
-                DatabaseField fieldToAdd = mapping.getFields().get(i);
-                Object value = ((AbstractTransformationMapping)mapping).valueFromObject(object, fieldToAdd, getSession());
-                transformationRow.add(fieldToAdd, value);
-            }
-            record.setRow(transformationRow);
-            if (handledMappings != null) {
-                handledMappings.add(mapping);
-            }
-        } else {
-            getSession().log(SessionLog.FINEST, SessionLog.QUERY, "field_for_unsupported_mapping_returned", field, getDescriptor());
         }
     }
 
@@ -913,34 +856,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
      */
     protected void updateObjectAndRowWithSequenceNumber() throws DatabaseException {
         WriteObjectQuery writeQuery = getWriteObjectQuery();
-        Object object = writeQuery.getObject();
-        ClassDescriptor descriptor = writeQuery.getDescriptor();
-        ObjectBuilder objectBuilder = descriptor.getObjectBuilder();
-        AbstractSession session = writeQuery.getSession();
-        Object sequenceValue = objectBuilder.assignSequenceNumber(object, session);
-        if (sequenceValue == null) {
-            return;
-        }
-        Object primaryKey = objectBuilder.extractPrimaryKeyFromObject(object, getSession());
-        writeQuery.setPrimaryKey(primaryKey);
-        DatabaseField sequenceNumberField = descriptor.getSequenceNumberField();
-        AbstractRecord modifyRow = getModifyRow();
-        // Update the row.
-        modifyRow.put(sequenceNumberField, sequenceValue);
-        if (descriptor.hasMultipleTables()) {
-            objectBuilder.addPrimaryKeyForNonDefaultTable(modifyRow, object, session);
-        }
-        // Update the changeSet if there is one.
-        if (session.isUnitOfWork()) {
-            ObjectChangeSet objectChangeSet = writeQuery.getObjectChangeSet();
-            if ((objectChangeSet == null) && (((UnitOfWorkImpl)session).getUnitOfWorkChangeSet() != null)) {
-                objectChangeSet = (ObjectChangeSet)((UnitOfWorkImpl)session).getUnitOfWorkChangeSet().getObjectChangeSetForClone(object);
-            }
-            if (objectChangeSet != null) {
-                updateChangeSet(descriptor, objectChangeSet, sequenceNumberField, object);
-                objectChangeSet.setId(primaryKey);
-            }
-        }
+        writeQuery.getDescriptor().getObjectBuilder().assignSequenceNumber(writeQuery);
     }
 
     /**
@@ -1011,6 +927,9 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
                     ((VersionLockingPolicy)policy).writeLockValueIntoRow(writeQuery, object);
                 }
             }
+            if (descriptor.hasSerializedObjectPolicy()) {
+                descriptor.getSerializedObjectPolicy().putObjectIntoRow(getModifyRow(), object, session);
+            }
 
             // PERF: Avoid events if no listeners.
             if (eventManager.hasAnyEventListeners()) {
@@ -1110,8 +1029,8 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         // For a user defined update in the uow to row must be built twice to check if any update is required.
         writeQuery.setModifyRow(descriptor.getObjectBuilder().buildRowForUpdateWithChangeSet(writeQuery));
             
-    	Boolean shouldModifyVersionField = changeSet.shouldModifyVersionField();
-        if (!getModifyRow().isEmpty() || (shouldModifyVersionField != null) || changeSet.hasCmpPolicyForcedUpdate()) {
+        Boolean shouldModifyVersionField = changeSet.shouldModifyVersionField();
+        if (!getModifyRow().isEmpty() || shouldModifyVersionField != null || changeSet.hasCmpPolicyForcedUpdate()) {
             // If user defined the entire row is required. Must not be built until change is known.
             if (writeQuery.isUserDefined() || writeQuery.isCallQuery()) {
                 writeQuery.setModifyRow(descriptor.getObjectBuilder().buildRow(object, session, WriteType.UNDEFINED));
@@ -1121,14 +1040,30 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
             // Update the write lock field if required.
             if (lockingPolicy != null) {
                 lockingPolicy.addLockValuesToTranslationRow(writeQuery);
-
-                if (!getModifyRow().isEmpty() || shouldModifyVersionField.booleanValue()) {
-                    // Update the row with newer lock value.
-                    lockingPolicy.updateRowAndObjectForUpdate(writeQuery, object);
-                } else if (!shouldModifyVersionField.booleanValue() && (lockingPolicy instanceof VersionLockingPolicy)) {
-                    // Add the existing write lock value to the for a "read" lock (requires something to update).
-                    ((VersionLockingPolicy)lockingPolicy).writeLockValueIntoRow(writeQuery, object);
+                // Do not lock an object that has previously been optimistically locked within the RWUoW
+                boolean existingOptimisticLock = false;
+                if (session instanceof RepeatableWriteUnitOfWork) {
+                    RepeatableWriteUnitOfWork uow = (RepeatableWriteUnitOfWork)session;
+                    if (uow.getOptimisticReadLockObjects().get(object) != null && uow.getCumulativeUOWChangeSet() != null 
+                            && uow.getCumulativeUOWChangeSet().getObjectChangeSetForClone(object) != null) {
+                        existingOptimisticLock = true;
+                    }
                 }
+                if (!existingOptimisticLock) {
+                    // update the row and object if shouldModifyVersionField is non null and has a value of true (a forced update),
+                    // or if there is no forced update and modifyRow has modifications
+                    if ((shouldModifyVersionField != null && shouldModifyVersionField) || !getModifyRow().isEmpty()) {
+                        // Update the row with newer lock value.
+                        lockingPolicy.updateRowAndObjectForUpdate(writeQuery, object);
+                    } else if (!shouldModifyVersionField && (lockingPolicy instanceof VersionLockingPolicy)) {
+                        // Add the existing write lock value to the for a "read" lock (requires something to update).
+                        ((VersionLockingPolicy)lockingPolicy).writeLockValueIntoRow(writeQuery, object);
+                    }
+                }
+            }
+            
+            if (descriptor.hasSerializedObjectPolicy()) {
+                descriptor.getSerializedObjectPolicy().putObjectIntoRow(getModifyRow(), object, session);
             }
 
             // PERF: Avoid events if no listeners.
@@ -1168,4 +1103,11 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
             eventManager.executeEvent(new DescriptorEvent(DescriptorEventManager.PostUpdateEvent, writeQuery));
         }
     }
+    
+    /**
+     * Unprepare the call if required.
+     */
+    public void unprepare() {
+        
+    }
 }
