@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -41,6 +41,8 @@ public class CommitManager {
     protected static final Integer POST = Integer.valueOf(2);
     /** The commit is complete for the object. */
     protected static final Integer COMPLETE = Integer.valueOf(3);
+    /** This object should be ignored. */
+    protected static final Integer IGNORE = Integer.valueOf(4);
     
     /** Set of objects that had partial row written to resolve constraints. */
     protected Map shallowCommits;
@@ -169,6 +171,8 @@ public class CommitManager {
                     this.session.deleteObject(objects.get(index));
                 }
             }
+            
+            this.session.commitTransaction();
         } catch (RuntimeException exception) {
             this.session.rollbackTransaction();
             throw exception;
@@ -176,8 +180,6 @@ public class CommitManager {
             reinitialize();
             this.isActive = false;
         }
-
-        this.session.commitTransaction();
     }
 
     /**
@@ -288,6 +290,8 @@ public class CommitManager {
                     deleteAllObjects(theClass, objects, session);
                 }
             }
+
+            session.commitTransaction();
         } catch (RuntimeException exception) {
             try {
                 session.rollbackTransaction();
@@ -298,8 +302,6 @@ public class CommitManager {
             reinitialize();
             this.isActive = false;
         }
-
-        session.commitTransaction();
     }
 
     /**
@@ -307,6 +309,11 @@ public class CommitManager {
      */
     public void deleteAllObjects(Class theClass, List objects, AbstractSession session) {
         ClassDescriptor descriptor = null;
+        
+        if (((UnitOfWorkImpl)session).shouldOrderUpdates()) {// bug 331064 - Sort the delete order
+            objects = sort(theClass, objects);
+        }
+        
         int size = objects.size();
         for (int index = 0; index < size; index++) {
             Object objectToDelete = objects.get(index);
@@ -331,6 +338,24 @@ public class CommitManager {
         }
     }
     
+    /**
+     * Sort the objects based on PK. 
+     */
+    // bug 331064 - Sort the delete order based on PKs.
+    private List sort (Class theClass, List objects) {
+        ClassDescriptor descriptor = session.getDescriptor(theClass);
+        org.eclipse.persistence.internal.descriptors.ObjectBuilder objectBuilder = descriptor.getObjectBuilder();
+        int size = objects.size();
+        TreeMap sortedObjects = new TreeMap();
+        for (int index = 0; index < size; index++) {
+            Object objectToDelete = objects.get(index);
+            if (objectToDelete.getClass() == theClass) {
+                sortedObjects.put(objectBuilder.extractPrimaryKeyFromObject(objectToDelete, session), objectToDelete);
+            }
+        }
+        return new ArrayList(sortedObjects.values());
+    }
+    
     /**
      * Return the order in which objects should be committed to the database.
      * This order is based on ownership in the descriptors and is require for referential integrity.
@@ -485,9 +510,9 @@ public class CommitManager {
      * This should be called by any query that is writing an object,
      * if true the query should not write the object.
      */
-    public boolean isCommitCompletedOrInPost(Object object) {
+    public boolean isCommitCompletedInPostOrIgnore(Object object) {
         Integer state = getCommitState().get(object);
-        return (state == COMPLETE) || (state == POST);
+        return (state == COMPLETE) || (state == POST) || (state == IGNORE);
     }
 
     /**
@@ -531,6 +556,10 @@ public class CommitManager {
             return;
         }
     }
+    
+    public void markIgnoreCommit(Object object){
+        getCommitState().put(object, IGNORE);
+    }
 
     /**
      * Add an object as being in progress of being committed.
