@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,16 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     14/05/2012-2.4 Guy Pelletier  
+ *       - 376603: Provide for table per tenant support for multitenant applications
+ *     22/05/2012-2.4 Guy Pelletier  
+ *       - 380008: Multitenant persistence units with a dedicated emf should force tenant property specification up front.
+ *     31/05/2012-2.4 Guy Pelletier  
+ *       - 381196: Multitenant persistence units with a dedicated emf should allow for DDL generation.
+ *     12/24/2012-2.5 Guy Pelletier 
+ *       - 389090: JPA 2.1 DDL Generation Support
+ *     01/11/2013-2.5 Guy Pelletier 
+ *       - 389090: JPA 2.1 DDL Generation Support
  ******************************************************************************/  
 package org.eclipse.persistence.internal.sessions;
 
@@ -16,23 +26,33 @@ import java.sql.Connection;
 import java.sql.SQLException;
 import java.util.*;
 
+import org.eclipse.persistence.config.PersistenceUnitProperties;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
+import org.eclipse.persistence.descriptors.partitioning.PartitioningPolicy;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.internal.helper.DBPlatformHelper;
 import org.eclipse.persistence.internal.databaseaccess.Platform;
 import org.eclipse.persistence.internal.sequencing.Sequencing;
 import org.eclipse.persistence.internal.sequencing.SequencingHome;
 import org.eclipse.persistence.internal.sequencing.SequencingFactory;
+import org.eclipse.persistence.sequencing.Sequence;
 import org.eclipse.persistence.sequencing.SequencingControl;
 import org.eclipse.persistence.logging.SessionLog;
 import org.eclipse.persistence.sessions.Login;
 import org.eclipse.persistence.sessions.DatasourceLogin;
 import org.eclipse.persistence.sessions.SessionProfiler;
+import org.eclipse.persistence.tools.tuning.SessionTuner;
 import org.eclipse.persistence.platform.database.DatabasePlatform;
 import org.eclipse.persistence.platform.database.OraclePlatform;
+import org.eclipse.persistence.platform.database.events.DatabaseEventListener;
 import org.eclipse.persistence.platform.server.ServerPlatform;
 import org.eclipse.persistence.platform.server.NoServerPlatform;
 import org.eclipse.persistence.platform.server.ServerPlatformBase;
+import org.eclipse.persistence.queries.AttributeGroup;
+import org.eclipse.persistence.queries.DatabaseQuery;
+import org.eclipse.persistence.queries.ObjectLevelReadQuery;
+import org.eclipse.persistence.queries.QueryResultsCachePolicy;
+import org.eclipse.persistence.queries.ReadQuery;
 
 /**
  * Implementation of org.eclipse.persistence.sessions.DatabaseSession
@@ -59,22 +79,32 @@ import org.eclipse.persistence.platform.server.ServerPlatformBase;
  */
 public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.persistence.sessions.DatabaseSession {
 
+    /**
+     * Database event listener, this allows database events to invalidate the cache.
+     */
+    protected DatabaseEventListener databaseEventListener;
+
     /**
      * INTERNAL:
      * sequencingHome for this session.
      */
-    private SequencingHome sequencingHome;
+    protected SequencingHome sequencingHome;
 
     /**
      * Used to store the server platform that handles server-specific functionality for Oc4j, WLS,  etc.
      */
-    private ServerPlatform serverPlatform;
+    protected ServerPlatform serverPlatform;
+    
+    /**
+     * Stores the tuner used to tune the configuration of this session.
+     */
+    protected SessionTuner tuner;
 
     /**
      * INTERNAL:
      * connectedTime indicates the exact time this session was logged in.
      */
-    private long connectedTime;
+    protected long connectedTime;
 
     /**
      * INTERNAL
@@ -83,7 +113,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
 
     //Bug#3440544 Used to stop the attempt to login more than once. 
     protected volatile boolean isLoggedIn;
-
+    
     /**
      * INTERNAL:
      * Set the SequencingHome object used by the session.
@@ -113,6 +143,110 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         return isLoggedIn;
     }
     
+    /**
+     * Return the database event listener, this allows database events to invalidate the cache.
+     */
+    public DatabaseEventListener getDatabaseEventListener() {
+        return databaseEventListener;
+    }
+    
+    /**
+     * PUBLIC:
+     * Set the database event listener, this allows database events to invalidate the cache.
+     */
+    public void setDatabaseEventListener(DatabaseEventListener databaseEventListener) {
+        this.databaseEventListener = databaseEventListener;
+    }
+    
+    /**
+     * INTERNAL:
+     * Issue any pre connect and post connect without an actual connection to 
+     * the database. Descriptors are initialized in postConnectDatasource and
+     * are used in DDL generation. This will look to set the schema platform
+     * via the JPA 2.1 properties or through a detection on the connection
+     * before DDL generation.
+     */
+    public void setDatasourceAndInitialize() throws DatabaseException {
+        preConnectDatasource();
+        setOrDetectDatasource(false);
+        postConnectDatasource();
+    }
+    
+    /**
+     * INTERNAL:
+     * Will set the platform from specified schema generation properties or 
+     * by detecting it through the connection (if one is available).
+     * Any connection that is open for detection is closed before this method
+     * returns.
+     * 
+     * @param throwException - set to true if the caller cares to throw exceptions, false to swallow them.
+     */
+    protected void setOrDetectDatasource(boolean throwException) {
+        // Try to set the platform from JPA 2.1 schema properties first before attempting a detection.
+        if (getProperties().containsKey(PersistenceUnitProperties.SCHEMA_DATABASE_PRODUCT_NAME)) {
+            String vendorNameAndVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_PRODUCT_NAME);
+            
+            String majorVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_MAJOR_VERSION);
+            if (majorVersion != null) {
+                vendorNameAndVersion += majorVersion;
+            }
+            
+            String minorVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_MINOR_VERSION);
+            if (minorVersion != null) {
+                vendorNameAndVersion += minorVersion;
+            }
+
+            getLogin().setPlatformClassName(DBPlatformHelper.getDBPlatform(vendorNameAndVersion, getSessionLog()));
+        } else {
+            Connection conn = null;
+            
+            try {
+                conn = (Connection) getReadLogin().connectToDatasource(null, this);
+                // null out the cached platform because the platform on the login will be changed by the following line of code
+                this.platform = null;
+                String platformName = null;
+                
+                try {
+                    String vendorNameAndVersion = conn.getMetaData().getDatabaseProductName() + conn.getMetaData().getDatabaseMajorVersion();
+                    platformName = DBPlatformHelper.getDBPlatform(vendorNameAndVersion, getSessionLog());
+                    getLogin().setPlatformClassName(platformName);
+                } catch (EclipseLinkException classNotFound) {
+                    if (platformName.indexOf("Oracle") != -1) {
+                        // If we are running against Oracle, it is possible that we are running in an environment where
+                        // the OracleXPlatform class can not be loaded. Try using OraclePlatform class before giving up
+                        getLogin().setPlatformClassName(OraclePlatform.class.getName());
+                    } else {
+                        throw classNotFound;
+                    }
+                }
+                
+                getLogin().getPlatform().setDriverName(conn.getMetaData().getDriverName());
+            } catch (SQLException ex) {
+                if (throwException) {
+                    DatabaseException dbEx =  DatabaseException.errorRetrieveDbMetadataThroughJDBCConnection();
+                    // Typically exception would occur if user did not provide correct connection
+                    // parameters. The root cause of exception should be propagated up
+                    dbEx.initCause(ex);
+                    throw dbEx;
+                }
+            } finally {
+                if (conn != null) {
+                    try {
+                        conn.close();
+                    } catch (SQLException ex) {
+                        if (throwException) {
+                            DatabaseException dbEx =  DatabaseException.errorRetrieveDbMetadataThroughJDBCConnection();
+                            // Typically exception would occur if user did not provide correct connection
+                            // parameters. The root cause of exception should be propagated up
+                            dbEx.initCause(ex);
+                            throw dbEx;
+                        }
+                    }
+                }
+            }
+        }
+    }
+    
     /**
      * PUBLIC:
      * Return  SequencingControl which used for sequencing setup and
@@ -148,6 +282,15 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         getSequencingHome().onConnect();
     }
 
+    /**
+     * INTERNAL:
+     * If sequencing is connected then initializes sequences referenced by the passed descriptors,
+     * otherwise connects sequencing.
+     */
+    public void addDescriptorsToSequencing(Collection descriptors) {
+        getSequencingHome().onAddDescriptors(descriptors);
+    }
+
     /**
      * INTERNAL:
      * Called in the end of beforeCompletion of external transaction synchronization listener.
@@ -165,6 +308,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
     public DatabaseSessionImpl() {
         super();
         this.setServerPlatform(new NoServerPlatform(this));
+        this.shouldOptimizeResultSetAccess = ObjectLevelReadQuery.isResultSetAccessOptimizedQueryDefault;
     }
 
     /**
@@ -191,6 +335,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
     public DatabaseSessionImpl(org.eclipse.persistence.sessions.Project project) {
         super(project);
         this.setServerPlatform(new NoServerPlatform(this));
+        this.shouldOptimizeResultSetAccess = ObjectLevelReadQuery.isResultSetAccessOptimizedQueryDefault;
     }
 
     /**
@@ -238,6 +383,21 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         getProject().addDescriptors(project, this);
     }
 
+    /**
+     * PUBLIC:
+     * Add the sequence to the session.
+     * Allows to add a new sequence to the session even if the session is connected.
+     * If the session is connected then the sequence is added only 
+     * if there is no sequence with the same name already in use.
+     * Call this method before addDescriptor(s) if need to add new descriptor 
+     * with a new non-default sequence to connected session.
+     *
+     * @see #addSequences(Collection)
+     */
+    public void addSequence(Sequence sequence) {
+        getProject().getLogin().getDatasourcePlatform().addSequence(sequence, this.getSequencingHome().isConnected());        
+    }
+    
     /**
      * INTERNAL:
      * Connect the session only.
@@ -373,10 +533,12 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * Normally the descriptors are added before login, then initialized on login.
      */
     public void initializeDescriptorIfSessionAlive(ClassDescriptor descriptor) {
-        if (isConnected() && (descriptor.requiresInitialization())) {
+        if (isConnected() && (descriptor.requiresInitialization(this))) {
             try {
                 try {
-                    initializeSequencing();
+                    Collection descriptorsToAdd = new ArrayList(1);
+                    descriptorsToAdd.add(descriptor);
+                    addDescriptorsToSequencing(descriptorsToAdd);
                     descriptor.preInitialize(this);
                     descriptor.initialize(this);
                     descriptor.postInitialize(this);
@@ -403,10 +565,37 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      */
     public void initializeDescriptors() {
         // Must clone to avoid modification of the map while enumerating.
-        initializeDescriptors((Map)((HashMap)getDescriptors()).clone());
+        initializeDescriptors((Map)((HashMap)getDescriptors()).clone(), true);
+        // Initialize serializer
+        if (this.serializer != null) {
+        	this.serializer.initialize(null, null, this);
+        }
+        // Initialize partitioning policies.
+        for (PartitioningPolicy policy : getProject().getPartitioningPolicies().values()) {
+            policy.initialize(this);
+        }
         // Process JPA named queries and add as session queries,
         // this must be done after descriptor init as requires to parse the JPQL.
         processJPAQueries();
+        
+        // Configure default query cache for all named queries.
+        QueryResultsCachePolicy defaultQueryCachePolicy = getProject().getDefaultQueryResultsCachePolicy();
+        if (defaultQueryCachePolicy != null) {
+            for (List<DatabaseQuery> queries : getQueries().values()) {
+                for (DatabaseQuery query : queries) {
+                    if (query.isReadQuery() && (query.getDescriptor() != null) && !query.getDescriptor().getCachePolicy().isIsolated()) {
+                        ReadQuery readQuery = (ReadQuery)query;
+                        if (!readQuery.shouldCacheQueryResults()) {
+                            readQuery.setQueryResultsCachePolicy(defaultQueryCachePolicy.clone());
+                        }
+                    }
+                }
+            }
+        }
+        for (AttributeGroup group : getProject().getAttributeGroups().values()){
+            getAttributeGroups().put(group.getName(), group);
+            this.getDescriptor(group.getType()).addAttributeGroup(group);
+        }
     }
 
     /**
@@ -415,22 +604,58 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * This is done in two passes to allow the inheritance to be resolved first.
      * Normally the descriptors are added before login, then initialized on login.
      * The descriptors session must be used, not the broker.
+     * Sequencing is (re)initialized: disconnected (if has been already connected), then connected.
      */
     public void initializeDescriptors(Map descriptors) {
-        initializeSequencing();
+        initializeDescriptors(descriptors.values(), false);
+    }
+    public void initializeDescriptors(Collection descriptors) {
+        initializeDescriptors(descriptors, false);
+    }
+
+    /**
+     * INTERNAL:
+     * Allow each descriptor to initialize any dependencies on this session.
+     * This is done in two passes to allow the inheritance to be resolved first.
+     * Normally the descriptors are added before login, then initialized on login.
+     * The descriptors session must be used, not the broker.
+     * If shouldInitializeSequencing parameter is true then sequencing is (re)initialized:
+     * disconnected (if has been connected), then connected.
+     * If shouldInitializeSequencing parameter is false then
+     *   if sequencing has been already connected, then it stays connected:
+     *     only the new sequences used by the passed descriptors are initialized;
+     *   otherwise, if sequencing has NOT been connected then it is connected 
+     *     (just like in shouldInitializeSequencing==true case);
+     *   disconnected (if has been connected), then connected.
+     */
+    public void initializeDescriptors(Map descriptors, boolean shouldInitializeSequencing) {
+        initializeDescriptors(descriptors.values(), shouldInitializeSequencing);
+    }
+    public void initializeDescriptors(Collection descriptors, boolean shouldInitializeSequencing) {
+        if (shouldInitializeSequencing) {
+            initializeSequencing();
+        } else {
+            addDescriptorsToSequencing(descriptors);
+        }
+        
         try {
             // First initialize basic properties (things that do not depend on anything else)
-            Iterator iterator = descriptors.values().iterator();
+            Iterator iterator = descriptors.iterator();
             while (iterator.hasNext()) {
                 ClassDescriptor descriptor = (ClassDescriptor)iterator.next();
                 try {
                     AbstractSession session = getSessionForClass(descriptor.getJavaClass());
-                    if (descriptor.requiresInitialization()) {
+                    if (descriptor.requiresInitialization(session)) {
                         descriptor.preInitialize(session);
+                    } else if (descriptor.hasTablePerMultitenantPolicy()) {
+                        // If the descriptor doesn't require initialization and
+                        // has a table per tenant policy then add to the list
+                        // to be cloned and initialized per client session.
+                        addTablePerTenantDescriptor(descriptor);
                     }
-
+                    
                     //check if inheritance is involved in aggregate relationship, and let the parent know the child descriptor
-                    if ((descriptor.isAggregateDescriptor() || descriptor.isAggregateCollectionDescriptor()) && descriptor.isChildDescriptor()) {
+                    if (descriptor.isDescriptorTypeAggregate() && descriptor.isChildDescriptor()) {
                         descriptor.initializeAggregateInheritancePolicy(session);
                     }
                 } catch (RuntimeException exception) {
@@ -439,12 +664,12 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             }
 
             // Second initialize basic mappings
-            iterator = descriptors.values().iterator();
+            iterator = descriptors.iterator();
             while (iterator.hasNext()) {
                 ClassDescriptor descriptor = (ClassDescriptor)iterator.next();
                 try {
                     AbstractSession session = getSessionForClass(descriptor.getJavaClass());
-                    if (descriptor.requiresInitialization()) {
+                    if (descriptor.requiresInitialization(session)) {
                         descriptor.initialize(session);
                     }
                 } catch (RuntimeException exception) {
@@ -453,12 +678,12 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             }
 
             // Third initialize child dependencies
-            iterator = descriptors.values().iterator();
+            iterator = descriptors.iterator();
             while (iterator.hasNext()) {
                 ClassDescriptor descriptor = (ClassDescriptor)iterator.next();
                 try {
                     AbstractSession session = getSessionForClass(descriptor.getJavaClass());
-                    if (descriptor.requiresInitialization()) {
+                    if (descriptor.requiresInitialization(session)) {
                         descriptor.postInitialize(session);
                     }
                 } catch (RuntimeException exception) {
@@ -477,75 +702,6 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         getCommitManager().initializeCommitOrder();
     }
 
-    /**
-     * INTERNAL:
-     * Allow each descriptor to initialize any dependencies on this session.
-     * This is done in two passes to allow the inheritance to be resolved first.
-     * Normally the descriptors are added before login, then initialized on login.
-     * The descriptors session must be used, not the broker.
-     */
-    public void initializeDescriptors(Collection descriptors) {
-        initializeSequencing();
-        try {
-            // First initialize basic properties (things that do not depend on anything else)
-            for (Iterator descriptorEnum = descriptors.iterator(); descriptorEnum.hasNext();) {
-                try {
-                    ClassDescriptor descriptor = (ClassDescriptor)descriptorEnum.next();
-                    AbstractSession session = getSessionForClass(descriptor.getJavaClass());
-                    if (descriptor.requiresInitialization()) {
-                        descriptor.preInitialize(session);
-                    }
-
-                    //check if inheritance is involved in aggregate relationship, and let the parent know the child descriptor
-                    if (descriptor.isAggregateDescriptor() && descriptor.isChildDescriptor()) {
-                        descriptor.initializeAggregateInheritancePolicy(session);
-                    }
-                } catch (RuntimeException exception) {
-                    getIntegrityChecker().handleError(exception);
-                }
-            }
-
-            // Second basic initialize mappings
-            for (Iterator descriptorEnum = descriptors.iterator(); descriptorEnum.hasNext();) {
-                try {
-                    ClassDescriptor descriptor = (ClassDescriptor)descriptorEnum.next();
-                    AbstractSession session = getSessionForClass(descriptor.getJavaClass());
-                    if (descriptor.requiresInitialization()) {
-                        descriptor.initialize(session);
-                    }
-                } catch (RuntimeException exception) {
-                    getIntegrityChecker().handleError(exception);
-                }
-            }
-
-            // Third initialize child dependencies
-            for (Iterator descriptorEnum = descriptors.iterator(); descriptorEnum.hasNext();) {
-                try {
-                    ClassDescriptor descriptor = (ClassDescriptor)descriptorEnum.next();
-                    AbstractSession session = getSessionForClass(descriptor.getJavaClass());
-                    if (descriptor.requiresInitialization()) {
-                        descriptor.postInitialize(session);
-                    }
-                } catch (RuntimeException exception) {
-                    getIntegrityChecker().handleError(exception);
-                }
-            }
-
-            try {
-                getCommitManager().initializeCommitOrder();
-            } catch (RuntimeException exception) {
-                getIntegrityChecker().handleError(exception);
-            }
-
-            if (getIntegrityChecker().hasErrors()) {
-                //CR#4011
-                handleException(new IntegrityException(getIntegrityChecker()));
-            }
-        } finally {
-            clearIntegrityChecker();
-        }
-    }
-
     /**
      * INTERNAL:
      * Return if this session is a database session.
@@ -554,6 +710,14 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         return true;
     }
 
+    /**
+     * PUBLIC:
+     * Returns true if Protected Entities should be built within this session
+     */
+    public boolean isProtectedSession(){
+        return false;
+    }
+
     /**
      * INTERNAL:
      * Return the login for the read connection.  Used by the platform autodetect feature
@@ -561,7 +725,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
     protected Login getReadLogin(){
         return getDatasourceLogin();
     }
-
+    
     /**
      * PUBLIC:
      * Connect to the database using the predefined login.
@@ -574,43 +738,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      */
     public void loginAndDetectDatasource() throws DatabaseException {
         preConnectDatasource();
-        Connection conn = null;
-        try{
-            conn = (Connection)getReadLogin().connectToDatasource(null,this);
-            // null out the cached platform because the platform on the login will be changed by the following line of code
-            this.platform = null;
-            String platformName = null;
-            try {
-                platformName = DBPlatformHelper.getDBPlatform(conn.getMetaData().getDatabaseProductName(), getSessionLog());
-                getLogin().setPlatformClassName(platformName);
-            } catch (EclipseLinkException classNotFound) {
-                if (platformName.indexOf("Oracle") != -1) {
-                    // If we are running against Oracle, it is possible that we are running in an environment where
-                    // the OracleXPlatform class can not be loaded. Try using OraclePlatform class before giving up
-                    getLogin().setPlatformClassName(OraclePlatform.class.getName());
-                } else {
-                    throw classNotFound;
-                }
-            }
-        }catch (SQLException ex){
-            DatabaseException dbEx =  DatabaseException.errorRetrieveDbMetadataThroughJDBCConnection();
-            // Typically exception would occur if user did not provide correct connection
-            // parameters. The root cause of exception should be propagated up
-            dbEx.initCause(ex);
-            throw dbEx;
-        }finally{
-            if (conn != null){
-                try{
-                    conn.close();
-                }catch (SQLException ex){
-                    DatabaseException dbEx =  DatabaseException.errorRetrieveDbMetadataThroughJDBCConnection();
-                    // Typically exception would occur if user did not provide correct connection
-                    // parameters. The root cause of exception should be propagated up
-                    dbEx.initCause(ex);
-                    throw dbEx;
-                }
-            }
-        }
+        setOrDetectDatasource(true);
         connect();
         postConnectDatasource();
     }
@@ -627,6 +755,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         connect();
         postConnectDatasource();
     }
+    
     /**
      * INTERNAL:
      * This method includes all of the code that is issued before the datasource
@@ -642,17 +771,20 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             getProfiler().initialize();
         }
         updateProfile(SessionProfiler.LoginTime, new Date(System.currentTimeMillis()));
+        updateProfile(SessionProfiler.SessionName, getName());
 
         // Login and initialize
         if (this.eventManager != null) {
             this.eventManager.preLogin(this);
         }
-        //setup the external transaction controller
-        getServerPlatform().initializeExternalTransactionController();
-        log(SessionLog.INFO, null, "topLink_version", DatasourceLogin.getVersion());
-        if (getServerPlatform().getServerNameAndVersion() != null && 
-                !getServerPlatform().getServerNameAndVersion().equals(ServerPlatformBase.DEFAULT_SERVER_NAME_AND_VERSION)) {
-            log(SessionLog.INFO, null, "application_server_name_and_version", getServerPlatform().getServerNameAndVersion());
+        if (!hasBroker()) {
+            //setup the external transaction controller
+            getServerPlatform().initializeExternalTransactionController();
+            log(SessionLog.INFO, null, "topLink_version", DatasourceLogin.getVersion());
+            if (getServerPlatform().getServerNameAndVersion() != null && 
+                    !getServerPlatform().getServerNameAndVersion().equals(ServerPlatformBase.DEFAULT_SERVER_NAME_AND_VERSION)) {
+                log(SessionLog.INFO, null, "application_server_name_and_version", getServerPlatform().getServerNameAndVersion());
+            }
         }
         this.isLoggingOff = (getLogLevel() == SessionLog.OFF);
     }
@@ -663,28 +795,73 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * is connected to.
      */
     protected void postConnectDatasource(){
-        initializeDescriptors();
-        //added to process ejbQL query strings
-        if (getCommandManager() != null) {
-            getCommandManager().initialize();
+        if (!hasBroker()) {
+            initializeDescriptors();
+            
+            //added to process ejbQL query strings
+            if (getCommandManager() != null) {
+                getCommandManager().initialize();
+            }
         }
-        log(SessionLog.INFO, null, "login_successful", this.getName());
-        if (this.eventManager != null) {
-            this.eventManager.postLogin(this);
+        
+        // Once the descriptors are initialized we can check if there are
+        // multitenant entities and if this session (emf) is shared or not. If
+        // not shared, all multitenant properties must be available and set by 
+        // the user at this point for us to validate (meaning they must be set 
+        // in a persitence.xml or passed into the create EMF call).
+        if (getProperties().containsKey(PersistenceUnitProperties.MULTITENANT_SHARED_EMF)) {
+            String value = (String) getProperties().get(PersistenceUnitProperties.MULTITENANT_SHARED_EMF);
+            if (! new Boolean(value)) {
+                for (String property : getMultitenantContextProperties()) {
+                    if (! getProperties().containsKey(property)) {
+                        throw ValidationException.multitenantContextPropertyForNonSharedEMFNotSpecified(property);
+                    }
+                }
+                
+                // Once the properties are validated we can allow ddl generation to happen (if needed).
+                project.setAllowTablePerMultitenantDDLGeneration(true);
+            }
+        }
+        
+        log(SessionLog.INFO, SessionLog.CONNECTION, "login_successful", this.getName());
+        // postLogin event should not be risen before descriptors have been initialized 
+        if (!hasBroker()) {
+            postLogin();
         }
 
         initializeConnectedTime();
         this.isLoggedIn = true;
         this.platform = null;
         
-        //register the MBean
-        getServerPlatform().registerMBean();
+        if (!hasBroker()) {
+            //register the MBean
+            getServerPlatform().registerMBean();
+        }
         this.descriptors = getDescriptors();
-        // EclipseLink 23869 - Initialize plaformOperators eagerly to avoid concurrency issues.
-        getDatasourcePlatform().initialize();
-        getIdentityMapAccessorInstance().getIdentityMapManager().checkIsCacheAccessPreCheckRequired();
+        if (!isBroker()) {
+            // EclipseLink 23869 - Initialize plaformOperators eagerly to avoid concurrency issues.
+            getDatasourcePlatform().initialize();
+            getIdentityMapAccessorInstance().getIdentityMapManager().checkIsCacheAccessPreCheckRequired();
+        }
+        if (this.databaseEventListener != null) {
+            this.databaseEventListener.register(this);
+        }
+        if ((getDatasourcePlatform() instanceof DatabasePlatform) && getPlatform().getBatchWritingMechanism() != null) {
+            getPlatform().getBatchWritingMechanism().initialize(this);
+        }
+        
     }
 
+    /**
+     * INTERNAL:
+     * Rise postLogin event.
+     */
+    public void postLogin() {
+        if (this.eventManager != null) {
+            this.eventManager.postLogin(this);
+        }
+    }
+    
     /**
      * PUBLIC:
      * Connect to the database using the given user name and password.
@@ -718,6 +895,12 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * or a general error occurs.
      */
     public void logout() throws DatabaseException {
+        if (this.eventManager != null) {
+            this.eventManager.preLogout(this);
+        }
+        
+        cleanUpEntityListenerInjectionManager();
+        
         // Reset cached data, as may be invalid later on.
         this.lastDescriptorAccessed = null;
 
@@ -729,20 +912,29 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             return;
         }
         
-        // We're logging out so turn off change propagation.
-        this.setShouldPropagateChanges(false);
-        
-        if (getCommandManager() != null) {
-            getCommandManager().shutdown();
+        if (this.databaseEventListener != null) {
+            this.databaseEventListener.remove(this);
         }
+        
+        // We're logging out so turn off change propagation.
+        setShouldPropagateChanges(false);
+        
+        if (!hasBroker()) {
+            if (getCommandManager() != null) {
+                getCommandManager().shutdown();
+            }
 
-        // Unregister the JMX MBean before logout to avoid a javax.naming.NameNotFoundException
-        getServerPlatform().unregisterMBean();
+            // Unregister the JMX MBean before logout to avoid a javax.naming.NameNotFoundException
+            getServerPlatform().shutdown();
+        }
         
         disconnect();
         getIdentityMapAccessor().initializeIdentityMaps();
-        isLoggedIn = false;
-        log(SessionLog.INFO, null, "logout_successful", this.getName());
+        this.isLoggedIn = false;
+        if (this.eventManager != null) {
+            this.eventManager.postLogout(this);
+        }
+        log(SessionLog.INFO, SessionLog.CONNECTION, "logout_successful", this.getName());
 	   
     }
 
@@ -795,4 +987,53 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             writeObject(objectsEnum.nextElement());
         }
     }
+    
+    /**
+     * INTERNAL:
+     * A query execution failed due to an invalid query.
+     * Re-connect and retry the query.
+     */
+    @Override
+    public Object retryQuery(DatabaseQuery query, AbstractRecord row, DatabaseException databaseException, int retryCount, AbstractSession executionSession) {
+        if (getClass() != DatabaseSessionImpl.class) {
+            return super.retryQuery(query, row, databaseException, retryCount, executionSession);
+        }
+        //attempt to reconnect connection:
+        int count = getLogin().getQueryRetryAttemptCount();
+        while (retryCount < count) {
+            try {
+                // if database session then re-establish connection
+                // else the session will just get a new
+                // connection from the pool
+                databaseException.getAccessor().reestablishConnection(this);
+                break;
+            } catch (DatabaseException ex) {
+                // failed to get connection because of
+                // database error.
+                ++retryCount;
+                try {
+                    // Give the failover time to recover.
+                    Thread.currentThread().sleep(getLogin().getDelayBetweenConnectionAttempts());
+                    log(SessionLog.INFO, "communication_failure_attempting_query_retry", (Object[])null, null);
+                } catch (InterruptedException intEx) {
+                    break;
+                }
+            }
+        }
+        return executionSession.executeQuery(query, row, retryCount);
+    }
+    
+    /**
+     * Return the tuner used to tune the configuration of this session.
+     */
+    public SessionTuner getTuner() {
+        return tuner;
+    }
+    
+    /**
+     * Set the tuner used to tune the configuration of this session.
+     */
+    public void setTuner(SessionTuner tuner) {
+        this.tuner = tuner;
+    }
 }
