@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -15,9 +15,9 @@ package org.eclipse.persistence.internal.sessions.factories.model.platform;
 /**
  * INTERNAL:
  */
-public class WebLogic_8_1_PlatformConfig extends ServerPlatformConfig {
-    public WebLogic_8_1_PlatformConfig() {
-        super("org.eclipse.persistence.platform.server.wls.WebLogic_8_1_Platform");
+public class WebLogic_7_0_PlatformConfig extends ServerPlatformConfig {
+    public WebLogic_7_0_PlatformConfig() {
+        super("org.eclipse.persistence.platform.server.wls.WebLogicPlatform");
         isSupported = false;
     }
 }
