@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
  * which accompanies this distribution.
@@ -36,6 +36,7 @@ public class Result {
     protected Attachment attachment;
     protected SimpleXMLFormat simpleXMLFormat;
     protected Boolean isCollection = null;
+    protected Integer jdbcType = null;
 
     public Result() {
     }
@@ -70,4 +71,25 @@ public class Result {
     public boolean isCollection () {
         return isCollection == null ? false : isCollection.booleanValue();
     }
+
+    /**
+     * Indicates if the JDBC type should be set on the call.
+     */
+    public boolean isJdbcTypeSet() {
+        return jdbcType != null;
+    }
+
+    /**
+     * Indicates the JDBC type code to be set on the call.
+     */
+    public int getJdbcType() {
+        return jdbcType;
+    }
+
+    /**
+     * Set the JDBC type code to be set on the call.
+     */
+    public void setJdbcType(int jdbcType) {
+        this.jdbcType = jdbcType;
+    }
 }
