@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -16,8 +16,9 @@ import javax.xml.bind.JAXBElement;
 import javax.xml.namespace.QName;
 
 import org.eclipse.persistence.exceptions.XMLMarshalException;
+import org.eclipse.persistence.internal.oxm.XPathFragment;
+import org.eclipse.persistence.internal.oxm.mappings.Descriptor;
 import org.eclipse.persistence.oxm.XMLContext;
-import org.eclipse.persistence.oxm.XMLDescriptor;
 import org.eclipse.persistence.sessions.Session;
 
 /**
@@ -53,11 +54,11 @@ public class JAXBIntrospector extends javax.xml.bind.JAXBIntrospector {
             if(session == null) {
                 return false;
             }
-            XMLDescriptor descriptor = (XMLDescriptor)session.getDescriptor(obj);
+            Descriptor descriptor = (Descriptor)session.getDescriptor(obj);
             if(descriptor == null) {
                 return false;
             }
-            return descriptor.getDefaultRootElement().length() != 0;
+            return descriptor.getDefaultRootElementField() != null;
         } catch(XMLMarshalException e) {
             return false;
         }
@@ -72,17 +73,9 @@ public class JAXBIntrospector extends javax.xml.bind.JAXBIntrospector {
             return ((JAXBElement) obj).getName();
         }
         try {
-            XMLDescriptor descriptor = (XMLDescriptor)context.getSession(obj).getDescriptor(obj);
-            String rootElem = descriptor.getDefaultRootElement();
-            int prefixIndex = rootElem.indexOf(":");
-            if(prefixIndex == -1) {
-                return new QName(rootElem);
-            } else {
-                String prefix = rootElem.substring(0, prefixIndex);
-                String localPart = rootElem.substring(prefixIndex + 1);
-                String URI = descriptor.getNamespaceResolver().resolveNamespacePrefix(prefix);
-                return new QName(URI, localPart);
-            }
+            Descriptor descriptor = (Descriptor)context.getSession(obj).getDescriptor(obj);
+            XPathFragment rootFragment = descriptor.getDefaultRootElementField().getXPathFragment(); 
+            return new QName(rootFragment.getNamespaceURI(), rootFragment.getLocalName()); 
         } catch(XMLMarshalException e) {
             return null;
         }
