@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,6 +12,7 @@
  ******************************************************************************/
 package org.eclipse.persistence.jaxb.compiler;
 
+import java.lang.reflect.Field;
 import java.lang.reflect.GenericArrayType;
 import java.lang.reflect.Method;
 import java.lang.reflect.ParameterizedType;
@@ -31,12 +32,18 @@ import javax.xml.bind.annotation.XmlElement;
 import javax.xml.bind.annotation.XmlList;
 import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
 
+import org.eclipse.persistence.internal.jaxb.AccessorFactoryWrapper;
+import org.eclipse.persistence.internal.jaxb.JaxbClassLoader;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
 import org.eclipse.persistence.jaxb.JAXBContextFactory;
 import org.eclipse.persistence.jaxb.JAXBContext;
 import org.eclipse.persistence.jaxb.TypeMappingInfo;
 import org.eclipse.persistence.jaxb.javamodel.Helper;
 import org.eclipse.persistence.jaxb.javamodel.JavaClass;
+import org.eclipse.persistence.jaxb.javamodel.JavaField;
+import org.eclipse.persistence.jaxb.javamodel.JavaMethod;
+import org.eclipse.persistence.jaxb.javamodel.reflection.JavaClassImpl;
+import org.eclipse.persistence.jaxb.xmlmodel.XmlJoinNodes;
 
 /**
  * Helper class for code that needs to be shared between AnnotationsProcessor,
@@ -47,6 +54,43 @@ public class CompilerHelper {
     private static JAXBContext xmlBindingsModelContext;
     private static final String METADATA_MODEL_PACKAGE = "org.eclipse.persistence.jaxb.xmlmodel";
 
+    private static final String XML_ACCESSOR_FACTORY_ANNOTATION_NAME = "com.sun.xml.bind.XmlAccessorFactory";
+    private static final String INTERNAL_ACCESSOR_FACTORY_ANNOTATION_NAME = "com.sun.xml.internal.bind.XmlAccessorFactory";
+    public static final String XML_LOCATION_ANNOTATION_NAME = "com.sun.xml.bind.annotation.XmlLocation";
+    public static final String INTERNAL_XML_LOCATION_ANNOTATION_NAME = "com.sun.xml.internal.bind.annotation.XmlLocation";
+    public static Class ACCESSOR_FACTORY_ANNOTATION_CLASS = null;
+    public static Method ACCESSOR_FACTORY_VALUE_METHOD = null;
+    public static Class INTERNAL_ACCESSOR_FACTORY_ANNOTATION_CLASS = null;
+    public static Method INTERNAL_ACCESSOR_FACTORY_VALUE_METHOD = null;
+    public static Class XML_LOCATION_ANNOTATION_CLASS = null;
+    public static Class INTERNAL_XML_LOCATION_ANNOTATION_CLASS = null;
+    
+    static {
+        try {
+            ACCESSOR_FACTORY_ANNOTATION_CLASS = PrivilegedAccessHelper.getClassForName(XML_ACCESSOR_FACTORY_ANNOTATION_NAME, true, CompilerHelper.class.getClassLoader());
+            ACCESSOR_FACTORY_VALUE_METHOD = PrivilegedAccessHelper.getDeclaredMethod(ACCESSOR_FACTORY_ANNOTATION_CLASS, "value", new Class[]{});            
+        } catch (Exception ex) {        
+        }
+        
+        try {
+            XML_LOCATION_ANNOTATION_CLASS = PrivilegedAccessHelper.getClassForName(XML_LOCATION_ANNOTATION_NAME, true, CompilerHelper.class.getClassLoader());
+        } catch (Exception ex) {
+        }
+        
+        try{
+            INTERNAL_ACCESSOR_FACTORY_ANNOTATION_CLASS = PrivilegedAccessHelper.getClassForName(INTERNAL_ACCESSOR_FACTORY_ANNOTATION_NAME);
+            INTERNAL_ACCESSOR_FACTORY_VALUE_METHOD = PrivilegedAccessHelper.getDeclaredMethod(INTERNAL_ACCESSOR_FACTORY_ANNOTATION_CLASS, "value", new Class[]{});
+        } catch (Exception ex) {         
+        }
+        
+        try{
+            INTERNAL_XML_LOCATION_ANNOTATION_CLASS = PrivilegedAccessHelper.getClassForName(INTERNAL_XML_LOCATION_ANNOTATION_NAME);
+        }catch (Exception ex) {         
+        }
+        
+        
+    }
+   
     /**
      * If 2 TypeMappingInfo objects would generate the same generated class (and
      * therefore complex type) then return the existing class otherwise return
@@ -256,6 +300,33 @@ public class CompilerHelper {
         return null;
 
     }
+    
+    /**
+     * If adapter class is null return null If there is a marshal method that
+     * returns something other than Object on the adapter class return the
+     * return type of that method Otherwise return Object.class
+     */
+    public static JavaClass getTypeFromAdapterClass(JavaClass adapterClass, Helper helper) {
+        if (adapterClass != null) {
+            //JavaClass declJavaType = Object.class;
+            JavaClass declJavaType = helper.getJavaClass(Object.class);
+            // look for marshal method
+            Object[] tacMethods = (Object[]) adapterClass.getMethods().toArray();
+            for (int i = 0; i < tacMethods.length; i++) {
+                JavaMethod method = (JavaMethod)tacMethods[i];
+                if (method.getName().equals("marshal")) {
+                    JavaClass returnType = method.getReturnType();
+                    if (!(returnType.getQualifiedName().equals(declJavaType.getQualifiedName()))) {
+                        declJavaType = returnType;
+                        return declJavaType;
+                    }
+                }
+            }
+            return declJavaType;
+        }
+        return null;
+
+    }       
 
     /**
      * Return true if the type is a Collection, List or Set
@@ -298,7 +369,7 @@ public class CompilerHelper {
                                 Class typeClass = ((XmlElement) nextAnnotation).type();
                                 if (!typeClass.getName().equals("javax.xml.bind.annotation.XmlElement.DEFAULT")) {
                                     final Type tmiType = tmi.getType();
-                                    if (tmiType instanceof java.util.List) {
+                                    if (isCollectionType(tmiType)) {
                                         final Class itemType = typeClass;
                                         Type parameterizedType = new ParameterizedType() {
                                             Type[] typeArgs = { itemType };
@@ -405,4 +476,82 @@ public class CompilerHelper {
         
         return superClass;
     }
+    
+    public static void addClassToClassLoader(JavaClass cls, ClassLoader loader) {
+    
+        if(loader.getClass() == JaxbClassLoader.class && cls.getClass() == JavaClassImpl.class) {
+            Class wrappedClass = ((JavaClassImpl)cls).getJavaClass();
+            ((JaxbClassLoader)loader).putClass(wrappedClass.getName(), wrappedClass);
+        }
+    }
+    
+    static boolean hasNonAttributeJoinNodes(Property property) {
+        if(property.isSetXmlJoinNodes()) {
+            for(XmlJoinNodes.XmlJoinNode next: property.getXmlJoinNodes().getXmlJoinNode()) {
+                if(!(next.getXmlPath().startsWith("@"))) {
+                    return true;
+                }
+            }
+        } else if(property.isSetXmlJoinNodesList()) {
+            for(XmlJoinNodes nextNodes:property.getXmlJoinNodesList()) {
+                for(XmlJoinNodes.XmlJoinNode next: nextNodes.getXmlJoinNode()) {
+                    if(!(next.getXmlPath().startsWith("@"))) {
+                        return true;
+                    }
+                }
+            }
+            
+        }
+        return false;
+    }
+
+    public static Object createAccessorFor(JavaClass jClass, Property property, Helper helper, AccessorFactoryWrapper accessorFactory) {
+        if(!(jClass.getClass() == JavaClassImpl.class)) {
+            return null;
+        }
+        Class beanClass = ((JavaClassImpl)jClass).getJavaClass();
+        if(property.isMethodProperty()) {
+            try {
+                Method getMethod = null;
+                Method setMethod = null;
+                if(property.getGetMethodName() != null) {
+                    getMethod = PrivilegedAccessHelper.getMethod(beanClass, property.getGetMethodName(), new Class[]{}, true);
+                }
+                if(property.getSetMethodName() != null) {
+                    String setMethodParamTypeName = property.getType().getName();
+                    JavaClassImpl paramType = (JavaClassImpl)helper.getJavaClass(setMethodParamTypeName);
+                    Class[] setMethodParams = new Class[]{paramType.getJavaClass()};
+                    setMethod = PrivilegedAccessHelper.getMethod(beanClass, property.getSetMethodName(), setMethodParams, true);
+                }
+                return accessorFactory.createPropertyAccessor(beanClass, getMethod, setMethod);
+            } catch(Exception ex) {}      
+        }  else {
+            try {
+                Field field = PrivilegedAccessHelper.getField(beanClass, ((JavaField)property.getElement()).getName(), true);
+                return accessorFactory.createFieldAccessor(beanClass, field, property.isReadOnly());
+            } catch(Exception ex) {
+                ex.printStackTrace();
+            }
+        }
+        return null;
+    }
+
+    public static boolean isSimpleType(TypeInfo info) {
+        if (info.isEnumerationType()) {
+            return true;
+        }
+        Property xmlValueProperty = info.getXmlValueProperty();
+
+        boolean hasMappedAttributes = false;
+
+        for (Property nextProp : info.getPropertyList()) {
+            if (nextProp.isAttribute() && !nextProp.isTransient()) {
+                hasMappedAttributes = true;
+            }
+        }
+        hasMappedAttributes = hasMappedAttributes || info.hasPredicateProperties();
+
+        return (xmlValueProperty != null && !hasMappedAttributes);
+    }
+
 }
