@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -16,20 +16,26 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
 import org.eclipse.persistence.config.DescriptorCustomizer;
+import org.eclipse.persistence.internal.jaxb.AccessorFactoryWrapper;
+import org.eclipse.persistence.internal.oxm.mappings.Descriptor;
 import org.eclipse.persistence.internal.oxm.schema.model.ComplexType;
 import org.eclipse.persistence.internal.oxm.schema.model.Schema;
 import org.eclipse.persistence.internal.oxm.schema.model.SimpleType;
 import org.eclipse.persistence.internal.oxm.schema.model.TypeDefParticle;
 
-import org.eclipse.persistence.oxm.XMLDescriptor;
+import org.eclipse.persistence.oxm.XMLNameTransformer;
 
+import org.eclipse.persistence.jaxb.DefaultXMLNameTransformer;
 import org.eclipse.persistence.jaxb.javamodel.Helper;
 import org.eclipse.persistence.jaxb.javamodel.JavaClass;
 
+import org.eclipse.persistence.jaxb.xmlmodel.XmlNamedObjectGraph;
 import org.eclipse.persistence.jaxb.xmlmodel.XmlAccessOrder;
 import org.eclipse.persistence.jaxb.xmlmodel.XmlAccessType;
+import org.eclipse.persistence.jaxb.xmlmodel.XmlVirtualAccessMethods;
 import org.eclipse.persistence.jaxb.xmlmodel.XmlJavaTypeAdapter;
 import org.eclipse.persistence.jaxb.xmlmodel.XmlRootElement;
 import org.eclipse.persistence.jaxb.xmlmodel.XmlType;
@@ -56,9 +62,9 @@ import org.eclipse.persistence.jaxb.xmlmodel.XmlType;
  */
 
 public class TypeInfo {
-    private XMLDescriptor descriptor;
+    private Descriptor descriptor;
+    private String javaClassName;
     private ComplexType complexType;
-    private boolean hasRootElement;
     private String elementRefsPropertyName;
     private Schema schema;
     private SimpleType simpleType;
@@ -69,12 +75,20 @@ public class TypeInfo {
     private ArrayList<String> propertyNames;
     private ArrayList<Property> propertyList;// keep the keys in a list to preserve order
     private HashMap<String, Property> properties;
+    private HashMap<String, Property> originalProperties;
+    private Map<String, List<Property>> additionalProperties;
     private Property idProperty; // if there is an XmlID annotation, set the property for mappings gen
     private HashMap<String, JavaClass> packageLevelAdaptersByClass;
     private String objectFactoryClassName;
     private String factoryMethodName;
     private String[] factoryMethodParamTypes;
     private Property xmlValueProperty;
+    private String classExtractorName;
+    private Map<Object, Object> userProperties;
+    private List<Property> xmlKeyProperties;
+    private List<Property> predicateProperties;
+    
+    private List<XmlNamedObjectGraph> objectGraphs;
 
     private boolean isMixed;
     private boolean isTransient;
@@ -89,27 +103,44 @@ public class TypeInfo {
     private XmlAccessOrder xmlAccessOrder;
     private XmlJavaTypeAdapter xmlJavaTypeAdapter;
     private String xmlCustomizer;
+    private XmlVirtualAccessMethods xmlExtensible;
 
     private String anyAttributePropertyName;
     private String anyElementPropertyName;
     
     private boolean isBinaryDataInlined;
+    
+    private String xmlDiscriminatorNode;
+    private String xmlDiscriminatorValue;
+    
+    private AccessorFactoryWrapper xmlAccessorFactory;
+    private AccessorFactoryWrapper packageLevelXmlAccessorFactory;
 
-    /**
+    private static String EMPTY_STRING = "";
+    
+    private XMLNameTransformer xmlNameTransformer;
+    
+    private boolean isLocationAware = false;
+    private JavaClass javaClass;
+
+    public static XMLNameTransformer DEFAULT_NAME_TRANSFORMER = new DefaultXMLNameTransformer();
+ 
+	/**
      * This constructor sets the Helper to be used throughout XML and Annotations
      * processing.  Other fields are initialized here as well.
      * 
      * @param helper
      */
-    public TypeInfo(Helper helper) {
+    public TypeInfo(Helper helper, JavaClass javaClass) {
         propertyNames = new ArrayList<String>();
         properties = new HashMap<String, Property>();
+        originalProperties = new HashMap<String, Property>();
         propertyList = new ArrayList<Property>();
-        packageLevelAdaptersByClass = new HashMap<String, JavaClass>();
-
+        xmlNameTransformer = DEFAULT_NAME_TRANSFORMER;
         isSetXmlTransient = false;
         isPreBuilt = false;
         isPostBuilt = false;
+        this.javaClass = javaClass;
     }
 
     /**
@@ -117,7 +148,7 @@ public class TypeInfo {
      * 
      * @return the XmlDescriptor set on this TypeInfo, or null if none
      */
-    public XMLDescriptor getDescriptor() {
+    public Descriptor getDescriptor() {
         return descriptor;
     }
 
@@ -126,7 +157,7 @@ public class TypeInfo {
      * 
      * @param desc
      */
-    public void setDescriptor(XMLDescriptor desc) {
+    public void setDescriptor(Descriptor desc) {
     	descriptor = desc;
     }
 
@@ -233,12 +264,50 @@ public class TypeInfo {
     }
 
     /**
-     * Put a Property in the Map of Properties for this TypeInfo.
+     * Put a Property in the Map of Properties for this TypeInfo. Here, the
+     * original property is put in the originalProperties map, the 
+     * properties map, and the propertyList list.  It is assumed that
+     * the originalProperties map will remain unchanged during processing.
      * 
-     * @param name
-     * @param property
      */
     public void addProperty(String name, Property property) {
+        if(originalProperties.get(name) != null) {
+            // Duplicate property, see if either is transient and only keep the non-transient one
+            Property existingProperty = originalProperties.get(name);
+            if (existingProperty.isTransient() && property.isTransient()) {
+                // Both transient, keep the one that matches AccessType
+                if (this.getXmlAccessType().equals(XmlAccessType.FIELD)) {
+                    if (!property.isMethodProperty()) {
+                        // Existing property must be method property
+                        // Remove it and use the new property
+                        propertyList.remove(existingProperty);
+                        propertyNames.remove(name);
+                    } else {
+                        return;
+                    }
+                } else {
+                    if (property.isMethodProperty()) {
+                        // Existing property must be field property
+                        // Remove it and use the new property
+                        propertyList.remove(existingProperty);
+                        propertyNames.remove(name);
+                    } else {
+                        return;
+                    }
+                }
+            } else if (existingProperty.isTransient() || existingProperty.isSuperClassProperty()) {
+                // Continue, and overwrite it
+                propertyList.remove(existingProperty);
+                propertyNames.remove(name);
+            } else if (property.isTransient()) {
+                // Do nothing
+                return;
+            } else {
+                // Neither is transient - truly a duplicate property collision
+                throw org.eclipse.persistence.exceptions.JAXBException.duplicatePropertyName(name, this.getJavaClassName());
+            }
+        }
+        originalProperties.put(name, property);
         properties.put(name, property);
         propertyNames.add(name);
         propertyList.add(property);
@@ -264,7 +333,7 @@ public class TypeInfo {
             for (int i = 0; i < properties.size(); i++) {
                 Property next = properties.get(i);
                 this.addProperty(next.getPropertyName(), next);
-            }
+            }            
         }
     }
 
@@ -282,6 +351,13 @@ public class TypeInfo {
         }
     }
 
+    /**
+     * Indicates if this type info represents an enumeration.  Since 
+     * EnumTypeInfo is used for enum types, this method will always
+     * return false.
+     * 
+     * @return false
+     */
     public boolean isEnumerationType() {
         return false;
     }
@@ -323,7 +399,14 @@ public class TypeInfo {
      * @return
      */
     public JavaClass getPackageLevelAdapterClass(JavaClass boundType) {
-        return getPackageLevelAdaptersByClass().get(boundType.getQualifiedName());
+        if(hasPackageLevelAdaptersByClass()) {
+            return getPackageLevelAdaptersByClass().get(boundType.getQualifiedName());
+        }
+        return null;
+    }
+    
+    public boolean isAnonymousComplexType() {
+        return this.schemaTypeName == null || this.schemaTypeName.equals("");
     }
 
     /**
@@ -333,7 +416,10 @@ public class TypeInfo {
      * @return
      */
     public JavaClass getPackageLevelAdapterClass(String boundTypeName) {
-        return getPackageLevelAdaptersByClass().get(boundTypeName);
+        if(hasPackageLevelAdaptersByClass()) {
+            return getPackageLevelAdaptersByClass().get(boundTypeName);
+        }
+        return null;
     }
 
     /**
@@ -342,9 +428,16 @@ public class TypeInfo {
      * @return
      */
     public HashMap<String, JavaClass> getPackageLevelAdaptersByClass() {
+        if(!hasPackageLevelAdaptersByClass()) {
+            packageLevelAdaptersByClass = new HashMap<String, JavaClass>();
+        }
         return packageLevelAdaptersByClass;
     }
 
+    boolean hasPackageLevelAdaptersByClass() {
+        return null != packageLevelAdaptersByClass;
+    }
+
     /**
      * Put a bound type class to adapter class entry in the Map.
      * 
@@ -352,15 +445,11 @@ public class TypeInfo {
      * @param boundType
      */
     public void addPackageLevelAdapterClass(JavaClass adapterClass, JavaClass boundType) {
-    	packageLevelAdaptersByClass.put(boundType.getQualifiedName(), adapterClass);
+        getPackageLevelAdaptersByClass().put(boundType.getQualifiedName(), adapterClass);
     }
 
     public boolean hasRootElement() {
-        return hasRootElement;
-    }
-
-    public void setHasRootElement(boolean hasRoot) {
-        hasRootElement = hasRoot;
+        return xmlRootElement != null;
     }
 
     /**
@@ -456,42 +545,58 @@ public class TypeInfo {
      * 
      * @return
      */
-    public java.util.List<Property> getNonTransientPropertiesInPropOrder() {
-        java.util.List<Property> propertiesInOrder = new ArrayList<Property>();
+    public List<Property> getNonTransientPropertiesInPropOrder() {
+        List<Property> propertiesInOrder = new ArrayList<Property>();
         String[] propOrder = getPropOrder();
-        if (propOrder.length == 0 || propOrder[0].equals("")) {
+        if (propOrder.length == 0 || propOrder[0].equals(EMPTY_STRING)) {
             ArrayList<String> propertyNames = getPropertyNames();
             for (int i = 0; i < propertyNames.size(); i++) {
-                String nextPropertyKey = propertyNames.get(i);
-                Property next = getProperties().get(nextPropertyKey);
-                if (next != null && !next.isTransient()) {
-                    propertiesInOrder.add(next);
-                }
+                addPropertyToList(propertiesInOrder, propertyNames.get(i), null);
             }
         } else {
             ArrayList<String> propertyNamesCopy = new ArrayList<String>(getPropertyNames());
             for (int i = 0; i < propOrder.length; i++) {
                 // generate mappings based on the propOrder.
-                String propertyName = propOrder[i];
-                Property next = getProperties().get(propertyName);
-                if (next != null && !next.isTransient()) {
-                    propertyNamesCopy.remove(propertyName);
-                    propertiesInOrder.add(next);
-                }
+                addPropertyToList(propertiesInOrder, propOrder[i], propertyNamesCopy);
             }
-            // attributes may not be in the prop order in which case we need to generate those
-            // mappings also
+            // attributes may not be in the prop order in which 
+            // case we need to generate those mappings also
             for (int i = 0; i < propertyNamesCopy.size(); i++) {
-                String nextPropertyKey = propertyNamesCopy.get(i);
-                Property next = getProperties().get(nextPropertyKey);
-                if (next != null && !next.isTransient()) {
-                    propertiesInOrder.add(next);
-                }
+                addPropertyToList(propertiesInOrder, propertyNamesCopy.get(i), null);
             }
         }
         return propertiesInOrder;
     }
 
+    /**
+     * Convenience method that adds non-null, non-transient properties to a given 
+     * List<Property>.  The propertyName parameter is used to lookup the Property.
+     * If propertyNamesCopy is non-null, the Property will be removed from that 
+     * List.  Any additional properties that exist for propertyName (as in the 
+     * case of multiple mappings to the same field) will be added as well.
+     *  
+     */
+    private void addPropertyToList(List<Property> propertiesInOrder, String propertyName, ArrayList<String> propertyNamesCopy) {
+        Property next = getProperties().get(propertyName);
+        if (next != null && !next.isTransient()) {
+            if (propertyNamesCopy != null) { 
+                propertyNamesCopy.remove(propertyName);
+            }
+            propertiesInOrder.add(next);
+            // check for additional properties (i.e. multiple mappings to the same field)
+            if(hasAdditionalProperties()) {
+                List<Property> addProps = getAdditionalProperties().get(propertyName);
+                if (addProps != null) {
+                    for (Property addProp : addProps) {
+                        if (!addProp.isTransient()) {
+                            propertiesInOrder.add(addProp);
+                        }
+                    }
+                }
+            }
+        }
+    }
+    
     /**
      * Indicates if XmlTransient is set.
      * 
@@ -810,4 +915,297 @@ public class TypeInfo {
     public void setInlineBinaryData(boolean isBinaryDataInlined) {
         this.isBinaryDataInlined = isBinaryDataInlined;
     }
-}
+
+    /**
+     * Indicates if a ClassExtractor class name is set, 
+     * i.e. is non-null  
+     * 
+     * @return true if classExtractorName != null, otherwise false
+     */
+    public boolean isSetClassExtractorName() {
+        return classExtractorName != null;
+    }
+
+    /**
+     * Return the ClassExtractor class name.  
+     * 
+     * @return classExtractorName
+     */
+    public String getClassExtractorName() {
+        return classExtractorName;
+    }
+
+    /**
+     * Set the ClassExtractor class name.
+     * 
+     * @param classExtractorName
+     */
+    public void setClassExtractorName(String classExtractorName) {
+        this.classExtractorName = classExtractorName;
+    }
+
+    /**
+     * Return the Map of user-defined properties.
+     * 
+     * @return
+     */
+    public Map<Object, Object> getUserProperties() {
+        return userProperties;
+    }
+
+    /**
+     * Set the Map of user-defined properties.
+     * 
+     * @param userProperties
+     */
+    public void setUserProperties(Map<Object, Object> userProperties) {
+        this.userProperties = userProperties;
+    }
+
+    /**
+     * Return the JavaClass name for this typeinfo.
+     * 
+     * @return
+     */
+    public String getJavaClassName() {
+        return javaClassName;
+    }
+
+    /**
+     * Set the JavaClass name for this typeinfo.
+     * 
+     * @param javaClassName
+     */
+    public void setJavaClassName(String javaClassName) {
+        this.javaClassName = javaClassName;
+    }
+
+    /**
+     * Return the class indicator field name for this type info.
+     * 
+     * @return
+     */
+    public String getXmlDiscriminatorNode() {
+        return xmlDiscriminatorNode;
+    }
+
+    /**
+     * Sets the class indicator field name (for use with inheritance).  
+     *  
+     * @param xmlDiscriminatorNode
+     */
+    public void setXmlDiscriminatorNode(String xmlDiscriminatorNode) {
+        this.xmlDiscriminatorNode = xmlDiscriminatorNode;
+    }
+
+    /**
+     * Indicates if the class indicator field name is set, i.e. is non-null  
+     * 
+     * @return true if xmlDiscriminatorNode != null, otherwise false
+     */
+    public boolean isSetXmlDiscriminatorNode() {
+        return xmlDiscriminatorNode != null;
+    }
+
+    /**
+     * Return the value of the class indicator field for the java-type 
+     * associated with this type info.
+     * 
+     * @return
+     */
+    public String getXmlDiscriminatorValue() {
+        return xmlDiscriminatorValue;
+    }
+
+    /**
+     * Set the value of the class indicator field for the java-type 
+     * associated with this type info.
+     *    
+     * @param xmlDiscriminatorValue
+     */
+    public void setXmlDiscriminatorValue(String xmlDiscriminatorValue) {
+        this.xmlDiscriminatorValue = xmlDiscriminatorValue;
+    }
+    
+    /**
+     * Indicates if a class indicator field value has been set, 
+     * i.e. is non-null  
+     * 
+     * @return true if xmlDiscriminatorValue != null, otherwise false
+     */
+    public boolean isSetXmlDiscriminatorValue() {
+        return xmlDiscriminatorValue != null;
+    }
+
+    /**
+     * Add an XmlKey property to the list.  Each entry will be 
+     * set as a primary key on the owning descriptor.
+     * 
+     * @param xmlKeyProp
+     */
+    public void addXmlKeyProperty(Property xmlKeyProp) {
+        if (xmlKeyProperties == null) {
+            xmlKeyProperties = new ArrayList<Property>();
+        }
+        xmlKeyProperties.add(xmlKeyProp);
+    }
+    
+    /**
+     * Return the list of XmlKey property entries.  Each entry
+     * will be set as a primary key on the owning descriptor.
+     * 
+     * @return
+     */
+    public List<Property> getXmlKeyProperties() {
+        return xmlKeyProperties;
+    }
+    
+    /**
+     * Indicates if the list of XmlKey property entries has 
+     * been set, i.e. is non-null.
+     *  
+     * @return
+     */
+    public boolean hasXmlKeyProperties() {
+        return xmlKeyProperties != null;
+    }
+
+    /**
+     * Return a Map of property names to list of properties.  This Map will
+     * contain entries when one or more properties exist for a given field,
+     * as in the case of multiple mappings for a single field.
+     * 
+     */
+    public Map<String, List<Property>> getAdditionalProperties() {
+        if (!hasAdditionalProperties()) {
+            additionalProperties = new HashMap<String, List<Property>>();
+        }
+        return additionalProperties;
+    }
+
+    boolean hasAdditionalProperties() {
+        return null != additionalProperties;
+    }
+
+    /**
+     * Return the list of original properties for this type info.  These
+     * properties were created during annotations processing, and have 
+     * not been modified since.  
+     * 
+     * @return
+     */
+    public HashMap<String, Property> getOriginalProperties() {
+        return originalProperties;
+    }
+
+    public XmlVirtualAccessMethods getXmlVirtualAccessMethods() {
+        return xmlExtensible;
+    }
+
+    public void setXmlVirtualAccessMethods(XmlVirtualAccessMethods xmlExtensible) {
+        this.xmlExtensible = xmlExtensible;
+    }
+
+    
+    /**
+     * Return the XMLNameTransformer used when converting Java names to XML names
+     * @return
+     */
+    public XMLNameTransformer getXmlNameTransformer() {    	
+        return xmlNameTransformer;
+    }
+
+    /**
+     * Set the XMLNameTransformer used when converting Java names to XML names
+     * @param xmlNameTransformer
+     */
+    public void setXmlNameTransformer(XMLNameTransformer xmlNameTransformer) {
+        this.xmlNameTransformer = xmlNameTransformer;
+    }
+
+    
+    /**
+     * INTERNAL:
+     * Return the list of predicate properties. Lazy initializes the list.
+     * @return
+     */
+    public List<Property> getPredicateProperties() {
+        if(predicateProperties == null) {
+            predicateProperties = new ArrayList<Property>();
+        }
+        return predicateProperties;
+    }
+
+    /**
+     * INTERNAL:
+     * Return true if the property handed in is a duplicate of any existing predicate property.
+     */
+    public boolean hasPredicateProperty(Property property) {
+        if(this.predicateProperties != null) {
+            for(Property next:this.predicateProperties) {
+                if(next.getXmlPath().equals(property.getXmlPath())) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+    
+    /**
+     * INTERNAL:
+     * Return true if the list of predicate properties hasn't been initialized.
+     * @return
+     */
+    public boolean hasPredicateProperties() {
+        return this.predicateProperties != null && this.predicateProperties.size() > 0;
+    }
+
+    public boolean isLocationAware() {
+        return this.isLocationAware;
+    }
+
+    public void setLocationAware(boolean value) {
+        this.isLocationAware = value;
+    }
+
+    /**
+     * This event is called when all of the metadata for this type has been 
+     * processed and provides a chance to deference anything that is no longer 
+     * needed to reduce the memory footprint of this object.
+     */
+    void postInitialize() {
+        this.originalProperties = null;
+    }
+
+    public AccessorFactoryWrapper getXmlAccessorFactory() {
+        return xmlAccessorFactory;
+    }
+
+    public void setXmlAccessorFactory(AccessorFactoryWrapper xmlAccessorFactory) {
+        this.xmlAccessorFactory = xmlAccessorFactory;
+    }
+
+    public AccessorFactoryWrapper getPackageLevelXmlAccessorFactory() {
+        return packageLevelXmlAccessorFactory;
+    }
+
+    public void setPackageLevelXmlAccessorFactory(AccessorFactoryWrapper packageLevelXmlAccessorFactory) {
+        this.packageLevelXmlAccessorFactory = packageLevelXmlAccessorFactory;
+    }
+    
+    public List<XmlNamedObjectGraph> getObjectGraphs() {
+        if(this.objectGraphs == null) {
+            this.objectGraphs = new ArrayList<XmlNamedObjectGraph>();
+        }
+        return this.objectGraphs;
+    }
+
+    public JavaClass getJavaClass() {
+        return javaClass;
+    }
+
+    public void setJavaClass(JavaClass javaClass) {
+        this.javaClass = javaClass;
+    }
+
+}
\ No newline at end of file
