@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,6 +12,7 @@
  ******************************************************************************/  
 package org.eclipse.persistence.jaxb.javamodel.reflection;
 
+import org.eclipse.persistence.exceptions.JAXBException;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
 import org.eclipse.persistence.jaxb.javamodel.JavaAnnotation;
 import org.eclipse.persistence.jaxb.javamodel.JavaClass;
@@ -29,6 +30,7 @@ import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
+import java.lang.reflect.TypeVariable;
 import java.lang.reflect.WildcardType;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -54,19 +56,27 @@ public class JavaClassImpl implements JavaClass {
     protected ParameterizedType jType;
     protected Class jClass;
     protected JavaModelImpl javaModelImpl;
+    protected boolean isMetadataComplete;
+    protected JavaClass superClassOverride;
+
+    protected static String XML_REGISTRY_CLASS_NAME = "javax.xml.bind.annotation.XmlRegistry";
 
     public JavaClassImpl(Class javaClass, JavaModelImpl javaModelImpl) {
         this.jClass = javaClass;
         this.javaModelImpl = javaModelImpl;
+        isMetadataComplete = false;
     }
 
     public JavaClassImpl(ParameterizedType javaType, Class javaClass, JavaModelImpl javaModelImpl) {
         this.jType = javaType;
         this.jClass = javaClass;
         this.javaModelImpl = javaModelImpl;
-        
+        isMetadataComplete = false;
     }
 
+    public void setJavaModelImpl(JavaModelImpl javaModel) {
+        this.javaModelImpl = javaModel;
+    }
     public Collection getActualTypeArguments() {
         ArrayList<JavaClass> argCollection = new ArrayList<JavaClass>();
         if (jType instanceof ParameterizedType) {
@@ -90,6 +100,14 @@ public class JavaClassImpl implements JavaClass {
                     Class genericTypeClass = (Class)((GenericArrayType)type).getGenericComponentType();
                     genericTypeClass = java.lang.reflect.Array.newInstance(genericTypeClass, 0).getClass();
                     argCollection.add(javaModelImpl.getClass(genericTypeClass));
+                } else if(type instanceof TypeVariable) {
+                    Type[] boundTypes = ((TypeVariable) type).getBounds();
+                    if(boundTypes.length > 0) {
+                        Type boundType = boundTypes[0];
+                        if(boundType instanceof Class) {
+                            argCollection.add(javaModelImpl.getClass((Class) boundType));
+                        }
+                    }
                 }
             }
         }
@@ -104,7 +122,8 @@ public class JavaClassImpl implements JavaClass {
      * Assumes JavaType is a JavaClassImpl instance
      */
     public JavaAnnotation getAnnotation(JavaClass arg0) {
-        if (arg0 != null) {
+        // the only annotation we will return if isMetadataComplete == true is XmlRegistry
+        if (arg0 != null && (!isMetadataComplete || arg0.getQualifiedName().equals(XML_REGISTRY_CLASS_NAME))) {
             Class annotationClass = ((JavaClassImpl) arg0).getJavaClass();
             if (javaModelImpl.getAnnotationHelper().isAnnotationPresent(getAnnotatedElement(), annotationClass)) {
                 return new JavaAnnotationImpl(this.javaModelImpl.getAnnotationHelper().getAnnotation(getAnnotatedElement(), annotationClass));
@@ -115,9 +134,11 @@ public class JavaClassImpl implements JavaClass {
 
     public Collection getAnnotations() {
         ArrayList<JavaAnnotation> annotationCollection = new ArrayList<JavaAnnotation>();
-        Annotation[] annotations = javaModelImpl.getAnnotationHelper().getAnnotations(getAnnotatedElement());
-        for (Annotation annotation : annotations) {
-            annotationCollection.add(new JavaAnnotationImpl(annotation));
+        if (!isMetadataComplete) {
+            Annotation[] annotations = javaModelImpl.getAnnotationHelper().getAnnotations(getAnnotatedElement());
+            for (Annotation annotation : annotations) {
+                annotationCollection.add(new JavaAnnotationImpl(annotation));
+            }
         }
         return annotationCollection;
     }
@@ -211,7 +232,7 @@ public class JavaClassImpl implements JavaClass {
             }
         }
         try {
-            return new JavaConstructorImpl(jClass.getDeclaredConstructor(params), javaModelImpl);
+            return new JavaConstructorImpl(PrivilegedAccessHelper.getDeclaredConstructorFor(this.jClass, params, true), javaModelImpl);
         } catch (NoSuchMethodException nsme) {
             return null;
         }
@@ -293,14 +314,20 @@ public class JavaClassImpl implements JavaClass {
     }
 
     public JavaPackage getPackage() {
-        return new JavaPackageImpl(jClass.getPackage(), javaModelImpl);
+        return new JavaPackageImpl(jClass.getPackage(), javaModelImpl, isMetadataComplete);
     }
 
     public String getPackageName() {
         if(jClass.getPackage() != null){
             return jClass.getPackage().getName();
         }else{
-            String className = jClass.getCanonicalName();
+            Class nonInnerClass = jClass;
+            Class enclosingClass = jClass.getEnclosingClass();
+            while(enclosingClass != null){
+            	nonInnerClass = enclosingClass;
+            	enclosingClass = nonInnerClass.getEnclosingClass();
+            }
+            String className = nonInnerClass.getCanonicalName();            
             if(className !=null){
                 int index = className.lastIndexOf(".");
                 if(index > -1){
@@ -320,25 +347,51 @@ public class JavaClassImpl implements JavaClass {
     }
 
     public JavaClass getSuperclass() {
+        if(this.superClassOverride != null) {
+            return this.superClassOverride;
+        }
+        if(jClass.isInterface()) {
+            Class[] superInterfaces = jClass.getInterfaces();
+            if(superInterfaces != null) {
+                if(superInterfaces.length == 1) {
+                    return javaModelImpl.getClass(superInterfaces[0]);
+                } else {
+                    Class parent = null;
+                    for(Class next:superInterfaces) {
+                        if(!(next.getName().startsWith("java.") || next.getName().startsWith("javax."))) {
+                            if(parent == null) {
+                                parent = next;
+                            } else {
+                                throw JAXBException.invalidInterface(jClass.getName());
+                            }
+                        }
+                    }
+                    return javaModelImpl.getClass(parent);
+                }
+            }
+        }
         return javaModelImpl.getClass(jClass.getSuperclass());
     }
 
-    public boolean hasActualTypeArguments() {
-        if (jType instanceof ParameterizedType) {
-            ParameterizedType pType = (ParameterizedType) jType;
-            if (pType.getActualTypeArguments() != null && pType.getActualTypeArguments().length > 0) {
-                return true;
-            }
-        }
-        return false;
+    @Override
+    public Type[] getGenericInterfaces() {
+        return jClass.getGenericInterfaces();
     }
+
+    public Type getGenericSuperclass() {
+        return jClass.getGenericSuperclass();
+    }
+
+    public boolean hasActualTypeArguments() {
+        return getActualTypeArguments().size() > 0;        
+    }    
     
     public JavaField getJavaField(Field field) {
-    	return new JavaFieldImpl(field, javaModelImpl);
+    	return new JavaFieldImpl(field, javaModelImpl, isMetadataComplete);
     }
     
     public JavaMethod getJavaMethod(Method method) {
-    	return new JavaMethodImpl(method, javaModelImpl);
+    	return new JavaMethodImpl(method, javaModelImpl, isMetadataComplete);
     }
 
     public JavaClass getOwningClass() {
@@ -357,13 +410,63 @@ public class JavaClassImpl implements JavaClass {
     	return jClass;
     }
 
-    /**
-     * Assumes JavaType is a JavaClassImpl instance
-     */
     public boolean isAssignableFrom(JavaClass arg0) {
+        if (!(arg0 instanceof JavaClassImpl)) {
+            return false;
+        }
+        if(hasCustomSuperClass(arg0)) {
+            return this.customIsAssignableFrom(arg0);
+        }
         return jClass.isAssignableFrom(((JavaClassImpl) arg0).getJavaClass());
     }
 
+    private boolean customIsAssignableFrom(JavaClass arg0) {
+        JavaClassImpl jClass = (JavaClassImpl)arg0;
+        Class cls = jClass.getJavaClass();
+        
+        if(cls == this.jClass) {
+            return true;
+        }
+        Class[] interfaces = cls.getInterfaces();
+        for(Class nextInterface:interfaces) {
+            if(nextInterface == this.jClass) {
+                return true;
+            }
+            if(customIsAssignableFrom(javaModelImpl.getClass(nextInterface))) {
+                return true;
+            }
+        }
+        
+        if(!(jClass.isInterface())) {
+            JavaClassImpl superJavaClass = (JavaClassImpl)jClass.getSuperclass();
+            if(superJavaClass.getName().equals("java.lang.Object")) {
+                return this.jClass == superJavaClass.getJavaClass(); 
+            }
+            return customIsAssignableFrom(superJavaClass);
+        } 
+        return false;
+    }
+
+    private boolean hasCustomSuperClass(JavaClass arg0) {
+        if(arg0 == null) {
+            return false;
+        }
+        if(!this.javaModelImpl.hasXmlBindings()) {
+        	return false;
+        }
+        if(!(arg0.getClass() == this.getClass())) { 
+            return false;
+        }
+        if(arg0.getName().equals("java.lang.Object")) {
+            return false;
+        }
+        JavaClassImpl jClass = (JavaClassImpl)arg0;
+        if(jClass.getSuperClassOverride() != null) {
+            return true;
+        }
+        return hasCustomSuperClass(jClass.getSuperclass());
+    }
+
     public boolean isEnum() {
         return jClass.isEnum();
     }
@@ -412,7 +515,6 @@ public class JavaClassImpl implements JavaClass {
         return jClass.isSynthetic();
     }
 
-//---------------- unimplemented methods ----------------//
     public JavaClass getComponentType() {
         if(!isArray()) {
             return null;
@@ -420,12 +522,48 @@ public class JavaClassImpl implements JavaClass {
         return javaModelImpl.getClass(this.jClass.getComponentType());
     }
 
+    public JavaClass getSuperClassOverride() {
+        return superClassOverride;
+    }
+
+    public void setSuperClassOverride(JavaClass superClassOverride) {
+        this.superClassOverride = superClassOverride;
+    }
+    /**
+     * Set the indicator for XML metadata complete - if true, 
+     * annotations will be ignored.
+     * 
+     * @param isMetadataComplete
+     */
+    void setIsMetadataComplete(Boolean isMetadataComplete) {
+       if(isMetadataComplete != null){
+            this.isMetadataComplete = isMetadataComplete;
+        }      
+    }
+
     public JavaAnnotation getDeclaredAnnotation(JavaClass arg0) {
+        // the only annotation we will return if isMetadataComplete == true is XmlRegistry
+        if (arg0 != null && (!isMetadataComplete || arg0.getQualifiedName().equals(XML_REGISTRY_CLASS_NAME))) {
+            Class annotationClass = ((JavaClassImpl) arg0).getJavaClass();
+            Annotation[] annotations = javaModelImpl.getAnnotationHelper().getDeclaredAnnotations(getAnnotatedElement());
+            for (Annotation annotation : annotations) {
+                if (annotation.annotationType().equals(annotationClass)) {
+                    return new JavaAnnotationImpl(annotation);
+                }
+            }
+        }
         return null;
     }
 
     public Collection getDeclaredAnnotations() {
-        return null;
+        ArrayList<JavaAnnotation> annotationCollection = new ArrayList<JavaAnnotation>();
+        if (!isMetadataComplete) {
+            Annotation[] annotations = javaModelImpl.getAnnotationHelper().getDeclaredAnnotations(getAnnotatedElement());
+            for (Annotation annotation : annotations) {
+                annotationCollection.add(new JavaAnnotationImpl(annotation));
+            }
+        }
+        return annotationCollection;
     }
 
 }
\ No newline at end of file
