@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
  * which accompanies this distribution.
@@ -71,6 +71,9 @@ import javax.xml.bind.annotation.XmlType;
  *       &lt;/all>
  *       &lt;attribute name="xml-accessor-type" type="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-access-type" default="PUBLIC_MEMBER" />
  *       &lt;attribute name="xml-accessor-order" type="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-access-order" default="UNDEFINED" />
+ *       &lt;attribute name="xml-mapping-metadata-complete" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
+ *       &lt;attribute name="package-name" type="{http://www.w3.org/2001/XMLSchema}string" default="##default" />
+ *       &lt;attribute name="xml-name-transformer" type="{http://www.w3.org/2001/XMLSchema}string" />
  *     &lt;/restriction>
  *   &lt;/complexContent>
  * &lt;/complexType>
@@ -103,6 +106,12 @@ public class XmlBindings {
     protected org.eclipse.persistence.jaxb.xmlmodel.XmlAccessType xmlAccessorType;
     @XmlAttribute(name = "xml-accessor-order")
     protected XmlAccessOrder xmlAccessorOrder;
+    @XmlAttribute(name = "xml-mapping-metadata-complete")
+    protected Boolean xmlMappingMetadataComplete;
+    @XmlAttribute(name = "package-name")
+    protected String packageName;
+    @XmlAttribute(name = "xml-name-transformer")
+    protected String xmlNameTransformer;
 
     /**
      * Gets the value of the xmlSchema property.
@@ -346,6 +355,85 @@ public class XmlBindings {
         return xmlAccessorOrder != null;
     }
 
+    /**
+     * Gets the value of the xmlMappingMetadataComplete property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link Boolean }
+     *     
+     */
+    public boolean isXmlMappingMetadataComplete() {
+        if (xmlMappingMetadataComplete == null) {
+            return false;
+        } else {
+            return xmlMappingMetadataComplete;
+        }
+    }
+
+    /**
+     * Sets the value of the xmlMappingMetadataComplete property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link Boolean }
+     *     
+     */
+    public void setXmlMappingMetadataComplete(Boolean value) {
+        this.xmlMappingMetadataComplete = value;
+    }
+
+    /**
+     * Gets the value of the packageName property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link String }
+     *     
+     */
+    public String getPackageName() {
+        if (packageName == null) {
+            return "##default";
+        } else {
+            return packageName;
+        }
+    }
+
+    /**
+     * Sets the value of the packageName property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link String }
+     *     
+     */
+    public void setPackageName(String value) {
+        this.packageName = value;
+    }
+
+    /**
+     * Gets the value of the xmlNameTransformer property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link String }
+     *     
+     */
+    public String getXmlNameTransformer() {
+        return xmlNameTransformer;
+    }
+
+    /**
+     * Sets the value of the xmlNameTransformer property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link String }
+     *     
+     */
+    public void setXmlNameTransformer(String value) {
+        this.xmlNameTransformer = value;
+    }
 
 
     /**
@@ -526,5 +614,9 @@ public class XmlBindings {
         }
 
     }
+    
+    public boolean isSetXmlMappingMetadataComplete() {
+        return this.xmlMappingMetadataComplete != null;
+    }
 
 }
