@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
  * which accompanies this distribution.
@@ -33,8 +33,18 @@ import javax.xml.bind.annotation.XmlType;
  *         &lt;element ref="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-abstract-null-policy" minOccurs="0"/>
  *         &lt;element ref="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-access-methods" minOccurs="0"/>
  *         &lt;element ref="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-element-wrapper" minOccurs="0"/>
+ *         &lt;element name="xml-inverse-reference" minOccurs="0">
+ *           &lt;complexType>
+ *             &lt;complexContent>
+ *               &lt;restriction base="{http://www.w3.org/2001/XMLSchema}anyType">
+ *                 &lt;attribute name="mapped-by" type="{http://www.w3.org/2001/XMLSchema}string" />
+ *               &lt;/restriction>
+ *             &lt;/complexContent>
+ *           &lt;/complexType>
+ *         &lt;/element>
  *         &lt;element ref="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-java-type-adapter" minOccurs="0"/>
  *         &lt;element ref="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-map" minOccurs="0"/>
+ *         &lt;element ref="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-properties" minOccurs="0"/>
  *         &lt;element ref="{http://www.eclipse.org/eclipselink/xsds/persistence/oxm}xml-schema-type" minOccurs="0"/>
  *       &lt;/all>
  *       &lt;attribute name="name" type="{http://www.w3.org/2001/XMLSchema}string" default="##default" />
@@ -42,9 +52,11 @@ import javax.xml.bind.annotation.XmlType;
  *       &lt;attribute name="default-value" type="{http://www.w3.org/2001/XMLSchema}string" />
  *       &lt;attribute name="nillable" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *       &lt;attribute name="required" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
+ *       &lt;attribute name="container-type" type="{http://www.w3.org/2001/XMLSchema}string" default="##default" />
  *       &lt;attribute name="type" type="{http://www.w3.org/2001/XMLSchema}string" default="javax.xml.bind.annotation.XmlElement.DEFAULT" />
  *       &lt;attribute name="xml-id" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *       &lt;attribute name="xml-idref" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
+ *       &lt;attribute name="xml-key" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *       &lt;attribute name="xml-list" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *       &lt;attribute name="xml-inline-binary-data" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *       &lt;attribute name="xml-attachment-ref" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
@@ -53,6 +65,7 @@ import javax.xml.bind.annotation.XmlType;
  *       &lt;attribute name="write-only" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *       &lt;attribute name="cdata" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *       &lt;attribute name="xml-path" type="{http://www.w3.org/2001/XMLSchema}string" />
+ *       &lt;attribute name="xml-location" type="{http://www.w3.org/2001/XMLSchema}boolean" default="false" />
  *     &lt;/extension>
  *   &lt;/complexContent>
  * &lt;/complexType>
@@ -65,8 +78,10 @@ import javax.xml.bind.annotation.XmlType;
     "xmlAbstractNullPolicy",
     "xmlAccessMethods",
     "xmlElementWrapper",
+    "xmlInverseReference",
     "xmlJavaTypeAdapter",
     "xmlMap",
+    "xmlProperties",
     "xmlSchemaType"
 })
 public class XmlElement
@@ -79,28 +94,36 @@ public class XmlElement
     protected XmlAccessMethods xmlAccessMethods;
     @javax.xml.bind.annotation.XmlElement(name = "xml-element-wrapper")
     protected XmlElementWrapper xmlElementWrapper;
+    @javax.xml.bind.annotation.XmlElement(name = "xml-inverse-reference")
+    protected XmlElement.XmlInverseReference xmlInverseReference;
     @javax.xml.bind.annotation.XmlElement(name = "xml-java-type-adapter")
     protected XmlJavaTypeAdapter xmlJavaTypeAdapter;
     @javax.xml.bind.annotation.XmlElement(name = "xml-map")
     protected XmlMap xmlMap;
+    @javax.xml.bind.annotation.XmlElement(name = "xml-properties")
+    protected XmlProperties xmlProperties;
     @javax.xml.bind.annotation.XmlElement(name = "xml-schema-type")
     protected XmlSchemaType xmlSchemaType;
-    @XmlAttribute
+    @XmlAttribute(name = "name")
     protected String name;
-    @XmlAttribute
+    @XmlAttribute(name = "namespace")
     protected String namespace;
     @XmlAttribute(name = "default-value")
     protected String defaultValue;
-    @XmlAttribute
+    @XmlAttribute(name = "nillable")
     protected Boolean nillable;
-    @XmlAttribute
+    @XmlAttribute(name = "required")
     protected Boolean required;
-    @XmlAttribute
+    @XmlAttribute(name = "container-type")
+    protected String containerType;
+    @XmlAttribute(name = "type")
     protected String type;
     @XmlAttribute(name = "xml-id")
     protected Boolean xmlId;
     @XmlAttribute(name = "xml-idref")
     protected Boolean xmlIdref;
+    @XmlAttribute(name = "xml-key")
+    protected Boolean xmlKey;
     @XmlAttribute(name = "xml-list")
     protected Boolean xmlList;
     @XmlAttribute(name = "xml-inline-binary-data")
@@ -113,19 +136,21 @@ public class XmlElement
     protected Boolean readOnly;
     @XmlAttribute(name = "write-only")
     protected Boolean writeOnly;
-    @XmlAttribute
+    @XmlAttribute(name = "cdata")
     protected Boolean cdata;
     @XmlAttribute(name = "xml-path")
     protected String xmlPath;
+    @XmlAttribute(name = "xml-location")
+    protected Boolean xmlLocation;
 
     /**
      * Gets the value of the xmlAbstractNullPolicy property.
      * 
      * @return
      *     possible object is
+     *     {@link JAXBElement }{@code <}{@link XmlNullPolicy }{@code >}
      *     {@link JAXBElement }{@code <}{@link XmlAbstractNullPolicy }{@code >}
      *     {@link JAXBElement }{@code <}{@link XmlIsSetNullPolicy }{@code >}
-     *     {@link JAXBElement }{@code <}{@link XmlNullPolicy }{@code >}
      *     
      */
     public JAXBElement<? extends XmlAbstractNullPolicy> getXmlAbstractNullPolicy() {
@@ -137,9 +162,9 @@ public class XmlElement
      * 
      * @param value
      *     allowed object is
+     *     {@link JAXBElement }{@code <}{@link XmlNullPolicy }{@code >}
      *     {@link JAXBElement }{@code <}{@link XmlAbstractNullPolicy }{@code >}
      *     {@link JAXBElement }{@code <}{@link XmlIsSetNullPolicy }{@code >}
-     *     {@link JAXBElement }{@code <}{@link XmlNullPolicy }{@code >}
      *     
      */
     public void setXmlAbstractNullPolicy(JAXBElement<? extends XmlAbstractNullPolicy> value) {
@@ -194,6 +219,30 @@ public class XmlElement
         this.xmlElementWrapper = value;
     }
 
+    /**
+     * Gets the value of the xmlInverseReference property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link XmlElement.XmlInverseReference }
+     *     
+     */
+    public XmlElement.XmlInverseReference getXmlInverseReference() {
+        return xmlInverseReference;
+    }
+
+    /**
+     * Sets the value of the xmlInverseReference property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link XmlElement.XmlInverseReference }
+     *     
+     */
+    public void setXmlInverseReference(XmlElement.XmlInverseReference value) {
+        this.xmlInverseReference = value;
+    }
+
     /**
      * Gets the value of the xmlJavaTypeAdapter property.
      * 
@@ -242,6 +291,30 @@ public class XmlElement
         this.xmlMap = value;
     }
 
+    /**
+     * Gets the value of the xmlProperties property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link XmlProperties }
+     *     
+     */
+    public XmlProperties getXmlProperties() {
+        return xmlProperties;
+    }
+
+    /**
+     * Sets the value of the xmlProperties property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link XmlProperties }
+     *     
+     */
+    public void setXmlProperties(XmlProperties value) {
+        this.xmlProperties = value;
+    }
+
     /**
      * Gets the value of the xmlSchemaType property.
      * 
@@ -405,6 +478,34 @@ public class XmlElement
         this.required = value;
     }
 
+    /**
+     * Gets the value of the containerType property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link String }
+     *     
+     */
+    public String getContainerType() {
+        if (containerType == null) {
+            return "##default";
+        } else {
+            return containerType;
+        }
+    }
+
+    /**
+     * Sets the value of the containerType property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link String }
+     *     
+     */
+    public void setContainerType(String value) {
+        this.containerType = value;
+    }
+
     /**
      * Gets the value of the type property.
      * 
@@ -489,6 +590,34 @@ public class XmlElement
         this.xmlIdref = value;
     }
 
+    /**
+     * Gets the value of the xmlKey property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link Boolean }
+     *     
+     */
+    public boolean isXmlKey() {
+        if (xmlKey == null) {
+            return false;
+        } else {
+            return xmlKey;
+        }
+    }
+
+    /**
+     * Sets the value of the xmlKey property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link Boolean }
+     *     
+     */
+    public void setXmlKey(Boolean value) {
+        this.xmlKey = value;
+    }
+
     /**
      * Gets the value of the xmlList property.
      * 
@@ -741,4 +870,83 @@ public class XmlElement
         this.xmlPath = value;
     }
 
+    /**
+     * Gets the value of the xmlLocation property.
+     * 
+     * @return
+     *     possible object is
+     *     {@link Boolean }
+     *     
+     */
+    public boolean isXmlLocation() {
+        if (xmlLocation == null) {
+            return false;
+        } else {
+            return xmlLocation;
+        }
+    }
+
+    /**
+     * Sets the value of the xmlLocation property.
+     * 
+     * @param value
+     *     allowed object is
+     *     {@link Boolean }
+     *     
+     */
+    public void setXmlLocation(Boolean value) {
+        this.xmlLocation = value;
+    }
+
+
+    /**
+     * <p>Java class for anonymous complex type.
+     * 
+     * <p>The following schema fragment specifies the expected content contained within this class.
+     * 
+     * <pre>
+     * &lt;complexType>
+     *   &lt;complexContent>
+     *     &lt;restriction base="{http://www.w3.org/2001/XMLSchema}anyType">
+     *       &lt;attribute name="mapped-by" type="{http://www.w3.org/2001/XMLSchema}string" />
+     *     &lt;/restriction>
+     *   &lt;/complexContent>
+     * &lt;/complexType>
+     * </pre>
+     * 
+     * 
+     */
+    @XmlAccessorType(XmlAccessType.FIELD)
+    @XmlType(name = "")
+    public static class XmlInverseReference {
+
+        @XmlAttribute(name = "mapped-by")
+        protected String mappedBy;
+
+        /**
+         * Gets the value of the mappedBy property.
+         * 
+         * @return
+         *     possible object is
+         *     {@link String }
+         *     
+         */
+        public String getMappedBy() {
+            return mappedBy;
+        }
+
+        /**
+         * Sets the value of the mappedBy property.
+         * 
+         * @param value
+         *     allowed object is
+         *     {@link String }
+         *     
+         */
+        public void setMappedBy(String value) {
+            this.mappedBy = value;
+        }
+
+    }
+
 }
