@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -148,6 +148,9 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      * null value is default behavior of determining from log level.
      */
     protected Boolean shouldPrintConnection;
+    
+    /** Used to determine if bingdparameters should be logged or hidden. */
+    protected Boolean shouldDisplayData;
 
 
     /**
@@ -220,7 +223,7 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      *
      * @return the log level
      * </p><p>
-     * @param category  the string representation of a TopLink category, e.g. "sql", "transaction" ...
+     * @param category  the string representation of a EclipseLink category, e.g. "sql", "transaction" ...
      * </p>
      */
     public int getLevel(String category) {
@@ -247,13 +250,27 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      * </p><p>
      *
      * @param level     the new log level
-     * @param category  the string representation of a TopLink category, e.g. "sql", "transaction" ...
+     * @param category  the string representation of an EclipseLink category, e.g. "sql", "transaction" ...
      * </p>
      */
     public void setLevel(int level, String category) {
         this.level = level;
     }
 
+    /**
+     * PUBLIC:
+     * Return true if SQL logging should log visible bind parameters. If the 
+     * shouldDisplayData is not set, check the session log level and return 
+     * true for a level greater than CONFIG.
+     */
+    public boolean shouldDisplayData() {
+        if (this.shouldDisplayData != null) {
+            return shouldDisplayData.booleanValue();
+        } else {
+            return this.level < SessionLog.CONFIG;
+        }
+    }
+    
     /**
      * PUBLIC:
      * <p>
@@ -281,7 +298,7 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      * @return true if the given message level will be logged
      * </p><p>
      * @param level  the log request level
-     * @param category  the string representation of a TopLink category, e.g. "sql", "transaction" ...* </p>
+     * @param category  the string representation of an EclipseLink category, e.g. "sql", "transaction" ...* </p>
      * </p>
      */
     public boolean shouldLog(int level, String category) {
@@ -362,7 +379,7 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         if (!shouldLog(level)) {
             return;
         }
-		//Bug#4566524  Pass in false for external use
+        //Bug#4566524  Pass in false for external use
         log(level, message, (Object[])null, false);
     }
 
@@ -386,6 +403,26 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         log(level, message, new Object[] { param });
     }
 
+    /**
+     * PUBLIC:
+     * <p>
+     * Log a message with one parameter that needs to be translated.
+     * </p><p>
+     *
+     * @param level  the log request level value
+     * </p><p>
+     * @param message  the string message
+     * </p><p>
+     * @param param  a parameter of the message
+     * </p>
+     */
+    public void log(int level, String category, String message, Object param) {
+        if (!shouldLog(level, category)) {
+            return;
+        }
+        log(level, category, message, new Object[] { param }, true);
+    }
+
     /**
      * PUBLIC:
      * <p>
@@ -408,6 +445,28 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         log(level, message, new Object[] { param1, param2 });
     }
 
+    /**
+     * PUBLIC:
+     * <p>
+     * Log a message with two parameters that needs to be translated.
+     * </p><p>
+     *
+     * @param level the log request level value
+     * </p><p>
+     * @param message the string message
+     * </p><p>
+     * @param param1  a parameter of the message
+     * </p><p>
+     * @param param2  second parameter of the message
+     * </p>
+     */
+    public void log(int level, String category, String message, Object param1, Object param2) {
+        if (!shouldLog(level)) {
+            return;
+        }
+        log(level, category, message, new Object[] { param1, param2 }, true);
+    }
+
     /**
      * PUBLIC:
      * <p>
@@ -432,6 +491,30 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         log(level, message, new Object[] { param1, param2, param3 });
     }
 
+    /**
+     * PUBLIC:
+     * <p>
+     * Log a message with three parameters that needs to be translated.
+     * </p><p>
+     *
+     * @param level the log request level value
+     * </p><p>
+     * @param message the string message
+     * </p><p>
+     * @param param1  a parameter of the message
+     * </p><p>
+     * @param param2  second parameter of the message
+     * </p><p>
+     * @param param3  third parameter of the message
+     * </p>
+     */
+    public void log(int level, String category, String message, Object param1, Object param2, Object param3) {
+        if (!shouldLog(level)) {
+            return;
+        }
+        log(level, category, message, new Object[] { param1, param2, param3 }, true);
+    }
+
     /**
      * PUBLIC:
      * <p>
@@ -457,6 +540,32 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         }
         log(level, message, new Object[] { param1, param2, param3, param4 });
     }
+
+    /**
+     * PUBLIC:
+     * <p>
+     * Log a message with four parameters that needs to be translated.
+     * </p><p>
+     *
+     * @param level the log request level value
+     * </p><p>
+     * @param message the string message
+     * </p><p>
+     * @param param1  a parameter of the message
+     * </p><p>
+     * @param param2  second parameter of the message
+     * </p><p>
+     * @param param3  third parameter of the message
+     * </p><p>
+     * @param param4  third parameter of the message
+     * </p>
+     */
+    public void log(int level, String category, String message, Object param1, Object param2, Object param3, Object param4) {
+        if (!shouldLog(level)) {
+            return;
+        }
+        log(level, category, message, new Object[] { param1, param2, param3, param4 }, true);
+    }
     
     /**
      * PUBLIC:
@@ -474,6 +583,23 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
     public void log(int level, String message, Object[] params) {
         log(level, message, params, true);
     }
+    
+    /**
+     * PUBLIC:
+     * <p>
+     * Log a message with an array of parameters that needs to be translated.
+     * </p><p>
+     *
+     * @param level the log request level value
+     * </p><p>
+     * @param message the string message
+     * </p><p>
+     * @param params array of parameters to the message
+     * </p>
+     */
+    public void log(int level, String category, String message, Object[] params) {
+        log(level, category, message, params, true);
+    }
 
     /**
      * PUBLIC:
@@ -496,6 +622,30 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         }
         log(new SessionLogEntry(level, null, message, params, null, shouldTranslate));
     }
+    
+    /**
+     * PUBLIC:
+     * <p>
+     * Log a message.  shouldTranslate determines if the message needs to be translated.
+     * </p><p>
+     *
+     * @param level the log request level
+     * </p><p>
+     * @param message the string message
+     * </p><p>
+     * @param category the log category
+     * </p><p>
+     * @param params array of parameters to the message
+     * </p><p>
+     * @param shouldTranslate true if the message needs to be translated
+     * </p>
+     */
+    public void log(int level, String category, String message, Object[] params, boolean shouldTranslate) {
+        if (!shouldLog(level, category)) {
+            return;
+        }
+        log(new SessionLogEntry(level, category, null, message, params, null, shouldTranslate));
+    }
 
     /**
      * PUBLIC:
@@ -558,6 +708,14 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         }
     }
 
+    /**
+     * PUBLIC:
+     * Set whether bind parameters should be displayed when logging SQL.
+     */
+    public void setShouldDisplayData(Boolean shouldDisplayData) {
+        this.shouldDisplayData = shouldDisplayData;
+    }
+    
     /**
      * By default the stack is logged for FINER or less (finest).
      * The logging of the stack can also be explicitly turned on or off.
@@ -743,7 +901,7 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      */
 
     //Bug3135111  Prefix strings are not translated until the first time they are used.
-    protected void printPrefixString(int level) {
+    protected void printPrefixString(int level, String category) {
         try {
             switch (level) {
             case SEVERE:
@@ -794,6 +952,10 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
                 }
                 this.getWriter().write(TOPLINK_PREFIX);
             }
+            if (category != null) {
+                this.getWriter().write(category);
+                this.getWriter().write(": ");
+            }
         } catch (IOException exception) {
             throw ValidationException.logIOError(exception);
         }
@@ -1010,6 +1172,24 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
         }
     }
 
+    /**
+     * PUBLIC:
+     * <p>
+     * Log a throwable with level.
+     * </p><p>
+     *
+     * @param level  the log request level value
+     * </p><p>
+     * @param throwable  a Throwable
+     * </p>
+     */
+    public void logThrowable(int level, String category, Throwable throwable) {
+        // Must not create the log if not logging as is a performance issue.
+        if (shouldLog(level, category)) {
+            log(new SessionLogEntry(null, level, category, throwable));
+        }
+    }
+
     /**
      * INTERNAL:
      * Check if the log level is set to off.
