@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -27,31 +27,36 @@ import org.eclipse.persistence.sessions.Session;
  * <p>
  * This class defines Eclipselink logging levels (that are used throughout EclipseLink code) with the following integer values:
  * <table>
- * <tr><td>&nbsp;</td><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
- * <tr><td>&nbsp;</td><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
- * <tr><td>&nbsp;</td><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
- * <tr><td>&nbsp;</td><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
- * <tr><td>&nbsp;</td><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
- * <tr><td>&nbsp;</td><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
- * <tr><td>&nbsp;</td><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
- * <tr><td>&nbsp;</td><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
- * <tr><td>&nbsp;</td><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+ * <tr><td>&nbsp;</td><td>ALL</td>    <td>&nbsp;</td><td>= 0</td></tr>
+ * <tr><td>&nbsp;</td><td>FINEST</td> <td>&nbsp;</td><td>= 1</td></tr>
+ * <tr><td>&nbsp;</td><td>FINER</td>  <td>&nbsp;</td><td>= 2</td></tr>
+ * <tr><td>&nbsp;</td><td>FINE</td>   <td>&nbsp;</td><td>= 3</td></tr>
+ * <tr><td>&nbsp;</td><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td></tr>
+ * <tr><td>&nbsp;</td><td>INFO</td>   <td>&nbsp;</td><td>= 5</td></tr>
+ * <tr><td>&nbsp;</td><td>WARNING</td><td>&nbsp;</td><td>= 6</td></tr>
+ * <tr><td>&nbsp;</td><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td></tr>
+ * <tr><td>&nbsp;</td><td>OFF</td>    <td>&nbsp;</td><td>= 8</td></tr>
  * </table>
  * <p>
  * In addition, EclipseLink categories used for logging name space are defined with the following String values:
  * <table>
- * <tr><td>&nbsp;</td><td>SQL</td>           <td>&nbsp;</td><td>= "sql"</td>
- * <tr><td>&nbsp;</td><td>TRANSACTION</td>   <td>&nbsp;</td><td>= "transaction"</td>
- * <tr><td>&nbsp;</td><td>EVENT</td>         <td>&nbsp;</td><td>= "event"</td>
- * <tr><td>&nbsp;</td><td>QUERY</td>         <td>&nbsp;</td><td>= "query"</td>
- * <tr><td>&nbsp;</td><td>CACHE</td>         <td>&nbsp;</td><td>= "cache"</td>
- * <tr><td>&nbsp;</td><td>PROPAGATION</td>   <td>&nbsp;</td><td>= "propagation"</td>
- * <tr><td>&nbsp;</td><td>SEQUENCING</td>    <td>&nbsp;</td><td>= "sequencing"</td>
- * <tr><td>&nbsp;</td><td>EJB</td>           <td>&nbsp;</td><td>= "ejb"</td>
- * <tr><td>&nbsp;</td><td>DMS</td>           <td>&nbsp;</td><td>= "dms"</td>
- * <tr><td>&nbsp;</td><td>EJB_ANNOTATION</td><td>&nbsp;</td><td>= "ejb_annotation"</td>
- * <tr><td>&nbsp;</td><td>WEAVER</td>        <td>&nbsp;</td><td>= "weaver"</td>
+ * <tr><td>&nbsp;</td><td>{@link #SQL}</td>           <td>&nbsp;</td><td>= {@value #SQL}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #TRANSACTION}</td>   <td>&nbsp;</td><td>= {@value #TRANSACTION}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #EVENT}</td>         <td>&nbsp;</td><td>= {@value #EVENT}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #CONNECTION}</td>         <td>&nbsp;</td><td>= {@value #CONNECTION}</td></tr> 
+ * <tr><td>&nbsp;</td><td>{@link #QUERY}</td>         <td>&nbsp;</td><td>= {@value #QUERY}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #CACHE}</td>         <td>&nbsp;</td><td>= {@value #CACHE}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #PROPAGATION}</td>   <td>&nbsp;</td><td>= {@value #PROPAGATION}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #SEQUENCING}</td>    <td>&nbsp;</td><td>= {@value #SEQUENCING}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #EJB}</td>           <td>&nbsp;</td><td>= {@value #EJB}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #DMS}</td>           <td>&nbsp;</td><td>= {@value #DMS}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #METADATA}</td><td>&nbsp;</td><td>= {@value #METADATA} </td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #METAMODEL}</td><td>&nbsp;</td><td>= {@value #METAMODEL}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #WEAVER}</td>        <td>&nbsp;</td><td>= {@value #WEAVER}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #PROPERTIES}</td>        <td>&nbsp;</td><td>= {@value #PROPERTIES}</td></tr>
+ * <tr><td>&nbsp;</td><td>{@link #SERVER}</td>        <td>&nbsp;</td><td>= {@value #SERVER}</td></tr>
  * </table>
+ * 
  * @see AbstractSessionLog
  * @see SessionLogEntry
  * @see Session
@@ -102,14 +107,44 @@ public interface SessionLog extends Cloneable {
     public static final String CACHE = "cache";
     public static final String PROPAGATION = "propagation";
     public static final String SEQUENCING = "sequencing";
+    public static final String JPA = "jpa";
     public static final String EJB = "ejb";
     public static final String DMS = "dms";
-    public static final String EJB_OR_METADATA = "ejb_or_metadata";
-    public static final String METAMODEL = "jpa_metamodel";
+    public static final String METADATA = "metadata";
+    public static final String MONITORING = "monitoring";
+    /**
+     * @deprecated use {@link #METADATA}
+     */
+    @Deprecated 
+    public static final String EJB_OR_METADATA = METADATA;
+    public static final String METAMODEL = "metamodel";
     public static final String WEAVER = "weaver";
     public static final String PROPERTIES = "properties";
     public static final String SERVER = "server";
-    public final String[] loggerCatagories = new String[] { SQL ,TRANSACTION ,EVENT ,CONNECTION ,QUERY ,CACHE ,PROPAGATION ,SEQUENCING ,EJB ,DMS ,EJB_OR_METADATA, METAMODEL ,WEAVER ,PROPERTIES ,SERVER};
+    public static final String DDL = "ddl";
+    public static final String JPARS = "jpars";
+    
+    public final String[] loggerCatagories = new String[] { 
+        SQL,
+        TRANSACTION,
+        EVENT,
+        CONNECTION,
+        QUERY,
+        CACHE,
+        PROPAGATION,
+        SEQUENCING, 
+        JPA, 
+        EJB, 
+        DMS, 
+        METADATA,
+        MONITORING,
+        METAMODEL, 
+        WEAVER,
+        PROPERTIES,
+        SERVER,
+        DDL,
+        JPARS
+    };
 
     /**
      * PUBLIC:
@@ -137,7 +172,12 @@ public interface SessionLog extends Cloneable {
      * By default the thread is logged at FINE or less level, this can be turned off.
      */
     public boolean shouldPrintThread();
-
+    
+    /**
+     * Return whether bind parameters should be displayed when logging SQL, default is true.
+     */
+    public boolean shouldDisplayData();
+    
     /**
      * By default the connection is always printed whenever available, this can be turned off.
      */
@@ -148,6 +188,11 @@ public interface SessionLog extends Cloneable {
      */
     public boolean shouldPrintSession();
 
+    /**
+     * Set whether bind parameters should be displayed when logging SQL.
+     */
+    public void setShouldDisplayData(Boolean shouldDisplayData);
+    
     /**
      * By default stack trace is logged for SEVERE all the time and at FINER level for WARNING or less.
      * This can be turned off.
@@ -194,17 +239,17 @@ public interface SessionLog extends Cloneable {
      * PUBLIC:
      * Return the log level.  Used when session is not available.
      * <p>
-     * The EclipseLink logging levels returned correspond to:
+     * The EclipseLink logging levels returned correspond to:<br>
      * <table>
-     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
-     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
-     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
-     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
-     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
-     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
-     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
-     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
-     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * <tr><td>{@link #ALL}</td>    <td>&nbsp;</td><td>= {@value #ALL}</td>
+     * <tr><td>{@link #FINEST}</td> <td>&nbsp;</td><td>= {@value #FINEST}</td>
+     * <tr><td>{@link #FINER}</td>  <td>&nbsp;</td><td>= {@value #FINER}</td>
+     * <tr><td>{@link #FINE}</td>   <td>&nbsp;</td><td>= {@value #FINE}</td>
+     * <tr><td>{@link #CONFIG}</td> <td>&nbsp;</td><td>= {@value #CONFIG}</td>
+     * <tr><td>{@link #INFO}</td>   <td>&nbsp;</td><td>= {@value #INFO}</td>
+     * <tr><td>{@link #WARNING}</td><td>&nbsp;</td><td>= {@value #WARNING}</td>
+     * <tr><td>{@link #SEVERE}</td> <td>&nbsp;</td><td>= {@value #SEVERE}</td> 
+     * <tr><td>{@link #OFF}</td>    <td>&nbsp;</td><td>= {@value #OFF}</td>
      * </table>
      */
     public int getLevel();
@@ -221,34 +266,36 @@ public interface SessionLog extends Cloneable {
      * Return the log level; category is only needed where name space
      * is available.
      * <p>
-     * The EclipseLink logging levels returned correspond to:
+     * The EclipseLink logging levels returned correspond to:<br>
      * <table>
-     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
-     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
-     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
-     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
-     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
-     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
-     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
-     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
-     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td></tr>
+     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td></tr>
+     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td></tr>
+     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td></tr>
+     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td></tr>
+     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td></tr>
+     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td></tr>
+     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td></tr>
+     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td></tr>
      * </table>
      * <p>
-     * The EclipseLink categories for logging name space are: 
+     * The EclipseLink categories for the logging name space are:<br> 
      * <table>
-     * <tr><td>SQL</td>              <td>&nbsp;</td><td>= "sql"</td>
-     * <tr><td>TRANSACTION</td>   <td>&nbsp;</td><td>= "transaction"</td>
-     * <tr><td>EVENT</td>          <td>&nbsp;</td><td>= "event"</td>
-     * <tr><td>QUERY</td>         <td>&nbsp;</td><td>= "query"</td>
-     * <tr><td>CACHE</td>         <td>&nbsp;</td><td>= "cache"</td>
-     * <tr><td>PROPAGATION</td>   <td>&nbsp;</td><td>= "propagation"</td>
-     * <tr><td>SEQUENCING</td>    <td>&nbsp;</td><td>= "sequencing"</td>
-     * <tr><td>EJB</td>           <td>&nbsp;</td><td>= "ejb"</td>
-     * <tr><td>DMS</td>           <td>&nbsp;</td><td>= "dms"</td>
-     * <tr><td>EJB_ANNOTATION</td><td>&nbsp;</td><td>= "ejb_annotation"</td>
-     * <tr><td>WEAVER</td>        <td>&nbsp;</td><td>= "weaver"</td>
-     * <tr><td>PROPERTIES</td>        <td>&nbsp;</td><td>= "properties"</td>
-     * <tr><td>SERVER</td>        <td>&nbsp;</td><td>= "server"</td>
+     * <tr><td>&nbsp;</td><td>SQL</td>           <td>&nbsp;</td><td>= "sql"</td></tr>
+     * <tr><td>&nbsp;</td><td>TRANSACTION</td>   <td>&nbsp;</td><td>= "transaction"</td></tr>
+     * <tr><td>&nbsp;</td><td>EVENT</td>         <td>&nbsp;</td><td>= "event"</td></tr>
+     * <tr><td>&nbsp;</td><td>CONNECTION</td>         <td>&nbsp;</td><td>= "connection"</td></tr> 
+     * <tr><td>&nbsp;</td><td>QUERY</td>         <td>&nbsp;</td><td>= "query"</td></tr>
+     * <tr><td>&nbsp;</td><td>CACHE</td>         <td>&nbsp;</td><td>= "cache"</td></tr>
+     * <tr><td>&nbsp;</td><td>PROPAGATION</td>   <td>&nbsp;</td><td>= "propagation"</td></tr>
+     * <tr><td>&nbsp;</td><td>SEQUENCING</td>    <td>&nbsp;</td><td>= "sequencing"</td></tr>
+     * <tr><td>&nbsp;</td><td>EJB</td>           <td>&nbsp;</td><td>= "ejb"</td></tr>
+     * <tr><td>&nbsp;</td><td>DMS</td>           <td>&nbsp;</td><td>= "dms"</td></tr>
+     * <tr><td>&nbsp;</td><td>EJB_OR_METADATA</td><td>&nbsp;</td><td>= "ejb_or_metadata"</td></tr>
+     * <tr><td>&nbsp;</td><td>METAMODEL</td><td>&nbsp;</td><td>= "metamodel"</td></tr>
+     * <tr><td>&nbsp;</td><td>WEAVER</td>        <td>&nbsp;</td><td>= "weaver"</td></tr>
+     * <tr><td>&nbsp;</td><td>PROPERTIES</td>        <td>&nbsp;</td><td>= "properties"</td></tr>
+     * <tr><td>&nbsp;</td><td>SERVER</td>        <td>&nbsp;</td><td>= "server"</td></tr>
      * </table>
      */
     public int getLevel(String category);
@@ -257,7 +304,7 @@ public interface SessionLog extends Cloneable {
      * PUBLIC:
      * Set the log level.  Used when session is not available.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -277,7 +324,7 @@ public interface SessionLog extends Cloneable {
      * Set the log level.  Category is only needed where name space
      * is available.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -290,19 +337,23 @@ public interface SessionLog extends Cloneable {
      * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
      * </table>
      * <p>
-     * The EclipseLink categories for logging name space are: 
+     * The EclipseLink categories for logging name space are:<br> 
      * <table>
-     * <tr><td>SQL</td>              <td>&nbsp;</td><td>= "sql"</td>
-     * <tr><td>TRANSACTION</td>   <td>&nbsp;</td><td>= "transaction"</td>
-     * <tr><td>EVENT</td>          <td>&nbsp;</td><td>= "event"</td>
-     * <tr><td>QUERY</td>         <td>&nbsp;</td><td>= "query"</td>
-     * <tr><td>CACHE</td>         <td>&nbsp;</td><td>= "cache"</td>
-     * <tr><td>PROPAGATION</td>   <td>&nbsp;</td><td>= "propagation"</td>
-     * <tr><td>SEQUENCING</td>    <td>&nbsp;</td><td>= "sequencing"</td>
-     * <tr><td>EJB</td>           <td>&nbsp;</td><td>= "ejb"</td>
-     * <tr><td>DMS</td>           <td>&nbsp;</td><td>= "dms"</td>
-     * <tr><td>EJB_ANNOTATION</td><td>&nbsp;</td><td>= "ejb_annotation"</td>
-     * <tr><td>WEAVER</td>        <td>&nbsp;</td><td>= "weaver"</td>
+     * <tr><td>&nbsp;</td><td>SQL</td>           <td>&nbsp;</td><td>= "sql"</td></tr>
+     * <tr><td>&nbsp;</td><td>TRANSACTION</td>   <td>&nbsp;</td><td>= "transaction"</td></tr>
+     * <tr><td>&nbsp;</td><td>EVENT</td>         <td>&nbsp;</td><td>= "event"</td></tr>
+     * <tr><td>&nbsp;</td><td>CONNECTION</td>         <td>&nbsp;</td><td>= "connection"</td></tr> 
+     * <tr><td>&nbsp;</td><td>QUERY</td>         <td>&nbsp;</td><td>= "query"</td></tr>
+     * <tr><td>&nbsp;</td><td>CACHE</td>         <td>&nbsp;</td><td>= "cache"</td></tr>
+     * <tr><td>&nbsp;</td><td>PROPAGATION</td>   <td>&nbsp;</td><td>= "propagation"</td></tr>
+     * <tr><td>&nbsp;</td><td>SEQUENCING</td>    <td>&nbsp;</td><td>= "sequencing"</td></tr>
+     * <tr><td>&nbsp;</td><td>EJB</td>           <td>&nbsp;</td><td>= "ejb"</td></tr>
+     * <tr><td>&nbsp;</td><td>DMS</td>           <td>&nbsp;</td><td>= "dms"</td></tr>
+     * <tr><td>&nbsp;</td><td>EJB_OR_METADATA</td><td>&nbsp;</td><td>= "ejb_or_metadata"</td></tr>
+     * <tr><td>&nbsp;</td><td>METAMODEL</td><td>&nbsp;</td><td>= "metamodel"</td></tr>
+     * <tr><td>&nbsp;</td><td>WEAVER</td>        <td>&nbsp;</td><td>= "weaver"</td></tr>
+     * <tr><td>&nbsp;</td><td>PROPERTIES</td>        <td>&nbsp;</td><td>= "properties"</td></tr>
+     * <tr><td>&nbsp;</td><td>SERVER</td>        <td>&nbsp;</td><td>= "server"</td></tr>
      * </table>
      */
     public void setLevel(int level, String category);
@@ -332,7 +383,7 @@ public interface SessionLog extends Cloneable {
      * Check if a message of the given level would actually be logged.
      * Category is only needed where name space is available.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -345,19 +396,23 @@ public interface SessionLog extends Cloneable {
      * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
      * </table>
      * <p>
-     * The EclipseLink categories for logging name space are: 
+     * The EclipseLink categories for logging name space are:<br> 
      * <table>
-     * <tr><td>SQL</td>              <td>&nbsp;</td><td>= "sql"</td>
-     * <tr><td>TRANSACTION</td>   <td>&nbsp;</td><td>= "transaction"</td>
-     * <tr><td>EVENT</td>          <td>&nbsp;</td><td>= "event"</td>
-     * <tr><td>QUERY</td>         <td>&nbsp;</td><td>= "query"</td>
-     * <tr><td>CACHE</td>         <td>&nbsp;</td><td>= "cache"</td>
-     * <tr><td>PROPAGATION</td>   <td>&nbsp;</td><td>= "propagation"</td>
-     * <tr><td>SEQUENCING</td>    <td>&nbsp;</td><td>= "sequencing"</td>
-     * <tr><td>EJB</td>           <td>&nbsp;</td><td>= "ejb"</td>
-     * <tr><td>DMS</td>           <td>&nbsp;</td><td>= "dms"</td>
-     * <tr><td>EJB_ANNOTATION</td><td>&nbsp;</td><td>= "ejb_annotation"</td>
-     * <tr><td>WEAVER</td>        <td>&nbsp;</td><td>= "weaver"</td>
+     * <tr><td>&nbsp;</td><td>SQL</td>           <td>&nbsp;</td><td>= "sql"</td></tr>
+     * <tr><td>&nbsp;</td><td>TRANSACTION</td>   <td>&nbsp;</td><td>= "transaction"</td></tr>
+     * <tr><td>&nbsp;</td><td>EVENT</td>         <td>&nbsp;</td><td>= "event"</td></tr>
+     * <tr><td>&nbsp;</td><td>CONNECTION</td>         <td>&nbsp;</td><td>= "connection"</td></tr> 
+     * <tr><td>&nbsp;</td><td>QUERY</td>         <td>&nbsp;</td><td>= "query"</td></tr>
+     * <tr><td>&nbsp;</td><td>CACHE</td>         <td>&nbsp;</td><td>= "cache"</td></tr>
+     * <tr><td>&nbsp;</td><td>PROPAGATION</td>   <td>&nbsp;</td><td>= "propagation"</td></tr>
+     * <tr><td>&nbsp;</td><td>SEQUENCING</td>    <td>&nbsp;</td><td>= "sequencing"</td></tr>
+     * <tr><td>&nbsp;</td><td>EJB</td>           <td>&nbsp;</td><td>= "ejb"</td></tr>
+     * <tr><td>&nbsp;</td><td>DMS</td>           <td>&nbsp;</td><td>= "dms"</td></tr>
+     * <tr><td>&nbsp;</td><td>EJB_OR_METADATA</td><td>&nbsp;</td><td>= "ejb_or_metadata"</td></tr>
+     * <tr><td>&nbsp;</td><td>METAMODEL</td><td>&nbsp;</td><td>= "metamodel"</td></tr>
+     * <tr><td>&nbsp;</td><td>WEAVER</td>        <td>&nbsp;</td><td>= "weaver"</td></tr>
+     * <tr><td>&nbsp;</td><td>PROPERTIES</td>        <td>&nbsp;</td><td>= "properties"</td></tr>
+     * <tr><td>&nbsp;</td><td>SERVER</td>        <td>&nbsp;</td><td>= "server"</td></tr>
      * </table>
      */
     public boolean shouldLog(int level, String category);
@@ -367,7 +422,7 @@ public interface SessionLog extends Cloneable {
      * Log a message that does not need to be translated.  This method is intended for 
      * external use when logging messages are wanted within the EclipseLink output.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -386,7 +441,7 @@ public interface SessionLog extends Cloneable {
      * PUBLIC:
      * Log a message with one parameter that needs to be translated.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -400,12 +455,31 @@ public interface SessionLog extends Cloneable {
      * </table>
      */
     public void log(int level, String message, Object param);
+    
+    /**
+     * PUBLIC:
+     * Log a message with one parameter that needs to be translated.
+     * <p>
+     * The EclipseLink logging levels available are:<br>
+     * <table>
+     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
+     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
+     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
+     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
+     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
+     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
+     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
+     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
+     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * </table>
+     */
+    public void log(int level, String category, String message, Object param);
 
     /**
      * PUBLIC:
      * Log a message with two parameters that needs to be translated.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -419,12 +493,31 @@ public interface SessionLog extends Cloneable {
      * </table>
      */
     public void log(int level, String message, Object param1, Object param2);
+    
+    /**
+     * PUBLIC:
+     * Log a message with two parameters that needs to be translated.
+     * <p>
+     * The EclipseLink logging levels available are:<br>
+     * <table>
+     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
+     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
+     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
+     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
+     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
+     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
+     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
+     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
+     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * </table>
+     */
+    public void log(int level, String category, String message, Object param1, Object param2);
 
     /**
      * PUBLIC:
      * Log a message with three parameters that needs to be translated.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -438,12 +531,31 @@ public interface SessionLog extends Cloneable {
      * </table>
      */
     public void log(int level, String message, Object param1, Object param2, Object param3);
+    
+    /**
+     * PUBLIC:
+     * Log a message with three parameters that needs to be translated.
+     * <p>
+     * The EclipseLink logging levels available are:<br>
+     * <table>
+     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
+     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
+     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
+     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
+     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
+     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
+     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
+     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
+     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * </table>
+     */
+    public void log(int level, String category, String message, Object param1, Object param2, Object param3);
 
     /**
      * PUBLIC:
      * Log a message with four parameters that needs to be translated.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -458,12 +570,31 @@ public interface SessionLog extends Cloneable {
      */
     public void log(int level, String message, Object param1, Object param2, Object param3, Object param4);
     
+    /**
+     * PUBLIC:
+     * Log a message with four parameters that needs to be translated.
+     * <p>
+     * The EclipseLink logging levels available are:<br>
+     * <table>
+     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
+     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
+     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
+     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
+     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
+     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
+     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
+     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
+     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * </table>
+     */
+    public void log(int level, String category, String message, Object param1, Object param2, Object param3, Object param4);
+    
     /**
      * PUBLIC:
      * This method is called when the log request is from somewhere session is not available.
      * The message needs to be translated.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -477,13 +608,33 @@ public interface SessionLog extends Cloneable {
      * </table>
      */
     public void log(int level, String message, Object[] arguments);
+    
+    /**
+     * PUBLIC:
+     * This method is called when the log request is from somewhere session is not available.
+     * The message needs to be translated.
+     * <p>
+     * The EclipseLink logging levels available are:<br>
+     * <table>
+     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
+     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
+     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
+     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
+     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
+     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
+     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
+     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
+     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * </table>
+     */
+    public void log(int level, String category, String message, Object[] arguments);
 
     /**
      * PUBLIC:
      * This method is called when the log request is from somewhere session is not available.
      * shouldTranslate flag determines if the message needs to be translated.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
      * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
      * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
@@ -498,6 +649,26 @@ public interface SessionLog extends Cloneable {
      */
     public void log(int level, String message, Object[] arguments, boolean shouldTranslate);
 
+    /**
+     * PUBLIC:
+     * This method is called when the log request is from somewhere session is not available.
+     * shouldTranslate flag determines if the message needs to be translated.
+     * <p>
+     * The EclipseLink logging levels available are:<br>
+     * <table>
+     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
+     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
+     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
+     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
+     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
+     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
+     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
+     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
+     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * </table>
+     */
+    public void log(int level, String category, String message, Object[] arguments, boolean shouldTranslate);
+    
     /**
      * PUBLIC:
      * This method is called when a throwable at finer level needs to be logged.
@@ -555,22 +726,41 @@ public interface SessionLog extends Cloneable {
 
     /**
      * PUBLIC:
-     * Log a throwable with level.
+     * Log a {@link Throwable} with level.
      * <p>
-     * The EclipseLink logging levels available are:
+     * The EclipseLink logging levels available are:<br>
      * <table>
-     * <tr><td>ALL</td>    <td>&nbsp;</td><td>= 0</td>
-     * <tr><td>FINEST</td> <td>&nbsp;</td><td>= 1</td>
-     * <tr><td>FINER</td>  <td>&nbsp;</td><td>= 2</td>
-     * <tr><td>FINE</td>   <td>&nbsp;</td><td>= 3</td>
-     * <tr><td>CONFIG</td> <td>&nbsp;</td><td>= 4</td>
-     * <tr><td>INFO</td>   <td>&nbsp;</td><td>= 5</td>
-     * <tr><td>WARNING</td><td>&nbsp;</td><td>= 6</td>
-     * <tr><td>SEVERE</td> <td>&nbsp;</td><td>= 7</td>
-     * <tr><td>OFF</td>    <td>&nbsp;</td><td>= 8</td>
+     * <tr><td>{@link #ALL}</td>    <td>&nbsp;</td><td>= {@value #ALL}</td>
+     * <tr><td>{@link #FINEST}</td> <td>&nbsp;</td><td>= {@value #FINEST}</td>
+     * <tr><td>{@link #FINER}</td>  <td>&nbsp;</td><td>= {@value #FINER}</td>
+     * <tr><td>{@link #FINE}</td>   <td>&nbsp;</td><td>= {@value #FINE}</td>
+     * <tr><td>{@link #CONFIG}</td> <td>&nbsp;</td><td>= {@value #CONFIG}</td>
+     * <tr><td>{@link #INFO}</td>   <td>&nbsp;</td><td>= {@value #INFO}</td>
+     * <tr><td>{@link #WARNING}</td><td>&nbsp;</td><td>= {@value #WARNING}</td>
+     * <tr><td>{@link #SEVERE}</td> <td>&nbsp;</td><td>= {@value #SEVERE}</td> 
+     * <tr><td>{@link #OFF}</td>    <td>&nbsp;</td><td>= {@value #OFF}</td>
      * </table>
      */
     public void logThrowable(int level, Throwable throwable);
+    
+    /**
+     * PUBLIC:
+     * Log a throwable with level.
+     * <p>
+     * The EclipseLink logging levels available are:<br>
+     * <table>
+     * <tr><td>{@link #ALL}</td>    <td>&nbsp;</td><td>= {@value #ALL}</td>
+     * <tr><td>{@link #FINEST}</td> <td>&nbsp;</td><td>= {@value #FINEST}</td>
+     * <tr><td>{@link #FINER}</td>  <td>&nbsp;</td><td>= {@value #FINER}</td>
+     * <tr><td>{@link #FINE}</td>   <td>&nbsp;</td><td>= {@value #FINE}</td>
+     * <tr><td>{@link #CONFIG}</td> <td>&nbsp;</td><td>= {@value #CONFIG}</td>
+     * <tr><td>{@link #INFO}</td>   <td>&nbsp;</td><td>= {@value #INFO}</td>
+     * <tr><td>{@link #WARNING}</td><td>&nbsp;</td><td>= {@value #WARNING}</td>
+     * <tr><td>{@link #SEVERE}</td> <td>&nbsp;</td><td>= {@value #SEVERE}</td> 
+     * <tr><td>{@link #OFF}</td>    <td>&nbsp;</td><td>= {@value #OFF}</td>
+     * </table>
+     */
+    public void logThrowable(int level, String category, Throwable throwable);
 
     /**
      * PUBLIC:
