@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -13,6 +13,8 @@
 package org.eclipse.persistence.mappings;
 
 import java.io.*;
+
+import org.eclipse.persistence.core.mappings.CoreAttributeAccessor;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.internal.helper.ClassConstants;
 
@@ -25,7 +27,7 @@ import org.eclipse.persistence.internal.helper.ClassConstants;
  *    @author James
  *    @since OracleAS TopLink 10<i>g</i> (10.0.3)
  */
-public abstract class AttributeAccessor implements Cloneable, Serializable {
+public abstract class AttributeAccessor implements CoreAttributeAccessor, Cloneable, Serializable {
 
     /** Stores the name of the attribute */
     protected String attributeName;
@@ -87,11 +89,23 @@ public abstract class AttributeAccessor implements Cloneable, Serializable {
     public boolean isInstanceVariableAttributeAccessor() {
         return false;
     }
+
+    public boolean isMapValueAttributeAccessor(){
+        return false;
+    }
     
     public boolean isMethodAttributeAccessor() {
         return false;
     }
     
+    public boolean isValuesAccessor() {
+    	return false;
+    }
+    
+    public boolean isVirtualAttributeAccessor(){
+        return false;
+    }
+    
     /**
      * INTERNAL:
      * @return
@@ -126,10 +140,12 @@ public abstract class AttributeAccessor implements Cloneable, Serializable {
     /**
      * Return the attribute value from the object.
      */
+    @Override
     public abstract Object getAttributeValueFromObject(Object object) throws DescriptorException;
 
     /**
      * Set the attribute value into the object.
      */
+    @Override
     public abstract void setAttributeValueInObject(Object object, Object value) throws DescriptorException;
 }
