@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,16 +9,20 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     07/19/2011-2.2.1 Guy Pelletier 
+ *       - 338812: ManyToMany mapping in aggregate object violate integrity constraint on deletion
  ******************************************************************************/  
 package org.eclipse.persistence.mappings;
 
 import java.util.*;
 
+import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.expressions.*;
 import org.eclipse.persistence.history.*;
 import org.eclipse.persistence.internal.expressions.*;
 import org.eclipse.persistence.internal.helper.*;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.queries.*;
 import org.eclipse.persistence.internal.sessions.*;
 import org.eclipse.persistence.mappings.foundation.MapComponentMapping;
@@ -42,7 +46,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
     protected static final String ObjectAdded = "objectAdded";
 
     /** Mechanism holds relationTable and all fields and queries associated with it. */
-    protected transient RelationTableMechanism mechanism;
+    protected RelationTableMechanism mechanism;
     protected HistoryPolicy historyPolicy;
 
     /**
@@ -67,6 +71,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
     /**
      * INTERNAL:
      */
+    @Override
     public boolean isOwned(){
         return !isReadOnly;
     }
@@ -74,6 +79,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
     /**
      * INTERNAL:
      */
+    @Override
     public boolean isRelationalMapping() {
         return true;
     }
@@ -114,10 +120,21 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         this.mechanism.addTargetRelationKeyFieldName(targetRelationKeyFieldName, targetPrimaryKeyFieldName);
     }
 
+    /**
+     * INTERNAL: 
+     * This method is used to store the FK fields that can be cached that correspond to noncacheable mappings
+     * the FK field values will be used to re-issue the query when cloning the shared cache entity
+     */
+    @Override
+    public void collectQueryParameters(Set<DatabaseField> cacheFields){
+        this.mechanism.collectQueryParameters(cacheFields);
+    }
+
     /**
      * INTERNAL:
      * The mapping clones itself to create deep copy.
      */
+    @Override
     public Object clone() {
         ManyToManyMapping clone = (ManyToManyMapping)super.clone();        
         clone.mechanism = (RelationTableMechanism)this.mechanism.clone();
@@ -127,23 +144,29 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
 
     /**
      * INTERNAL:
-     * This method is called to update collection tables prior to commit.
+     * Delete join tables before the start of the deletion process to avoid constraint errors.
      */
     @Override
-    public void earlyPreDelete(DeleteObjectQuery query){
+    public void earlyPreDelete(DeleteObjectQuery query, Object object) {
         AbstractSession querySession = query.getSession();
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), querySession);
-        querySession.executeQuery(getDeleteAllQuery(), query.getTranslationRow());
+        if (!this.isCascadeOnDeleteSetOnDatabase) {
+            prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), querySession);
+            querySession.executeQuery(this.deleteAllQuery, query.getTranslationRow());
+        }
 
-        if ((getHistoryPolicy() != null) && getHistoryPolicy().shouldHandleWrites()) {
-            getHistoryPolicy().mappingLogicalDelete(getDeleteAllQuery(), query.getTranslationRow(), querySession);
+        if ((this.historyPolicy != null) && this.historyPolicy.shouldHandleWrites()) {
+            if (this.isCascadeOnDeleteSetOnDatabase) {
+                prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), querySession);
+            }
+            this.historyPolicy.mappingLogicalDelete(this.deleteAllQuery, query.getTranslationRow(), querySession);
         }
     }
+    
     /**
      * INTERNAL
      * Called when a DatabaseMapping is used to map the key in a collection.  Returns the key.
      */
-    public Object createMapComponentFromRow(AbstractRecord dbRow, ObjectBuildingQuery query, AbstractSession session){
+    public Object createMapComponentFromRow(AbstractRecord dbRow, ObjectBuildingQuery query, CacheKey parentCacheKey, AbstractSession session, boolean isTargetProtected){
         return session.executeQuery(getSelectionQuery(), dbRow);
     }
     
@@ -151,6 +174,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Adds locking clause to the target query to extend pessimistic lock scope.
      */
+    @Override
     protected void extendPessimisticLockScopeInTargetQuery(ObjectLevelReadQuery targetQuery, ObjectBuildingQuery sourceQuery) {
         this.mechanism.setRelationTableLockingClause(targetQuery, sourceQuery);
     }
@@ -165,6 +189,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * This method must be implemented in subclasses that allow
      * setting shouldExtendPessimisticLockScopeInSourceQuery to true.
      */
+    @Override
     public void extendPessimisticLockScopeInSourceQuery(ObjectLevelReadQuery sourceQuery) {
         Expression exp = sourceQuery.getSelectionCriteria();
         exp = this.mechanism.joinRelationTableField(exp, sourceQuery.getExpressionBuilder());
@@ -217,7 +242,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
             } else if (builder.getAsOfClause() == null) {
                 builder.asOf(AsOfClause.NO_CLAUSE);
             }
-            twisted = twisted.and(this.historyPolicy.additionalHistoryExpression(builder));
+            twisted = twisted.and(this.historyPolicy.additionalHistoryExpression(builder, builder));
             mappingBatchQuery.setSelectionCriteria(twisted);
         }
     }
@@ -242,6 +267,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * Should be overridden by subclass that allows setting
      * extendPessimisticLockScope to DEDICATED_QUERY. 
      */
+    @Override
     protected ReadQuery getExtendPessimisticLockScopeDedicatedQuery(AbstractSession session, short lockMode) {
         if(this.mechanism != null) {
             return this.mechanism.getLockRelationTableQueryClone(session, lockMode);            
@@ -250,6 +276,15 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         }
     }
     
+    /**
+     * INTERNAL:
+     * Return source key fields for translation by an AggregateObjectMapping
+     */
+    @Override
+    public Collection getFieldsForTranslationInAggregate() {
+        return getRelationTableMechanism().getSourceKeyFields();
+    }
+    
     protected DataModifyQuery getInsertQuery() {
         return this.mechanism.getInsertQuery();
     }
@@ -260,10 +295,10 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * is used to read reference objects across the tables from the database.
      */
     @Override
-    public Expression getJoinCriteria(QueryKeyExpression exp) {
+    public Expression getJoinCriteria(ObjectExpression context, Expression base) {
         if (getHistoryPolicy() != null) {
-            Expression result = super.getJoinCriteria(exp);
-            Expression historyCriteria = getHistoryPolicy().additionalHistoryExpression(exp);
+            Expression result = super.getJoinCriteria(context, base);
+            Expression historyCriteria = getHistoryPolicy().additionalHistoryExpression(context, base);
             if (result != null) {
                 return result.and(historyCriteria);
             } else if (historyCriteria != null) {
@@ -272,12 +307,13 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
                 return null;
             }
         } else {
-            return super.getJoinCriteria(exp);
+            return super.getJoinCriteria(context, base);
         }
     }
 
     /**
      * PUBLIC:
+     * Allows history tracking on the m-m join table.
      */
     public HistoryPolicy getHistoryPolicy() {
         return historyPolicy;
@@ -323,23 +359,6 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         return this.mechanism.getRelationTableQualifiedName();
     }
 
-    /**
-     * INTERNAL:
-     * Returns the selection criteria stored in the mapping selection query. This criteria
-     * is used to read reference objects from the database.
-     */
-    public Expression getSelectionCriteria() {
-        return getSelectionQuery().getSelectionCriteria();
-    }
-
-    /**
-     * INTERNAL:
-     * Returns the read query assoicated with the mapping.
-     */
-    public ReadQuery getSelectionQuery() {
-        return selectionQuery;
-    }
-
     /**
      * PUBLIC:
      * Return the source key field names associated with the mapping.
@@ -420,17 +439,27 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * The join table is a dependency if not read-only.
      */
+    @Override
     public boolean hasDependency() {
-        return isPrivateOwned() || (!isReadOnly());
+        return this.isPrivateOwned || (!this.isReadOnly);
     }
 
     /**
      * INTERNAL:
      * Initialize mappings
      */
+    @Override
     public void initialize(AbstractSession session) throws DescriptorException {
+        if (session.hasBroker()) {
+            if (getReferenceClass() == null) {
+                throw DescriptorException.referenceClassNotSpecified(this);
+            }
+            // substitute session that owns the mapping for the session that owns reference descriptor.
+            session = session.getBroker().getSessionForClass(getReferenceClass());
+        }
+        
         super.initialize(session);
-        getDescriptor().getPreDeleteMappings().add(this);
+        getDescriptor().addPreDeleteMapping(this);
 
         if(this.mechanism != null) {
             this.mechanism.initialize(session, this);
@@ -467,6 +496,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * Verifies listOrderField's table: it must be relation table.
      * Precondition: listOrderField != null.
      */
+    @Override
     protected void buildListOrderField() {
         if(this.listOrderField.hasTableName()) {
             if(!getRelationTable().equals(this.listOrderField.getTable())) {
@@ -482,6 +512,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Indicates whether getListOrderFieldExpression method should create field expression on table expression.  
      */
+    @Override
     public boolean shouldUseListOrderFieldTableExpression() {
         return true;
     }
@@ -542,7 +573,10 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         if (!getDeleteAllQuery().hasSessionName()) {
             getDeleteAllQuery().setSessionName(session.getName());
         }
-
+        getDeleteAllQuery().setName(getAttributeName());
+        if (getDeleteAllQuery().getPartitioningPolicy() == null) {
+            getDeleteAllQuery().setPartitioningPolicy(getPartitioningPolicy());
+        }
         if (hasCustomDeleteAllQuery()) {
             return;
         }
@@ -592,7 +626,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
     protected void insertAddedObjectEntry(ObjectLevelModifyQuery query, Object objectAdded, Map extraData) throws DatabaseException, OptimisticLockException {
         //cr 3819 added the line below to fix the translationtable to ensure that it
         // contains the required values
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getSession());
+        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), query.getSession());
         AbstractRecord databaseRow = this.mechanism.buildRelationTableSourceAndTargetRow(query.getTranslationRow(), containerPolicy.unwrapIteratorResult(objectAdded), query.getSession(), this);
         ContainerPolicy.copyMapDataToRow(getContainerPolicy().getKeyMappingDataForWriteQuery(objectAdded, query.getSession()), databaseRow);
         
@@ -600,7 +634,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
             databaseRow.put(listOrderField, extraData.get(listOrderField));
         }
         
-        query.getSession().executeQuery(this.mechanism.getInsertQuery(), databaseRow);
+        query.getExecutionSession().executeQuery(this.mechanism.getInsertQuery(), databaseRow);
         if ((getHistoryPolicy() != null) && getHistoryPolicy().shouldHandleWrites()) {
             getHistoryPolicy().mappingLogicalInsert(this.mechanism.getInsertQuery(), databaseRow, query.getSession());
         }
@@ -626,7 +660,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
             return;
         }
 
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getSession());
+        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), query.getSession());
         AbstractRecord databaseRow = this.mechanism.buildRelationTableSourceRow(query.getTranslationRow());
 
         int orderIndex = 0;
@@ -634,7 +668,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         for (Object iter = cp.iteratorFor(objects); cp.hasNext(iter);) {
             Object wrappedObject = cp.nextEntry(iter, query.getSession());
             Object object = cp.unwrapIteratorResult(wrappedObject);
-            databaseRow = this.mechanism.addRelationTableTargetRow(object, query.getSession(), databaseRow, this);
+            databaseRow = this.mechanism.addRelationTableTargetRow(object, query.getExecutionSession(), databaseRow, this);
 
             ContainerPolicy.copyMapDataToRow(cp.getKeyMappingDataForWriteQuery(wrappedObject, query.getSession()), databaseRow);
             
@@ -642,7 +676,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
                 databaseRow.put(listOrderField, orderIndex++);
             }
 
-            query.getSession().executeQuery(this.mechanism.getInsertQuery(), databaseRow);
+            query.getExecutionSession().executeQuery(this.mechanism.getInsertQuery(), databaseRow);
             if ((getHistoryPolicy() != null) && getHistoryPolicy().shouldHandleWrites()) {
                 getHistoryPolicy().mappingLogicalInsert(this.mechanism.getInsertQuery(), databaseRow, query.getSession());
             }
@@ -702,8 +736,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
 
     /**
      * INTERNAL:
-     * Return whether this mapping was originally defined as a OneToMany
-     * @return
+     * Return whether this mapping was originally defined as a OneToMany.
      */
     public boolean isDefinedAsOneToManyMapping() {
         return isDefinedAsOneToManyMapping;
@@ -713,28 +746,31 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Return if this mapping support joining.
      */
+    @Override
     public boolean isJoiningSupported() {
         return true;
     }
 
-    /**
-     * INTERNAL:
-     */
+    @Override
     public boolean isManyToManyMapping() {
         return true;
     }
-
+    
     /**
-     * For Many To Many mappings referenced objects are deleted one by one.
+     * INTERNAL:
+     * Ensure the container policy is post initialized
      */
-    protected boolean mustDeleteReferenceObjectsOneByOne() {
-        return true;
+    @Override
+    public void postInitialize(AbstractSession session) {
+        super.postInitialize(session);
+        this.mustDeleteReferenceObjectsOneByOne = true;
     }
 
     /**
      * INTERNAL:
      * An object was added to the collection during an update, insert it if private.
      */
+    @Override
     protected void objectAddedDuringUpdate(ObjectLevelModifyQuery query, Object objectAdded, ObjectChangeSet changeSet, Map extraData) throws DatabaseException, OptimisticLockException {
         // First insert/update object.
         super.objectAddedDuringUpdate(query, objectAdded, changeSet, extraData);
@@ -757,6 +793,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * An object was removed to the collection during an update, delete it if private.
      */
+    @Override
     protected void objectRemovedDuringUpdate(ObjectLevelModifyQuery query, Object objectDeleted, Map extraData) throws DatabaseException, OptimisticLockException {
         Object unwrappedObjectDeleted = getContainerPolicy().unwrapIteratorResult(objectDeleted);
         AbstractRecord databaseRow = this.mechanism.buildRelationTableSourceAndTargetRow(query.getTranslationRow(), unwrappedObjectDeleted, query.getSession(), this);
@@ -780,8 +817,9 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         super.objectRemovedDuringUpdate(query, objectDeleted, extraData);
     }
 
+    @Override
     protected void objectOrderChangedDuringUpdate(WriteObjectQuery query, Object orderChangedObject, int orderIndex) {
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getSession());
+        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), query.getSession());
         AbstractRecord databaseRow = this.mechanism.buildRelationTableSourceAndTargetRow(query.getTranslationRow(), orderChangedObject, query.getSession(), this);
         databaseRow.put(listOrderField, orderIndex);
   
@@ -793,6 +831,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * Perform the commit event.
      * This is used in the uow to delay data modifications.
      */
+    @Override
     public void performDataModificationEvent(Object[] event, AbstractSession session) throws DatabaseException, DescriptorException {
         // Hey I might actually want to use an inner class here... ok array for now.
         if (event[0] == PostInsert) {
@@ -818,6 +857,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * <p>- execute the statement.
      * <p>- Repeat above three statements until all the target objects are done.
      */
+    @Override
     public void postInsert(WriteObjectQuery query) throws DatabaseException {
         insertTargetObjects(query);
         // Batch data modification in the uow
@@ -838,8 +878,9 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * Delete entries removed, insert entries added.
      * If private also insert/delete/update target objects.
      */
+    @Override
     public void postUpdate(WriteObjectQuery query) throws DatabaseException {
-        if (isReadOnly()) {
+        if (this.isReadOnly) {
             return;
         }
 
@@ -847,15 +888,13 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         if (!isAttributeValueInstantiatedOrChanged(query.getObject())) {
             return;
         }
-
-        Object objectsInMemoryModel = getRealCollectionAttributeValueFromObject(query.getObject(), query.getSession());
-
-        // This accesses the backup in uow otherwise goes to database (may be better of to delete all in non uow case).
-        Object currentObjectsInDB = readPrivateOwnedForObject(query);
-        if (currentObjectsInDB == null) {
-            currentObjectsInDB = getContainerPolicy().containerInstance(1);
+        if (query.getObjectChangeSet() != null) {
+            // UnitOfWork
+            writeChanges(query.getObjectChangeSet(), query);
+        } else {
+            // OLD COMMIT
+            compareObjectsAndWrite(query);
         }
-        compareObjectsAndWrite(currentObjectsInDB, objectsInMemoryModel, query);
     }
 
     /**
@@ -864,14 +903,11 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      */
     @Override
     public void preDelete(DeleteObjectQuery query) throws DatabaseException {
-        AbstractSession querySession = query.getSession();
-        if (querySession != null && querySession.isUnitOfWork()){
-            return;
-        }
+        AbstractSession session = query.getSession();
         Object objectsIterator = null;
         ContainerPolicy containerPolicy = getContainerPolicy();
         
-        if (isReadOnly()) {
+        if (this.isReadOnly) {
             return;
         }
         Object objects = null;
@@ -879,12 +915,17 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
         boolean cascade = shouldObjectModifyCascadeToParts(query);
         if (containerPolicy.propagatesEventsToCollection() || cascade) {
             // if processed during UnitOfWork commit process the private owned delete will occur during change calculation
-            objects = getRealCollectionAttributeValueFromObject(query.getObject(), querySession);
+            objects = getRealCollectionAttributeValueFromObject(query.getObject(), session);
             //this must be done up here because the select must be done before the entry in the relation table is deleted.
+            // TODO: Hmm given the below code, the rows are already deleted, so this code is broken.
+            // Assuming it was a cascade remove, it will have been instantiated, so may be ok?
             objectsIterator = containerPolicy.iteratorFor(objects);
         }
-        
-            earlyPreDelete(query);
+
+        // This has already been done in a unit of work.
+        if (!session.isUnitOfWork()) {
+            earlyPreDelete(query, query.getObject());
+        }
 
         // If privately owned delete the objects, this does not handle removed objects (i.e. verify delete, not req in uow).
         // Does not try to optimize delete all like 1-m, (rarely used and hard to do).
@@ -892,14 +933,17 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
             if (objects != null) {
                 //objectsIterator will not be null because cascade check will still return true.
                 while (containerPolicy.hasNext(objectsIterator)) {
-                    Object wrappedObject = containerPolicy.nextEntry(objectsIterator, query.getSession());
+                    Object wrappedObject = containerPolicy.nextEntry(objectsIterator, session);
                     Object object = containerPolicy.unwrapIteratorResult(wrappedObject);
                     if (cascade){
-                        DeleteObjectQuery deleteQuery = new DeleteObjectQuery();
-                        deleteQuery.setIsExecutionClone(true);
-                        deleteQuery.setObject(object);
-                        deleteQuery.setCascadePolicy(query.getCascadePolicy());
-                        query.getSession().executeQuery(deleteQuery);
+                        // PERF: Avoid query execution if already deleted.
+                        if (!session.getCommitManager().isCommitCompletedInPostOrIgnore(object)) {
+                            DeleteObjectQuery deleteQuery = new DeleteObjectQuery();
+                            deleteQuery.setIsExecutionClone(true);
+                            deleteQuery.setObject(object);
+                            deleteQuery.setCascadePolicy(query.getCascadePolicy());
+                            session.executeQuery(deleteQuery);
+                        }
                     }
                     containerPolicy.propogatePreDelete(query, wrappedObject);
                 }
@@ -911,13 +955,14 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * The translation row may require additional fields than the primary key if the mapping in not on the primary key.
      */
-    protected void prepareTranslationRow(AbstractRecord translationRow, Object object, AbstractSession session) {
+    @Override
+    protected void prepareTranslationRow(AbstractRecord translationRow, Object object, ClassDescriptor descriptor, AbstractSession session) {
         // Make sure that each source key field is in the translation row.
         for (Enumeration sourceFieldsEnum = getSourceKeyFields().elements();
                  sourceFieldsEnum.hasMoreElements();) {
             DatabaseField sourceKey = (DatabaseField)sourceFieldsEnum.nextElement();
             if (!translationRow.containsKey(sourceKey)) {
-                Object value = getDescriptor().getObjectBuilder().extractValueFromObjectForField(object, sourceKey, session);
+                Object value = descriptor.getObjectBuilder().extractValueFromObjectForField(object, sourceKey, session);
                 translationRow.put(sourceKey, value);
             }
         }
@@ -1035,6 +1080,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
     
     /**
      * PUBLIC:
+     * Enable history tracking on the m-m join table.
      */
     public void setHistoryPolicy(HistoryPolicy policy) {
         this.historyPolicy = policy;
@@ -1058,6 +1104,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * This can be used by the session broker to override the default session
      * to be used for the target class.
      */
+    @Override
     public void setSessionName(String name) {
         super.setSessionName(name);
         this.mechanism.setSessionName(name);
@@ -1155,6 +1202,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Append the temporal selection to the query selection criteria.
      */
+    @Override
     protected ReadQuery prepareHistoricalQuery(ReadQuery targetQuery, ObjectBuildingQuery sourceQuery, AbstractSession executionSession) {
         if (getHistoryPolicy() != null) {
             if (targetQuery == getSelectionQuery()) {
@@ -1169,7 +1217,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
             } else if (((ObjectLevelReadQuery)targetQuery).getAsOfClause() == null) {
                 ((ObjectLevelReadQuery)targetQuery).setAsOfClause(AsOfClause.NO_CLAUSE);
             }
-            Expression temporalExpression = (this).getHistoryPolicy().additionalHistoryExpression(targetQuery.getSelectionCriteria().getBuilder());
+            Expression temporalExpression = (this).getHistoryPolicy().additionalHistoryExpression(targetQuery.getSelectionCriteria().getBuilder(), targetQuery.getSelectionCriteria().getBuilder());
             targetQuery.setSelectionCriteria(targetQuery.getSelectionCriteria().and(temporalExpression));
         }
         return targetQuery;
