@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,8 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     07/19/2011-2.2.1 Guy Pelletier 
+ *       - 338812: ManyToMany mapping in aggregate object violate integrity constraint on deletion
  ******************************************************************************/  
 package org.eclipse.persistence.mappings;
 
@@ -44,10 +46,10 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
     protected static final String ObjectAdded = "objectAdded";
     
     /** The target foreign key fields that reference the sourceKeyFields. */
-    protected transient Vector<DatabaseField> targetForeignKeyFields;
+    protected Vector<DatabaseField> targetForeignKeyFields;
 
     /** The (typically primary) source key fields that are referenced by the targetForeignKeyFields. */
-    protected transient Vector<DatabaseField> sourceKeyFields;
+    protected Vector<DatabaseField> sourceKeyFields;
 
     /** This maps the target foreign key fields to the corresponding (primary) source key fields. */
     protected transient Map<DatabaseField, DatabaseField> targetForeignKeysToSourceKeys;
@@ -77,8 +79,8 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      *   where 1 is id of the source, and 2 is the id of the target to be added.  
      *  Used only in case data modification events required.
      **/
-    protected transient DataModifyQuery addTargetQuery;
-    protected transient boolean hasCustomAddTargetQuery;
+    protected DataModifyQuery addTargetQuery;
+    protected boolean hasCustomAddTargetQuery;
     
     /** 
      * Query used to update a single target row changing its foreign key value from the one pointing to the source to null. 
@@ -90,8 +92,8 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      *   where 1 is id of the source, and 2 is the id of the target to be removed.  
      *  Used only in case data modification events required.
      **/
-    protected transient DataModifyQuery removeTargetQuery;
-    protected transient boolean hasCustomRemoveTargetQuery;
+    protected DataModifyQuery removeTargetQuery;
+    protected boolean hasCustomRemoveTargetQuery;
 
     /** 
      * Query used to update all target rows changing target foreign key value from the one pointing to the source to null. 
@@ -103,8 +105,8 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      *   where 1 is id of the source to be deleted.  
      *  Used only in case data modification events required.
      **/
-    protected transient DataModifyQuery removeAllTargetsQuery;
-    protected transient boolean hasCustomRemoveAllTargetsQuery;
+    protected DataModifyQuery removeAllTargetsQuery;
+    protected boolean hasCustomRemoveAllTargetsQuery;
     
     /**
      * PUBLIC:
@@ -129,6 +131,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
     /**
      * INTERNAL:
      */
+    @Override
     public boolean isRelationalMapping() {
         return true;
     }
@@ -137,6 +140,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * Add the associated fields to the appropriate collections.
      */
+    @Override
     public void addTargetForeignKeyField(DatabaseField targetForeignKeyField, DatabaseField sourceKeyField) {
         getTargetForeignKeyFields().addElement(targetForeignKeyField);
         getSourceKeyFields().addElement(sourceKeyField);
@@ -235,10 +239,23 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
         return selectionCriteria;
     }
 
+    /**
+     * INTERNAL: 
+     * This method is used to store the FK fields that can be cached that correspond to noncacheable mappings
+     * the FK field values will be used to re-issue the query when cloning the shared cache entity
+     */
+    @Override
+    public void collectQueryParameters(Set<DatabaseField> cacheFields){
+        for (DatabaseField field : getSourceKeyFields()) {
+            cacheFields.add(field);
+        }
+    }
+
     /**
      * INTERNAL:
      * Clone the appropriate attributes.
      */
+    @Override
     public Object clone() {
         OneToManyMapping clone = (OneToManyMapping)super.clone();
         clone.setTargetForeignKeysToSourceKeys(new HashMap(getTargetForeignKeysToSourceKeys()));
@@ -256,24 +273,28 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL
      * Called when a DatabaseMapping is used to map the key in a collection.  Returns the key.
      */
-    public Object createMapComponentFromRow(AbstractRecord dbRow, ObjectBuildingQuery query, AbstractSession session){
+    public Object createMapComponentFromRow(AbstractRecord dbRow, ObjectBuildingQuery query, CacheKey parentCacheKey, AbstractSession session, boolean isTargetProtected){
         return session.executeQuery(getSelectionQuery(), dbRow);
     }
     
     /**
      * Delete all the reference objects with a single query.
      */
-    protected void deleteAll(DeleteObjectQuery query) throws DatabaseException {
-        Object attribute = getAttributeAccessor().getAttributeValueFromObject(query.getObject()); 
-        if(usesIndirection()) {
-           if(attribute == null || !getIndirectionPolicy().objectIsInstantiated(attribute)) {
+    protected void deleteAll(DeleteObjectQuery query, AbstractSession session) throws DatabaseException {
+        Object attribute = getAttributeValueFromObject(query.getObject()); 
+        if (usesIndirection()) {
+           if (!this.indirectionPolicy.objectIsInstantiated(attribute)) {
                // An empty Vector indicates to DeleteAllQuery that no objects should be removed from cache
-               ((DeleteAllQuery)getDeleteAllQuery()).executeDeleteAll(query.getSession().getSessionForClass(getReferenceClass()), query.getTranslationRow(), new Vector(0));
+               ((DeleteAllQuery)this.deleteAllQuery).executeDeleteAll(session.getSessionForClass(this.referenceClass), query.getTranslationRow(), new Vector(0));
                return;
            }
         }
-        Object referenceObjects = this.getRealCollectionAttributeValueFromObject(query.getObject(), query.getSession());
-        ((DeleteAllQuery)getDeleteAllQuery()).executeDeleteAll(query.getSession().getSessionForClass(getReferenceClass()), query.getTranslationRow(), getContainerPolicy().vectorFor(referenceObjects, query.getSession()));
+        Object referenceObjects = getRealCollectionAttributeValueFromObject(query.getObject(), session);
+        // PERF: Avoid delete if empty.
+        if (session.isUnitOfWork() && this.containerPolicy.isEmpty(referenceObjects)) {
+            return;
+        }
+        ((DeleteAllQuery)this.deleteAllQuery).executeDeleteAll(session.getSessionForClass(getReferenceClass()), query.getTranslationRow(), this.containerPolicy.vectorFor(referenceObjects, session));
      }
 
     /**
@@ -339,6 +360,25 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
         }        
         return new CacheId(key);
     }
+    
+    /**
+     * Overrides CollectionMappig because this mapping requires a DeleteAllQuery instead of a ModifyQuery.  
+     */
+    protected ModifyQuery getDeleteAllQuery() {
+        if (deleteAllQuery == null) {
+            deleteAllQuery = new DeleteAllQuery();//this is casted to a DeleteAllQuery
+        }
+        return deleteAllQuery;
+    }
+    
+    /**
+     * INTERNAL:
+     * Return source key fields for translation by an AggregateObjectMapping
+     */
+    @Override
+    public Collection getFieldsForTranslationInAggregate() {
+        return getSourceKeyFields();
+    }
 
     /**
      * PUBLIC:
@@ -368,6 +408,9 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * Return the source/target key fields.
      */
     public Map<DatabaseField, DatabaseField> getSourceKeysToTargetForeignKeys() {
+        if (sourceKeysToTargetForeignKeys == null) {
+            sourceKeysToTargetForeignKeys = new HashMap(2);
+        }
         return sourceKeysToTargetForeignKeys;
     }
 
@@ -407,6 +450,9 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * Return the target/source key fields.
      */
     public Map<DatabaseField, DatabaseField> getTargetForeignKeysToSourceKeys() {
+        if (targetForeignKeysToSourceKeys == null) {
+            targetForeignKeysToSourceKeys = new HashMap(2);
+        }
         return targetForeignKeysToSourceKeys;
     }
 
@@ -425,6 +471,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * Return whether the mapping has any inverse constraint dependencies,
      * such as foreign keys and join tables.
      */
+    @Override
     public boolean hasInverseConstraintDependency() {
         return true;
     }
@@ -433,7 +480,16 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * Initialize the mapping.
      */
+    @Override
     public void initialize(AbstractSession session) throws DescriptorException {
+        if (session.hasBroker()) {
+            if (getReferenceClass() == null) {
+                throw DescriptorException.referenceClassNotSpecified(this);
+            }
+            // substitute session that owns the mapping for the session that owns reference descriptor.
+            session = session.getBroker().getSessionForClass(getReferenceClass());
+        }
+        
         super.initialize(session);
 
         getContainerPolicy().initialize(session, getReferenceDescriptor().getDefaultTable());
@@ -441,7 +497,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
             setSelectionCriteria(buildDefaultSelectionCriteriaAndAddFieldsToQuery());
         }
 
-        initializeDeleteAllQuery();
+        initializeDeleteAllQuery(session);
         
         if (requiresDataModificationEvents() || getContainerPolicy().requiresDataModificationEvents()) {
             initializeAddTargetQuery(session);
@@ -476,7 +532,10 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
         if(modifyRow.isEmpty()) {
             return;
         }
-        addTargetQuery = new DataModifyQuery();
+        
+        if (!hasCustomAddTargetQuery){
+            addTargetQuery = new DataModifyQuery();
+        }
         
         if (!addTargetQuery.hasSessionName()) {
             addTargetQuery.setSessionName(session.getName());
@@ -486,7 +545,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
         }
         
         // all fields in modifyRow must have the same table
-        DatabaseTable table = ((DatabaseField)modifyRow.getFields().get(0)).getTable();
+        DatabaseTable table = (modifyRow.getFields().get(0)).getTable();
         
         // Build where clause expression.
         Expression whereClause = null;
@@ -564,8 +623,10 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * This query is used to delete the collection of objects from the
      * database.
      */
-    protected void initializeDeleteAllQuery() {
+    protected void initializeDeleteAllQuery(AbstractSession session) {
         ((DeleteAllQuery)getDeleteAllQuery()).setReferenceClass(getReferenceClass());
+        getDeleteAllQuery().setName(getAttributeName());
+        ((DeleteAllQuery)getDeleteAllQuery()).setIsInMemoryOnly(isCascadeOnDeleteSetOnDatabase());
         if (!hasCustomDeleteAllQuery()) {
             // the selection criteria are re-used by the delete all query
             if (getSelectionCriteria() == null) {
@@ -574,6 +635,12 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
                 getDeleteAllQuery().setSelectionCriteria(getSelectionCriteria());
             }
         }
+        if (!getDeleteAllQuery().hasSessionName()) {
+            getDeleteAllQuery().setSessionName(session.getName());
+        }
+        if (getDeleteAllQuery().getPartitioningPolicy() == null) {
+            getDeleteAllQuery().setPartitioningPolicy(getPartitioningPolicy());
+        }
     }
 
     /**
@@ -687,6 +754,11 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
             setSourceKeyFields(org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(getDescriptor().getPrimaryKeyFields()));
         }
         initializeTargetForeignKeysToSourceKeys();
+        if (usesIndirection()) {
+            for (DatabaseField field : getSourceKeyFields()) {
+                field.setKeepInRow(true);
+            }
+        }
         if(requiresDataModificationEvents() || getContainerPolicy().requiresDataModificationEvents()) {
             initializeTargetPrimaryKeyFields();
         }
@@ -773,6 +845,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
     /**
      * INTERNAL:
      */
+    @Override
     public boolean isOneToManyMapping() {
         return true;
     }
@@ -789,6 +862,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * An object was added to the collection during an update, insert it if private.
      */
+    @Override
     protected void objectAddedDuringUpdate(ObjectLevelModifyQuery query, Object objectAdded, ObjectChangeSet changeSet, Map extraData) throws DatabaseException, OptimisticLockException {
         // First insert/update object.
         super.objectAddedDuringUpdate(query, objectAdded, changeSet, extraData);
@@ -813,6 +887,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * An object was removed to the collection during an update, delete it if private.
      */
+    @Override
     protected void objectRemovedDuringUpdate(ObjectLevelModifyQuery query, Object objectDeleted, Map extraData) throws DatabaseException, OptimisticLockException {
         if(!isPrivateOwned()) {
             if (requiresDataModificationEvents() || containerPolicy.requiresDataModificationEvents()){
@@ -840,6 +915,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * Perform the commit event.
      * This is used in the uow to delay data modifications.
      */
+    @Override
     public void performDataModificationEvent(Object[] event, AbstractSession session) throws DatabaseException, DescriptorException {
         // Hey I might actually want to use an inner class here... ok array for now.
         if (event[0] == PostInsert) {
@@ -857,6 +933,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * Insert the reference objects.
      */
+    @Override
     public void postInsert(WriteObjectQuery query) throws DatabaseException, OptimisticLockException {
         if (isReadOnly()) {
             return;
@@ -915,7 +992,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      */
     @Override
     public void postUpdate(WriteObjectQuery query) throws DatabaseException, OptimisticLockException {
-        if (isReadOnly()) {
+        if (this.isReadOnly) {
             return;
         }
         
@@ -927,12 +1004,14 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
         if (!isAttributeValueInstantiatedOrChanged(query.getObject())) {
             return;
         }
-    
-        // manage objects added and removed from the collection
-        Object objectsInMemory = getRealCollectionAttributeValueFromObject(query.getObject(), query.getSession());
-        Object objectsInDB = readPrivateOwnedForObject(query);
-    
-        compareObjectsAndWrite(objectsInDB, objectsInMemory, query);
+        
+        if (query.getObjectChangeSet() != null) {
+            // UnitOfWork
+            writeChanges(query.getObjectChangeSet(), query);
+        } else {
+            // OLD COMMIT            
+            compareObjectsAndWrite(query);
+        }
     }
     
     /**
@@ -948,11 +1027,9 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
             for (DatabaseField targetForeignKeyField : this.targetForeignKeyFields) {
                 fields.add(builder.getField(targetForeignKeyField));
             }
-            return builder.value(fields).in(
-                    builder.getParameter(ForeignReferenceMapping.QUERY_BATCH_PARAMETER));
+            return query.getSession().getPlatform().buildBatchCriteriaForComplexId(builder, fields);
         } else {
-            return builder.getField(this.targetForeignKeyFields.get(0)).in(
-                    builder.getParameter(ForeignReferenceMapping.QUERY_BATCH_PARAMETER));
+            return query.getSession().getPlatform().buildBatchCriteria(builder, builder.getField(this.targetForeignKeyFields.get(0)));
         }
     }
 
@@ -963,43 +1040,58 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
     @Override
     public void preDelete(DeleteObjectQuery query) throws DatabaseException, OptimisticLockException {
         if (!shouldObjectModifyCascadeToParts(query)) {
-            if(this.listOrderField != null) {
+            if (this.listOrderField != null) {
                 updateTargetRowPreDeleteSource(query);
             }
             return;
         }
+        AbstractSession session = query.getSession();
 
-        Object objects = getRealCollectionAttributeValueFromObject(query.getObject(), query.getSession());
-
-        ContainerPolicy cp = getContainerPolicy();
-
-        // if privately-owned parts have their privately-owned sub-parts, delete them one by one;
-        // else delete everything in one shot
+        // If privately-owned parts have their privately-owned sub-parts, delete them one by one;
+        // else delete everything in one shot.
         if (mustDeleteReferenceObjectsOneByOne()) {
-            for (Object iter = cp.iteratorFor(objects); cp.hasNext(iter);) {
-                Object wrappedObject = cp.nextEntry(iter, query.getSession());
-                Object object = cp.unwrapIteratorResult(wrappedObject);
-                DeleteObjectQuery deleteQuery = new DeleteObjectQuery();
-                deleteQuery.setIsExecutionClone(true);
-                deleteQuery.setObject(object);
-                deleteQuery.setCascadePolicy(query.getCascadePolicy());
-                query.getSession().executeQuery(deleteQuery);
-                containerPolicy.propogatePreDelete(deleteQuery, wrappedObject);
+            Object objects = getRealCollectionAttributeValueFromObject(query.getObject(), session);
+            ContainerPolicy cp = getContainerPolicy();
+            if (this.isCascadeOnDeleteSetOnDatabase && session.isUnitOfWork()) {
+                for (Object iterator = cp.iteratorFor(objects); cp.hasNext(iterator);) {
+                    Object wrappedObject = cp.nextEntry(iterator, session);
+                    Object object = cp.unwrapIteratorResult(wrappedObject);
+                    ((UnitOfWorkImpl)session).getCascadeDeleteObjects().add(object);
+                }
             }
-            if (!query.getSession().isUnitOfWork()) {
+            int cascade = query.getCascadePolicy();
+            for (Object iterator = cp.iteratorFor(objects); cp.hasNext(iterator);) {
+                Object wrappedObject = cp.nextEntry(iterator, session);
+                Object object = cp.unwrapIteratorResult(wrappedObject);
+                // PERF: Avoid query execution if already deleted.
+                if (!session.getCommitManager().isCommitCompletedInPostOrIgnore(object) || this.containerPolicy.propagatesEventsToCollection()) {
+                    if (session.isUnitOfWork() && ((UnitOfWorkImpl)session).isObjectNew(object) ){
+                        session.getCommitManager().markIgnoreCommit(object);
+                    } else {
+                        DeleteObjectQuery deleteQuery = new DeleteObjectQuery();
+                        deleteQuery.setIsExecutionClone(true);
+                        deleteQuery.setObject(object);
+                        deleteQuery.setCascadePolicy(cascade);
+                        session.executeQuery(deleteQuery);
+                        this.containerPolicy.propogatePreDelete(deleteQuery, wrappedObject);
+                    }
+                }
+            }
+            if (!session.isUnitOfWork()) {
                 // This deletes any objects on the database, as the collection in memory may have been changed.
                 // This is not required for unit of work, as the update would have already deleted these objects,
                 // and the backup copy will include the same objects causing double deletes.
                 deleteReferenceObjectsLeftOnDatabase(query);
             }
         } else {
-            deleteAll(query);
+            deleteAll(query, session);
         }
     }
     
     /**
      * Prepare a cascade locking policy.
      */
+    @Override
     public void prepareCascadeLockingPolicy() {
         CascadeLockingPolicy policy = new CascadeLockingPolicy(getDescriptor(), getReferenceDescriptor());
         policy.setQueryKeyFields(getSourceKeysToTargetForeignKeys());
@@ -1010,7 +1102,6 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * Returns whether this mapping uses data modification events to complete its writes
      * @see UnidirectionalOneToManyMapping
-     * @return
      */
     public boolean requiresDataModificationEvents(){
         return this.listOrderField != null;
@@ -1071,6 +1162,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * <p>
      * Example: "delete from PHONE where OWNER_ID = #EMPLOYEE_ID"
      */
+    @Override
     public void setDeleteAllSQLString(String sqlString) {
         DeleteAllQuery query = new DeleteAllQuery();
         query.setSQLString(sqlString);
@@ -1084,6 +1176,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * This can be used by the session broker to override the default session
      * to be used for the target class.
      */
+    @Override
     public void setSessionName(String name) {
         super.setSessionName(name);
         if (addTargetQuery != null){
@@ -1195,6 +1288,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * should also affect its parts.
      * Used by write, insert, update, and delete.
      */
+    @Override
     protected boolean shouldObjectModifyCascadeToParts(ObjectLevelModifyQuery query) {
         if (isReadOnly()) {
             return false;
@@ -1223,6 +1317,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL
      * Return true if this mapping supports cascaded version optimistic locking.
      */
+    @Override
     public boolean isCascadedLockingSupported() {
         return true;
     }
@@ -1231,6 +1326,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * Return if this mapping support joining.
      */
+    @Override
     public boolean isJoiningSupported() {
         return true;
     }
@@ -1250,7 +1346,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
             return;
         }
 
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getSession());
+        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), query.getSession());
 
         AbstractRecord keyRow = buildKeyRowForTargetUpdate(query);
         
@@ -1274,7 +1370,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
             query.getSession().executeQuery(addTargetQuery, databaseRow);
         }
     }
-    
+
     protected AbstractRecord buildKeyRowForTargetUpdate(ObjectLevelModifyQuery query){
         return new DatabaseRecord();
     }
@@ -1293,7 +1389,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
         }
 
         ContainerPolicy cp = getContainerPolicy();
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getSession());
+        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), query.getSession());
         AbstractRecord databaseRow = buildKeyRowForTargetUpdate(query);
 
         // Extract target field and its value. Construct insert statement and execute it
@@ -1325,7 +1421,7 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
             return;
         }
         AbstractSession session = query.getSession();
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), session);
+        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getDescriptor(), session);
         AbstractRecord translationRow = new DatabaseRecord();
 
         // Extract primary key and value from the source (use translation row).
@@ -1400,9 +1496,10 @@ public class OneToManyMapping extends CollectionMapping implements RelationalMap
      * INTERNAL:
      * Used to verify whether the specified object is deleted or not.
      */
+    @Override
     public boolean verifyDelete(Object object, AbstractSession session) throws DatabaseException {
-        if (this.isPrivateOwned()) {
-            Object objects = this.getRealCollectionAttributeValueFromObject(object, session);
+        if (this.isPrivateOwned() || isCascadeRemove()) {
+            Object objects = getRealCollectionAttributeValueFromObject(object, session);
 
             ContainerPolicy containerPolicy = getContainerPolicy();
             for (Object iter = containerPolicy.iteratorFor(objects); containerPolicy.hasNext(iter);) {
