@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -10,14 +10,20 @@
  * Contributors:
  *     07/16/2009 Andrei Ilitchev 
  *       - Bug 282553: JPA 2.0 JoinTable support for OneToOne and ManyToOne
+ *     14/05/2012-2.4 Guy Pelletier   
+ *       - 376603: Provide for table per tenant support for multitenant applications
  ******************************************************************************/  
 package org.eclipse.persistence.mappings;
 
 import java.util.ArrayList;
 import java.util.Enumeration;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
 import java.util.Vector;
 
+import org.eclipse.persistence.descriptors.ClassDescriptor;
+import org.eclipse.persistence.descriptors.TablePerMultitenantPolicy;
 import org.eclipse.persistence.exceptions.DescriptorException;
 import org.eclipse.persistence.expressions.Expression;
 import org.eclipse.persistence.expressions.ExpressionBuilder;
@@ -51,7 +57,7 @@ import org.eclipse.persistence.sessions.DatabaseRecord;
  * that was originally defined in ManyToManyMapping
  * and now is shared with OneToOneMapping. 
  */
-public class RelationTableMechanism  implements Cloneable {
+public class RelationTableMechanism  implements Cloneable, java.io.Serializable {
     /** The intermediate relation table. */
     protected DatabaseTable relationTable;
 
@@ -133,34 +139,26 @@ public class RelationTableMechanism  implements Cloneable {
     Expression buildSelectionCriteria(ForeignReferenceMapping mapping, Expression criteria) {
         return buildSelectionCriteriaAndAddFieldsToQueryInternal(mapping, criteria, true, false);
     }
+    
     Expression buildSelectionCriteriaAndAddFieldsToQuery(ForeignReferenceMapping mapping, Expression criteria) {
         return buildSelectionCriteriaAndAddFieldsToQueryInternal(mapping, criteria, true, true);
     }
-    protected Expression buildSelectionCriteriaAndAddFieldsToQueryInternal(ForeignReferenceMapping mapping, Expression criteria, boolean shouldAddTargetFields, boolean shouldAddFieldsToQuery) {
-        DatabaseField relationKey;
-        DatabaseField sourceKey;
-        DatabaseField targetKey;
-        Expression exp1;
-        Expression exp2;
-        Expression expression;
+    
+    /**
+     * INTERNAL:
+     * Build the selection criteria to join the source, relation, and target tables.
+     */
+    public Expression buildSelectionCriteriaAndAddFieldsToQueryInternal(ForeignReferenceMapping mapping, Expression criteria, boolean shouldAddTargetFields, boolean shouldAddFieldsToQuery) {
         Expression builder = new ExpressionBuilder();
-        Enumeration relationKeyEnum;
-        Enumeration sourceKeyEnum;
-        Enumeration targetKeyEnum;
-
         Expression linkTable = builder.getTable(this.relationTable);
 
-        if(shouldAddTargetFields) {
-            targetKeyEnum = getTargetKeyFields().elements();
-            relationKeyEnum = getTargetRelationKeyFields().elements();
-            for (; targetKeyEnum.hasMoreElements();) {
-                relationKey = (DatabaseField)relationKeyEnum.nextElement();
-                targetKey = (DatabaseField)targetKeyEnum.nextElement();
-    
-                exp1 = builder.getField(targetKey);
-                exp2 = linkTable.getField(relationKey);
-                expression = exp1.equal(exp2);
-    
+        if (shouldAddTargetFields) {
+            Iterator<DatabaseField> targetKeyIterator = getTargetKeyFields().iterator();
+            Iterator<DatabaseField> relationKeyIterator = getTargetRelationKeyFields().iterator();
+            while (targetKeyIterator.hasNext()) {
+                DatabaseField relationKey = relationKeyIterator.next();
+                DatabaseField targetKey = targetKeyIterator.next();    
+                Expression expression = builder.getField(targetKey).equal(linkTable.getField(relationKey));    
                 if (criteria == null) {
                     criteria = expression;
                 } else {
@@ -169,17 +167,13 @@ public class RelationTableMechanism  implements Cloneable {
             }
         }
 
-        relationKeyEnum = getSourceRelationKeyFields().elements();
-        sourceKeyEnum = getSourceKeyFields().elements();
-
-        for (; relationKeyEnum.hasMoreElements();) {
-            relationKey = (DatabaseField)relationKeyEnum.nextElement();
-            sourceKey = (DatabaseField)sourceKeyEnum.nextElement();
-
-            exp1 = linkTable.getField(relationKey);
-            exp2 = builder.getParameter(sourceKey);
-            expression = exp1.equal(exp2);
+        Iterator<DatabaseField> relationKeyIterator = getSourceRelationKeyFields().iterator();
+        Iterator<DatabaseField> sourceKeyIterator = getSourceKeyFields().iterator();
 
+        while (relationKeyIterator.hasNext()) {
+            DatabaseField relationKey = relationKeyIterator.next();
+            DatabaseField sourceKey = sourceKeyIterator.next();            
+            Expression expression = linkTable.getField(relationKey).equal(builder.getParameter(sourceKey));
             if (criteria == null) {
                 criteria = expression;
             } else {
@@ -187,13 +181,24 @@ public class RelationTableMechanism  implements Cloneable {
             }
         }
         
-        if(shouldAddFieldsToQuery && mapping.isCollectionMapping()) {
+        if (shouldAddFieldsToQuery && mapping.isCollectionMapping()) {
             ((CollectionMapping)mapping).getContainerPolicy().addAdditionalFieldsToQuery(mapping.getSelectionQuery(), linkTable);
         }
         
         return criteria;
     }
     
+    /**
+     * INTERNAL: 
+     * This method is used to store the FK fields that can be cached that correspond to noncacheable mappings
+     * the FK field values will be used to re-issue the query when cloning the shared cache entity
+     */
+    protected void collectQueryParameters(Set<DatabaseField> cacheFields){
+        for (DatabaseField field : getSourceKeyFields()) {
+            cacheFields.add(field);
+        }
+    }
+
     /**
      * INTERNAL:
      * The mapping clones itself to create deep copy.
@@ -321,7 +326,7 @@ public class RelationTableMechanism  implements Cloneable {
      * INTERNAL:
      * Return the selection criteria used to IN batch fetching.
      */
-    public Expression buildBatchCriteria(ExpressionBuilder builder, ObjectLevelReadQuery query) {
+    protected Expression buildBatchCriteria(ExpressionBuilder builder, ObjectLevelReadQuery query) {
         Expression linkTable = builder.getTable(this.relationTable);
         Expression criteria = null;
         int size = this.targetRelationKeyFields.size();
@@ -337,11 +342,9 @@ public class RelationTableMechanism  implements Cloneable {
             for (DatabaseField sourceRelationKeyField : this.sourceRelationKeyFields) {
                 fields.add(linkTable.getField(sourceRelationKeyField));
             }            
-            return criteria.and(builder.value(fields).in(
-                    builder.getParameter(ForeignReferenceMapping.QUERY_BATCH_PARAMETER)));
+            return criteria.and(query.getSession().getPlatform().buildBatchCriteriaForComplexId(builder, fields));
         } else {
-            return criteria.and(linkTable.getField(this.sourceRelationKeyFields.get(0)).in(
-                    builder.getParameter(ForeignReferenceMapping.QUERY_BATCH_PARAMETER)));
+            return criteria.and(query.getSession().getPlatform().buildBatchCriteria(builder, linkTable.getField(this.sourceRelationKeyFields.get(0))));
         }
     }
 
@@ -443,6 +446,18 @@ public class RelationTableMechanism  implements Cloneable {
         return fieldNames;
     }
 
+    /**
+     * INTERNAL:
+     * Return the relation field for the target field.
+     */
+    public DatabaseField getRelationFieldForTargetField(DatabaseField targetField) {
+        int index = this.targetKeyFields.indexOf(targetField);
+        if (index == -1) {
+            return null;
+        }
+        return this.targetRelationKeyFields.get(index);
+    }
+
     /**
      * INTERNAL:
      * Return all the target keys associated with the mapping.
@@ -496,13 +511,13 @@ public class RelationTableMechanism  implements Cloneable {
      */
     public void initialize(AbstractSession session, ForeignReferenceMapping mapping) throws DescriptorException {
         initializeRelationTable(session, mapping);
-        initializeSourceRelationKeys(session, mapping);
-        initializeTargetRelationKeys(session, mapping);
+        initializeSourceRelationKeys(mapping);
+        initializeTargetRelationKeys(mapping);
 
         if (isSingleSourceRelationKeySpecified()) {
-            initializeSourceKeysWithDefaults(session, mapping);
+            initializeSourceKeysWithDefaults(mapping);
         } else {
-            initializeSourceKeys(session, mapping);
+            initializeSourceKeys(mapping);
         }
 
         if (isSingleTargetRelationKeySpecified()) {
@@ -542,6 +557,10 @@ public class RelationTableMechanism  implements Cloneable {
         if (!getDeleteQuery().hasSessionName()) {
             getDeleteQuery().setSessionName(session.getName());
         }
+        if (getDeleteQuery().getPartitioningPolicy() == null) {
+            getDeleteQuery().setPartitioningPolicy(mapping.getPartitioningPolicy());
+        }
+        getInsertQuery().setName(mapping.getAttributeName());
         if (hasCustomDeleteQuery()) {
             return;
         }
@@ -550,19 +569,13 @@ public class RelationTableMechanism  implements Cloneable {
         Expression whereClause = null;
         Expression builder = new ExpressionBuilder();
 
-        Enumeration relationKeyEnum = getSourceRelationKeyFields().elements();
-        for (; relationKeyEnum.hasMoreElements();) {
-            DatabaseField relationKey = (DatabaseField)relationKeyEnum.nextElement();
-
+        for (DatabaseField relationKey : getSourceRelationKeyFields()) {
             Expression expression = builder.getField(relationKey).equal(builder.getParameter(relationKey));
             whereClause = expression.and(whereClause);
         }
 
-        if(mapping.isCollectionMapping()) {
-            relationKeyEnum = getTargetRelationKeyFields().elements();
-            for (; relationKeyEnum.hasMoreElements();) {
-                DatabaseField relationKey = (DatabaseField)relationKeyEnum.nextElement();
-    
+        if (mapping.isCollectionMapping()) {
+            for (DatabaseField relationKey : getTargetRelationKeyFields()) {    
                 Expression expression = builder.getField(relationKey).equal(builder.getParameter(relationKey));
                 whereClause = expression.and(whereClause);
             }
@@ -611,6 +624,10 @@ public class RelationTableMechanism  implements Cloneable {
         if (!getInsertQuery().hasSessionName()) {
             getInsertQuery().setSessionName(session.getName());
         }
+        if (getInsertQuery().getPartitioningPolicy() == null) {
+            getInsertQuery().setPartitioningPolicy(mapping.getPartitioningPolicy());
+        }
+        getInsertQuery().setName(mapping.getAttributeName());
         if (hasCustomInsertQuery()) {
             return;
         }
@@ -618,17 +635,15 @@ public class RelationTableMechanism  implements Cloneable {
         SQLInsertStatement statement = new SQLInsertStatement();
         statement.setTable(getRelationTable());
         AbstractRecord joinRow = new DatabaseRecord();
-        for (Enumeration targetEnum = getTargetRelationKeyFields().elements();
-                 targetEnum.hasMoreElements();) {
-            joinRow.put((DatabaseField)targetEnum.nextElement(), null);
+        for (DatabaseField field : getTargetRelationKeyFields()) {
+            joinRow.put(field, null);
         }
-        for (Enumeration sourceEnum = getSourceRelationKeyFields().elements();
-                 sourceEnum.hasMoreElements();) {
-            joinRow.put((DatabaseField)sourceEnum.nextElement(), null);
+        for (DatabaseField field : getSourceRelationKeyFields()) {
+            joinRow.put(field, null);
         }
-        if(mapping.isCollectionMapping()) {
+        if (mapping.isCollectionMapping()) {
             CollectionMapping collectionMapping = (CollectionMapping)mapping;
-            if(collectionMapping.getListOrderField() != null) {
+            if (collectionMapping.getListOrderField() != null) {
                 joinRow.put(collectionMapping.getListOrderField(), null);
             }
             collectionMapping.getContainerPolicy().addFieldsForMapKey(joinRow);
@@ -661,6 +676,13 @@ public class RelationTableMechanism  implements Cloneable {
     protected void initializeRelationTable(AbstractSession session, ForeignReferenceMapping mapping) throws DescriptorException {
         Platform platform = session.getDatasourcePlatform();
 
+        // We need to look up the relation table name from the reference
+        // descriptor if we are the non owning side of a bidirectional mapping
+        // to a table per tenant descriptor.
+        if (mapping.isReadOnly() && mapping.getReferenceDescriptor().hasTablePerMultitenantPolicy()) {
+            setRelationTable(((TablePerMultitenantPolicy) mapping.getReferenceDescriptor().getMultitenantPolicy()).getTable(getRelationTable()));
+        }
+        
         if (!hasRelationTable()) {
             throw DescriptorException.noRelationTable(mapping);
         }
@@ -675,9 +697,12 @@ public class RelationTableMechanism  implements Cloneable {
      * INTERNAL:
      * All the source key field names are converted to DatabaseField and stored.
      */
-    protected void initializeSourceKeys(AbstractSession session, ForeignReferenceMapping mapping) {
+    protected void initializeSourceKeys(ForeignReferenceMapping mapping) {
         for (int index = 0; index < getSourceKeyFields().size(); index++) {
             DatabaseField field = mapping.getDescriptor().buildField(getSourceKeyFields().get(index));
+            if (((ForeignReferenceMapping)mapping).usesIndirection()) {
+                field.setKeepInRow(true);
+            }
             getSourceKeyFields().set(index, field);
         }
     }
@@ -686,10 +711,14 @@ public class RelationTableMechanism  implements Cloneable {
      * INTERNAL:
      * If a user does not specify the source key then the primary keys of the source table are used.
      */
-    protected void initializeSourceKeysWithDefaults(AbstractSession session, DatabaseMapping mapping) {
+    protected void initializeSourceKeysWithDefaults(DatabaseMapping mapping) {
         List<DatabaseField> primaryKeyFields = mapping.getDescriptor().getPrimaryKeyFields();
         for (int index = 0; index < primaryKeyFields.size(); index++) {
-            getSourceKeyFields().addElement(primaryKeyFields.get(index));
+            DatabaseField field = primaryKeyFields.get(index);
+            if (((ForeignReferenceMapping)mapping).usesIndirection()) {
+                field.setKeepInRow(true);
+            }
+            getSourceKeyFields().addElement(field);
         }
     }
 
@@ -697,13 +726,20 @@ public class RelationTableMechanism  implements Cloneable {
      * INTERNAL:
      * All the source relation key field names are converted to DatabaseField and stored.
      */
-    protected void initializeSourceRelationKeys(AbstractSession session, ForeignReferenceMapping mapping) throws DescriptorException {
+    protected void initializeSourceRelationKeys(ForeignReferenceMapping mapping) throws DescriptorException {
         if (getSourceRelationKeyFields().size() == 0) {
             throw DescriptorException.noSourceRelationKeysSpecified(mapping);
         }
 
         for (Enumeration entry = getSourceRelationKeyFields().elements(); entry.hasMoreElements();) {
             DatabaseField field = (DatabaseField)entry.nextElement();
+            
+            // Update the fields table first if the mapping is from a table per tenant entity.
+            ClassDescriptor sourceDescriptor = mapping.getDescriptor();
+            if (sourceDescriptor.hasTablePerMultitenantPolicy()) {
+                field.setTable(((TablePerMultitenantPolicy) sourceDescriptor.getMultitenantPolicy()).getTable(field.getTable()));
+            }
+            
             if (field.hasTableName() && (!(field.getTableName().equals(getRelationTable().getName())))) {
                 throw DescriptorException.relationKeyFieldNotProperlySpecified(field, mapping);
             }
@@ -737,14 +773,20 @@ public class RelationTableMechanism  implements Cloneable {
      * INTERNAL:
      * All the target relation key field names are converted to DatabaseField and stored.
      */
-    protected void initializeTargetRelationKeys(AbstractSession session, ForeignReferenceMapping mapping) {
+    protected void initializeTargetRelationKeys(ForeignReferenceMapping mapping) {
         if (getTargetRelationKeyFields().size() == 0) {
             throw DescriptorException.noTargetRelationKeysSpecified(mapping);
         }
 
-        for (Enumeration targetEnum = getTargetRelationKeyFields().elements();
-                 targetEnum.hasMoreElements();) {
+        for (Enumeration targetEnum = getTargetRelationKeyFields().elements(); targetEnum.hasMoreElements();) {
             DatabaseField field = (DatabaseField)targetEnum.nextElement();
+            
+            // Update the fields table first if the mapping is from a table per tenant entity.
+            ClassDescriptor referenceDescriptor = mapping.getReferenceDescriptor();
+            if (referenceDescriptor.hasTablePerMultitenantPolicy()) {
+                field.setTable(((TablePerMultitenantPolicy) referenceDescriptor.getMultitenantPolicy()).getTable(field.getTable()));
+            }
+            
             if (field.hasTableName() && (!(field.getTableName().equals(getRelationTable().getName())))) {
                 throw DescriptorException.relationKeyFieldNotProperlySpecified(field, mapping);
             }
