@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,8 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     14/05/2012-2.4 Guy Pelletier   
+ *       - 376603: Provide for table per tenant support for multitenant applications
  ******************************************************************************/  
 package org.eclipse.persistence.mappings.structures;
 
@@ -44,9 +46,11 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
         this.orderedFields = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
     }
     
-    /** INTERNAL:
+    /**
+     * INTERNAL:
      * Auto-Default orderedFields to fields
      */
+    @Override
     public void initialize(AbstractSession session) throws DescriptorException {
         super.initialize(session);
         if (orderedFields==null || orderedFields.size()==0){
@@ -71,6 +75,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * Return them in a vector.
      * The field value better be an Array.
      */
+    @Override
     public Vector buildDirectValuesFromFieldValue(Object fieldValue) throws DatabaseException {
         
         if(fieldValue == null) {
@@ -86,6 +91,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * set of direct values.
      * The database better be expecting an ARRAY.
      */
+    @Override
     public Object buildFieldValueFromDirectValues(Vector directValues, String elementDataTypeName, AbstractSession session) throws DatabaseException {
         Object[] fields = Helper.arrayFromVector(directValues);
         try {
@@ -104,6 +110,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * Build and return the field value from the specified nested database row.
      * The database better be expecting a Struct.
      */
+    @Override
     public Object buildFieldValueFromNestedRow(AbstractRecord nestedRow, AbstractSession session) throws DatabaseException {
         java.sql.Connection connection = ((DatabaseAccessor)session.getAccessor()).getConnection();
         return this.buildStructureFromRow(nestedRow, session, connection);
@@ -116,6 +123,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * The database better be expecting an ARRAY.
      * It looks like we can ignore inheritance here....
      */
+    @Override
     public Object buildFieldValueFromNestedRows(Vector nestedRows, String structureName, AbstractSession session) throws DatabaseException {
         Object[] fields = new Object[nestedRows.size()];
         java.sql.Connection connection = ((DatabaseAccessor)session.getAccessor()).getConnection();
@@ -143,6 +151,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
             }
         }
     }
+    
      /**
       * INTERNAL:
       * Build and return the nested rows from the specified field value.
@@ -201,7 +210,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
                     }
                 }
                 arrayValue = descriptor.getObjectBuilder().buildNewInstance();
-                descriptor.getObjectBuilder().buildAttributesIntoObject(arrayValue, nestedRow, query, null, false);
+                descriptor.getObjectBuilder().buildAttributesIntoObject(arrayValue, null, nestedRow, query, null, null, false, session);
             } else if (isNestedStructure && (arrayValue instanceof Array)){
                 arrayValue = buildContainerFromArray((Array)arrayValue, (ObjectRelationalDatabaseField)nestedType, session);
             }             
@@ -216,6 +225,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * Build and return the nested database row from the specified field value.
      * The field value better be an Struct.
      */
+     @Override
     public AbstractRecord buildNestedRowFromFieldValue(Object fieldValue) throws DatabaseException {
         
         AbstractRecord row = new DatabaseRecord();
@@ -234,6 +244,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * Build and return the nested rows from the specified field value.
      * The field value better be an ARRAY.
      */
+    @Override
     public Vector buildNestedRowsFromFieldValue(Object fieldValue, AbstractSession session) throws DatabaseException {
         
         if(fieldValue==null){
@@ -315,8 +326,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
     /**
      * INTERNAL:
      * Build array of objects for Array data type.
-     */
-    
+     */    
     public static Object buildArrayObjectFromArray(Object array) throws DatabaseException {
         Object[] objects = null;
         if(array==null){
@@ -346,7 +356,6 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * INTERNAL:
      * Build array of objects for Struct data type.
      */
-    
     public static Object buildArrayObjectFromStruct(Object structure) throws DatabaseException{
         Object[] attributes = null;
         if(structure==null){
@@ -376,6 +385,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * INTERNAL:
      * Aggregates use a dummy table as default.
      */
+    @Override
     protected DatabaseTable extractDefaultTable() {
         if (isAggregateDescriptor()) {
             return new DatabaseTable();
@@ -408,7 +418,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
 
         ValueReadQuery valueQuery = new ValueReadQuery();
         valueQuery.setSQLStatement(statement);
-        valueQuery.prepareCall(session, new DatabaseRecord());
+        valueQuery.checkPrepare(session, new DatabaseRecord(), true);
         // Must return unwrapped Ref on WLS.
         valueQuery.getCall().setIsNativeConnectionRequired(true);
 
@@ -430,6 +440,7 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      *  PUBLIC:
      *  Return if this is an ObjectRelationalDataTypeDescriptor.
      */
+    @Override
     public boolean isObjectRelationalDataTypeDescriptor(){
         return true;
     }
@@ -438,10 +449,12 @@ public class ObjectRelationalDataTypeDescriptor extends RelationalDescriptor {
      * INTERNAL:
      * Aggregates obj-rel are initialized normally as no cloning is required.
      */
-    public boolean requiresInitialization() {
+    @Override
+    public boolean requiresInitialization(AbstractSession session) {
         return true;
     }
 
+    @Override
     protected void validateMappingType(DatabaseMapping mapping) {
         //do nothing
     }
