@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,8 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     14/05/2012-2.4 Guy Pelletier   
+ *       - 376603: Provide for table per tenant support for multitenant applications
  ******************************************************************************/
 package org.eclipse.persistence.oxm;
 
@@ -17,27 +19,47 @@ import java.util.Collections;
 import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map.Entry;
 import java.util.StringTokenizer;
 import java.util.Vector;
+
 import javax.xml.namespace.QName;
+
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.descriptors.InheritancePolicy;
 import org.eclipse.persistence.exceptions.DatabaseException;
 import org.eclipse.persistence.exceptions.DescriptorException;
+import org.eclipse.persistence.exceptions.XMLMarshalException;
+import org.eclipse.persistence.internal.descriptors.InstantiationPolicy;
+import org.eclipse.persistence.internal.descriptors.ObjectBuilder;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.helper.DatabaseTable;
 import org.eclipse.persistence.internal.helper.Helper;
 import org.eclipse.persistence.internal.helper.NonSynchronizedVector;
-import org.eclipse.persistence.internal.identitymaps.AbstractIdentityMap;
+import org.eclipse.persistence.internal.oxm.Root;
 import org.eclipse.persistence.internal.oxm.TreeObjectBuilder;
+import org.eclipse.persistence.internal.oxm.Unmarshaller;
 import org.eclipse.persistence.internal.oxm.XPathFragment;
+import org.eclipse.persistence.internal.oxm.XPathQName;
+import org.eclipse.persistence.internal.oxm.mappings.Descriptor;
+import org.eclipse.persistence.internal.oxm.record.UnmarshalRecord;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
+import org.eclipse.persistence.mappings.AggregateMapping;
+import org.eclipse.persistence.mappings.AttributeAccessor;
 import org.eclipse.persistence.mappings.DatabaseMapping;
+import org.eclipse.persistence.mappings.foundation.AbstractDirectMapping;
+import org.eclipse.persistence.oxm.mappings.XMLChoiceCollectionMapping;
+import org.eclipse.persistence.oxm.mappings.XMLChoiceObjectMapping;
+import org.eclipse.persistence.oxm.mappings.XMLCompositeCollectionMapping;
+import org.eclipse.persistence.oxm.mappings.XMLCompositeDirectCollectionMapping;
+import org.eclipse.persistence.oxm.mappings.XMLCompositeObjectMapping;
 import org.eclipse.persistence.oxm.mappings.XMLDirectMapping;
-import org.eclipse.persistence.oxm.record.UnmarshalRecord;
+import org.eclipse.persistence.oxm.mappings.XMLMapping;
 import org.eclipse.persistence.oxm.record.XMLRecord;
 import org.eclipse.persistence.oxm.schema.XMLSchemaReference;
+import org.eclipse.persistence.queries.AttributeGroup;
+import org.eclipse.persistence.queries.DoesExistQuery;
 
 /**
  * Use an XML project for nontransactional, nonpersistent (in-memory) conversions between Java objects and XML documents.
@@ -48,7 +70,7 @@ import org.eclipse.persistence.oxm.schema.XMLSchemaReference;
  *
  * @see org.eclipse.persistence.oxm.mappings
  */
-public class XMLDescriptor extends ClassDescriptor {
+public class XMLDescriptor extends ClassDescriptor implements Descriptor<AttributeAccessor, DatabaseMapping, DatabaseField, InheritancePolicy, InstantiationPolicy, NamespaceResolver, ObjectBuilder, UnmarshalRecord, XMLUnmarshaller>{
 
     private static final Vector EMPTY_VECTOR = NonSynchronizedVector.newInstance(1);
 
@@ -60,6 +82,9 @@ public class XMLDescriptor extends ClassDescriptor {
     private boolean isWrapper = false;
     private boolean resultAlwaysXMLRoot = false;
     private boolean lazilyInitialized = false;
+    private AttributeAccessor locationAccessor = null;
+    private boolean hasReferenceMappings = false;
+    
 
     /**
      * PUBLIC:
@@ -68,7 +93,7 @@ public class XMLDescriptor extends ClassDescriptor {
     public XMLDescriptor() {
         this.tables = NonSynchronizedVector.newInstance(3);
         this.mappings = NonSynchronizedVector.newInstance();
-        this.primaryKeyFields = new ArrayList(1);
+        this.primaryKeyFields = null;
         this.fields = NonSynchronizedVector.newInstance();
         this.allFields = NonSynchronizedVector.newInstance();
         this.constraintDependencies = EMPTY_VECTOR;
@@ -76,32 +101,21 @@ public class XMLDescriptor extends ClassDescriptor {
         this.queryKeys = Collections.EMPTY_MAP;
         this.initializationStage = UNINITIALIZED;
         this.interfaceInitializationStage = UNINITIALIZED;
-        this.shouldAlwaysRefreshCache = false;
-        this.shouldOnlyRefreshCacheIfNewerVersion = false;
-        this.shouldDisableCacheHits = false;
-        this.identityMapSize = 100;
-        this.remoteIdentityMapSize = -1;
-        this.identityMapClass = AbstractIdentityMap.getDefaultIdentityMapClass();
-        this.remoteIdentityMapClass = null;
         this.descriptorType = NORMAL;
-        this.shouldAlwaysRefreshCacheOnRemote = false;
-        this.shouldDisableCacheHitsOnRemote = false;
         this.shouldOrderMappings = true;
         this.shouldBeReadOnly = false;
         this.shouldAlwaysConformResultsInUnitOfWork = false;
         this.shouldAcquireCascadedLocks = false;
         this.hasSimplePrimaryKey = false;
-        this.isIsolated = false;
         this.idValidation = null;
         this.derivesIdMappings = Collections.EMPTY_MAP;
         this.additionalWritableMapKeyFields = Collections.EMPTY_LIST;
 
         // Policies        
         this.objectBuilder = new TreeObjectBuilder(this);
-        this.cascadeLockingPolicies = null;
 
         this.shouldOrderMappings = false;
-        this.descriptorIsAggregate();
+        descriptorIsAggregate();
     }
 
     /**
@@ -166,11 +180,15 @@ public class XMLDescriptor extends ClassDescriptor {
             int index = getTableNames().indexOf(newDefaultRootElement);
             if (index == 0) {
                 return;
-            } else if (index >= 0) {
+            } 
+            DatabaseTable databaseTable = new DatabaseTable();
+            databaseTable.setUseDelimiters(false);
+            databaseTable.setName(newDefaultRootElement);
+            if (index >= 0) {
                 getTables().remove(index);
-                getTables().add(0, new DatabaseTable(newDefaultRootElement));
+                getTables().add(0, databaseTable);
             } else {
-                getTables().add(0, new DatabaseTable(newDefaultRootElement));
+                getTables().add(0, databaseTable);
             }
         }
     }
@@ -238,6 +256,15 @@ public class XMLDescriptor extends ClassDescriptor {
         schemaReference = newSchemaReference;
     }
 
+    /**
+     * PUBLIC:
+     * Return if the descriptor maps to XML.
+     */
+    @Override
+    public boolean isXMLDescriptor() {
+        return true;
+    }
+    
     /**
      * If true, the descriptor may be lazily initialized.  This is useful if the
      * descriptor may not get used.
@@ -254,6 +281,14 @@ public class XMLDescriptor extends ClassDescriptor {
         this.lazilyInitialized = shouldLazyInitiailize;
     }
 
+    @Override
+    public Vector<String> getPrimaryKeyFieldNames() {
+        if(null == primaryKeyFields) {
+            return new Vector<String>(0);
+        }
+        return super.getPrimaryKeyFieldNames();
+    }
+
     protected void validateMappingType(DatabaseMapping mapping) {
         if (!(mapping.isXMLMapping())) {
             throw DescriptorException.invalidMappingType(mapping);
@@ -309,6 +344,38 @@ public class XMLDescriptor extends ClassDescriptor {
         }
         return (Vector) fieldValue;
     }
+    
+    /**
+     * Return a new direct/basic mapping for this type of descriptor.
+     */
+    @Override
+    public AbstractDirectMapping newDirectMapping() {
+        return new XMLDirectMapping();
+    }
+    
+    /**
+     * Return a new aggregate/embedded mapping for this type of descriptor.
+     */
+    @Override
+    public AggregateMapping newAggregateMapping() {
+        return new XMLCompositeObjectMapping();
+    }
+    
+    /**
+     * Return a new aggregate collection/element collection mapping for this type of descriptor.
+     */
+    @Override
+    public DatabaseMapping newAggregateCollectionMapping() {
+        return new XMLCompositeCollectionMapping();
+    }
+    
+    /**
+     * Return a new direct collection/element collection mapping for this type of descriptor.
+     */
+    @Override
+    public DatabaseMapping newDirectCollectionMapping() {
+        return new XMLCompositeDirectCollectionMapping();
+    }
 
     /**
     * PUBLIC:
@@ -351,7 +418,7 @@ public class XMLDescriptor extends ClassDescriptor {
 
     @Override
     public void addPrimaryKeyFieldName(String fieldName) {
-        super.addPrimaryKeyField(new XMLField(fieldName));
+        addPrimaryKeyField(new XMLField(fieldName));
     }
 
     @Override
@@ -360,11 +427,17 @@ public class XMLDescriptor extends ClassDescriptor {
             String fieldName = field.getName();
             field = new XMLField(fieldName);
         }
+        if(null == primaryKeyFields) {
+            primaryKeyFields = new ArrayList<DatabaseField>(1);
+        }
         super.addPrimaryKeyField(field);
     }
 
     @Override
     public void setPrimaryKeyFields(List<DatabaseField> thePrimaryKeyFields) {
+        if(null == thePrimaryKeyFields) {
+            return;
+        }
         List<DatabaseField> xmlFields = new ArrayList(thePrimaryKeyFields.size());
         Iterator<DatabaseField> it = thePrimaryKeyFields.iterator();
 
@@ -435,7 +508,7 @@ public class XMLDescriptor extends ClassDescriptor {
             xmlField.setNamespaceResolver(this.getNamespaceResolver());
             xmlField.initialize();
         } catch (ClassCastException e) {
-            // Assumes fields are always XML.
+            // Assumes field is always an XMLField
         }
         return super.buildField(field);
     }
@@ -489,27 +562,54 @@ public class XMLDescriptor extends ClassDescriptor {
             }
         }
 
+        getCachePolicy().useNoIdentityMap();
+        getQueryManager().getDoesExistQuery().setExistencePolicy(DoesExistQuery.CheckDatabase);
+ 
         validateBeforeInitialization(session);
 
         preInitializeInheritancePolicy(session);
 
-        // Make sure that parent is already preinitialized
-        if (hasInheritance()) {
-            // The default table will be set in this call once the duplicate
-            // tables have been removed.
-            getInheritancePolicy().preInitialize(session);
-        } else {
-            // This must be done now, after validate, before init anything else.
-            setInternalDefaultTable();
-        }
-
         verifyTableQualifiers(session.getDatasourcePlatform());
         initializeProperties(session);
 
         if (hasInterfacePolicy()) {
             preInterfaceInitialization(session);
         }
+        getCachePolicy().assignDefaultValues(session);
+    }
 
+    
+    @Override
+    protected void preInitializeInheritancePolicy(AbstractSession session) throws DescriptorException {
+        super.preInitializeInheritancePolicy(session);
+        // Make sure that parent is already preinitialized
+        if (hasInheritance()) {
+            if(isChildDescriptor()) {
+                XMLDescriptor parentDescriptor = (XMLDescriptor) getInheritancePolicy().getParentDescriptor();
+                NamespaceResolver parentNamespaceResolver = parentDescriptor.getNamespaceResolver();
+                if(null != parentNamespaceResolver && parentNamespaceResolver != namespaceResolver) {
+                    if(null == namespaceResolver) {
+                        namespaceResolver = getNonNullNamespaceResolver();
+                    }
+                    if(parentNamespaceResolver.hasPrefixesToNamespaces()) {
+                        for(Entry<String, String> entry : parentNamespaceResolver.getPrefixesToNamespaces().entrySet()) {
+                            String namespaceURI = namespaceResolver.resolveNamespacePrefix(entry.getKey());
+                            if(null == namespaceURI) {
+                                namespaceResolver.put(entry.getKey(), entry.getValue());
+                            } else if(!namespaceURI.equals(entry.getValue())) {
+                                throw XMLMarshalException.subclassAttemptedToOverrideNamespaceDeclaration(entry.getKey(), getJavaClassName(), namespaceURI, parentDescriptor.getJavaClassName(), entry.getValue());
+                            }
+                        }
+                    }
+                }
+            }
+            // The default table will be set in this call once the duplicate
+            // tables have been removed.
+            getInheritancePolicy().preInitialize(session);
+        } else {
+            // This must be done now, after validate, before init anything else.
+            setInternalDefaultTable();
+        }
     }
 
     /**
@@ -543,14 +643,16 @@ public class XMLDescriptor extends ClassDescriptor {
         }
 
         //PERF: Ensure that the identical primary key fields are used to avoid equals.
-        for (int index = (getPrimaryKeyFields().size() - 1); index >= 0; index--) {
-            DatabaseField primaryKeyField = getPrimaryKeyFields().get(index);
-            int fieldIndex = getFields().indexOf(primaryKeyField);
+        if(null != primaryKeyFields) {
+            for (int index = (primaryKeyFields.size() - 1); index >= 0; index--) {
+                DatabaseField primaryKeyField = getPrimaryKeyFields().get(index);
+                int fieldIndex = getFields().indexOf(primaryKeyField);
 
-            // Aggregate/agg-collections may not have a mapping for pk field.
-            if (fieldIndex != -1) {
-                primaryKeyField = getFields().get(fieldIndex);
-                getPrimaryKeyFields().set(index, primaryKeyField);
+                // Aggregate/agg-collections may not have a mapping for pk field.
+                if (fieldIndex != -1) {
+                    primaryKeyField = getFields().get(fieldIndex);
+                    getPrimaryKeyFields().set(index, primaryKeyField);
+                }
             }
         }
 
@@ -586,11 +688,26 @@ public class XMLDescriptor extends ClassDescriptor {
             defaultRootElementField.setNamespaceResolver(this.namespaceResolver);
             defaultRootElementField.initialize();
         }
+        if(schemaReference != null && schemaReference.getSchemaContext() != null && (schemaReference.getType() == XMLSchemaReference.COMPLEX_TYPE || schemaReference.getType() == XMLSchemaReference.SIMPLE_TYPE) && getDefaultRootElementType() == null){
+        	
+        	if(hasInheritance() && isChildDescriptor()){
+        		XMLField parentField = ((XMLDescriptor)getInheritancePolicy().getParentDescriptor()).getDefaultRootElementField();
+        		//if this descriptor has a root element field different than it's parent set the leaf element type of the default root field
+        		if(parentField == null || (parentField !=null && !defaultRootElementField.getXPathFragment().equals(parentField.getXPathFragment()))){
+        			setDefaultRootElementType(schemaReference.getSchemaContextAsQName(getNamespaceResolver()));
+        		}
+        	}else{
+        		setDefaultRootElementType(schemaReference.getSchemaContextAsQName(getNamespaceResolver()));
+        	}
+       
+        }
 
-        for(int x = 0, primaryKeyFieldsSize = this.primaryKeyFields.size(); x<primaryKeyFieldsSize; x++) {
-            XMLField pkField = (XMLField) this.primaryKeyFields.get(x);
-            pkField.setNamespaceResolver(this.namespaceResolver);
-            pkField.initialize();
+        if(null != primaryKeyFields) {
+            for(int x = 0, primaryKeyFieldsSize = this.primaryKeyFields.size(); x<primaryKeyFieldsSize; x++) {
+                XMLField pkField = (XMLField) this.primaryKeyFields.get(x);
+                pkField.setNamespaceResolver(this.namespaceResolver);
+                pkField.initialize();
+            }
         }
 
         // These cached settings on the project must be set even if descriptor is initialized.
@@ -608,14 +725,36 @@ public class XMLDescriptor extends ClassDescriptor {
 
         // make sure that parent mappings are initialized?
         if (isChildDescriptor()) {
-            getInheritancePolicy().getParentDescriptor().initialize(session);
+            ClassDescriptor parentDescriptor = getInheritancePolicy().getParentDescriptor();
+            parentDescriptor.initialize(session);
+            if(parentDescriptor.hasEventManager()) {
+                getEventManager();
+            }
         }
 
         for (Enumeration mappingsEnum = getMappings().elements(); mappingsEnum.hasMoreElements();) {
             DatabaseMapping mapping = (DatabaseMapping) mappingsEnum.nextElement();
             validateMappingType(mapping);
             mapping.initialize(session);
-
+            if(mapping.isObjectReferenceMapping()) {
+                this.hasReferenceMappings = true;
+            }
+            if(mapping instanceof XMLChoiceObjectMapping) {
+                XMLChoiceObjectMapping choiceMapping = ((XMLChoiceObjectMapping)mapping);
+                for(XMLMapping next : choiceMapping.getChoiceElementMappings().values()) {
+                    if(((DatabaseMapping)next).isObjectReferenceMapping()) {
+                        this.hasReferenceMappings = true;
+                    }
+                }
+            }
+            if(mapping instanceof XMLChoiceCollectionMapping) {
+                XMLChoiceCollectionMapping choiceMapping = ((XMLChoiceCollectionMapping)mapping);
+                for(XMLMapping next : choiceMapping.getChoiceElementMappings().values()) {
+                    if(((DatabaseMapping)next).isObjectReferenceMapping()) {
+                        this.hasReferenceMappings = true;
+                    }
+                }
+            }
             // Add all the fields in the mapping to myself.
             Helper.addAllUniqueToVector(getFields(), mapping.getFields());
         }
@@ -635,12 +774,24 @@ public class XMLDescriptor extends ClassDescriptor {
         if (hasReturningPolicy()) {
             getReturningPolicy().initialize(session);
         }
-        getEventManager().initialize(session);
-        getCopyPolicy().initialize(session);
+        if (eventManager != null) {
+            eventManager.initialize(session);
+        }
+        if (copyPolicy != null) {
+            copyPolicy.initialize(session);
+        }
         getInstantiationPolicy().initialize(session);
         if (getSchemaReference() != null) {
             getSchemaReference().initialize(session);
         }
+        // If a Location Accessor is set on a superclass, inherit it
+        if (getInheritancePolicyOrNull() != null && getInheritancePolicy().getParentDescriptor() != null) {
+            XMLDescriptor d = (XMLDescriptor) getInheritancePolicy().getParentDescriptor();
+            locationAccessor = d.getLocationAccessor();
+        }
+        if (locationAccessor != null) {
+            locationAccessor.initializeAttributes(getJavaClass());
+        }
     }
 
     /**
@@ -648,7 +799,8 @@ public class XMLDescriptor extends ClassDescriptor {
      * XML descriptors are initialized normally, since they do
      * not need to be cloned by XML aggregate mappings.
      */
-    public boolean requiresInitialization() {
+    @Override
+    public boolean requiresInitialization(AbstractSession session) {
         return (!isDescriptorForInterface());
     }
 
@@ -663,7 +815,7 @@ public class XMLDescriptor extends ClassDescriptor {
      * INTERNAL:
      * Determines the appropriate object to return from the unmarshal
      * call.  The method will either return the object created in the
-     * xmlReader.parse() call or an instance of XMLRoot.  An XMLRoot
+     * xmlReader.parse() call or an instance of Root.  An Root
      * instance will be returned if the DOMRecord element being
      * unmarshalled does not equal the descriptor's default root
      * element.
@@ -674,8 +826,8 @@ public class XMLDescriptor extends ClassDescriptor {
     public Object wrapObjectInXMLRoot(UnmarshalRecord unmarshalRecord, boolean forceWrap) {
         String elementLocalName = unmarshalRecord.getLocalName();
         String elementNamespaceUri = unmarshalRecord.getRootElementNamespaceUri();
-        if (forceWrap || shouldWrapObject(unmarshalRecord.getCurrentObject(), elementNamespaceUri, elementLocalName, null)) {
-            XMLRoot xmlRoot = new XMLRoot();
+        if (forceWrap || shouldWrapObject(unmarshalRecord.getCurrentObject(), elementNamespaceUri, elementLocalName, null, unmarshalRecord.isNamespaceAware())) {
+            Root xmlRoot = new XMLRoot();
             xmlRoot.setLocalName(elementLocalName);
             xmlRoot.setNamespaceURI(elementNamespaceUri);
             xmlRoot.setObject(unmarshalRecord.getCurrentObject());
@@ -684,6 +836,8 @@ public class XMLDescriptor extends ClassDescriptor {
             xmlRoot.setSchemaLocation(unmarshalRecord.getSchemaLocation());
             xmlRoot.setNoNamespaceSchemaLocation(unmarshalRecord.getNoNamespaceSchemaLocation());
             xmlRoot.setNil(unmarshalRecord.isNil());
+            setDeclaredTypeOnXMLRoot(xmlRoot, elementNamespaceUri, elementLocalName, unmarshalRecord.isNamespaceAware(), unmarshalRecord.getUnmarshaller());
+            
             return xmlRoot;
         }
         return unmarshalRecord.getCurrentObject();
@@ -693,7 +847,7 @@ public class XMLDescriptor extends ClassDescriptor {
       * INTERNAL:
       * Determines the appropriate object to return from the unmarshal
       * call.  The method will either return the object created in the
-      * xmlReader.parse() call or an instance of XMLRoot.  An XMLRoot
+      * xmlReader.parse() call or an instance of Root.  An Root
       * instance will be returned if the DOMRecord element being
       * unmarshalled does not equal the descriptor's default root
       * element.
@@ -704,37 +858,58 @@ public class XMLDescriptor extends ClassDescriptor {
       * @param elementPrefix
       * @return object
       */
-    public Object wrapObjectInXMLRoot(Object object, String elementNamespaceUri, String elementLocalName, String elementPrefix, boolean forceWrap) {
+    public Object wrapObjectInXMLRoot(Object object, String elementNamespaceUri, String elementLocalName, String elementPrefix, boolean forceWrap, boolean isNamespaceAware, XMLUnmarshaller xmlUnmarshaller) {
 
-        if (forceWrap || shouldWrapObject(object, elementNamespaceUri, elementLocalName, elementPrefix)) {
+        if (forceWrap || shouldWrapObject(object, elementNamespaceUri, elementLocalName, elementPrefix, isNamespaceAware)) {
             // if the DOMRecord element != descriptor's default 
-            // root element, create an XMLRoot, populate and return it
-            XMLRoot xmlRoot = new XMLRoot();
+            // root element, create an Root, populate and return it
+            Root xmlRoot = new XMLRoot();
             xmlRoot.setLocalName(elementLocalName);
             xmlRoot.setNamespaceURI(elementNamespaceUri);
             xmlRoot.setObject(object);
+            
+            setDeclaredTypeOnXMLRoot(xmlRoot, elementNamespaceUri, elementLocalName, isNamespaceAware, xmlUnmarshaller);
+            
             return xmlRoot;
         }
         return object;
     }
 
-    public Object wrapObjectInXMLRoot(Object object, String elementNamespaceUri, String elementLocalName, String elementPrefix, String encoding, String version, boolean forceWrap) {
-        if (forceWrap || shouldWrapObject(object, elementNamespaceUri, elementLocalName, elementPrefix)) {
+    /**
+     * INTERNAL:
+     * @return
+     */
+    public Object wrapObjectInXMLRoot(Object object, String elementNamespaceUri, String elementLocalName, String elementPrefix, String encoding, String version, boolean forceWrap, boolean isNamespaceAware, XMLUnmarshaller unmarshaller) {
+        if (forceWrap || shouldWrapObject(object, elementNamespaceUri, elementLocalName, elementPrefix, isNamespaceAware)) {
             // if the DOMRecord element != descriptor's default 
             // root element, create an XMLRoot, populate and return it
-            XMLRoot xmlRoot = new XMLRoot();
+            Root xmlRoot = new XMLRoot();
             xmlRoot.setLocalName(elementLocalName);
             xmlRoot.setNamespaceURI(elementNamespaceUri);
             xmlRoot.setObject(object);
             xmlRoot.setEncoding(encoding);
-            xmlRoot.setVersion(version);                        
+            xmlRoot.setVersion(version);  
+            setDeclaredTypeOnXMLRoot(xmlRoot, elementNamespaceUri, elementLocalName, isNamespaceAware, unmarshaller);
+
             return xmlRoot;
         }
         return object;
 
     }
     
-    public boolean shouldWrapObject(Object object, String elementNamespaceUri, String elementLocalName, String elementPrefix) {
+    private void setDeclaredTypeOnXMLRoot(Root xmlRoot, String elementNamespaceUri, String elementLocalName, boolean isNamespaceAware, Unmarshaller unmarshaller){
+    	XPathQName xpathQName = new XPathQName(elementNamespaceUri, elementLocalName, isNamespaceAware);
+    	Descriptor desc = unmarshaller.getContext().getDescriptor(xpathQName);
+    	if(desc != null){
+    		xmlRoot.setDeclaredType(desc.getJavaClass());
+    	}
+    }
+    
+    /**
+     * INTERNAL:
+     * @return
+     */
+    public boolean shouldWrapObject(Object object, String elementNamespaceUri, String elementLocalName, String elementPrefix, boolean isNamespaceAware) {
         if(resultAlwaysXMLRoot){
             return true;
         }
@@ -747,13 +922,19 @@ public class XMLDescriptor extends ClassDescriptor {
             // resolve namespace prefix if one exists
             String defaultRootName = defaultRootField.getXPathFragment().getLocalName();
             String defaultRootNamespaceUri = defaultRootField.getXPathFragment().getNamespaceURI();
-
             // if the DOMRecord element == descriptor's default 
             // root element, return the object as per usual
-            if ((((defaultRootNamespaceUri == null) && (elementNamespaceUri == null)) || ((defaultRootNamespaceUri == null) && (elementNamespaceUri.length() == 0)) || ((elementNamespaceUri == null) && (defaultRootNamespaceUri.length() == 0)) || (((defaultRootNamespaceUri != null) && (elementNamespaceUri != null)) && (defaultRootNamespaceUri
-                    .equals(elementNamespaceUri))))
-                    && (defaultRootName.equals(elementLocalName))) {
-                return false;
+
+            if(isNamespaceAware){
+            	if ((((defaultRootNamespaceUri == null) && (elementNamespaceUri == null)) || ((defaultRootNamespaceUri == null) && (elementNamespaceUri.length() == 0)) || ((elementNamespaceUri == null) && (defaultRootNamespaceUri.length() == 0)) || (((defaultRootNamespaceUri != null) && (elementNamespaceUri != null)) && (defaultRootNamespaceUri
+                        .equals(elementNamespaceUri))))
+                        && (defaultRootName.equals(elementLocalName))) {
+                    return false;
+                }
+            }else{
+                if (defaultRootName.equals(elementLocalName)) {
+                    return false;
+                }
             }
         }
         return true;
@@ -771,6 +952,9 @@ public class XMLDescriptor extends ClassDescriptor {
             setDefaultRootElementField((XMLField) null);
             return false;
         }
+        if(getDefaultRootElementField() != null && newDefaultRootElement.equals(getDefaultRootElementField().getName())){
+        	return false;
+        }
         // create the root element xml field based on default root element name
         setDefaultRootElementField(new XMLField(newDefaultRootElement));
         return true;
@@ -828,6 +1012,15 @@ public class XMLDescriptor extends ClassDescriptor {
     public void setResultAlwaysXMLRoot(boolean resultAlwaysXMLRoot) {
         this.resultAlwaysXMLRoot = resultAlwaysXMLRoot;
     }
+    
+    /**
+     * INTERNAL:
+     * Returns true if any of the mappings on this descriptor are key-based reference
+     * mappings.
+     */
+    public boolean hasReferenceMappings() {
+        return this.hasReferenceMappings;
+    }
 
     @Override
     public DatabaseField getTypedField(DatabaseField field) {
@@ -885,4 +1078,37 @@ public class XMLDescriptor extends ClassDescriptor {
         return null; 
     }
 
+    /**
+     * INTERNAL:
+     * Returns this Descriptor's location accessor, if one is defined.
+     */
+    public AttributeAccessor getLocationAccessor() {
+        return locationAccessor;
+    }
+
+    /**
+     * INTERNAL:
+     * Set this Descriptor's location accessor.
+     */
+    public void setLocationAccessor(AttributeAccessor value) {
+        this.locationAccessor = value;
+    }
+    
+    
+    /**
+     * INTERNAL:
+     * Convert all the class-name-based settings in this Descriptor to actual class-based
+     * settings. This method is used when converting a project that has been built
+     * with class names to a project with classes.
+     * @param classLoader 
+     */
+    @Override
+    public void convertClassNamesToClasses(ClassLoader classLoader){
+        super.convertClassNamesToClasses(classLoader);
+        if(this.attributeGroups != null) {
+            for(AttributeGroup next:attributeGroups.values()) {
+                next.convertClassNamesToClasses(classLoader);
+            }
+        }
+    }
 }
\ No newline at end of file
