@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -21,26 +21,33 @@ import java.net.URL;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.Properties;
+
 import javax.xml.transform.Source;
 import javax.xml.validation.Schema;
 
+import org.eclipse.persistence.core.sessions.CoreSession;
+import org.eclipse.persistence.exceptions.EclipseLinkException;
 import org.eclipse.persistence.exceptions.XMLMarshalException;
-import org.eclipse.persistence.internal.oxm.ReferenceResolver;
 import org.eclipse.persistence.internal.oxm.StrBuffer;
+import org.eclipse.persistence.internal.oxm.Unmarshaller;
 import org.eclipse.persistence.internal.oxm.record.PlatformUnmarshaller;
+import org.eclipse.persistence.internal.oxm.record.UnmarshalRecord;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
+import org.eclipse.persistence.oxm.attachment.XMLAttachmentUnmarshaller;
 import org.eclipse.persistence.oxm.platform.XMLPlatform;
+import org.eclipse.persistence.oxm.record.XMLRootRecord;
 import org.eclipse.persistence.oxm.schema.XMLSchemaReference;
 import org.eclipse.persistence.platform.xml.XMLParser;
-import org.eclipse.persistence.sessions.DatabaseSession;
 import org.w3c.dom.Node;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.ErrorHandler;
 import org.xml.sax.InputSource;
+import org.xml.sax.SAXException;
+import org.xml.sax.SAXParseException;
 import org.xml.sax.XMLReader;
-import org.eclipse.persistence.oxm.attachment.*;
 
 /**
  * <p>Class used to unmarshal XML to objects.
@@ -68,7 +75,7 @@ import org.eclipse.persistence.oxm.attachment.*;
  *
  * @see org.eclipse.persistence.oxm.XMLContext
  */
-public class XMLUnmarshaller {
+public class XMLUnmarshaller extends Unmarshaller<AbstractSession, XMLContext, XMLDescriptor, IDResolver, MediaType, XMLRoot, XMLUnmarshallerHandler> implements Cloneable {
     public static final int NONVALIDATING = XMLParser.NONVALIDATING;
     public static final int SCHEMA_VALIDATION = XMLParser.SCHEMA_VALIDATION;
     public static final int DTD_VALIDATION = XMLParser.DTD_VALIDATION;
@@ -90,8 +97,33 @@ public class XMLUnmarshaller {
     private static Constructor xmlStreamReaderInputSourceConstructor;
     private static Constructor xmlEventReaderReaderConstructor;
     private static Constructor xmlEventReaderInputSourceConstructor;
+    
+   	/**
+     * @since EclipseLink 2.4
+     */
+    private static final ErrorHandler DEFAULT_ERROR_HANDLER = new ErrorHandler() {
+
+        public void warning(SAXParseException exception)
+                throws SAXException {
+            if(exception.getException() instanceof EclipseLinkException) {
+                throw (EclipseLinkException) exception.getCause();
+            }
+        }
+
+        public void error(SAXParseException exception) throws SAXException {
+            if(exception.getException() instanceof EclipseLinkException) {
+                throw exception;
+            }
+        }
+
+        public void fatalError(SAXParseException exception)
+                throws SAXException {
+            throw exception;
+            
+        }
+
+    };
 
-    private XMLContext xmlContext;
     private XMLUnmarshallerHandler xmlUnmarshallerHandler;
     private PlatformUnmarshaller platformUnmarshaller;
     private boolean schemasAreInitialized;
@@ -101,6 +133,16 @@ public class XMLUnmarshaller {
 
     private Class unmappedContentHandlerClass;
     private StrBuffer stringBuffer;
+    private MediaType mediaType = MediaType.APPLICATION_XML;
+    private IDResolver idResolver;
+    private String valueWrapper = XMLConstants.VALUE_WRAPPER;
+    private char namespaceSeparator = XMLConstants.DOT;
+    private String attributePrefix;
+    private boolean includeRoot = true;
+    private NamespaceResolver namespaceResolver;    
+    private boolean autoDetectMediaType = false;
+    private Object unmarshalAttributeGroup;
+    private boolean wrapperAsCollectionName = false;
 
     static {
         try {
@@ -131,21 +173,22 @@ public class XMLUnmarshaller {
     }
 
     protected XMLUnmarshaller(XMLContext xmlContext, Map<String, Boolean> parserFeatures) {
-        setXMLContext(xmlContext);
+        super(xmlContext);
         stringBuffer = new StrBuffer();
         initialize(parserFeatures);
+        setErrorHandler(DEFAULT_ERROR_HANDLER);
     }
 
     private void initialize(Map<String, Boolean> parserFeatures) {
-        DatabaseSession session = xmlContext.getSession(0);
-        XMLPlatform xmlPlatform = (XMLPlatform)session.getDatasourceLogin().getDatasourcePlatform();
-        platformUnmarshaller = xmlPlatform.newPlatformUnmarshaller(this, parserFeatures);
-        platformUnmarshaller.setWhitespacePreserving(false);
-
-        // Waiting on XDK to fix bug #3697940 to enable this code
-        //initializeSchemas();
-        setValidationMode(NONVALIDATING);
-    }
+	    CoreSession session = context.getSession();
+	    XMLPlatform xmlPlatform = (XMLPlatform)session.getDatasourceLogin().getDatasourcePlatform();
+	    platformUnmarshaller = xmlPlatform.newPlatformUnmarshaller(this, parserFeatures);
+	    platformUnmarshaller.setWhitespacePreserving(false);
+	
+	    // Waiting on XDK to fix bug #3697940 to enable this code
+	    //initializeSchemas();
+	    setValidationMode(NONVALIDATING);
+	}
 
     private void initializeSchemas() {
         if (!schemasAreInitialized) {
@@ -153,9 +196,9 @@ public class XMLUnmarshaller {
             Iterator xmlDescriptors;
             XMLDescriptor xmlDescriptor;
             XMLSchemaReference xmlSchemaReference;
-            int numberOfSessions = xmlContext.getSessions().size();
+            int numberOfSessions = context.getSessions().size();
             for (int x = 0; x < numberOfSessions; x++) {
-                xmlDescriptors = ((DatabaseSession)xmlContext.getSessions().get(x)).getDescriptors().values().iterator();
+                xmlDescriptors = ((CoreSession)context.getSessions().get(x)).getDescriptors().values().iterator();
                 URL schemaURL;
                 while (xmlDescriptors.hasNext()) {
                     xmlDescriptor = (XMLDescriptor)xmlDescriptors.next();
@@ -173,20 +216,46 @@ public class XMLUnmarshaller {
             schemasAreInitialized = true;
         }
     }
+    
+    /**
+     * Set the MediaType for this xmlUnmarshaller.
+     * See org.eclipse.persistence.oxm.MediaType for the media types supported by EclipseLink MOXy
+     * @since 2.4
+     * @param mediaType
+     */
+    public void setMediaType(MediaType mediaType) {
+    	if(this.mediaType != mediaType){
+    		this.mediaType = mediaType;
+            if(platformUnmarshaller != null){
+            	platformUnmarshaller.mediaTypeChanged();
+            }	
+    	}    	
+    }
 
+    /**
+     * Get the MediaType for this xmlUnmarshaller.
+     * See org.eclipse.persistence.oxm.MediaType for the media types supported by EclipseLink MOXy
+     * If not set the default is MediaType.APPLICATION_XML
+     * @since 2.4
+     * @return MediaType
+     */
+    public MediaType getMediaType(){
+    	return mediaType;
+    }
+    
     /**
      * Return the instance of XMLContext that was used to create this instance
      * of XMLUnmarshaller.
      */
     public XMLContext getXMLContext() {
-        return xmlContext;
+        return getContext();
     }
 
     /** 
      * Set the XMLContext used by this instance of XMLUnmarshaller.
      */
     public void setXMLContext(XMLContext value) {
-        xmlContext =  value;
+        context =  value;
     }
     
     /**
@@ -198,17 +267,6 @@ public class XMLUnmarshaller {
         return platformUnmarshaller.getValidationMode();
     }
 
-    /**
-     * INTERNAL
-     * @param unitOfWork
-     */
-    public void resolveReferences(AbstractSession unitOfWork) {
-        ReferenceResolver resolver = ReferenceResolver.getInstance(unitOfWork);
-        if (resolver != null) {
-            resolver.resolveReferences(unitOfWork);
-        }
-    }
-
     /**
     * Set the validation mode.
     * This method sets the validation mode of the parser to one of the 3 types:
@@ -252,7 +310,11 @@ public class XMLUnmarshaller {
      * @param errorHandler the ErrorHandler to set on this XMLUnmarshaller
      */
     public void setErrorHandler(ErrorHandler errorHandler) {
-        platformUnmarshaller.setErrorHandler(errorHandler);
+        if(null == errorHandler) {
+            platformUnmarshaller.setErrorHandler(DEFAULT_ERROR_HANDLER);
+        } else {
+            platformUnmarshaller.setErrorHandler(errorHandler);
+        }
     }
 
     public XMLUnmarshalListener getUnmarshalListener() {
@@ -571,7 +633,7 @@ public class XMLUnmarshaller {
         if ((null == source) || (null == clazz)) {
             throw XMLMarshalException.nullArgumentException();
         }
-        if (source.getClass() == this.staxSourceClass) {
+        if (source.getClass() == this.staxSourceClass) {        	
             try {
                 Object xmlStreamReader = PrivilegedAccessHelper.invokeMethod(this.staxSourceGetStreamReaderMethod, source);
                 if(xmlStreamReader != null) {
@@ -631,5 +693,218 @@ public class XMLUnmarshaller {
     public Schema getSchema() {
         return this.platformUnmarshaller.getSchema();
     }
+    
+    /**
+     * Value that will be used to prefix attributes.  
+     * Ignored unmarshalling XML.   
+     * @return
+     * @since 2.4
+     */
+    public String getAttributePrefix() {
+        return attributePrefix;
+    }
+    
+    /**
+     * Value that will be used to prefix attributes.  
+     * Ignored unmarshalling XML.
+     * @since 2.4	 
+     */
+    public void setAttributePrefix(String attributePrefix) {
+        this.attributePrefix = attributePrefix;
+    }
+    
+    /**
+     * Name of the property to marshal/unmarshal as a wrapper on the text() mappings   
+     * Ignored unmarshalling XML.  
+     * @since 2.4	 
+     */	
+    public String getValueWrapper() {
+        return valueWrapper;
+    }
 
-}
+    /**
+     * Name of the property to marshal/unmarshal as a wrapper on the text() mappings   
+     * Ignored unmarshalling XML.  
+     * @since 2.4	 
+     */
+    public void setValueWrapper(String valueWrapper) {
+        this.valueWrapper = valueWrapper;
+    }
+        
+    /**
+     * Get the namespace separator used during unmarshal operations.
+     * If mediaType is application/json '.' is the default
+     * Ignored unmarshalling XML.   
+     * @since 2.4
+     */
+    public char getNamespaceSeparator() {    	
+        return namespaceSeparator;
+    }
+
+    /**
+     * Set the namespace separator used during unmarshal operations.
+     * If mediaType is application/json '.' is the default
+     * Ignored unmarshalling XML.   
+     * @since 2.4
+     */
+	public void setNamespaceSeparator(char namespaceSeparator) {
+		this.namespaceSeparator = namespaceSeparator;
+	}
+    
+    /**
+     * Determine if the @XMLRootElement should be marshalled when present.  
+     * Ignored unmarshalling XML.   
+     * @return
+     * @since 2.4
+     */
+    public boolean isIncludeRoot() {
+            return includeRoot;
+    }
+
+    /**
+     * Determine if the @XMLRootElement should be marshalled when present.  
+     * Ignored unmarshalling XML.   
+     * @return
+     * @since 2.4
+     */
+    public void setIncludeRoot(boolean includeRoot) {
+         this.includeRoot = includeRoot;
+    }
+    
+    /**
+     * Return if this XMLUnmarshaller should try to automatically determine
+     * the MediaType of the document (instead of using the MediaType set
+     * by setMediaType)
+     */
+    public boolean isAutoDetectMediaType() {
+		return autoDetectMediaType;
+	}
+
+    /**
+     * Set if this XMLUnmarshaller should try to automatically determine
+     * the MediaType of the document (instead of using the MediaType set
+     * by setMediaType)
+     */
+	public void setAutoDetectMediaType(boolean autoDetectMediaType) {
+		this.autoDetectMediaType = autoDetectMediaType;
+	}
+
+    
+    
+    /**
+     * Name of the NamespaceResolver to be used during unmarshal
+     * Ignored unmarshalling XML.  
+     * @since 2.4	 
+     */	
+    public NamespaceResolver getNamespaceResolver() {
+        return namespaceResolver;
+    }
+
+    /**
+     * Get the NamespaceResolver to be used during unmarshal
+     * Ignored unmarshalling XML.  
+     * @since 2.4	 
+     */
+    public void setNamespaceResolver(NamespaceResolver namespaceResolver) {
+        this.namespaceResolver = namespaceResolver;
+    }
+    
+    /**
+     * @since 2.4.2
+     */
+    @Override
+    public boolean isWrapperAsCollectionName() {
+        return wrapperAsCollectionName;
+    }
+
+    /**
+     * @since 2.4.2
+     */
+    public void setWrapperAsCollectionName(boolean wrapperAsCollectionName) {
+        this.wrapperAsCollectionName = wrapperAsCollectionName;
+    }
+
+    @Override
+    public XMLUnmarshaller clone() {
+        XMLUnmarshaller clone = new XMLUnmarshaller(context);
+        clone.setAttachmentUnmarshaller(attachmentUnmarshaller);
+        clone.setEntityResolver(getEntityResolver());
+        clone.setErrorHandler(getErrorHandler());
+        for(Entry entry : unmarshalProperties.entrySet()) {
+            clone.getProperties().put(entry.getKey(), entry.getValue());
+        }
+        clone.setResultAlwaysXMLRoot(platformUnmarshaller.isResultAlwaysXMLRoot());
+        try {
+            Schema schema = getSchema();
+            if(null != schema) {
+                clone.setSchema(schema);
+            }
+        } catch(UnsupportedOperationException e) {}
+        clone.setUnmappedContentHandlerClass(unmappedContentHandlerClass);
+        clone.setUnmarshalListener(unmarshalListener);
+        clone.setValidationMode(getValidationMode());
+        return clone;
+    }
+
+    /**
+     * Return this Unmarshaller's custom IDResolver.
+     * @see IDResolver
+     * @since 2.3.3
+     * @return the custom IDResolver, or null if one has not been specified.
+     */
+    public IDResolver getIDResolver() {
+        return idResolver;
+    }
+
+    /**
+     * Set this Unmarshaller's custom IDResolver.
+     * @see IDResolver
+     * @since 2.3.3
+     */
+    public void setIDResolver(IDResolver idResolver) {
+        this.idResolver = idResolver;
+    }
+
+    /**
+     * INTERNAL
+     * @since 2.5.0
+     */
+    public XMLRoot createRoot() {
+        return new XMLRoot();
+    }
+
+    /**
+     * INTERNAL
+     * @since 2.5.0
+     */
+    @Override
+    public UnmarshalRecord createRootUnmarshalRecord(Class clazz) {
+        XMLRootRecord rootUnmarshalRecord = new XMLRootRecord(clazz);
+        rootUnmarshalRecord.setSession((AbstractSession) context.getSession());
+        return rootUnmarshalRecord;
+    }
+
+    /**
+     * INTERNAL
+     * @since 2.5.0
+     */
+    @Override
+    public UnmarshalRecord createUnmarshalRecord(XMLDescriptor xmlDescriptor, AbstractSession session) {
+        org.eclipse.persistence.oxm.record.UnmarshalRecord wrapper = (org.eclipse.persistence.oxm.record.UnmarshalRecord) xmlDescriptor.getObjectBuilder().createRecord((AbstractSession) session);
+        return wrapper.getUnmarshalRecord();
+    }
+
+    /**
+     * INTERNAL:
+     * Returns the AttributeGroup or the name of the AttributeGroup to be used to 
+     * unmarshal. 
+     */
+    public Object getUnmarshalAttributeGroup() {
+        return this.unmarshalAttributeGroup;
+    }
+    
+    public void setUnmarshalAttributeGroup(Object attributeGroup) {
+        this.unmarshalAttributeGroup = attributeGroup;
+    }
+
+}
\ No newline at end of file
