@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,7 +12,17 @@
  ******************************************************************************/ 
 package org.eclipse.persistence.oxm.mappings;
 
-public interface MimeTypePolicy {
+/**
+ * <p>Purpose: </p> A MimeTypePolicy is used in conjunction with an XMLBinaryData/CollectionMapping
+ * in order to allow customisation of the mime type for a specific property at runtime rather than
+ * at design time. By default, a FixedMimeTypePolicy is used.
+ * 
+ * @see XMLBinaryDataMapping
+ * @see XMLBinaryCollectionMapping
+ * @see FixedMimeTypePolicy
+ *
+ */
+public interface MimeTypePolicy extends org.eclipse.persistence.internal.oxm.mappings.MimeTypePolicy {
 	/**
 	 * return a MIME type string
 	 * @param anObject - fixed non-dynamic implementors will ignore this parameter
