@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -17,14 +17,19 @@ import java.util.Vector;
 import javax.activation.DataHandler;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
+import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.exceptions.XMLMarshalException;
 import org.eclipse.persistence.internal.helper.ClassConstants;
+import org.eclipse.persistence.internal.helper.DatabaseField;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.oxm.XMLBinaryDataHelper;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
+import org.eclipse.persistence.internal.oxm.mappings.BinaryDataCollectionMapping;
 import org.eclipse.persistence.internal.queries.ContainerPolicy;
 import org.eclipse.persistence.internal.queries.JoinedAttributeManager;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
+import org.eclipse.persistence.mappings.AttributeAccessor;
 import org.eclipse.persistence.mappings.converters.Converter;
 import org.eclipse.persistence.oxm.NamespaceResolver;
 import org.eclipse.persistence.oxm.XMLConstants;
@@ -32,10 +37,12 @@ import org.eclipse.persistence.oxm.XMLDescriptor;
 import org.eclipse.persistence.oxm.XMLField;
 import org.eclipse.persistence.oxm.XMLMarshaller;
 import org.eclipse.persistence.oxm.XMLUnmarshaller;
-import org.eclipse.persistence.oxm.mappings.converters.XMLConverter;
+import org.eclipse.persistence.oxm.mappings.nullpolicy.AbstractNullPolicy;
+import org.eclipse.persistence.oxm.mappings.nullpolicy.XMLNullRepresentationType;
 import org.eclipse.persistence.oxm.record.DOMRecord;
 import org.eclipse.persistence.oxm.record.XMLRecord;
 import org.eclipse.persistence.queries.ObjectBuildingQuery;
+import org.eclipse.persistence.sessions.Session;
 
 /**
  * <p><b>Purpose:</b>Provide a mapping for a collection of binary data values that can be treated
@@ -72,7 +79,7 @@ import org.eclipse.persistence.queries.ObjectBuildingQuery;
  *  @see org.eclipse.persistence.oxm.mappings.MimeTypePolicy
  *  @since   TopLink 11.1.1.0.0g
  */
-public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollectionMapping {
+public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollectionMapping implements BinaryDataCollectionMapping<AbstractSession, AttributeAccessor, ContainerPolicy, Converter, ClassDescriptor, DatabaseField, XMLMarshaller, MimeTypePolicy, Session, XMLUnmarshaller, XMLRecord> {
     private boolean shouldInlineBinaryData;
     private MimeTypePolicy mimeTypePolicy;
     private boolean isSwaRef;
@@ -197,10 +204,25 @@ public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollection
         for (Object iter = cp.iteratorFor(attributeValue); cp.hasNext(iter);) {
             Object element = cp.next(iter, session);
             element = getValueToWrite(element, object, record, field, includeField, session);
-            if(element.getClass() == ClassConstants.ABYTE) {
-                inline = true;
-            }
-            if (element != null) {
+            if(element == null){
+                AbstractNullPolicy nullPolicy = getNullPolicy();
+                if (nullPolicy == null) {
+                    elements.addElement(null);
+                } else {
+                    if (nullPolicy.getMarshalNullRepresentation() == XMLNullRepresentationType.XSI_NIL) {
+                        elements.addElement(XMLRecord.NIL);
+                    } else if (nullPolicy.getMarshalNullRepresentation() == XMLNullRepresentationType.ABSENT_NODE) {
+                        // Do nothing
+                    } else {
+                        elements.addElement(XMLConstants.EMPTY_STRING);
+                    }
+                }
+            }else{
+            
+                
+                if(element.getClass() == ClassConstants.ABYTE) {
+                    inline = true;
+                }
                 elements.addElement(element);
             }
         }
@@ -217,17 +239,13 @@ public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollection
 
     public Object getValueToWrite(Object value, Object parent, XMLRecord record, XMLField field, XMLField includeField, AbstractSession session) {
         XMLMarshaller marshaller = record.getMarshaller();
-        Object element = value;
+        Object element = convertObjectValueToDataValue(value, session, record.getMarshaller());
         boolean isAttribute = ((XMLField) getField()).getLastXPathFragment().isAttribute();
-        if (getValueConverter() != null) {
-            Converter converter = getValueConverter();
-            if (converter instanceof XMLConverter) {
-                element = ((XMLConverter) converter).convertObjectValueToDataValue(element, session, record.getMarshaller());
-            } else {
-                element = converter.convertObjectValueToDataValue(element, session);
-            }
-        }
 
+        if(element == null){
+            return null;
+        }
+        
         if (isAttribute) {
             if (isSwaRef() && (marshaller.getAttachmentMarshaller() != null)) {
                 //should be a DataHandler here
@@ -287,7 +305,7 @@ public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollection
                         Object obj = include.getIndicatingNoEntry(incField);
                         if (obj != null && obj instanceof DOMRecord) {
                             if (((DOMRecord) obj).getDOM().getNodeType() == Node.ELEMENT_NODE) {
-                                ((Element) ((DOMRecord) obj).getDOM()).setAttributeNS(XMLConstants.XMLNS_URL, XMLConstants.XMLNS + XMLConstants.COLON + XMLConstants.XOP_PREFIX, XMLConstants.XOP_URL);
+                                ((Element) ((DOMRecord) obj).getDOM()).setAttributeNS(javax.xml.XMLConstants.XMLNS_ATTRIBUTE_NS_URI, javax.xml.XMLConstants.XMLNS_ATTRIBUTE + XMLConstants.COLON + XMLConstants.XOP_PREFIX, XMLConstants.XOP_URL);
                             }
                         }
                     }
@@ -351,7 +369,7 @@ public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollection
         record.add(field, valueToWrite);
     }
 
-    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, AbstractSession executionSession) {
+    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, CacheKey cacheKey, AbstractSession executionSession, boolean isTargetProtected, Boolean[] wasCacheUsed) {
         ContainerPolicy cp = this.getContainerPolicy();
 
         Object fieldValue = row.getValues(this.getField());
@@ -403,59 +421,56 @@ public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollection
             } else {
                 //this was an element, so do the XOP/SWAREF/Inline binary cases for an element
                 XMLRecord record = (XMLRecord) element;
+                if (getNullPolicy().valueIsNull((Element) record.getDOM())) {
+                    fieldValue = null;
+                }
+                else{
                     record.setSession(executionSession);
 
-                if ((unmarshaller.getAttachmentUnmarshaller() != null) && unmarshaller.getAttachmentUnmarshaller().isXOPPackage() && !this.isSwaRef() && !this.shouldInlineBinaryData()) {
-                    //look for the include element:
-                    String xpath = XMLConstants.EMPTY_STRING;
-
-                    //  need a prefix for XOP
-                    String prefix = null;
-                    NamespaceResolver descriptorResolver = ((XMLDescriptor) getDescriptor()).getNamespaceResolver();
-                    if (descriptorResolver != null) {
-                        prefix = descriptorResolver.resolveNamespaceURI(XMLConstants.XOP_URL);
-                    }
-                    if (prefix == null) {
-                        prefix = XMLConstants.XOP_PREFIX;
-                    }
-                    NamespaceResolver tempResolver = new NamespaceResolver();
-                    tempResolver.put(prefix, XMLConstants.XOP_URL);
-                    xpath = prefix + XMLConstants.COLON + INCLUDE + "/@href";
-                    XMLField field = new XMLField(xpath);
-                    field.setNamespaceResolver(tempResolver);
-                    String includeValue = (String) record.get(field);
-                    if (element != null && includeValue != null) {
-                    	if ((getAttributeElementClass() == ClassConstants.ABYTE) || (getAttributeElementClass() == ClassConstants.APBYTE)) {
-                            fieldValue = unmarshaller.getAttachmentUnmarshaller().getAttachmentAsByteArray(includeValue);
+                    if ((unmarshaller.getAttachmentUnmarshaller() != null) && unmarshaller.getAttachmentUnmarshaller().isXOPPackage() && !this.isSwaRef() && !this.shouldInlineBinaryData()) {
+                        //look for the include element:
+                        String xpath = XMLConstants.EMPTY_STRING;
+    
+                        //  need a prefix for XOP
+                        String prefix = null;
+                        NamespaceResolver descriptorResolver = ((XMLDescriptor) getDescriptor()).getNamespaceResolver();
+                        if (descriptorResolver != null) {
+                            prefix = descriptorResolver.resolveNamespaceURI(XMLConstants.XOP_URL);
+                        }
+                        if (prefix == null) {
+                            prefix = XMLConstants.XOP_PREFIX;
+                        }
+                        NamespaceResolver tempResolver = new NamespaceResolver();
+                        tempResolver.put(prefix, XMLConstants.XOP_URL);
+                        xpath = prefix + XMLConstants.COLON + INCLUDE + "/@href";
+                        XMLField field = new XMLField(xpath);
+                        field.setNamespaceResolver(tempResolver);
+                        String includeValue = (String) record.get(field);
+                        if (element != null && includeValue != null) {
+                        	if ((getAttributeElementClass() == ClassConstants.ABYTE) || (getAttributeElementClass() == ClassConstants.APBYTE)) {
+                                fieldValue = unmarshaller.getAttachmentUnmarshaller().getAttachmentAsByteArray(includeValue);
+                            } else {
+                                fieldValue = unmarshaller.getAttachmentUnmarshaller().getAttachmentAsDataHandler(includeValue);
+                            }
                         } else {
-                            fieldValue = unmarshaller.getAttachmentUnmarshaller().getAttachmentAsDataHandler(includeValue);
+                            //If we didn't find the Include element, check for inline
+                            fieldValue = record.get(XMLConstants.TEXT);
+                            //should be a base64 string
+                            fieldValue = ((XMLConversionManager) executionSession.getDatasourcePlatform().getConversionManager()).convertSchemaBase64ToByteArray(fieldValue);
+                        }
+                    } else if ((unmarshaller.getAttachmentUnmarshaller() != null) && isSwaRef()) {
+                        String refValue = (String) record.get(XMLConstants.TEXT);
+                        if (refValue != null) {
+                            fieldValue = unmarshaller.getAttachmentUnmarshaller().getAttachmentAsDataHandler(refValue);
                         }
                     } else {
-                        //If we didn't find the Include element, check for inline
                         fieldValue = record.get(XMLConstants.TEXT);
                         //should be a base64 string
                         fieldValue = ((XMLConversionManager) executionSession.getDatasourcePlatform().getConversionManager()).convertSchemaBase64ToByteArray(fieldValue);
                     }
-                } else if ((unmarshaller.getAttachmentUnmarshaller() != null) && isSwaRef()) {
-                    String refValue = (String) record.get(XMLConstants.TEXT);
-                    if (refValue != null) {
-                        fieldValue = unmarshaller.getAttachmentUnmarshaller().getAttachmentAsDataHandler(refValue);
-                    }
-                } else {
-                    fieldValue = record.get(XMLConstants.TEXT);
-                    //should be a base64 string
-                    fieldValue = ((XMLConversionManager) executionSession.getDatasourcePlatform().getConversionManager()).convertSchemaBase64ToByteArray(fieldValue);
                 }
             }
-            Object attributeValue = fieldValue;
-            if (getValueConverter() != null) {
-                if (getValueConverter() instanceof XMLConverter) {
-                    attributeValue = ((XMLConverter) getValueConverter()).convertDataValueToObjectValue(fieldValue, executionSession, unmarshaller);
-                } else {
-                    attributeValue = getValueConverter().convertDataValueToObjectValue(fieldValue, executionSession);
-                }
-            }
-
+            Object attributeValue = convertDataValueToObjectValue(fieldValue, executionSession, unmarshaller);
             cp.addInto(attributeValue, result, query.getSession());
         }
         return result;
@@ -472,5 +487,28 @@ public class XMLBinaryDataCollectionMapping extends XMLCompositeDirectCollection
     public Class getCollectionContentType() {
     	return getAttributeElementClass();
     }
+    
+    /**
+     * PUBLIC:
+     * Set the class each element in the object's
+     * collection should be converted to, before the collection
+     * is inserted into the object.
+     * This is optional - if left null, the elements will be added
+     * to the object's collection unconverted.
+     */
+    @Override
+    public void setAttributeElementClass(Class attributeElementClass) {
+        super.setAttributeElementClass(attributeElementClass);
+        this.collectionContentType = attributeElementClass;
+    }
+    
+    @Override
+    public Class getAttributeElementClass() {
+        Class elementClass = super.getAttributeElementClass();
+        if(elementClass == null) {
+            return this.collectionContentType;
+        }
+        return elementClass;
+    }
 
 }
\ No newline at end of file
