@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -17,13 +17,20 @@ import javax.activation.DataHandler;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 
+import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.exceptions.XMLMarshalException;
 import org.eclipse.persistence.internal.helper.ClassConstants;
+import org.eclipse.persistence.internal.helper.DatabaseField;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.oxm.XMLBinaryDataHelper;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
+import org.eclipse.persistence.internal.oxm.mappings.BinaryDataMapping;
+import org.eclipse.persistence.internal.oxm.mappings.Field;
+import org.eclipse.persistence.internal.queries.ContainerPolicy;
 import org.eclipse.persistence.internal.queries.JoinedAttributeManager;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
+import org.eclipse.persistence.mappings.AttributeAccessor;
 import org.eclipse.persistence.mappings.converters.Converter;
 import org.eclipse.persistence.oxm.NamespaceResolver;
 import org.eclipse.persistence.oxm.XMLConstants;
@@ -31,10 +38,10 @@ import org.eclipse.persistence.oxm.XMLDescriptor;
 import org.eclipse.persistence.oxm.XMLField;
 import org.eclipse.persistence.oxm.XMLMarshaller;
 import org.eclipse.persistence.oxm.XMLUnmarshaller;
-import org.eclipse.persistence.oxm.mappings.converters.XMLConverter;
 import org.eclipse.persistence.oxm.record.DOMRecord;
 import org.eclipse.persistence.oxm.record.XMLRecord;
 import org.eclipse.persistence.queries.ObjectBuildingQuery;
+import org.eclipse.persistence.sessions.Session;
 
 /**
  * <p><b>Purpose:</b>Provide a mapping for binary data that can be treated as either inline or as
@@ -71,7 +78,7 @@ import org.eclipse.persistence.queries.ObjectBuildingQuery;
  *  @see org.eclipse.persistence.oxm.mappings.MimeTypePolicy
  *  @since   TopLink 11.1.1.0.0g
  */
-public class XMLBinaryDataMapping extends XMLDirectMapping {
+public class XMLBinaryDataMapping extends XMLDirectMapping implements BinaryDataMapping<AbstractSession, AttributeAccessor, ContainerPolicy, Converter, ClassDescriptor, DatabaseField, XMLMarshaller, MimeTypePolicy, Session, XMLUnmarshaller, XMLRecord> {
     private boolean shouldInlineBinaryData;
     private MimeTypePolicy mimeTypePolicy;
     private boolean isSwaRef;
@@ -150,21 +157,19 @@ public class XMLBinaryDataMapping extends XMLDirectMapping {
     public void writeFromObjectIntoRow(Object object, AbstractRecord row, AbstractSession session, WriteType writeType) {
         Object attributeValue = getAttributeValueFromObject(object);
         if (attributeValue == null) {
-            return;
+            XMLField field = (XMLField) getField();
+
+            if(getNullPolicy() != null && !field.getXPathFragment().isSelfFragment()) {
+               getNullPolicy().directMarshal((Field) this.getField(), (XMLRecord) row, object);
+            }
+            return;            
         }
         writeSingleValue(attributeValue, object, (XMLRecord) row, session);
     }
 
     public void writeSingleValue(Object attributeValue, Object parent, XMLRecord record, AbstractSession session) {
         XMLMarshaller marshaller = record.getMarshaller();
-        if (getConverter() != null) {
-            Converter converter = getConverter();
-            if (converter instanceof XMLConverter) {
-                attributeValue = ((XMLConverter) converter).convertObjectValueToDataValue(attributeValue, session, record.getMarshaller());
-            } else {
-                attributeValue = converter.convertObjectValueToDataValue(attributeValue, session);
-            }
-        }
+        attributeValue = convertObjectValueToDataValue(attributeValue, session, record.getMarshaller());
         XMLField field = (XMLField) getField();
         if (field.getLastXPathFragment().isAttribute()) {
             if (isSwaRef() && (marshaller.getAttachmentMarshaller() != null)) {
@@ -290,7 +295,7 @@ public class XMLBinaryDataMapping extends XMLDirectMapping {
                 Object obj = record.getIndicatingNoEntry(incField);
                 if (!prefixAlreadyDefined && obj != null && obj instanceof DOMRecord) {
                     if (((DOMRecord) obj).getDOM().getNodeType() == Node.ELEMENT_NODE) {
-                        ((Element) ((DOMRecord) obj).getDOM()).setAttributeNS(XMLConstants.XMLNS_URL, XMLConstants.XMLNS + ":" + prefix, XMLConstants.XOP_URL);
+                        ((Element) ((DOMRecord) obj).getDOM()).setAttributeNS(javax.xml.XMLConstants.XMLNS_ATTRIBUTE_NS_URI, javax.xml.XMLConstants.XMLNS_ATTRIBUTE + XMLConstants.COLON + prefix, XMLConstants.XOP_URL);
                     }
                 }
             }
@@ -332,7 +337,7 @@ public class XMLBinaryDataMapping extends XMLDirectMapping {
         }
     }
 
-    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, AbstractSession executionSession) {
+    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, CacheKey cacheKey, AbstractSession executionSession, boolean isTargetProtected, Boolean[] wasCacheUsed) {
         // PERF: Direct variable access.
         Object value = row.get(this.field);
         if (value == null) {
@@ -357,7 +362,13 @@ public class XMLBinaryDataMapping extends XMLDirectMapping {
         } else {
             //this was an element, so do the XOP/SWAREF/Inline binary cases for an element
             XMLRecord record = (XMLRecord) value;
-                record.setSession(executionSession);
+
+            if (getNullPolicy().valueIsNull((Element) record.getDOM())) {
+                return null;
+            }
+            
+           
+            record.setSession(executionSession);
 
             if ((unmarshaller.getAttachmentUnmarshaller() != null) && unmarshaller.getAttachmentUnmarshaller().isXOPPackage() && !this.isSwaRef() && !this.shouldInlineBinaryData()) {
                 //look for the include element:
@@ -405,16 +416,8 @@ public class XMLBinaryDataMapping extends XMLDirectMapping {
                 }
             }
         }
-        Object attributeValue = fieldValue;
-        if (getConverter() != null) {
-            if (getConverter() instanceof XMLConverter) {
-                attributeValue = ((XMLConverter) getConverter()).convertDataValueToObjectValue(fieldValue, executionSession, unmarshaller);
-            } else {
-                attributeValue = getConverter().convertDataValueToObjectValue(fieldValue, executionSession);
-            }
-        }
-
-        attributeValue = XMLBinaryDataHelper.getXMLBinaryDataHelper().convertObject(attributeValue, getAttributeClassification(), executionSession);
+        Object attributeValue = convertDataValueToObjectValue(fieldValue, executionSession, unmarshaller);
+        attributeValue = XMLBinaryDataHelper.getXMLBinaryDataHelper().convertObject(attributeValue, getAttributeClassification(), executionSession, null);
 
         return attributeValue;
     }
@@ -422,4 +425,10 @@ public class XMLBinaryDataMapping extends XMLDirectMapping {
     public boolean isAbstractDirectMapping() {
         return false;
     }
+
+    @Override
+    public boolean isAbstractColumnMapping() {
+        return false;
+    }
+
 }
