@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -28,19 +28,25 @@ import org.eclipse.persistence.internal.helper.ClassConstants;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.helper.NonSynchronizedVector;
 import org.eclipse.persistence.internal.identitymaps.CacheId;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.oxm.Reference;
 import org.eclipse.persistence.internal.oxm.ReferenceResolver;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
+import org.eclipse.persistence.internal.oxm.mappings.CollectionReferenceMapping;
+import org.eclipse.persistence.internal.oxm.mappings.XMLContainerMapping;
+import org.eclipse.persistence.internal.oxm.record.UnmarshalRecord;
 import org.eclipse.persistence.internal.queries.CollectionContainerPolicy;
 import org.eclipse.persistence.internal.queries.ContainerPolicy;
 import org.eclipse.persistence.internal.queries.JoinedAttributeManager;
+import org.eclipse.persistence.internal.queries.ListContainerPolicy;
 import org.eclipse.persistence.internal.queries.MapContainerPolicy;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
+import org.eclipse.persistence.mappings.AttributeAccessor;
 import org.eclipse.persistence.mappings.ContainerMapping;
-import org.eclipse.persistence.oxm.XMLDescriptor;
 import org.eclipse.persistence.oxm.XMLField;
-import org.eclipse.persistence.oxm.record.UnmarshalRecord;
+import org.eclipse.persistence.oxm.mappings.nullpolicy.AbstractNullPolicy;
+import org.eclipse.persistence.oxm.record.DOMRecord;
 import org.eclipse.persistence.oxm.record.XMLRecord;
 import org.eclipse.persistence.queries.ObjectBuildingQuery;
 
@@ -61,12 +67,14 @@ import org.eclipse.persistence.queries.ObjectBuildingQuery;
  * @see XMLObjectReferenceMapping
  * @see ContainerMapping
  */
-public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping implements ContainerMapping {
+public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping implements CollectionReferenceMapping<AbstractSession, AttributeAccessor, ContainerPolicy, ClassDescriptor, DatabaseField, UnmarshalRecord, XMLField, XMLRecord>, ContainerMapping {
     protected ContainerPolicy containerPolicy; // type of container used to hold the aggregate objects
     private static final String SPACE = " ";
     private DatabaseField field;
+    private boolean defaultEmptyContainer = XMLContainerMapping.EMPTY_CONTAINER_DEFAULT;
     private boolean usesSingleNode;
     private boolean reuseContainer;
+    private AbstractNullPolicy wrapperNullPolicy;
 
     /**
      * PUBLIC:
@@ -148,24 +156,26 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
      * Create (if necessary) and populate a reference object that will be used
      * during the mapping reference resolution phase after unmarshalling is
      * complete.
-     * 
-     * @param record
-     * @param xmlField
-     * @param object
-     * @param session
-     * @return
      */
-    public void buildReference(UnmarshalRecord record, XMLField xmlField, Object object, AbstractSession session) {
-        ReferenceResolver resolver = ReferenceResolver.getInstance(session);
+    public void buildReference(UnmarshalRecord record, XMLField xmlField, Object object, AbstractSession session, Object container) {
+         buildReference(record.getCurrentObject(), xmlField, object, session, container, record.getReferenceResolver());
+    }
+
+    /**
+     * INTERNAL:
+     * Create (if necessary) and populate a reference object that will be used
+     * during the mapping reference resolution phase after unmarshalling is
+     * complete.
+     */
+    public void buildReference(Object srcObject, XMLField xmlField, Object object, AbstractSession session, Object container, ReferenceResolver resolver) {
         if (resolver == null) {
             return;
         }
 
-        Object srcObject = record.getCurrentObject();
-        Reference reference = resolver.getReference(this, srcObject);
+        Reference reference = resolver.getReference(this, srcObject, xmlField);
         if (reference == null) {
             // if reference is null, create a new instance and set it on the resolver
-            reference = new Reference(this, srcObject, referenceClass, new HashMap());
+            reference = new Reference(this, srcObject, referenceClass, new HashMap(), container);
             resolver.addReference(reference);
         }
         CacheId primaryKeys;
@@ -180,6 +190,19 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
             if(usesSingleNode) {
                 for (StringTokenizer stok = new StringTokenizer((String) object); stok.hasMoreTokens();) {
                     pks.add(stok.nextToken());
+                    reference = resolver.getReference(this, srcObject, xmlField);
+                    if (reference == null) {
+                        // if reference is null, create a new instance and set it on the resolver
+                        reference = new Reference(this, srcObject, referenceClass, new HashMap(), container);
+                        resolver.addReference(reference);
+                    }
+                    primaryKeyMap = reference.getPrimaryKeyMap();
+                    pks = (CacheId) primaryKeyMap.get(null);
+                    if (pks == null){
+                        Object[] pkValues = new Object[1];
+                        pks = new CacheId(pkValues);
+                        primaryKeyMap.put(null, pks);
+                    }
                 }
             } else {
                 pks.add(object);
@@ -194,7 +217,6 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
                 pks = new CacheId(new Object[0]);
                 primaryKeyMap.put(tgtXPath, pks);
             }
-
             Class type = descriptor.getTypedField(tgtFld).getType();
             XMLConversionManager xmlConversionManager = (XMLConversionManager) session.getDatasourcePlatform().getConversionManager();
             if(usesSingleNode) {
@@ -203,6 +225,18 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
                     if (value != null) {
                         pks.add(value);
                     }
+                    reference = resolver.getReference(this, srcObject, xmlField);
+                    if (reference == null) {
+                        // if reference is null, create a new instance and set it on the resolver
+                        reference = new Reference(this, srcObject, referenceClass, new HashMap(), container);
+                        resolver.addReference(reference);
+                    }
+                    primaryKeyMap = reference.getPrimaryKeyMap();
+                    pks = (CacheId) primaryKeyMap.get(null);
+                    if (pks == null){
+                        pks = new CacheId(new Object[0]);
+                        primaryKeyMap.put(tgtXPath, pks);
+                    }                    
                 }
             } else {
                 Object value = xmlConversionManager.convertObject(object, type);
@@ -239,20 +273,6 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
             }
         }
 
-        // iterate over each source & target XMLField and set the 
-        // appropriate namespace resolver
-        XMLDescriptor descriptor = (XMLDescriptor) this.getDescriptor();
-        XMLDescriptor targetDescriptor = (XMLDescriptor) getReferenceDescriptor();
-        for (int index = 0; index < sourceToTargetKeys.size(); index++) {
-            XMLField sourceField = (XMLField) sourceToTargetKeys.get(index);
-            XMLField targetField = (XMLField) sourceToTargetKeyFieldAssociations.remove(sourceField);
-            sourceField = (XMLField) descriptor.buildField(sourceField);
-            sourceToTargetKeys.set(index, sourceField);
-            if(null != targetField) {
-                targetField = (XMLField) targetDescriptor.buildField(targetField);
-            }
-            sourceToTargetKeyFieldAssociations.put(sourceField, targetField);
-        }
     }
 
     /**
@@ -261,51 +281,58 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
      * org.eclipse.persistence.internal.oxm.Reference instance and stored it 
      * on the session's org.eclipse.persistence.internal.oxm.ReferenceResolver.
      */
-    public Object readFromRowIntoObject(AbstractRecord databaseRow, JoinedAttributeManager joinManager, Object targetObject, ObjectBuildingQuery sourceQuery, AbstractSession executionSession) throws DatabaseException {
+    public Object readFromRowIntoObject(AbstractRecord databaseRow, JoinedAttributeManager joinManager, Object targetObject, CacheKey parentCacheKey, ObjectBuildingQuery sourceQuery, AbstractSession executionSession, boolean isTargetProtected) throws DatabaseException {
+    	  ContainerPolicy cp = getContainerPolicy();                
+          
+          Object container = null;
+          
+          if (reuseContainer) {
+              Object currentObject = ((XMLRecord) databaseRow).getCurrentObject();
+              container = getAttributeAccessor().getAttributeValueFromObject(currentObject);
+              
+          } 
+          if(container == null){
+          	container = cp.containerInstance();
+          }
+          return readFromRowIntoObject(databaseRow, joinManager, targetObject, parentCacheKey, sourceQuery, executionSession, isTargetProtected, container);
+    	
+    }
+
+    /**
+     * INTERNAL:
+     * Extract the primary key values from the row, then create an 
+     * org.eclipse.persistence.internal.oxm.Reference instance and stored it 
+     * on the session's org.eclipse.persistence.internal.oxm.ReferenceResolver.
+     */
+    
+    public Object readFromRowIntoObject(AbstractRecord databaseRow, JoinedAttributeManager joinManager, Object targetObject, CacheKey parentCacheKey, ObjectBuildingQuery sourceQuery, AbstractSession executionSession, boolean isTargetProtected, Object container) throws DatabaseException {
         ClassDescriptor descriptor = sourceQuery.getSession().getClassDescriptor(getReferenceClass());
-        ContainerPolicy cp = getContainerPolicy();
-        HashMap primaryKeyMap = new HashMap();
+        
+        if(container == null){
+        	readFromRowIntoObject(databaseRow, joinManager, targetObject, parentCacheKey, sourceQuery, executionSession, isTargetProtected);
+        }
+        
         // for each source xmlField, get the value from the row and store
         for (Iterator fieldIt = getFields().iterator(); fieldIt.hasNext();) {
             XMLField fld = (XMLField) fieldIt.next();
             //
             Object fieldValue = databaseRow.getValues(fld);
             if ((fieldValue == null) || (fieldValue instanceof String) || !(fieldValue instanceof Vector)) {
-                return cp.containerInstance();
+                return container;
             }
             // fix for bug# 5687430
             // need to get the actual type of the target (i.e. int, String, etc.) 
             // and use the converted value when checking the cache.
-            XMLConversionManager xmlConversionManager = (XMLConversionManager) executionSession.getDatasourcePlatform().getConversionManager();
-            XMLField tgtFld = (XMLField) getSourceToTargetKeyFieldAssociations().get(fld);
-            CacheId newValues = new CacheId(new Object[0]);
             for (Iterator valIt = ((Vector) fieldValue).iterator(); valIt.hasNext();) {
-                for (StringTokenizer stok = new StringTokenizer((String) valIt.next()); stok.hasMoreTokens();) {
-                    Object value;
-                    if(null == tgtFld) {
-                        value = stok.nextToken();
-                    } else {
-                        value = xmlConversionManager.convertObject(stok.nextToken(), descriptor.getTypedField(tgtFld).getType());
-                    }
-                    if (value != null) {
-                        newValues.add(value);
-                    }
-                }
+                Object nextValue = valIt.next();
+                DOMRecord domRecord = (DOMRecord) databaseRow;
+                this.buildReference(domRecord.getCurrentObject(), fld, nextValue, sourceQuery.getSession(), container, domRecord.getReferenceResolver());
             }
-            if(null == tgtFld) {
-                primaryKeyMap.put(null, newValues);
-            } else {
-                primaryKeyMap.put(tgtFld.getXPath(), newValues);
-            }
-        }
-        // store the Reference instance on the resolver for use during mapping resolution phase
-        ReferenceResolver resolver = ReferenceResolver.getInstance(sourceQuery.getSession());
-        if (resolver != null) {
-            resolver.addReference(new Reference(this, targetObject, referenceClass, primaryKeyMap));
         }
         return null;
     }
 
+    
     /**
      * ADVANCED:
      * Set the mapping's containerPolicy.
@@ -333,6 +360,10 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
         this.setContainerPolicy(new CollectionContainerPolicy(concreteContainerClassName));
     }
 
+    public void useListClassName(String concreteContainerClassName) {
+        this.setContainerPolicy(new ListContainerPolicy(concreteContainerClassName));
+    }
+
     /**
      * PUBLIC:
      * Configure the mapping to use an instance of the specified container class
@@ -354,6 +385,27 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
         this.setContainerPolicy(policy);
     }
 
+    /**
+     * PUBLIC:
+     * Configure the mapping to use an instance of the specified container class
+     * to hold the target objects. The key used to index the value in the Map
+     * is the value returned by a call to the specified zero-argument method.
+     * The method must be implemented by the class (or a superclass) of the
+     * value to be inserted into the Map.
+     * <p>jdk1.2.x: The container class must implement (directly or indirectly) the Map interface.
+     * <p>jdk1.1.x: The container class must be a subclass of Hashtable.
+     * <p>The referenceClass must be set before calling this method.
+     */
+    public void useMapClassName(String concreteContainerClass, String methodName) {
+        // the reference class has to be specified before coming here
+        if (this.getReferenceClass() == null) {
+            throw DescriptorException.referenceClassNotSpecified(this);
+        }
+        MapContainerPolicy policy = new MapContainerPolicy(concreteContainerClass);
+        policy.setKeyName(methodName, getReferenceClass().getName());
+        this.setContainerPolicy(policy);
+    }
+
     /**
      * INTERNAL:
      * For the purpose of XMLCollectionReferenceMappings, 'usesSingleNode' 
@@ -461,4 +513,32 @@ public class XMLCollectionReferenceMapping extends XMLObjectReferenceMapping imp
         this.reuseContainer = reuseContainer;
     }
 
+    /**
+     * INTERNAL
+     * Return true if an empty container should be set on the object if there
+     * is no presence of the collection in the XML document.
+     * @since EclipseLink 2.3.3
+     */
+    public boolean isDefaultEmptyContainer() {
+        return defaultEmptyContainer;
+    }
+
+    /**
+     * INTERNAL
+     * Indicate whether by default an empty container should be set on the 
+     * field/property if the collection is not present in the XML document.
+     * @since EclipseLink 2.3.3
+     */
+    public void setDefaultEmptyContainer(boolean defaultEmptyContainer) {
+        this.defaultEmptyContainer = defaultEmptyContainer;
+    }
+
+    public AbstractNullPolicy getWrapperNullPolicy() {
+        return this.wrapperNullPolicy;
+    }
+
+    public void setWrapperNullPolicy(AbstractNullPolicy policy) {
+        this.wrapperNullPolicy = policy;
+    }
+
 }
\ No newline at end of file
