@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
  * which accompanies this distribution.
@@ -15,39 +15,50 @@ package org.eclipse.persistence.oxm.mappings;
 import java.lang.reflect.Modifier;
 import java.util.Enumeration;
 import java.util.Vector;
-import javax.xml.namespace.QName;
 
-import org.w3c.dom.Element;
-import org.w3c.dom.Node;
-import org.w3c.dom.NodeList;
-import org.w3c.dom.Text;
+import javax.xml.namespace.QName;
 
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.exceptions.DatabaseException;
 import org.eclipse.persistence.exceptions.DescriptorException;
 import org.eclipse.persistence.exceptions.XMLMarshalException;
+import org.eclipse.persistence.internal.helper.DatabaseField;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
+import org.eclipse.persistence.internal.oxm.XMLObjectBuilder;
 import org.eclipse.persistence.internal.oxm.XPathEngine;
 import org.eclipse.persistence.internal.oxm.XPathFragment;
-import org.eclipse.persistence.oxm.XMLConstants;
-import org.eclipse.persistence.oxm.XMLContext;
-import org.eclipse.persistence.oxm.XMLField;
-import org.eclipse.persistence.internal.oxm.XMLObjectBuilder;
+import org.eclipse.persistence.internal.oxm.XPathQName;
+import org.eclipse.persistence.internal.oxm.mappings.CompositeCollectionMapping;
+import org.eclipse.persistence.internal.oxm.mappings.Mapping;
+import org.eclipse.persistence.internal.oxm.mappings.XMLContainerMapping;
 import org.eclipse.persistence.internal.queries.ContainerPolicy;
 import org.eclipse.persistence.internal.queries.JoinedAttributeManager;
 import org.eclipse.persistence.internal.queries.MapContainerPolicy;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.mappings.AttributeAccessor;
+import org.eclipse.persistence.mappings.converters.Converter;
 import org.eclipse.persistence.mappings.foundation.AbstractCompositeCollectionMapping;
+import org.eclipse.persistence.oxm.XMLConstants;
+import org.eclipse.persistence.oxm.XMLContext;
 import org.eclipse.persistence.oxm.XMLDescriptor;
+import org.eclipse.persistence.oxm.XMLField;
+import org.eclipse.persistence.oxm.XMLMarshaller;
+import org.eclipse.persistence.oxm.XMLUnmarshaller;
 import org.eclipse.persistence.oxm.mappings.converters.XMLConverter;
 import org.eclipse.persistence.oxm.mappings.nullpolicy.AbstractNullPolicy;
 import org.eclipse.persistence.oxm.mappings.nullpolicy.NullPolicy;
+import org.eclipse.persistence.oxm.mappings.nullpolicy.XMLNullRepresentationType;
 import org.eclipse.persistence.oxm.record.DOMRecord;
 import org.eclipse.persistence.oxm.record.XMLRecord;
 import org.eclipse.persistence.platform.xml.XMLPlatformFactory;
 import org.eclipse.persistence.queries.ObjectBuildingQuery;
+import org.eclipse.persistence.sessions.Session;
+import org.w3c.dom.Element;
+import org.w3c.dom.Node;
+import org.w3c.dom.NodeList;
+import org.w3c.dom.Text;
 
 /**
  * <p>Composite collection XML mappings map an attribute that contains a homogeneous collection of objects
@@ -145,11 +156,13 @@ import org.eclipse.persistence.queries.ObjectBuildingQuery;
  *
  * @since Oracle TopLink 10<i>g</i> Release 2 (10.1.3)
  */
-public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMapping implements XMLMapping, XMLNillableMapping {
+public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMapping implements CompositeCollectionMapping<AbstractSession, AttributeAccessor, ContainerPolicy, Converter, ClassDescriptor, DatabaseField, XMLMarshaller, Session, UnmarshalKeepAsElementPolicy, XMLUnmarshaller, XMLRecord>, XMLMapping, XMLNillableMapping {
     AbstractNullPolicy nullPolicy;
+    private AbstractNullPolicy wrapperNullPolicy;
     private UnmarshalKeepAsElementPolicy keepAsElementPolicy;
     private XMLInverseReferenceMapping inverseReferenceMapping;
 
+    private boolean defaultEmptyContainer = XMLContainerMapping.EMPTY_CONTAINER_DEFAULT;
     private boolean isWriteOnly;
     private boolean reuseContainer = false;
 
@@ -283,6 +296,13 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         return true;
     }
 
+    
+    public void convertClassNamesToClasses(ClassLoader classLoader){        
+        if(XMLConstants.UNKNOWN_OR_TRANSIENT_CLASS.equals(referenceClassName)){            
+            return;
+        }
+        super.convertClassNamesToClasses(classLoader);
+    }
     /**
      * INTERNAL:
      * The mapping is initialized with the given session. This mapping is fully initialized
@@ -290,23 +310,13 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
      */
     public void initialize(AbstractSession session) throws DescriptorException {
         //modified so that reference class on composite mappings is no longer mandatory
-        if ((getReferenceClass() == null) && (getReferenceClassName() != null)) {
-            setReferenceClass(session.getDatasourcePlatform().getConversionManager().convertClassNameToClass(getReferenceClassName()));
-        }
-        if (getReferenceClass() != null) {
-            super.initialize(session);
-        } else {
-            //below should be the same as AbstractCompositeCollectionMapping.initialize
-            if (getField() == null) {
-                throw DescriptorException.fieldNameNotSetInMapping(this);
-            }
-            setField(getDescriptor().buildField(getField()));
-            setFields(collectFields());
-            if (hasConverter()) {
-                getConverter().initialize(this, session);
+        String referenceClassName = getReferenceClassName();
+        if (this.referenceClass == null && referenceClassName != null) {
+            if (!referenceClassName.equals(XMLConstants.UNKNOWN_OR_TRANSIENT_CLASS)) {
+                setReferenceClass(session.getDatasourcePlatform().getConversionManager().convertClassNameToClass(referenceClassName));
             }
         }
-
+        initializeReferenceDescriptorAndField(session);
         ContainerPolicy cp = getContainerPolicy();
         if (cp != null) {
             if (cp.getContainerClass() == null) {
@@ -314,14 +324,38 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
                 cp.setContainerClass(cls);
             }
             if (cp instanceof MapContainerPolicy) {
-                ((MapContainerPolicy) cp).setElementClass(this.getReferenceClass());
+            	initializeMapContainerPolicy(session, (MapContainerPolicy)cp);
+                               
             }
         }
+       
+
+      
         if(null != getContainerAccessor()) {
             getContainerAccessor().initializeAttributes(this.referenceClass);
         }
 
     }
+        
+    protected void initializeMapContainerPolicy(AbstractSession session, MapContainerPolicy cp){
+    	 cp.setElementClass(this.referenceClass);
+    }
+    
+    protected void initializeReferenceDescriptorAndField(AbstractSession session){
+    	 if (this.referenceClass != null) {
+             super.initialize(session);
+         } else {
+        	  //below should be the same as AbstractCompositeCollectionMapping.initialize
+             if (this.field == null) {
+                 throw DescriptorException.fieldNameNotSetInMapping(this);
+             }
+             setField(getDescriptor().buildField(this.field));
+             setFields(collectFields());   
+             if (hasConverter()) {
+                 getConverter().initialize(this, session);
+             }
+         }    	
+    }
 
     /**
      * Get the XPath String
@@ -341,7 +375,8 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         this.setField(new XMLField(xpathString));
     }
 
-    protected Object buildCompositeObject(ClassDescriptor descriptor, AbstractRecord nestedRow, ObjectBuildingQuery query, JoinedAttributeManager joinManger) {
+    @Override
+    protected Object buildCompositeObject(ClassDescriptor descriptor, AbstractRecord nestedRow, ObjectBuildingQuery query, CacheKey parentCacheKey, JoinedAttributeManager joinManger, AbstractSession targetSession) {
         return descriptor.getObjectBuilder().buildObject(query, nestedRow, joinManger);
     }
 
@@ -361,14 +396,7 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         XMLRecord parent = (XMLRecord) parentRow;
 
         if (classDesc != null) {
-            XMLObjectBuilder objectBuilder = (XMLObjectBuilder) classDesc.getObjectBuilder();
-
-            
-            XMLRecord child = (XMLRecord) objectBuilder.createRecordFor(attributeValue, (XMLField) getField(), parent, this);
-            child.setNamespaceResolver(parent.getNamespaceResolver());
-            child.setSession(session);
-            objectBuilder.buildIntoNestedRow(child, attributeValue, session, (XMLDescriptor)getReferenceDescriptor(), (XMLField) getField());
-            return child;
+        	return buildCompositeRowForDescriptor(classDesc, attributeValue, session, parent, writeType);
         } else {
             if (attributeValue instanceof Element && getKeepAsElementPolicy() == UnmarshalKeepAsElementPolicy.KEEP_UNKNOWN_AS_ELEMENT) {
                 return new DOMRecord((Element) attributeValue);
@@ -380,6 +408,18 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         }
     }
 
+
+   	protected AbstractRecord buildCompositeRowForDescriptor(ClassDescriptor classDesc, Object attributeValue, AbstractSession session, XMLRecord parentRow, WriteType writeType) {
+	    XMLObjectBuilder objectBuilder = (XMLObjectBuilder) classDesc.getObjectBuilder();
+	
+	    
+	    XMLRecord child = (XMLRecord) objectBuilder.createRecordFor(attributeValue, (XMLField) getField(), parentRow, this);
+	    child.setNamespaceResolver(parentRow.getNamespaceResolver());
+	    child.setSession(session);
+	    objectBuilder.buildIntoNestedRow(child, attributeValue, session, (XMLDescriptor)getReferenceDescriptor(), (XMLField) getField());
+	    return child;
+    }
+    
     /**
      * INTERNAL:
      */
@@ -398,17 +438,25 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         ContainerPolicy cp = this.getContainerPolicy();
 
         Vector nestedRows = new Vector(cp.sizeFor(attributeValue));
-        for (Object iter = cp.iteratorFor(attributeValue); cp.hasNext(iter);) {
-            Object element = cp.next(iter, session);
-            // convert the value - if necessary
-            if (hasConverter()) {
-                if (getConverter() instanceof XMLConverter) {
-                    element = ((XMLConverter) getConverter()).convertObjectValueToDataValue(element, session, ((XMLRecord) row).getMarshaller());
+        Object iter = cp.iteratorFor(attributeValue);
+        if(null != iter) {
+            while(cp.hasNext(iter)) {
+                Object element = cp.next(iter, session);
+                // convert the value - if necessary
+                element = convertObjectValueToDataValue(element, session, ((XMLRecord) row).getMarshaller());
+                if(element == null) {
+                    XMLNullRepresentationType nullRepresentation = getNullPolicy().getMarshalNullRepresentation();
+                    if(nullRepresentation == XMLNullRepresentationType.XSI_NIL) {
+                        nestedRows.add(XMLRecord.NIL);
+                    } else if(nullRepresentation == XMLNullRepresentationType.EMPTY_NODE) {
+                        Node emptyNode = XPathEngine.getInstance().createUnownedElement(((XMLRecord)row).getDOM(), (XMLField)field);
+                        DOMRecord nestedRow = new DOMRecord(emptyNode);
+                        nestedRows.add(nestedRow);
+                    }
                 } else {
-                    element = getConverter().convertObjectValueToDataValue(element, session);
+                    nestedRows.addElement(buildCompositeRow(element, session, row, writeType));
                 }
             }
-            nestedRows.addElement(buildCompositeRow(element, session, row, writeType));
         }
 
         Object fieldValue = null;
@@ -418,7 +466,7 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         row.put(this.getField(), fieldValue);
     }
 
-    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery sourceQuery, AbstractSession executionSession) throws DatabaseException {
+    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery sourceQuery, CacheKey cacheKey, AbstractSession executionSession, boolean isTargetProtected, Boolean[] wasCacheUsed) throws DatabaseException {
         ContainerPolicy cp = this.getContainerPolicy();
 
         Object fieldValue = row.getValues(this.getField());
@@ -455,8 +503,13 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         }
 
         for (Enumeration stream = nestedRows.elements(); stream.hasMoreElements();) {
-            AbstractRecord nestedRow = (AbstractRecord) stream.nextElement();
-            Object objectToAdd = buildObjectFromNestedRow(nestedRow, joinManager, sourceQuery, executionSession);
+            XMLRecord nestedRow = (XMLRecord) stream.nextElement();
+            Object objectToAdd;
+            if (getNullPolicy().valueIsNull((Element) nestedRow.getDOM())) {
+                objectToAdd = null;
+            } else {
+                objectToAdd = buildObjectFromNestedRow(nestedRow, joinManager, sourceQuery, executionSession, isTargetProtected);
+            }
             cp.addInto(objectToAdd, result, sourceQuery.getSession());
             if(null != getContainerAccessor()) {
                 Object currentObject = ((XMLRecord)row).getCurrentObject();
@@ -475,7 +528,7 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         return result;
     }
 
-    public Object buildObjectFromNestedRow(AbstractRecord nestedRow, JoinedAttributeManager joinManager, ObjectBuildingQuery sourceQuery, AbstractSession executionSession) {
+    public Object buildObjectFromNestedRow(AbstractRecord nestedRow, JoinedAttributeManager joinManager, ObjectBuildingQuery sourceQuery, AbstractSession executionSession, boolean isTargetProtected) {
         Object objectToAdd = null;
         ClassDescriptor aDescriptor = getReferenceDescriptor((DOMRecord) nestedRow);
         
@@ -483,13 +536,7 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
             if ((getKeepAsElementPolicy() == UnmarshalKeepAsElementPolicy.KEEP_UNKNOWN_AS_ELEMENT) || (getKeepAsElementPolicy() == UnmarshalKeepAsElementPolicy.KEEP_ALL_AS_ELEMENT)) {
                 XMLPlatformFactory.getInstance().getXMLPlatform().namespaceQualifyFragment((Element) ((DOMRecord)nestedRow).getDOM());
                 objectToAdd = ((DOMRecord)nestedRow).getDOM();
-                if (getConverter() != null) {     
-                    if (getConverter() instanceof XMLConverter) {
-                        objectToAdd = ((XMLConverter) getConverter()).convertDataValueToObjectValue(objectToAdd, executionSession, ((XMLRecord) nestedRow).getUnmarshaller());
-                    } else {
-                        objectToAdd = getConverter().convertDataValueToObjectValue(objectToAdd, executionSession);
-                    }
-                }
+                convertDataValueToObjectValue(objectToAdd, executionSession, ((XMLRecord) nestedRow).getUnmarshaller());
                 //simple case
                 objectToAdd = convertToSimpleTypeIfPresent(objectToAdd, nestedRow,executionSession);
             } else {
@@ -498,7 +545,12 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
                     Node nextNode = children.item(i);
                     if(nextNode.getNodeType() == nextNode.ELEMENT_NODE){
                         //complex child
-                        throw XMLMarshalException.noDescriptorFound(this);                              
+                        String type = ((Element) ((DOMRecord)nestedRow).getDOM()).getAttributeNS(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI, XMLConstants.SCHEMA_TYPE_ATTRIBUTE);
+                        if(type != null && type.length() > 0) {
+                            throw XMLMarshalException.unknownXsiTypeValue(type, (Mapping) this);
+                        } else {
+                            throw XMLMarshalException.noDescriptorFound((Mapping) this);
+                        }
                     }
                 }
                  //simple case
@@ -511,8 +563,9 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
                 if (newElementClass == null) {
                     // no xsi:type attribute - look for type indicator on the field
                     QName leafElementType = ((XMLField) getField()).getLeafElementType();
-                    if (leafElementType != null) {
-                        Object indicator = aDescriptor.getInheritancePolicy().getClassIndicatorMapping().get(leafElementType);
+                    if (leafElementType != null) {                    	
+                    	XPathQName leafElementXPathQName = new XPathQName(leafElementType, ((XMLRecord) nestedRow).isNamespaceAware());
+                        Object indicator = aDescriptor.getInheritancePolicy().getClassIndicatorMapping().get(leafElementXPathQName);
                         if(indicator != null) {
                             newElementClass = (Class) indicator;
                         }
@@ -531,14 +584,8 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
             }
 
             //Object element 
-            objectToAdd = buildCompositeObject(aDescriptor, nestedRow, sourceQuery, joinManager);
-            if (hasConverter()) {
-                if (getConverter() instanceof XMLConverter) {
-                    objectToAdd = ((XMLConverter) getConverter()).convertDataValueToObjectValue(objectToAdd, executionSession, ((XMLRecord) nestedRow).getUnmarshaller());
-                } else {
-                    objectToAdd = getConverter().convertDataValueToObjectValue(objectToAdd, executionSession);
-                }
-            }
+            objectToAdd = buildCompositeObject(aDescriptor, nestedRow, sourceQuery, null, joinManager, executionSession);
+            objectToAdd = convertDataValueToObjectValue(objectToAdd, executionSession, ((XMLRecord) nestedRow).getUnmarshaller());
         }
         return objectToAdd;
     }
@@ -558,7 +605,7 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
             return objectToAdd;
         }
 
-        String type = theElement.getAttributeNS(XMLConstants.SCHEMA_INSTANCE_URL, XMLConstants.SCHEMA_TYPE_ATTRIBUTE);
+        String type = theElement.getAttributeNS(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI, XMLConstants.SCHEMA_TYPE_ATTRIBUTE);
         if ((null != type) && type.length() > 0) {
             XPathFragment typeFragment = new XPathFragment(type);
             String namespaceURI = ((DOMRecord)nestedRow).resolveNamespacePrefix(typeFragment.getPrefix());
@@ -577,7 +624,7 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
 
         if (returnDescriptor == null) {
             // Try to find a descriptor based on the schema type
-            String type = ((Element) xmlRecord.getDOM()).getAttributeNS(XMLConstants.SCHEMA_INSTANCE_URL, XMLConstants.SCHEMA_TYPE_ATTRIBUTE);
+            String type = ((Element) xmlRecord.getDOM()).getAttributeNS(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI, XMLConstants.SCHEMA_TYPE_ATTRIBUTE);
 
             if ((null != type) && type.length() >0 ) {
                 XPathFragment typeFragment = new XPathFragment(type);
@@ -625,14 +672,7 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
     }
 
     public void writeSingleValue(Object value, Object parent, XMLRecord record, AbstractSession session) {
-        Object element = value;
-        if (hasConverter()) {
-            if (getConverter() instanceof XMLConverter) {
-                element = ((XMLConverter) getConverter()).convertObjectValueToDataValue(element, session, record.getMarshaller());
-            } else {
-                element = getConverter().convertObjectValueToDataValue(element, session);
-            }
-        }
+        Object element = convertObjectValueToDataValue(value, session, record.getMarshaller());
         XMLRecord nestedRow = (XMLRecord) buildCompositeRow(element, session, record, WriteType.UNDEFINED);
         record.add(getField(), nestedRow);
     }
@@ -726,4 +766,62 @@ public class XMLCompositeCollectionMapping extends AbstractCompositeCollectionMa
         this.inverseReferenceMapping = inverseReferenceMapping;
     }
 
+    /**
+     * INTERNAL
+     * Return true if an empty container should be set on the object if there
+     * is no presence of the collection in the XML document.
+     * @since EclipseLink 2.3.3
+     */
+    public boolean isDefaultEmptyContainer() {
+        return defaultEmptyContainer;
+    }
+
+    /**
+     * INTERNAL
+     * Indicate whether by default an empty container should be set on the 
+     * field/property if the collection is not present in the XML document.
+     * @since EclipseLink 2.3.3
+     */
+    public void setDefaultEmptyContainer(boolean defaultEmptyContainer) {
+        this.defaultEmptyContainer = defaultEmptyContainer;
+    }
+
+    public AbstractNullPolicy getWrapperNullPolicy() {
+        return this.wrapperNullPolicy;
+    }
+
+    public void setWrapperNullPolicy(AbstractNullPolicy policy) {
+        this.wrapperNullPolicy = policy;
+    }
+
+    /**
+     * INTERNAL
+     * @since EclipseLink 2.5.0
+     */
+    public Object convertObjectValueToDataValue(Object value, Session session, XMLMarshaller marshaller) {
+        if (hasConverter()) {
+            if (converter instanceof XMLConverter) {
+                return ((XMLConverter)converter).convertObjectValueToDataValue(value, (AbstractSession) session, marshaller);
+            } else {
+                return converter.convertObjectValueToDataValue(value, (AbstractSession) session);
+            }
+        }
+        return value;
+    }
+
+    /**
+     * INTERNAL
+     * @since EclipseLink 2.5.0
+     */
+    public Object convertDataValueToObjectValue(Object value, Session session, XMLUnmarshaller unmarshaller) {
+        if (hasConverter()) {
+            if (converter instanceof XMLConverter) {
+                return ((XMLConverter)converter).convertDataValueToObjectValue(value, session, unmarshaller);
+            } else {
+                return converter.convertDataValueToObjectValue(value, session);
+            }
+        }
+        return value;
+    }
+
 }
\ No newline at end of file
