@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,14 +12,27 @@
  ******************************************************************************/  
 package org.eclipse.persistence.oxm.mappings;
 
+import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.exceptions.ConversionException;
 import org.eclipse.persistence.exceptions.DescriptorException;
+import org.eclipse.persistence.internal.core.sessions.CoreAbstractSession;
 import org.eclipse.persistence.internal.helper.DatabaseField;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
+import org.eclipse.persistence.internal.oxm.XMLConversionManager;
+import org.eclipse.persistence.internal.oxm.mappings.DirectMapping;
+import org.eclipse.persistence.internal.oxm.mappings.Field;
+import org.eclipse.persistence.internal.oxm.record.AbstractMarshalRecord;
+import org.eclipse.persistence.internal.oxm.record.AbstractUnmarshalRecord;
+import org.eclipse.persistence.internal.queries.ContainerPolicy;
 import org.eclipse.persistence.internal.queries.JoinedAttributeManager;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.oxm.XMLConstants;
 import org.eclipse.persistence.oxm.XMLField;
+import org.eclipse.persistence.oxm.XMLMarshaller;
+import org.eclipse.persistence.oxm.XMLUnmarshaller;
+import org.eclipse.persistence.mappings.AttributeAccessor;
+import org.eclipse.persistence.mappings.converters.Converter;
 import org.eclipse.persistence.mappings.foundation.AbstractDirectMapping;
 import org.eclipse.persistence.oxm.mappings.converters.XMLConverter;
 import org.eclipse.persistence.oxm.mappings.nullpolicy.AbstractNullPolicy;
@@ -27,6 +40,7 @@ import org.eclipse.persistence.oxm.mappings.nullpolicy.NullPolicy;
 import org.eclipse.persistence.oxm.record.DOMRecord;
 import org.eclipse.persistence.oxm.record.XMLRecord;
 import org.eclipse.persistence.queries.ObjectBuildingQuery;
+import org.eclipse.persistence.sessions.Session;
 
 /**
  * <p>XML Direct Mappings map a Java attribute directly to XML attribute or text node.
@@ -181,11 +195,14 @@ import org.eclipse.persistence.queries.ObjectBuildingQuery;
  *
  * @since Oracle TopLink 10<i>g</i> Release 2 (10.1.3)
  */
-public class XMLDirectMapping extends AbstractDirectMapping implements XMLMapping, XMLNillableMapping {
+public class XMLDirectMapping extends AbstractDirectMapping implements XMLMapping, DirectMapping<AbstractSession, AttributeAccessor, ContainerPolicy, Converter, ClassDescriptor, DatabaseField, XMLMarshaller, Session, XMLUnmarshaller, XMLRecord>, XMLNillableMapping {
    
     AbstractNullPolicy nullPolicy;
     public boolean isCDATA;
     private boolean isWriteOnly;
+    private boolean isCollapsingStringValues;
+    private boolean isNormalizingStringValues;
+    private boolean isNullValueMarshalled = false;
 
     public XMLDirectMapping() {
         super();
@@ -246,12 +263,12 @@ public class XMLDirectMapping extends AbstractDirectMapping implements XMLMappin
      * INTERNAL:
      * Allows for subclasses to convert the attribute value.
      */
-    public Object getAttributeValue(Object fieldValue, AbstractSession session, XMLRecord record) {
+    public Object getAttributeValue(Object fieldValue, AbstractSession session, AbstractUnmarshalRecord record) {
     	// Unmarshal DOM
         // If attribute is empty string representing (null) then return the nullValue
-        boolean isNullRepresentedByEmptyNode = getNullPolicy().isNullRepresentedByEmptyNode();
-        boolean isNullRepresentedByXsiNil = getNullPolicy().isNullRepresentedByXsiNil();
-        if (XMLConstants.EMPTY_STRING.equals(fieldValue) && isNullRepresentedByEmptyNode) {
+        boolean isNullRepresentedByEmptyNode = nullPolicy.isNullRepresentedByEmptyNode();
+        boolean isNullRepresentedByXsiNil = nullPolicy.isNullRepresentedByXsiNil();
+        if (isNullRepresentedByEmptyNode && XMLConstants.EMPTY_STRING.equals(fieldValue)) {
            fieldValue = null;
         } else if (null == fieldValue && !isNullRepresentedByEmptyNode) {
             fieldValue = XMLConstants.EMPTY_STRING;
@@ -275,12 +292,8 @@ public class XMLDirectMapping extends AbstractDirectMapping implements XMLMappin
         }
 
         // Allow for user defined conversion to the object value.       
-        if (hasConverter()) {
-            if (getConverter() instanceof XMLConverter) {
-                attributeValue = ((XMLConverter)getConverter()).convertDataValueToObjectValue(attributeValue, session, record.getUnmarshaller());
-            } else {
-                attributeValue = getConverter().convertDataValueToObjectValue(attributeValue, session);
-            }
+        if (converter != null) {
+            attributeValue = convertDataValueToObjectValue(attributeValue, session, (XMLUnmarshaller) record.getUnmarshaller());
         } else {
             // PERF: Avoid conversion check when not required.
             if ((attributeValue == null) || (attributeValue.getClass() != this.attributeObjectClassification)) {
@@ -303,31 +316,29 @@ public class XMLDirectMapping extends AbstractDirectMapping implements XMLMappin
      * Convert the attribute value to a field value.
      * Process any converter if defined, and check for null values.
      */
-    public Object getFieldValue(Object attributeValue, AbstractSession session, XMLRecord record) {
+    public Object getFieldValue(Object attributeValue, CoreAbstractSession session, AbstractMarshalRecord record) {
     	// Marshal
         // PERF: This method is a major performance code point,
         // so has been micro optimized and uses direct variable access.
         Object fieldValue = attributeValue;
-        if ((this.nullValue != null) && (this.nullValue.equals(fieldValue))) {
+
+        if ((this.nullValue != null) && (this.nullValue.equals(fieldValue)) && !this.isNullValueMarshalled && !((XMLField)field).isRequired()) {
             return null;
         }
 
         // Allow for user defined conversion to the object value.       
-        if (hasConverter()) {
-            if (getConverter() instanceof XMLConverter) {
-                fieldValue = ((XMLConverter)getConverter()).convertObjectValueToDataValue(fieldValue, session, record.getMarshaller());
-            } else {
-                fieldValue = getConverter().convertObjectValueToDataValue(fieldValue, session);
-            }
-        }
-        Class fieldClassification = getFieldClassification(getField());
-        // PERF: Avoid conversion if not required.
-        if ((fieldValue != null) && (fieldClassification != fieldValue.getClass())) {
-            try {
-                fieldValue = session.getPlatform(descriptor.getJavaClass()).convertObject(fieldValue, fieldClassification);
-            } catch (ConversionException exception) {
-                throw ConversionException.couldNotBeConverted(this, descriptor, exception);
-            }
+        fieldValue = convertObjectValueToDataValue(fieldValue, (AbstractSession) session, (XMLMarshaller) record.getMarshaller());
+   
+        if (fieldValue != null) {
+        	 Class fieldClassification = getFieldClassification(field);
+             // PERF: Avoid conversion if not required.
+        	 if(fieldClassification != fieldValue.getClass()){
+        		 try {
+                     fieldValue = session.getPlatform(descriptor.getJavaClass()).convertObject(fieldValue, fieldClassification);
+                 } catch (ConversionException exception) {
+                     throw ConversionException.couldNotBeConverted(this, descriptor, exception);
+                 }
+        	 }
         }
         return fieldValue;
     }
@@ -347,7 +358,7 @@ public class XMLDirectMapping extends AbstractDirectMapping implements XMLMappin
      * directly from a row, the session set in the query will not know which platform to use
      * for converting the value.  Allows the correct session to be passed in.
      */
-    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, AbstractSession executionSession) {
+    public Object valueFromRow(AbstractRecord row, JoinedAttributeManager joinManager, ObjectBuildingQuery query, CacheKey cacheKey, AbstractSession executionSession, boolean isTargetProtected, Boolean[] wasCacheUsed) {
         // PERF: Direct variable access.
         boolean shouldCheckForXsiNil = getNullPolicy().isNullRepresentedByXsiNil();
         return getAttributeValue(((DOMRecord)row).getIndicatingNoEntry(this.field, false, shouldCheckForXsiNil), executionSession, (XMLRecord)row);
@@ -371,7 +382,11 @@ public class XMLDirectMapping extends AbstractDirectMapping implements XMLMappin
     
     public void writeSingleValue(Object value, Object parent, XMLRecord row, AbstractSession session) {
         Object fieldValue = getFieldValue(value, session, row);
-        writeValueIntoRow(row, getField(), fieldValue);
+        if(fieldValue == null && getNullPolicy() != null) {
+            getNullPolicy().directMarshal((Field) this.getField(), row, parent);
+        } else {
+            writeValueIntoRow(row, getField(), fieldValue);
+        }
     }
     
     public void setAttributeValueInObject(Object object, Object value) throws DescriptorException {
@@ -383,6 +398,13 @@ public class XMLDirectMapping extends AbstractDirectMapping implements XMLMappin
             if(value == XMLRecord.noEntry) {
                 return;                    
             }
+            if (value != null && value instanceof String) {
+                if(isCollapsingStringValues) {
+                    value = XMLConversionManager.getDefaultXMLManager().collapseStringValue((String)value);
+                } else if(isNormalizingStringValues) {
+                    value = XMLConversionManager.getDefaultXMLManager().normalizeStringValue((String)value);
+                }
+            }
             this.attributeAccessor.setAttributeValueInObject(object, value);
         } catch (DescriptorException exception) {
             exception.setMapping(this);
@@ -405,11 +427,108 @@ public class XMLDirectMapping extends AbstractDirectMapping implements XMLMappin
     public void setIsWriteOnly(boolean b) {
         this.isWriteOnly = b;
     }
-    
+
+    @Override
+    public boolean isCloningRequired() {
+        return false;
+    }
+
     public void preInitialize(AbstractSession session) throws DescriptorException {
         getAttributeAccessor().setIsWriteOnly(this.isWriteOnly());
         getAttributeAccessor().setIsReadOnly(this.isReadOnly());
         super.preInitialize(session);
     }
+    
+    
+    /**
+     * PUBLIC: 
+     * Returns true if this mapping is normalizing string values on unmarshal before setting
+     * them in the object. Normalize replaces any CR, LF or Tab characters with a 
+     * single space character. 
+     */
+    public boolean isNormalizingStringValues() {
+        return this.isNormalizingStringValues;
+    }
+    
+    /**
+     * PUBLIC:
+     * Indicates that this mapping should normalize all string values before setting them
+     * in the object on unmarshal. Normalize replaces any CR, LF or Tab characters with a 
+     * single space character. 
+     * @param normalize
+     */
+    public void setNormalizingStringValues(boolean normalize) {
+        this.isNormalizingStringValues = normalize;
+    }
+    
+    
+    /**
+     * PUBLIC:
+     * Indicates that this mapping should collapse all string values before setting them
+     * in the object on unmarshal. Collapse removes leading and trailing whitespaces, and replaces
+     * any sequence of whitepsace characters with a single space. 
+     * @param collapse
+     */
+    public void setCollapsingStringValues(boolean collapse) {
+        this.isCollapsingStringValues = collapse;
+    }
+    
+    /**
+     * PUBLIC:
+     * Returns true if this mapping should collapse all string values before setting them
+     * in the object on unmarshal. Collapse removes leading and trailing whitespaces, and replaces
+     * any sequence of whitepsace characters with a single space. 
+     */
+    public boolean isCollapsingStringValues() {
+        return this.isCollapsingStringValues;
+    }
 
-}
+    /**
+     * PUBLIC:
+     * Returns true if this mapping's value should be marshalled, in the case that 
+     * it is equal to the default null value. 
+     */
+    public boolean isNullValueMarshalled() {
+        return this.isNullValueMarshalled;
+    }
+
+    /**
+     * PUBLIC:
+     * Set whether this mapping's value should be marshalled, in the case that 
+     * it is equal to the default null value. 
+     */
+    public void setNullValueMarshalled(boolean value) {
+        this.isNullValueMarshalled = value;
+    }
+
+    /**
+     * INTERNAL
+     * @since EclipseLink 2.5.0
+     */
+    public Object convertObjectValueToDataValue(Object value, Session session, XMLMarshaller marshaller) {
+        if (hasConverter()) {
+            if (converter instanceof XMLConverter) {
+                return ((XMLConverter)converter).convertObjectValueToDataValue(value, session, marshaller);
+            } else {
+                return converter.convertObjectValueToDataValue(value, session);
+            }
+        }
+        return value;
+    }
+
+    /**
+     * INTERNAL
+     * @since EclipseLink 2.5.0
+     */
+    public Object convertDataValueToObjectValue(Object fieldValue, Session session, XMLUnmarshaller unmarshaller) {
+        if (hasConverter()) {
+            if (converter instanceof XMLConverter) {
+                return ((XMLConverter)converter).convertDataValueToObjectValue(fieldValue, session, unmarshaller);
+            } else {
+                return converter.convertDataValueToObjectValue(fieldValue, session);
+            }
+        }
+        return fieldValue;
+    }
+
+}
\ No newline at end of file
