@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -24,19 +24,23 @@ import org.eclipse.persistence.internal.descriptors.ObjectBuilder;
 import org.eclipse.persistence.internal.helper.ClassConstants;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.identitymaps.CacheId;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.internal.oxm.Reference;
-import org.eclipse.persistence.internal.oxm.ReferenceListener;
 import org.eclipse.persistence.internal.oxm.ReferenceResolver;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
+import org.eclipse.persistence.internal.oxm.mappings.ObjectReferenceMapping;
+import org.eclipse.persistence.internal.oxm.record.UnmarshalRecord;
+import org.eclipse.persistence.internal.queries.ContainerPolicy;
 import org.eclipse.persistence.internal.queries.JoinedAttributeManager;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.sessions.UnitOfWorkImpl;
 import org.eclipse.persistence.mappings.AggregateMapping;
+import org.eclipse.persistence.mappings.AttributeAccessor;
 import org.eclipse.persistence.oxm.XMLDescriptor;
 import org.eclipse.persistence.oxm.XMLField;
 import org.eclipse.persistence.oxm.XMLUnionField;
-import org.eclipse.persistence.oxm.record.UnmarshalRecord;
+import org.eclipse.persistence.oxm.record.DOMRecord;
 import org.eclipse.persistence.oxm.record.XMLRecord;
 import org.eclipse.persistence.queries.ObjectBuildingQuery;
 
@@ -51,7 +55,7 @@ import org.eclipse.persistence.queries.ObjectBuildingQuery;
  * to locate the correct target object instance in the session cache when resolving
  * mapping references.
  */
-public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMapping {
+public class XMLObjectReferenceMapping extends AggregateMapping implements ObjectReferenceMapping<AbstractSession, AttributeAccessor, ContainerPolicy, ClassDescriptor, DatabaseField, UnmarshalRecord, XMLField, XMLRecord>, XMLMapping {
     protected HashMap sourceToTargetKeyFieldAssociations;
     protected Vector sourceToTargetKeys; // maintain the order of the keys
     private boolean isWriteOnly;
@@ -83,6 +87,11 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
             sourceToTargetKeyFieldAssociations.put(srcFld, new XMLField(tgtXPath));
         }
     }
+    
+    public void addSourceToTargetKeyFieldAssociation(XMLField srcField, XMLField tgtField) {
+        sourceToTargetKeys.add(srcField);
+        sourceToTargetKeyFieldAssociations.put(srcField, tgtField);
+    }
 
     /**    
      * INTERNAL:
@@ -134,7 +143,7 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
      * @return
      */
     public void buildReference(UnmarshalRecord record, XMLField xmlField, Object object, AbstractSession session) {
-        ReferenceResolver resolver = ReferenceResolver.getInstance(session);
+        ReferenceResolver resolver = record.getReferenceResolver();
         if (resolver == null) {
             return;
         }
@@ -169,7 +178,7 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
                 primaryKeys = (CacheId) reference.getPrimaryKey();
             }
             XMLField tgtFld = (XMLField) getSourceToTargetKeyFieldAssociations().get(xmlField);
-            int idx = pkFieldNames.indexOf(tgtFld.getXPath());
+            int idx = pkFieldNames.indexOf(tgtFld.getQualifiedName());
             // fix for bug# 5687430
             // need to get the actual type of the target (i.e. int, String, etc.) 
             // and use the converted value when checking the cache.
@@ -323,11 +332,6 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
             super.initialize(session);
         }
 
-        ReferenceListener listener = new ReferenceListener();
-        if (!(session.getEventManager().getListeners().contains(listener))) {
-            session.getEventManager().addListener(listener);
-        }
-
         // iterate over each source & target XMLField and set the 
         // appropriate namespace resolver
         XMLDescriptor descriptor = (XMLDescriptor) this.getDescriptor();
@@ -340,8 +344,18 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
             if(null != targetField) {
                 if(null == targetDescriptor) {
                     throw DescriptorException.referenceClassNotSpecified(this);
-                }
-                targetField = (XMLField) targetDescriptor.buildField(targetField);
+                }               
+                //primary key field from ref desc
+                List<DatabaseField > pkFields  = targetDescriptor.getPrimaryKeyFields();
+                for(int i=0; i<pkFields.size(); i++){
+                	XMLField nextPKField = (XMLField)pkFields.get(i);
+                	if(targetField.equals(nextPKField)){
+                		targetField = (XMLField) targetDescriptor.buildField(nextPKField);
+                		sourceField.setSchemaType(targetField.getSchemaType());
+                		break;
+                	}
+                } 
+
             }
             sourceToTargetKeyFieldAssociations.put(sourceField, targetField);
         }
@@ -374,7 +388,7 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
      * org.eclipse.persistence.internal.oxm.Reference instance and store it 
      * on the session's org.eclipse.persistence.internal.oxm.ReferenceResolver.
      */
-    public Object readFromRowIntoObject(AbstractRecord databaseRow, JoinedAttributeManager joinManager, Object targetObject, ObjectBuildingQuery sourceQuery, AbstractSession executionSession) throws DatabaseException {
+    public Object readFromRowIntoObject(AbstractRecord databaseRow, JoinedAttributeManager joinManager, Object targetObject, CacheKey parentCacheKey, ObjectBuildingQuery sourceQuery, AbstractSession executionSession, boolean isTargetProtected) throws DatabaseException {
         // the order in which the primary keys are added to the vector is
         // relevant for cache lookup - it must match the ordering of the 
         // reference descriptor's primary key entries
@@ -411,7 +425,7 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
         }
         // store the Reference instance on the resolver for use during mapping
         // resolution phase
-        ReferenceResolver resolver = ReferenceResolver.getInstance(sourceQuery.getSession());
+        ReferenceResolver resolver = ((DOMRecord) databaseRow).getReferenceResolver();
         if (resolver != null) {
             resolver.addReference(new Reference(this, targetObject, referenceClass, primaryKeys));
         }
@@ -482,5 +496,9 @@ public class XMLObjectReferenceMapping extends AggregateMapping implements XMLMa
     void setInverseReferenceMapping(XMLInverseReferenceMapping inverseReferenceMapping) {
         this.inverseReferenceMapping = inverseReferenceMapping;
     }
+    @Override
+    public boolean isObjectReferenceMapping() {
+        return true;
+    }
     
 }
\ No newline at end of file
