@@ -1,5 +1,5 @@
 /***************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -12,35 +12,40 @@
  ******************************************************************************/  
 package org.eclipse.persistence.oxm.record;
 
+import java.io.CharArrayWriter;
 import java.io.OutputStream;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Set;
 
 import org.eclipse.persistence.exceptions.XMLMarshalException;
-import org.eclipse.persistence.internal.helper.Helper;
+import org.eclipse.persistence.internal.oxm.Constants;
+import org.eclipse.persistence.internal.oxm.NamespaceResolver;
+import org.eclipse.persistence.internal.oxm.XMLMarshaller;
 import org.eclipse.persistence.internal.oxm.XPathFragment;
+import org.eclipse.persistence.internal.oxm.record.ExtendedContentHandler;
 import org.eclipse.persistence.internal.oxm.record.XMLFragmentReader;
-import org.eclipse.persistence.oxm.NamespaceResolver;
-import org.eclipse.persistence.oxm.XMLConstants;
+import org.eclipse.persistence.oxm.CharacterEscapeHandler;
 import org.w3c.dom.Attr;
 import org.w3c.dom.Node;
 import org.xml.sax.Attributes;
-import org.xml.sax.ContentHandler;
 import org.xml.sax.Locator;
 import org.xml.sax.SAXException;
 import org.xml.sax.ext.LexicalHandler;
 
 /**
  * <p>Use this type of MarshalRecord when the marshal target is an OutputStream and the
- * XML should be not be formatted with carriage returns and indenting.  This type is only
+ * XML should not be formatted with carriage returns or indenting.  This type is only
  * used if the encoding of the OutputStream is UTF-8</p>
  * <p><code>
  * XMLContext xmlContext = new XMLContext("session-name");<br>
  * XMLMarshaller xmlMarshaller = xmlContext.createMarshaller();<br>
  * OutputStreamRecord record = new OutputStreamRecord();<br>
- * record.setOutputStreamr(myOutputStream);<br>
+ * record.setOutputStream(myOutputStream);<br>
  * xmlMarshaller.marshal(myObject, record);<br>
  * </code></p>
  * <p>If the marshal(OutputStream) and setFormattedOutput(false) method is called on
@@ -54,39 +59,25 @@ import org.xml.sax.ext.LexicalHandler;
  * </code></p>
  * @see org.eclipse.persistence.oxm.XMLMarshaller
  */
-public class OutputStreamRecord extends MarshalRecord {
-    protected static byte[] OPEN_XML_PI_AND_VERSION_ATTRIBUTE;
-    protected static byte[] OPEN_ENCODING_ATTRIBUTE;
-    protected static byte[] CLOSE_PI;
-    protected static byte SPACE = (byte) ' ';
-    protected static byte[] CR;
-    protected static byte CLOSE_ATTRIBUTE_VALUE = (byte) '"';
-    protected static byte[] OPEN_CDATA;
-    protected static byte[] CLOSE_CDATA;
-    protected static byte[] OPEN_COMMENT;
-    protected static byte[] CLOSE_COMMENT;
-    protected static byte OPEN_START_ELEMENT = (byte) '<';
-    protected static byte CLOSE_ELEMENT = (byte) '>';
-    protected static byte[] AMP;
-    protected static byte[] LT;
-    protected static byte[] ENCODING;
+public class OutputStreamRecord extends MarshalRecord<XMLMarshaller> {
 
-    static {
-        try {
-            OPEN_XML_PI_AND_VERSION_ATTRIBUTE = "<?xml version=\"".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            OPEN_ENCODING_ATTRIBUTE = " encoding=\"".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            CLOSE_PI = "?>".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            CR = Helper.cr().getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            OPEN_CDATA = "<![CDATA[".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            CLOSE_CDATA = "]]>".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            OPEN_COMMENT = "<!--".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            CLOSE_COMMENT = "-->".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            AMP = "&amp;".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            LT = "&lt;".getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-            ENCODING = XMLConstants.DEFAULT_XML_ENCODING.getBytes(XMLConstants.DEFAULT_XML_ENCODING);
-        } catch (UnsupportedEncodingException e) {
-        }
-    }
+    protected static final byte[] OPEN_XML_PI_AND_VERSION_ATTRIBUTE = "<?xml version=\"".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] OPEN_ENCODING_ATTRIBUTE = " encoding=\"".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] CLOSE_PI = "?>".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte SPACE = (byte) ' ';
+    protected static final byte CLOSE_ATTRIBUTE_VALUE = (byte) '"';
+    protected static final byte[] OPEN_CDATA = "<![CDATA[".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] CLOSE_CDATA = "]]>".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] OPEN_COMMENT = "<!--".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] CLOSE_COMMENT = "-->".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte OPEN_START_ELEMENT = (byte) '<';
+    protected static final byte CLOSE_ELEMENT = (byte) '>';
+    protected static final byte[] AMP = "&amp;".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] LT = "&lt;".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] QUOT = "&quot;".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] ENCODING = Constants.DEFAULT_XML_ENCODING.getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] SLASH_N = "&#xa;".getBytes(Constants.DEFAULT_CHARSET);
+    protected static final byte[] SLASH_R = "&#xd;".getBytes(Constants.DEFAULT_CHARSET);
 
     protected OutputStream outputStream;
     protected boolean isStartElementOpen = false;
@@ -125,7 +116,7 @@ public class OutputStreamRecord extends MarshalRecord {
     public void startDocument(String encoding, String version) {
         try {
             outputStreamWrite(OPEN_XML_PI_AND_VERSION_ATTRIBUTE);
-            outputStreamWrite(version.getBytes(XMLConstants.DEFAULT_XML_ENCODING));
+            outputStreamWrite(version.getBytes(Constants.DEFAULT_XML_ENCODING));
             outputStreamWrite(CLOSE_ATTRIBUTE_VALUE);
             if (null != encoding) {
                 outputStreamWrite(OPEN_ENCODING_ATTRIBUTE);
@@ -133,12 +124,18 @@ public class OutputStreamRecord extends MarshalRecord {
                 outputStreamWrite(CLOSE_ATTRIBUTE_VALUE);
             }
             outputStreamWrite(CLOSE_PI);
-            outputStreamWrite(CR);
         } catch(UnsupportedEncodingException e) {
             throw XMLMarshalException.marshalException(e);
         }
     }
 
+    /**
+     * INTERNAL
+     */
+    public void writeHeader() {
+        outputStreamWrite(getMarshaller().getXmlHeader().getBytes());
+    }
+
     /**
      * INTERNAL:
      */
@@ -154,7 +151,15 @@ public class OutputStreamRecord extends MarshalRecord {
         }
         isStartElementOpen = true;
         outputStreamWrite(OPEN_START_ELEMENT);
-        outputStreamWrite(xPathFragment.getShortNameBytes());
+        byte[] prefixBytes = getPrefixBytes(xPathFragment);
+        if(null != prefixBytes) {
+            outputStreamWrite(prefixBytes);
+            outputStreamWrite((byte)':');
+        }
+        outputStreamWrite(xPathFragment.getLocalNameBytes());
+        if(xPathFragment.isGeneratedPrefix()){
+            namespaceDeclaration(xPathFragment.getPrefix(), xPathFragment.getNamespaceURI());
+        }
     }
 
     /**
@@ -166,7 +171,10 @@ public class OutputStreamRecord extends MarshalRecord {
             isStartElementOpen = false;
         }
         outputStreamWrite(OPEN_START_ELEMENT);
-        outputStreamWrite(frag.getShortNameBytes());
+        try {
+            outputStreamWrite(getNameForFragment(frag).getBytes(Constants.DEFAULT_XML_ENCODING));
+        } catch (UnsupportedEncodingException e) {
+        }
         outputStreamWrite((byte)'/');
         outputStreamWrite((byte)'>');
     }
@@ -175,7 +183,7 @@ public class OutputStreamRecord extends MarshalRecord {
      * INTERNAL:
      */
     public void attribute(XPathFragment xPathFragment, NamespaceResolver namespaceResolver, String value) {
-        attribute(null, xPathFragment.getLocalName(), xPathFragment.getShortName(), value);
+        attribute(null, xPathFragment.getLocalName(), getNameForFragment(xPathFragment), value);
     }
 
     /**
@@ -184,10 +192,10 @@ public class OutputStreamRecord extends MarshalRecord {
     public void attribute(String namespaceURI, String localName, String qName, String value) {
         try {
             outputStreamWrite(SPACE);
-            outputStreamWrite(qName.getBytes(XMLConstants.DEFAULT_XML_ENCODING));
+            outputStreamWrite(qName.getBytes(Constants.DEFAULT_XML_ENCODING));
             outputStreamWrite((byte)'=');
             outputStreamWrite((byte)'"');
-            writeValue(value, false);
+            writeValue(value, true, true, this.outputStream);
             outputStreamWrite(CLOSE_ATTRIBUTE_VALUE);
         } catch (UnsupportedEncodingException e) {
             throw XMLMarshalException.marshalException(e);
@@ -210,7 +218,12 @@ public class OutputStreamRecord extends MarshalRecord {
         } else {
             outputStreamWrite((byte)'<');
             outputStreamWrite((byte)'/');
-            outputStreamWrite(xPathFragment.getShortNameBytes());
+            byte[] prefixBytes = getPrefixBytes(xPathFragment);
+            if(null != prefixBytes) {
+                outputStreamWrite(prefixBytes);
+                outputStreamWrite((byte)':');
+            }
+            outputStreamWrite(xPathFragment.getLocalNameBytes());
             outputStreamWrite(CLOSE_ELEMENT);
         }
         isStartElementOpen = false;
@@ -237,7 +250,7 @@ public class OutputStreamRecord extends MarshalRecord {
                 outputStreamWrite(CLOSE_ELEMENT);
             }
             outputStreamWrite(OPEN_CDATA);
-            outputStreamWrite(value.getBytes(XMLConstants.DEFAULT_XML_ENCODING));
+            outputStreamWrite(value.getBytes(Constants.DEFAULT_XML_ENCODING));
             outputStreamWrite(CLOSE_CDATA);
         } catch(UnsupportedEncodingException e) {
             throw XMLMarshalException.marshalException(e);
@@ -248,6 +261,29 @@ public class OutputStreamRecord extends MarshalRecord {
      * INTERNAL:
      */
     protected void writeValue(String value, boolean escapeChars) {
+        writeValue(value, escapeChars, false, this.outputStream);
+    }
+
+    /**
+     * INTERNAL:
+     */
+    protected void writeValue(String value, boolean escapeChars, boolean isAttribute, OutputStream os) {
+        if (escapeChars) {
+            CharacterEscapeHandler escapeHandler = marshaller.getCharacterEscapeHandler();
+            if (escapeHandler != null) {
+                try {
+                    CharArrayWriter out = new CharArrayWriter();
+                    escapeHandler.escape(value.toCharArray(), 0, value.length(), isAttribute, out);
+                    byte[] bytes = out.toString().getBytes();
+                    outputStreamWrite(bytes);
+                    out.close();
+                } catch (IOException e) {
+                    throw XMLMarshalException.marshalException(e);
+                }
+                return;
+            }
+        }
+
         // UTF-8 Byte Representations:
         //     0x0 - 0x7F                                 0xxxxxxx
         //    0x80 - 0x7FF                       110yyyxx 10xxxxxx
@@ -260,43 +296,58 @@ public class OutputStreamRecord extends MarshalRecord {
                     if((character >= Character.MIN_HIGH_SURROGATE) && (character <= Character.MAX_LOW_SURROGATE)) {
                         int uc = (((character & 0x3ff) << 10) | (value.charAt(++x) & 0x3ff)) + 0x10000;
                         // 11110zzz
-                        outputStreamWrite((byte)(0xF0 | ((uc >> 18))));
+                        outputStreamWrite((byte)(0xF0 | ((uc >> 18))), os);
                         // 10zzyyyy
-                        outputStreamWrite((byte)(0x80 | ((uc >> 12) & 0x3F)));
+                        outputStreamWrite((byte)(0x80 | ((uc >> 12) & 0x3F)), os);
                         // 10yyyyxx
-                        outputStreamWrite((byte)(0x80 | ((uc >> 6) & 0x3F)));
+                        outputStreamWrite((byte)(0x80 | ((uc >> 6) & 0x3F)), os);
                         // 10xxxxxx
-                        outputStreamWrite((byte)(0x80 + (uc & 0x3F)));
+                        outputStreamWrite((byte)(0x80 + (uc & 0x3F)), os);
                        continue;
                     } else {
                         // 1110yyyy
-                        outputStreamWrite((byte)(0xE0 + (character >> 12)));
+                        outputStreamWrite((byte)(0xE0 + (character >> 12)), os);
                     }
                     // 10yyyyxx
-                    outputStreamWrite((byte)(0x80 + ((character >> 6) & 0x3F)));
+                    outputStreamWrite((byte)(0x80 + ((character >> 6) & 0x3F)), os);
                 } else {
                     // 110yyyxx
-                    outputStreamWrite((byte)(0xC0 + (character >> 6)));
+                    outputStreamWrite((byte)(0xC0 + (character >> 6)), os);
                 }
-                // outputStreamWrite((byte)(0x80 + (character & 0x3F)));
-                outputStreamWrite((byte)(0x80 + (character & 0x3F)));
+                outputStreamWrite((byte)(0x80 + (character & 0x3F)), os);
             } else {
                 // 0xxxxxxx
                 if(escapeChars) {
                     switch (character) {
                     case '&': {
-                        outputStreamWrite(AMP);
+                        outputStreamWrite(AMP, os);
                         break;
                     }
                     case '<': {
-                        outputStreamWrite(LT);
+                        outputStreamWrite(LT, os);
+                        break;
+                    }
+                    case '"': {
+                        outputStreamWrite(QUOT, os);
+                        break;
+                    }
+                    case '\n' : {
+                        if(isAttribute) {
+                            outputStreamWrite(SLASH_N, os);
+                        } else {
+                            outputStreamWrite((byte) character, os);
+                        }
+                        break;
+                    }
+                    case '\r': {
+                        outputStreamWrite(SLASH_R, os);
                         break;
                     }
                     default:
-                        outputStreamWrite((byte) character);
+                        outputStreamWrite((byte) character, os);
                     }
                 } else {
-                    outputStreamWrite((byte) character);
+                    outputStreamWrite((byte) character, os);
                 }
             }
         }
@@ -308,22 +359,23 @@ public class OutputStreamRecord extends MarshalRecord {
      * @param namespaceResolver The NamespaceResolver can be used to resolve the
      * namespace URI/prefix of the node
      */
-    public void node(Node node, NamespaceResolver namespaceResolver) {
+    public void node(Node node, NamespaceResolver namespaceResolver, String uri, String localName) {
         if (node.getNodeType() == Node.ATTRIBUTE_NODE) {
             Attr attr = (Attr) node;
             String resolverPfx = null;
-            if (namespaceResolver != null) {
-                resolverPfx = namespaceResolver.resolveNamespaceURI(attr.getNamespaceURI());
+            if (getNamespaceResolver() != null) {
+                resolverPfx = this.getNamespaceResolver().resolveNamespaceURI(attr.getNamespaceURI());
             }
             // If the namespace resolver contains a prefix for the attribute's URI,
             // use it instead of what is set on the attribute
             if (resolverPfx != null) {
-                attribute(attr.getNamespaceURI(), XMLConstants.EMPTY_STRING, resolverPfx+XMLConstants.COLON+attr.getLocalName(), attr.getNodeValue());
+                attribute(attr.getNamespaceURI(), Constants.EMPTY_STRING, resolverPfx+Constants.COLON+attr.getLocalName(), attr.getNodeValue());
             } else {
-                attribute(attr.getNamespaceURI(), XMLConstants.EMPTY_STRING, attr.getName(), attr.getNodeValue());
+                attribute(attr.getNamespaceURI(), Constants.EMPTY_STRING, attr.getName(), attr.getNodeValue());
                 // May need to declare the URI locally
-                if (attr.getNamespaceURI() != null) {
-                    attribute(XMLConstants.XMLNS_URL, XMLConstants.EMPTY_STRING,XMLConstants.XMLNS + XMLConstants.COLON + attr.getPrefix(), attr.getNamespaceURI());
+                if (attr.getNamespaceURI() != null) {                    
+                    namespaceDeclaration(attr.getPrefix(), attr.getNamespaceURI());
+                    this.getNamespaceResolver().put(attr.getPrefix(), attr.getNamespaceURI());
                 }
             }
         } else if (node.getNodeType() == Node.TEXT_NODE) {
@@ -333,8 +385,8 @@ public class OutputStreamRecord extends MarshalRecord {
                 OutputStreamRecordContentHandler handler = new OutputStreamRecordContentHandler();
                 XMLFragmentReader xfragReader = new XMLFragmentReader(namespaceResolver);
                 xfragReader.setContentHandler(handler);
-                xfragReader.setProperty("http://xml.org/sax/properties/lexical-handler", handler);
-                xfragReader.parse(node);
+                xfragReader.setProperty(Constants.LEXICAL_HANDLER_PROPERTY, handler);
+                xfragReader.parse(node, uri, localName);
             } catch (SAXException sex) {
                 throw XMLMarshalException.marshalException(sex);
             }
@@ -349,7 +401,7 @@ public class OutputStreamRecord extends MarshalRecord {
      *
      * @see org.eclipse.persistence.internal.oxm.record.XMLFragmentReader
      */
-    protected class OutputStreamRecordContentHandler implements ContentHandler, LexicalHandler {
+    protected class OutputStreamRecordContentHandler implements ExtendedContentHandler, LexicalHandler {
         Map<String, String> prefixMappings;
 
         OutputStreamRecordContentHandler() {
@@ -363,7 +415,7 @@ public class OutputStreamRecord extends MarshalRecord {
                     outputStreamWrite(CLOSE_ELEMENT);
                 }
                 outputStreamWrite(OPEN_START_ELEMENT);
-                outputStreamWrite(qName.getBytes(XMLConstants.DEFAULT_XML_ENCODING));
+                outputStreamWrite(qName.getBytes(Constants.DEFAULT_XML_ENCODING));
                 isStartElementOpen = true;
                 // Handle attributes
                 handleAttributes(atts);
@@ -382,7 +434,7 @@ public class OutputStreamRecord extends MarshalRecord {
                 } else {
                     outputStreamWrite((byte) '<');
                     outputStreamWrite((byte) '/');
-                    outputStreamWrite(qName.getBytes(XMLConstants.DEFAULT_XML_ENCODING));
+                    outputStreamWrite(qName.getBytes(Constants.DEFAULT_XML_ENCODING));
                     outputStreamWrite(CLOSE_ELEMENT);
                 }
                 isStartElementOpen = false;
@@ -399,8 +451,13 @@ public class OutputStreamRecord extends MarshalRecord {
         }
 
         public void characters(char[] ch, int start, int length) throws SAXException {
+            String characters = new String (ch, start, length);
+            characters(characters);
+        }
+
+        public void characters(CharSequence characters) throws SAXException {
             if (isProcessingCData) {
-                cdata(new String (ch, start, length));
+                cdata(characters.toString());
                 return;
             }
 
@@ -408,7 +465,7 @@ public class OutputStreamRecord extends MarshalRecord {
                 outputStreamWrite(CLOSE_ELEMENT);
                 isStartElementOpen = false;
             }
-            writeValue(new String(ch, start, length), true);
+            writeValue(characters.toString(), true);
         }
 
         // --------------------- LEXICALHANDLER METHODS --------------------- //
@@ -432,17 +489,23 @@ public class OutputStreamRecord extends MarshalRecord {
         protected void writePrefixMappings() {
             try {
                 if (!prefixMappings.isEmpty()) {
-                    for (java.util.Iterator<String> keys = prefixMappings.keySet().iterator(); keys.hasNext();) {
-                        String prefix = keys.next();
+                	Set<Entry<String, String>> entries = prefixMappings.entrySet();
+                	Iterator<Entry<String, String>> iter = entries.iterator();
+                	while(iter.hasNext()){
+                		Entry<String, String> nextEntry = iter.next();
+                        String prefix = nextEntry.getKey();
                         outputStreamWrite(SPACE);
-                        outputStreamWrite(XMLConstants.XMLNS.getBytes(XMLConstants.DEFAULT_XML_ENCODING));
-                        if(prefix.length() > 0) {
-                            outputStreamWrite((byte)XMLConstants.COLON);
-                            outputStreamWrite(prefix.getBytes(XMLConstants.DEFAULT_XML_ENCODING));
+                        outputStreamWrite(javax.xml.XMLConstants.XMLNS_ATTRIBUTE.getBytes(Constants.DEFAULT_XML_ENCODING));
+                        if(null != prefix && prefix.length() > 0) {
+                            outputStreamWrite((byte)Constants.COLON);
+                            outputStreamWrite(prefix.getBytes(Constants.DEFAULT_XML_ENCODING));
                         }
                         outputStreamWrite((byte)'=');
                         outputStreamWrite((byte)'"');
-                        outputStreamWrite(prefixMappings.get(prefix).getBytes(XMLConstants.DEFAULT_XML_ENCODING));
+                        String uri = nextEntry.getValue();
+                        if(null != uri) {
+                            outputStreamWrite(uri.getBytes(Constants.DEFAULT_XML_ENCODING));
+                        }
                         outputStreamWrite(CLOSE_ATTRIBUTE_VALUE);
                     }
                     prefixMappings.clear();
@@ -455,7 +518,7 @@ public class OutputStreamRecord extends MarshalRecord {
         protected void handleAttributes(Attributes atts) {
             for (int i=0, attsLength = atts.getLength(); i<attsLength; i++) {
                 String qName = atts.getQName(i);
-                if((qName != null && (qName.startsWith(XMLConstants.XMLNS + XMLConstants.COLON) || qName.equals(XMLConstants.XMLNS)))) {
+                if((qName != null && (qName.startsWith(javax.xml.XMLConstants.XMLNS_ATTRIBUTE + Constants.COLON) || qName.equals(javax.xml.XMLConstants.XMLNS_ATTRIBUTE)))) {
                     continue;
                 }
                 attribute(atts.getURI(i), atts.getLocalName(i), qName, atts.getValue(i));
@@ -485,6 +548,10 @@ public class OutputStreamRecord extends MarshalRecord {
         public void endEntity(String name) throws SAXException {}
         public void startDTD(String name, String publicId, String systemId) throws SAXException {}
         public void endDTD() throws SAXException {}
+
+
+        @Override
+        public void setNil(boolean isNil) {}
     }
 
     public void flush() {
@@ -498,13 +565,26 @@ public class OutputStreamRecord extends MarshalRecord {
     }
 
     protected void outputStreamWrite(byte[] bytes) {
+        outputStreamWrite(bytes, this.outputStream);
+    }
+
+    protected void outputStreamWrite(byte[] bytes, OutputStream os) {
+        if (os != this.outputStream) {
+            // Not using our buffer
+            try {
+                os.write(bytes);
+                return;
+            } catch (IOException e) {
+                throw XMLMarshalException.marshalException(e);
+            }
+        }
         int bytesLength = bytes.length;
         if(bufferIndex + bytesLength >= BUFFER_SIZE) {
             try {
-                outputStream.write(buffer, 0, bufferIndex);
+                os.write(buffer, 0, bufferIndex);
                 bufferIndex = 0;
                 if(bytesLength > BUFFER_SIZE) {
-                    outputStream.write(bytes);
+                    os.write(bytes);
                     return;
                 }
             } catch(IOException e) {
@@ -516,9 +596,22 @@ public class OutputStreamRecord extends MarshalRecord {
     }
 
     protected void outputStreamWrite(byte aByte) {
+        outputStreamWrite(aByte, this.outputStream);
+    }
+
+    protected void outputStreamWrite(byte aByte, OutputStream os) {
+        if (os != this.outputStream) {
+            // Not using our buffer
+            try {
+                os.write(aByte);
+                return;
+            } catch (IOException e) {
+                throw XMLMarshalException.marshalException(e);
+            }
+        }
         if(bufferIndex == BUFFER_SIZE) {
             try {
-                outputStream.write(buffer, 0, BUFFER_SIZE);
+                os.write(buffer, 0, BUFFER_SIZE);
                 bufferIndex = 0;
             } catch(IOException e) {
                 throw XMLMarshalException.marshalException(e);
