@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -14,26 +14,32 @@ package org.eclipse.persistence.oxm.record;
 
 import java.io.Writer;
 import java.io.IOException;
+import java.nio.charset.Charset;
+import java.nio.charset.CharsetEncoder;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Set;
 
 import org.eclipse.persistence.exceptions.XMLMarshalException;
-import org.eclipse.persistence.internal.helper.Helper;
+import org.eclipse.persistence.internal.oxm.Constants;
+import org.eclipse.persistence.internal.oxm.NamespaceResolver;
+import org.eclipse.persistence.internal.oxm.XMLMarshaller;
 import org.eclipse.persistence.internal.oxm.XPathFragment;
+import org.eclipse.persistence.internal.oxm.record.ExtendedContentHandler;
 import org.eclipse.persistence.internal.oxm.record.XMLFragmentReader;
-import org.eclipse.persistence.oxm.NamespaceResolver;
-import org.eclipse.persistence.oxm.XMLConstants;
+import org.eclipse.persistence.oxm.CharacterEscapeHandler;
 import org.w3c.dom.Attr;
 import org.w3c.dom.Node;
 import org.xml.sax.Attributes;
-import org.xml.sax.ContentHandler;
 import org.xml.sax.Locator;
 import org.xml.sax.SAXException;
 import org.xml.sax.ext.LexicalHandler;
 
 /**
  * <p>Use this type of MarshalRecord when the marshal target is a Writer and the
- * XML should be not be formatted with carriage returns and indenting.</p>
+ * XML should not be formatted with carriage returns or indenting.</p>
  * <p><code>
  * XMLContext xmlContext = new XMLContext("session-name");<br>
  * XMLMarshaller xmlMarshaller = xmlContext.createMarshaller();<br>
@@ -52,11 +58,12 @@ import org.xml.sax.ext.LexicalHandler;
  * </code></p>
  * @see org.eclipse.persistence.oxm.XMLMarshaller
  */
-public class WriterRecord extends MarshalRecord {
+public class WriterRecord extends MarshalRecord<XMLMarshaller> {
 
     protected Writer writer;
     protected boolean isStartElementOpen = false;
     protected boolean isProcessingCData = false;
+    protected CharsetEncoder encoder;
 
     /**
      * Return the Writer that the object will be marshalled to.
@@ -88,7 +95,17 @@ public class WriterRecord extends MarshalRecord {
                 writer.write('\"');
             }
             writer.write("?>");
-            writer.write(Helper.cr());
+        } catch (IOException e) {
+            throw XMLMarshalException.marshalException(e);
+        }
+    }
+
+    /**
+     * INTERNAL
+     */
+    public void writeHeader() {
+        try {
+            writer.write(getMarshaller().getXmlHeader());
         } catch (IOException e) {
             throw XMLMarshalException.marshalException(e);
         }
@@ -110,7 +127,10 @@ public class WriterRecord extends MarshalRecord {
             }
             isStartElementOpen = true;
             writer.write('<');
-            writer.write(xPathFragment.getShortName());
+            writer.write(getNameForFragment(xPathFragment));
+            if(xPathFragment.isGeneratedPrefix()){
+    		    namespaceDeclaration(xPathFragment.getPrefix(), xPathFragment.getNamespaceURI());
+    	    }
         } catch (IOException e) {
             throw XMLMarshalException.marshalException(e);
         }
@@ -126,7 +146,7 @@ public class WriterRecord extends MarshalRecord {
                 isStartElementOpen = false;
             }
             writer.write('<');
-            writer.write(frag.getShortName());
+            writer.write(getNameForFragment(frag));
             writer.write('/');
             writer.write('>');
         } catch (IOException e) {
@@ -138,7 +158,7 @@ public class WriterRecord extends MarshalRecord {
      * INTERNAL:
      */
     public void attribute(XPathFragment xPathFragment, NamespaceResolver namespaceResolver, String value) {
-        attribute(null, xPathFragment.getLocalName(), xPathFragment.getShortName(), value);
+        attribute(null, xPathFragment.getLocalName(), getNameForFragment(xPathFragment), value);
     }
 
     /**
@@ -148,13 +168,6 @@ public class WriterRecord extends MarshalRecord {
     public void startPrefixMappings(NamespaceResolver namespaceResolver) {
     }
 
-    /**
-     * INTERNAL:
-     * override so we don't iterate over namespaces when endPrefixMapping doesn't do anything
-     */
-    public void endPrefixMappings(NamespaceResolver namespaceResolver) {
-    }
-
     /**
      * INTERNAL:
      */
@@ -164,7 +177,7 @@ public class WriterRecord extends MarshalRecord {
             writer.write(qName);
             writer.write('=');
             writer.write('\"');
-            writeValue(value);
+            writeValue(value, true, this.writer);
             writer.write('\"');
         } catch (IOException e) {
             throw XMLMarshalException.marshalException(e);
@@ -188,7 +201,7 @@ public class WriterRecord extends MarshalRecord {
             } else {
                 writer.write('<');
                 writer.write('/');
-                writer.write(xPathFragment.getShortName());
+                writer.write(getNameForFragment(xPathFragment));
                 writer.write('>');
             }
             isStartElementOpen = false;
@@ -233,8 +246,28 @@ public class WriterRecord extends MarshalRecord {
      * INTERNAL:
      */
     protected void writeValue(String value) {
+        writeValue(value, false, this.writer);
+    }
+
+    /**
+     * INTERNAL:
+     */
+    protected void writeValue(String value, boolean isAttribute, Writer writer) {
+        CharacterEscapeHandler escapeHandler = null;
+        if (marshaller != null) {
+            escapeHandler = marshaller.getCharacterEscapeHandler();
+        }
+        if (escapeHandler != null) {
+            try {
+                escapeHandler.escape(value.toCharArray(), 0, value.length(), isAttribute, writer);
+            } catch (IOException e) {
+                throw XMLMarshalException.marshalException(e);
+            }
+            return;
+        }
+
         try {
-            if(value.indexOf('&') > -1 || value.indexOf('<') > -1) {
+            if((null != encoder && encoder.maxBytesPerChar() < 4) || value.indexOf('"') > -1 || value.indexOf('&') > -1 || value.indexOf('<') > -1 || value.indexOf('\n') > -1 || value.indexOf('\r') > -1) {
                   char[] chars = value.toCharArray();
                   for (int x = 0, charsSize = chars.length; x < charsSize; x++) {
                       char character = chars[x];
@@ -251,8 +284,29 @@ public class WriterRecord extends MarshalRecord {
                           writer.write("&quot;");
                           break;
                       }
+                      case '\n' : {
+                          if(isAttribute) {
+                              writer.write("&#xa;");
+                          } else {
+                              writer.write('\n');
+                          }
+                          break;
+                      }
+                      case '\r': {
+                          writer.write("&#xd;");
+                          break;
+                      }
                       default:
-                          writer.write(character);
+                          if(null == encoder) {
+                              encoder = Constants.DEFAULT_CHARSET.newEncoder();
+                          }
+                          if(encoder.canEncode(character)) {
+                              writer.write(character);
+                          } else {
+                              writer.write("&#");
+                              writer.write(String.valueOf((int) character));
+                              writer.write(';');
+                          }
                       }
                   }
             } else {
@@ -269,22 +323,24 @@ public class WriterRecord extends MarshalRecord {
      * @param namespaceResolver The NamespaceResolver can be used to resolve the
      * namespace URI/prefix of the node
      */
-    public void node(Node node, NamespaceResolver namespaceResolver) {
+    public void node(Node node, NamespaceResolver namespaceResolver, String newNamespace, String newName) {
         if (node.getNodeType() == Node.ATTRIBUTE_NODE) {
             Attr attr = (Attr) node;
             String resolverPfx = null;
-            if (namespaceResolver != null) {
-                resolverPfx = namespaceResolver.resolveNamespaceURI(attr.getNamespaceURI());
-            }
+            if (getNamespaceResolver() != null) {
+                resolverPfx = this.getNamespaceResolver().resolveNamespaceURI(attr.getNamespaceURI());
+            } 
+            String namespaceURI = attr.getNamespaceURI();
             // If the namespace resolver contains a prefix for the attribute's URI,
             // use it instead of what is set on the attribute
             if (resolverPfx != null) {
-                attribute(attr.getNamespaceURI(), XMLConstants.EMPTY_STRING, resolverPfx+XMLConstants.COLON+attr.getLocalName(), attr.getNodeValue());
+                attribute(attr.getNamespaceURI(), Constants.EMPTY_STRING, resolverPfx+Constants.COLON+attr.getLocalName(), attr.getNodeValue());
             } else {
-                attribute(attr.getNamespaceURI(), XMLConstants.EMPTY_STRING, attr.getName(), attr.getNodeValue());
+                attribute(attr.getNamespaceURI(), Constants.EMPTY_STRING, attr.getName(), attr.getNodeValue());
                 // May need to declare the URI locally
                 if (attr.getNamespaceURI() != null) {
-                    attribute(XMLConstants.XMLNS_URL, XMLConstants.EMPTY_STRING,XMLConstants.XMLNS + XMLConstants.COLON + attr.getPrefix(), attr.getNamespaceURI());
+                    namespaceDeclaration(attr.getPrefix(), attr.getNamespaceURI());
+                    this.getNamespaceResolver().put(attr.getPrefix(), attr.getNamespaceURI());
                 }
             }
         } else if (node.getNodeType() == Node.TEXT_NODE) {
@@ -295,7 +351,7 @@ public class WriterRecord extends MarshalRecord {
                 XMLFragmentReader xfragReader = new XMLFragmentReader(namespaceResolver);
                 xfragReader.setContentHandler(wrcHandler);
                 xfragReader.setProperty("http://xml.org/sax/properties/lexical-handler", wrcHandler);
-                xfragReader.parse(node);
+                xfragReader.parse(node, newNamespace, newName);
             } catch (SAXException sex) {
                 throw XMLMarshalException.marshalException(sex);
             }
@@ -310,7 +366,7 @@ public class WriterRecord extends MarshalRecord {
      *
      * @see org.eclipse.persistence.internal.oxm.record.XMLFragmentReader
      */
-    protected class WriterRecordContentHandler implements ContentHandler, LexicalHandler {
+    protected class WriterRecordContentHandler implements ExtendedContentHandler, LexicalHandler {
         Map<String, String> prefixMappings;
 
         WriterRecordContentHandler() {
@@ -361,8 +417,13 @@ public class WriterRecord extends MarshalRecord {
         }
 
         public void characters(char[] ch, int start, int length) throws SAXException {
+            String characters = new String (ch, start, length);
+            characters(characters);
+        }
+
+        public void characters(CharSequence characters) throws SAXException {
             if (isProcessingCData) {
-                cdata(new String (ch, start, length));
+                cdata(characters.toString());
                 return;
             }
 
@@ -374,9 +435,9 @@ public class WriterRecord extends MarshalRecord {
                     throw XMLMarshalException.marshalException(e);
                 }
             }
-            writeValue(new String(ch, start, length));
+            writeValue(characters.toString());
         }
-
+        
         // --------------------- LEXICALHANDLER METHODS --------------------- //
         public void comment(char[] ch, int start, int length) throws SAXException {
             try {
@@ -402,17 +463,23 @@ public class WriterRecord extends MarshalRecord {
         protected void writePrefixMappings() {
             try {
                 if (!prefixMappings.isEmpty()) {
-                    for (java.util.Iterator<String> keys = prefixMappings.keySet().iterator(); keys.hasNext();) {
-                        String prefix = keys.next();
+                	Set<Entry<String, String>> entries = prefixMappings.entrySet();
+                	Iterator<Entry<String, String>> iter = entries.iterator();
+                	while(iter.hasNext()){
+                		Entry<String, String> nextEntry = iter.next();
+                        String prefix = nextEntry.getKey();
                         writer.write(' ');
-                        writer.write(XMLConstants.XMLNS);
-                        if(prefix.length() > 0) {
-                            writer.write(XMLConstants.COLON);
+                        writer.write(javax.xml.XMLConstants.XMLNS_ATTRIBUTE);
+                        if(null != prefix && prefix.length() > 0) {
+                            writer.write(Constants.COLON);
                             writer.write(prefix);
                         }
                         writer.write('=');
                         writer.write('"');
-                        writer.write(prefixMappings.get(prefix));
+                        String uri = nextEntry.getValue();
+                        if(null != uri) {
+                            writer.write(uri);
+                        }
                         writer.write('"');
                     }
                     prefixMappings.clear();
@@ -425,7 +492,7 @@ public class WriterRecord extends MarshalRecord {
         protected void handleAttributes(Attributes atts) {
             for (int i=0, attsLength = atts.getLength(); i<attsLength; i++) {
                 String qName = atts.getQName(i);
-                if((qName != null && (qName.startsWith(XMLConstants.XMLNS + XMLConstants.COLON) || qName.equals(XMLConstants.XMLNS)))) {
+                if((qName != null && (qName.startsWith(javax.xml.XMLConstants.XMLNS_ATTRIBUTE + Constants.COLON) || qName.equals(javax.xml.XMLConstants.XMLNS_ATTRIBUTE)))) {
                     continue;
                 }
                 attribute(atts.getURI(i), atts.getLocalName(i), qName, atts.getValue(i));
@@ -472,6 +539,24 @@ public class WriterRecord extends MarshalRecord {
         public void endEntity(String name) throws SAXException {}
         public void startDTD(String name, String publicId, String systemId) throws SAXException {}
         public void endDTD() throws SAXException {}
+		@Override
+		public void setNil(boolean isNil) {}
+
+    }
+
+    @Override
+    public void setMarshaller(XMLMarshaller marshaller) {
+        super.setMarshaller(marshaller);
+        encoder = Charset.forName(marshaller.getEncoding()).newEncoder();
+    }
+
+    @Override
+    public void flush() {
+        try {
+            writer.flush();
+        } catch(IOException e) {
+            throw XMLMarshalException.marshalException(e);
+        }
     }
 
 }
