@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright (c) 1998, 2010 Oracle. All rights reserved.
+* Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
 * which accompanies this distribution.
@@ -12,7 +12,7 @@
 ******************************************************************************/
 package org.eclipse.persistence.oxm.record;
 
-import org.eclipse.persistence.oxm.XMLField;
+import org.eclipse.persistence.internal.oxm.mappings.Field;
 
 /**
  * INTERNAL:
@@ -24,13 +24,13 @@ import org.eclipse.persistence.oxm.XMLField;
  */
 public class XMLEntry {
     private Object value;
-    private XMLField xmlField;
+    private Field xmlField;
     
     public Object getValue() {
         return value;
     }
     
-    public XMLField getXMLField() {
+    public Field getXMLField() {
         return xmlField;
     }
     
@@ -38,7 +38,7 @@ public class XMLEntry {
         this.value = value;
     }
     
-    public void setXMLField(XMLField xmlField) {
+    public void setXMLField(Field xmlField) {
         this.xmlField = xmlField;
     }
 
