@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -10,6 +10,10 @@
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
  *     James Sutherland - Update to HSQL 1.8.1 and LRG
+ *     11/04/2010-2.2 Michael O'Brien 
+ *       - 319592: HSQL 2.0.0 requires VARCHAR length specified
+ *     09/14/2011-2.3.1 Guy Pelletier 
+ *       - 357533: Allow DDL queries to execute even when Multitenant entities are part of the PU
  ******************************************************************************/  
 package org.eclipse.persistence.platform.database;
 
@@ -69,6 +73,9 @@ public class HSQLPlatform extends DatabasePlatform {
         fieldTypeMapping.put(Character[].class, new FieldTypeDefinition("LONGVARCHAR", false));
         fieldTypeMapping.put(byte[].class, new FieldTypeDefinition("LONGVARBINARY", false));
         fieldTypeMapping.put(char[].class, new FieldTypeDefinition("LONGVARCHAR", false));
+        // 319592: HSQL 2.0.0 requires VARCHAR length specified
+        fieldTypeMapping.put(String.class, new FieldTypeDefinition("VARCHAR",DEFAULT_VARCHAR_SIZE));
+        
         fieldTypeMapping.put(java.sql.Blob.class, new FieldTypeDefinition("LONGVARBINARY", false));
         fieldTypeMapping.put(java.sql.Clob.class, new FieldTypeDefinition("LONGVARCHAR", false));
         
@@ -198,7 +205,7 @@ public class HSQLPlatform extends DatabasePlatform {
     @Override
     public void printFieldIdentityClause(Writer writer) throws ValidationException {
         try {
-            writer.write(" GENERATED BY DEFAULT AS IDENTITY (START WITH 1, INCREMENT BY 1)");
+            writer.write(" GENERATED BY DEFAULT AS IDENTITY (START WITH 1 INCREMENT BY 1)");
         } catch (IOException ex) {
             throw ValidationException.fileError(ex);
         }
@@ -346,6 +353,7 @@ public class HSQLPlatform extends DatabasePlatform {
         if (this.timestampQuery == null) {
             this.timestampQuery = new ValueReadQuery();
             this.timestampQuery.setSQLString("CALL CURRENT_TIMESTAMP");
+            this.timestampQuery.setAllowNativeSQLQuery(true);
         }
         return this.timestampQuery;
     }
