@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,21 +9,27 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
- *     Zoltan NAGY & tware - added implementation of updateMaxRowsForQuery 
+ *     Zoltan NAGY & tware - added implementation of updateMaxRowsForQuery
+ *     09/14/2011-2.3.1 Guy Pelletier 
+ *       - 357533: Allow DDL queries to execute even when Multitenant entities are part of the PU 
+ *     02/04/2013-2.5 Guy Pelletier 
+ *       - 389090: JPA 2.1 DDL Generation Support
  ******************************************************************************/  
 package org.eclipse.persistence.platform.database;
 
 import java.io.*;
-import java.sql.Statement;
 import java.util.*;
+
 import org.eclipse.persistence.exceptions.ValidationException;
 import org.eclipse.persistence.expressions.ExpressionOperator;
 import org.eclipse.persistence.internal.databaseaccess.DatabaseCall;
+import org.eclipse.persistence.internal.databaseaccess.DatasourcePlatform;
 import org.eclipse.persistence.internal.databaseaccess.FieldTypeDefinition;
 import org.eclipse.persistence.internal.expressions.FunctionExpression;
 import org.eclipse.persistence.internal.expressions.ExpressionSQLPrinter;
 import org.eclipse.persistence.internal.expressions.SQLSelectStatement;
 import org.eclipse.persistence.internal.helper.*;
+import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.queries.StoredProcedureCall;
 import org.eclipse.persistence.queries.ValueReadQuery;
@@ -56,6 +62,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * Appends an MySQL specific date if usesNativeSQL is true otherwise use the ODBC format.
      * Native FORMAT: 'YYYY-MM-DD'
      */
+    @Override
     protected void appendDate(java.sql.Date date, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("'");
@@ -70,6 +77,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * Appends an MySQL specific time if usesNativeSQL is true otherwise use the ODBC format.
      * Native FORMAT: 'HH:MM:SS'.
      */
+    @Override
     protected void appendTime(java.sql.Time time, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("'");
@@ -84,6 +92,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * Appends an MySQL specific Timestamp, if usesNativeSQL is true otherwise use the ODBC format.
      * Native Format: 'YYYY-MM-DD HH:MM:SS' 
      */
+    @Override
     protected void appendTimestamp(java.sql.Timestamp timestamp, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("'");
@@ -98,6 +107,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * Appends an MySQL specific Timestamp, if usesNativeSQL is true otherwise use the ODBC format.
      * Native Format: 'YYYY-MM-DD HH:MM:SS'
      */
+    @Override
     protected void appendCalendar(Calendar calendar, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("'");
@@ -111,6 +121,7 @@ public class MySQLPlatform extends DatabasePlatform {
     /**
      * Return the mapping of class types to database types for the schema framework.
      */
+    @Override
     protected Hashtable buildFieldTypes() {
         Hashtable fieldTypeMapping;
 
@@ -148,6 +159,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * Build the identity query for native sequencing.
      */
+    @Override
     public ValueReadQuery buildSelectQueryForIdentity() {
         ValueReadQuery selectQuery = new ValueReadQuery();
         StringWriter writer = new StringWriter();
@@ -157,11 +169,11 @@ public class MySQLPlatform extends DatabasePlatform {
     }
 
     /**
-     * Return the proc syntax for this platform.
+     * Return the stored procedure syntax for this platform.
      */
     @Override
-    public String buildProcedureCallString(StoredProcedureCall call, AbstractSession session) {
-        return "{ " + super.buildProcedureCallString(call, session);
+    public String buildProcedureCallString(StoredProcedureCall call, AbstractSession session, AbstractRecord row) {
+        return "{ " + super.buildProcedureCallString(call, session, row);
     }
     
     /**
@@ -184,33 +196,13 @@ public class MySQLPlatform extends DatabasePlatform {
     public int computeMaxRowsForSQL(int firstResultIndex, int maxResults){
         return maxResults - ((firstResultIndex >= 0) ? firstResultIndex : 0);
     }
-    
-    /**
-     * Internal: This gets called on each batch statement execution
-     * Needs to be implemented so that it returns the number of rows successfully modified
-     * by this statement for optimistic locking purposes (if useNativeBatchWriting is enabled, and 
-     * the call uses optimistic locking).  
-     * 
-     * @param isStatementPrepared - flag is set to true if this statement is prepared 
-     * @return - number of rows modified/deleted by this statement
-     */
-    public int executeBatch(Statement statement, boolean isStatementPrepared) throws java.sql.SQLException {
-       int[] updateResult = statement.executeBatch();
-       int updateCount = 0;
-       for (int count : updateResult){
-           if (count == Statement.SUCCESS_NO_INFO){
-               count = 1;
-           }
-           updateCount+=count;
-       }
-       return updateCount;
-    }
 
     /**
      * INTERNAL:
      * Supports Batch Writing with Optimistic Locking.
      */
-    public boolean canBatchWriteWithOptimisticLocking(){
+    @Override
+    public boolean canBatchWriteWithOptimisticLocking(DatabaseCall call){
         return true;
     }
 
@@ -218,17 +210,27 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * Used for constraint deletion.
      */
+    @Override
     public String getConstraintDeletionString() {
         return " DROP FOREIGN KEY ";
     }
 
+    /**
+     * INTERNAL:
+     * Used for unique constraint deletion.
+     */
+    @Override
+    public String getUniqueConstraintDeletionString() {
+    	return " DROP KEY ";
+    }
+    
     /**
      * Used for stored function calls.
      */
     @Override
     public String getFunctionCallHeader() {
         return "? " + getAssignmentString() + getProcedureCallHeader();
-            // different order -  CALL clause ^^^ comes AFTER assignment operator
+        // different order -  CALL clause ^^^ comes AFTER assignment operator
     }
 
     /**
@@ -243,19 +245,27 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * Used for pessimistic locking.
      */
+    @Override
     public String getSelectForUpdateString() {
         return " FOR UPDATE";
     }
     
+    @Override
+    public boolean isForUpdateCompatibleWithDistinct() {
+        return false;
+    }
+
     /**
      * INTERNAL:
      * This method returns the query to select the timestamp
      * from the server for MySQL.
      */
+    @Override
     public ValueReadQuery getTimestampQuery() {
         if (timestampQuery == null) {
             timestampQuery = new ValueReadQuery();
             timestampQuery.setSQLString("SELECT NOW()");
+            timestampQuery.setAllowNativeSQLQuery(true);
         }
         return timestampQuery;
     }
@@ -263,6 +273,7 @@ public class MySQLPlatform extends DatabasePlatform {
     /**
      * Answers whether platform is MySQL.
      */
+    @Override
     public boolean isMySQL() {
         return true;
     }
@@ -270,6 +281,7 @@ public class MySQLPlatform extends DatabasePlatform {
     /**
      * Initialize any platform-specific operators.
      */
+    @Override
     protected void initializePlatformOperators() {
         super.initializePlatformOperators();
         addOperator(logOperator());
@@ -424,6 +436,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * Append the receiver's field 'identity' constraint clause to a writer.
      */
+    @Override
     public void printFieldIdentityClause(Writer writer) throws ValidationException {
         try {
             writer.write(" AUTO_INCREMENT");
@@ -436,6 +449,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * JDBC defines an outer join syntax which many drivers do not support. So we normally avoid it.
      */
+    @Override
     public boolean shouldUseJDBCOuterJoinSyntax() {
         return false;
     }
@@ -445,14 +459,34 @@ public class MySQLPlatform extends DatabasePlatform {
      * Indicates whether the platform supports identity.
      * MySQL supports native sequencing through AUTO_INCREMENT field types.
      */
+    @Override
     public boolean supportsIdentity() {
         return true;
     }
 
+    /**
+     * INTERNAL:
+     * Indicates whether the platform supports the count distinct function with multiple fields.
+     */
+    @Override
+    public boolean supportsCountDistinctWithMultipleFields() {
+        return true;
+    }
+    
+    /**
+     * INTERNAL:
+     * Return if this database requires the table name when dropping an index.
+     */
+    @Override
+    public boolean requiresTableInIndexDropDDL() {
+        return true;
+    }
+    
     /**
      * INTERNAL:
      * MySQL supports temp tables for update-all, delete-all queries.
      */
+    @Override
     public boolean supportsGlobalTempTables() {
         return true;
     }
@@ -466,6 +500,7 @@ public class MySQLPlatform extends DatabasePlatform {
      *   on SQLServer platform (method returns true):
      *     SELECT t0.EMP_ID..., t1.SALARY FROM EMPLOYEE t0, SALARY t1 WITH (UPDLOCK) WHERE ...
      */
+    @Override
     public boolean supportsIndividualTableLocking() {
         return false;
     }
@@ -480,6 +515,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * UPDATE OL_PHONE SET PHONE_ORDER_VARCHAR = (PHONE_ORDER_VARCHAR + 1) WHERE ...
      * SELECT ... WHERE  ... t0.MANAGED_ORDER_VARCHAR BETWEEN 1 AND 4 ...
      */
+    @Override
     public boolean supportsAutoConversionToNumericForArithmeticOperations() {
         return true;
     }
@@ -488,14 +524,24 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * MySQL temp table syntax, used for update-all, delete-all queries.
      */
+    @Override
     protected String getCreateTempTableSqlPrefix() {
         return "CREATE TEMPORARY TABLE IF NOT EXISTS ";
     }
+    
+    /**
+     * Return the drop schema definition. Subclasses should override as needed.
+     */
+    @Override
+    public String getDropDatabaseSchemaString(String schema) {
+        return "DROP SCHEMA " + schema + " IF EXISTS";
+    }
 
     /**
      * INTERNAL:
      * MySQL supports temp tables for update-all, delete-all queries.
      */
+    @Override
     public boolean shouldAlwaysUseTempStorageForModifyAll() {
         return true;
     }
@@ -504,8 +550,8 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * MySQL stored procedure calls do not require the argument name be printed in the call string
      * e.g. call MyStoredProc(?) instead of call MyStoredProc(myvariable = ?)
-     * @return
      */
+    @Override
     public boolean shouldPrintStoredProcedureArgumentNameInCall(){
 	    return false;
     }
@@ -517,6 +563,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * @see getStartDelimiter()
      * @see getEndDelimiter()
      */
+    @Override
     public String getIdentifierQuoteCharacter() {
         return "`";
     }
@@ -525,6 +572,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * MySQL uses the INOUT keyword for this.
      */
+    @Override
     public String getInOutputProcedureToken() {
         return "INOUT";
     }
@@ -532,6 +580,7 @@ public class MySQLPlatform extends DatabasePlatform {
     /**
      * MySQL does not use the AS token.
      */
+    @Override
     public String getProcedureAsString() {
         return "";
     }
@@ -540,6 +589,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * MySQL requires the direction at the start of the argument.
      */
+    @Override
     public boolean shouldPrintOutputTokenAtStart() {
         return true;
     }
@@ -548,6 +598,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * Used for stored procedure calls.
      */
+    @Override
     public String getProcedureCallHeader() {
         return "CALL ";
     }
@@ -556,6 +607,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * MySQL requires BEGIN.
      */
+    @Override
     public String getProcedureBeginString() {
         return "BEGIN ";
     }
@@ -564,6 +616,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * MySQL requires END.
      */
+    @Override
     public String getProcedureEndString() {
         return "END";
     }
@@ -572,6 +625,7 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * Writes MySQL specific SQL for accessing temp tables for update-all queries.
      */
+    @Override
     public void writeUpdateOriginalFromTempTableSql(Writer writer, DatabaseTable table,
                                                     Collection pkFields,
                                                     Collection assignedFields) throws IOException 
@@ -590,9 +644,10 @@ public class MySQLPlatform extends DatabasePlatform {
      * INTERNAL:
      * Writes MySQL specific SQL for accessing temp tables for delete-all queries.
      */
+    @Override
     public void writeDeleteFromTargetTableUsingTempTableSql(Writer writer, DatabaseTable table, DatabaseTable targetTable,
-                                                        Collection pkFields, 
-                                                        Collection targetPkFields) throws IOException 
+            Collection pkFields, 
+            Collection targetPkFields, DatasourcePlatform platform) throws IOException 
     {
         writer.write("DELETE FROM ");
         String targetTableName = targetTable.getQualifiedNameDelimited(this);
@@ -627,6 +682,14 @@ public class MySQLPlatform extends DatabasePlatform {
         call.setIgnoreFirstRowSetting(true);
         call.setIgnoreMaxResultsSetting(true);
     }
+
+    /**
+     * Used for stored procedure creation: MySQL platforms need brackets around arguments declaration even if no arguments exist.
+     */
+    @Override
+    public boolean requiresProcedureBrackets() {
+        return true;
+    }
     
     /**
      * INTERNAL:
