@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -10,6 +10,12 @@
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
  *     Markus Karg - bug fix for log operator
+ *     09/09/2011-2.3.1 Guy Pelletier 
+ *       - 356197: Add new VPD type to MultitenantType
+ *     09/14/2011-2.3.1 Guy Pelletier 
+ *       - 357533: Allow DDL queries to execute even when Multitenant entities are part of the PU
+ *     02/04/2013-2.5 Guy Pelletier 
+ *       - 389090: JPA 2.1 DDL Generation Support
  ******************************************************************************/  
 package org.eclipse.persistence.platform.database;
 
@@ -33,10 +39,13 @@ import org.eclipse.persistence.internal.expressions.ExpressionSQLPrinter;
 import org.eclipse.persistence.internal.expressions.FunctionExpression;
 import org.eclipse.persistence.internal.expressions.RelationExpression;
 import org.eclipse.persistence.internal.expressions.SQLSelectStatement;
+import org.eclipse.persistence.internal.helper.ClassConstants;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.helper.Helper;
 import org.eclipse.persistence.internal.helper.NonSynchronizedVector;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
+import org.eclipse.persistence.queries.DataModifyQuery;
+import org.eclipse.persistence.queries.DatabaseQuery;
 import org.eclipse.persistence.queries.SQLCall;
 import org.eclipse.persistence.queries.ValueReadQuery;
 
@@ -51,7 +60,9 @@ import org.eclipse.persistence.queries.ValueReadQuery;
  * @since TOPLink/Java 1.0
  */
 public class OraclePlatform extends org.eclipse.persistence.platform.database.DatabasePlatform {
-   
+    protected static DataModifyQuery vpdSetIdentifierQuery;
+    protected static DataModifyQuery vpdClearIdentifierQuery;
+    
     /** 
      * Advanced attribute indicating whether identity is supported,
      * see comment to setSupportsIdentity method.
@@ -66,6 +77,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     /*
      * Used for stored procedure definitions.
      */
+    @Override
     public boolean allowsSizeInProcedureArguments() {
         return false;
     }
@@ -75,6 +87,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * If using native SQL then print a byte[] literally as a hex string otherwise use ODBC format
      * as provided in DatabasePlatform.
      */
+    @Override
     protected void appendByteArray(byte[] bytes, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write('\'');
@@ -90,6 +103,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * Appends an Oracle specific date if usesNativeSQL is true otherwise use the ODBC format.
      * Native FORMAT: to_date('1997-11-06','yyyy-mm-dd')
      */
+    @Override
     protected void appendDate(java.sql.Date date, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("to_date('");
@@ -105,6 +119,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * Appends an Oracle specific time if usesNativeSQL is true otherwise use the ODBC format.
      * Native FORMAT: to_date(#####, 'sssss').
      */
+    @Override
     protected void appendTime(java.sql.Time time, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("to_date('");
@@ -120,6 +135,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * Appends an Oracle specific Timestamp, if usesNativeSQL is true otherwise use the ODBC format.
      * Native Format: to_date ('1997-11-06 10:35:45.0' , 'yyyy-mm-dd hh:mm:ss.n')
      */
+    @Override
     protected void appendTimestamp(java.sql.Timestamp timestamp, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("to_date('");
@@ -135,6 +151,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * Appends an Oracle specific Timestamp, if usesNativeSQL is true otherwise use the ODBC format.
      * Native Format: to_date ('1997-11-06 10:35:45.0' , 'yyyy-mm-dd hh:mm:ss.n')
      */
+    @Override
     protected void appendCalendar(Calendar calendar, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("to_date('");
@@ -156,6 +173,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * INTERNAL:
      */
+    @Override
     protected Hashtable buildFieldTypes() {
         Hashtable fieldTypeMapping;
 
@@ -192,10 +210,22 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return fieldTypeMapping;
     }
 
+    /**
+     * Build the hint string used for first rows.
+     * 
+     * Allows it to be overridden
+     * @param max
+     * @return
+     */
+    protected String buildFirstRowsHint(int max){
+        return HINT_START + HINT_END;
+    }
+    
     /**
      * INTERNAL:
      * Returns null unless the platform supports call with returning
      */
+    @Override
     public DatabaseCall buildCallWithReturning(SQLCall sqlCall, Vector returnFields) {
         SQLCall call = new SQLCall();
         call.setParameters(sqlCall.getParameters());
@@ -242,6 +272,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * In case this method returns true, buildCallWithReturning method
      * may be called.
      */
+    @Override
     public boolean canBuildCallWithReturning() {
         return true;
     }
@@ -257,6 +288,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Used for stored function calls.
      */
+    @Override
     public String getAssignmentString() {
         return ":= ";
     }
@@ -272,6 +304,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Used for batch writing and sp defs.
      */
+    @Override
     public String getBatchBeginString() {
         return "BEGIN ";
     }
@@ -279,15 +312,58 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Used for batch writing and sp defs.
      */
+    @Override
     public String getBatchEndString() {
         return "END;";
     }
 
+    /**
+     * Used for batch writing for row count return.
+     */
+    @Override
+    public String getBatchRowCountDeclareString() {
+        return "DECLARE EL_COUNTER NUMBER := 0; ";
+    }
+    
+    /**
+     * Oracle does not return the row count from PLSQL anon blocks,
+     * so an output parameter is required for this.
+     */
+    @Override
+    public boolean isRowCountOutputParameterRequired() {
+        return true;
+    }
+
+    /**
+     * Used for batch writing for row count return.
+     */
+    @Override
+    public String getBatchRowCountReturnString() {
+        return "? := EL_COUNTER; ";
+    }
+    
+    /**
+     * Return the drop schema definition. Subclasses should override as needed.
+     */
+    @Override
+    public String getDropDatabaseSchemaString(String schema) {
+        return "DROP SCHEMA " + schema + " RESTRICT";
+    }
+    
+    /**
+     * Used for batch writing for row count return.
+     */
+    @Override
+    public String getBatchRowCountAssignString() {
+        return "EL_COUNTER := EL_COUNTER + SQL%ROWCOUNT; ";
+    }
+    
     /**
      * INTERNAL:
      * returns the maximum number of characters that can be used in a field
      * name on this platform.
      */
+    @Override
     public int getMaxFieldNameSize() {
         return 30;
     }
@@ -319,6 +395,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Used for sp calls.
      */
+    @Override
     public String getProcedureArgumentSetter() {
         return "=>";
     }
@@ -326,17 +403,28 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Used for sp calls.
      */
+    @Override
     public String getProcedureCallHeader() {
-        return "BEGIN ";
+        return useJDBCStoredProcedureSyntax() ? "{CALL " : "BEGIN ";  
     }
 
     /**
      * Used for sp calls.
      */
+    @Override
     public String getProcedureCallTail() {
-        return "; END;";
+        return useJDBCStoredProcedureSyntax() ? "}" : "; END;";
+    }
+    
+    /**
+     * Allows DROP TABLE to cascade dropping of any dependent constraints if the database supports this option.
+     */
+    @Override
+    public String getDropCascadeString() {
+        return " CASCADE CONSTRAINTS";
     }
 
+    @Override
     public String getSelectForUpdateString() {
         return " FOR UPDATE";
     }
@@ -346,10 +434,12 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return " FOR UPDATE WAIT " + waitTimeout.intValue();
     }
 
+    @Override
     public String getStoredProcedureParameterPrefix() {
         return "P_";
     }
 
+    @Override
     public String getStoredProcedureTerminationToken() {
         return "";
     }
@@ -372,20 +462,82 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * This method returns the query to select the timestamp
      * from the server for Oracle.
      */
+    @Override
     public ValueReadQuery getTimestampQuery() {
         if (timestampQuery == null) {
             timestampQuery = new ValueReadQuery();
             timestampQuery.setSQLString("SELECT SYSDATE FROM DUAL");
+            timestampQuery.setAllowNativeSQLQuery(true);
         }
         return timestampQuery;
     }
     
+    /**
+     * INTERNAL:
+     * Return an Oracle defined VPD clear identifier query.
+     */
+    @Override
+    public DatabaseQuery getVPDClearIdentifierQuery(String vpdIdentifier) {
+        if (vpdClearIdentifierQuery == null) {
+            vpdClearIdentifierQuery = new DataModifyQuery("CALL DBMS_SESSION.CLEAR_IDENTIFIER()");
+        }
+    
+        return vpdClearIdentifierQuery;
+    }
+    
+    /**
+     * INTERNAL:
+     * Return an Oracle defined VPD identifier function. Used for DDL generation.
+     */
+    @Override
+    public String getVPDCreationFunctionString(String tableName, String tenantFieldName) {
+        String functionName = tableName + "_ident_func";
+        return "CREATE OR REPLACE FUNCTION " + functionName + " (p_schema in VARCHAR2 default NULL, p_object in VARCHAR2 default NULL) RETURN VARCHAR2 AS BEGIN return '" + tenantFieldName + " = sys_context(''userenv'', ''client_identifier'')'; END;";
+    }
+    
+    /**
+     * INTERNAL:
+     * Return an Oracle defined VPD identifier policy. Used for DDL generation.
+     */
+    @Override
+    public String getVPDCreationPolicyString(String tableName, AbstractSession session) {
+        String functionName = tableName + "_ident_func";
+        String schemaName = session.getDatasourceLogin().getUserName();
+        String policyName = tableName + "_todo_list_policy";
+        return "\nCALL DBMS_RLS.ADD_POLICY ('" + schemaName + "', '" + tableName + "', '" + policyName + "', '" + schemaName + "', '" + functionName +"', 'select, update, delete')\n";
+    }
+    
+    /**
+     * INTERNAL:
+     * Return an Oracle defined VPD identifier policy deletion. Used for DDL generation.
+     */
+    @Override
+    public String getVPDDeletionString(String tableName, AbstractSession session) {
+        String schemaName = session.getDatasourceLogin().getUserName();
+        String policyName = tableName + "_todo_list_policy";
+        return "\nCALL DBMS_RLS.DROP_POLICY ('" + schemaName + "', '" + tableName + "', '" + policyName + "')";
+    }
+    
+    /**
+     * INTERNAL:
+     * Return an Oracle defined VPD set identifier query.
+     */
+    @Override
+    public DatabaseQuery getVPDSetIdentifierQuery(String vpdIdentifier) {
+        if (vpdSetIdentifierQuery == null) {
+            vpdSetIdentifierQuery = new DataModifyQuery("CALL DBMS_SESSION.SET_IDENTIFIER(#" + vpdIdentifier + ")");
+        }
+        
+        return vpdSetIdentifierQuery;
+    }
+    
     /**
      * INTERNAL:
      * Get a timestamp value from a result set.
      * Overrides the default behavior to specifically return a timestamp.  Added
      * to overcome an issue with the oracle 9.0.1.4 JDBC driver.
      */
+    @Override
     public Object getObjectFromResultSet(ResultSet resultSet, int columnNumber, int type, AbstractSession session) throws java.sql.SQLException {
         //Bug#3381652 10G Drivers return sql.Date instead of timestamp on DATE field
         if ((type == Types.TIMESTAMP) || (type == Types.DATE)) {
@@ -398,6 +550,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Initialize any platform-specific operators
      */
+    @Override
     protected void initializePlatformOperators() {
         super.initializePlatformOperators();
         addOperator(operatorOuterJoin());
@@ -413,6 +566,44 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         addOperator(ExpressionOperator.oracleDateName());
         addOperator(operatorLocate());
         addOperator(operatorLocate2());
+        addOperator(regexpOperator());
+        addOperator(exceptOperator());
+    }
+    
+    /**
+     * INTERNAL:
+     * Create the EXCEPT operator, MINUS in Oracle.
+     */
+    public static ExpressionOperator exceptOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Except);
+        exOperator.printsAs("MINUS ");
+        exOperator.bePostfix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Create the REGEXP_LIKE operator.
+     */
+    public static ExpressionOperator regexpOperator() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.Regexp);
+        result.setType(ExpressionOperator.FunctionOperator);
+        Vector v = NonSynchronizedVector.newInstance(3);
+        v.add("REGEXP_LIKE(");
+        v.add(", ");
+        v.add(")");
+        result.printsAs(v);
+        result.bePrefix();
+        result.setNodeClass(ClassConstants.FunctionExpression_Class);
+        v = NonSynchronizedVector.newInstance(2);
+        v.add(".regexp(");
+        v.add(")");
+        result.printsJavaAs(v);
+        return result;
     }
 
     /**
@@ -424,7 +615,8 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     public boolean isNativeConnectionRequiredForLobLocator() {
         return false;
     }
-    
+
+    @Override
     public boolean isOracle() {
         return true;
     }
@@ -450,6 +642,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * might also be useful to end users attempting to sanitize values.
      * <p><b>NOTE</b>: BigInteger & BigDecimal maximums are dependent upon their precision & Scale
      */
+    @Override
     public Hashtable maximumNumericValues() {
         Hashtable values = new Hashtable();
 
@@ -469,6 +662,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * might also be useful to end users attempting to sanitize values.
      * <p><b>NOTE</b>: BigInteger & BigDecimal minimums are dependent upon their precision & Scale
      */
+    @Override
     public Hashtable minimumNumericValues() {
         Hashtable values = new Hashtable();
 
@@ -489,6 +683,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * and returns it. Currently implemented on Oracle only.
      * @param qualifiedSeqName known by Oracle to be a defined sequence
      */
+    @Override
     public ValueReadQuery buildSelectQueryForSequenceObject(String qualifiedSeqName, Integer size) {
         return new ValueReadQuery("SELECT " + qualifiedSeqName + ".NEXTVAL FROM DUAL");
     }
@@ -499,6 +694,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * see comment to setSupportsIdentity method.
      * @param qualifiedSeqName known by Oracle to be a defined sequence
      */
+    @Override
     public ValueReadQuery buildSelectQueryForIdentity(String qualifiedSeqName, Integer size) {
         return new ValueReadQuery("SELECT " + qualifiedSeqName + ".CURRVAL FROM DUAL");
     }
@@ -557,6 +753,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Append the receiver's field 'NULL' constraint clause to a writer.
      */
+    @Override
     public void printFieldNullClause(Writer writer) throws ValidationException {
         try {
             writer.write(" NULL");
@@ -572,10 +769,20 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return "SYSDATE";
     }
 
-
+    /**
+     * INTERNAL:
+     * Should the variable name of a stored procedure call be printed as part of the procedure call
+     * e.g. EXECUTE PROCEDURE MyStoredProc(myvariable = ?)
+     */
+    @Override
+    public boolean shouldPrintStoredProcedureArgumentNameInCall() {
+        return ! useJDBCStoredProcedureSyntax();
+    }
+    
     /**
      * JDBC defines and outer join syntax, many drivers do not support this. So we normally avoid it.
      */
+    @Override
     public boolean shouldUseJDBCOuterJoinSyntax() {
         return false;
     }
@@ -585,6 +792,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * UPDATE OL_PHONE SET PHONE_ORDER_VARCHAR = (PHONE_ORDER_VARCHAR + 1) WHERE ...
      * SELECT ... WHERE  ... t0.MANAGED_ORDER_VARCHAR BETWEEN 1 AND 4 ...
      */
+    @Override
     public boolean supportsAutoConversionToNumericForArithmeticOperations() {
         return true;
     }
@@ -594,6 +802,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * Indicates whether the platform supports sequence objects.
      * This method is to be used *ONLY* by sequencing classes
      */
+    @Override
     public boolean supportsSequenceObjects() {
         return true;
     }
@@ -603,6 +812,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * Indicates whether the platform supports identity.
      * This method is to be used *ONLY* by sequencing classes
      */
+    @Override
     public boolean supportsIdentity() {
         return supportsIdentity;
     }
@@ -630,9 +840,18 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Return if database stored functions are supported.
      */
+    @Override
     public boolean supportsStoredFunctions() {
         return true;
     }
+    
+    /**
+     * Oracle db supports VPD.
+     */
+    @Override
+    public boolean supportsVPD() {
+        return true;
+    }
 
     /**
      * Returns true if the database supports SQL syntax not to wait on a SELECT..FOR UPADTE
@@ -673,13 +892,25 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return connection;
     }
     
+    /**
+     * Return true if JDBC syntax should be used for stored procedure calls.
+     */
+    public boolean useJDBCStoredProcedureSyntax() {
+        if (useJDBCStoredProcedureSyntax == null) {
+            useJDBCStoredProcedureSyntax = this.driverName != null && this.driverName.equals("Oracle");
+        }
+        
+        return useJDBCStoredProcedureSyntax;
+    }
+    
     //Oracle Rownum support
-    private static final String SELECT = "SELECT * FROM (SELECT ";
-    private static final String HINT = "/*+ FIRST_ROWS */ ";
-    private static final String FROM = "a.*, ROWNUM rnum  FROM (";
-    private static final String END_FROM = ") a ";
-    private static final String MAX_ROW = "WHERE ROWNUM <= ";
-    private static final String MIN_ROW = ") WHERE rnum > ";
+    protected String SELECT = "SELECT * FROM (SELECT ";
+    protected String HINT_START = "/*+ FIRST_ROWS";
+    protected String HINT_END = " */ ";
+    protected String FROM = "a.*, ROWNUM rnum  FROM (";
+    protected String END_FROM = ") a ";
+    protected String MAX_ROW = "WHERE ROWNUM <= ";
+    protected String MIN_ROW = ") WHERE rnum > ";
     
     /**
      * INTERNAL:
@@ -687,6 +918,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * in the DatabaseCall.  This implementation works MaxRows and FirstResult into the SQL using
      * Oracle's ROWNUM to filter values if shouldUseRownumFiltering is true.  
      */
+    @Override
     public void printSQLSelectStatement(DatabaseCall call, ExpressionSQLPrinter printer, SQLSelectStatement statement){
         int max = 0;
         int firstRow = 0;
@@ -702,7 +934,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         } else if ( max > 0 ){
             statement.setUseUniqueFieldAliases(true);
             printer.printString(SELECT);
-            printer.printString(HINT);
+            printer.printString(buildFirstRowsHint(max));
             printer.printString(FROM);
             
             call.setFields(statement.printSQL(printer));
@@ -730,6 +962,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * Override this method if the platform supports sequence objects
      * and it's possible to alter sequence object's increment in the database.
      */
+    @Override
     public boolean isAlterSequenceObjectSupported() {
         return true;
     }
@@ -738,10 +971,20 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Indicates whether SELECT DISTINCT ... FOR UPDATE is allowed by the platform (Oracle doesn't allow this).
      */
+    @Override
     public boolean isForUpdateCompatibleWithDistinct() {
         return false;
     }
     
+    /**
+     * INTERNAL:
+     * Indicates whether SELECT DISTINCT lob FROM ... (where lob is BLOB or CLOB) is allowed by the platform (Oracle doesn't allow this).
+     */
+    @Override
+    public boolean isLobCompatibleWithDistinct() {
+        return false;
+    }
+    
     /**
      * Return true if the given exception occurred as a result of a lock
      * time out exception (WAIT clause).
@@ -759,14 +1002,14 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * If this platform is unable to determine if the error was communication based it will return
      * false forcing the error to be thrown to the user.
      */
-    
+    @Override
     public boolean wasFailureCommunicationBased(SQLException exception, Connection connection, AbstractSession sessionForProfile){
         if (exception != null){
             if (exception.getErrorCode() == 17410){
                 return true;
             }
             if (exception.getErrorCode() == 17002){
-        	      return true;
+                return true;
             }
             if (exception.getErrorCode() == 2399){
                 return true;
