@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,25 +9,21 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     09/14/2011-2.3.1 Guy Pelletier 
+ *       - 357533: Allow DDL queries to execute even when Multitenant entities are part of the PU
  ******************************************************************************/  
 package org.eclipse.persistence.platform.database;
 
 import java.io.*;
-import java.sql.CallableStatement;
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
-import java.sql.SQLException;
 import java.util.*;
 
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.expressions.*;
 import org.eclipse.persistence.internal.expressions.*;
 import org.eclipse.persistence.internal.helper.*;
-import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.databaseaccess.*;
 import org.eclipse.persistence.queries.*;
-import org.eclipse.persistence.sessions.SessionProfiler;
 
 /**
  * <p><b>Purpose</b>: Provides SQL Server specific behavior.
@@ -305,6 +301,7 @@ public class SQLServerPlatform extends org.eclipse.persistence.platform.database
         if (timestampQuery == null) {
             timestampQuery = new ValueReadQuery();
             timestampQuery.setSQLString("SELECT GETDATE()");
+            timestampQuery.setAllowNativeSQLQuery(true);
         }
         return timestampQuery;
     }
@@ -339,56 +336,6 @@ public class SQLServerPlatform extends org.eclipse.persistence.platform.database
         return " WITH (UPDLOCK, NOWAIT)";
     }
 
-    /**
-     * because each platform has different requirements for accessing stored procedures and
-     * the way that we can combine resultsets and output params the stored procedure call
-     * is being executed on the platform.  This entire process needs some serious refactoring to eliminate
-     * the chance of bugs.
-     */
-    public Object executeStoredProcedure(DatabaseCall dbCall, PreparedStatement statement, DatabaseAccessor accessor, AbstractSession session) throws SQLException {
-        Object result = null;
-        ResultSet resultSet = null;
-        if (!dbCall.getReturnsResultSet()) {
-            accessor.executeDirectNoSelect(statement, dbCall, session);
-            result = accessor.buildOutputRow((CallableStatement)statement, dbCall, session);
-
-            //ReadAllQuery may be returning just output params, or they may be executing a DataReadQuery, which also
-            //assumes a vector
-            if (dbCall.areManyRowsReturned()) {
-                Vector tempResult = new Vector();
-                (tempResult).add(result);
-                result = tempResult;
-            }
-        } else {
-            // start the process of processing the result set and the output params.  this is specific to Sybase JConnect 5.5
-            // as we must process the result set before the output params.
-            session.startOperationProfile(SessionProfiler.STATEMENT_EXECUTE, dbCall.getQuery(), SessionProfiler.ALL);
-            try {
-                resultSet = statement.executeQuery();
-            } finally {
-                session.endOperationProfile(SessionProfiler.STATEMENT_EXECUTE, dbCall.getQuery(), SessionProfiler.ALL);
-            }
-            dbCall.matchFieldOrder(resultSet, accessor, session);
-
-            // cursored result set and output params not supported because of database limitation
-            if (dbCall.isCursorReturned()) {
-                dbCall.setStatement(statement);
-                dbCall.setResult(resultSet);
-                return dbCall;
-            }
-            result = processResultSet(resultSet, dbCall, statement, accessor, session);
-
-            if (dbCall.shouldBuildOutputRow()) {
-                AbstractRecord outputRow = accessor.buildOutputRow((CallableStatement)statement, dbCall, session);
-                dbCall.getQuery().setProperty("output", outputRow);
-                session.getEventManager().outputParametersDetected(outputRow, dbCall);
-            }
-            return result;
-            // end special sybase behavior.
-        }
-        return result;
-    }
-
     /**
      * INTERNAL:
      * Indicates whether locking clause should be printed after where clause by SQLSelectStatement.
@@ -445,8 +392,83 @@ public class SQLServerPlatform extends org.eclipse.persistence.platform.database
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.Length, "LEN"));
         addOperator(modOperator());
         addOperator(ExpressionOperator.simpleAggregate(ExpressionOperator.StandardDeviation, "STDEV", "standardDeviation"));
+        addOperator(trimOperator());
+        addOperator(trim2Operator());
+        addOperator(extractOperator());
     }
 
+    /**
+     * INTERNAL:
+     * Derby does not support EXTRACT, but does have DATEPART.
+     */
+    public static ExpressionOperator extractOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Extract);
+        exOperator.setName("EXTRACT");
+        Vector v = NonSynchronizedVector.newInstance(5);
+        v.add("DATEPART(");
+        v.add(",");
+        v.add(")");
+        exOperator.printsAs(v);
+        int[] indices = new int[2];
+        indices[0] = 1;
+        indices[1] = 0;
+        exOperator.setArgumentIndices(indices);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+    
+    /**
+     * INTERNAL:
+     * Use RTRIM(LTRIM(?)) function for trim.
+     */
+    public static ExpressionOperator trimOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Trim);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(2);
+        v.add("RTRIM(LTRIM(");
+        v.add("))");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+    
+    /**
+     * INTERNAL:
+     * Build Trim operator.
+     */
+    public static ExpressionOperator trim2Operator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Trim2);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(5);
+        v.add("RTRIM(");
+        v.add(" FROM LTRIM(");
+        v.add(" FROM ");
+        v.add("))");
+        exOperator.printsAs(v);
+        int[] argumentIndices = new int[3];
+        argumentIndices[0] = 1;
+        argumentIndices[1] = 1;
+        argumentIndices[2] = 0;
+        exOperator.setArgumentIndices(argumentIndices);
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Return true if output parameters can be built with result sets.
+     */
+    @Override
+    public boolean isOutputAllowWithResultSet() {
+        return false;
+    }
+    
     public boolean isSQLServer() {
         return true;
     }
