@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,8 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     09/14/2011-2.3.1 Guy Pelletier 
+ *       - 357533: Allow DDL queries to execute even when Multitenant entities are part of the PU
  ******************************************************************************/  
 package org.eclipse.persistence.platform.database;
 
@@ -18,18 +20,14 @@ import java.util.*;
 
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.expressions.*;
-import org.eclipse.persistence.internal.databaseaccess.DatabaseAccessor;
-import org.eclipse.persistence.internal.databaseaccess.DatabaseCall;
 import org.eclipse.persistence.internal.databaseaccess.FieldTypeDefinition;
 import org.eclipse.persistence.internal.expressions.*;
 import org.eclipse.persistence.internal.helper.*;
-import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.queries.*;
-import org.eclipse.persistence.sessions.SessionProfiler;
 
 /**
- *    <p><b>Purpose</b>: Provides Sybase specific behavior.
+ *    <p><b>Purpose</b>: Provides Sybase ASE specific behavior.
  *    <p><b>Responsibilities</b>:<ul>
  *    <li> Native SQL for byte[], Date, Time, & Timestamp.
  *    <li> Native sequencing using @@IDENTITY.
@@ -91,8 +89,23 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     /**
-     *    If using native SQL then print a byte[] as '0xFF...'
+     * Sybase and SQL Anywhere do not support BLOB/CLOB but require LONGVARBINARY/LONGVARCHAR.
      */
+    @Override
+    public int getJDBCType(Class javaType) {
+        if (javaType == ClassConstants.BLOB)  {
+            return Types.LONGVARBINARY;
+        } else if (javaType == ClassConstants.CLOB) {
+            return Types.LONGVARCHAR;
+        } else {
+            return super.getJDBCType(javaType);
+        }
+    }
+    
+    /**
+     * If using native SQL then print a byte[] as '0xFF...'
+     */
+    @Override
     protected void appendByteArray(byte[] bytes, Writer writer) throws IOException {
         if (usesNativeSQL() && (!usesByteArrayBinding())) {
             writer.write("0x");
@@ -103,9 +116,10 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     /**
-     *    Answer a platform correct string representation of a Date, suitable for SQL generation.
-     *    Native format: 'yyyy-mm-dd
+     * Answer a platform correct string representation of a Date, suitable for SQL generation.
+     * Native format: 'yyyy-mm-dd
      */
+    @Override
     protected void appendDate(java.sql.Date date, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("'");
@@ -140,9 +154,10 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     /**
-     *    Answer a platform correct string representation of a Time, suitable for SQL generation.
-     *    The time is printed in the ODBC platform independent format {t'hh:mm:ss'}.
+     * Answer a platform correct string representation of a Time, suitable for SQL generation.
+     * The time is printed in the ODBC platform independent format {t'hh:mm:ss'}.
      */
+    @Override
     protected void appendTime(java.sql.Time time, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("'");
@@ -154,9 +169,10 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     /**
-     *    Answer a platform correct string representation of a Timestamp, suitable for SQL generation.
-     *    The date is printed in the ODBC platform independent format {d'YYYY-MM-DD'}.
+     * Answer a platform correct string representation of a Timestamp, suitable for SQL generation.
+     * The date is printed in the ODBC platform independent format {d'YYYY-MM-DD'}.
      */
+    @Override
     protected void appendTimestamp(java.sql.Timestamp timestamp, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             appendSybaseTimestamp(timestamp, writer);
@@ -169,6 +185,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * Answer a platform correct string representation of a Calendar, suitable for SQL generation.
      * The date is printed in the ODBC platform independent format {d'YYYY-MM-DD'}.
      */
+    @Override
     protected void appendCalendar(Calendar calendar, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             appendSybaseCalendar(calendar, writer);
@@ -194,6 +211,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         return ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Atan2, "ATN2");
     }
 
+    @Override
     protected Hashtable buildFieldTypes() {
         Hashtable fieldTypeMapping;
 
@@ -230,6 +248,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Build the identity query for native sequencing.
      */
+    @Override
     public ValueReadQuery buildSelectQueryForIdentity() {
         ValueReadQuery selectQuery = new ValueReadQuery();
         StringWriter writer = new StringWriter();
@@ -238,65 +257,17 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         return selectQuery;
     }
 
-    /**
-     * because each platform has different requirements for accessing stored procedures and
-     * the way that we can combine resultsets and output params the stored procedure call
-     * is being executed on the platform.  This entire process needs some serious refactoring to eliminate
-     * the chance of bugs.
-     */
-    public Object executeStoredProcedure(DatabaseCall dbCall, PreparedStatement statement, DatabaseAccessor accessor, AbstractSession session) throws SQLException {
-        Object result = null;
-        ResultSet resultSet = null;
-        if (!dbCall.getReturnsResultSet()) {
-            accessor.executeDirectNoSelect(statement, dbCall, session);
-            result = accessor.buildOutputRow((CallableStatement)statement, dbCall, session);
-
-            //ReadAllQuery may be returning just output params, or they may be executing a DataReadQuery, which also
-            //assumes a vector
-            if (dbCall.areManyRowsReturned()) {
-                Vector tempResult = new Vector();
-                (tempResult).add(result);
-                result = tempResult;
-            }
-        } else {
-            // start the process of processing the result set and the output params.  this is specific to Sybase JConnect 5.5
-            // as we must process the result set before the output params.
-            session.startOperationProfile(SessionProfiler.STATEMENT_EXECUTE, dbCall.getQuery(), SessionProfiler.ALL);
-            try {
-                resultSet = statement.executeQuery();
-            } finally {
-                session.endOperationProfile(SessionProfiler.STATEMENT_EXECUTE, dbCall.getQuery(), SessionProfiler.ALL);
-            }
-            dbCall.matchFieldOrder(resultSet, accessor, session);
-
-            // cursored result set and output params not supported because of database limitation
-            if (dbCall.isCursorReturned()) {
-                dbCall.setStatement(statement);
-                dbCall.setResult(resultSet);
-                return dbCall;
-            }
-            result = processResultSet(resultSet, dbCall, statement, accessor, session);
-
-            if (dbCall.shouldBuildOutputRow()) {
-                AbstractRecord outputRow = accessor.buildOutputRow((CallableStatement)statement, dbCall, session);
-                dbCall.getQuery().setProperty("output", outputRow);
-                session.getEventManager().outputParametersDetected(outputRow, dbCall);
-            }
-            return result;
-            // end special sybase behavior.
-        }
-        return result;
-    }
-
     /**
      * The sybase syntax for obtaining pessimistic lock is "SELECT ADDRESS_ID, ... FROM ADDRESS WITH (HOLDLOCK) WHERE (ADDRESS_ID = ?)"
      * Please note that above only obtains shared lock. Apparently there is no way to obtain exclusive lock on Sybase
      * using only a select statement
      */
+    @Override
     public boolean shouldPrintLockingClauseAfterWhereClause() {
         return false;
     }
 
+    @Override
     public String getSelectForUpdateString() {
         return " HOLDLOCK";
     }
@@ -304,6 +275,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Used for batch writing and sp defs.
      */
+    @Override
     public String getBatchDelimiterString() {
         return "";
     }
@@ -311,6 +283,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /* This method is used to print the required output parameter token for the
      * specific platform.  Used when stored procedures are created.
      */
+    @Override
     public String getCreationInOutputProcedureToken() {
         return getInOutputProcedureToken();
     }
@@ -319,6 +292,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * this method was added because SQLServer requires the output paramater token
      * to be set on creation but not on execution.
      */
+    @Override
     public String getCreationOutputProcedureToken() {
         return getOutputProcedureToken();
     }
@@ -326,6 +300,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /* This method is used to print the output parameter token when stored
      * procedures are called
      */
+    @Override
     public String getInOutputProcedureToken() {
         return "OUT";
     }
@@ -334,6 +309,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Returns the type name corresponding to the jdbc type
      */
+    @Override
     public String getJdbcTypeName(int jdbcType) {
         return (String)getTypeStrings().get(new Integer(jdbcType));
     }
@@ -343,6 +319,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * returns the maximum number of characters that can be used in a field
      * name on this platform.
      */
+    @Override
     public int getMaxFieldNameSize() {
         return 22;
     }
@@ -375,13 +352,15 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /* This method is used to print the output parameter token when stored
      * procedures are called
      */
+    @Override
     public String getOutputProcedureToken() {
-        return "OUTPUT";
+        return useJDBCStoredProcedureSyntax() ? "" : "OUTPUT";
     }
 
     /**
      * Used for sp defs.
      */
+    @Override
     public String getProcedureArgumentString() {
         return "@";
     }
@@ -389,19 +368,36 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Used for sp calls.
      */
+    @Override
     public String getProcedureCallHeader() {
-        return "EXECUTE ";
+        return useJDBCStoredProcedureSyntax() ? "{Call " : "EXECUTE ";
+    }
+    public String getProcedureCallTail() {
+        return useJDBCStoredProcedureSyntax() ? "}" : "";
+    }
+    /**
+     * Return true if this platform is to use the JDBC supported syntax for executing stored procedures.
+     * If the driver is known to be the DataDirec driver, and the value is not set, then set to true and return.
+     */
+    public boolean useJDBCStoredProcedureSyntax() {
+    
+        if (useJDBCStoredProcedureSyntax == null) {
+            useJDBCStoredProcedureSyntax = this.driverName != null && this.driverName.equals("Sybase");
+        }
+        return useJDBCStoredProcedureSyntax;
     }
 
+    @Override
     public String getStoredProcedureParameterPrefix() {
         return "@";
     }
 
     /**
      * INTERNAL:
-     *    This method returns the delimiter between stored procedures in multiple stored procedure
+     * This method returns the delimiter between stored procedures in multiple stored procedure
      * calls.
      */
+    @Override
     public String getStoredProcedureTerminationToken() {
         return " go";
     }
@@ -411,10 +407,12 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * This method returns the query to select the timestamp
      * from the server for Sybase.
      */
+    @Override
     public ValueReadQuery getTimestampQuery() {
         if (timestampQuery == null) {
             timestampQuery = new ValueReadQuery();
             timestampQuery.setSQLString("SELECT GETDATE()");
+            timestampQuery.setAllowNativeSQLQuery(true);
         }
         return timestampQuery;
     }
@@ -422,6 +420,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * Initialize any platform-specific operators
      */
+    @Override
     protected void initializePlatformOperators() {
         super.initializePlatformOperators();
         addOperator(operatorOuterJoin());
@@ -434,6 +433,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.CurrentTime, "GETDATE"));
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.Length, "CHAR_LENGTH"));
         addOperator(ExpressionOperator.sybaseLocateOperator());
+        addOperator(ExpressionOperator.sybaseLocate2Operator());
         addOperator(ExpressionOperator.simpleThreeArgumentFunction(ExpressionOperator.Substring, "SUBSTRING"));
         addOperator(singleArgumentSubstringOperator());
         addOperator(ExpressionOperator.addDate());
@@ -459,17 +459,90 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         addOperator(ExpressionOperator.sybaseToCharOperator());
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.Ceil, "CEILING"));
         addOperator(modOperator());
+        addOperator(trimOperator());
+        addOperator(trim2Operator());
+        // Sybase does not define REPLACE only STR_REPLACE
+        addOperator(ExpressionOperator.simpleThreeArgumentFunction(ExpressionOperator.Replace, "STR_REPLACE"));
+        addOperator(extractOperator());
     }
 
+    /**
+     * INTERNAL:
+     * Derby does not support EXTRACT, but does have DATEPART.
+     */
+    public static ExpressionOperator extractOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Extract);
+        exOperator.setName("EXTRACT");
+        Vector v = NonSynchronizedVector.newInstance(5);
+        v.add("DATEPART(");
+        v.add(",");
+        v.add(")");
+        exOperator.printsAs(v);
+        int[] indices = new int[2];
+        indices[0] = 1;
+        indices[1] = 0;
+        exOperator.setArgumentIndices(indices);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+    
+    /**
+     * INTERNAL:
+     * Use RTRIM(LTRIM(?)) function for trim.
+     */
+    public static ExpressionOperator trimOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Trim);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(2);
+        v.add("RTRIM(LTRIM(");
+        v.add("))");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+    
+    /**
+     * INTERNAL:
+     * Build Trim operator.
+     */
+    public static ExpressionOperator trim2Operator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Trim2);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(5);
+        v.add("STR_REPLACE(");
+        v.add(", ");
+        v.add(", NULL)");
+        exOperator.printsAs(v);
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Return true if output parameters can be built with result sets.
+     */
+    @Override
+    public boolean isOutputAllowWithResultSet() {
+        return false;
+    }
+    
+    @Override
     public boolean isSybase() {
         return true;
     }
 
     /**
-     *    Builds a table of maximum numeric values keyed on java class. This is used for type testing but
+     * Builds a table of maximum numeric values keyed on java class. This is used for type testing but
      * might also be useful to end users attempting to sanitize values.
      * <p><b>NOTE</b>: BigInteger & BigDecimal maximums are dependent upon their precision & Scale
      */
+    @Override
     public Hashtable maximumNumericValues() {
         Hashtable values = new Hashtable();
 
@@ -485,10 +558,11 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     /**
-     *    Builds a table of minimum numeric values keyed on java class. This is used for type testing but
+     * Builds a table of minimum numeric values keyed on java class. This is used for type testing but
      * might also be useful to end users attempting to sanitize values.
      * <p><b>NOTE</b>: BigInteger & BigDecimal minimums are dependent upon their precision & Scale
      */
+    @Override
     public Hashtable minimumNumericValues() {
         Hashtable values = new Hashtable();
 
@@ -532,6 +606,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     /** Append the receiver's field 'identity' constraint clause to a writer.*/
+    @Override
     public void printFieldIdentityClause(Writer writer) throws ValidationException {
         try {
             writer.write(" IDENTITY");
@@ -541,6 +616,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     /** Append the receiver's field 'NULL' constraint clause to a writer.*/
+    @Override
     public void printFieldNullClause(Writer writer) throws ValidationException {
         try {
             writer.write(" NULL");
@@ -553,6 +629,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * This method is used to register output parameter on Callable Statements for Stored Procedures
      * as each database seems to have a different method.
      */
+    @Override
     public void registerOutputParameter(CallableStatement statement, int index, int jdbcType) throws SQLException {
         statement.registerOutParameter(index, jdbcType, (String)getTypeStrings().get(new Integer(jdbcType)));
     }
@@ -560,13 +637,15 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * USed for sp calls.
      */
+    @Override
     public boolean requiresProcedureCallBrackets() {
-        return false;
+        return useJDBCStoredProcedureSyntax();
     }
 
     /**
      * Used for sp calls.  Sybase must print output after output params.
      */
+    @Override
     public boolean requiresProcedureCallOuputToken() {
         return true;
     }
@@ -576,6 +655,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * Indicates whether the version of CallableStatement.registerOutputParameter method
      * that takes type name should be used.
      */
+    @Override
     public boolean requiresTypeNameToRegisterOutputParameter() {
         return true;
     }
@@ -583,6 +663,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /* This is required in the construction of the stored procedures with
      * output parameters
      */
+    @Override
     public boolean shouldPrintInOutputTokenBeforeType() {
         return false;
     }
@@ -591,6 +672,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /* This is required in the construction of the stored procedures with
      * output parameters
      */
+    @Override
     public boolean shouldPrintOutputTokenBeforeType() {
         return false;
     }
@@ -598,6 +680,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * JDBC defines and outer join syntax, many drivers do not support this. So we normally avoid it.
      */
+    @Override
     public boolean shouldUseJDBCOuterJoinSyntax() {
         return false;
     }
@@ -627,25 +710,37 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     }
     
     /**
-     *  INTERNAL:
-     *  Indicates whether the platform supports identity.
-     *  Sybase does through IDENTITY field types.
-     *  This method is to be used *ONLY* by sequencing classes
+     * INTERNAL:
+     * Indicates whether the platform supports identity.
+     * Sybase does through IDENTITY field types.
+     * This method is to be used *ONLY* by sequencing classes
      */
+    @Override
     public boolean supportsIdentity() {
         return true;
     }
 
+    /**
+     * Sybase (as of Sybase ASE 15, does not support delete on cascade).
+     * Sybase ASA (SQL Anywhere does, so must use different platform).
+     */
+    @Override
+    public boolean supportsDeleteOnCascade() {
+        return false;
+    }
+    
     /**
      * INTERNAL:
      */
-    public boolean supportsLocalTempTables() {
+    @Override
+    public boolean supportsGlobalTempTables() {
         return true;
     }
      
     /**
      * INTERNAL:
      */
+    @Override
     protected String getCreateTempTableSqlPrefix() {
         return "CREATE TABLE ";
     }          
@@ -653,6 +748,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * INTERNAL:
      */
+    @Override
     public DatabaseTable getTempTableForTable(DatabaseTable table) {
         return new DatabaseTable("#" + table.getName(), table.getTableQualifier(), table.shouldUseDelimiters(), getStartDelimiter(), getEndDelimiter());
     }          
@@ -660,6 +756,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
     /**
      * INTERNAL:
      */
+    @Override
     public void writeUpdateOriginalFromTempTableSql(Writer writer, DatabaseTable table,
                                                     Collection pkFields,
                                                     Collection assignedFields) throws IOException 
