@@ -10,6 +10,8 @@
  * Contributors:
  * Fujitsu Ltd. (Dies Koper) - based on TLE-based implementation in Fujitsu's
  *                             Interstage Application Server V9.2 (JPA 1.0)
+ *     09/14/2011-2.3.1 Guy Pelletier 
+ *       - 357533: Allow DDL queries to execute even when Multitenant entities are part of the PU
  *
  ******************************************************************************/
 package org.eclipse.persistence.platform.database;
@@ -85,9 +87,14 @@ import org.eclipse.persistence.queries.ValueReadQuery;
  * <li> The standard deviation (STDDEV) and variance (VARIANCE) functions cannot be used. (bug 304909)
  * <li> '= NULL' and '<> NULL' cannot be used for null comparisons in the WHERE clause. Use 'IS (NOT) NULL' instead.
  * <li> A scrollable cursor policy of CONCUR_UPDATABLE mode cannot be used with queries that select from multiple tables.
- * <li> Columns and literals of different type may need casting to allow them to be compared or assigned. For example: 
+ * <li> Columns and literals of different type may need casting to allow them to be compared or assigned. (bug 372178) For example: 
  *   'SELECT ... WHERE CAST(PHONE_ORDER_VARCHAR AS INTEGER) BETWEEN 0 AND 1'
  * <li> Subqueries with joins to the outer query are not supported. (see rfe 298193)
+ * <li> Stored functions are not supported. (bug 342409)
+ * <li> Using subquery in select list for query specification or single-row SELECT statement is not allowed. (bug 372172)
+ * <li> The CascadeOnDelete doesn't work on a relation where CascadeType.Remove or CascadeType.All is specified. (bug 342413)
+ * <li> A subquery cannot be specified on both sides of a comparison predicate or a quantified predicate. (bug 378313)
+ * <li> A base table name to be updated cannot be identical to table name in from clause in query or subquery specification (bug 381302) 
  * </ul>
  * <p/><b>Additional Notes</b><br/>
  * ----------------
@@ -853,7 +860,19 @@ public class SymfowarePlatform extends DatabasePlatform {
      */
     @Override
     public int computeMaxRowsForSQL(int firstResultIndex, int maxResults){
-        return maxResults - ((firstResultIndex >= 0) ? firstResultIndex : 0);
+         // In Symfoware, this API is used in the follow scenario.
+         //   1. construct SELECT ... WITH OPTION LIMIT(comupteMaxRowForSQL());
+         //   2. move the cursor of ResultSet to the row number index of firstResultIndex;
+         //   3. get (computeMaxRowForSQL() - firstResultIndex) number of rows from row number index of firstResultIndex.
+         // There are two different Queries which depend on this API.
+         //   Query#setFirstResult(), Query#setMaxResults() in JPA
+         //   ReadQuery#setFirstResult, ReadQuery#setMaxRows() in EclipseLink specific API.
+         // Note that each method of Query delegates one of ReadQuery respectively.
+         // ReadQuery#setMaxRow() is always passed by a converted maxRows value according to below condition.
+         //   int maxRows = maxResults + ((firstResultIndex >= 0) ? firstResultIndex : 0);
+         // Actually Query#setFirstResult(3) and Query#setMaxResults(6) is equals to ReadQuery#setFirstResult(3) and ReadQuery#setMaxRows(9).
+         // Therefore we don't need to compute a value of MaxRows here and just return maxResults.
+        return maxResults;
     }
 
     /**
@@ -880,19 +899,25 @@ public class SymfowarePlatform extends DatabasePlatform {
      *            qualified name of the table the index is to be created on
      * @param indexName
      *            name of the index
+     * @param qualifier
+     *            qualifier to construct qualified name of index if needed
+     * @param isUnique
+     *            Indicates whether uniqe index is created
      * @param columnNames
-     *            list of columns the index is created for
+     *            one or more columns the index is created for
      */
     @Override
-    public String buildCreateIndex(String fullTableName, String indexName,
-            String... columnNames) {
-        String columns = columnNames[0];
+    public String buildCreateIndex(String fullTableName, String indexName, String qualifier, 
+            boolean isUnique, String... columnNames) {
+        StringBuilder queryString = new StringBuilder();
+        queryString.append("CREATE INDEX ");
+        queryString.append(fullTableName).append(".").append(indexName).append(" KEY (");
+        queryString.append(columnNames[0]);
         for (int i = 1; i < columnNames.length; i++) {
-            columns += ", " + columnNames[i];
+            queryString.append(", ").append(columnNames[i]);
         }
-
-        return "CREATE INDEX " + fullTableName + "." + indexName + " KEY ("
-                + columns + ")";
+        queryString.append(")");
+        return queryString.toString();
     }
 
     /**
@@ -902,10 +927,14 @@ public class SymfowarePlatform extends DatabasePlatform {
      *            qualified name of the table the index is to be created on
      * @param indexName
      *            name of the index
+     * @param qualifier
+     *            qualifier to construct qualified name of index if needed
      */
     @Override
-    public String buildDropIndex(String fullTableName, String indexName) {
-        return "DROP INDEX " + fullTableName + "." + indexName;
+    public String buildDropIndex(String fullTableName, String indexName, String qualifier) {
+        StringBuilder queryString = new StringBuilder();
+        queryString.append("DROP INDEX ").append(fullTableName).append(".").append(indexName);
+        return queryString.toString();
     }
 
     /**
@@ -939,6 +968,18 @@ public class SymfowarePlatform extends DatabasePlatform {
         return "\"SEQUENCE\"";
     }
 
+    /**
+     * Used to allow platforms to define their own index prefixes
+     * @param isUniqueField
+     * @return
+     */
+    public String getIndexNamePrefix(boolean isUniqueSetOnField){
+        if (isUniqueSetOnField){
+            return "UIX_";
+        }
+        return super.getIndexNamePrefix(isUniqueSetOnField);
+    }
+    
     /**
      * This method is used to print the output parameter token when stored
      * procedures are called.
@@ -966,7 +1007,7 @@ public class SymfowarePlatform extends DatabasePlatform {
     public int getMaxFieldNameSize() {
         return 36;
     }
-
+    
     /**
      * Symfoware does not use the AS token.
      */
@@ -1026,10 +1067,19 @@ public class SymfowarePlatform extends DatabasePlatform {
         if (timestampQuery == null) {
             timestampQuery = new ValueReadQuery();
             timestampQuery.setSQLString("SELECT CURRENT_TIMESTAMP FROM RDBII_SYSTEM.RDBII_ASSISTTABLE");
+            timestampQuery.setAllowNativeSQLQuery(true);
         }
         return timestampQuery;
     }
 
+    /**
+     * INTERNAL:
+     * Returns the minimum time increment supported by the platform.
+     */
+    public long minimumTimeIncrement() {
+        return 1000;
+    }
+    
     /**
      * Print the pagination SQL using Symfoware syntax
      * " WITH OPTION LIMIT (<max>)". There is no equivalent to 'OFFSET'.<br/>
@@ -1263,11 +1313,13 @@ public class SymfowarePlatform extends DatabasePlatform {
 
     /**
      * Indicates whether the platform supports stored functions.<br/>
-     * Symfoware does.
+     * Although Symfoware supports some stored functions as function routines,
+     * their functionality is incompatible with the one EclipseLink provides.
+     * So, return false;
      */
     @Override
     public boolean supportsStoredFunctions() {
-        return true;
+        return false;
     }
 
     /**
