@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -9,6 +9,8 @@
  *
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
+ *     09/14/2011-2.3.1 Guy Pelletier 
+ *       - 357533: Allow DDL queries to execute even when Multitenant entities are part of the PU
  ******************************************************************************/  
 package org.eclipse.persistence.platform.database.oracle;
 
@@ -17,7 +19,6 @@ import java.io.Writer;
 import java.lang.reflect.Constructor;
 import java.security.AccessController;
 import java.sql.Connection;
-import java.sql.Date;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
@@ -43,6 +44,7 @@ import org.eclipse.persistence.exceptions.ConversionException;
 import org.eclipse.persistence.exceptions.DatabaseException;
 import org.eclipse.persistence.exceptions.QueryException;
 import org.eclipse.persistence.internal.databaseaccess.*;
+import org.eclipse.persistence.platform.database.OraclePlatform;
 import org.eclipse.persistence.queries.Call;
 import org.eclipse.persistence.queries.ValueReadQuery;
 import org.eclipse.persistence.internal.helper.*;
@@ -96,6 +98,13 @@ public class Oracle9Platform extends Oracle8Platform {
      */
     protected transient boolean isConnectionDataInitialized;
     
+    /** Indicates whether time component of java.sql.Date should be truncated (hours, minutes, seconds all set to zero)
+     * before been passed as a parameter to PreparedStatement. 
+     * Starting with version 12.1 oracle jdbc Statement.setDate no longer zeroes sql.Date's entire time component (only milliseconds).
+     * Set this flag to true to make the platform to truncate days/hours/minutes before passing the date to Statement.setDate method.
+     */
+    protected boolean shouldTruncateDate;
+    
     private XMLTypeFactory xmlTypeFactory;
 
     /**
@@ -150,12 +159,25 @@ public class Oracle9Platform extends Oracle8Platform {
         }
     }
 
+    /**
+     * Copy the state into the new platform.
+     */
+    public void copyInto(Platform platform) {
+        super.copyInto(platform);
+        if (!(platform instanceof Oracle9Platform)) {
+            return;
+        }
+        Oracle9Platform oracle9Platform = (Oracle9Platform)platform;
+        oracle9Platform.setShouldTruncateDate(shouldTruncateDate());
+    }
+    
     /**
      * INTERNAL:
      * Get a timestamp value from a result set.
      * Overrides the default behavior to specifically return a timestamp.  Added
      * to overcome an issue with the oracle 9.0.1.4 JDBC driver.
      */
+    @Override
     public Object getObjectFromResultSet(ResultSet resultSet, int columnNumber, int type, AbstractSession session) throws java.sql.SQLException {
         //Bug#3381652 10G Drivers return sql.Date instead of timestamp on DATE field
         if ((type == Types.TIMESTAMP) || (type == Types.DATE)) {
@@ -164,7 +186,9 @@ public class Oracle9Platform extends Oracle8Platform {
             return getTIMESTAMPTZFromResultSet(resultSet, columnNumber, type, session);
         } else if (type == oracle.jdbc.OracleTypes.TIMESTAMPLTZ) {
             return getTIMESTAMPLTZFromResultSet(resultSet, columnNumber, type, session);
-        } else if (type == OracleTypes.OPAQUE) {
+        } else if (type == OracleTypes.ROWID) {
+            return resultSet.getString(columnNumber);
+        } else if (type == OracleTypes.OPAQUE || type == Types_SQLXML) {
             try {
                 Object result = resultSet.getObject(columnNumber);
                 if(!(result instanceof OPAQUE)) {
@@ -199,7 +223,7 @@ public class Oracle9Platform extends Oracle8Platform {
             //It has been fixed in the next version for both streams
             Timestamp timestampToWrap = tsTZ.timestampValue(connection);
             TimeZone timezoneToWrap = TIMESTAMPHelper.extractTimeZone(tsTZ.toBytes());
-            return new TIMESTAMPTZWrapper(timestampToWrap, timezoneToWrap, isTimestampInGmt(connection));
+            return new TIMESTAMPTZWrapper(timestampToWrap, timezoneToWrap, this.isTimestampInGmt);
         }
         return null;
     }
@@ -218,7 +242,7 @@ public class Oracle9Platform extends Oracle8Platform {
             Timestamp timestampToWrap = TIMESTAMPLTZ.toTimestamp(connection, tsLTZ.toBytes());
             String sessionTimeZone = ((OracleConnection)connection).getSessionTimeZone();
             //Bug#4364359  Add a separate wrapper for TIMESTAMPLTZ.  
-            return new TIMESTAMPLTZWrapper(timestampToWrap, sessionTimeZone, isLtzTimestampInGmt(connection));
+            return new TIMESTAMPLTZWrapper(timestampToWrap, sessionTimeZone, this.isLtzTimestampInGmt);
         }
         return null;
     }
@@ -232,6 +256,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * kept in sync: shouldCustomModifyInDatabaseCall should return true if and only if the field
      * is handled by customModifyInDatabaseCall.
      */
+    @Override
     public boolean shouldUseCustomModifyForCall(DatabaseField field) {
         Class type = field.getType();
         if ((type != null) && isOracle9Specific(type)) {
@@ -244,9 +269,10 @@ public class Oracle9Platform extends Oracle8Platform {
      * INTERNAL:
      * Allow the use of XMLType operators on this platform.
      */
+    @Override
     protected void initializePlatformOperators() {
         super.initializePlatformOperators();
-        addOperator(ExpressionOperator.extract());
+        addOperator(ExpressionOperator.extractXml());
         addOperator(ExpressionOperator.extractValue());
         addOperator(ExpressionOperator.existsNode());
         addOperator(ExpressionOperator.isFragment());
@@ -263,6 +289,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * Add XMLType as the default database type for org.w3c.dom.Documents.
      * Add TIMESTAMP, TIMESTAMP WITH TIME ZONE and TIMESTAMP WITH LOCAL TIME ZONE
      */
+    @Override
     protected Hashtable buildFieldTypes() {
         Hashtable fieldTypes = super.buildFieldTypes();
         fieldTypes.put(org.w3c.dom.Document.class, new FieldTypeDefinition("sys.XMLType"));
@@ -275,10 +302,22 @@ public class Oracle9Platform extends Oracle8Platform {
         return fieldTypes;
     }
 
+    /**
+     * Build the hint string used for first rows.
+     * 
+     * Allows it to be overridden
+     * @param max
+     * @return
+     */
+    protected String buildFirstRowsHint(int max){
+        return HINT_START + '(' + max + ')'+ HINT_END;
+    }
+    
     /**
      * INTERNAL:
      * Add TIMESTAMP, TIMESTAMP WITH TIME ZONE and TIMESTAMP WITH LOCAL TIME ZONE
      */
+    @Override
     protected Map<String, Class> buildClassTypes() {
         Map<String, Class> classTypeMapping = super.buildClassTypes();
         classTypeMapping.put("TIMESTAMP", ORACLE_SQL_TIMESTAMP);
@@ -287,6 +326,7 @@ public class Oracle9Platform extends Oracle8Platform {
         return classTypeMapping;
     }
 
+    @Override
     public Object clone() {
         Oracle9Platform clone = (Oracle9Platform)super.clone();
         clone.clearConnectionData();
@@ -297,10 +337,14 @@ public class Oracle9Platform extends Oracle8Platform {
      * INTERNAL:
      * Allow for conversion from the Oracle type to the Java type.
      */
+    @Override
     public Object convertObject(Object sourceObject, Class javaClass) throws ConversionException, DatabaseException {
         if ((javaClass == null) || ((sourceObject != null) && (sourceObject.getClass() == javaClass))) {
             return sourceObject;
         }
+        if (sourceObject == null) {
+            return super.convertObject(sourceObject, javaClass);
+        }
 
         Object valueToConvert = sourceObject;
 
@@ -309,8 +353,8 @@ public class Oracle9Platform extends Oracle8Platform {
             return sourceObject;
         }
         
-        if(javaClass == TIMESTAMPTypes.TIMESTAMPTZ_CLASS) {
-            if(sourceObject instanceof java.util.Date) {
+        if (javaClass == TIMESTAMPTypes.TIMESTAMPTZ_CLASS) {
+            if (sourceObject instanceof java.util.Date) {
                 Calendar cal = Calendar.getInstance();
                 cal.setTimeInMillis(((java.util.Date)sourceObject).getTime());
                 return cal;
@@ -371,6 +415,7 @@ public class Oracle9Platform extends Oracle8Platform {
      *    Appends an Oracle specific Timestamp, if usesNativeSQL is true otherwise use the ODBC format.
      *    Native Format: to_timestamp ('1997-11-06 10:35:45.656' , 'yyyy-mm-dd hh:mm:ss.ff')
      */
+    @Override
     protected void appendTimestamp(java.sql.Timestamp timestamp, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("to_timestamp('");
@@ -389,6 +434,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * (DST) to_timestamp_tz ('1997-11-06 10:35:45.345 America/Los_Angeles','yyyy-mm-dd hh:mm:ss.ff TZR TZD')
      * (non-DST) to_timestamp_tz ('1997-11-06 10:35:45.345 America/Los_Angeles','yyyy-mm-dd hh:mm:ss.ff TZR')
      */
+    @Override
     protected void appendCalendar(Calendar calendar, Writer writer) throws IOException {
         if (usesNativeSQL()) {
             writer.write("to_timestamp_tz('");
@@ -407,77 +453,38 @@ public class Oracle9Platform extends Oracle8Platform {
     /**
      * INTERNAL:
      */
-    protected synchronized void initializeConnectionData(Connection conn) throws SQLException {
-        if(isConnectionDataInitialized) {
+    @Override
+    public void initializeConnectionData(Connection connection) throws SQLException {
+        if (this.isConnectionDataInitialized || (connection == null) || (connection.getMetaData() == null)) {
             return;
         }
-        driverVersion = conn.getMetaData().getDriverVersion();
+        this.driverVersion = connection.getMetaData().getDriverVersion();
         // printCalendar for versions greater or equal 9 and less than 10.2.0.4
-        shouldPrintCalendar = Helper.compareVersions("9", driverVersion) <= 0 && Helper.compareVersions(driverVersion, "10.2.0.4") < 0;
-        if(Helper.compareVersions(driverVersion, "11.1.0.7") < 0) {
-            isTimestampInGmt = false;
-        } else {
-            if(conn instanceof OracleConnection) {
-                String timestampTzInGmtPropStr = ((OracleConnection)conn).getProperties().getProperty("oracle.jdbc.timestampTzInGmt", "true");
-                isTimestampInGmt = timestampTzInGmtPropStr.equalsIgnoreCase("true");
+        this.shouldPrintCalendar = Helper.compareVersions("9", this.driverVersion) <= 0 && Helper.compareVersions(this.driverVersion, "10.2.0.4") < 0;
+        if( Helper.compareVersions(this.driverVersion, "11.1.0.7") >= 0) {
+            if(connection instanceof OracleConnection) {
+                String timestampTzInGmtPropStr = ((OracleConnection)connection).getProperties().getProperty("oracle.jdbc.timestampTzInGmt", "true");
+                this.isTimestampInGmt = timestampTzInGmtPropStr.equalsIgnoreCase("true");
             } else {
-                isTimestampInGmt = true;
+                this.isTimestampInGmt = true;
+            }
+            if (Helper.compareVersions(this.driverVersion, "11.2.0.2") >= 0) {
+                this.isLtzTimestampInGmt = true;
             }
-            isLtzTimestampInGmt = Helper.compareVersions(driverVersion, "11.2.0.2") >= 0;
         }
-        isConnectionDataInitialized = true;
+        this.isConnectionDataInitialized = true;
     }
     
-    public synchronized void clearConnectionData() {
-        driverVersion = null;
-        isConnectionDataInitialized = false;
-    }
-    
-    /**
-     * INTERNAL:
-     * Return driverVersion.
-     */
-    public String getDriverVersion(Connection conn) throws SQLException {
-        if(!isConnectionDataInitialized) {
-            initializeConnectionData(conn);
-        }
-        return driverVersion;
-    }
-    
-    /**
-     * INTERNAL:
-     */
-    public boolean shouldPrintCalendar(Connection conn) throws SQLException {
-        if(!isConnectionDataInitialized) {
-            initializeConnectionData(conn);
-        }
-        return shouldPrintCalendar;
-    }
-    
-    /**
-     * INTERNAL:
-     */
-    public boolean isTimestampInGmt(Connection conn) throws SQLException {
-        if(!isConnectionDataInitialized) {
-            initializeConnectionData(conn);
-        }
-        return isTimestampInGmt;
-    }
-    
-    /**
-     * INTERNAL:
-     */
-    public boolean isLtzTimestampInGmt(Connection conn) throws SQLException {
-        if(!isConnectionDataInitialized) {
-            initializeConnectionData(conn);
-        }
-        return isLtzTimestampInGmt;
+    public void clearConnectionData() {
+        this.driverVersion = null;
+        this.isConnectionDataInitialized = false;
     }
     
     /**
      * INTERNAL:
      * Clears both implicit and explicit caches of OracleConnection
      */
+    @Override
     public void clearOracleConnectionCache(Connection conn) {
         if(conn instanceof OracleConnection){
             OracleConnection oracleConnection = (OracleConnection)conn;
@@ -504,12 +511,16 @@ public class Oracle9Platform extends Oracle8Platform {
      *    Binding starts with a 1 not 0, so make sure that index > 0.
      *  Treat Calendar separately. Bind Calendar as TIMESTAMPTZ.
      */
+    @Override
     public void setParameterValueInDatabaseCall(Object parameter, PreparedStatement statement, int index, AbstractSession session) throws SQLException {
         if (parameter instanceof Calendar) {
             Calendar calendar = (Calendar)parameter;
             Connection conn = getConnection(session, statement.getConnection());
-            TIMESTAMPTZ tsTZ = TIMESTAMPHelper.buildTIMESTAMPTZ(calendar, conn, shouldPrintCalendar(conn));
+            TIMESTAMPTZ tsTZ = TIMESTAMPHelper.buildTIMESTAMPTZ(calendar, conn, this.shouldPrintCalendar);
             statement.setObject(index, tsTZ);
+        } else if (this.shouldTruncateDate && parameter instanceof java.sql.Date) {
+            // hours, minutes, seconds all set to zero
+            statement.setDate(index, Helper.truncateDateIgnoreMilliseconds((java.sql.Date)parameter));
         } else {
             super.setParameterValueInDatabaseCall(parameter, statement, index, session);
         }
@@ -519,6 +530,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * INTERNAL:
      * Answer the timestamp from the server. Convert TIMESTAMPTZ to Timestamp
      */
+    @Override
     public java.sql.Timestamp getTimestampFromServer(AbstractSession session, String sessionName) {
         if (getTimestampQuery() != null) {
             getTimestampQuery().setSessionName(sessionName);
@@ -533,10 +545,12 @@ public class Oracle9Platform extends Oracle8Platform {
      * This method returns the query to select the SYSTIMESTAMP as TIMESTAMPTZ
      * from the server for Oracle9i.
      */
+    @Override
     public ValueReadQuery getTimestampQuery() {
         if (timestampQuery == null) {
             timestampQuery = new ValueReadQuery();
             timestampQuery.setSQLString("SELECT SYSTIMESTAMP FROM DUAL");
+            timestampQuery.setAllowNativeSQLQuery(true);
         }
         return timestampQuery;
     }
@@ -545,6 +559,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * INTERNAL:
      * Return the current SYSTIMESTAMP as TIMESTAMPTZ from the server.
      */
+    @Override
     public String serverTimestampString() {
         return "SYSTIMESTAMP";
     }
@@ -576,32 +591,13 @@ public class Oracle9Platform extends Oracle8Platform {
         return vec;
     }
 
-    /**
-     * PUBLIC:
-     * Set if the locator is required for the LOB write. The default is true.
-     * For Oracle thin driver, the locator is recommended for large size
-     * ( >4k for Oracle8, >5.9K for Oracle9) BLOB/CLOB value write.
-     */
-    public void setShouldUseLocatorForLOBWrite(boolean usesLocatorForLOBWrite) {
-        this.usesLocatorForLOBWrite = usesLocatorForLOBWrite;
-    }
-
-    /**
-     * PUBLIC:
-     * Return if the locator is required for the LOB write. The default is true.
-     * For Oracle thin driver, the locator is recommended for large size
-     * ( >4k for Oracle8, >5.9K for Oracle9) BLOB/CLOB value write.
-     */
-    public boolean shouldUseLocatorForLOBWrite() {
-        return usesLocatorForLOBWrite;
-    }
-
     /**
      * PUBLIC:
      * Return the BLOB/CLOB value limits on thin driver. The default value is 0.
      * If usesLocatorForLOBWrite is true, locator will be used in case the
      * lob's size is larger than lobValueLimit.
      */
+    @Override
     public int getLobValueLimits() {
         return lobValueLimits;
     }
@@ -612,6 +608,7 @@ public class Oracle9Platform extends Oracle8Platform {
     * If usesLocatorForLOBWrite is true, locator will be used in case the
     * lob's size is larger than lobValueLimit.
     */
+    @Override
     public void setLobValueLimits(int lobValueLimits) {
         this.lobValueLimits = lobValueLimits;
     }    
@@ -629,6 +626,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * INTERNAL:
      * Used in write LOB method only to identify a CLOB.
      */
+    @Override
     protected boolean isClob(Class type) {
         return NCLOB.equals(type) || super.isClob(type);
     }
@@ -640,6 +638,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * In these special cases the method returns a wrapper object
      * which knows whether it should be bound or appended and knows how to do that.
      */
+    @Override
     public Object getCustomModifyValueForCall(Call call, Object value, DatabaseField field, boolean shouldBind) {
         Class type = field.getType();
         if ((type != null) && isOracle9Specific(type)) {
@@ -681,6 +680,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * @param javaClass - the class that is converted from
      * @return - a vector of classes
      */
+    @Override
     public Vector getDataTypesConvertedFrom(Class javaClass) {
         if (dataTypesConvertedFromAClass == null) {
             dataTypesConvertedFromAClass = new Hashtable(5);
@@ -711,6 +711,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * @param javaClass - the class that is converted to
      * @return - a vector of classes
      */
+    @Override
     public Vector getDataTypesConvertedTo(Class javaClass) {
         if (dataTypesConvertedToAClass == null) {
             dataTypesConvertedToAClass = new Hashtable(5);
@@ -732,12 +733,13 @@ public class Oracle9Platform extends Oracle8Platform {
     
     
     /**
-     * Return the JDBC type for the given database field.
+     * Return the JDBC type for the given database field to be passed to Statement.setNull
      * The Oracle driver does not like the OPAQUE type so VARCHAR must be used.
      */
-    public int getJDBCType(DatabaseField field) {
-        int type = super.getJDBCType(field);
-        if (type == OracleTypes.OPAQUE) {
+    @Override
+    public int getJDBCTypeForSetNull(DatabaseField field) {
+        int type = getJDBCType(field);
+        if (type == OracleTypes.OPAQUE || type == Types_SQLXML) {
             // VARCHAR seems to work, driver does not like OPAQUE.
             return java.sql.Types.VARCHAR;
         }
@@ -748,6 +750,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * Return the JDBC type for the Java type.
      * The Oracle driver does not like the OPAQUE type so VARCHAR must be used.
      */
+    @Override
     public int getJDBCType(Class javaType) {
         if (javaType == XMLTYPE) {
             //return OracleTypes.OPAQUE;
@@ -766,10 +769,17 @@ public class Oracle9Platform extends Oracle8Platform {
      * @param isStatementPrepared - flag is set to true if this statement is prepared 
      * @return - number of rows modified/deleted by this statement
      */
-    public int executeBatch(Statement statement,  boolean isStatementPrepared)throws java.sql.SQLException {
-        if (usesNativeBatchWriting() && isStatementPrepared){
-            return((OraclePreparedStatement)statement).sendBatch(); 
-        }else {
+    @Override
+    public int executeBatch(Statement statement,  boolean isStatementPrepared) throws SQLException {
+        if (usesNativeBatchWriting() && isStatementPrepared) {
+            int rowCount = 0;
+            try {
+                rowCount = ((OraclePreparedStatement)statement).sendBatch();
+            } finally {
+                ((OraclePreparedStatement) statement).setExecuteBatch(1);
+            }
+            return rowCount;
+        } else {
             return super.executeBatch(statement, isStatementPrepared);
         }
     }
@@ -782,6 +792,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * 
      * @return - number of rows modified/deleted by this statement if it was executed (0 if it wasn't)
      */
+    @Override
     public int addBatch(PreparedStatement statement) throws java.sql.SQLException {
         if (usesNativeBatchWriting()){
             return statement.executeUpdate(); 
@@ -796,10 +807,11 @@ public class Oracle9Platform extends Oracle8Platform {
      * 
      * @return - statement to be used for batch writing
      */
-    public Statement prepareBatchStatement(Statement statement) throws java.sql.SQLException {
+    @Override
+    public Statement prepareBatchStatement(Statement statement, int maxBatchWritingSize) throws java.sql.SQLException {
         if (usesNativeBatchWriting()){
             //add max statement setting
-            ((OraclePreparedStatement) statement).setExecuteBatch(getMaxBatchWritingSize()); 
+            ((OraclePreparedStatement) statement).setExecuteBatch(maxBatchWritingSize); 
         }
         return statement;
     }
@@ -835,6 +847,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * Indicates whether the passed object is an instance of XDBDocument.
      * @return boolean
      */
+    @Override
     public boolean isXDBDocument(Object obj) {
         return getXMLTypeFactory().isXDBDocument(obj);
     }
@@ -843,6 +856,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * INTERNAL:
      * Indicates whether this Oracle platform can unwrap Oracle connection.
      */
+    @Override
     public boolean canUnwrapOracleConnection() {
         return true;
     }
@@ -851,6 +865,7 @@ public class Oracle9Platform extends Oracle8Platform {
      * INTERNAL:
      * If can unwrap returns unwrapped Oracle connection, otherwise original connection.
      */
+    @Override
     public Connection unwrapOracleConnection(Connection connection) {
         //Bug#4607977  Use getPhysicalConnection() instead of physicalConnectionWithin() because it's not suppported in 9.2 driver
         if(connection instanceof oracle.jdbc.internal.OracleConnection){
@@ -864,13 +879,15 @@ public class Oracle9Platform extends Oracle8Platform {
      * PUBLIC:
      * Return is this is the Oracle 9 platform. 
      */
+    @Override
     public boolean isOracle9() {
-	   return true;
+        return true;
     }
 
     /**
      * INTERNAL:
      */
+    @Override
     public ConnectionCustomizer createConnectionCustomizer(Accessor accessor, AbstractSession session) {
         Object proxyTypeValue = session.getProperty(PersistenceUnitProperties.ORACLE_PROXY_TYPE);
         if (proxyTypeValue == null || ((proxyTypeValue instanceof String) && ((String)proxyTypeValue).length() == 0)) {
@@ -879,4 +896,47 @@ public class Oracle9Platform extends Oracle8Platform {
             return new OracleJDBC_10_1_0_2ProxyConnectionCustomizer(accessor, session);
         }
     }
+    
+    /**
+     * INTERNAL: Return the driver version.
+     */
+    public String getDriverVersion() {
+        return driverVersion;
+    }
+    
+    /**
+     * INTERNAL: Return if timestamps are returned in GMT by the driver.
+     */
+    public boolean isTimestampInGmt() {
+        return isTimestampInGmt;
+    }
+    
+    /**
+     * INTERNAL: Return if ltz timestamps are returned in GMT by the driver.
+     */
+    public boolean isLtzTimestampInGmt() {
+        return isLtzTimestampInGmt;
+    }
+
+    /**
+     * PUBLIC:
+     * Indicates whether time component of java.sql.Date should be truncated (hours, minutes, seconds all set to zero)
+     * before been passed as a parameter to PreparedStatement. 
+     * Starting with version 12.1 oracle jdbc Statement.setDate no longer zeroes sql.Date's entire time component (only milliseconds).
+     * "true" indicates that the platform truncates days/hours/minutes before passing the date to Statement.setDate method.
+     */
+    public boolean shouldTruncateDate() {
+        return shouldTruncateDate;
+    }
+
+    /**
+     * PUBLIC:
+     * Indicates whether time component of java.sql.Date should be truncated (hours, minutes, seconds all set to zero)
+     * before been passed as a parameter to PreparedStatement. 
+     * Starting with version 12.1 oracle jdbc Statement.setDate no longer zeroes sql.Date's entire time component (only milliseconds).
+     * Set this flag to true to make the platform to truncate days/hours/minutes before passing the date to Statement.setDate method.
+     */
+    public void setShouldTruncateDate(boolean shouldTruncateDate) {
+        this.shouldTruncateDate = shouldTruncateDate;
+    }
 }
