@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -129,6 +129,12 @@ public class SDOProperty implements Property, Serializable {
         setType(aType);
     }
     
+    public SDOProperty(HelperContext aContext, String aName, SDOType aType, boolean hasMany) {
+        this(aContext, aName);
+        setType(aType);
+        this.hasMany = hasMany;
+    }
+    
     public SDOProperty(HelperContext aContext, String aUri, String aName, SDOType aType) {
     	this(aContext, aName, aType);
     	this.setUri(aUri);
@@ -723,11 +729,11 @@ public class SDOProperty implements Property, Serializable {
         mapping.addFieldTransformer(xpath, transformer);
         
         NamespaceResolver nsr = new NamespaceResolver();
-        nsr.put(XMLConstants.XMLNS, XMLConstants.XMLNS_URL);
+        nsr.put(javax.xml.XMLConstants.XMLNS_ATTRIBUTE, javax.xml.XMLConstants.XMLNS_ATTRIBUTE_NS_URI);
 
         XMLField field = new XMLField();
         field.setNamespaceResolver(nsr);
-        field.setXPath(xpathMinusText + "/@" + XMLConstants.XMLNS + ":" + QNameTransformer.QNAME_NAMESPACE_PREFIX);
+        field.setXPath(xpathMinusText + "/@" + javax.xml.XMLConstants.XMLNS_ATTRIBUTE + ":" + QNameTransformer.QNAME_NAMESPACE_PREFIX);
         
         mapping.addFieldTransformer(field, new NamespaceURITransformer());
         
@@ -736,6 +742,7 @@ public class SDOProperty implements Property, Serializable {
     
     private DatabaseMapping buildXMLDirectMapping(String mappingUri) {
         XMLDirectMapping mapping = new XMLDirectMapping();
+        mapping.setNullValueMarshalled(true);
         mapping.setAttributeName(getName());
         String xpath = getQualifiedXPath(mappingUri, true);
         mapping.setXPath(xpath);
@@ -760,7 +767,13 @@ public class SDOProperty implements Property, Serializable {
       		// elements or attributes
             setIsSetOptionalPolicyOnMapping(mapping, propertyName);
         }
-        mapping.getNullPolicy().setNullRepresentedByEmptyNode(true);
+        if(this.isDefaultSet()) {
+            mapping.setNullValue(getDefault());
+            mapping.getNullPolicy().setNullRepresentedByEmptyNode(false);
+            
+        } else {
+            mapping.getNullPolicy().setNullRepresentedByEmptyNode(true);
+        }
         return mapping;
     }
 
