@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -28,7 +28,6 @@ import org.eclipse.persistence.sdo.helper.SDOXMLHelper;
 import commonj.sdo.helper.HelperContext;
 import commonj.sdo.helper.XMLDocument;
 import commonj.sdo.impl.ExternalizableDelegator;
-import commonj.sdo.impl.HelperProvider;
 
 /**
  * INTERNAL:
@@ -123,9 +122,9 @@ public class SDOResolvable implements ExternalizableDelegator.Resolvable {
 	/** Visibility reduced from [public] in 2.1.0. May 15 2007 */
     /** hold the context containing all helpers so that we can preserve inter-helper relationships */
     private transient HelperContext aHelperContext;
-
+    
     public SDOResolvable() {
-        aHelperContext = HelperProvider.getDefaultContext();
+        aHelperContext = SDOHelperContext.getHelperContext();
     }
 
     /**
