@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -33,10 +33,10 @@ import org.eclipse.persistence.sdo.SDOProperty;
 import org.eclipse.persistence.sdo.SDOType;
 import org.eclipse.persistence.exceptions.ConversionException;
 import org.eclipse.persistence.exceptions.SDOException;
+import org.eclipse.persistence.internal.core.sessions.CoreAbstractSession;
 import org.eclipse.persistence.internal.helper.ClassConstants;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
-import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.mappings.DatabaseMapping;
 import org.eclipse.persistence.oxm.XMLConstants;
 
@@ -62,8 +62,8 @@ public class SDODataHelper implements DataHelper {
     }
 
     /**
-     * The specified TimeZone will be used for all String to date object
-     * conversions. By default the TimeZone from the JVM is used.
+     * The specified TimeZone will be used for all String to Date object
+     * conversions.  By default the GMT time zone is used.
      */
     public void setTimeZone(TimeZone timeZone) {
         if (null == timeZone) {
@@ -186,7 +186,14 @@ public class SDODataHelper implements DataHelper {
         if (calendar == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.DATE_TIME_QNAME);
+
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.DATE_TIME_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+
+        return s;
     }
 
     /**
@@ -212,7 +219,14 @@ public class SDODataHelper implements DataHelper {
         if (calendar == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_DAY_QNAME);
+
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_DAY_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+
+        return s;
     }
 
     /**
@@ -282,7 +296,13 @@ public class SDODataHelper implements DataHelper {
         if (calendar == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_MONTH_QNAME);
+
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_MONTH_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+        return s;
     }
 
     /**
@@ -295,7 +315,13 @@ public class SDODataHelper implements DataHelper {
         if (date == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromDate(date, XMLConstants.G_MONTH_QNAME);
+        
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromDate(date, XMLConstants.G_MONTH_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+        return s;
     }
 
     /**
@@ -386,7 +412,13 @@ public class SDODataHelper implements DataHelper {
         if (calendar == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.TIME_QNAME);
+
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.TIME_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+        return s;
     }
 
     /**
@@ -412,7 +444,13 @@ public class SDODataHelper implements DataHelper {
         if (calendar == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_YEAR_QNAME);
+
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_YEAR_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+        return s;
     }
 
     /**
@@ -464,7 +502,13 @@ public class SDODataHelper implements DataHelper {
         if (calendar == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_YEAR_MONTH_QNAME);
+
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.G_YEAR_MONTH_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+        return s;
     }
 
     /**
@@ -478,7 +522,13 @@ public class SDODataHelper implements DataHelper {
         if (calendar == null) {
             return null;
         }
-        return getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.DATE_QNAME);
+
+        // Temporarily turn off TZ qualification
+        boolean wasTimezoneQualified = getXMLConversionManager().isTimeZoneQualified();
+        getXMLConversionManager().setTimeZoneQualified(false);
+        String s = getXMLConversionManager().stringFromCalendar(calendar, XMLConstants.DATE_QNAME);
+        getXMLConversionManager().setTimeZoneQualified(wasTimezoneQualified);
+        return s;
     }
 
     /**
@@ -540,6 +590,8 @@ public class SDODataHelper implements DataHelper {
             return toDate(value);
         } else if (convertClass == ClassConstants.CALENDAR) {
             return toCalendar(value);
+        } else if(value == SDOConstants.EMPTY_STRING && (convertClass == ClassConstants.LONG || convertClass == ClassConstants.SHORT || convertClass == ClassConstants.INTEGER || convertClass == ClassConstants.FLOAT || convertClass == ClassConstants.DOUBLE || convertClass == ClassConstants.BOOLEAN || convertClass == ClassConstants.CHAR || convertClass == ClassConstants.QNAME )){
+       	    return null;        	
         } else {
             return getXMLConversionManager().convertObject(value, convertClass);
         }
@@ -661,10 +713,10 @@ public class SDODataHelper implements DataHelper {
             try {
                 Class binaryDataHelper = PrivilegedAccessHelper.getClassForName("org.eclipse.persistence.internal.oxm.XMLBinaryDataHelper");
                 java.lang.reflect.Method getHelperMethod = PrivilegedAccessHelper.getMethod(binaryDataHelper, "getXMLBinaryDataHelper", new Class[] {}, false);
-                java.lang.reflect.Method stringToDataHandlerMethod = PrivilegedAccessHelper.getMethod(binaryDataHelper, "stringFromDataHandler", new Class[] { Object.class, QName.class, AbstractSession.class }, false);
+                java.lang.reflect.Method stringToDataHandlerMethod = PrivilegedAccessHelper.getMethod(binaryDataHelper, "stringFromDataHandler", new Class[] { Object.class, QName.class, CoreAbstractSession.class }, false);
 
                 Object helper = PrivilegedAccessHelper.invokeMethod(getHelperMethod, binaryDataHelper, new Object[] {});
-                String result = (String) PrivilegedAccessHelper.invokeMethod(stringToDataHandlerMethod, helper, new Object[] { value, xsdType, ((SDOXMLHelper) getHelperContext().getXMLHelper()).getXmlContext().getSession(0) });
+                String result = (String) PrivilegedAccessHelper.invokeMethod(stringToDataHandlerMethod, helper, new Object[] { value, xsdType, ((SDOXMLHelper) getHelperContext().getXMLHelper()).getXmlContext().getSession() });
                 return result;
             } catch (Exception ex) {
                 return (String) getXMLConversionManager().convertObject(value, ClassConstants.STRING, xsdType);
@@ -722,23 +774,27 @@ public class SDODataHelper implements DataHelper {
         if (null == property) {
             throw new IllegalArgumentException(SDOException.conversionError(null));
         }
-        Type convertType = property.getType();
-        if (property.isMany()) {
-            if (value == null) {
-                return null;
-            } else if (!(value instanceof List)) {
-                throw new IllegalArgumentException(SDOException.conversionError(null));
-            } else {
-                List theList = (List) value;
-                Object nextItem = null;
-                for (int i = 0; i < theList.size(); i++) {
-                    nextItem = theList.get(i);
-                    theList.set(i, convert(convertType, nextItem));
+        try {
+            Type convertType = property.getType();
+            if (property.isMany()) {
+                if (value == null) {
+                    return null;
+                } else if (!(value instanceof List)) {
+                    throw new IllegalArgumentException(SDOException.conversionError(null));
+                } else {
+                    List theList = (List) value;
+                    Object nextItem = null;
+                    for (int i = 0; i < theList.size(); i++) {
+                        nextItem = theList.get(i);
+                        theList.set(i, convert(convertType, nextItem));
+                    }
+                    return theList;
                 }
-                return theList;
+            } else {
+                return convert(convertType, value);
             }
-        } else {
-            return convert(convertType, value);
+        } catch (ConversionException e) {
+            throw SDOException.invalidPropertyValue(property.getName(), property.getType().getURI() + '#' + property.getType().getName(), value.getClass().getName(), value.toString(), e);
         }
     }
 
