@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -26,7 +26,7 @@ import org.eclipse.persistence.sdo.SDOProperty;
 import org.eclipse.persistence.sdo.SDOSetting;
 import org.eclipse.persistence.sdo.SDOType;
 import org.eclipse.persistence.sessions.Session;
-import org.eclipse.persistence.internal.descriptors.Namespace;
+import org.eclipse.persistence.internal.oxm.Namespace;
 import org.eclipse.persistence.internal.oxm.XMLConversionManager;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.oxm.NamespaceResolver;
@@ -78,12 +78,12 @@ public class SDOMarshalListener implements XMLMarshalListener {
 
             String rootElementName = this.marshalledObjectRootQName.getLocalPart();
             String rootNamespaceUri = this.marshalledObjectRootQName.getNamespaceURI();
-            if(rootNamespaceUri != null && !rootNamespaceUri.equals("")) {
-                NamespaceResolver resolver = getRootMarshalRecord().getNamespaceResolver();
+            if(rootNamespaceUri != null && !rootNamespaceUri.equals(XMLConstants.EMPTY_STRING)) {            	
+                org.eclipse.persistence.internal.oxm.NamespaceResolver resolver = getRootMarshalRecord().getNamespaceResolver();
                 if(resolver != null) {
                     String prefix = resolver.resolveNamespaceURI(this.marshalledObjectRootQName.getNamespaceURI());
                     if(prefix != null) {
-                        rootElementName = prefix + ":" + rootElementName;
+                        rootElementName = prefix + XMLConstants.COLON + rootElementName;                        
                     }
                 }
             }
@@ -170,14 +170,14 @@ public class SDOMarshalListener implements XMLMarshalListener {
                 for (int j = 0; j < namespaces.size(); j++) {
                     Namespace next = (Namespace)namespaces.get(j);
                     if (declareNamespace(next.getNamespaceURI(), next.getPrefix(), changeSummary.getRootObject())) {
-                        csNode.setAttributeNS(XMLConstants.XMLNS_URL, XMLConstants.XMLNS + ":" + next.getPrefix(), next.getNamespaceURI());
+                        csNode.setAttributeNS(javax.xml.XMLConstants.XMLNS_ATTRIBUTE_NS_URI, javax.xml.XMLConstants.XMLNS_ATTRIBUTE + XMLConstants.COLON + next.getPrefix(), next.getNamespaceURI());
                     }
                 }
 
                 List nextDOSettings = changeSummary.getOldValues(nextModifiedDO);
                 DOMRecord row = new DOMRecord(csNode);
                 
-                Session session = ((SDOXMLHelper)typeHelper.getHelperContext().getXMLHelper()).getXmlContext().getSession(0);
+                Session session = ((SDOXMLHelper)typeHelper.getHelperContext().getXMLHelper()).getXmlContext().getSession();
                 row.setSession((AbstractSession) session);
 
                 //Iterate through SDOSettings for the current modified Object
@@ -234,7 +234,7 @@ public class SDOMarshalListener implements XMLMarshalListener {
         if (value == null) {
             //Marshal out xsi:nil=true   
             DOMRecord row = new DOMRecord(csNode);
-            Session session = ((SDOXMLHelper)typeHelper.getHelperContext().getXMLHelper()).getXmlContext().getSession(0);
+            Session session = ((SDOXMLHelper)typeHelper.getHelperContext().getXMLHelper()).getXmlContext().getSession();
             row.setSession((AbstractSession) session);
             marshalNilAttribute(prop, row);                            
             return;
@@ -305,15 +305,15 @@ public class SDOMarshalListener implements XMLMarshalListener {
 
                 if (schemaContext != null) {
                     String typeValue = schemaContext.substring(1, schemaContext.length());
-                    String schemaInstancePrefix = ((SDOType)value.getType()).getXmlDescriptor().getNonNullNamespaceResolver().resolveNamespaceURI(XMLConstants.SCHEMA_INSTANCE_URL);
+                    String schemaInstancePrefix = ((SDOType)value.getType()).getXmlDescriptor().getNonNullNamespaceResolver().resolveNamespaceURI(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI);
 
                     //may or may not need the xmlns declaration added.
                     String schemaContextUri = schemaContextQName.getNamespaceURI();
                     String schemaContextPrefix = ((SDOType)value.getType()).getXmlDescriptor().getNonNullNamespaceResolver().resolveNamespaceURI(schemaContextUri);
                     if (schemaContextPrefix != null) {
-                        modifiedElement.setAttributeNS(XMLConstants.XMLNS_URL, XMLConstants.XMLNS + ":" + schemaContextPrefix, schemaContextQName.getNamespaceURI());
+                        modifiedElement.setAttributeNS(javax.xml.XMLConstants.XMLNS_ATTRIBUTE_NS_URI, javax.xml.XMLConstants.XMLNS_ATTRIBUTE + XMLConstants.COLON + schemaContextPrefix, schemaContextQName.getNamespaceURI());
                     }
-                    modifiedElement.setAttributeNS(XMLConstants.SCHEMA_INSTANCE_URL, schemaInstancePrefix + ":" + XMLConstants.SCHEMA_TYPE_ATTRIBUTE, typeValue);
+                    modifiedElement.setAttributeNS(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI, schemaInstancePrefix + XMLConstants.COLON + XMLConstants.SCHEMA_TYPE_ATTRIBUTE, typeValue);
                 }
             }
         }
@@ -398,10 +398,10 @@ public class SDOMarshalListener implements XMLMarshalListener {
     }
 
     private String getXPathForProperty(SDOProperty prop) {
-        return getXPathForProperty(prop, false);
+        return getXPathForProperty(prop, false, typeHelper.getNamespaceResolver());
     }
 
-    private String getXPathForProperty(SDOProperty prop, boolean removeText) {
+    private String getXPathForProperty(SDOProperty prop, boolean removeText, NamespaceResolver namespaceResolver) {
         if ((prop).getXmlMapping() != null) {
             String xpath = prop.getXmlMapping().getField().getName();
             if (removeText && xpath.endsWith("/text()")) {
@@ -413,10 +413,10 @@ public class SDOMarshalListener implements XMLMarshalListener {
             if (prop.isOpenContent()) {
                 String uri = prop.getUri();
                 if (uri != null) {
-                    String prefix = typeHelper.getNamespaceResolver().resolveNamespaceURI(uri);
+                    String prefix = namespaceResolver.resolveNamespaceURI(uri);
 
                     if ((prefix != null) && !prefix.equals(SDOConstants.EMPTY_STRING)) {
-                        return prefix + ":" + name;
+                        return prefix + XMLConstants.COLON + name;
                     }
                 }
             }
@@ -577,16 +577,35 @@ public class SDOMarshalListener implements XMLMarshalListener {
 
     private void marshalNilAttribute(SDOProperty property, DOMRecord row) {
         //Marshal out xsi:nil=true   
-        String xsiPrefix = typeHelper.getNamespaceResolver().resolveNamespaceURI(XMLConstants.SCHEMA_INSTANCE_URL);
-        if ((xsiPrefix == null) || xsiPrefix.equals("")) {
-            xsiPrefix = typeHelper.getNamespaceResolver().generatePrefix("xsi");
-            typeHelper.getNamespaceResolver().put(xsiPrefix, XMLConstants.SCHEMA_INSTANCE_URL);
+        NamespaceResolver resolver = null;
+        if(property.getContainingType() != null) {
+            resolver = property.getContainingType().getXmlDescriptor().getNamespaceResolver();
+        } 
+        if(null == resolver) {
+            resolver = typeHelper.getNamespaceResolver();
         }
-        String xPath = getXPathForProperty(property, true);
-        xPath = xPath + "/@" + xsiPrefix + ":nil";
+        String xsiPrefix = resolver.resolveNamespaceURI(javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI);
+        if ((xsiPrefix == null) || xsiPrefix.equals(SDOConstants.EMPTY_STRING)) {
+            NamespaceResolver nsResolverWithXsi = new NamespaceResolver();
+            copyNamespaces(resolver, nsResolverWithXsi);
+            resolver = nsResolverWithXsi;
+            xsiPrefix = resolver.generatePrefix(XMLConstants.SCHEMA_INSTANCE_PREFIX);
+            resolver.put(xsiPrefix, javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI);
+        }
+        String xPath = getXPathForProperty(property, true, resolver);
+        xPath = xPath + "/@" + xsiPrefix + XMLConstants.COLON + XMLConstants.SCHEMA_NIL_ATTRIBUTE;
 
         XMLField field = new XMLField(xPath);
-        field.setNamespaceResolver(typeHelper.getNamespaceResolver());
-        row.put(field, "true");
+        field.setNamespaceResolver(resolver);
+        row.put(field, XMLConstants.BOOLEAN_STRING_TRUE);
+    }
+
+    private void copyNamespaces(NamespaceResolver source, NamespaceResolver target) {
+        if (null != source && null != target) {            
+            if(source.hasPrefixesToNamespaces()) {
+                target.getPrefixesToNamespaces().putAll(source.getPrefixesToNamespaces());
+            }
+            target.setDefaultNamespaceURI(source.getDefaultNamespaceURI());
+        }        
     }
 }
