@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -29,6 +29,7 @@ import org.eclipse.persistence.exceptions.DescriptorException;
 import org.eclipse.persistence.exceptions.XMLMarshalException;
 import org.eclipse.persistence.internal.oxm.StrBuffer;
 import org.eclipse.persistence.internal.oxm.TreeObjectBuilder;
+import org.eclipse.persistence.internal.oxm.record.XMLRecord;
 import org.eclipse.persistence.internal.oxm.record.namespaces.UnmarshalNamespaceResolver;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.oxm.XMLConstants;
@@ -113,7 +114,7 @@ public class SDOUnmappedContentHandler implements UnmappedContentHandler {
             String stringValue = atts.getValue(i);
             String uri = atts.getURI(i);
             String attrName = atts.getLocalName(i);
-            if (XMLConstants.SCHEMA_INSTANCE_URL.equals(uri) && XMLConstants.SCHEMA_TYPE_ATTRIBUTE.equals(attrName)) {
+            if (javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI.equals(uri) && XMLConstants.SCHEMA_TYPE_ATTRIBUTE.equals(attrName)) {
                 int colonIndex = stringValue.indexOf(':');
                 String localPrefix = stringValue.substring(0, colonIndex);
                 String localURI = unmarshalNamespaceResolver.getNamespaceURI(localPrefix);
@@ -138,13 +139,13 @@ public class SDOUnmappedContentHandler implements UnmappedContentHandler {
             String uri = atts.getURI(i);
             String attrName = atts.getLocalName(i);
 
-            if ((atts.getQName(i) != null) && atts.getQName(i).startsWith(XMLConstants.XMLNS + ":")) {
+            if ((atts.getQName(i) != null) && atts.getQName(i).startsWith(javax.xml.XMLConstants.XMLNS_ATTRIBUTE + ":")) {
                 //namespace declaration - do nothing because namespaces were already handled                             
             } else if (isRoot && XMLConstants.SCHEMA_LOCATION.equals(attrName)) {
                 getXmlDocument().setSchemaLocation(stringValue);
             } else if (isRoot && XMLConstants.NO_NS_SCHEMA_LOCATION.equals(attrName)) {
                 getXmlDocument().setNoNamespaceSchemaLocation(stringValue);
-            } else if (XMLConstants.SCHEMA_INSTANCE_URL.equals(uri) && XMLConstants.SCHEMA_TYPE_ATTRIBUTE.equals(attrName)) {
+            } else if (javax.xml.XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI.equals(uri) && XMLConstants.SCHEMA_TYPE_ATTRIBUTE.equals(attrName)) {
                 //do nothing
             } else if (SDOConstants.CHANGESUMMARY_REF.equals(attrName) && SDOConstants.SDO_URL.equals(uri)) {
                 ((SDODataObject)dataObject)._setSdoRef(stringValue);
@@ -413,6 +414,9 @@ public class SDOUnmappedContentHandler implements UnmappedContentHandler {
             unmarshalRecord = new UnmarshalRecord((TreeObjectBuilder)xmlDescriptor.getObjectBuilder());
             unmarshalRecord.setUnmarshalNamespaceResolver(unmarshalNamespaceResolver);
             unmarshalRecord.setAttributes(atts);
+            if(parentRecord != null){
+                unmarshalRecord.setXMLReader(parentRecord.getXMLReader());
+            }
             Class classValue = xmlDescriptor.getInheritancePolicy().classFromRow(unmarshalRecord, session);
             if (classValue == null) {
                 // no xsi:type attribute - look for type indicator on the default root element
@@ -434,7 +438,7 @@ public class SDOUnmappedContentHandler implements UnmappedContentHandler {
                 // sure it is non-abstract
                 if (Modifier.isAbstract(xmlDescriptor.getJavaClass().getModifiers())) {
                     // need to throw an exception here
-                    throw DescriptorException.missingClassIndicatorField(unmarshalRecord, xmlDescriptor.getInheritancePolicy().getDescriptor());
+                    throw DescriptorException.missingClassIndicatorField((XMLRecord) unmarshalRecord, xmlDescriptor.getInheritancePolicy().getDescriptor());
                 }
             }
         }
