@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
  * which accompanies this distribution.
@@ -513,7 +513,7 @@ public class SDOTypeHelperDelegate implements SDOTypeHelper {
             return sdoType.getInstanceClass();
         }
         Class javaClass = null;
-        if (sdoType.getBaseTypes() != null) {
+        if (sdoType.isSubType()) {
             for (int i = 0; i < sdoType.getBaseTypes().size(); i++) {
                 Type baseType = (Type)sdoType.getBaseTypes().get(i);
                 javaClass = getJavaWrapperTypeForSDOType(baseType);
@@ -666,9 +666,9 @@ public class SDOTypeHelperDelegate implements SDOTypeHelper {
         }
         for (int i = 0; i < types.size(); i++) {
             SDOType nextType = (SDOType)types.get(i);
-            if ((!nextType.isDataType() && nextType.getBaseTypes() == null || nextType.getBaseTypes().size() == 0) && nextType.getSubTypes().size() > 0) {
+            if ((!nextType.isDataType() && !nextType.isSubType()) && nextType.isBaseType()) {
                 nextType.setupInheritance(null);
-            } else if (!nextType.isDataType() && nextType.getBaseTypes().size() > 0 && !types.contains(nextType.getBaseTypes().get(0))) {
+            } else if (!nextType.isDataType() && nextType.isSubType() && !types.contains(nextType.getBaseTypes().get(0))) {
                 SDOType baseType = (SDOType)nextType.getBaseTypes().get(0);
                 while (baseType != null && !baseType.isDataType()) {
                     descriptorsToAdd.add(baseType);
@@ -726,6 +726,8 @@ public class SDOTypeHelperDelegate implements SDOTypeHelper {
         addType(type);
         types.add(type);
 
+        type.setAppInfoElements((List) dataObject.get(SDOConstants.APPINFO));
+
         type.setAbstract(dataObject.getBoolean("abstract"));
 
         List baseTypes = dataObject.getList("baseType");
@@ -770,7 +772,7 @@ public class SDOTypeHelperDelegate implements SDOTypeHelper {
         type.setOpen(dataObject.getBoolean("open"));
 
         if (type.isDataType()) {
-            QName typeQName = new QName(XMLConstants.SCHEMA_URL, name);
+            QName typeQName = new QName(javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI, name);
             if (typeHelper.getWrappersHashMap().get(typeQName) != null) {
                 // In the case of a non-user-defined simple type, we already have built-in
                 // wrappers created, so do not continue on building a new WrapperType.
@@ -785,9 +787,9 @@ public class SDOTypeHelperDelegate implements SDOTypeHelper {
             // conversions (eg. "myBinaryElement" could be a restriction of base64Binary
             // or hexBinary.
             QName currentTypeQName = null;
-            if (!type.getBaseTypes().isEmpty()) {
+            if (type.isSubType()) {
                 SDOType baseType = (SDOType) type.getBaseTypes().get(0);
-                currentTypeQName = new QName(XMLConstants.SCHEMA_URL, baseType.getName());
+                currentTypeQName = new QName(javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI, baseType.getName());
             }
 
             // Create the new WrapperType
@@ -808,8 +810,8 @@ public class SDOTypeHelperDelegate implements SDOTypeHelper {
     }
 
     private boolean isBaseTypeBytes(Type theType) {
-        List baseTypes = ((SDOType)theType).getBaseTypes();
-        if (baseTypes.size() > 0) {
+        if (((SDOType) theType).isSubType()) {
+            List baseTypes = ((SDOType)theType).getBaseTypes();
             Type nextType = (Type)baseTypes.get(0);
             if (nextType == SDOConstants.SDO_BYTES) {
                 return true;
@@ -849,6 +851,8 @@ public class SDOTypeHelperDelegate implements SDOTypeHelper {
             }
         }
 
+        newProperty.setAppInfoElements((List) dataObject.get(SDOConstants.APPINFO));
+        
         if (dataObject.isSet("containment")) {
             newProperty.setContainment(dataObject.getBoolean("containment"));
         } else {
