@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -199,7 +199,14 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
                     SDOType theType = (SDOType)optionsDataObject.get(SDOConstants.TYPE_LOAD_OPTION);
                     try{
                         if (theType != null) {
-                            unmarshalledObject = anXMLUnmarshaller.unmarshal(inputSource, theType.getImplClass());
+                            if(theType.isDataType()) {
+                                theType = (SDOType)((SDOTypeHelper)this.aHelperContext.getTypeHelper()).getWrappersHashMap().get(theType.getQName());
+                            }  
+                            if(theType != null) {
+                                unmarshalledObject = anXMLUnmarshaller.unmarshal(inputSource, theType.getImplClass());
+                            } else {
+                                unmarshalledObject = anXMLUnmarshaller.unmarshal(inputSource);
+                            }
                         }else{
                             unmarshalledObject = anXMLUnmarshaller.unmarshal(inputSource);
                         }
@@ -273,7 +280,14 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
                     SDOType theType = (SDOType)optionsDataObject.get(SDOConstants.TYPE_LOAD_OPTION);
                     try{
                         if (theType != null) {
-                            unmarshalledObject = anXMLUnmarshaller.unmarshal(source, theType.getImplClass());
+                            if(theType.isDataType()) {
+                                theType = (SDOType)((SDOTypeHelper)this.aHelperContext.getTypeHelper()).getWrappersHashMap().get(theType.getQName());
+                            }  
+                            if(theType != null) {
+                                unmarshalledObject = anXMLUnmarshaller.unmarshal(source, theType.getImplClass());
+                            } else {
+                                unmarshalledObject = anXMLUnmarshaller.unmarshal(source);
+                            }
                         }else{
                             unmarshalledObject = anXMLUnmarshaller.unmarshal(source);
                         }
@@ -360,10 +374,13 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
     }
 
     public void serialize(XMLDocument xmlDocument, OutputStream outputStream, Object options) throws IOException {
-        OutputStreamWriter writer = new OutputStreamWriter(outputStream, getXmlMarshaller().getEncoding());
-        XMLMarshaller xmlMarshaller = getXmlContext().createMarshaller();
-        xmlMarshaller.setMarshalListener(new SDOMarshalListener(xmlMarshaller, (SDOTypeHelper) aHelperContext.getTypeHelper()));
-        save(xmlDocument, writer, xmlMarshaller);
+    	XMLMarshaller xmlMarshaller = getXmlMarshaller();    	
+    	XMLAttachmentMarshaller attachmentMarshaller = xmlMarshaller.getAttachmentMarshaller();
+    	//temporarily null out the attachment marshaller as it should not be used during serialization
+    	xmlMarshaller.setAttachmentMarshaller(null);
+    	OutputStreamWriter writer = new OutputStreamWriter(outputStream, xmlMarshaller.getEncoding());    	
+        save(xmlDocument, writer, xmlMarshaller);    	
+        xmlMarshaller.setAttachmentMarshaller(attachmentMarshaller);
     }
 
     /**
@@ -434,11 +451,25 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
         writerRecord.setWriter(outputWriter);
         writerRecord.setMarshaller(anXMLMarshaller);
         
-        ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setMarshalledObject(xmlDocument.getRootObject());
-        ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setMarshalledObjectRootQName(new QName(xmlDocument.getRootElementURI(), xmlDocument.getRootElementName()));
-        ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setRootMarshalRecord(writerRecord);
+        SDOMarshalListener listener = ((SDOMarshalListener)anXMLMarshaller.getMarshalListener());
         
-        anXMLMarshaller.marshal(xmlDocument, writerRecord);
+        listener.setMarshalledObject(xmlDocument.getRootObject());
+        listener.setMarshalledObjectRootQName(new QName(xmlDocument.getRootElementURI(), xmlDocument.getRootElementName()));
+        listener.setRootMarshalRecord(writerRecord);
+        
+        try{
+            anXMLMarshaller.marshal(xmlDocument, writerRecord);
+        }catch(XMLMarshalException xme){
+            if(xme.getErrorCode() == XMLMarshalException.DESCRIPTOR_NOT_FOUND_IN_PROJECT){
+    		    if(aHelperContext != ((SDOType)xmlDocument.getRootObject().getType()).getHelperContext()){
+    			     throw SDOException.dataObjectNotFromHelperContext();
+    		   }    	
+            }
+    	} finally{
+    	    listener.setMarshalledObject(null);
+    	    listener.setMarshalledObjectRootQName(null);
+    	    listener.setRootMarshalRecord(null);
+    	}
         outputWriter.flush();
     }
 
@@ -466,20 +497,23 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
             anXMLMarshaller.setEncoding(xmlDocument.getEncoding());
             anXMLMarshaller.setSchemaLocation(xmlDocument.getSchemaLocation());
             anXMLMarshaller.setNoNamespaceSchemaLocation(xmlDocument.getNoNamespaceSchemaLocation());
-            ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setMarshalledObject(xmlDocument.getRootObject());
-            ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setMarshalledObjectRootQName(new QName(xmlDocument.getRootElementURI(), xmlDocument.getRootElementName()));
+            
+            SDOMarshalListener listener = ((SDOMarshalListener)anXMLMarshaller.getMarshalListener());
+            
+            listener.setMarshalledObject(xmlDocument.getRootObject());
+            listener.setMarshalledObjectRootQName(new QName(xmlDocument.getRootElementURI(), xmlDocument.getRootElementName()));
             
             if(result instanceof SAXResult) {
                 ContentHandlerRecord marshalRecord = new ContentHandlerRecord();
                 marshalRecord.setContentHandler(((SAXResult)result).getHandler());
                 marshalRecord.setMarshaller(anXMLMarshaller);
-                ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setRootMarshalRecord(marshalRecord);
+                listener.setRootMarshalRecord(marshalRecord);
                 anXMLMarshaller.marshal(xmlDocument, marshalRecord);
             } else if(result instanceof DOMResult) {
                 NodeRecord marshalRecord = new NodeRecord();
                 marshalRecord.setDOM(((DOMResult)result).getNode());
                 marshalRecord.setMarshaller(anXMLMarshaller);
-                ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setRootMarshalRecord(marshalRecord);
+                listener.setRootMarshalRecord(marshalRecord);
                 anXMLMarshaller.marshal(xmlDocument, marshalRecord);
             } else {
                 StringWriter writer = new StringWriter();
@@ -488,6 +522,9 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
                 StreamSource source = new StreamSource(new java.io.StringReader(xml));
                 anXMLMarshaller.getTransformer().transform(source, result);
             }
+            listener.setMarshalledObject(null);
+            listener.setMarshalledObjectRootQName(null);
+            listener.setRootMarshalRecord(null);
         }
 
     }
@@ -547,10 +584,27 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
         writerRecord.setWriter(writer);
         writerRecord.setMarshaller(anXMLMarshaller);
         
-        ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setMarshalledObject(rootObject);
-        ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setMarshalledObjectRootQName(new QName(rootElementURI, rootElementName));
-        ((SDOMarshalListener)anXMLMarshaller.getMarshalListener()).setRootMarshalRecord(writerRecord);
-        anXMLMarshaller.marshal(xmlDocument, writerRecord);
+        SDOMarshalListener listener = ((SDOMarshalListener)anXMLMarshaller.getMarshalListener());
+        
+        listener.setMarshalledObject(rootObject);
+        listener.setMarshalledObjectRootQName(new QName(rootElementURI, rootElementName));
+        listener.setRootMarshalRecord(writerRecord);
+  
+        try{
+            anXMLMarshaller.marshal(xmlDocument, writerRecord);
+        }catch(XMLMarshalException xme){
+            if(xme.getErrorCode() == XMLMarshalException.DESCRIPTOR_NOT_FOUND_IN_PROJECT){
+    		    if(aHelperContext != ((SDOType)rootObject.getType()).getHelperContext()){
+    			     throw SDOException.dataObjectNotFromHelperContext();
+    		   }    	
+            }
+    	}finally{
+    	    listener.setMarshalledObject(null);
+    	    listener.setMarshalledObjectRootQName(null);
+    	    listener.setRootMarshalRecord(null);
+    	}
+    	
+        
         try {
             writer.flush();
         } catch(IOException ex) {
@@ -583,11 +637,13 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
     }
    
     public void initializeDescriptor(XMLDescriptor descriptor){
-        AbstractSession theSession = (AbstractSession)getXmlContext().getSession(0);
+        AbstractSession theSession = (AbstractSession)getXmlContext().getSession();
+
         //do initialization for new descriptor;        
         descriptor.preInitialize(theSession);
         descriptor.initialize(theSession);
         descriptor.postInitialize(theSession);
+                
         descriptor.getObjectBuilder().initializePrimaryKey(theSession);
         getXmlContext().storeXMLDescriptorByQName(descriptor);                           
     }
@@ -753,6 +809,7 @@ public class SDOXMLHelperDelegate implements SDOXMLHelper {
     }
 
     public XMLConversionManager getXmlConversionManager() {
-        return (XMLConversionManager)getXmlContext().getSession(0).getDatasourceLogin().getDatasourcePlatform().getConversionManager();
+        return (XMLConversionManager) getXmlContext().getSession().getDatasourceLogin().getDatasourcePlatform().getConversionManager();
     }
-}
+
+}
\ No newline at end of file
