@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010 Oracle. All rights reserved.
+ * Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -10,11 +10,14 @@
  * Contributors:
  *     ailitchev - Bug 244124 - Added to handle copying using AttributeGroup
  *                              Functionality copied from ObjectCopyPolicy 
+ *     09 Jan 2013-2.5 Gordon Yorke
+ *       - 397772: JPA 2.1 Entity Graph Support
  ******************************************************************************/  
 package org.eclipse.persistence.sessions;
 
 import java.util.*;
 
+import org.eclipse.persistence.core.queries.CoreAttributeGroup;
 import org.eclipse.persistence.internal.localization.*;
 import org.eclipse.persistence.queries.AttributeGroup;
 
@@ -33,7 +36,7 @@ import org.eclipse.persistence.queries.AttributeGroup;
 public class CopyGroup extends AttributeGroup {
     protected boolean shouldResetPrimaryKey;
     protected boolean shouldResetVersion;
-    protected org.eclipse.persistence.internal.sessions.AbstractSession session;
+    protected transient org.eclipse.persistence.internal.sessions.AbstractSession session;
     protected Map copies;
 
     /** Policy depth that determines how the copy will cascade to the object's
@@ -236,12 +239,12 @@ public class CopyGroup extends AttributeGroup {
     }
 
     @Override
-    public void addAttribute(String attributeNameOrPath, AttributeGroup group) {
+    public void addAttribute(String attributeNameOrPath, CoreAttributeGroup group) {
         cascadeTree();
         if(group == null) {
-            super.addAttribute(attributeNameOrPath, null);
+            super.addAttribute(attributeNameOrPath, (AttributeGroup)null);
         } else {
-            addAttribute(attributeNameOrPath, group.toCopyGroup());
+            addAttribute(attributeNameOrPath, ((AttributeGroup)group).toCopyGroup());
         }
     }
 
@@ -275,7 +278,7 @@ public class CopyGroup extends AttributeGroup {
     }
 
     @Override
-    protected CopyGroup newGroup(String name, AttributeGroup parent) {
+    protected CopyGroup newGroup(String name, CoreAttributeGroup parent) {
         CopyGroup copyGroup = new CopyGroup(name);
         copyGroup.cascadeTree();
         if(parent != null) {
