@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -14,12 +14,15 @@ package org.eclipse.persistence.sessions;
 
 import java.util.*;
 import java.io.*;
+import org.eclipse.persistence.config.ReferenceMode;
+import org.eclipse.persistence.core.sessions.CoreSession;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
+import org.eclipse.persistence.descriptors.partitioning.PartitioningPolicy;
 import org.eclipse.persistence.expressions.*;
 import org.eclipse.persistence.platform.server.ServerPlatform;
 import org.eclipse.persistence.platform.database.DatabasePlatform;
 import org.eclipse.persistence.queries.*;
-import org.eclipse.persistence.sessions.factories.ReferenceMode;
+import org.eclipse.persistence.sessions.serializers.Serializer;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.internal.databaseaccess.Platform;
 import org.eclipse.persistence.internal.sessions.UnitOfWorkImpl;
@@ -46,7 +49,7 @@ import org.eclipse.persistence.logging.SessionLogEntry;
  * @see org.eclipse.persistence.sessions.server.ServerSession
  * @see org.eclipse.persistence.sessions.server.ClientSession
  */
-public interface Session {
+public interface Session extends CoreSession<ClassDescriptor, Login, Platform, Project, SessionEventManager> {
 
     /**
      * ADVANCED:
@@ -433,12 +436,14 @@ public interface Session {
      * by one of the classes stored in the map, that descriptor will be stored under the
      * new class.
      */
+    @Override
     public ClassDescriptor getDescriptor(Class theClass);
 
     /**
      * ADVANCED:
      * Return the descriptor specified for the object's class.
      */
+    @Override
     public ClassDescriptor getDescriptor(Object domainObject);
 
     /**
@@ -990,4 +995,29 @@ public interface Session {
      * and that do not have a default timeout defined in their descriptor.
      */
     public void setQueryTimeoutDefault(int queryTimeoutDefault);
+    
+    /**
+     * PUBLIC:
+     * Return the session's partitioning policy.
+     */
+    public PartitioningPolicy getPartitioningPolicy();
+    
+    /**
+     * PUBLIC:
+     * Set the session's partitioning policy.
+     * A PartitioningPolicy is used to partition, load-balance or replicate data across multiple difference databases
+     * or across a database cluster such as Oracle RAC.
+     * Partitioning can provide improved scalability by allowing multiple database machines to service requests.
+     */
+    public void setPartitioningPolicy(PartitioningPolicy partitioningPolicy);
+    
+    /**
+     * Return the Serializer to use by default for serialization.
+     */
+    public Serializer getSerializer();
+
+    /**
+     * Set the Serializer to use by default for serialization.
+     */
+    public void setSerializer(Serializer serializer);
 }
