@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -17,9 +17,12 @@ import java.util.*;
 import java.io.Writer;
 import org.eclipse.persistence.queries.*;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
+import org.eclipse.persistence.descriptors.partitioning.PartitioningPolicy;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.history.*;
 import org.eclipse.persistence.logging.SessionLog;
+import org.eclipse.persistence.sequencing.Sequence;
+import org.eclipse.persistence.sessions.ExternalTransactionController;
 import org.eclipse.persistence.sessions.SessionProfiler;
 import org.eclipse.persistence.internal.databaseaccess.*;
 import org.eclipse.persistence.internal.identitymaps.*;
@@ -45,6 +48,7 @@ public class SessionBroker extends DatabaseSessionImpl {
     protected Map<Class, String> sessionNamesByClass;
     protected Map<String, AbstractSession> sessionsByName;
     protected Sequencing sequencing;
+    protected boolean shouldUseDescriptorAliases;
 
     /**
      * PUBLIC:
@@ -76,10 +80,23 @@ public class SessionBroker extends DatabaseSessionImpl {
      * acquire session broker is done under the covers on each session inside
      * the session broker, and a new broker is returned.
      *
-     * NOTE: when finished with the client broker, it should be releases.  See
-     * releaseClientSessionBroker.
+     * NOTE: when finished with the client broker, it should be released.
      */
     public SessionBroker acquireClientSessionBroker() {
+        return acquireClientSessionBroker(null, null);
+    }
+    
+    /**
+     * PUBLIC:
+     * Return a session broker that behaves as a client session broker.  An
+     * acquire session broker is done under the covers on each session inside
+     * the session broker, and a new broker is returned.
+     *
+     * NOTE: when finished with the client broker, it should be released.
+     * @param connectionPolicies maps session name to connectionPolicy to be used for this session;
+     * @param mapOfProperties maps session name to properties to be used for this session. 
+     */
+    public SessionBroker acquireClientSessionBroker(Map<String, ConnectionPolicy> connectionPolicies, Map mapOfProperties) {
         log(SessionLog.FINER, SessionLog.CONNECTION, "acquire_client_session_broker");
         SessionBroker clientBroker = copySessionBroker();
         clientBroker.parent = this;
@@ -89,16 +106,28 @@ public class SessionBroker extends DatabaseSessionImpl {
         clientBroker.externalTransactionController = getExternalTransactionController();
         clientBroker.setServerPlatform(getServerPlatform());
         String sessionName;
-        AbstractSession serverSession;
+        AbstractSession session;
         Iterator names = this.getSessionsByName().keySet().iterator();
         while (names.hasNext()) {
             sessionName = (String)names.next();
-            serverSession = getSessionForName(sessionName);
-            if (serverSession instanceof org.eclipse.persistence.sessions.server.ServerSession) {
-                if (serverSession.getProject().hasIsolatedClasses()) {
+            session = getSessionForName(sessionName);
+            if (session.isServerSession()) {
+                ServerSession serverSession = (ServerSession)session;
+                if (serverSession.getProject().hasIsolatedCacheClassWithoutUOWIsolation()) {
                     throw ValidationException.isolatedDataNotSupportedInSessionBroker(sessionName);
                 }
-                clientBroker.internalRegisterSession(sessionName, ((org.eclipse.persistence.sessions.server.ServerSession)serverSession).acquireClientSession());
+                ConnectionPolicy connectionPolicy = null;
+                if (connectionPolicies != null) {
+                    connectionPolicy = connectionPolicies.get(sessionName);
+                }
+                if (connectionPolicy == null) {
+                    connectionPolicy = serverSession.getDefaultConnectionPolicy();
+                }
+                Map properties = null;
+                if (mapOfProperties != null) {
+                    properties = (Map)mapOfProperties.get(sessionName);
+                }
+                clientBroker.registerSession(sessionName, serverSession.acquireClientSession(connectionPolicy, properties));
             } else {
                 throw ValidationException.cannotAcquireClientSessionFromSession();
             }
@@ -119,13 +148,14 @@ public class SessionBroker extends DatabaseSessionImpl {
 
         // ? logMessage("acquire_client_session_broker", (Object[])null);
         SessionBroker historicalBroker = copySessionBroker();
+        historicalBroker.parent = this;
         String sessionName;
         AbstractSession session;
         Iterator names = this.getSessionsByName().keySet().iterator();
         while (names.hasNext()) {
             sessionName = (String)names.next();
             session = getSessionForName(sessionName);
-            historicalBroker.registerSession(sessionName, session.acquireHistoricalSession(clause));
+            historicalBroker.registerSession(sessionName, (AbstractSession)session.acquireHistoricalSession(clause));
         }
         return historicalBroker;
     }
@@ -137,10 +167,20 @@ public class SessionBroker extends DatabaseSessionImpl {
      * if applicable releases accessor.
      */
     public void releaseJTSConnection() {
+        RuntimeException exception = null;
         for (Iterator sessionEnum = getSessionsByName().values().iterator();
                  sessionEnum.hasNext();) {
             AbstractSession session = (AbstractSession)sessionEnum.next();
-            session.releaseJTSConnection();
+            try {
+                session.releaseJTSConnection();
+            } catch (RuntimeException ex) {
+                if (exception == null) {
+                    exception = ex;
+                }
+            }
+        }
+        if (exception != null) {
+            throw exception;
         }
     }
 
@@ -183,6 +223,14 @@ public class SessionBroker extends DatabaseSessionImpl {
         throw ValidationException.cannotAddDescriptorsToSessionBroker();
     }
 
+    /**
+     * PUBLIC:
+     * You cannot add a sequence to a session broker, you must add it to its session.
+     */
+    public void addSequence(Sequence sequence) {
+        throw ValidationException.cannotAddSequencesToSessionBroker();
+    }
+
     /**
      * INTERNAL:
      * Begin the transaction on all child sessions.
@@ -236,7 +284,7 @@ public class SessionBroker extends DatabaseSessionImpl {
             throw globalException;
         }
     }
-
+    
     /**
      * PUBLIC:
      * Return true if the pre-defined query is defined on the session.
@@ -270,13 +318,18 @@ public class SessionBroker extends DatabaseSessionImpl {
     protected SessionBroker copySessionBroker() {
         SessionBroker broker = new SessionBroker(getSessionNamesByClass());
 
-        broker.accessor = getAccessor();
+        broker.accessors = getAccessors();
         broker.name = getName();
+        broker.isLoggingOff = isLoggingOff();
         broker.sessionLog = getSessionLog();
+        broker.profiler = getProfiler();
+        broker.isInProfile = isInProfile();
         broker.project = project;
         if (hasEventManager()) {
             broker.eventManager = getEventManager().clone(broker);
         }
+        broker.exceptionHandler = getExceptionHandler();
+        broker.descriptors = getDescriptors();
         broker.shouldPropagateChanges = shouldPropagateChanges;
 	
         return broker;
@@ -284,31 +337,21 @@ public class SessionBroker extends DatabaseSessionImpl {
 
     /**
      * INTERNAL:
-     * Return the lowlevel database accessor.
+     * Return the low-level database accessors.
      * The database accessor is used for direct database access.
-     * The right accessor for this
-     * broker will be returned.
+     * The right accessor for this broker will be returned.
      */
-    public Accessor getAccessor(Class domainClass) {
-        if (domainClass == null) {
+    public Collection<Accessor> getAccessors(Call call, AbstractRecord translationRow, DatabaseQuery query) {
+        if (query.getSessionName() != null) {
+            return getSessionForName(query.getSessionName()).getAccessors(call, translationRow, query);
+        }
+        if (query.getReferenceClass() == null) {
             // CR#... this occurs if a data query is used without a session-name, needs to throw an error
             throw QueryException.unnamedQueryOnSessionBroker(null);
         }
 
-        //session may be a broker too.  Eventually a non session broker wil be found
-        return getSessionForClass(domainClass).getAccessor(domainClass);
-    }
-
-    /**
-     * INTERNAL:
-     * Return the lowlevel database accessor.
-     * The database accesor is used for direct database access.
-     * The right accessor for this
-     * broker will be returned.
-     */
-    public Accessor getAccessor(String sessionName) {
-        //session may be a broker too.  Eventually a non session broker wil be found
-        return getSessionForName(sessionName).getAccessor(sessionName);
+        //session may be a broker too.  Eventually a non session broker will be found
+        return getSessionForClass(query.getReferenceClass()).getAccessors(call, translationRow, query);
     }
 
     /**
@@ -327,8 +370,8 @@ public class SessionBroker extends DatabaseSessionImpl {
             return ((AbstractSession)enumtr.next()).getAsOfClause();
         }
         return null;
-    }
-
+    }    
+    
     /**
      * INTERNAL:
      * Gets the parent SessionBroker.
@@ -337,29 +380,6 @@ public class SessionBroker extends DatabaseSessionImpl {
         return parent;
     }
 
-    /**
-     * INTERNAL:
-     * Gets the next link in the chain of sessions followed by a query's check
-     * early return, the chain of sessions with identity maps all the way up to
-     * the root session.
-     * <p>
-     * Used for session broker which delegates to registered sessions, or UnitOfWork
-     * which checks parent identity map also.
-     * @param canReturnSelf true when method calls itself.  If the path
-     * starting at <code>this</code> is acceptable.  Sometimes true if want to
-     * move to the first valid session, i.e. executing on ClientSession when really
-     * should be on ServerSession.
-     * @param terminalOnly return the session we will execute the call on, not
-     * the next step towards it.
-     * @return this if there is no next link in the chain
-     */
-    public AbstractSession getParentIdentityMapSession(DatabaseQuery query, boolean canReturnSelf, boolean terminalOnly) {
-        if (query == null) {
-            return this;
-        }
-        return getSessionForQuery(query).getParentIdentityMapSession(query, canReturnSelf, terminalOnly);
-    }
-
     /**
      * INTERNAL:
      * Gets the session which this query will be executed on.
@@ -394,62 +414,47 @@ public class SessionBroker extends DatabaseSessionImpl {
         return getSessionForClass(domainClass).getDatasourcePlatform();
     }
 
-    /**
-     * PUBLIC:
-     * Return the query from the session pre-defined queries with the given name.
-     * This allows for common queries to be pre-defined, reused and executed by name.
-     */
-    public DatabaseQuery getQuery(String name) {
-        //Bug#3473441 Should always call super first to ensure any change in Session will be reflected here.
-        DatabaseQuery query = super.getQuery(name);
-        if (isClientSessionBroker() && (query == null)) {
-            String sessionName = null;
-            AbstractSession ssession = null;
-            Iterator names = this.getSessionsByName().keySet().iterator();
-            while (names.hasNext()) {
-                sessionName = (String)names.next();
-                ssession = getSessionForName(sessionName);
-                if (ssession instanceof org.eclipse.persistence.sessions.server.ClientSession) {
-                    query = ((ClientSession)ssession).getParent().getBroker().getQuery(name);
-                    if (query != null) {
-                        return query;
-                    }
-                }
-            }
-        }
-
-        return query;
-    }
-
     /**
      * PUBLIC:
      * Return the query from the session pre-defined queries with the given name and argument types.
      * This allows for common queries to be pre-defined, reused and executed by name.
      * This method should be used if the Session has multiple queries with the same name but
      * different arguments.
+     * 
+     * The search order is: 
+     *    for ClientSessionBroker:
+     *      the broker; 
+     *      it's member ClientSessions (but not their parent ServerSessions);
+     *      the parent SessionBroker.
+     *      
+     *    for ServerSession or DatabaseSession SessionBroker:
+     *      the broker;
+     *      it's member ServerSessions (or DatabaseSessions). 
      */
-
     //Bug#3551263  Override getQuery(String name, Vector arguments) in Session search through 
     //the server session broker as well
-    public DatabaseQuery getQuery(String name, Vector arguments) {
-        DatabaseQuery query = super.getQuery(name, arguments);
-        if (isClientSessionBroker() && (query == null)) {
-            String sessionName = null;
-            AbstractSession ssession = null;
-            Iterator names = this.getSessionsByName().keySet().iterator();
-            while (names.hasNext()) {
-                sessionName = (String)names.next();
-                ssession = getSessionForName(sessionName);
-                if (ssession instanceof org.eclipse.persistence.sessions.server.ClientSession) {
-                    query = ((ClientSession)ssession).getParent().getBroker().getQuery(name, arguments);
-                    if (query != null) {
-                        return query;
-                    }
-                }
+    public DatabaseQuery getQuery(String name, Vector arguments, boolean shouldSearchParent) {
+        // First search the broker
+        DatabaseQuery query = super.getQuery(name, arguments, shouldSearchParent);
+        if(query != null) {
+            return query;
+        }
+        Iterator<AbstractSession> it = this.sessionsByName.values().iterator();
+        while(it.hasNext()) {
+            // Note that ClientSession's parent ServerSessions should not be searched at this time
+            query = it.next().getQuery(name, arguments, false);
+            if(query != null) {
+                return query;
             }
         }
-
-        return query;
+        if(shouldSearchParent) {
+            AbstractSession parent = getParent();
+            if(parent != null) {
+                // Search SessionBroker and it's member ServerSessions
+                return parent.getQuery(name, arguments, true);
+            }
+        }
+        return null;
     }
 
     /**
@@ -523,6 +528,7 @@ public class SessionBroker extends DatabaseSessionImpl {
      * Allow each descriptor to initialize any dependencies on this session.
      * This is done in two passes to allow the inheritance to be resolved first.
      * Normally the descriptors are added before login, then initialized on login.
+     * Should not be called on client SessoionBroker
      */
     public void initializeDescriptors() {
         // ClientSession initializes sequencing during construction,
@@ -530,16 +536,40 @@ public class SessionBroker extends DatabaseSessionImpl {
         // in initializeDescriptors method. 
         // Because initializeDescriptors() is not called for a session-member of a SessionBroker,
         // initializeSequencing() for the sessions should be called here.
-        if (!isClientSessionBroker()) {
-            for (Iterator enumtr = getSessionsByName().values().iterator(); enumtr.hasNext();) {
-                DatabaseSessionImpl databaseSession = (DatabaseSessionImpl)enumtr.next();
-                databaseSession.initializeSequencing();
+        for (Iterator enumtr = getSessionsByName().values().iterator(); enumtr.hasNext();) {
+            DatabaseSessionImpl databaseSession = (DatabaseSessionImpl)enumtr.next();                
+            String sessionName = databaseSession.getName();
+
+            // Initialize partitioning policies.
+            for (PartitioningPolicy policy : databaseSession.getProject().getPartitioningPolicies().values()) {
+                policy.initialize(this);
             }
-            if(hasExternalTransactionController()) {
-                getExternalTransactionController().initializeSequencingListeners();
+            
+            // Copy jpa queries from member sessions into SessionBroker before descriptors' initialization.
+            if (!databaseSession.isJPAQueriesProcessed()) {
+                for(DatabaseQuery query: databaseSession.getJPAQueries()) {
+                    query.setSessionName(sessionName);
+                    getJPAQueries().add(query);
+                }
+                //prevent them from being re-added in the case of a logout/login
+                databaseSession.setJPAQueriesProcessed(true);
             }
 
+            // assign session name to each query
+            Iterator<List<DatabaseQuery>> it = databaseSession.getQueries().values().iterator();
+            while(it.hasNext()) {
+                List<DatabaseQuery> queryList = it.next();
+                for(int i=0; i < queryList.size(); i++) {
+                    queryList.get(i).setSessionName(sessionName);
+                }
+            }
+            
+            databaseSession.initializeSequencing();
         }
+        if(hasExternalTransactionController()) {
+            getExternalTransactionController().initializeSequencingListeners();
+        }
+
         super.initializeDescriptors();
         // Must reset project options to session broker project, as initialization occurs
         // with local projects.
@@ -551,9 +581,15 @@ public class SessionBroker extends DatabaseSessionImpl {
             if (databaseSession.getProject().hasIsolatedClasses()) {
                 getProject().setHasIsolatedClasses(true);
             }
+            if (databaseSession.getProject().hasMappingsPostCalculateChangesOnDeleted()) {
+                getProject().setHasMappingsPostCalculateChangesOnDeleted(true);
+            }
             if (databaseSession.getProject().hasNonIsolatedUOWClasses()) {
                 getProject().setHasNonIsolatedUOWClasses(true);
             }
+            if (databaseSession.getProject().hasProxyIndirection()) {
+                getProject().setHasProxyIndirection(true);
+            }
             getProject().getDefaultReadOnlyClasses().addAll(databaseSession.getProject().getDefaultReadOnlyClasses());
         }
         
@@ -662,39 +698,43 @@ public class SessionBroker extends DatabaseSessionImpl {
      * This connects all of the child sessions and expects that they are in a valid state to be connected.
      */
     public void login() throws DatabaseException {
-        //Bug#3440544 Check if logged in already to stop the attempt to login more than once
-        if (isLoggedIn) {
-            throw ValidationException.alreadyLoggedIn(this.getName());
-        } else {
-            if (this.eventManager != null) {
-                this.eventManager.preLogin(this);
+        preConnectDatasource();
+        // Connection all sessions and initialize
+        for (Iterator sessionEnum = getSessionsByName().values().iterator();
+                 sessionEnum.hasNext();) {
+            DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
+            if (!session.isConnected()) {
+                session.login();
             }
-            // Bug 3848021 - ensure the external transaction controller is initialized
-            if (!isConnected()) {
-                getServerPlatform().initializeExternalTransactionController();
-            }
-
-            // Connection all sessions and initialize
-            for (Iterator sessionEnum = getSessionsByName().values().iterator();
-                     sessionEnum.hasNext();) {
-                DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
-                if (session.hasEventManager()) {
-                    session.getEventManager().preLogin(session);
-                }
-                if (!session.isConnected()) {
-                    session.connect();
-                }
-            }
-            initializeDescriptors();
-            if (getCommandManager() != null) {
-                getCommandManager().initialize();
-            }
-            isLoggedIn = true;
-            if (this.eventManager != null) {
-                this.eventManager.postLogin(this);
-            }
-
         }
+        postConnectDatasource();
+    }
+
+    /**
+     * PUBLIC:
+     * Connect to the database using the predefined login.
+     * During connection, attempt to auto detect the required database platform.
+     * This method can be used in systems where for ease of use developers have
+     * EclipseLink autodetect the platform.
+     * To be safe, however, the platform should be configured directly.
+     * The login must have been assigned when or after creating the session.
+     *
+     */
+    public void loginAndDetectDatasource() throws DatabaseException {
+        preConnectDatasource();
+        // Connection all sessions and initialize
+        for (Iterator sessionEnum = getSessionsByName().values().iterator();
+                 sessionEnum.hasNext();) {
+            DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
+            if (!session.isConnected()) {
+                if(session.getDatasourcePlatform().getClass().getName().equals("org.eclipse.persistence.platform.database.DatabasePlatform")) {
+                    session.loginAndDetectDatasource();
+                } else {
+                    session.login();
+                }
+            }
+        }
+        postConnectDatasource();
     }
 
     /**
@@ -703,35 +743,18 @@ public class SessionBroker extends DatabaseSessionImpl {
      * This connects all of the child sessions and expects that they are in a valid state to be connected.
      */
     public void login(String userName, String password) throws DatabaseException {
-        //Bug#3440544 Check if logged in already to stop the attempt to login more than once
-        if (isLoggedIn) {
-            throw ValidationException.alreadyLoggedIn(this.getName());
-        } else {
-            if (this.eventManager != null) {
-                this.eventManager.preLogin(this);
+        preConnectDatasource();
+        // Connection all sessions and initialize
+        for (Iterator sessionEnum = getSessionsByName().values().iterator();
+                 sessionEnum.hasNext();) {
+            DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
+            session.getDatasourceLogin().setUserName(userName);
+            session.getDatasourceLogin().setPassword(password);
+            if (!session.isConnected()) {
+                session.login();
             }
-            // Bug 3848021 - ensure the external transaction controller is initialized
-            if (!isConnected()) {
-                getServerPlatform().initializeExternalTransactionController();
-            }
-
-            // Connection all sessions and initialize
-            for (Iterator sessionEnum = getSessionsByName().values().iterator();
-                     sessionEnum.hasNext();) {
-                DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
-                if (session.hasEventManager()) {
-                    session.getEventManager().preLogin(session);
-                }
-                session.getDatasourceLogin().setUserName(userName);
-                session.getDatasourceLogin().setPassword(password);
-
-                if (!session.isConnected()) {
-                    session.connect();
-                }
-            }
-            initializeDescriptors();
-            this.isLoggedIn = true;
         }
+        postConnectDatasource();
     }
 
     /**
@@ -743,40 +766,80 @@ public class SessionBroker extends DatabaseSessionImpl {
      * or a general error occurs.
      */
     public void logout() throws DatabaseException {
-        for (Iterator sessionEnum = getSessionsByName().values().iterator();
-                 sessionEnum.hasNext();) {
-            DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
-            session.logout();
+        if(!isLoggedIn) {
+            return;
+        }
+        if (this.eventManager != null) {
+            this.eventManager.preLogout(this);
         }
         if (!isClientSessionBroker()) {
-            if(hasExternalTransactionController()) {
-                getExternalTransactionController().clearSequencingListeners();
+            for (Iterator sessionEnum = getSessionsByName().values().iterator();
+                     sessionEnum.hasNext();) {
+                DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
+                session.logout();
             }
         }
-        sequencing = null;
-        isLoggedIn = false;
+        this.sequencing = null;
+        this.isLoggedIn = false;
+        if (this.eventManager != null) {
+            this.eventManager.postLogout(this);
+        }
     }
 
+    /**
+     * INTERNAL:
+     * Rise postLogin events for member sessions and the SessionBroker.
+     */
+    public void postLogin() {
+        for (Iterator sessionEnum = getSessionsByName().values().iterator();
+            sessionEnum.hasNext();) {
+           DatabaseSessionImpl session = (DatabaseSessionImpl)sessionEnum.next();
+           session.postLogin();
+        }
+        super.postLogin();
+    }
+    
     /**
      * PUBLIC:
      * Register the session under its name.
      * All of the session's descriptors must have already been registered.
-     * The session should not be connected and descriptors should not be initialized.
+     * DatabaseSession/ServerSession should not be connected and descriptors should not be initialized.
      */
     public void registerSession(String name, AbstractSession session) {
         session.setIsInBroker(true);
         getSessionsByName().put(name, session);
         session.setBroker(this);
         session.setName(name);
-
-        // The keys/classes must also be used as some descriptors may be under multiple classes.
-        Iterator descriptors = session.getDescriptors().values().iterator();
-        Iterator classes = session.getDescriptors().keySet().iterator();
-        while (descriptors.hasNext()) {
-            ClassDescriptor descriptor = (ClassDescriptor)descriptors.next();
-            Class descriptorClass = (Class)classes.next();
-            getSessionNamesByClass().put(descriptorClass, name);
-            getDescriptors().put(descriptorClass, descriptor);
+        if (session.isDatabaseSession()) {
+            if (hasEventManager()) {
+                // add broker's listeners to member sessions.
+                // each time a listener added to broker it is added to member sessions, too
+                // (see SessionEventManager#addListener) therefore at all times
+                // each member session has all broker's listeners.
+                session.getEventManager().getListeners().addAll(getEventManager().getListeners());
+            }
+    
+            // The keys/classes must also be used as some descriptors may be under multiple classes.
+            Iterator descriptors = session.getDescriptors().values().iterator();
+            Iterator classes = session.getDescriptors().keySet().iterator();
+            while (descriptors.hasNext()) {
+                ClassDescriptor descriptor = (ClassDescriptor)descriptors.next();
+                Class descriptorClass = (Class)classes.next();
+                getSessionNamesByClass().put(descriptorClass, name);
+                if(this.shouldUseDescriptorAliases) {
+                    String alias = descriptor.getAlias();
+                    if(alias != null && alias.length() > 0) {
+                        ClassDescriptor anotherDescriptor = getDescriptorForAlias(alias);
+                        if(anotherDescriptor != null) {
+                            if(anotherDescriptor.getJavaClass() != descriptor.getJavaClass()) {
+                                throw ValidationException.sharedDescriptorAlias(alias, descriptor.getJavaClass().getName(), anotherDescriptor.getJavaClass().getName());
+                            }
+                        }
+                        addAlias(alias, descriptor);
+                    }
+                }
+                getDescriptors().put(descriptorClass, descriptor);
+            }
         }
     }
 
@@ -784,37 +847,12 @@ public class SessionBroker extends DatabaseSessionImpl {
      * PUBLIC:
      * Register the session under its name.
      * All of the session's descriptors must have already been registered.
-     * The session should not be connected and descriptors should not be initialized.
+     * DatabaseSession/ServerSession should not be connected and descriptors should not be initialized.
      */
     public void registerSession(String name, org.eclipse.persistence.sessions.Session session) {
         registerSession(name, (AbstractSession)session);
     }
 
-    /**
-     * INTERNAL:
-     * Register the session under its name.
-     * All of the session's descriptors must have already been registered.
-     * The session should not be connected and descriptors should not be initialized.
-     * Used for client session broker
-     */
- 
-    public void internalRegisterSession(String name, AbstractSession session) {
-        //Bug#3911318  Removed session brokers and sessions point to the same descriptors.  
-        //They don't need to be reassigned.  Also the code to add read only classes has been removed
-        //because client broker and server broker point to the same project.
-        session.setIsInBroker(true);
-        getSessionsByName().put(name, session);
-        session.setBroker(this);
-        session.setName(name);
-
- /*       // The keys/classes must also be used as some descriptors may be under multiple classes.
-        Iterator classes = session.getDescriptors().keySet().iterator();
-        while (classes.hasNext()) {
-            Class descriptorClass = (Class)classes.next();
-            getSessionNamesByClass().put(descriptorClass, session);
-        }
-  */  }
-
     /**
      * PUBLIC:
      * Release the session.
@@ -824,12 +862,51 @@ public class SessionBroker extends DatabaseSessionImpl {
         if (isClientSessionBroker()) {
             log(SessionLog.FINER, SessionLog.CONNECTION, "releasing_client_session_broker");
         }
+        RuntimeException exception = null;
         AbstractSession session;
         for (Iterator enumtr = getSessionsByName().values().iterator(); enumtr.hasNext();) {
             session = (AbstractSession)enumtr.next();
-            session.release();
+            try {
+                session.release();
+            } catch (RuntimeException ex) {
+                if (exception == null) {
+                    exception = ex;
+                }
+            }
         }
         super.release();
+        if (exception != null) {
+            throw exception;
+        }
+    }
+
+    /**
+     * INTERNAL:
+     * A query execution failed due to an invalid query.
+     * Re-connect and retry the query.
+     */
+    @Override
+    public Object retryQuery(DatabaseQuery query, AbstractRecord row, DatabaseException databaseException, int retryCount, AbstractSession executionSession) {
+        // If not in a transaction and has a write connection, must release it if invalid.
+        if (isClientSessionBroker()) {
+            for (Iterator it = getSessionsByName().values().iterator(); it.hasNext();) {
+                ClientSession clientSession = (ClientSession)it.next();
+                clientSession.getParent().releaseInvalidClientSession(clientSession);
+            }
+        }
+        return super.retryQuery(query, row, databaseException, retryCount, executionSession);
+    }
+
+    /**
+     * INTERNAL:
+     * Used for JTS integration internally by ServerPlatform.
+     */
+    public void setExternalTransactionController(ExternalTransactionController externalTransactionController) {
+        super.setExternalTransactionController(externalTransactionController);
+        for (AbstractSession session : getSessionsByName().values()) {
+            DatabaseSessionImpl dbSession = (DatabaseSessionImpl)session;
+            dbSession.setExternalTransactionController(externalTransactionController);
+        }
     }
 
     /**
@@ -1000,4 +1077,22 @@ public class SessionBroker extends DatabaseSessionImpl {
     public boolean isSequencingCallbackRequired() {
         return howManySequencingCallbacks() > 0;
     }
+    
+    /**
+     * PUBLIC:
+     * Indicates whether descriptors should use aliasDescriptors map.
+     * If aliasDescriptors is used then descriptors' aliases should be unique.
+     */
+    public boolean shouldUseDescriptorAliases() {
+        return this.shouldUseDescriptorAliases;
+    }
+
+    /**
+     * PUBLIC:
+     * Indicates whether descriptors should use aliasDescriptors map.
+     * If aliasDescriptors is used then descriptors' aliases should be unique.
+     */
+    public void setShouldUseDescriptorAliases(boolean shouldUseDescriptorAliases) {
+        this.shouldUseDescriptorAliases = shouldUseDescriptorAliases;
+    }
 }
