@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -14,8 +14,10 @@ package org.eclipse.persistence.sessions.interceptors;
 
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.Map;
 
 import org.eclipse.persistence.descriptors.ClassDescriptor;
+import org.eclipse.persistence.exceptions.QueryException;
 import org.eclipse.persistence.indirection.ValueHolderInterface;
 import org.eclipse.persistence.internal.helper.Helper;
 import org.eclipse.persistence.internal.identitymaps.CacheKey;
@@ -23,6 +25,7 @@ import org.eclipse.persistence.internal.identitymaps.IdentityMap;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.mappings.ForeignReferenceMapping;
 
+
 /**
  * <p>
  * <b>Purpose</b>: Define a class through which Cache access can be
@@ -61,16 +64,16 @@ public abstract class CacheInterceptor implements IdentityMap {
      * This first thread will get an active lock.
      * Other threads will get deferred locks, all threads will wait until all other threads are complete before releasing their locks.
      */
-    public CacheKey acquireDeferredLock(Object primaryKey){
-        return createCacheKeyInterceptor(this.targetIdentityMap.acquireDeferredLock(primaryKey));
+    public CacheKey acquireDeferredLock(Object primaryKey, boolean isCacheCheckComplete){
+        return createCacheKeyInterceptor(this.targetIdentityMap.acquireDeferredLock(primaryKey, isCacheCheckComplete));
     }
 
     /**
      * Acquire an active lock on the object.
      * This is used by reading (when using indirection or no relationships) and by merge.
      */
-    public CacheKey acquireLock(Object primaryKey, boolean forMerge){
-        return createCacheKeyInterceptor(this.targetIdentityMap.acquireLock(primaryKey, forMerge));
+    public CacheKey acquireLock(Object primaryKey, boolean forMerge, boolean isCacheCheckComplete){
+        return createCacheKeyInterceptor(this.targetIdentityMap.acquireLock(primaryKey, forMerge, isCacheCheckComplete));
     }
 
     /**
@@ -161,11 +164,33 @@ public abstract class CacheInterceptor implements IdentityMap {
         return this.targetIdentityMap.get(primaryKey);
     }
     
+    /**
+     * ADVANCED:
+     * Using a list of Entity PK this method will attempt to bulk load the entire list from the cache.
+     * In certain circumstances this can have large performance improvements over loading each item individually.
+     * @param pkList List of Entity PKs to extract from the cache
+     * @param ClassDescriptor Descriptor type to be retrieved.
+     * @return Map of Entity PKs associated to the Entities that were retrieved
+     * @throws QueryException
+     */
+    public abstract Map<Object, Object> getAllFromIdentityMapWithEntityPK(Object[] pkList, ClassDescriptor descriptor, AbstractSession session);
+
+    /**
+     * ADVANCED:
+     * Using a list of Entity PK this method will attempt to bulk load the entire list from the cache.
+     * In certain circumstances this can have large performance improvements over loading each item individually.
+     * @param pkList List of Entity PKs to extract from the cache
+     * @param ClassDescriptor Descriptor type to be retrieved.
+     * @return Map of Entity PKs associated to the Entities that were retrieved
+     * @throws QueryException
+     */
+    public abstract Map<Object, CacheKey> getAllCacheKeysFromIdentityMapWithEntityPK(Object[] pkList, ClassDescriptor descriptor, AbstractSession session);
+
     /**
      * Get the cache key (with object) for the primary key.
      */
-    public CacheKey getCacheKey(Object primaryKey) {
-        return this.targetIdentityMap.getCacheKey(primaryKey);
+    public CacheKey getCacheKey(Object primaryKey, boolean forMerge) {
+        return this.targetIdentityMap.getCacheKey(primaryKey, forMerge);
     }
 
     /**
@@ -255,8 +280,8 @@ public abstract class CacheInterceptor implements IdentityMap {
      * Notify the cache that a lazy relationship has been triggered in the object
      * and the cache may need to be updated
      */
-    public void lazyRelationshipLoaded(Object object, ValueHolderInterface valueHolder, ForeignReferenceMapping mapping){
-    	this.targetIdentityMap.lazyRelationshipLoaded(object, mapping);
+    public void lazyRelationshipLoaded(Object rootEntity, ValueHolderInterface valueHolder, ForeignReferenceMapping mapping){
+        this.targetIdentityMap.lazyRelationshipLoaded(rootEntity, valueHolder, mapping);
     }
 
     /**
