@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -73,13 +73,32 @@ public class RMIConnection extends RemoteConnection {
         }
     }
 
+    /**
+     * INTERNAL:
+     * Begin an early unit of work transaction.
+     */
+    public void beginEarlyTransaction() {
+        try {
+            Transporter transporter = getRemoteSessionController().beginEarlyTransaction();
+            if (!transporter.wasOperationSuccessful()) {
+                throw transporter.getException();
+            }
+        } catch (RemoteException exception) {
+            throw CommunicationException.errorInInvocation(exception);
+        }
+    }
+
     /**
      * INTERNAL:
      * Commit root unit of work from the client side to the server side.
      */
     public RemoteUnitOfWork commitRootUnitOfWork(RemoteUnitOfWork theRemoteUnitOfWork) {
         try {
-            Transporter transporter = getRemoteSessionController().commitRootUnitOfWork(new Transporter(theRemoteUnitOfWork));
+            Transporter transporter = new Transporter();
+            transporter.setObject(theRemoteUnitOfWork);
+            transporter.prepare(this.session);
+            transporter = getRemoteSessionController().commitRootUnitOfWork(transporter);
+            transporter.expand(this.session);
             if (!transporter.wasOperationSuccessful()) {
                 throw transporter.getException();
             } else {
@@ -131,7 +150,7 @@ public class RMIConnection extends RemoteConnection {
     /**
      * Retrieve next page size of objects from the remote cursored stream
      */
-    public Vector cursoredStreamNextPage(RemoteCursoredStream remoteCursoredStream, ReadQuery query, RemoteSession session, int pageSize) {
+    public Vector cursoredStreamNextPage(RemoteCursoredStream remoteCursoredStream, ReadQuery query, DistributedSession session, int pageSize) {
         Transporter transporter = null;
         try {
             transporter = getRemoteSessionController().cursoredStreamNextPage(new Transporter(remoteCursoredStream.getID()), pageSize);
@@ -199,7 +218,7 @@ public class RMIConnection extends RemoteConnection {
             remoteCursoredStream.setPolicy(policy);
 
             if (policy.getQuery().isReadAllQuery() && (!policy.getQuery().isReportQuery())) {// could be DataReadQuery
-                fixObjectReferences(transporter, (ObjectLevelReadQuery)policy.getQuery(), (RemoteSession)session);
+                fixObjectReferences(transporter, (ObjectLevelReadQuery)policy.getQuery(), session);
             }
             return remoteCursoredStream;
         } catch (RemoteException exception) {
@@ -228,27 +247,6 @@ public class RMIConnection extends RemoteConnection {
         }
     }
 
-    /**
-     * INTERNAL:
-     * An object has been serialized from the server to the remote client.
-     * Replace the transient attributes of the remote value holders with client-side objects.
-     * Being used for the cursored stream only
-     */
-    public void fixObjectReferences(Transporter remoteCursoredStream, ObjectLevelReadQuery query, RemoteSession session) {
-        RemoteCursoredStream stream = (RemoteCursoredStream)remoteCursoredStream.getObject();
-        List remoteObjectCollection = stream.getObjectCollection();
-        if (query.isReadAllQuery() && (!query.isReportQuery())) {// could be DataReadQuery
-            Vector clientObjectCollection = new Vector(remoteObjectCollection.size());
-
-            // find next power-of-2 size
-            Map recursiveSet = new IdentityHashMap(remoteObjectCollection.size() + 1);
-            for (Object serverSideDomainObject : remoteObjectCollection) {
-                clientObjectCollection.addElement(session.getObjectCorrespondingTo(serverSideDomainObject, remoteCursoredStream.getObjectDescriptors(), recursiveSet, query));
-            }
-            stream.setObjectCollection(clientObjectCollection);
-        }
-    }
-
     /**
      * INTERNAL
      * Return the read-only classes
@@ -283,6 +281,23 @@ public class RMIConnection extends RemoteConnection {
         }
     }
 
+    /**
+     * INTERNAL:
+     * Return the table descriptor specified for the alias.
+     */
+    public ClassDescriptor getDescriptorForAlias(String alias) {
+        try {
+            Transporter transporter = getRemoteSessionController().getDescriptorForAlias(new Transporter(alias));
+            if (!transporter.wasOperationSuccessful()) {
+                throw transporter.getException();
+            } else {
+                return (ClassDescriptor)transporter.getObject();
+            }
+        } catch (RemoteException exception) {
+            throw CommunicationException.errorInInvocation(exception);
+        }
+    }
+
     /**
      * INTERNAL:
      * Return the table descriptor specified for the class.
@@ -348,10 +363,14 @@ public class RMIConnection extends RemoteConnection {
      */
     public Transporter instantiateRemoteValueHolderOnServer(RemoteValueHolder remoteValueHolder) {
         try {
-            Transporter transporter = getRemoteSessionController().instantiateRemoteValueHolderOnServer(new Transporter(remoteValueHolder));
+            Transporter transporter = new Transporter();
+            transporter.setObject(remoteValueHolder);
+            transporter.prepare(this.session);
+            transporter = getRemoteSessionController().instantiateRemoteValueHolderOnServer(transporter);
             if (!transporter.wasOperationSuccessful()) {
                 throw transporter.getException();
             }
+            transporter.expand(this.session);
             return transporter;
         } catch (RemoteException exception) {
             throw CommunicationException.errorInInvocation(exception);
@@ -364,7 +383,11 @@ public class RMIConnection extends RemoteConnection {
      */
     public Transporter remoteExecute(DatabaseQuery query) {
         try {
-            Transporter transporter = getRemoteSessionController().executeQuery(new Transporter(query));
+            Transporter transporter = new Transporter();
+            transporter.setObject(query);
+            transporter.prepare(this.session);
+            transporter = getRemoteSessionController().executeQuery(transporter);
+            transporter.expand(this.session);
             if (!transporter.wasOperationSuccessful()) {
                 throw transporter.getException();
             }
@@ -620,7 +643,7 @@ public class RMIConnection extends RemoteConnection {
     /**
      * Retrieve next object from the remote scrollable cursor
      */
-    public Object scrollableCursorNextObject(ObjID remoteScrollableCursorOid, ReadQuery query, RemoteSession session) {
+    public Object scrollableCursorNextObject(ObjID remoteScrollableCursorOid, ReadQuery query, DistributedSession session) {
         Transporter transporter = null;
         try {
             transporter = getRemoteSessionController().scrollableCursorNextObject(new Transporter(remoteScrollableCursorOid));
@@ -651,7 +674,7 @@ public class RMIConnection extends RemoteConnection {
     /**
      * Retrieve previous object from the remote scrollable cursor
      */
-    public Object scrollableCursorPreviousObject(ObjID remoteScrollableCursorOid, ReadQuery query, RemoteSession session) {
+    public Object scrollableCursorPreviousObject(ObjID remoteScrollableCursorOid, ReadQuery query, DistributedSession session) {
         Transporter transporter = null;
         try {
             transporter = getRemoteSessionController().scrollableCursorPreviousObject(new Transporter(remoteScrollableCursorOid));
