@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -10,13 +10,20 @@
  * Contributors:
  *     Oracle - initial API and implementation from Oracle TopLink
  *     Dies Koper - add support for creating indices on tables
+ *     01/11/2013-2.5 Guy Pelletier 
+ *       - 389090: JPA 2.1 DDL Generation Support
+ *     02/04/2013-2.5 Guy Pelletier 
+ *       - 389090: JPA 2.1 DDL Generation Support
+ *     04/12/2013-2.5 Guy Pelletier 
+ *       - 405640: JPA 2.1 schema generation drop operation fails to include dropping defaulted fk constraints.
  ******************************************************************************/  
 package org.eclipse.persistence.tools.schemaframework;
 
 import java.io.Writer;
+import java.net.URL;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Iterator;
+import java.util.Set;
 import java.util.Vector;
 
 import org.eclipse.persistence.descriptors.ClassDescriptor;
@@ -28,6 +35,8 @@ import org.eclipse.persistence.internal.helper.Helper;
 import org.eclipse.persistence.internal.sequencing.Sequencing;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.sessions.DatabaseSessionImpl;
+import org.eclipse.persistence.logging.SessionLog;
+import org.eclipse.persistence.queries.DataReadQuery;
 import org.eclipse.persistence.sequencing.DefaultSequence;
 import org.eclipse.persistence.sequencing.NativeSequence;
 import org.eclipse.persistence.sequencing.Sequence;
@@ -54,6 +63,15 @@ public class SchemaManager {
     /** Allow table creator to occur "fast" by just deleting all the rows. */
     public static boolean FAST_TABLE_CREATOR = false;
     
+    /** Allow replacing of table to force the drop, this may require several passes. */
+    public static boolean FORCE_DROP = true;
+    
+    /** Flag to determine if database schemas should be created during DDL generation */
+    protected boolean createDatabaseSchemas = false;
+    protected HashSet<String> createdDatabaseSchemas = new HashSet<String>();
+    protected HashSet<String> createdDatabaseSchemasOnDatabase = new HashSet<String>();
+    protected HashMap<String, DatabaseObjectDefinition> dropDatabaseSchemas = new HashMap<String, DatabaseObjectDefinition>();
+    
     public SchemaManager(DatabaseSessionImpl session) {
         this.session = session;
     }
@@ -61,13 +79,9 @@ public class SchemaManager {
     public SchemaManager(org.eclipse.persistence.sessions.DatabaseSession session) {
         this.session = ((DatabaseSessionImpl)session);
     }
-
+    
     protected Writer getDropSchemaWriter() {
-        if (null == dropSchemaWriter) {
-            return createSchemaWriter;
-        } else {
-            return dropSchemaWriter;
-        }
+        return (dropSchemaWriter == null) ? createSchemaWriter : dropSchemaWriter;
     }
 
     /**
@@ -126,6 +140,19 @@ public class SchemaManager {
             throw ValidationException.fileError(ioException);
         }
     }
+    
+    /**
+     * INTERNAL:
+     * Called when dropping tables. Will build a map of those schemas (and a 
+     * table that references it) that should be dropped.
+     */
+    protected void collectDatabaseSchemasForDrop(DatabaseObjectDefinition databaseObjectDefinition) {
+        if (createDatabaseSchemas && databaseObjectDefinition.hasDatabaseSchema()) {
+            if (! dropDatabaseSchemas.containsKey(databaseObjectDefinition.getDatabaseSchema())) {
+                dropDatabaseSchemas.put(databaseObjectDefinition.getDatabaseSchema(), databaseObjectDefinition);
+            }
+        }
+    }
 
     /**
      * Use the table definition to add the constraints to the database, this is normally done
@@ -143,8 +170,8 @@ public class SchemaManager {
             if (shouldWriteToDatabase()) {
                 tableDefinition.createConstraintsOnDatabase(getSession());
             } else {
-            tableDefinition.setCreateSQLFiles(createSQLFiles);
-            tableDefinition.createConstraints(getSession(), createSchemaWriter);
+                tableDefinition.setCreateSQLFiles(createSQLFiles);
+                tableDefinition.createConstraints(getSession(), createSchemaWriter);
             }
         } finally {
             if (usesBatchWriting) {
@@ -185,15 +212,32 @@ public class SchemaManager {
 
         try {
             if (shouldWriteToDatabase()) {
+                // Check if we should create a database schema for this 
+                // database object definition on the database. It is only 
+                // create once and for the first database object definition
+                // that references it.
+                if (shouldCreateDatabaseSchema(databaseObjectDefinition, createdDatabaseSchemasOnDatabase)) {
+                    databaseObjectDefinition.createDatabaseSchemaOnDatabase(getSession(), createdDatabaseSchemasOnDatabase);
+                }
+                
                 databaseObjectDefinition.createOnDatabase(getSession());
             } else {
+                // Check if we should create a database schema for this 
+                // database object definition on the database. It is only 
+                // create once and for the first database object definition
+                // that references it.
+                if (shouldCreateDatabaseSchema(databaseObjectDefinition, createdDatabaseSchemas)) {
+                    databaseObjectDefinition.createDatabaseSchema(getSession(), createSchemaWriter, createdDatabaseSchemas);
+                    appendToDDLWriter(createSchemaWriter, "\n");
+                }
+                
                 databaseObjectDefinition.createObject(getSession(), createSchemaWriter);
                 if (createSQLFiles){
                     this.appendToDDLWriter(createSchemaWriter, getSession().getPlatform().getStoredProcedureTerminationToken());
                 }
                 this.appendToDDLWriter(createSchemaWriter, "\n");
-                databaseObjectDefinition.postCreateObject(getSession(), createSchemaWriter, createSQLFiles);
             }
+            databaseObjectDefinition.postCreateObject(getSession(), createSchemaWriter, createSQLFiles);
         } finally {
             if (usesBatchWriting) {
                 getSession().getPlatform().setUsesBatchWriting(true);
@@ -208,6 +252,14 @@ public class SchemaManager {
         createOrReplaceSequences(true);
     }
 
+    /**
+     * INTERNAL:
+     * Set to true if database schemas should be built during the DDL generation.
+     */
+    public void setCreateDatabaseSchemas(boolean createDatabaseSchemas) {
+        this.createDatabaseSchemas = createDatabaseSchemas;
+    }
+    
     public void setCreateSQLFiles(boolean genFlag) {
         this.createSQLFiles = genFlag;
     }
@@ -232,42 +284,154 @@ public class SchemaManager {
      * @param createSequenceTables - true to create the sequences tables, false to replace them (dropped then create)
      * @param createSequenceObjects - true to create the sequences objects, false to replace them (dropped then create)
      */
-    protected void createOrReplaceSequences(boolean createSequenceTables, boolean createSequenceObjects) throws EclipseLinkException {
+    protected void createOrReplaceSequences(boolean createSequenceTables, boolean createSequences) throws EclipseLinkException {
         // PERF: Allow a special "fast" flag to be set on the session causes a delete from the table instead of a replace.
         boolean fast = FAST_TABLE_CREATOR;
         if (fast) {
             // Assume sequences already created.
             return;
         }
+        
+        processSequenceDefinitions(createSequenceTables, createSequences, true);
+    } 
+    
+    /**
+     * This will drop the database schemas if managing the database schemas.
+     */
+    protected void dropSequences() {
+        processSequenceDefinitions(false, false, false);
+    }
+    
+    /**
+     * Method creates database tables/objects. If create is true, it will 
+     * attempt to create the object and silently ignore exceptions. If create 
+     * is false, it will drop the object ignoring any exceptions, then create 
+     * it if the replace flag is true (otherwise a drop only).
+     *   
+     * @param definition - the sequence definition
+     * @param createTable - true if table sequence table definitions should be created.
+     * @param createSequences - true if the sequence definition should be created, 
+     *        false if it should be dropped.
+     * @param replace - true if table definitions and sequence definitions should be replaced.
+     * @throws EclipseLinkException
+     */
+    protected void processSequenceDefinition(SequenceDefinition definition, final boolean createTables, final boolean createSequences, final boolean replace, HashSet<String> createdTableNames, HashSet<String> droppedTableNames) throws EclipseLinkException {
+        try {
+            // Handle the table definitions first.
+            if (definition.isTableSequenceDefinition()) {
+                TableDefinition tableDefinition = ((TableSequenceDefinition) definition).buildTableDefinition();
+                // Check that we haven't already created the table.
+                if (! createdTableNames.contains(tableDefinition.getFullName())) {
+                    createdTableNames.add(tableDefinition.getFullName());
+                
+                    // Check if it exists on the database. NOTE: when writing to scripts only with 
+                    // no connection, this of course will always return false hence the need for
+                    // the createdSequenceTableNames collection above.
+                    boolean exists = checkTableExists(tableDefinition);
+                    
+                    if (createTables) {
+                        // Don't create it if it already exists on the database.
+                        // In all all other cases, write it out.
+                        if ((shouldWriteToDatabase() && ! exists) || ! shouldWriteToDatabase()) {
+                            createObject(tableDefinition);
+                        }
+                    } else {
+                        // Don't check exists since if writing to scripts only with no connection, 
+                        // we'll never write the sql out. When executing to the database, the drop 
+                        // will fail and we'll ignore it. Note: TableSequenceDefinition's will drop 
+                        // their table definitions as needed (i.e.) when the jpa create database 
+                        // schemas flag is set and the table definition has a schema. Otherwise,
+                        // we should not drop sequence tables since they may be re-used across
+                        // persistence units (default behavior right now).
+                        // TODO: We should drop them really unless it is the default SEQUENCE table??
+                        if (replace) {
+                            dropObject(tableDefinition);
+                            createObject(tableDefinition);
+                        }
+                    }
+                }
+            }
+        } catch (DatabaseException exception) {
+            // ignore any database exceptions here and keep going.
+        }
+        
+        // Handle the sequence objects second.
+        try {
+            if (createSequences) {
+                createObject(definition);
+            } else {
+                try {
+                    // If the sequence definition has and will drop a table definition, then check 
+                    // if we have already dropped it. Table definitions are dropped as a whole if 
+                    // they have a schema name and the jpa create database schemas flag is set to true.
+                    if (definition.isTableSequenceDefinition()) {
+                        if (((TableSequenceDefinition) definition).shouldDropTableDefinition()) {
+                            String tableDefinitionTableName = ((TableSequenceDefinition) definition).getSequenceTableName();
+                            
+                            // If we have already dropped it, move on, otherwise drop it!
+                            if (droppedTableNames.contains(tableDefinitionTableName)) {
+                                return; // cut out early, we've already seen this table.
+                            } else {
+                                droppedTableNames.add(tableDefinitionTableName);
+                            }
+                        }
+                    }
+                    
+                    dropObject(definition);
+                } catch (DatabaseException exception) {
+                    // Ignore table not found for first creation
+                } 
+            
+                // Drop only scripts we don't want to replace.
+                if (replace) {
+                    createObject(definition);
+                }
+            }
+        } catch (Exception exception) {
+            // ignore any database exceptions here and keep chugging
+        }
+    }
+    
+    /**
+     * Common implementor for createSequence and replaceSequence, distinguishes between sequence tables and sequence objects
+     * @param createSequenceTables - true to create the sequences tables, false to replace them (dropped then create)
+     * @param createSequenceObjects - true to create the sequences objects, false to replace them (dropped then create)
+     * @param replaceSequences - true to actually replace, false to drop only.
+     */
+    protected void processSequenceDefinitions(boolean createSequenceTables, boolean createSequences, boolean replaceSequences) throws EclipseLinkException {
         Sequencing sequencing = getSession().getSequencing();
 
-        if ((sequencing == null) || (sequencing.whenShouldAcquireValueForAll() == Sequencing.AFTER_INSERT)) {
-            // Not required on Sybase native etc.
-            return;
+        // Not required on Sybase native etc.
+        if (sequencing != null && sequencing.whenShouldAcquireValueForAll() != Sequencing.AFTER_INSERT) {
+            // Build the sequence definitions.
+            HashSet<SequenceDefinition> sequenceDefinitions = buildSequenceDefinitions();
+        
+            // Now process the sequence definitions.
+            // CR 3870467, do not log stack
+            boolean shouldLogExceptionStackTrace = session.getSessionLog().shouldLogExceptionStackTrace();
+            session.getSessionLog().setShouldLogExceptionStackTrace(false);
+            HashSet<String> createdSequenceTableNames = new HashSet();
+            HashSet<String> droppedSequenceTableNames = new HashSet();
+        
+            for (SequenceDefinition sequenceDefinition : sequenceDefinitions) {
+                processSequenceDefinition(sequenceDefinition, createSequenceTables, createSequences, replaceSequences, createdSequenceTableNames, droppedSequenceTableNames);
+            }
+        
+            // Set the log stack trace flag back.
+            session.getSessionLog().setShouldLogExceptionStackTrace(shouldLogExceptionStackTrace);
         }
-
-        // Prepare table and sequence definitions
-        // table name mapped to TableDefinition
-        HashMap tableDefinitions = new HashMap();
-
-        // sequence name to SequenceDefinition
-        HashSet sequenceDefinitions = new HashSet();
-
-        // remember the processed - to handle each sequence just once.
+    } 
+    
+    /**
+     * INTERNAL:
+     * Build the sequence definitions.
+     */
+    protected HashSet<SequenceDefinition> buildSequenceDefinitions() {
+        // Remember the processed - to handle each sequence just once.
         HashSet processedSequenceNames = new HashSet();
-
-        buildTableAndSequenceDefinitions(sequenceDefinitions, processedSequenceNames, tableDefinitions);
-        processTableDefinitions(tableDefinitions, createSequenceTables);
-        processSequenceDefinitions(sequenceDefinitions, createSequenceObjects);
-    }    
-
-    private void buildTableAndSequenceDefinitions(final HashSet sequenceDefinitions, 
-            final HashSet processedSequenceNames, final HashMap tableDefinitions) {
-        Iterator descriptors = getSession().getDescriptors().values().iterator();
-
-        while (descriptors.hasNext()) {
-            ClassDescriptor descriptor = (ClassDescriptor) descriptors.next();
-
+        HashSet<SequenceDefinition> sequenceDefinitions = new HashSet<SequenceDefinition>();
+        
+        for (ClassDescriptor descriptor : getSession().getDescriptors().values()) {
             if (descriptor.usesSequenceNumbers()) {
                 String seqName = descriptor.getSequenceNumberName();
 
@@ -275,115 +439,47 @@ public class SchemaManager {
                     seqName = getSession().getDatasourcePlatform().getDefaultSequence().getName();
                 }
 
-                if (processedSequenceNames.contains(seqName)) {
-                    continue;
-                }
+                if (! processedSequenceNames.contains(seqName)) {
+                    processedSequenceNames.add(seqName);
 
-                processedSequenceNames.add(seqName);
+                    Sequence sequence = getSession().getDatasourcePlatform().getSequence(seqName);
+                    SequenceDefinition sequenceDefinition = buildSequenceDefinition(sequence);
 
-                Sequence sequence = getSession().getDatasourcePlatform().getSequence(seqName);
-
-                SequenceDefinition sequenceDefinition = buildSequenceDefinition(sequence);
-
-                if (sequenceDefinition == null) {
-                    continue;
-                }
-
-                sequenceDefinitions.add(sequenceDefinition);
-
-                TableDefinition tableDefinition = sequenceDefinition.buildTableDefinition();
-
-                if (tableDefinition != null) {
-                    String tableName = tableDefinition.getName();
-                    TableDefinition otherTableDefinition = (TableDefinition) tableDefinitions.get(tableName);
-
-                    if (otherTableDefinition != null) {
-                        // check for a conflict; if there is one - throw a ValidationException
-                    } else {
-                        tableDefinitions.put(tableName, tableDefinition);
+                    if (sequenceDefinition != null) {
+                        sequenceDefinitions.add(sequenceDefinition);
                     }
                 }
             }
         }
-    }
-
-    /**
-     * Method creates database sequence tables.  If create is true, it will attempt to create the sequence tables and silently 
-     * ignore exceptions.  If create is false, it will drop the tables ignoring any exceptions, then create it.  
-     * @param tableDefinitions - HashMap of Sequence table definitions
-     * @param create - true if tables should be created, false if they should be replaced (dropped then created)
-     * @throws EclipseLinkException
-     */
-    private void processTableDefinitions(final HashMap tableDefinitions, final boolean create) throws EclipseLinkException {
-
-        // create tables
-        Iterator itTableDefinitions = tableDefinitions.values().iterator();
         
-        // CR 3870467, do not log stack
-        boolean shouldLogExceptionStackTrace = session.getSessionLog().shouldLogExceptionStackTrace();
-
-        while (itTableDefinitions.hasNext()) {
-            TableDefinition tableDefinition = (TableDefinition) itTableDefinitions.next();
-
-            processDatabaseObjectDefinition(tableDefinition, create, shouldLogExceptionStackTrace);
-        }
+        return sequenceDefinitions;
     }
-
+    
     /**
-     * Method creates database sequence objects.  If create is true, it will attempt to create the sequence and silently ignore
-     * exceptions.  If create is false, it will drop the sequence ignoring any exceptions, then create it.  
-     * @param sequenceDefinitions - HashSet of Sequence object definitions
-     * @param create - true if sequenceDefinitions should be created, false if they should be replaced (dropped then created)
-     * @throws EclipseLinkException
+     * Check if the table exists by issuing a select.
      */
-    private void processSequenceDefinitions(final HashSet sequenceDefinitions, final boolean create) throws EclipseLinkException {
-        
-        // CR 3870467, do not log stack
-        boolean shouldLogExceptionStackTrace = session.getSessionLog().shouldLogExceptionStackTrace();
-        // create sequence objects
-        Iterator itSequenceDefinitions = sequenceDefinitions.iterator();
-        
-        while (itSequenceDefinitions.hasNext()) {
-            SequenceDefinition sequenceDefinition = (SequenceDefinition) itSequenceDefinitions.next();
-            
-            processDatabaseObjectDefinition(sequenceDefinition, create, shouldLogExceptionStackTrace);
+    public boolean checkTableExists(TableDefinition table) {
+        String column = null;
+        for (FieldDefinition field : table.getFields()) {
+            if (column == null) {
+                column = field.getName();
+            } else if (field.isPrimaryKey()) {
+                column = field.getName();
+                break;
+            }                        
         }
-    }
-
-    /**
-     * Method creates database tables/objects.  If create is true, it will attempt to create the object and silently ignore
-     * exceptions.  If create is false, it will drop the object ignoring any exceptions, then create it.  
-     * @param definition -the object definition
-     * @param create - true if the definition should be created, false if it should be replaced (dropped then created)
-     * @throws EclipseLinkException
-     */
-    private void processDatabaseObjectDefinition(DatabaseObjectDefinition definition, final boolean create, final boolean shouldLogExceptionStackTrace) throws EclipseLinkException {
-        if (shouldLogExceptionStackTrace) {
-            session.getSessionLog().setShouldLogExceptionStackTrace(false);
-        }
-
-        if (create) {
-            try {
-                createObject(definition);
-            } catch (DatabaseException exception) {
-                // Ignore already created
-            } finally {
-                if (shouldLogExceptionStackTrace) {
-                    session.getSessionLog().setShouldLogExceptionStackTrace(true);
-                }
-            }
-        } else {
-            try {
-                dropObject(definition);
-            } catch (DatabaseException exception) {
-                // Ignore table not found for first creation
-            } finally {
-                if (shouldLogExceptionStackTrace) {
-                    session.getSessionLog().setShouldLogExceptionStackTrace(true);
-                }
-            }
-
-            createObject(definition);
+        String sql = "SELECT " + column + " FROM " + table.getFullName() + " WHERE " + column + " <> " + column;
+        DataReadQuery query = new DataReadQuery(sql);
+        query.setMaxRows(1);
+        int level = this.session.getSessionLog().getLevel();
+        try {
+            this.session.getSessionLog().setLevel(SessionLog.OFF);
+            this.session.executeQuery(query);
+            return true;
+        } catch (Exception notFound) {
+            return false;
+        } finally {
+            this.session.getSessionLog().setLevel(level);
         }
     }
 
@@ -393,12 +489,21 @@ public class SchemaManager {
         }
         if (sequence instanceof TableSequence ||
             (sequence instanceof DefaultSequence && ((DefaultSequence)sequence).getDefaultSequence() instanceof TableSequence)) {
-            return new TableSequenceDefinition(sequence);
+            return new TableSequenceDefinition(sequence, createDatabaseSchemas);
         } else if (sequence instanceof UnaryTableSequence ||
                    (sequence instanceof DefaultSequence && ((DefaultSequence)sequence).getDefaultSequence() instanceof UnaryTableSequence)) {
-            return new UnaryTableSequenceDefinition(sequence);
+            return new UnaryTableSequenceDefinition(sequence, createDatabaseSchemas);
         } else if (sequence instanceof NativeSequence || 
                    (sequence instanceof DefaultSequence && ((DefaultSequence)sequence).getDefaultSequence() instanceof NativeSequence)) {
+            NativeSequence nativeSequence = null;
+            if (sequence instanceof NativeSequence) {
+                nativeSequence = (NativeSequence)sequence;
+            } else {
+                nativeSequence = (NativeSequence)((DefaultSequence)sequence).getDefaultSequence();
+            }
+            if (nativeSequence.hasDelegateSequence()) {
+                return buildSequenceDefinition(((NativeSequence)sequence).getDelegateSequence());
+            }
             return new SequenceObjectDefinition(sequence);
         } else {
             return null;
@@ -444,14 +549,19 @@ public class SchemaManager {
         }
 
         try {
+            // If the object definition has a database schema collect it.
+            collectDatabaseSchemasForDrop(databaseObjectDefinition);
+            
+            databaseObjectDefinition.preDropObject(getSession(), getDropSchemaWriter(), this.createSQLFiles);
             if (shouldWriteToDatabase()) {
                 // drop actual object
                 databaseObjectDefinition.dropFromDatabase(getSession());
             } else {
                 Writer dropSchemaWriter = getDropSchemaWriter();
+                
                 // drop actual object
                 databaseObjectDefinition.dropObject(getSession(), dropSchemaWriter, createSQLFiles);
-                if (createSQLFiles){
+                if (this.createSQLFiles){
                     this.appendToDDLWriter(dropSchemaWriter, getSession().getPlatform().getStoredProcedureTerminationToken());
                 }
                 this.appendToDDLWriter(dropSchemaWriter, "\n");
@@ -462,7 +572,7 @@ public class SchemaManager {
             }
         }
     }
-
+    
     /**
      * Drop (delete) the table named tableName from the database.
      */
@@ -814,32 +924,35 @@ public class SchemaManager {
         this.createSchemaWriter = null;
         this.dropSchemaWriter = null;
     }
-
+    
     /**
      * PUBLIC:
      * Output all DDL statements to a file writer specified by the name in the parameter.
      */
     public void outputDDLToFile(String fileName) {
-        try {
-            this.createSchemaWriter = new java.io.FileWriter(fileName);
-        } catch (java.io.IOException ioException) {
-            throw ValidationException.fileError(ioException);
-        }
+        this.createSchemaWriter = getWriter(fileName);
     }
 
     public void outputCreateDDLToFile(String fileName) {
-        try {
-            this.createSchemaWriter = new java.io.FileWriter(fileName);
-        } catch (java.io.IOException ioException) {
-            throw ValidationException.fileError(ioException);
-        }
+        this.createSchemaWriter = getWriter(fileName);
     }
 
     public void outputDropDDLToFile(String fileName) {
+        this.dropSchemaWriter = getWriter(fileName);
+    }
+    
+    protected Writer getWriter(String fileName) {
         try {
-            this.dropSchemaWriter = new java.io.FileWriter(fileName);
+            return new java.io.FileWriter(fileName);
         } catch (java.io.IOException ioException) {
-            throw ValidationException.fileError(ioException);
+            // Try a url next, otherwise throw the existing error.
+            try {
+                URL url = new URL(fileName);
+                return new java.io.FileWriter(url.getFile());
+            } catch (Exception e) {
+                // MalformedURLException and IOException
+                throw ValidationException.fileError(ioException);
+            }
         }
     }
 
@@ -905,8 +1018,8 @@ public class SchemaManager {
     }
     
     /**
-    * Create the default table schema for the TopLink project this session associated with.
-    */
+     * Create the default table schema for the project this session associated with.
+     */
     public void createDefaultTables(boolean generateFKConstraints) {
         //Create each table w/o throwing exception and/or exit if some of them are already existed in the db. 
         //If a table is already existed, skip the creation.
@@ -916,52 +1029,118 @@ public class SchemaManager {
 
         try {
             TableCreator tableCreator = getDefaultTableCreator(generateFKConstraints);
-            tableCreator.createTables(session, this);
+            tableCreator.createTables(this.session, this);
         } catch (DatabaseException ex) {
             // Ignore error
         } finally {
             getSession().getSessionLog().setShouldLogExceptionStackTrace(shouldLogExceptionStackTrace);
         }
-    }
-
-    /**
-     * Drop and recreate the default table schema for the TopLink project this session associated with.
-     */
-    public void replaceDefaultTables() throws EclipseLinkException {
-        boolean shouldLogExceptionStackTrace = getSession().getSessionLog().shouldLogExceptionStackTrace();
-        getSession().getSessionLog().setShouldLogExceptionStackTrace(false);
-
-        try {
-            TableCreator tableCreator = getDefaultTableCreator(true);
-            tableCreator.replaceTables(session, this);
-        } catch (DatabaseException exception) {
-            // Ignore error
-        } finally {
-            getSession().getSessionLog().setShouldLogExceptionStackTrace(shouldLogExceptionStackTrace);
+        // Reset database change events to new tables.
+        if (this.session.getDatabaseEventListener() != null) {
+            this.session.getDatabaseEventListener().remove(this.session);
+            this.session.getDatabaseEventListener().register(this.session);
         }
     }
     
     /**
-     * Drop and recreate the default table schema for the TopLink project this session associated with.
+     * INTERNAL:
+     * Iterate over the schemas that need to be dropped.
      */
-    public void replaceDefaultTables(boolean keepSequenceTables, boolean generateFKConstraints) throws EclipseLinkException {
+    public void dropDatabaseSchemas() {
+        for (String databaseSchema : dropDatabaseSchemas.keySet()) {
+            if (shouldWriteToDatabase()) {
+                dropDatabaseSchemas.get(databaseSchema).dropDatabaseSchemaOnDatabase(getSession());
+            } else {
+                dropDatabaseSchemas.get(databaseSchema).dropDatabaseSchema(getSession(), getDropSchemaWriter());
+                appendToDDLWriter(getDropSchemaWriter(), "\n");
+            }
+        }
+    }
+    
+    /**
+     * Create the default table schema for the project this session associated with.
+     */
+    public void dropDefaultTables() {
+        // Drop each table w/o throwing exception and/or exit if some don't exist.
         boolean shouldLogExceptionStackTrace = getSession().getSessionLog().shouldLogExceptionStackTrace();
         getSession().getSessionLog().setShouldLogExceptionStackTrace(false);
 
         try {
-            TableCreator tableCreator = getDefaultTableCreator(generateFKConstraints);
-            tableCreator.replaceTables(session, this, keepSequenceTables);
-        } catch (DatabaseException exception) {
+            // Drop the tables.
+            TableCreator tableCreator = getDefaultTableCreator(true);
+            tableCreator.dropTables(this.session, this);
+            
+            // Drop the sequences.
+            dropSequences();
+            
+            // Drop all the database schemas now if set to do so. This must be 
+            // called after all the constraints, tables etc. are dropped.
+            dropDatabaseSchemas();
+        } catch (DatabaseException ex) {
             // Ignore error
         } finally {
             getSession().getSessionLog().setShouldLogExceptionStackTrace(shouldLogExceptionStackTrace);
         }
+        // Reset database change events to new tables.
+        if (this.session.getDatabaseEventListener() != null) {
+            this.session.getDatabaseEventListener().remove(this.session);
+            this.session.getDatabaseEventListener().register(this.session);
+        }
+    }
+
+    /**
+     * Drop and recreate the default table schema for the project this session associated with.
+     */
+    public void replaceDefaultTables() throws EclipseLinkException {
+        replaceDefaultTables(true, true, true);
+    }
+
+    /**
+     * Drop and recreate the default table schema for the project this session associated with.
+     */
+    public void replaceDefaultTables(boolean createSequenceTables, boolean generateFKConstraints) throws EclipseLinkException {
+        replaceDefaultTables(createSequenceTables, false, generateFKConstraints);
+    }
+    
+    /**
+     * Drop and recreate the default table schema for the project this session associated with.
+     */
+    public void replaceDefaultTables(boolean createSequenceTables, boolean createSequences, boolean generateFKConstraints) throws EclipseLinkException {
+        boolean shouldLogExceptionStackTrace = getSession().getSessionLog().shouldLogExceptionStackTrace();
+        this.session.getSessionLog().setShouldLogExceptionStackTrace(false);
+
+        try {
+            TableCreator tableCreator = getDefaultTableCreator(generateFKConstraints);
+            tableCreator.replaceTables(this.session, this, createSequenceTables, createSequences);
+            
+            // Drop all the database schemas now if set to do so. This must be 
+            // called after all the constraints, tables etc. are dropped.
+            dropDatabaseSchemas();
+        } catch (DatabaseException exception) {
+            // Ignore error
+        } finally {
+            this.session.getSessionLog().setShouldLogExceptionStackTrace(shouldLogExceptionStackTrace);
+        }
+        // Reset database change events to new tables.
+        if (this.session.getDatabaseEventListener() != null) {
+            this.session.getDatabaseEventListener().remove(this.session);
+            this.session.getDatabaseEventListener().register(this.session);
+        }
     }
 
     public void setSession(DatabaseSessionImpl session) {
         this.session = session;
     }
 
+    /**
+     * INTERNAL:
+     * Returns true if a database schema should be created during ddl generation
+     * for the given databaseObjectDefinition.
+     */
+    protected boolean shouldCreateDatabaseSchema(DatabaseObjectDefinition databaseObjectDefinition, Set<String> createdDatabaseSchemas) {
+        return (createDatabaseSchemas && databaseObjectDefinition.shouldCreateDatabaseSchema(createdDatabaseSchemas));
+    }
+    
     /**
      * PUBLIC:
      * Return true if this SchemaManager should write to the database directly
@@ -969,7 +1148,7 @@ public class SchemaManager {
     public boolean shouldWriteToDatabase() {
         return ((this.createSchemaWriter == null) && (this.dropSchemaWriter == null));
     }
-
+    
     /**
      * Use the definition to alter sequence.
      */
@@ -998,4 +1177,26 @@ public class SchemaManager {
         }
     }
 
+    /**
+     * Create or extend the default table schema for the project this session associated with.
+     */
+    public void extendDefaultTables(boolean generateFKConstraints) throws EclipseLinkException {
+        boolean shouldLogExceptionStackTrace = getSession().getSessionLog().shouldLogExceptionStackTrace();
+        this.session.getSessionLog().setShouldLogExceptionStackTrace(false);
+
+        try {
+            TableCreator tableCreator = getDefaultTableCreator(generateFKConstraints);
+            tableCreator.extendTables(this.session, this);
+        } catch (DatabaseException exception) {
+            // Ignore error
+        } finally {
+            this.session.getSessionLog().setShouldLogExceptionStackTrace(shouldLogExceptionStackTrace);
+        }
+        // Reset database change events to new tables.
+        if (this.session.getDatabaseEventListener() != null) {
+            this.session.getDatabaseEventListener().remove(this.session);
+            this.session.getDatabaseEventListener().register(this.session);
+        }
+    }
+
 }
