@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+ * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
  * This program and the accompanying materials are made available under the 
  * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
  * which accompanies this distribution. 
@@ -107,7 +107,7 @@ public class StoredProcedureGenerator {
         while (tokenIndex != -1) {
             stringWriter.write(stringToModify.substring(startIndex, tokenIndex));
             startIndex = tokenIndex + 1;
-            Object parameter = call.getParameters().elementAt(nextParamIndex);
+            Object parameter = call.getParameters().get(nextParamIndex);
             if (parameter instanceof DatabaseField) {
                 stringWriter.write(replacementToken);
                 stringWriter.write(((DatabaseField)parameter).getName());
@@ -576,7 +576,7 @@ public class StoredProcedureGenerator {
         this.generateSequenceStoredProcedures(getSession().getProject());
         while (iterator.hasNext()) {
             desc = (ClassDescriptor)descriptors.get(iterator.next());
-            if (desc.isDescriptorForInterface() || desc.isAggregateDescriptor() || desc.isAggregateCollectionDescriptor()) {
+            if (desc.isDescriptorForInterface() || desc.isDescriptorTypeAggregate()) {
                 continue;
             }
             definition = this.generateInsertStoredProcedure(desc);
